/**
 * Exemplo de Usos do ORM
 * Mais info em: https://knorm.netlify.com/
 */
const dotenv = require('dotenv');
const config = require('./config.json');
dotenv.config();

if (process.env && process.env.DEV_HOST) {
    console.log(`DEV Host informado no arquivo .env ${process.env.DEV_HOST}`);
    config.database.host = process.env.DEV_HOST;
    config.asterisk.host = process.env.DEV_HOST;
}

const md5 = require('md5');

const Supervisores = require('./models/Supervisores');
const Modulos = require('./models/Modulos');
const SupervisoresModulos = require('./models/SupervisoresModulos');
const SupervisoresFilas = require('./models/SupervisoresFilas')

// Usando Extends com filtro na classe, olhe na classe Turno; ela extende de Horarios mas com tipo = T.
const getTurnos = async () => {
    var turnos = await Turno.fetch();
}
// Extends com filtro na classe, mesmo caso do acima
const getFeriados = async () => {
    var feriados = await Feriado.fetch();
}

// Sem on, só pelo que foi definido nas referencias da classe
const getFilaComTurnos = async (nome) => {
    const ret = await Fila.query
        .leftJoin(FilaTurnos)
        .where({ name: nome })
        .fetch();
}

const getFilasComTurnos = async () => {
    const ret = await Fila.query
        .leftJoin(
            FilaTurnos.query
                .on('queue')
                .leftJoin(Horarios.query.on('TurnoID').as('Turnos'))
        )
        .fetch();
}

// dados de várias tabelas em arrays.
// O ORM Executa uma query só e não consegue agrupar vários joins em arrays com relações Many-to-many que envolve uma terceira tabela, então só resta fazer várias querys mesmo e colocar em arrays!!
const getFilaComTurnosERespostas = async (nome) => {
    var ret = await Fila.query
        .leftJoin(Pesquisa)
        .where({ name: nome })
        .fetch();

    var respostas = await FilaRespostasAutomaticas.query.where({ fila: nome }).fetch();
    ret[0].RespostasAutomaticas = respostas;

    //var turnos = await FilaTurnos.query.fields(['id']).leftJoin(Horarios.query.as('Turno')).where({FilaID: nome}).fetch();
    var turnos = await Horarios.query.fields(['id', 'descricao']).innerJoin(FilaTurnos.query.where({ FilaID: nome })).fetch();
    ret[0].turnos = turnos;

}

// Com transações...Exemplo de Update, delete e insert com transações.
const UpdateFila = async (request) => {
    // Crie a transação
    const transaction = new db.orm.Transaction();

    // Crie Objetos do ORM com transações. Aqui nesse caso são 3
    const FilaT = transaction.models.Fila;
    const FilaTurnosT = transaction.models.FilaTurnos;
    const FilaRespostasAutomaticasT = transaction.models.FilaRespostasAutomaticas;

    try {
        var { turnos, RespostasAutomaticas, ...FilaUp } = request.payload;
        FilaUp.name = FilaUp.name ? FilaUp.name : request.params.FILA_NOME;

        // Update no primeiro objeto
        var fila = await FilaT.update(FilaUp, { require: true });

        // Delete e insert no segundo objeto se vier dados para ele...
        // require(false) no detete para não retornar erro se não tiver registros para apagar!!
        if (turnos) {
            await FilaTurnosT.query
                .require(false)
                .where({ FilaID: FilaUp.name })
                .delete();

            FilaTurnosT.insert(request.payload.turnos);
        }

        // Delete e insert no terceiro objeto se vier dados para ele...
        if (RespostasAutomaticas) {
            await FilaRespostasAutomaticasT.query
                .require(false)
                .where({ fila: FilaUp.name })
                .delete();

            FilaRespostasAutomaticasT.insert(request.payload.RespostasAutomaticas);
        }

        // Commit da transação..
        await transaction.commit();
    } catch (e) {
        await transaction.rollback();
    }
}

const teste1 = async () => {
    // Sempre passar require: true onde quiser q se der erro abort, lembrando q select, delete qualquer query se não afetar ou retornar registro com require: true vai retornar erro.
    // Por exemplo: Cliente.update(request.payload, {require: true})


    // Lista todos
    //const clientes = await Cliente.fetch();
    //console.log(clientes); 

    // Apenas 1 por Numero    
    //const cliente = await Cliente.fetchByNumero('555130305501');
    //console.log(cliente);
    // ou
    //var cliente = await Cliente.query
    //.where({ numero: '123456' })
    //.first()
    //.fetch();

    // Com left join
    //const ClientesComGrupo = await Cliente.query
    //.leftJoin(ClienteGrupo.query.on('idGrupo'))
    //.fetch();

    //console.log(ClientesComGrupo);

    // Se nao tem cria
    try {
        const sup = await Supervisores.query
            .where({ nome: 'admin' })
            .where({ senha: md5('admin') })
            .require(true)
            .first()
            .fetch();

        const supfilas = await SupervisoresFilas.query
            .where({ id_supervisor: sup.id })
            .require(false)
            .fetch();

        if (supfilas) {
            sup.filas = supfilas.map(f => {
                return f.id_fila;
            })
        }


        const supModulos = await SupervisoresModulos.query
            .innerJoin(Modulos)
            .require(false)
            .where({ id_supervisor: sup.id })
            .fetch();

        if (supModulos) {
            const somodulos = supModulos.map(modulo => {
                return modulo.modulos[0];
            })

            const ordemm = somodulos
                .slice(0)
                .sort((a, b) => {
                    if (a.parent == null) return 1
                    else return a.descricao.toUpperCase() > b.descricao.toUpperCase() ? 1 : -1
                });

            const nest = (items, id = null, link = 'parent') =>
                items
                    .filter(item => item[link] === id)
                    .map(item => ({ ...item, modulos: nest(items, item.id) }));


            sup.modulos = nest(ordemm);
        }

        console.log(JSON.stringify(sup, null, 2));

    } catch (error) {
        console.error(error.message);
    }
}

const getComCamposApelidos = async () => {
    const m = await AtendimentoMensagem.fetchById('370');

    /**
    Exibe:
    AtendimentoMensagem {
        id: 370,
        idAtendimento: 1071,
        inOut: 1,
        dataHora: 2019-06-17T07:08:01.000Z,
        agente: 'Fernando Rossato',
        texto: 'Boa noite' 
    }

    Executa:  SELECT ... "queueChatMessages"."idQueueChat" AS "queueChatMessages.idAtendimento" FROM "queueChatMessages"
     */
}

const saveComCamposApelidos = async () => {

    const mensagem = new AtendimentoMensagem();
    mensagem.idAtendimento = 1071;
    mensagem.inOut = 1;
    mensagem.texto = 'Oi';
    mensagem.agente = 'Fernando Rossato';
    var novamensagem = await mensagem.save();

    // Pode ser feito InLine tb: const mensagem = new AtendimentoMensagem({idAtendimento: 1071, inOut: 1, ...});
    /**
    Exibe:
    AtendimentoMensagem {
        idAtendimento: 1071,
        inOut: 1,
        texto: 'Oi',
        agente: 'Fernando Rossato',
        id: 6501,
        dataHora: 2019-08-17T02:27:12.000Z 
    }

    Executa:  INSERT INTO "queueChatMessages" AS "queueChatMessages" ("idQueueChat"
    parameters: $1 = '1071'
     */
}

const SelectSemRetornarErro = async () => {
    var cliente = await Cliente.query
        .require(false)
        .where({ numero: '1234568896655' })
        .fetch();

    // []
}

// first
const SelectOneSemRetornarErro = async () => {
    var cliente = await Cliente.query.require(false).first()
        .where({ numero: '555199545103' })
        .fetch();

    // []
}

const where = async () => {
    try {
        // Inline .where({ idCliente: 4, queue: 'Whats5510' })
        const r = await ClienteAtendimento.query
            .where({ idCliente: 4 })
            .where({ queue: 'Whats5510' })
            .fetch();


    } catch (error) {
        console.error(error.message);
    }
}

const whereComOr = async () => {
    try {
        const r = await ClienteAtendimento.query
            .where(ClienteAtendimento.where.or(
                ClienteAtendimento.where.equal({ queue: 'Whats5510' }),
                ClienteAtendimento.where.equal({ queue: 'Atendimento' })
            ))
            .fetch();
        // FROM "queueChat" AS "queueChat" WHERE "queueChat"."queue" = $1 OR "queueChat"."queue" = $2


    } catch (error) {
        console.error(error.message);
    }
}

const whereComOrEand = async () => {
    try {
        const r = await ClienteAtendimento.query
            .where(ClienteAtendimento.where.or(
                ClienteAtendimento.where.equal({ queue: 'Whats5510' }),
                ClienteAtendimento.where.equal({ queue: 'Atendimento' })
            ))
            .where({ idCliente: 4 })
            .fetch();
        // FROM "queueChat" AS "queueChat" WHERE ("queueChat"."queue" = $1 OR "queueChat"."queue" = $2) AND "queueChat"."idCliente" = $3


    } catch (error) {
        console.error(error.message);
    }
}

const whereComLike = async () => {
    try {
        const r = await ClienteAtendimento.query
            .where(ClienteAtendimento.where.like({ queue: 'Whats5510%' }))
            .fetch();
        // FROM "queueChat" AS "queueChat" WHERE "queueChat"."queue" LIKE $1
        // parameters: $1 = 'Whats5510%'


    } catch (error) {
        console.error(error.message);
    }
}

teste1();

//SelectOneSemRetornarErro();

//saveComCamposApelidos();

//getComCamposApelidos();

//SelectSemRetornarErro();

//whereComLike();

//whereComOrEand();

//whereComOr();

//where();

//teste1();