# Xcontact server - API para o Xcontact

## Instruções para desenvolvedores

A pasta node_modules está sendo ignorada então:

1. Clone este repositório
2. Rode `npm install`

Depois das depências instaladas para rodar use:

3. node app.js
4. Você pode acessar a página da API [http://localhost:8001](http://localhost:8001)

---

## .env

Você pode criar um arquivo `.env` na raiz do projeto para definir algumas variáveis uteis no desenvolvimento.
Exemplo:

```
DEV_HOST=172.30.25.177
SWAGGER_PROTOCOLO=http
NO_EMAIL=true
NO_DISCADOR=true
NO_CHAT=true
NO_WHATS=true
LOG=debug
WHATSAPI_ATIVADO=true
WHATSAPI_CLIENTID=Key do cliente
WHATSAPI_HOST=http://127.0.0.1:3000
VIDEO=true
VIDEO_HOST=ws://localhost:1000
```

---

## node .\admin_client.js

Você pode usar o cliente do serviço para chamar métodos.

Ex: Mudar o log level sem parar o serviço.

```sh
ADMIN CLI> setLog debug
```

---

## config.json

As configurações do serviço estão no arquivo config.json, as principais configurações são:

1. Acesso ao banco de dados.
2. Acesso ao Asterisk.
3. Definições de fuso horário.
4. Acesso ao servidor CSA (Xcontact Cloud server API)

## Eventos Omni Channel

#### novaMensagemAgente

---

Evento emitido quando o envia uma mensagem.

- **RETORNO**

```json
{
  "fila": "whats_gup",
  "data": "2023-03-09T15:37:35.200Z",
  "atendimentoId": "930312",
  "agente": "Teste Valeska"
}
```

#### removeAtendimentoNaFila

---

Evento emitido quando o atendimento e removido da fila de espera.

- **RETORNO**

```json
{
  "fila": "whats_gup",
  "data": "2023-03-09T15:37:35.200Z",
  "atendimentoId": "930312",
  "agente": "juca",
  "tipo": "transferencia || encerramento || atendido"
}
```

#### FilaInviteAtendimento

---

Evento emitido quando o invite do atendimento e emitido.

- **RETORNO**

```json
{
  "fila": "whats_gup",
  "data": "2023-03-09T15:37:35.200Z",
  "atendimentoId": "930312",
  "agente": "Juca"
}
```

#### agenteAceitouAtendimento

---

Evento emitido quando o agente aceita o invite do atendimento.

- **RETORNO**

```json
{
  "fila": "whats_gup",
  "data": "2023-03-09T15:37:35.200Z",
  "atendimentoId": "930312",
  "agente": "Teste Valeska"
}
```

#### agenteAddAtendimento

---

Evento emitido quando o agente cria o atendimento.

- **RETORNO**

```json
{
  "fila": "whats_gup",
  "data": "2023-03-09T15:37:35.200Z",
  "atendimentoId": "930312",
  "agente": "Teste Valeska"
}
```

#### filaAddAtendimento

---

Evento emitido quando o atendimento vai para aguardando na fila.

- **RETORNO**

```json
{
  "fila": "whats_gup",
  "data": "2023-03-09T15:37:35.200Z",
  "atendimentoId": "930312"
}
```

#### filaNovaMensagem

---

Evento emitido quando recebida nova mensagem.

- **RETORNO**

```json
{
  "fila": "whats_gup",
  "data": "2023-03-09T15:37:35.200Z",
  "atendimentoId": "930312",
  "agente": "Teste Valeska"
}
```

#### filaNovoAtendimento

---

Evento emitido quando o atendimento e criado (apenas para incrementar o contador de total de atendimentos da fila).

- **RETORNO**

```json
{
  "fila": "whats_gup",
  "data": "2023-03-09T15:37:35.200Z",
  "atendimentoId": "930312"
}
```
---
#### DOC OMNI CHANNEL

Para estabelecer uma conexão com o WebSocket e iniciar sessão no sistema, é essencial enviar um método `Login` imediatamente após a conexão ser estabelecida. Este processo garante que o agente seja identificado corretamente no sistema e receba os dados e permissões adequadas à sua conta.

### Conectando e Autenticando no WebSocket

1. **Estabelecer Conexão**: Primeiro, conecte-se ao WebSocket usando a URL fornecida. Este passo é crucial para iniciar a comunicação bidirecional entre o cliente e o servidor.

2. **Enviar Método `Login`**: Após a conexão ser estabelecida com sucesso, você deve emitir imediatamente o evento `Login` com as credenciais e informações do agente. Este passo é necessário para autenticar o agente no sistema e configurar a sessão corretamente.

### Estrutura do Método `Login`

Quando emitir o evento `Login`, inclua as seguintes informações:

- `xcontactID`: Identificador único do agente ou sessão. Pode ser um valor pré-definido para testes ou produção.
- `id`: ID da conexão WebSocket. Normalmente, é fornecido automaticamente pela biblioteca WebSocket.
- `token`: Token de autenticação para validar a sessão. Este pode ser um valor estático para fins de teste ou um token dinâmico obtido através de um processo de login anterior.
- `agente`: Nome completo do agente conectando-se ao sistema.
- `ativo`: Um valor booleano indicando se o agente está ativo. Geralmente, define-se como `1` para ativo.
- `ramal`: O ramal associado ao agente.
- `web`: Um valor booleano que indica se a conexão é feita através de um cliente web. Geralmente, define-se como `true`.
- `versao`: Versão do cliente ou protocolo sendo utilizado.

### Exemplo de Código

```javascript
ws.on('connect', async () => {
  console.log('Conectado');
  conectado.value = true;
  await new Promise((resolve) => {
    ws.emit('Login', {
      xcontactID: 'teste',
      id: ws.id,
      token: 'teste',
      agente: 'Juca Bala',
      ativo: 1,
      ramal: 'SIP/666',
      web: true,
      versao: '4.0'
    }, (retorno) => {
      console.log('Login efetuado com sucesso', retorno);
      resolve(retorno);
    });
  });
});
```

### Considerações

- Assegure-se de que todas as informações enviadas no método `Login` estejam corretas e atualizadas.
- O tratamento do retorno do `Login` é essencial para confirmar que a autenticação foi bem-sucedida e que o agente está pronto para receber e enviar mensagens.
- Adapte o exemplo de código conforme necessário para alinhar com a implementação específica do seu projeto e as estruturas de dados usadas.

Este processo é uma parte fundamental da configuração inicial da sessão do agente, permitindo uma comunicação segura e personalizada através do WebSocket.
---
# EVENTOS WEBSOCKET
---

## Escutando Novas Mensagens
### Conexão WebSocket

Para começar a receber eventos, primeiro estabeleça uma conexão WebSocket com o endereço fornecido:

```
wss://homologadev.xcontactcenter.com.br:8003/omni
```


### Evento: `newMessage`

Este evento é emitido pelo servidor para notificar o cliente sobre a chegada de uma nova mensagem em um atendimento ativo. Os clientes devem escutar este evento para atualizar suas interfaces de usuário em tempo real com as novas mensagens recebidas.


### Uso

Após estabelecer a conexão WebSocket, configure o cliente para escutar o evento `newMessage`:

```javascript
ws.on('newMessage', async (dados) => {
  // Processamento dos dados da mensagem recebida
});
```

### Estrutura dos Dados Recebidos

O objeto `dados` recebido pelo evento contém informações detalhadas sobre a mensagem e o atendimento, incluindo:

- `message`: Objeto contendo detalhes da mensagem, como texto, ID e anexos.
- `atendimento`: Objeto contendo detalhes do atendimento, como ID do cliente, status, e informações da fila.

Aqui está uma documentação concisa para os eventos WebSocket que seu sistema deve escutar:

---

### `TransfereAtendimento`

Informa sobre a transferência de um atendimento para outro agente ou fila.

**Estrutura de Dados Recebidos:**

  - `id`: ID do atendimento transferido.
  - `idCliente`: ID do cliente associado ao atendimento.
  - `ultimoAgente`: Nome do último agente antes da transferência.

---
### `addAtendimento`

Sinaliza a adição de um novo atendimento ao cliente.

**Estrutura de Dados Recebidos:**

  - `idCliente`: ID do cliente para o novo atendimento.

---
### `RemoveAtendimento`

Indica que um atendimento foi removido ou encerrado.

**Estrutura de Dados Recebidos:**

  - `id`: ID do atendimento encerrado.

---
### `inviteAtendimento`

Notifica o agente sobre um convite para atender a um novo cliente.

**Estrutura de Dados Recebidos:**

  - `atendimento`: Detalhes do convite de atendimento, incluindo o nome do remetente.

---
### `removeInviteAgente`

Informa sobre a remoção de um convite de atendimento direcionado ao agente.

**Estrutura de Dados Recebidos:**

  - `atendimento`: Detalhes do atendimento cujo convite foi removido.

---
### `SendSuccess`

Confirmação de envio bem-sucedido de uma mensagem.

**Estrutura de Dados Recebidos:**

  - `message`: Detalhes da mensagem enviada, incluindo o status de envio.

---
### `getTemplatesGupshup`

Fornece templates de mensagens aprovados pelo serviço Gupshup.

**Estrutura de Dados Recebidos:**

  - `dados`: String JSON contendo os templates aprovados.

---
### `getTemplatesZenvia`

Fornece templates de mensagens aprovados pelo serviço Zenvia.

**Estrutura de Dados Recebidos:**

  - `dados`: String JSON contendo os templates aprovados.

# METODOS WEBSOCKET
## Enviar Mensagem

### Método: `sendMsg`

O método `sendMsg` é responsável por enviar mensagens de um agente para um cliente. Este método deve ser emitido através da conexão WebSocket estabelecida e espera receber um callback com o resultado da operação.

#### Parâmetros do Método

- `idCliente` (Number/String): O identificador único do cliente com quem a mensagem será compartilhada.
- `message` (Object): Um objeto contendo os detalhes da mensagem a ser enviada.
- `atendimento` (Object): Um objeto contendo os detalhes do atendimento associado à mensagem.

#### Estrutura do Objeto `message`

```json
{
  "texto": "Texto da mensagem",
  "quotedMsg": {
    "mailMessageID": "Identificador da mensagem que está sendo citada, se houver"
  },
  // Outros campos relevantes para a mensagem
}
```

#### Uso

```javascript
// Exemplo de uso do método sendMsg
sendMessage(1234, {
  texto: 'Olá, como posso ajudar?',
  // ... outros campos da mensagem
}, atendimento);
```

#### Tratamento de Eventos

Ao enviar uma mensagem, você deve tratar os possíveis eventos de retorno:

- Em caso de sucesso, o callback receberá um objeto com os detalhes da mensagem enviada, incluindo um identificador da mensagem (`id`).
- Em caso de erro, o callback será chamado com um objeto contendo uma propriedade `error` e uma mensagem de erro (`message`).

#### Exemplo de Retorno de Sucesso

```javascript
{
  "id": "identificador-da-mensagem",
  // ... outros detalhes da mensagem
}
```

#### Exemplo de Retorno de Erro

```javascript
{
  "error": true,
  "message": "Descrição do erro"
}
```

#### Monitorando a Resposta

```javascript
// Exemplo de monitoramento do retorno após emitir sendMsg
ws.emit('sendMsg', message, (retorno) => {
  if (retorno.error) {
    // Lida com erro
    console.error(retorno.message);
  } else {
    // Lida com sucesso
    console.log('Mensagem enviada com sucesso. ID:', retorno.id);
  }
});
```
Aqui está um exemplo de documentação para o método `deleteMessage` que pode ser incluído no README do seu projeto:

---

## Deletar Mensagem

### Método: `deleteMessage`

O método `deleteMessage` é usado para solicitar a exclusão de uma mensagem específica em um atendimento.

#### Parâmetros do Método

- `idAtendimento` (Number/String): O identificador único do atendimento onde a mensagem está localizada.
- `mensagem` (Object/String): A mensagem ou o identificador da mensagem a ser deletada.

#### Uso

O método é utilizado invocando `deletarMensagem`, passando o identificador do atendimento e a mensagem ou o identificador da mensagem a ser deletada.

```javascript
// Exemplo de como deletar uma mensagem
deletarMensagem('idDoAtendimento', 'idOuObjetoDaMensagem');
```

#### Estrutura do Emit para WebSocket

```javascript
// Exemplo de estrutura emitida para WebSocket
ws.emit('deleteMessage', {
  protocolo: 'idDoAtendimento',
  msg: 'idOuObjetoDaMensagem',
  action: 'deleteMessage'
});
```


## Aceitar Atendimento

### Evento WebSocket: `aceitarAtendimento`

Emita este evento para aceitar um atendimento atribuído a um agente. Isso notificará o servidor que o agente está iniciando o atendimento.

### Uso

Chame a função `aceitarAtendimento` passando o objeto `atendimento`. A função emite um evento `aceitarAtendimento` através da conexão WebSocket e processa a resposta.

### Exemplo de Código

```javascript
// Função para emitir o evento de aceitar um atendimento via WebSocket
function aceitarAtendimento(atendimento) {
  // Substitua `ws` pela sua instância de WebSocket conectada.
  ws.emit('aceitarAtendimento', {
    protocolo: parseInt(atendimento.id),
    agente: atendimento.agenteFullName
  }, (retorno) => {
    if (retorno.error) {
      console.error('Erro ao aceitar atendimento:', retorno.error);
      // Trate o erro conforme necessário
    } else {
      console.log('Atendimento aceito com sucesso');
      // Prossiga com a lógica de atendimento
    }
  });
}

// Exemplo de como usar a função
aceitarAtendimento({
  id: '1234', // Substitua pelo ID real do atendimento
  agenteFullName: 'Nome do Agente' // Substitua pelo nome completo do agente
});
```

### Parâmetros

- `protocolo`: ID do atendimento a ser aceito.
- `agente`: Nome completo do agente aceitando o atendimento.

##### Resposta
O servidor pode responder com uma confirmação ou um erro, que deve ser manuseado no callback.
---


---
## Recusar Atendimento

### Método: `recusarAtendimento`

Este método é chamado quando um agente precisa recusar um atendimento. Ele envia um evento de socket para notificar o servidor de que o agente recusou o atendimento.

### Como Usar

Para recusar um atendimento, chame a função `recusarAtendimento` passando o objeto `atendimento`.

### Exemplo de Código

```javascript
// Função para recusar um atendimento via WebSocket
function recusarAtendimento(atendimento) {
  // Substitua `ws` pela sua instância de WebSocket conectada.
  // `agente` deve ser um estado reativo ou contexto com informações do agente atual.
  ws.emit('recusarAtendimento', {
    atendimentoId: atendimento.id, // Ou o objeto de atendimento completo, conforme necessário.
    agenteFullName: agente.value.fullname // Nome completo do agente.
  });
}

// Exemplo de uso
recusarAtendimento(atendimentoObjeto);
```

### Parâmetros do Evento

- `atendimentoId`: ID do atendimento que está sendo recusado.
- `agenteFullName`: Nome completo do agente que está recusando o atendimento.

### Tratamento de Eventos e Erros

O evento `recusarAtendimento` não espera uma resposta do servidor por padrão. No entanto, você pode adicionar um callback para processar qualquer confirmação ou erro retornado pelo servidor.

```javascript
ws.emit('recusarAtendimento', { atendimentoId, agenteFullName }, (retorno) => {
  if (retorno && retorno.error) {
    // Tratar erro
    console.error('Erro ao recusar atendimento:', retorno.message);
  } else {
    // Confirmação de recusa de atendimento
    console.log('Atendimento recusado com sucesso');
  }
});
```
---

## Solicitar Modelos de Mensagens (API Oficial WhatsApp)

### Evento WebSocket: `getTemplates`

O evento `getTemplates` é utilizado para requisitar um conjunto de modelos de mensagens associados a uma fila específica e um agente.

### Uso

Chame a função `getTemplates`, passando o objeto `atendimento` que contém as informações necessárias para a requisição dos modelos.

### Exemplo de Uso

```javascript
// Função para solicitar os modelos de mensagens via WebSocket
function getTemplates(atendimento) {
  // Substitua `ws` pela sua instância de WebSocket conectada.
  ws.emit('getTemplates', {
    queue: atendimento.queue,         // Nome da fila do atendimento.
    agente: atendimento.ultimoAgente, // Nome do último agente que atendeu.
    protocolo: atendimento.idQueueChat // ID do chat dentro da fila.
  });
}

// Exemplo de como chamar a função
getTemplates(atendimentoObjeto);
```

### Parâmetros do Evento

- `queue`: Nome da fila à qual o atendimento pertence.
- `agente`: Nome do último agente que gerenciou o atendimento.
- `protocolo`: Identificador único do chat dentro da fila de atendimento.

### Resposta do Servidor

Após emitir o evento `getTemplates`, o servidor pode responder com um array de modelos de mensagens. Essa resposta pode ser capturada adicionando um callback ao emitir o evento.

### Tratamento de Respostas e Erros

O cliente pode ouvir respostas do servidor após a emissão do evento. Um exemplo disso seria:

```javascript
ws.emit('getTemplates', { queue, agente, protocolo }, (templates) => {
  if (templates.error) {
    // Tratar erro
    console.error('Erro ao obter modelos de mensagens:', templates.message);
  } else {
    // Processar modelos de mensagens recebidos
    console.log('Modelos de mensagens recebidos:', templates);
  }
});
```
---
## Enviar Template de Mensagem

### Função: `enviarTemplate`

Esta função é utilizada para enviar um template de mensagem para um cliente específico durante um atendimento. Ela prepara os dados necessários e emite um evento `sendTemplate` via WebSocket.

### Como Usar

```javascript
enviarTemplate(templateObjeto, atendimentoObjeto);
```

- `templateObjeto`: Objeto representando o template de mensagem a ser enviado.
- `atendimentoObjeto`: Objeto representando o atendimento atual.

### Parâmetros

O objeto `template` deve conter os seguintes campos:
- `id`: Identificador único do template.
- `message`: Mensagem de texto do template.
- `file` (opcional): Objeto de arquivo, caso o template inclua um anexo.

O objeto `atendimento` deve conter, no mínimo, os seguintes campos:
- `agente`: Nome do agente responsável pelo atendimento.
- `idQueueChat`: Identificador único do chat dentro da fila de atendimento.
- `queue`: Nome da fila de atendimento.

### Exemplo de Código

```javascript
// Defina o template e os dados do atendimento
const template = {
  id: "1",
  message: "Olá, como posso ajudar?"
  // file: { ... }
};

const atendimento = {
  agente: "Nome do Agente",
  idQueueChat: "123",
  queue: "Suporte"
};

// Envie o template
enviarTemplate(template, atendimento);
```

### Evento WebSocket: `sendTemplate`

Quando chamada, a função `enviarTemplate` emite um evento `sendTemplate` para o servidor via WebSocket, incluindo os dados do template e do atendimento.

```javascript
ws.emit('sendTemplate', {
  ...data
}, (retorno) => {
  console.log('Retorno após enviar template:', retorno);
});
```

### Tratamento de Resposta

- O servidor pode responder com um objeto contendo o status do envio e um identificador da mensagem enviada.
- Use o callback do evento `sendTemplate` para processar a resposta do servidor.

### Notas

- Certifique-se de que a conexão WebSocket esteja estabelecida e que o agente esteja autenticado corretamente antes de tentar enviar um template de mensagem.
- Para templates que incluem arquivos, certifique-se de que o objeto `file` esteja corretamente formatado conforme as especificações do servidor.
---
## Marcar Mensagens como Lidas

### Método WebSocket: `readMessage`

Este método é utilizado para notificar o servidor que o agente leu as ultimas mensagens do atendimento `readMessage` via WebSocket, comunicando ao servidor que as mensagens devem ser marcadas como lidas.

### Uso

Para marcar uma ou mais mensagens como lidas, você deve chamar a função `readMessage`, passando um objeto `data` que identifica as mensagens específicas.

### Exemplo de Uso

```javascript
// Dados das mensagens a serem marcadas como lidas
const messageData = {
  id: "12345", // Identificador do cliente contido na conversa
  // Outros campos conforme necessário
};

// Chama a função para marcar a mensagem como lida
readMessage(messageData);
```

### Parâmetros

- `data`: Objeto contendo as informações das mensagens a serem marcadas como lidas. Deve incluir todos os identificadores e informações necessárias para que o servidor processe a requisição corretamente.

### Evento WebSocket: `readMessage`

A função `readMessage` emite o evento `readMessage` com o seguinte formato de dados:

```json
{
  "id": "Identificador do cliente contido na conversa",
}
```

### Tratamento de Resposta

O servidor pode responder ao evento com um objeto de retorno, indicando o sucesso ou falha da operação.

```javascript
ws.emit('readMessage', data, (retorno) => {
  if (!retorno.error) {
    console.log('Mensagens marcadas como lidas com sucesso.');
    // Atualize a interface do usuário conforme necessário
  } else {
    console.error('Erro ao marcar mensagens como lidas:', retorno.message);
    // Trate o erro conforme necessário
  }
});
```
---

## Transferir Atendimento

### Método WebSocket: `TransferenciaAtendimento` ou `TransfereAtendimentoParaFila`

Este método permite transferir um atendimento atual para outro agente ou para uma fila específica. A ação de transferência é comunicada ao servidor por meio de um evento WebSocket, que pode ser `TransferenciaAtendimento` para transferir diretamente para um agente ou `TransfereAtendimentoParaFila` para transferir o atendimento para outra fila.

### Uso

Chame a função `TransfereAtendimento`, fornecendo os dados necessários para a transferência, o ID do cliente e o objeto de atendimento atual.

### Exemplo de Uso

```javascript
// Dados para transferência do atendimento
const dataTransferencia = {
  nome: "Nome do Agente Destino", // Omitir ou deixar vazio para transferir para fila
  queue: "NomeDaFilaDestino"
};

// ID do cliente e objeto de atendimento atual
const idCliente = "12345";
const atendimentoAtual = {
  idQueueChat: "67890",
  queue: "NomeDaFilaOrigem",
  queueFrom: "NomeDaFilaOrigem", // Opcional
  // Outros dados do atendimento
};

// Chama a função para realizar a transferência
TransfereAtendimento(dataTransferencia, idCliente, atendimentoAtual);
```

### Parâmetros

- `data`: Objeto contendo os dados para a transferência.
  - `nome`: Nome do agente de destino (opcional).
  - `queue`: Nome da fila de destino.
- `idCliente`: Identificador único do cliente.
- `atendimento`: Objeto contendo dados do atendimento atual.

### Evento WebSocket

A função `TransfereAtendimento` emite um dos seguintes eventos WebSocket, dependendo dos dados fornecidos:

1. `TransferenciaAtendimento`: Quando um agente de destino é especificado.
   
   ```javascript
   {
     action: 'TransferenciaAtendimento',
     protocolo: 67890, // ID do chat do atendimento
     queue: "NomeDaFilaDestino",
     queueFrom: "NomeDaFilaOrigem",
     ramal: "RamalDoAgenteOrigem",
     nome: "Nome do Agente Destino"
   }
   ```

2. `TransfereAtendimentoParaFila`: Quando nenhum agente de destino é especificado, e o atendimento deve ser transferido para uma fila.
   
   ```javascript
   {
     action: 'TransfereAtendimentoParaFila',
     protocolo: 67890, // ID do chat do atendimento
     queue: "NomeDaFilaDestino",
     queueFrom: "NomeDaFilaOrigem",
     ramal: "RamalDoAgenteOrigem"
   }
   ```
---
## Adicionar Novo Atendimento

### Método WebSocket: `addAtendimento`

Este método inicia um novo atendimento, enviando um conjunto de informações sobre o cliente e o atendimento para o servidor via WebSocket. É uma parte crucial do sistema de gerenciamento de atendimentos, permitindo a criação de atendimentos de forma dinâmica e em tempo real.

### Estrutura de Dados para `addAtendimento`

Ao chamar a função `addAtendimento`, o objeto `data` deve conter os seguintes campos:

- `queue`: Nome da fila de atendimento onde o atendimento será adicionado.
- `numero`: Número de telefone ou identificador do cliente.
- `fromID`: Identificador único do cliente (`numero`).
- `instancia`: Identificador da instância de atendimento, se houver, ou a instância padrão do agente.
- `name`: Nome do cliente.
- `ativo`: Um booleano indicando se o atendimento está ativo.
- `agente`: Nome completo do agente responsável pelo atendimento.
- `ramal`: Ramal do agente.
- `queueFrom`: Fila de origem do atendimento, geralmente igual a `queue`.
- `queueTypeID`: Tipo de fila do atendimento, identificado numericamente.

### Exemplo de Uso

```javascript
const atendimentoData = {
  queue: fila.value.fila,
  numero: numero.value,
  fromID: numero.value,
  instancia: instancia.value ? instancia.value.fila : agente.value.default_instance,
  name: name.value,
  ativo: true,
  agente: agente.value.fullname,
  ramal: agente.value.ramal,
  queueFrom: instancia.value.fila,
  queueTypeID: fila.value.queue_type
};

// Chama a função para adicionar o atendimento
addAtendimento(atendimentoData);
```

### Envio do Evento WebSocket

```javascript
ws.emit('addAtendimento', atendimentoData, (retorno) => {
  if (retorno.msg) {
    // Tratamento de erro
    console.error('Erro ao adicionar atendimento:', retorno.msg);
  } else {
    // Atendimento adicionado com sucesso
    console.log('Novo atendimento adicionado.');
  }
});
```
---
## Encerrar Atendimento

### Método WebSocket: `EncerrarAtendimento`

Este método é utilizado para comunicar ao servidor que um atendimento específico foi concluído e deve ser marcado como encerrado. A ação de encerrar é realizada emitindo um evento `EncerrarAtendimento` via WebSocket, que carrega os detalhes necessários para identificar e atualizar o status do atendimento.

### Uso

Para encerrar um atendimento, chame a função `encerrarAtendimento`, fornecendo o ID do cliente e o objeto de atendimento relevante.

### Exemplo de Uso

```javascript
// Dados do atendimento a ser encerrado
const idCliente = "12345";
const atendimento = {
  idQueueChat: "67890",
  queue: "FilaDeSuporte",
  // outros dados do atendimento conforme necessário
};

// Chama a função para encerrar o atendimento
encerrarAtendimento(idCliente, atendimento);
```

### Parâmetros

- `idCliente`: Identificador único do cliente associado ao atendimento.
- `atendimento`: Objeto contendo detalhes do atendimento a ser encerrado.

### Estrutura do Objeto Enviado no Evento `EncerrarAtendimento`

```javascript
{
  "dataFim": "Data e hora do encerramento",
  "protocolo": "ID do chat do atendimento",
  "queue": "Nome da fila de atendimento",
  "action": "EncerrarAtendimento"
}
```

### Tratamento de Resposta

O servidor responderá ao evento `EncerrarAtendimento` com um objeto que pode indicar sucesso ou falha na operação:

```javascript
ws.emit('EncerrarAtendimento', aux, (retorno) => {
  if (!retorno.error) {
    // Sucesso no encerramento do atendimento
    console.log('Atendimento encerrado com sucesso.');
  } else {
    // Falha ao encerrar atendimento
    console.error('Falha ao encerrar atendimento:', retorno.error);
  }
});
```

---