const readline = require('readline');
const rl = readline.createInterface({
    input: process.stdin,
    output: process.stdout,
    prompt: 'UraArquivoCLI> '
});

const log = {
    debug: (msg) => { console.debug(msg); rl.prompt(); },
    info: (msg) => { console.log(msg); rl.prompt(); },
    warn: (msg) => { console.warn(msg); rl.prompt(); },
    error: (msg) => { console.error(msg); rl.prompt(); }
}

const UraProcessor = require('./bots/preAtendimento/UraProcessor');

const urafileConf = {
    "fila": "NOME_FILA",
    "arquivo": "./ura.json",
    "expirar": 1,
    "sessao_expirada": "⌛ sua sessão expirou, o processo irá recomeçar 🔃"
}

const urafile = new UraProcessor();
urafile.loadFromDb(1);

//urafile.loadFile(urafileConf.arquivo);
//urafile.setTimeExpire(urafileConf.expirar);
//urafile.setTimeExpireMsg(urafileConf.sessao_expirada);

// Envia uma mensagem para o usuário
urafile.on('UraMsg', dados => {
    //log.debug('URA_DEBUG: UraMsg');
    log.debug(`>> Seria enviado ao cliente: ${dados.msg}`);
});

// Envia o usuário para uma fila
urafile.on('UraGotoFila', async dados => {
    log.debug(`URA_DEBUG: UraGotoFila, Fila: ${dados.fila}, Fila origem: ${dados.route_data.instanceID}`);
    if (dados.msg) {
        log.debug(`URA_DEBUG: ENVIANDO: Msg: ${dados.msg}, Destino: ${dados.idCliente}, Fila: ${dados.route_data.instanceID}, Atendimento: ${dados.route_data.atendimentoId}`);
    }
});

// Envia o usuário para um agente
urafile.on('UraGotoAgente', async dados => {
    log.debug(`UraGotoAgente, Agente: ${dados.agente.nome}`);

    if (dados.msg) {
        log.debug(`URA_DEBUG: ENVIANDO: Msg: ${dados.msg}, Destino: ${dados.idCliente}, Fila: ${dados.route_data.instanceID}, Atendimento: ${dados.route_data.atendimentoId}`);
    }
});

// Encerra o atendimento
urafile.on('UraFim', async dados => {
    // ### Atigamente quando finalizava registrava o atendimento aberto na fila, agora registra como encerrado.
    log.debug(`URA_DEBUG: UraFim`);
    if (dados.msg) {
        log.debug(`URA_FIM: ENVIANDO: Msg: ${dados.msg}, Destino: ${dados.idCliente}, Fila: ${dados.route_data.instanceID}, Atendimento: ${dados.route_data.atendimentoId}`);
    }
});

// Mostra um log de erro
urafile.on('UraErro', dados => {
    //console.error('Erro URAFile');
    //console.error(dados);
    log.error('URA_ERROR: ' + dados.msg);
});

// Mostra um log de debug
urafile.on('UraDebug', dados => {
    let msg = dados.msg;
    if (typeof dados.msg === 'object') msg = JSON.stringify(dados.msg);
    if (dados.debug) msg = `[${dados.debug.toString()}] ${msg}`;
    log.debug('URA_DEBUG: ' + msg);
});

function routerUraFile(texto) {
    log.debug(`URA_DEBUG: ** routerUraFile **, Fila: ${urafileConf.fila}`);

    const msg = {
        idCliente: '1',
        texto: texto,
        instanceID: urafileConf.fila
    }

    urafile.routeMsg(msg.idCliente, msg.texto, { instanceID: msg.instanceID, atendimentoId: 1 });
}

rl.prompt();

rl.on('line', (line) => {
    switch (line.trim()) {
        case 'hello':
            console.log('world!');
            break;
        default:
            //console.log(`Parece que você escreveu '${line.trim()}, vou chamar o UraFile'`);
            routerUraFile(line.trim());
            break;
    }
    rl.prompt();
}).on('close', () => {
    console.log('Até mais!!');
    process.exit(0);
});