'use strict'
const config = require('../config');
const db = require("../util/db");
const moment = require('moment-timezone');
moment.tz.setDefault(config.servidor.timezone);

const log = require('simple-node-logger').createSimpleLogger({ logFilePath: '/var/log/xcontact.log', timestampFormat: 'DD-MM-YYYY HH:mm:ss.SSS' });
const log_level = config.log ? config.log : 'error';
log.setLevel(log_level);


class Fila {

    constructor(nome) {
        this.nome = nome;
        this.agentes = [];
        this.mensagens = [];
        this.atendimentos = [];
        this.atendimentosAguardando = [];
        this.recebidas = 0;
        this.efetuadas = 0;
        this.novosAtendimentos = 0;
        this.mensagensRecebidas = 0;
        this.mensagensEnviadas = 0;
        this.total_espera = 0;
        this.total_falando = 0;
        this.total_abandono = 0;

        // Arrays
        this.ligacoes = [];// Ligações aguardando
        this.ligacoes_atendidas = [];// Ligações que foram atendidas
        this.ligacoes_abandonadas = [];// Ligações que foram abandonadas

        // Para fins de compatibilidade, definimos as variáveis abaixo como o tamanho do array atrelado a informação que ela representa
        this.definirGetter("ligacoes", "aguardando");
        this.definirGetter("ligacoes_atendidas", "atendidas");
        this.definirGetter("ligacoes_abandonadas", "abandonadas");

        this.definirGetterTME();
        this.definirGetterTMA();
        this.definirGetterTMAB();
        this.definirGetterTMAXA();
        this.definirGetterTMAXAB();
        this.definirGetterTMAXE();

    }

    //#region Definição de tempos
    definirGetterTME() {
        Object.defineProperty(this, "tme", {

            enumerable: true,

            set: function () {
                log.warn(`[AMI] SET na variável TME!`);
                console.trace(`[AMI] SET na variável TME!`);
            },

            get: function () {
                let ligacoes = this.ligacoes_atendidas || [];
                ligacoes = ligacoes.filter(l => l.holdtime && l.holdtime != 0);
                let total = ligacoes.reduce((a, b) => a + parseInt(b.holdtime || 0), 0);

                return Math.round(total / ligacoes.length) || 0;
            }

        });
    }

    definirGetterTMA() {
        Object.defineProperty(this, "tma", {

            enumerable: true,

            set: function () {
                log.warn(`[AMI] SET na variável TMA!`);
                console.trace(`[AMI] SET na variável TMA!`);
            },

            get: function () {
                let ligacoes = this.ligacoes_atendidas || [];
                ligacoes = ligacoes.filter(l => l.talktime && l.talktime != 0);
                let total = ligacoes.reduce((a, b) => a + parseInt(b.talktime || 0), 0);

                return Math.round(total / ligacoes.length) || 0;
            }

        });
    }

    definirGetterTMAB() {
        Object.defineProperty(this, "tmab", {

            enumerable: true,

            set: function () {
                log.warn(`[AMI] SET na variável TMAB!`);
                console.trace(`[AMI] SET na variável TMAB!`);
            },

            get: function () {
                let ligacoes = this.ligacoes_abandonadas || [];
                let total = ligacoes.reduce((a, b) => a + parseInt(b.espera || 0), 0);
                let media = Math.round(total / ligacoes.length) || 0;
                // log.info(`[AMI] GET na variável TMAB -> ${media}`);
                return media;
            }

        });
    }

    definirGetterTMAXA() {
        Object.defineProperty(this, "tmaxa", {

            enumerable: true,

            set: function () {
                log.warn(`[AMI] SET na variável TMAXA!`);
                console.trace(`[AMI] SET na variável TMAXA!`);
            },

            get: function () {
                let ligacoes = this.ligacoes_atendidas || [];
                let maximo = Math.max(...ligacoes.map(x => x && (x.talktime || x.falando) ? (x.talktime || x.falando) : 0));
                //let maximo = ligacoes.reduce((a, b) => b && b.talktime > a ? b.talktime : a, 0);
                // log.info(`[AMI] GET na variável TMAXA -> ${maximo}`);
                return maximo;
            }

        });
    }

    definirGetterTMAXAB() {
        Object.defineProperty(this, "tmaxab", {

            enumerable: true,

            set: function () {
                log.warn(`[AMI] SET na variável TMAXAB!`);
                console.trace(`[AMI] SET na variável TMAXAB!`);
            },

            get: function () {
                let ligacoes = this.ligacoes_abandonadas || [];
                // let maximo = Math.max(...ligacoes.map(x => x && x.hold_time ? x.hold_time || 0 : 0));
                let maximo = Math.max(...ligacoes.map(x => x && (x.holdtime || x.espera) ? (x.holdtime || x.espera) : 0));
                //let maximo = ligacoes.reduce((a, b) => b && b.hold_time > a ? b.hold_time : a, 0);
                // log.info(`[AMI] GET na variável TMAXAB -> ${maximo}`);
                return maximo;
            }

        });
    }

    definirGetterTMAXE() {
        Object.defineProperty(this, "tmaxe", {

            enumerable: true,

            set: function () {
                log.warn(`[AMI] SET na variável TMAXE!`);
                console.trace(`[AMI] SET na variável TMAXE!`);
            },

            get: function () {
                let ligacoes = this.ligacoes_atendidas || [];
                // let maximo = Math.max(...ligacoes.map(x => x && x.espera ? x.espera || 0 : 0));
                let maximo = Math.max(...ligacoes.map(x => x && (x.holdtime || x.espera) ? (x.holdtime || x.espera) : 0));
                //let maximo = ligacoes.reduce((a, b) => b && b.espera > a ? b.espera : a, 0);
                // log.info(`[AMI] GET na variável TMAXE -> ${maximo}`);
                return maximo;
            }

        });
    }

    definirGetter(array, property) {
        // Define o getter personalizado
        Object.defineProperty(this, property, {

            enumerable: true,

            set: function () {
                log.warn(`[AMI] SET na variável ${property}!`);
                console.trace(`[AMI] SET na variável ${property}!`);
            },

            get: function () {
                return this && this[array] ? this[array].length : 0;
            }

        });
    }
    //#endregion

    addOrUpdateAgente(agente) {
        let tem = false;
        for (let i = 0; i < this.agentes.length; ++i) {
            if (this.agentes[i].ramal === agente.ramal) {
                //this.agentes[i] = agente;
                const ag = this.agentes[i];
                for (const co in agente) {
                    ag[co] = agente[co];
                }
                tem = true;
            }
        }
        if (!tem) {
            agente.fila = this.nome;
            agente.ligacoes_ativas = [];
            agente.ligacoes = 0;
            agente.finalizou = 0;
            agente.finalizada = 0;
            agente.transferiu = 0;
            agente.atendeu = 0;
            agente.naoatendeu = 0;
            agente.dump = 0;
            agente.efetuada = 0;
            agente.efetuada_naocompletou = 0;
            agente.efetuada_agente_finalizou = 0;
            agente.efetuada_cliente_finalizou = 0;
            this.agentes.push(agente);
        }
    }

    addAgente(agente) {
        agente.fila = this.nome;
        agente.ligacoes_ativas = [];
        agente.ligacoes = 0;
        agente.finalizou = 0;
        agente.finalizada = 0;
        agente.transferiu = 0;
        agente.atendeu = 0;
        agente.naoatendeu = 0;
        agente.dump = 0;
        agente.efetuada = 0;
        agente.efetuada_naocompletou = 0;
        agente.efetuada_agente_finalizou = 0;
        agente.efetuada_cliente_finalizou = 0;
        this.agentes.push(agente);
    }

    removeAgente(agente) {
        for (let i = 0; i < this.agentes.length; ++i) {
            if (this.agentes[i].ramal === agente.ramal) {
                this.agentes.splice(i, 1);
                break;
            }
        }
    }

    getAgente(ramal) {
        for (let i = 0; i < this.agentes.length; ++i) {
            if (this.agentes[i].ramal === ramal) {
                return this.agentes[i];
            }
        }
    }

    addOrUpdateLigacao(ligacao, callback) {
        let tem = false;
        for (let i = 0; i < this.ligacoes.length; ++i) {
            if (this.ligacoes[i].uniqueid === ligacao.uniqueid) {
                this.ligacoes[i] = ligacao;
                tem = true;
            }
        }

        if (!tem) {
            //this.aguardando = parseInt(this.aguardando) + 1;
            this.ligacoes.push(ligacao);
        }

        if (callback) callback(this);
    }

    removeLigacao(ligacao) {
        for (let i = 0; i < this.ligacoes.length; ++i) {
            if (this.ligacoes[i].uniqueid === ligacao.uniqueid) {
                //this.aguardando = parseInt(this.aguardando) === 0 ? 0 : parseInt(this.aguardando) - 1;
                this.ligacoes.splice(i, 1);
                //break;
            }
        }
    }

    getTotalLigacoesAtivas() {
        return this.ligacoes.length;
    }

    getNome() {
        return this.nome;
    }
}

module.exports = Fila;
