const EventEmitter = require('events');
class MyEmitter extends EventEmitter { }
const myEmitter = new MyEmitter();
const eventos = new MyEmitter();
/**
 * Eventos Emitos no emitter eventos
 * AgenteAdicionado
 * AgentePausa
 * AgenteDesconectou
 * AgenteStatus
 * AgenteRemovido
 * AgenteTocando - Agente e ligacao
 * AgenteAtendeu - Agente e ligacao
 * AgenteDesligou - Agente e ligacao
 * AgenteNaoAtendeu - Agente e ligacao
 * 
 */
const AgenteModel = require('../models/Agente');

exports.register = (server, options, next) => {
    const self = this;
    const db = require('../util/db');

    const ws = require('../ws');
    const moment = require('moment-timezone');
    moment.tz.setDefault(options.config.servidor.timezone);
    const uuid = require('uuid/v1');
    const discador = require('../discador');
    const Ligacao = require('./Ligacao');
    const Fila = require('./Fila');
    const Agente = require('./Agente');
    const request = require('request');
    var auxbd = false;
    const XClientes = require('../util/XClients');
    const xc = new XClientes();
    var cron = require('node-cron');
    const Depurador = require('../discador/Depurador');

    //const ChatEvents = require('../webrtc/chat/ChatEvents');
    const event = require('./Events');

    var filas = [];
    var agentesChats = [];
    const opts = {
        logFilePath: '/var/log/xcontact.log',
        timestampFormat: 'DD-MM-YYYY HH:mm:ss.SSS'
    };
    const log = require('simple-node-logger').createSimpleLogger(opts);
    const log_level = options.config.log ? options.config.log : 'error';
    log.setLevel(log_level);
    log.debug('AMI_START: LOG_LEVEL = ' + log_level);

    log.info(`AMI_START: Iniciando conexao com Asterisk no servidor ${options.config.asterisk.host}`);

    var namiLib = require("../plugins/nami");
    var nami = new (require("../plugins/nami").Nami)(
        {
            host: options.config.asterisk.host,
            port: options.config.asterisk.porta,
            username: options.config.asterisk.usuario,
            secret: options.config.asterisk.senha,
            logLevel: 0
        }
    );
    nami.logLevel = 0;
    //nami.logger.setLevel('ERROR');

    nami.on('namiConnectionClose', function (data) {
        log.warn('AMI_CONNECTION_CLOSE: Perdi a conexão com o Asterisk, Reconnecting em 5s...');
        setTimeout(function () { nami.open(); }, 5000);
    });

    nami.on('namiInvalidPeer', function (data) {
        log.warn('AMI_CONNECTION_CLOSE: Tem certeza que está tentando conectar em um AMI?');
        //process.exit();
    });
    nami.on('namiLoginIncorrect', function () {
        log.warn('AMI_CONNECTION_CLOSE: Usuário ou senha recusado.');
        //process.exit();
    });
    nami.on('namiEvent', function (event) {
        // log.debug('AMI_EVENTO: ' + JSON.stringify(event));
        try {
            myEmitter.emit(event.event, event);
        } catch (ex) {
            console.error('AMI_EVENTO: Erro ao tratar evento ' + event.event + ', motivo: ' + ex.toString());
        }
        // Publica todos os eventos do asterisk
        //ws.broadcast(event);
    });


    // Listener de eventos do Chat.
    //var chatevt = new ChatEvents();
    //chatevt.on('ChatEvento', (event) => {
    //  log.info('Recebi evento de chat....');
    //  log.info(event);
    //});

    function limpaEvento(ev) {
        const { lines, EOL, variables, privilege, ...limpo } = ev
        return limpo
    }

    /**
     * Reseta as estatisticas da Fila
     */
    function reset() {
        log.info('Função reset chamada.')
        filas.forEach(fila => {
            fila.ligacoes = [];
            fila.efetuadas = 0;
            fila.recebidas = 0;
            fila.ligacoes_atendidas = [];
            fila.ligacoes_abandonadas = [];
            /* fila.aguardando = 0;
            fila.atendidas = 0;
            fila.abandonadas = 0;
            fila.tme = 0;
            fila.tma = 0;
            fila.tmab = 0; */
            fila.total_espera = 0;
            fila.total_falando = 0;
            fila.total_abandono = 0;
        });

        Depurador.resetar();
        try {
            event.FilaStatus(filas);
        } catch (ex) {
            log.error("AMI_QUEUESTATUSCOMPLETE: Falha ao processar QueueStatusComplete. ERRO: " + ex.toString());
        }
    }

    cron.schedule(options.config.servidor.reset, () => {
        log.info('Rodando RESET agendado.');
        reset();
    });

    async function popularDadosExtrasDoAgente(agente) {
        try {
            // Popula os campos extras do banco de dados para o objeto do Agente
            let dados = await db.query(`SELECT extra1, extra2, tempo_auto_pausa FROM agentes WHERE fullname='${agente.nome}' LIMIT 1`);
            if (dados && dados.rows && dados.rows.length > 0) agente.data = dados.rows[0];
            else agente.data = {};
            console.log(`DADOS DO AGENTE ${agente.nome} -> ${JSON.stringify(agente.data)}`);
        } catch (e) {
            console.error(e);
        }
    }

    // Carrega filas do banco antes de conectar no asterisk
    exports.loadFilas = () => {
        log.info('AMI_LOADFILAS: Carregando filas do banco.');
        db.query("SELECT * FROM public.queue_table")
            .then(res => {
                filas = [];
                for (let i = 0; i < res.rows.length; ++i) {
                    const row = res.rows[i];
                    const fila = new Fila(row.name);
                    fila.id = row.id;
                    fila.context = row.context;
                    fila.timeout = row.timeout;
                    fila.max_ring_agente = row.timeout;
                    fila.announce_frequency = row.announce_frequency;
                    fila.strategy = row.strategy;
                    fila.queue_type = row.queue_type;
                    fila.verificaFeriado = row.verificaFeriado;
                    fila.emailConfig = row.emailConfig;
                    fila.obrigaClassificar = row.obrigaClassificar;
                    fila.id_pesquisa = row.id_pesquisa;
                    fila.ligacoes_abandonadas = [];
                    fila.timerTransbordo = row.queue_youarenext;
                    fila.timerPreAtendimento = row.queue_thereare;
                    fila.filaTransbordo = row.queue_thankyou;
                    filas.push(fila);
                }
                nami.open();
                log.info('AMI_LOADFILAS: Filas recarregadas com sucesso.');
                myEmitter.emit('filasCarregadas', filas);
            })
            .catch(e => {
                log.error('AMI_LOADFILAS: ERRO: ' + e);
            });
    };
    this.loadFilas();

    exports.loadFilaByName = (fila_nome) => {
        log.info(`AMI_LOADFILA: Chamou loadFilaByName para fila ${fila_nome}.`);
        const knex = db.knex('public.queue_table');
        knex.where('name', fila_nome);
        knex.select('*')
            .then(rows => {
                var tem = false;
                for (let i = 0; i < filas.length; ++i) {
                    if (filas[i].nome == fila_nome) {
                        tem = true;
                        // Atualiza apenas dados uteis no sistema para não zerar estatisticas ou recriar objeto
                        const row = rows[0];
                        filas[i].id = row.id;
                        filas[i].context = row.context;
                        filas[i].timeout = row.timeout;
                        filas[i].max_ring_agente = row.timeout;
                        filas[i].announce_frequency = row.announce_frequency;
                        filas[i].strategy = row.strategy;
                        filas[i].queue_type = row.queue_type;
                        filas[i].verificaFeriado = row.verificaFeriado;
                        filas[i].emailConfig = row.emailConfig;
                        filas[i].obrigaClassificar = row.obrigaClassificar;
                        filas[i].id_pesquisa = row.id_pesquisa;
                        filas[i].maximo = parseInt(row.maxlen);
                        filas[i].timerTransbordo = row.queue_youarenext;
                        filas[i].timerPreAtendimento = row.queue_thereare;
                        filas[i].filaTransbordo = row.queue_thankyou;
                        log.info('AMI_LOADFILAS: Fila ' + fila_nome + ' atualizada com sucesso.');
                    }
                }
                if (!tem) {
                    const row = rows[0];
                    const fila = new Fila(row.name);
                    fila.id = row.id;
                    fila.context = row.context;
                    fila.timeout = row.timeout;
                    fila.max_ring_agente = row.timeout;
                    fila.announce_frequency = row.announce_frequency;
                    fila.strategy = row.strategy;
                    fila.queue_type = row.queue_type;
                    fila.verificaFeriado = row.verificaFeriado;
                    fila.emailConfig = row.emailConfig;
                    fila.obrigaClassificar = row.obrigaClassificar;
                    fila.id_pesquisa = row.id_pesquisa;
                    fila.ligacoes_abandonadas = [];
                    fila.timerTransbordo = row.queue_youarenext;
                    fila.timerPreAtendimento = row.queue_thereare;
                    fila.filaTransbordo = row.queue_thankyou;
                    fila.maximo = parseInt(row.maxlen);

                    filas.push(fila);
                    log.info('AMI_LOADFILAS: Fila ' + fila_nome + ' carregada  com sucesso.');
                }
            })
            .catch(e => {
                log.error('AMI_LOADFILAS: ERRO: ' + e);
            });
    };

    exports.getFilas = () => {
        return filas;
    };

    exports.getFila = (fila) => {
        for (var i = 0; i < filas.length; ++i) {
            if (filas[i].nome === fila) {
                return filas[i];
            }
        }
    };

    exports.removeFila = (fila) => {
        for (var i = 0; i < filas.length; ++i) {
            if (filas[i].nome === fila) {
                filas.splice(i, 1);
            }
        }
    };
    exports.getAgenteLogadoFilas = (ramal, callback) => {
        const agentes_filas = [];
        for (let i = 0; i < filas.length; ++i) {
            for (let j = 0; j < filas[i].agentes.length; ++j) {
                if (filas[i].agentes[j].ramal === ramal && filas[i].agentes[j].status === 1) agentes_filas.push(filas[i].nome);
            }
        }
        if (callback) callback(agentes_filas);
        else return agentes_filas;
    };
    exports.getAgenteLogadoByRamal = (ramal, callback) => {
        const agentes_filas = [];
        for (let i = 0; i < filas.length; ++i) {
            for (let j = 0; j < filas[i].agentes.length; ++j) {
                if (filas[i].agentes[j].ramal === ramal) agentes_filas.push(filas[i].agentes[j]);
            }
        }
        if (callback) callback(agentes_filas);
        else return agentes_filas;
    };
    /**
     * Obtem o Agente idependente da Fila que ele estiver
     */
    exports.getAgenteLogadoByFullname = (fullname) => {
        //var agente = undefined;
        for (let i = 0; i < filas.length; ++i) {
            for (let j = 0; j < filas[i].agentes.length; ++j) {
                if (filas[i].agentes[j].nome === fullname) return filas[i].agentes[j];
            }
        }
    };

    exports.getAgentePeloRamal = (ramal) => {
        for (let i = 0; i < filas.length; ++i) {
            const a = filas[i].agentes.find(a => a.ramal === ramal)
            if(a) return a
        }
    };

    exports.getAgentePeloID = (id) => {
        for (let i = 0; i < filas.length; ++i) {
            const a = filas[i].agentes.find(a => a.id == id)
            if(a) return a
        }
    };

    exports.getAgenteLogadoByFilaRamal = (fila, ramal, callback) => {
        let agente = undefined;
        for (let i = 0; i < filas.length; ++i) {
            if (filas[i].nome === fila) {
                for (let j = 0; j < filas[i].agentes.length; ++j) {
                    if (filas[i].agentes[j].ramal === ramal) {
                        agente = filas[i].agentes[j];
                        break;
                    }
                }
            }
        }
        if (callback) callback(agente);
        else return agente;
    };
    exports.getFilasLogadasDoAgente = (agenteRamal) => {
        let filasLogadas = [];

        try {
            filasLogadas = filas.filter(f => f && f.agentes && f.agentes.find(a => a.ramal === agenteRamal));
        } catch (error) {
            console.trace(error);
            log.error(`[AMI] Erro ao buscar as filas do agente -> '${agente}'\n${error}`);
        }

        return filasLogadas;
    };

    exports.getAgentesOnline = (fila, callback) => {
        for (let i = 0; i < filas.length; ++i) {
            if (filas[i].nome === fila) {
                const agentes_online = [];
                for (let j = 0; j < filas[i].agentes.length; ++j) {
                    // if(filas[i].agentes[j].status === 1 && filas[i].agentes[j].chats_ativos.length  < filas[i].agentes[j].max_chat_atende ) {

                    if (filas[i].agentes[j].status === 1 && filas[i].agentes[j].pausa === 0) {
                        if (options.config.agente_descanso) {
                            let now = moment.tz(options.config.servidor.timezone);
                            const tempo_online = moment.duration(now.diff(filas[i].agentes[j].DataStatus));

                            if (tempo_online.asSeconds() >= options.config.agente_descanso) {
                                agentes_online.push(filas[i].agentes[j]);
                            } else {
                                log.info(`[AMI] Agente (${filas[i].agentes[j].ramal}) em descanso, tempo online ${tempo_online.asSeconds()}s, descanso definido em ${options.config.agente_descanso}s`);
                            }
                        } else {
                            agentes_online.push(filas[i].agentes[j]);
                        }
                    } else {
                        log.warn(`[AMI] O Agente(${filas[i].agentes[j].ramal}) não está com o status de disponível ou está em pausa!`);
                        log.warn(filas[i].agentes[j]);
                        //log.warn('O AGENTE  ESTA COM O NUMERO MAXIMO DE ATENDIMENTOS SIMULTANEOS  OU NAO ESTA DISPONIVEL****************');
                    }
                }

                if (callback) callback(agentes_online);
                else return agentes_online;
            }
        }
    };

    exports.getQtdAgentesOnline = (fila, callback) => {
        for (let i = 0; i < filas.length; ++i) {
            if (filas[i].nome === fila) {
                let total = 0;
                for (let j = 0; j < filas[i].agentes.length; ++j) {
                    if (filas[i].agentes[j].status === 1) total++;
                }
                if (callback) callback(total);
                else return total;
            }
        }
    };

    function ReplaceIntegracaoVars(url, dados) {
        url = url.replace(/X_AGENTE_NOME/gi, dados.nome);
        url = url.replace(/X_AGENTE_RAMAL/gi, dados.ramal);
        url = url.replace(/X_AGENTE_FILA/gi, dados.fila);
        url = url.replace(/X_AGENTE_STATUS_COD/gi, dados.status);
        url = url.replace(/X_AGENTE_STATUS_NOME/gi, dados.StatusNome);
        url = url.replace(/X_AGENTE_PAUSA_MOTIVO/gi, dados.pausa_motivo);
        url = url.replace(/X_AGENTE_DATA_STATUS/gi, dados.DataStatus);
        return url;
    }

    function NotificaIntegracaoStatus(dados) {
        log.info('INTEGRACAO_STATUS_AGENTE: Iniciando chamada de integracao de status de agente.');
        let replaced = ReplaceIntegracaoVars(JSON.stringify(options.config.integracao_status_agente.request), dados);
        let integracao = JSON.parse(replaced);

        request(integracao, (error, response, body) => {
            if (error) {
                log.error('INTEGRACAO_STATUS_AGENTE: ERRO: ' + JSON.stringify(error));
            }
            else {
                log.info('INTEGRACAO_STATUS_AGENTE: Retornou StatusCode: ' + (response && response.statusCode));
                log.info('INTEGRACAO_STATUS_AGENTE: Body: ' + body);
            }
        });
    }

    function groupBy(ar, col) {
        var r = [], o = {}, array = JSON.parse(JSON.stringify(ar));
        array.forEach(function (a) {
            if (!o[a[col]]) {
                //o[a[col]] = {};
                //o[a[col]][col] = a[col];
                o[a[col]] = a;
                o[a[col]]['filas'] = [];
                r.push(o[a[col]]);
            }
            o[a[col]]['filas'].push(a['fila']);
        });
        return r;
    }

    exports.getAgentes = (filtros, callback) => {
        const agentes = [];
        var filtro = [];

        try { filtro = JSON.parse(filtros); } catch (error) { console.trace(error); }

        for (let i = 0; i < filas.length; ++i) {
            for (let x = 0; x < filtro.length; x++) {
                if (filas[i].nome == filtro[x].fila) {
                    for (let j = 0; j < filas[i].agentes.length; ++j) {
                        agentes.push(filas[i].agentes[j]);
                        //  if(filas[i].agentes[j].status === 1) agentes_online.push(filas[i].agentes[j]);
                    }
                }
            }
            //agentes.push(filas[i].agentes);

        }
        if (filtros && filtros.agrupar) {
            var agentes2 = groupBy(agentes, 'ramal');
            if (callback) callback(agentes2);
            else return agentes2;
        }
        else {
            if (callback) callback(agentes);
            else return agentes;
        }
    };

    /**
     * Envia o Action QueueStatus para o Asterisk, se callback for uma function retorna o resultado.
     */
    exports.QueueStatus = (fila, callback) => {
        nami.send(new namiLib.Actions.QueueStatus(fila), function (response) {
            if (typeof callback === "function") {
                callback(response);
            }
        });
    };

    exports.queueShow = (fila) => {
        const qa = new namiLib.Actions.QueueStatus(fila);
        nami.send(qa);
    };

    exports.send = (dados, callback) => {
        if (typeof callback === "function") {
            nami.send(dados, callback);
        }
        else {
            nami.send(dados);
        }
    };

    exports.QueueAdd = function (dados, callback) {
        //Não pode ser Array de filas pelo Retorno Que não é Sync
        const qa = new namiLib.Actions.QueueAdd(dados.ramal, dados.fila, dados.pausado, dados.nome, dados.penalty);
        nami.send(qa, (resposta) => {
            if (callback) callback(resposta);
            else return resposta;
        });
    };

    exports.QueueRemove = function (dados, callback) {
        //Não pode ser Array de filas pelo Retorno Que não é Sync
        const qa = new namiLib.Actions.QueueRemove(dados.ramal, dados.fila);
        nami.send(qa, (resposta) => {
            if (callback) callback(resposta);
            else return resposta;
        });
    };

    exports.QueuePause = function (dados, callback) {
        //Não pode ser Array de filas pelo Retorno Que não é Sync
        const qa = new namiLib.Actions.QueuePause(dados.ramal, dados.fila, dados.motivo);
        nami.send(qa, (resposta) => {
            if (callback) callback(resposta);
            else return resposta;
        });
    };

    exports.QueueUnpause = function (dados, callback) {
        //Não pode ser Array de filas pelo Retorno Que não é Sync
        const qa = new namiLib.Actions.QueueUnpause(dados.ramal, dados.fila);
        nami.send(qa, (resposta) => {
            if (callback) callback(resposta);
            else return resposta;
        });
    };

    exports.Discar = function (dados, callback) {
        const ac = new namiLib.Actions.Originate();
        ac.channel = dados.origem;
        ac.exten = dados.destino;
        ac.callerid = dados.destino_nome ? dados.destino_nome + " <" + dados.destino + ">" : dados.destino;
        ac.priority = 1;
        ac.context = dados.rota;
        ac.timeout = dados.max_chamar ? dados.max_chamar * 1000 : 30 * 1000;
        ac.channelid = new Date().getTime();
        ac.async = true;
        let ar = [];
        ar.push('__SIPADDHEADER51=X-UNIQUE-ID:' + ac.channelid);
        if (dados.campos && Object.keys(dados.campos).length > 0) {
            let id = 2;
            Object.keys(dados.campos).forEach(function (key) {
                ar.push('__SIPADDHEADER5' + id + '=X-' + key + ':' + encodeURIComponent(dados.campos[key]));
                ++id;
            });
        }
        ac.variable = ar.join(",");

        nami.send(ac, (resposta) => {
            resposta.callid = ac.channelid;
            if (callback) callback(resposta);
            else return resposta;
        });
    };
    exports.Conferencia = function (dados, callback) {
        const ac = new namiLib.Actions.Originate();
        ac.channel = "Local/" + dados.destino + "@" + dados.rota;
        ac.application = "Confbridge";
        ac.data = dados.sala + ",conf_webrtc,user_webrtc";
        ac.timeout = dados.max_chamar ? dados.max_chamar * 1000 : 30 * 1000;
        ac.channelid = new Date().getTime();
        ac.async = true;
        let ar = [];
        ar.push('__SIPADDHEADER51=X-UNIQUE-ID:' + ac.channelid);
        if (dados.campos && Object.keys(dados.campos).length > 0) {
            let id = 2;
            Object.keys(dados.campos).forEach(function (key) {
                console.log(key);
                ar.push('__SIPADDHEADER5' + id + '=X-' + key + ':' + encodeURIComponent(dados.campos[key]));
                ++id;
            });
        }
        ac.variable = ar.join(",");
        nami.send(ac, (resposta) => {
            resposta.callid = ac.channelid;
            if (callback) {
                ws.broadcast({ dados: dados, event: 'ConferenciaAtt', errorMsg: null });
                callback(resposta);
            }
            else return resposta;
        });
    };

    exports.Desligar = function (dados, callback) {
        console.log(dados);

        const ac = new namiLib.Actions.Hangup();
        ac.channel = dados.destino ? '/^' + dados.destino + '-.*$/' : dados.canal;
        ac.cause = dados.causa ? dados.causa : 16;
        nami.send(ac, (resposta) => {
            console.log(resposta)
            resposta.callid = ac.channelid;
            if (callback) callback(resposta);
            else return resposta;
        });


    };

    exports.CoreShowChannels = function () {
        return new Promise((resolve, reject) => {
            log.debug(`[AMI] chamando CoreShowChannels`);
            const qa = new namiLib.Actions.CoreShowChannels();
            nami.send(qa, (result) => {
                if (result.response == "Success") {
                    resolve(result);
                }
                else {
                    reject(result.message);
                }
            });
        });
    };

    exports.SipShowPeer = function (peer) {
        return new Promise((resolve, reject) => {
            log.debug(`[AMI] chamando sipshowpeer ${peer}`);
            const qa = new namiLib.Actions.SipShowPeer();
            qa.peer = peer;
            nami.send(qa, (result) => {
                if (result.response == "Success") {
                    resolve(result);
                }
                else {
                    reject(result.message);
                }
            });
        });
    };

    exports.TransferirComConsulta = function ({ CanalOrigem, NumeroDestino, ContextoDestino } = {}) {
        return new Promise((resolve, reject) => {
            log.debug(`[AMI] chamando AttendedTransfer ${NumeroDestino}`);
            const qa = new namiLib.Actions.AttendedTransfer();
            qa.ActionID = uuid();
            qa.Channel = CanalOrigem;
            qa.Exten = NumeroDestino;
            qa.Context = ContextoDestino;

            log.debug(`Enviando Redirect; ActionID: ${qa.ActionID}, Canal: ${CanalOrigem}, Destino: ${NumeroDestino}`);

            nami.send(qa, (result) => {
                if (result.response == "Success") {
                    resolve(result);
                }
                else {
                    reject(result.message);
                }
            });
        });
    };

    exports.getConfig = function ({ arquivo, categoria } = {}) {
        return new Promise((resolve, reject) => {
            log.debug(`[AMI] chamando GetConfigJson ${arquivo}`);
            const qa = new namiLib.Actions.GetConfigJson();
            qa.Filename = arquivo;
            if (categoria) qa.Category = categoria;

            nami.send(qa, (result) => {
                if (result.response == "Success") {
                    resolve(JSON.parse(result.json));
                }
                else {
                    reject(result.message);
                }
            });
        });
    };

    /**
     * Atualiza um valor de um parametro em uma categoria de um arquivo.
     * @param {*} param0 
     * @returns 
     */
    exports.updateConfig = function ({ arquivo, categoria, acao, parametro, valor, match, reload } = {}) {
        return new Promise((resolve, reject) => {
            log.debug(`[AMI] chamando updateConfig ${arquivo} com acao ${acao}`);

            const qa = new namiLib.Actions.UpdateConfig();
            qa.SrcFilename = arquivo;
            qa.DstFilename = arquivo;
            qa.Reload = reload ? reload : false;
            qa['Action-000000'] = acao;
            qa['Cat-000000'] = categoria;
            if (parametro) qa['Var-000000'] = parametro;
            if (valor) qa['Value-000000'] = valor;
            if (match) qa['Match-000000'] = match;

            nami.send(qa, (result) => {
                if (result.response == "Success") {
                    resolve(result);
                }
                else {
                    reject(result.message);
                }
            });
        });
    };

    exports.updateTroncoConfig = function ({ tronco, registro_novo, registro_antigo, dados, isNew } = {}) {

        return new Promise((resolve, reject) => {
            console.log(`[AMI] chamando updateTroncoConfig`);

            // Remove registro
            if (registro_antigo && !registro_novo) {
                const sip = new namiLib.Actions.UpdateConfig();
                sip.SrcFilename = 'sip.conf';
                sip.DstFilename = 'sip.conf';
                sip.Reload = false;
                sip['Action-000000'] = 'Delete';
                sip['Cat-000000'] = 'general';
                sip['Var-000000'] = 'register';
                sip['Value-000000'] = registro_antigo;
                sip['Match-000000'] = registro_antigo;
                nami.send(sip);
            }
            // Atualiza registro
            else if (registro_novo && registro_antigo && registro_novo !== registro_antigo) {
                const sip = new namiLib.Actions.UpdateConfig();
                sip.SrcFilename = 'sip.conf';
                sip.DstFilename = 'sip.conf';
                sip.Reload = false;
                sip['Action-000000'] = 'Update';
                sip['Cat-000000'] = 'general';
                sip['Var-000000'] = 'register';
                sip['Value-000000'] = registro_novo;
                sip['Match-000000'] = registro_antigo;
                nami.send(sip);
            }
            // Novo registro
            else if (registro_novo && !registro_antigo) {
                const sip = new namiLib.Actions.UpdateConfig();
                sip.SrcFilename = 'sip.conf';
                sip.DstFilename = 'sip.conf';
                sip.Reload = false;
                sip['Action-000000'] = 'Append';
                sip['Cat-000000'] = 'general';
                sip['Var-000000'] = 'register';
                sip['Value-000000'] = registro_novo;
                nami.send(sip);
            }

            // Atualiza sip_trunk.conf
            const qa = new namiLib.Actions.UpdateConfig();
            qa.SrcFilename = 'sip_trunk.conf';
            qa.DstFilename = 'sip_trunk.conf';
            qa.Reload = 'chan_sip.so';

            // se for update Limpa a categoria
            if (!isNew) {
                qa['Action-000000'] = 'EmptyCat';
                qa['Cat-000000'] = tronco;
            }
            else {
                qa['Action-000000'] = 'NewCat';
                qa['Cat-000000'] = tronco;
            }

            for (let i = 0; i < dados.length; i++) {
                let valor = dados[i];
                qa[`Action-${(i + 1).toString().padStart(6, '0')}`] = 'Append';
                qa[`Cat-${(i + 1).toString().padStart(6, '0')}`] = tronco;
                qa[`Var-${(i + 1).toString().padStart(6, '0')}`] = valor.parametro;
                qa[`Value-${(i + 1).toString().padStart(6, '0')}`] = valor.valor;
            }

            nami.send(qa, (result) => {
                if (result.response == "Success") {
                    resolve(result);
                }
                else {
                    reject(result.message);
                }
            });
        });
    };

    exports.deleteTroncoConfig = function ({ tronco, registro } = {}) {

        return new Promise((resolve, reject) => {
            console.log(`[AMI] chamando deleteTroncoConfig`);

            // Remove registro
            if (registro) {
                const sip = new namiLib.Actions.UpdateConfig();
                sip.SrcFilename = 'sip.conf';
                sip.DstFilename = 'sip.conf';
                sip.Reload = false;
                sip['Action-000000'] = 'Delete';
                sip['Cat-000000'] = 'general';
                sip['Var-000000'] = 'register';
                sip['Value-000000'] = registro;
                sip['Match-000000'] = registro;
                nami.send(sip);
            }

            // Atualiza sip_trunk.conf
            const qa = new namiLib.Actions.UpdateConfig();
            qa.SrcFilename = 'sip_trunk.conf';
            qa.DstFilename = 'sip_trunk.conf';
            qa.Reload = 'chan_sip.so';

            qa['Action-000000'] = 'DelCat';
            qa['Cat-000000'] = tronco;

            nami.send(qa, (result) => {
                if (result.response == "Success") {
                    resolve(result);
                }
                else {
                    reject(result.message);
                }
            });
        });
    };

    exports.criaContextoConfig = function (contexto) {
        return new Promise((resolve, reject) => {
            const qa = new namiLib.Actions.UpdateConfig();
            qa.SrcFilename = 'extensions_custom.conf';
            qa.DstFilename = 'extensions_custom.conf';
            qa.Reload = 'pbx_config.so';
            qa['Action-000000'] = 'NewCat';
            qa['Cat-000000'] = contexto;

            qa['Action-000001'] = 'Append';
            qa['Cat-000001'] = contexto;
            qa['Var-000001'] = 'switch';
            qa['Value-000001'] = 'realtime';

            nami.send(qa, (result) => {
                if (result.response == "Success") {
                    resolve(result);
                }
                else {
                    reject(result.message);
                }
            });
        });
    };

    /**
     * Obtem os dados de uma variável do canal.
     * @param {*} param0 
     * @returns 
     */
    exports.getVar = function ({ Channel, Variable } = {}) {
        return new Promise((resolve, reject) => {
            log.debug(`[AMI] chamando getVar ${Variable} para Channel ${Channel}`);
            const qa = new namiLib.Actions.GetVar();
            qa.Channel = Channel;
            qa.Variable = Variable;
            nami.send(qa, (result) => {
                if (result.response == "Success") {
                    resolve(result);
                }
                else {
                    reject(result.message);
                }
            });
        });
    };

    exports.QueueStatusV2 = function ({ Queue, Member } = {}) {
        return new Promise((resolve, reject) => {
            log.debug(`[AMI] chamando QueueStatusV2 para fila ${Queue}, agente ${Member}`);
            const qa = new namiLib.Actions.QueueStatus();
            if (Queue) qa.Queue = Queue;
            if (Member) qa.Member = Member;
            nami.send(qa, (result) => {
                if (result.response == "Success") {
                    resolve(result);
                }
                else {
                    reject(result.message);
                }
            });
        });
    };

    exports.nami = nami;

    myEmitter.on('FullyBooted', (event) => {
        log.info('AMI_FULLYBOOTED: #### Conectado no asterisk #####');
        log.info('AMI_FULLYBOOTED: Asterisk Uptime: ' + event.uptime);
        log.info('AMI_FULLYBOOTED: Asterisk ultimo reload: ' + event.lastreload);
        setTimeout(function () {
            self.queueShow();
        }, 10000);
    });

    // Acontece quando o comando queueShow é chamado pela API (Start do serviço)
    myEmitter.on('QueueParams', (ev) => {
        ev = limpaEvento(ev)
        log.info('AMI_QUEUEPARAMS: Tratando QueueParams');
        try {
            const Evento = {
                Queue: ev.queue,
                Max: ev.max,
                Strategy: ev.strategy,
                Calls: parseInt(ev.calls),
                Holdtime: parseInt(ev.holdtime),
                TalkTime: parseInt(ev.talktime),
                Completed: parseInt(ev.completed),
                Abandoned: parseInt(ev.abandoned),
                ServiceLevel: ev.servicelevel,
                ServicelevelPerf: ev.servicelevelperf,
                ServicelevelPerf2: ev.servicelevelperf2,
                Weight: parseInt(ev.weight)
            };
            let add = true;
            // Percorre fila
            for (let i = 0; i < filas.length; ++i) {
                if (filas[i].nome === Evento.Queue) {
                    add = false;
                    filas[i].maximo = Evento.Max;
                    filas[i].estrategia = Evento.Strategy;
                    filas[i].efetuadas = 0;
                    filas[i].total_espera = filas[i].tme;
                    filas[i].total_falando = filas[i].tma;
                    filas[i].total_abandono = 0;
                    break;
                }
            }
            // Nova fila
            if (add) {
                const fila = new Fila(Evento.Queue);
                fila.maximo = Evento.Max;
                fila.estrategia = Evento.Strategy;
                fila.efetuadas = 0;
                fila.total_espera = fila.tme;
                fila.total_falando = fila.tma;
                fila.total_abandono = 0;
                filas.push(fila);
            }
        } catch (ex) {
            log.error('AMI_QUEUEPARAMS: Falha ao atualizar as filas. ERRO: ' + ex.toString());
        }
    });

    // Também Acontece quando o comando queueShow é chamado pela API (Start do serviço)
    myEmitter.on('QueueMember', async (ev) => {
        ev = limpaEvento(ev)
        // console.log(ev)
        try {
            const Evento = {
                Queue: ev.queue,
                Name: ev.name,
                Location: ev.location,
                StateInterface: ev.stateinterface,
                Membership: ev.membership,
                Penalty: ev.penalty,
                CallsTaken: parseInt(ev.callstaken),
                LastCall: parseInt(ev.lastcall),
                LastPause: parseInt(ev.lastpause),
                InCall: parseInt(ev.incall),
                Status: parseInt(ev.status),
                Paused: parseInt(ev.paused),
                PausedReason: ev.pausedreason
            };
            for (let i = 0; i < filas.length; ++i) {
                if (filas[i].nome === Evento.Queue) {
                    var chat = [];
                    const agente = filas[i].getAgente(Evento.Location);
                    if (agente) {
                        // Só atualiza se for problema, pois isso afeta o multifila
                        agente.status = Evento.Paused === 1 ? 10 : Evento.Status;
                        agente.StatusNome = agente.getStatusNome();
                        agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
                        agente.StatusDate = new Date();
                        agente.ligacoes = Evento.CallsTaken;
                        agente.tipo = Evento.Membership;
                        agente.ultimaligacao = Evento.LastCall;
                        agente.ultimapausa = Evento.LastPause;
                        agente.pausa = Evento.Paused;
                        agente.pausa_motivo = Evento.Reason;
                        agente.motivo = Evento.Reason;

                    } else {

                        const agente = new Agente(Evento.Location);
                        agente.nome = Evento.Name;
                        agente.status = Evento.Paused === 1 ? 10 : Evento.Status;
                        agente.StatusNome = agente.getStatusNome();
                        agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
                        agente.StatusDate = new Date();
                        agente.ligacoes = Evento.CallsTaken;
                        agente.tipo = Evento.Membership;
                        agente.ultimaligacao = Evento.LastCall;
                        agente.ultimapausa = Evento.LastPause;
                        agente.pausa = Evento.Paused;
                        agente.pausa_motivo = Evento.Reason;
                        agente.motivo = Evento.Reason;
                        agente.penalizacoes = Evento.Penalty;
                        agente.fila = Evento.Queue;
                        agente.time = 0;

                        // Load do agente do banco se for agente dynamico, agente statico não tem na tabela de agentes
                        if(ev.membership === 'dynamic') {
                            let ag = await AgenteModel.fetchByFullname(agente.nome);
                            agente.id = ag.id;
                            agente.max_chat_atende = ag.max_chat_atende;
                            agente.edita_rota = ag.edita_rota;
                            agente.visualiza_todos_clientes = ag.visualiza_todos_clientes;
                            agente.atalho_url = ag.atalho_url;
                            agente.abrir_url = ag.abrir_url;
                            agente.tempo_maximo_sessao = ag.tempo_maximo_sessao;
                            agente.tempo_auto_pausa = ag.tempo_auto_pausa;
                            agente.edita_config = ag.edita_config;
                            agente.default_instance = ag.default_instance;
                            agente.pode_selecionar_instancias = ag.pode_selecionar_instancias;
                        }

                        filas[i].addAgente(agente);

                    }
                }
            }
        } catch (ex) {
            log.error("'AMI_QUEUEMEMBER: Falha ao processar QueueMember. ERRO: " + ex.toString());
        }
    });

    // Acontece no final do QueueStatus, uma ótima hora para reportar as filas!!
    myEmitter.on('QueueStatusComplete', (ev) => {
        log.info('AMI_QUEUESTATUSCOMPLETE: Tratando QueueStatusComplete');

        event.FilaStatus(filas);
    });

    myEmitter.on('QueueCallerJoin', (ev) => {
        ev = limpaEvento(ev)
        log.info('AMI_QUEUECALLERJOIN: Tratando Join');
        ws.broadcastEvent(`QueueCallerJoin`, ev);

        try {
            const l = new Ligacao(ev.linkedid);
            l.origem = ev.calleridnum;
            l.origem_nome = ev.calleridname;
            l.sentido = 'Entrante';
            l.fila = ev.queue;
            l.posicao = parseInt(ev.position);
            l.total = parseInt(ev.count);
            l.inicio = moment.tz(options.config.servidor.timezone);

            for (var i = 0; i < filas.length; ++i) {
                if (filas[i].nome === l.fila) {
                    filas[i].addOrUpdateLigacao(l, (fila) => {
                        const Unifila = [];
                        Unifila.push(fila);
                        event.FilaStatus(Unifila);
                    });
                    break;
                }
            }
        } catch (ex) {
            log.error("AMI_QUEUECALLERJOIN: Falha ao processar QueueCallerJoin. ERRO: " + ex.toString());
        }
    });

    myEmitter.on('QueueCallerAbandon', (ev) => {
        ev = limpaEvento(ev)
        log.info('AMI_QUEUECALLERABANDON: Tratando QueueCallerAbandon');
        ws.broadcastEvent(`QueueCallerAbandon`, ev);

        try {
            myEmitter.emit('Abandono', ev);
            const l = new Ligacao(ev.linkedid);
            l.origem = ev.calleridnum;
            l.origem_nome = ev.calleridname;
            l.sentido = 'Entrante';
            l.fila = ev.queue;
            l.posicao = parseInt(ev.position);
            l.posicao_original = parseInt(ev.originalposition);
            l.total = parseInt(ev.count);
            l.espera = parseInt(ev.holdtime);
            l.fim = moment.tz(options.config.servidor.timezone);
            l.abandono = true;

            for (let i = 0; i < filas.length; ++i) {
                if (filas[i].nome === l.fila) {
                    // Discarta transbordos
                    if (ev.reason && ev.reason === 'NORMAL') {
                        //filas[i].abandonadas = filas[i].abandonadas ? filas[i].abandonadas + 1 : 1;
                        //filas[i].tmab = Math.round(filas[i].total_abandono / filas[i].abandonadas);
                        filas[i].total_abandono = filas[i].total_abandono ? filas[i].total_abandono + l.espera : l.espera;

                        if (!filas[i].ligacoes_abandonadas) filas[i].ligacoes_abandonadas = [];
                        filas[i].ligacoes_abandonadas.push(l);
                        log.info("[AMI] Nova ligação abandonada! linkedid->" + (l.uniqueid || ev.linkedid));
                    }

                    filas[i].removeLigacao(l);

                    // Ve se a ligacao ja nao foi para um agente.
                    for (let j = 0; j < filas[i].agentes.length; ++j) {
                        const agente = filas[i].agentes[j];
                        const lig = new Ligacao(ev.linkedid);
                        lig.fim = 'FilaAbandono';
                        agente.removeLigacao(lig, () => {
                            if (agente.ligacoes_ativas.length === 0) {
                                agente.status = agente.pausa ? 10 : 1;
                                agente.StatusNome = agente.getStatusNome();
                                agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
                                agente.StatusDate = new Date();
                            }
                        });
                    }

                    // Processamento do discador
                    try {
                        discador.ProcessaAbandono(ev);
                    } catch (ex) {
                        log.error("AMI_QUEUECALLERABANDON: Falha ao processar discador.ProcessaAbandono. ERRO: " + ex.toString());
                    }

                    const Unifila = [];
                    Unifila.push(filas[i]);
                    event.FilaStatus(Unifila);

                    break;
                }
            }
        } catch (ex) {
            log.error("AMI_QUEUECALLERABANDON: Falha ao processar QueueCallerAbandon. ERRO: " + ex.toString());
        }
    });

    myEmitter.on('QueueCallerLeave', (ev) => {
        ev = limpaEvento(ev)
        log.info('AMI_QUEUECALLERLEAVE: Tratando QueueCallerLeave');

        try {
            const l = new Ligacao(ev.linkedid);
            l.origem = ev.calleridnum;
            l.origem_nome = ev.calleridname;
            l.sentido = 'Entrante';
            l.fila = ev.queue;
            l.posicao = parseInt(ev.position);
            l.total = parseInt(ev.count);
            l.fim = moment.tz(options.config.servidor.timezone);

            for (let i = 0; i < filas.length; ++i) {
                if (filas[i].nome === l.fila) {
                    filas[i].removeLigacao(l);
                    const Unifila = [];
                    Unifila.push(filas[i]);
                    event.FilaStatus(Unifila);
                    break;
                }
            }
        } catch (ex) {
            log.error("AMI_QUEUECALLERLEAVE: Falha ao processar QueueCallerLeave. ERRO: " + ex.toString());
        }
    });

    myEmitter.on('QueueMemberAdded', async (ev) => {
        ev = limpaEvento(ev)

        try {
            const QueueMemberAdded = {
                Queue: ev.queue,
                MemberName: ev.membername,
                Interface: ev.interface,
                StateInterface: ev.stateinterface,
                Membership: ev.membership,
                Penalty: ev.penalty,
                CallsTaken: parseInt(ev.callstaken),
                LastCall: parseInt(ev.lastcall),
                LastPause: parseInt(ev.lastpause),
                InCall: parseInt(ev.incall),
                Status: parseInt(ev.status),
                Paused: parseInt(ev.paused),
                PausedReason: ev.pausedreason,
                Ringinuse: parseInt(ev.ringinuse)
            };
            for (let i = 0; i < filas.length; ++i) {

                if (filas[i].nome === QueueMemberAdded.Queue) {
                    let chat = [];
                    const agente = new Agente(QueueMemberAdded.Interface);
                    agente.nome = QueueMemberAdded.MemberName;
                    agente.status = QueueMemberAdded.Paused ? 10 : QueueMemberAdded.Status;
                    agente.StatusNome = agente.getStatusNome();
                    agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
                    agente.StatusDate = new Date();
                    agente.ligacoes = QueueMemberAdded.CallsTaken;
                    agente.tipo = QueueMemberAdded.Membership;
                    agente.DataUltimoAtendimento = 0
                    agente.ultimaligacao = QueueMemberAdded.LastCall;
                    agente.ultimapausa = QueueMemberAdded.LastPause;
                    agente.pausa = QueueMemberAdded.Paused;
                    agente.pausa_motivo = QueueMemberAdded.PausedReason;
                    agente.motivo = agente.pausa_motivo;
                    agente.penalizacoes = QueueMemberAdded.Penalty;
                    agente.fila = QueueMemberAdded.Queue;
                    agente.time = 0;

                    if(ev.membership === 'dynamic') {
                        let ag = await AgenteModel.fetchByFullname(agente.nome);
                        agente.id = ag.id;
                        agente.max_chat_atende = ag.max_chat_atende;
                        agente.edita_rota = ag.edita_rota;
                        agente.visualiza_todos_clientes = ag.visualiza_todos_clientes;
                        agente.atalho_url = ag.atalho_url;
                        agente.abrir_url = ag.abrir_url;
                        agente.tempo_maximo_sessao = ag.tempo_maximo_sessao;
                        agente.tempo_auto_pausa = ag.tempo_auto_pausa;
                        agente.edita_config = ag.edita_config;
                        agente.default_instance = ag.default_instance;
                        agente.pode_selecionar_instancias = ag.pode_selecionar_instancias;
                    }

                    filas[i].addOrUpdateAgente(agente);
                    event.AgenteAdicionado(agente);
                    // Precisa mudar isso no futuro, fazer esse event ouvir o eventos
                    eventos.emit('AgenteAdicionado', agente)

                    if (options.config.integracao_status_agente && options.config.integracao_status_agente.ativada) {
                        // Processamento de integração de status de agente.
                        try {
                            NotificaIntegracaoStatus(agente);
                        } catch (ex) {
                            log.error("INTEGRACAO_STATUS_AGENTE: Falha ao processar NotificaIntegracaoStatus(). ERRO: " + ex.toString());
                        }
                    }

                    // Processamento do discador
                    try {
                        discador.ProcessaCampanhasByFila(filas[i].nome);
                    } catch (ex) {
                        log.error("AMI_QUEUEMEMBERADD: Falha ao processar discador.ProcessaCampanhasByFila. ERRO: " + ex.toString());
                    }

                    break;
                }
            }
        } catch (ex) {
            log.error("AMI_QUEUEMEMBERADD: Falha ao processar QueueMemberAdded. ERRO: " + ex.toString());
        }
    });

    myEmitter.on('QueueMemberPause', async (ev) => {
        ev = limpaEvento(ev)
        try {
            const Evento = {
                Queue: ev.queue,
                MemberName: ev.membername,
                Interface: ev.interface,
                StateInterface: ev.stateinterface,
                Membership: ev.membership,
                Penalty: ev.penalty,
                CallsTaken: parseInt(ev.callstaken),
                LastCall: parseInt(ev.lastcall),
                LastPause: parseInt(ev.lastpause),
                InCall: parseInt(ev.incall),
                Status: parseInt(ev.status),
                Paused: parseInt(ev.paused),
                PausedReason: ev.pausedreason,
                Ringinuse: parseInt(ev.ringinuse),
                Reason: ev.reason,
            };
            for (let i = 0; i < filas.length; ++i) {
                var chat = [];
                if (filas[i].nome === Evento.Queue) {
                    const agente = filas[i].getAgente(Evento.Interface);
                    if (agente) {
                        // Só atualiza se for problema, pois isso afeta o multifila
                        agente.status = Evento.Paused === 1 ? 10 : Evento.Status;
                        agente.StatusNome = agente.getStatusNome();
                        agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
                        agente.StatusDate = new Date();
                        agente.ligacoes = Evento.CallsTaken;
                        agente.tipo = Evento.Membership;
                        agente.ultimaligacao = Evento.LastCall;
                        agente.ultimapausa = Evento.LastPause;
                        agente.pausa = Evento.Paused;
                        agente.pausa_motivo = Evento.Reason;
                        agente.motivo = Evento.Reason;

                        // Gera evento
                        event.AgentePausa(agente);
                        eventos.emit('AgentePausa', agente)

                        if (options.config.integracao_status_agente && options.config.integracao_status_agente.ativada) {
                            // Processamento de integração de status de agente.
                            try {
                                NotificaIntegracaoStatus(agente);
                            } catch (ex) {
                                log.error("INTEGRACAO_STATUS_AGENTE: Falha ao processar NotificaIntegracaoStatus(). ERRO: " + ex.toString());
                            }
                        }

                    } else {
                        const agente = new Agente(Evento.Interface);
                        agente.nome = Evento.MemberName;
                        agente.status = Evento.Paused === 1 ? 10 : Evento.Status;
                        agente.StatusNome = agente.getStatusNome();
                        agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
                        agente.StatusDate = new Date();
                        agente.ligacoes = Evento.CallsTaken;
                        agente.tipo = Evento.Membership;
                        agente.ultimaligacao = Evento.LastCall;
                        agente.ultimapausa = Evento.LastPause;
                        agente.pausa = Evento.Paused;
                        agente.pausa_motivo = Evento.Reason;
                        agente.motivo = Evento.Reason;
                        agente.penalizacoes = Evento.Penalty;
                        agente.fila = Evento.Queue;
                        agente.time = 0;
                        // await agente.getMaxChatAtendeFromDb(Evento.Interface);

                        // Load do agente do banco se for agente dinamico
                        if(ev.membership === 'dynamic') {
                            let ag = await AgenteModel.fetchByFullname(agente.nome);
                            agente.id = ag.id;
                            agente.max_chat_atende = ag.max_chat_atende;
                            agente.edita_rota = ag.edita_rota;
                            agente.visualiza_todos_clientes = ag.visualiza_todos_clientes;
                            agente.atalho_url = ag.atalho_url;
                            agente.abrir_url = ag.abrir_url;
                            agente.tempo_maximo_sessao = ag.tempo_maximo_sessao;
                            agente.tempo_auto_pausa = ag.tempo_auto_pausa;
                            agente.edita_config = ag.edita_config;
                            agente.default_instance = ag.default_instance;
                            agente.pode_selecionar_instancias = ag.pode_selecionar_instancias;
                        }
                        
                        filas[i].addAgente(agente);

                        // Evento
                        event.AgentePausa(agente);
                        eventos.emit('AgentePausa', agente)
                        // Tambem fazer essa notificação de integração ouvir o eventos

                        if (options.config.integracao_status_agente && options.config.integracao_status_agente.ativada) {
                            // Processamento de integração de status de agente.
                            try {
                                NotificaIntegracaoStatus(agente);
                            } catch (ex) {
                                log.error("INTEGRACAO_STATUS_AGENTE: Falha ao processar NotificaIntegracaoStatus(). ERRO: " + ex.toString());
                            }
                        }

                    }




                    // Processamento do discador, só quando sai da pausa
                    if (!Evento.Paused) {
                        try {
                            discador.ProcessaCampanhasByFila(filas[i].nome);
                        } catch (ex) {
                            log.error("AMI_QUEUEMEMBERPAUSE: Falha ao processar discador.ProcessaCampanhasByFila. ERRO: " + ex.toString());
                        }
                    }

                    break;
                }
            }
        } catch (ex) {
            log.error("AMI_QUEUEMEMBERPAUSE: Falha ao processar QueueMemberPause: ERRO: " + ex.toString());
        }
    });
    myEmitter.on('AgentDisconnected', async (data) => {
        data = limpaEvento(data)
        let agente = this.getAgenteLogadoByFullname(data.nome);
        if (agente) {
            agente.status = 0;
            agente.StatusNome = agente.getStatusNome();
            agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
            agente.StatusDate = new Date();
            console.log(`EMITINDO AGENTE STATUS`, agente);
            event.AgenteStatus(agente);
            eventos.emit('AgenteDesconectou', agente)
        }
    });
    myEmitter.on('QueueMemberStatus', async (ev) => {
        ev = limpaEvento(ev)
        try {
            const Evento = {
                Queue: ev.queue,
                MemberName: ev.membername,
                Interface: ev.interface,
                StateInterface: ev.stateinterface,
                Membership: ev.membership,
                Penalty: ev.penalty,
                CallsTaken: parseInt(ev.callstaken),
                LastCall: parseInt(ev.lastcall),
                LastPause: parseInt(ev.lastpause),
                InCall: parseInt(ev.incall),
                Status: parseInt(ev.status),
                Paused: parseInt(ev.paused),
                PausedReason: ev.pausedreason,
                Ringinuse: parseInt(ev.ringinuse)
            };
            for (let i = 0; i < filas.length; ++i) {
                let chat = [];
                if (filas[i].nome === Evento.Queue) {
                    const agente = filas[i].getAgente(ev.interface);
                    // Agente encontrado
                    if (agente) {
                        agente.update = 'QueueMemberStatus';
                        agente.status = Evento.Paused ? 10 : Evento.Status;
                        agente.pausa = Evento.Paused;
                        agente.StatusNome = agente.getStatusNome();
                        agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
                        agente.StatusDate = new Date();
                    }
                    else {
                        const agente = new Agente(Evento.Interface);
                        agente.update = 'QueueMemberStatus';
                        agente.nome = Evento.MemberName;
                        agente.status = Evento.Paused ? 10 : Evento.Status;
                        agente.StatusNome = agente.getStatusNome();
                        agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
                        agente.StatusDate = new Date();
                        agente.ligacoes = Evento.CallsTaken;
                        agente.ligacoes_ativas = [];
                        agente.atendeu = Evento.CallsTaken;
                        agente.tipo = Evento.Membership;
                        agente.ultimaligacao = Evento.LastCall;
                        agente.ultimapausa = Evento.LastPause;
                        agente.pausa = Evento.Paused;
                        agente.pausa_motivo = Evento.PausedReason;
                        agente.motivo = agente.pausa_motivo;
                        agente.penalizacoes = Evento.Penalty;
                        agente.fila = Evento.Queue;
                        agente.time = 0;

                        // Load do agente do banco
                        if(ev.membership === 'dynamic') {
                            let ag = await AgenteModel.fetchByFullname(agente.nome);
                            agente.id = ag.id;
                            agente.max_chat_atende = ag.max_chat_atende;
                            agente.edita_rota = ag.edita_rota;
                            agente.visualiza_todos_clientes = ag.visualiza_todos_clientes;
                            agente.atalho_url = ag.atalho_url;
                            agente.abrir_url = ag.abrir_url;
                            agente.tempo_maximo_sessao = ag.tempo_maximo_sessao;
                            agente.tempo_auto_pausa = ag.tempo_auto_pausa;
                            agente.edita_config = ag.edita_config;
                            agente.default_instance = ag.default_instance;
                            agente.pode_selecionar_instancias = ag.pode_selecionar_instancias;
                        }

                        filas[i].addOrUpdateAgente(agente);
                    }

                    event.AgenteStatus(agente);
                    eventos.emit('AgenteStatus', agente)

                    if (options.config.integracao_status_agente && options.config.integracao_status_agente.ativada) {
                        // Processamento de integração de status de agente.
                        try {
                            NotificaIntegracaoStatus(agente);
                        } catch (ex) {
                            log.error("INTEGRACAO_STATUS_AGENTE: Falha ao processar NotificaIntegracaoStatus(). ERRO: " + ex.toString());
                        }
                    }

                    break;
                }
            }
        } catch (ex) {
            log.error("AMI_QUEUEMEMBERSTATUS: Falha ao processar QueueMemberStatus: " + ex.toString());
        }
    });

    myEmitter.on('QueueMemberRemoved', (ev) => {
        ev = limpaEvento(ev)

        try {
            const QueueMemberRemoved = {
                Queue: ev.queue,
                MemberName: ev.membername,
                Interface: ev.interface,
                StateInterface: ev.stateinterface,
                Membership: ev.membership,
                Penalty: ev.penalty,
                CallsTaken: parseInt(ev.callstaken),
                LastCall: ev.lastcall,
                LastPause: ev.lastpause,
                InCall: parseInt(ev.incall),
                Status: parseInt(ev.status),
                Paused: parseInt(ev.paused),
                PausedReason: ev.pausedreason,
                Ringinuse: parseInt(ev.ringinuse)
            };
            for (let i = 0; i < filas.length; ++i) {
                if (filas[i].nome === QueueMemberRemoved.Queue) {
                    const agente = new Agente(QueueMemberRemoved.Interface);
                    agente.nome = QueueMemberRemoved.MemberName;
                    agente.status = 5;
                    agente.StatusNome = agente.getStatusNome();
                    agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
                    agente.StatusDate = new Date();
                    agente.emligacao = QueueMemberRemoved.InCall;
                    agente.Ringinuse = QueueMemberRemoved.Ringinuse;
                    agente.ligacoes = QueueMemberRemoved.CallsTaken;
                    agente.tipo = QueueMemberRemoved.Membership;
                    agente.ultimaligacao = QueueMemberRemoved.LastCall;
                    agente.ultimapausa = QueueMemberRemoved.LastPause;
                    agente.pausa = QueueMemberRemoved.Paused;
                    agente.pausa_motivo = QueueMemberRemoved.PausedReason;
                    agente.motivo = agente.pausa_motivo;
                    agente.penalizacoes = QueueMemberRemoved.Penalty;
                    agente.fila = QueueMemberRemoved.Queue;
                    filas[i].removeAgente(agente);

                    event.AgenteRemovido(agente);
                    eventos.emit('AgenteRemovido', agente)

                    if (options.config.integracao_status_agente && options.config.integracao_status_agente.ativada) {
                        // Processamento de integração de status de agente.
                        try {
                            NotificaIntegracaoStatus(agente);
                        } catch (ex) {
                            log.error("INTEGRACAO_STATUS_AGENTE: Falha ao processar NotificaIntegracaoStatus(). ERRO: " + ex.toString());
                        }
                    }

                    break;
                }
            }
        } catch (ex) {
            log.error("AMI_QUEUEMEMBERREMOVE: Falha ao processar QueueMemberRemoved. ERRO: " + ex.toString());
        }
    });

    // Acontece quando o agente é notificado de uma chamada. (Ringing)
    myEmitter.on('AgentCalled', (ev) => {
        ev = limpaEvento(ev)
        myEmitter.emit('Tocando', ev);
        try {
            // Ligação deverá ser alocada para o Agente em todas as filas que ele tiver logado.
            for (let i = 0; i < filas.length; ++i) {
                const agente = filas[i].getAgente(ev.interface);
                // Agente encontrado
                if (agente) {
                    // Vamos ver se essa ligação já não está no agente...
                    agente.getLigacao(ev.linkedid, (ligacao) => {
                        // Se a ligação na está na sua lista de ligações add
                        if (!ligacao) {
                            const l = new Ligacao(ev.linkedid);
                            l.origem = ev.calleridnum;
                            l.origem_nome = ev.calleridname;
                            l.sentido = 'Entrante';
                            l.status = 6;
                            l.StatusNome = 'Tocando';
                            l.fila = ev.queue;
                            if (!agente.ligacoes) agente.ligacoes = 0;
                            if (!agente.ligacoes_ativas) agente.ligacoes_ativas = [];
                            agente.ligacoes++;
                            agente.ligacoes_ativas.push(l);

                            agente.status = 6;
                            agente.update = 'AgentCalled';
                            agente.StatusNome = agente.getStatusNome();
                            agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
                            agente.StatusDate = new Date();
                        }
                        // Publica evento
                        event.AgenteStatus(agente);
                        eventos.emit('AgenteTocando', { agente, ligacao: ev })

                        if (options.config.integracao_status_agente && options.config.integracao_status_agente.ativada) {
                            // Processamento de integração de status de agente.
                            try {
                                NotificaIntegracaoStatus(agente);
                            } catch (ex) {
                                log.error("INTEGRACAO_STATUS_AGENTE: Falha ao processar NotificaIntegracaoStatus(). ERRO: " + ex.toString());
                            }
                        }
                    });
                }
            }
        } catch (ex) {
            log.error("AMI_AGENTCALLED: Falha ao processar AgentCalled, ERRO: " + ex.toString());
        }
    });

    // Acontece quando o agente atende a chamada
    myEmitter.on('AgentConnect', (ev) => {
        ev = limpaEvento(ev)
        myEmitter.emit('Atendeu', ev);
        try {
            // Status do agente deverá ser atualizado em todas as filas que ele tiver logado.
            for (let i = 0; i < filas.length; ++i) {
                const agente = filas[i].getAgente(ev.interface);
                // Agente encontrado
                if (agente) {
                    agente.update = 'AgentConnect';
                    agente.status = 3;
                    agente.StatusNome = agente.getStatusNome();
                    agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
                    agente.StatusDate = new Date();
                    agente.AtendeuEm = ev.ringtime;
                    //const ligacao = new Ligacao(ev.linkedid);
                    //ligacao.StatusNome = 'Atendida';
                    //agente.setStatusLigacao(ligacao);
                    if (!agente.atendeu) agente.atendeu = 0;
                    agente.atendeu++;
                    agente.LigacaoAguardou = ev.holdtime;

                    event.AgenteStatus(agente);
                    myEmitter.emit('AgenteAtendeu', agente);
                    eventos.emit('AgenteAtendeu', { agente, ligacao: ev })

                    if (options.config.integracao_status_agente && options.config.integracao_status_agente.ativada) {
                        // Processamento de integração de status de agente.
                        try {
                            NotificaIntegracaoStatus(agente);
                        } catch (ex) {
                            log.error("INTEGRACAO_STATUS_AGENTE: Falha ao processar NotificaIntegracaoStatus(). ERRO: " + ex.toString());
                        }
                    }
                }
            }
        } catch (ex) {
            log.error("AMI_AGENTCONNECT: Falha ao processar AgentConnect. ERRO: " + ex.toString());
        }
    });

    // Acontece quando a chamada termina
    myEmitter.on('AgentComplete', (ev) => {
        ev = limpaEvento(ev)
        myEmitter.emit('Desligou', ev);
        try {
            const ReasonMap = new Map();
            ReasonMap.set('caller', 'Cliente');
            ReasonMap.set('agent', 'Agente');
            ReasonMap.set('transfer', 'Transferida');
            // Status do agente deverá ser atualizado em todas as filas que ele tiver logado.
            for (let i = 0; i < filas.length; ++i) {
                // Atualiza informação de ligações só da fila que veio o evento
                if (filas[i].nome === ev.queue) {
                    // Verifica se a ligação não foi adicionada anteriormente
                    if (!filas[i].ligacoes_atendidas) filas[i].ligacoes_atendidas = [];
                    if (!filas[i].ligacoes_atendidas.find(x => x.linkedid === ev.linkedid)) {
                        filas[i].ligacoes_atendidas.push(ev);// Adiciona para o evento de atendidas
                        log.info("[AMI] Ligação " + ev.linkedid + " atendida!");
                    }

                    filas[i].total_falando = filas[i].total_falando ? filas[i].total_falando + parseInt(ev.talktime) : parseInt(ev.talktime);
                    // filas[i].tma = Math.round(filas[i].total_falando / filas[i].ligacoes_atendidas.length);
                    filas[i].total_espera = filas[i].total_espera ? filas[i].total_espera + parseInt(ev.holdtime) : parseInt(ev.holdtime);
                    // filas[i].tme = Math.round(filas[i].total_espera / filas[i].ligacoes_atendidas.length);
                }

                let agente = filas[i].getAgente(ev.interface);
                // Agente encontrado
                if (agente) {
                    let lig = new Ligacao(ev.linkedid);
                    lig.fim = ReasonMap.get(ev.reason);

                    agente.removeLigacao(lig, () => {
                        if (agente.ligacoes_ativas.length === 0) {
                            agente.status = agente.pausa ? 10 : 1;
                            agente.StatusNome = agente.getStatusNome();
                            agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
                            agente.StatusDate = new Date();
                            agente.DataUltimaLigacao = moment.tz(options.config.servidor.timezone).format();
                            agente.LigacaoDuracao = parseInt(ev.talktime);
                            agente.LigacaoAguardou = parseInt(ev.holdtime);
                            agente.LigacaoFim = ReasonMap.get(ev.reason);

                            event.AgenteStatus(agente);
                            eventos.emit('AgenteDesligou', { agente, ligacao: ev })

                            if (options.config.integracao_status_agente && options.config.integracao_status_agente.ativada) {
                                // Processamento de integração de status de agente.
                                try {
                                    NotificaIntegracaoStatus(agente);
                                } catch (ex) {
                                    log.error("INTEGRACAO_STATUS_AGENTE: Falha ao processar NotificaIntegracaoStatus(). ERRO: " + ex.toString());
                                }
                            }
                        }
                    });
                }
            }
            event.FilaStatus(filas);
        } catch (ex) {
            log.error("AMI_AGENTCOMPLETE: Falha ao processar AgentComplete. ERRO: " + ex.toString());
        }
    });

    // Acontece quando um agente não atende uma ligação
    myEmitter.on('AgentRingNoAnswer', (ev) => {
        ev = limpaEvento(ev)
        myEmitter.emit('NaoAtendeu', ev);
        try {
            const Evento = {
                Channel: ev.channel,
                ChannelState: ev.channelstate,
                ChannelStateDesc: ev.channelstatedesc,
                CallerIDNum: ev.calleridnum,
                CallerIDName: ev.calleridname,
                ConnectedLineNum: ev.connectedlinenum,
                ConnectedLineName: ev.connectedlinename,
                AccountCode: ev.accountcode,
                Context: ev.context,
                Exten: ev.exten,
                Priority: ev.priority,
                Uniqueid: ev.uniqueid,
                Linkedid: ev.linkedid,
                DestChannel: ev.destchannel,
                DestChannelState: ev.destchannelstate,
                DestChannelStateDesc: ev.destchannelstatedesc,
                DestCallerIDNum: ev.destcalleridnum,
                DestCallerIDName: ev.destcalleridname,
                DestConnectedLineNum: ev.destconnectedlinenum,
                DestConnectedLineName: ev.destconnectedlinename,
                DestAccountCode: ev.destaccountcode,
                DestContext: ev.destcontext,
                DestExten: ev.destexten,
                DestPriority: ev.destpriority,
                DestUniqueid: ev.destuniqueid,
                DestLinkedid: ev.destlinkedid,
                Queue: ev.queue,
                MemberName: ev.membername,
                Interface: ev.interface,
                RingTime: parseInt(ev.ringtime)
            };
            for (let i = 0; i < filas.length; ++i) {
                if (filas[i].nome === Evento.Queue) {
                    const agente = filas[i].getAgente(Evento.Interface);
                    if (agente) {
                        //////// MUDAR AQUI, INCLUIR REMOVER LIG COM CALLBACK E SO MUDAR STATUS SE FOR A ULTIMA LIGACAO
                        const lig = new Ligacao(Evento.Linkedid);
                        lig.fim = 'NaoAtendeu';
                        agente.removeLigacao(lig, () => {
                            if (agente.ligacoes_ativas.length === 0) {
                                agente.status = agente.pausa ? 10 : 1;
                                agente.StatusNome = agente.getStatusNome();
                                agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
                                agente.StatusDate = new Date();
                                agente.DataUltimoNaoAtendimento = moment.tz(options.config.servidor.timezone).format();
                                agente.LigacaoTocou = Evento.RingTime;

                                event.AgenteStatus(agente);
                                eventos.emit('AgenteNaoAtendeu', { agente, ligacao: ev })

                                if (options.config.integracao_status_agente && options.config.integracao_status_agente.ativada) {
                                    // Processamento de integração de status de agente.
                                    try {
                                        NotificaIntegracaoStatus(agente);
                                    } catch (ex) {
                                        log.error("INTEGRACAO_STATUS_AGENTE: Falha ao processar NotificaIntegracaoStatus(). ERRO: " + ex.toString());
                                    }
                                }
                            }
                        });
                    }
                }
            }
        } catch (ex) {
            log.error("AMI_AGENTRINGNOANSWER: Falha ao processar AgentRingNoAnswer. ERRO: " + ex.toString());
        }
    });

    // Ocorre quando a ligacao é interrompida.
    myEmitter.on('AgentDump', (ev) => {
        ev = limpaEvento(ev)
        try {
            const Evento = {
                Channel: ev.channel,
                ChannelState: ev.channelstate,
                ChannelStateDesc: ev.channelstatedesc,
                CallerIDNum: ev.calleridnum,
                CallerIDName: ev.calleridname,
                ConnectedLineNum: ev.connectedlinenum,
                ConnectedLineName: ev.connectedlinename,
                AccountCode: ev.accountcode,
                Context: ev.context,
                Exten: ev.exten,
                Priority: ev.priority,
                Uniqueid: ev.uniqueid,
                Linkedid: ev.linkedid,
                DestChannel: ev.destchannel,
                DestChannelState: ev.destchannelstate,
                DestChannelStateDesc: ev.destchannelstatedesc,
                DestCallerIDNum: ev.destcalleridnum,
                DestCallerIDName: ev.destcalleridname,
                DestConnectedLineNum: ev.destconnectedlinenum,
                DestConnectedLineName: ev.destconnectedlinename,
                DestAccountCode: ev.destaccountcode,
                DestContext: ev.destcontext,
                DestExten: ev.destexten,
                DestPriority: ev.destpriority,
                DestUniqueid: ev.destuniqueid,
                DestLinkedid: ev.destlinkedid,
                Queue: ev.queue,
                MemberName: ev.membername,
                Interface: ev.interface
            };
            myEmitter.emit('Interrompeu', Evento);
            for (let i = 0; i < filas.length; ++i) {
                if (filas[i].nome === Evento.Queue) {
                    const agente = filas[i].getAgente(Evento.Interface);
                    if (agente) {
                        //////// MUDAR AQUI, INCLUIR REMOVER LIG COM CALLBACK E SO MUDAR STATUS SE FOR A ULTIMA LIGACAO
                        const lig = new Ligacao(Evento.Linkedid);
                        lig.fim = 'Dump';
                        agente.removeLigacao(lig, () => {
                            if (agente.ligacoes_ativas.length === 0) {
                                agente.status = agente.pausa ? 10 : 1;
                                agente.StatusNome = agente.getStatusNome();
                                agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
                                agente.StatusDate = new Date();
                                agente.DataUltimoDump = moment.tz(options.config.servidor.timezone).format();

                                event.AgenteStatus(agente);

                                if (options.config.integracao_status_agente && options.config.integracao_status_agente.ativada) {
                                    // Processamento de integração de status de agente.
                                    try {
                                        NotificaIntegracaoStatus(agente);
                                    } catch (ex) {
                                        log.error("INTEGRACAO_STATUS_AGENTE: Falha ao processar NotificaIntegracaoStatus(). ERRO: " + ex.toString());
                                    }
                                }
                            }
                        });
                    }
                }
            }
        } catch (ex) {
            log.error("AMI_AGENTDUMP: Falha ao processar AgentDump. ERRO: " + ex.toString());
        }
    });

    myEmitter.on('Newchannel', (ev) => {
        // Publica esse evento para Apps não SIP identificarem o evento.
        ev = limpaEvento(ev)
        ws.broadcast(ev);
        try {
            const Newchannel = {
                Channel: ev.channel,
                ChannelState: ev.channelctate,
                ChannelStateDesc: ev.channelctatedesc,
                CallerIDNum: ev.calleridnum,
                CallerIDName: ev.calleridname,
                ConnectedLineNum: ev.connectedlinenum,
                ConnectedLineName: ev.connectedlinename,
                AccountCode: ev.accountcode,
                Context: ev.context,
                Exten: ev.exten,
                Priority: ev.priority,
                Uniqueid: ev.uniqueid,
                Linkedid: ev.linkedid
            };
        } catch (ex) {
            log.error("AMI_NEWCHANNEL: Falha ao processar Newchannel. ERRO: " + ex.toString());
        }
    });

    // Armazenar ligações efetuadas
    myEmitter.on('Newstate', (ev) => {
        // Publica esse evento para Apps não SIP identificarem o evento.
        ev = limpaEvento(ev)
        myEmitter.emit('Ligacao', ev);
        ws.broadcast(ev);
        try {
            const ChannelStateMap = new Map();
            ChannelStateMap.set(0, 'Down');
            ChannelStateMap.set(1, 'Rsrvd');
            ChannelStateMap.set(2, 'OffHook');
            ChannelStateMap.set(3, 'Dialing');
            ChannelStateMap.set(4, 'Ring');
            ChannelStateMap.set(5, 'Ringing');
            ChannelStateMap.set(6, 'Up');
            ChannelStateMap.set(7, 'Busy');
            ChannelStateMap.set(8, 'Dialing Offhook');
            ChannelStateMap.set(9, 'Pre-ring');
            ChannelStateMap.set(10, 'Unknown');

            // Ligação deverá ser alocada para o Agente em todas as filas que ele tiver logado.
            for (let i = 0; i < filas.length; ++i) {
                const agente = filas[i].getAgente(ev.channel.split('-')[0]);
                // Agente encontrado
                if (agente) {
                    // Vamos ver se essa ligação já não está no agente...
                    agente.getLigacao(ev.linkedid, (ligacao) => {
                        // Nova Ligação
                        if (!ligacao) {
                            const l = new Ligacao(ev.linkedid);
                            switch (parseInt(ev.channelstate)) {
                                case 4:
                                    l.sentido = 'Sainte'
                                    l.destino = ev.exten;
                                    agente.status = 6;
                                    break;
                                case 5:
                                    l.origem = ev.connectedlinenum;
                                    l.origem_nome = ev.connectedlinename;
                                    l.sentido = 'Entrante';
                                    agente.status = 6;
                                    break;
                                case 6:
                                    agente.status = 2;
                                    break;
                            }
                            agente.update = 'Newstate';
                            agente.StatusNome = agente.getStatusNome();
                            agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
                            agente.StatusDate = new Date();
                            l.status = parseInt(ev.channelstate);
                            l.StatusNome = ChannelStateMap.get(parseInt(ev.channelstate));
                            l.fila = filas[i].nome;
                            if (!agente.ligacoes) agente.ligacoes = 0;
                            agente.ligacoes++;
                            agente.addOrUpdateLigacao(l, filas[i]);

                            event.AgenteStatus(agente);

                            if (options.config.integracao_status_agente && options.config.integracao_status_agente.ativada) {
                                // Processamento de integração de status de agente.
                                try {
                                    NotificaIntegracaoStatus(agente);
                                } catch (ex) {
                                    log.error("INTEGRACAO_STATUS_AGENTE: Falha ao processar NotificaIntegracaoStatus(). ERRO: " + ex.toString());
                                }
                            }
                        }
                        else {
                            ligacao.status = parseInt(ev.channelstate);
                            ligacao.StatusNome = ChannelStateMap.get(parseInt(ev.channelstate));
                            // Ligação completada
                            if (ligacao.status === 6) agente.status = 2;
                            agente.StatusNome = agente.getStatusNome();
                            agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
                            agente.StatusDate = new Date();
                            agente.addOrUpdateLigacao(ligacao, filas[i]);
                        }
                        // Publica evento
                        event.AgenteStatus(agente);

                        if (options.config.integracao_status_agente && options.config.integracao_status_agente.ativada) {
                            // Processamento de integração de status de agente.
                            try {
                                NotificaIntegracaoStatus(agente);
                            } catch (ex) {
                                log.error("INTEGRACAO_STATUS_AGENTE: Falha ao processar NotificaIntegracaoStatus(). ERRO: " + ex.toString());
                            }
                        }
                    });
                }
            }
        } catch (ex) {
            log.error("AMI_NEWSTATE: Falha ao processar Newstate. ERRO: " + ex.toString());
        }
    });

    function removerLigacaoTransferida(e) {
        try {
            let ids = [];
            let { lines, ...evento } = e;

            if (evento.origtransfererlinkedid) ids.push(evento.origtransfererlinkedid);
            if (evento.origtransfereruniqueid) ids.push(evento.origtransfereruniqueid);

            if (evento.secondtransfererlinkedid) ids.push(evento.secondtransfererlinkedid);
            if (evento.secondtransfereruniqueid) ids.push(evento.secondtransfereruniqueid);

            if (evento.transfertargetlinkedid) ids.push(evento.transfertargetlinkedid);
            if (evento.transfertargetuniqueid) ids.push(evento.transfertargetuniqueid);

            if (evento.transfereelinkedid) ids.push(evento.transfereelinkedid);
            if (evento.transfereeuniqueid) ids.push(evento.transfereeuniqueid);

            if (ids.length > 0) {
                for (let fila of filas) {
                    fila.ligacoes.forEach((ligacao, i) => {
                        if (ids.indexOf(ligacao.uniqueid) !== -1) {
                            fila.ligacoes.splice(i, 1);
                            log.debug("[AMI] [Transfer] Removendo da fila -> " + ligacao.uniqueid, evento);
                        }
                    });

                    for (let agente of fila.agentes) {
                        agente.ligacoes_ativas.forEach((ligacao, i) => {
                            if (ids.indexOf(ligacao.uniqueid) !== -1) {
                                agente.ligacoes_ativas.splice(i, 1);
                                log.debug("[AMI] [Transfer] Removendo do agente -> " + ligacao.uniqueid, evento);
                            }
                        });
                    }
                }
            }
        } catch (e) {
            console.trace(e);
            log.error("[AMI] [Transfer] Erro ao remover ligação transferida -> ", e);
        }
    }

    myEmitter.on('BlindTransfer', (ev) => {
        console.log('EVENTO DE TRANSFERENCIA ===================================');
        ev = limpaEvento(ev)

        myEmitter.emit('TransferindoDireto', ev);
        removerLigacaoTransferida(ev);
    });

    myEmitter.on('AttendedTransfer', (ev) => {
        console.log('EVENTO DE TRANSFERENCIA ===================================');
        ev = limpaEvento(ev)

        myEmitter.emit('Transferindo', ev);
        removerLigacaoTransferida(ev);
    });

    myEmitter.on('Hangup', (ev) => {
        // Publica esse evento para Apps não SIP identificarem o evento.
        ev = limpaEvento(ev)
        ws.broadcast(ev);
        myEmitter.emit('Desliga', ev);
        const Evento = {
            Channel: ev.channel,
            ChannelState: ev.channelctate,
            ChannelStateDesc: ev.channelstatedesc,
            CallerIDNum: ev.calleridnum,
            CallerIDName: ev.calleridname,
            ConnectedLineNum: ev.connectedlinenum,
            ConnectedLineName: ev.connectedlinename,
            AccountCode: ev.accountcode,
            Context: ev.context,
            Exten: ev.exten,
            Priority: ev.priority,
            Uniqueid: ev.uniqueid,
            Linkedid: ev.linkedid,
            Cause: parseInt(ev.cause)
        };

        // Desativado pois fica sem motivo? q sem motivo
        setTimeout(function () {
            try {
                for (let i = 0; i < filas.length; ++i) {
                    // Asterisk BUG, ligações transferidas continuam no aguardando.
                    let l = new Ligacao(Evento.Linkedid);
                    // filas[i].removeLigacao(l);

                    let agente = filas[i].getAgente(Evento.Channel.split('-')[0]);
                    if (agente) {
                        filas[i].efetuada = filas[i].efetuada ? filas[i].efetuada + 1 : 1;

                        for (let j = 0; j < agente.ligacoes_ativas.length; ++j) {
                            if (agente.ligacoes_ativas[j].uniqueid === Evento.Linkedid) {
                                agente.ligacoes_ativas.splice(j, 1);
                                // Verifica se é a última ligação na lista
                                if (agente.ligacoes_ativas.length === 0) {
                                    agente.status = agente.pausa ? 10 : 1;
                                    agente.StatusNome = agente.getStatusNome();
                                    agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
                                    agente.StatusDate = new Date();

                                    event.FilaStatus(filas);
                                }
                            }
                        }
                    }
                }
            } catch (ex) {
                log.error("AMI_HANGUP: Falha ao processar Hangup. ERRO: " + ex.toString());
            }
        }, 5000);

        // Processamento do discador, no caso de abandono o hangup vem antes do evento de abandono dai remove a ligacao como se tivese completado.
        // Para contornar isso chamo 5s depois
        setTimeout(function () {
            log.info('AMI_HANGUP: Executando processamento de Hangup do discador depois de 5s');
            try {
                discador.ProcessaHangup(Evento);
            } catch (ex) {
                log.error("AMI_HANGUP: Falha ao processar discador.ProcessaHangup. ERRO: " + ex.toString());
            }
        }, 5000);
    });

    myEmitter.on('OriginateResponse', (event) => {
        event = limpaEvento(event)
        try {
            discador.ProcessaRetornoOriginate(event);
        } catch (ex) {
            log.error("AMI_ORIGINATERESPONSE: Falha ao processar discador.ProcessaRetornoOriginate. ERRO: " + ex.toString());
        }
    });

    myEmitter.on('AgenteAddChat', async (dados) => {
        this.addChatAgente(dados.fila, dados.ramal, dados.protocolo);

    });

    exports.ws = ws;

    next();
};

exports.register.attributes = {
    name: 'xcontact-ami'
};


exports.XCeventos = myEmitter;
exports.eventos = eventos;