const EventEmitter = require('events');
class MyEmitter extends EventEmitter {}
const myEmitter = new MyEmitter();

exports.register = (server, options, next) => {
  const self = this;  
  const db = require('../util/db');
  
  const ws = require('../ws');
  const moment = require('moment-timezone');
  moment.tz.setDefault(options.config.servidor.timezone);
  const uuid = require('uuid/v1');
  const discador = require('../discador');
  const Ligacao = require('./Ligacao');
  const Fila = require('./Fila');
  const Agente = require('./Agente');
  const request = require('request');
  var auxbd= false;
  const XClientes = require('../util/XClients');
  const xc = new XClientes();

  //const ChatEvents = require('../webrtc/chat/ChatEvents');
  
  var filas = [];
  var agentesChats = [];
  const opts = {
		logFilePath:'/var/log/xcontact.log',
		timestampFormat:'DD-MM-YYYY HH:mm:ss.SSS'
	};
  const log = require('simple-node-logger').createSimpleLogger();
	const log_level = options.config.log ? options.config.log : 'error';
	log.setLevel(log_level);
	log.debug('AMI_START: LOG_LEVEL = '+log_level);

  log.info('AMI_START: Iniciando serviço de AMI');

  var namiLib = require("nami");
  var nami = new (require("nami").Nami)(
    {
      host: options.config.asterisk.host,
      port: options.config.asterisk.porta,
      username: options.config.asterisk.usuario,
      secret: options.config.asterisk.senha,
      logLevel: 0
    }
  );
  nami.logLevel = 0;
  //nami.logger.setLevel('ERROR');

  nami.on('namiConnectionClose', function (data) {
      log.warn('AMI_CONNECTION_CLOSE: Perdi a conexão com o Asterisk, Reconnecting em 5s...');
      setTimeout(function () { nami.open(); }, 5000);
  });

  nami.on('namiInvalidPeer', function (data) {
  	log.warn('AMI_CONNECTION_CLOSE: Tem certeza que está tentando conectar em um AMI?');
  	//process.exit();
  });
  nami.on('namiLoginIncorrect', function () {
  	log.warn('AMI_CONNECTION_CLOSE: Usuário ou senha recusado.');
  	//process.exit();
  });
  nami.on('namiEvent', function (event) {
      log.debug('AMI_EVENTO: '+JSON.stringify(event));
      try {
        myEmitter.emit(event.event, event);
      }catch (ex){
          log.error('AMI_EVENTO: Erro ao tratar evento '+event.event+', motivo: '+ ex.toString());
      }
      // Publica todos os eventos do asterisk
      //ws.broadcast(event);
  });


  // Listener de eventos do Chat.
  //var chatevt = new ChatEvents();
  //chatevt.on('ChatEvento', (event) => {
  //  log.info('Recebi evento de chat....');
  //  log.info(event);
  //});


  // Carrega filas do banco antes de conectar no asterisk
  exports.loadFilas = () => {
    log.info('AMI_LOADFILAS: Carregando filas do banco.');
    db.query("SELECT * FROM public.queue_table")
    .then(res => {
        filas = [];
        for(let i=0; i < res.rows.length; ++i){
          const row = res.rows[i];
          const fila = new Fila(row.name);
          fila.id_lista_callback = row.id_lista_callback;
          fila.callback_tolerancia = row.callback_tolerancia;
          fila.max_ring_agente = row.timeout;
          fila.maximo = parseInt(row.maxlen);
          fila.estrategia = row.strategy;
          fila.autopause = row.autopause;
          filas.push(fila);
        }
        nami.open();
        log.info('AMI_LOADFILAS: Filas recarregadas com sucesso.');
    })
    .catch(e => {
        log.error('AMI_LOADFILAS: ERRO: '+ e);
    });
  };
  this.loadFilas();

  exports.loadFilaByName = (fila_nome) => {
    log.info('AMI_LOADFILA: Carregando fila '+fila_nome+' do banco.');
    const knex = db.knex('public.queue_table');
    knex.where('name', fila_nome);
    knex.select('*')
    .then(rows => {
        var tem = false;
        for(let i=0; i < filas.length; ++i){
          if(filas[i].nome == fila_nome){
            tem = true;
          }
        }
        if(!tem){
          const row = rows[0];
          const fila = new Fila(row.name);
          fila.id_lista_callback = row.id_lista_callback;
          fila.callback_tolerancia = row.callback_tolerancia;
          fila.max_ring_agente = row.timeout;
          fila.maximo = parseInt(row.maxlen);
          fila.estrategia = row.strategy;
          filas.push(fila);
          log.info('AMI_LOADFILAS: Fila '+fila_nome+' carregada  com sucesso.');
        }        
    })
    .catch(e => {
        log.error('AMI_LOADFILAS: ERRO: '+ e);
    });
  };

  exports.getFilas = () => {
    return filas;
  };

  exports.getFila = (fila) => {
    for(var i=0;i<filas.length;++i){
        if(filas[i].nome === fila){
           return filas[i];
        }
    }
  };

  exports.removeFila = (fila) => {
    for(var i=0;i<filas.length;++i){
        if(filas[i].nome === fila){
           filas.splice(i, 1);
        }
    }
  };
  exports.getAgenteLogadoFilas = (ramal,callback) => {
    const agentes_filas = [];
    for(let i=0;i<filas.length;++i){
      for(let j=0;j<filas[i].agentes.length;++j){
        if(filas[i].agentes[j].ramal === ramal && filas[i].agentes[j].status === 1) agentes_filas.push(filas[i].nome);
      }
    }
    if(callback) callback(agentes_filas);
    else return agentes_filas;
  };
  exports.getAgenteLogadoByRamal = (ramal,callback) => {
    const agentes_filas = [];
    for(let i=0;i<filas.length;++i){
      for(let j=0;j<filas[i].agentes.length;++j){
        if(filas[i].agentes[j].ramal === ramal) agentes_filas.push(filas[i].agentes[j]);
      }
    }
    if(callback) callback(agentes_filas);
    else return agentes_filas;
  };

  exports.getAgenteLogadoByFilaRamal = (fila,ramal,callback) => {
    let agente = undefined;
    for(let i=0;i<filas.length;++i){
        if(filas[i].nome === fila){
          for(let j=0;j<filas[i].agentes.length;++j){
            if(filas[i].agentes[j].ramal === ramal) {
              agente = filas[i].agentes[j];
              break;
            }
          }
        }
    }
    if(callback) callback(agente);
    else return agente;
  };
  exports.getChatsAgentes = (protocolo)=>{
    console.log('ENTROU NO GET CHATS ATIVOS');
    for(let i in agentesChats){
      var aux = agentesChats[i];
      console.log('ENTROU NO GET CHATS ATIVOS - FOR');
      if(aux.indexOf(protocolo) !== -1){
        console.log('ENTROU NO GET CHATS ATIVOS - IF', aux.indexOf(protocolo));
        return i;
      }
    }
    //return undefined;
  }
  exports.getChatsAtivos = (protocolo) => {
   
    for(let i=0;i<filas.length;++i){
      for(let j=0;j<filas[i].agentes.length;++j){
        for(let x=0; x <filas[i].agentes[j].chats_ativos.length; x++){
          if(filas[i].agentes[j].chats_ativos[x] === protocolo){
            return filas[i].agentes[j].ramal
          }
        }        
      }
    }
    return undefined;
  }

  exports.addChatAgente = async (fila,ramal,prt) =>{
    try {
      if(!agentesChats[ramal]){
        agentesChats[ramal] = [];
      }
      console.log('ADICIONANDO CHAT',agentesChats[ramal].prt);
      console.log(agentesChats[ramal].indexOf(prt));     
      if(agentesChats[ramal].indexOf(parseInt(prt)) === -1){
        agentesChats[ramal].push(parseInt(prt));
      }

      for(let i=0;i<filas.length;++i){
    
        for(let j =0; j < filas[i].agentes.length; j++){
          console.log('ramal  do agente', ramal);
          console.log(filas[i].agentes[j].ramal);
          if(filas[i].agentes[j].ramal === ramal){          
            var aux_autopause =  filas[i].agentes[j].max_chat_atende - agentesChats[ramal].length;
            if(aux_autopause === 0 && filas[i].autopause && filas[i].autopause !== null){
              var dados = {
                ramal: ramal,
                fila:fila,
                motivo: 'Automatica'
              }
               this.QueuePause(dados)
            } 
          }
        }
      } 
      console.log('CHAT ATIVO',agentesChats);
    } catch (error) {
      log.error('erro ao atualizar chat ativo do agente de ramal',ramal);
    }
  }
  exports.removeChatAgente = async (ramal,prt) =>{
    try {
      console.log('REMOVENDO',agentesChats[ramal])
      if(agentesChats[ramal]){
        var index = agentesChats[ramal].indexOf(parseInt(prt));
        console.log('REMOVENDO CHAT',agentesChats[ramal][index])
        agentesChats[ramal].splice(index,1);
        console.log('REMOVENDO CHAT 2',agentesChats[ramal])
      }  
    } catch (error) {
      log.error('erro ao remover chat ativo do agente de ramal',ramal);
    }
   
  }

  exports.addChatAtivo = async (fila,ramal,prt) =>{
   try {
    var agenteAux; 
    for(let i=0;i<filas.length;++i){
    
      for(let j =0; j < filas[i].agentes.length; j++){
        console.log('ramal  do agente', ramal);
        console.log(filas[i].agentes[j].ramal);
        if(filas[i].agentes[j].ramal === ramal){
          let chat = []
          agenteAux =  await filas[i].getAgente(ramal);
          if(agenteAux.chats_ativos && agenteAux.chats_ativos.length > 0 ){
            agenteAux.chats_ativos.forEach(element => {      
                chat.push(element)
            });                  
           
          }
          if(chat.indexOf(prt) === -1){
            console.log('ADICIONANDO PROTOCOLO NA LISTA ATIVA', prt);
            chat.push(prt)
          }         
          
          var aux_autopause =  agenteAux.max_chat_atende - chat.length;    
           
          if(aux_autopause === 0 && filas[i].autopause && filas[i].autopause !== null){
            var dados = {
              ramal: ramal,
              fila:fila,
              motivo: 'Automatica'
            }
             this.QueuePause(dados)
          }
          var agente = new Agente(ramal)
           agente.nome = agenteAux.nome;
           agente.status = agenteAux.status;
           agente.StatusNome =  agenteAux.StatusNome;
           agente.DataStatus =  agenteAux.DataStatus;
           agente.ligacoes = agenteAux.ligacoes;
           agente.tipo =  agenteAux.tipo;
           agente.ultimaligacao =agenteAux.ultimaligacao;
           agente.ultimapausa = agenteAux.ultimapausa;
           agente.pausa =  agenteAux.pausa;
           agente.pausa_motivo =  agenteAux.pausa_motivo;
           agente.motivo = agenteAux.motivo;
           agente.penalizacoes = agenteAux.penalizacoes;
           agente.fila =  agenteAux.fila;      
           agente.max_chat_atende = agenteAux.max_chat_atende;       
           agente.chats_ativos = await chat; 
           console.log('AGENTE', agente);
           filas[i].addOrUpdateAgente(agente); 
      
        }
      }
    }  
    auxAtendimento = {
      agente: agenteAux,
      protocolo: prt,
      fila: fila 
    }
    myEmitter.emit('AtendimentoSelecionado', auxAtendimento);
   } catch (error) {
     
   }
    
  }
  

  exports.removeChatAtivo =   (fila,ramal,prt)=>{   
    for(let i=0;i<filas.length;++i){    
    
        for(let j =0; j < filas[i].agentes.length; j++){
          if(filas[i].agentes[j].ramal === ramal){
            var agenteAux =  filas[i].getAgente(ramal);
            let chat = []
            var agenteAux =  filas[i].getAgente(ramal);
            if(agenteAux.chats_ativos && agenteAux.chats_ativos.length > 0 ){
              agenteAux.chats_ativos.forEach(element => {      
                  chat.push(element)
              });
            }
            for(let x =0; x < chat.length;x++){
              if(chat[x] === prt){
                chat.splice(x,1);
              }
         
            if(chat){              
            
            var agente = new Agente(ramal)
             agente.nome = agenteAux.nome;
             agente.status = agenteAux.status;
             agente.StatusNome =  agenteAux.StatusNome;
             agente.DataStatus =  agenteAux.DataStatus;
             agente.ligacoes = agenteAux.ligacoes;
             agente.tipo =  agenteAux.tipo;
             agente.ultimaligacao =agenteAux.ultimaligacao;
             agente.ultimapausa = agenteAux.ultimapausa;
             agente.pausa =  agenteAux.pausa;
             agente.pausa_motivo =  agenteAux.pausa_motivo;
             agente.motivo = agenteAux.motivo;
             agente.penalizacoes = agenteAux.penalizacoes;
             agente.fila =  agenteAux.fila;
             agente.max_chat_atende = agenteAux.max_chat_atende;
             agente.chats_ativos =  chat;
             console.log('AGENTE APOS REMOVER CHAT', agente);
             filas[i].addOrUpdateAgente(agente);   
            }     
           
          }
        }        
      }  
    }  
  }

  exports.getAgentesOnline = (fila,callback) => {
    console.log('GET AGENTES ONLINE', fila);
    for(let i=0;i<filas.length;++i){
      console.log(filas[i].nome)
        if(filas[i].nome === fila){
          console.log('AQUI$$$$$');
          const agentes_online = [];
          for(let j=0;j<filas[i].agentes.length;++j){
            console.log('chats do agente',agentesChats[filas[i].agentes[j].ramal]  )
            console.log(filas[i].agentes[j].chats_ativos.length)
            console.log(filas[i].agentes[j].status);
            console.log(filas[i].agentes[j].max_chat_atende);
            // if(filas[i].agentes[j].status === 1 && filas[i].agentes[j].chats_ativos.length  < filas[i].agentes[j].max_chat_atende ) {

            if(filas[i].agentes[j].status === 1 &&  agentesChats[filas[i].agentes[j].ramal].length < filas[i].agentes[j].max_chat_atende ) {
              console.log('AGENTE AINDA NAO ESTA COM O NUMERO MAXIMO',filas[i].agentes[j].chats_ativos  )             
              if(options.config.agente_descanso){                
                let now = moment.tz(options.config.servidor.timezone);
                const tempo_online = moment.duration(now.diff(filas[i].agentes[j].DataStatus));
                if(tempo_online.asSeconds() >= options.config.agente_descanso){                  
                  agentes_online.push(filas[i].agentes[j]);  
                }
                else {
                  log.info('AMI, getAgentesOnline: Agente em decanso, tempo online = '+tempo_online.asSeconds()+'s, descanso definido em : '+options.config.agente_descanso+'s');                  
                }
              }
              else {
                agentes_online.push(filas[i].agentes[j]);
              }
            }
          }
          console.log('AGENTES ONLINE', agentes_online);
          if(callback) callback(agentes_online);
          else return agentes_online;

          
        }

    }
  };
  exports.getQtdAgentesOnline = (fila,callback) => {
    for(let i=0;i<filas.length;++i){
        if(filas[i].nome === fila){
          let total = 0;
          for(let j=0;j<filas[i].agentes.length;++j){
            if(filas[i].agentes[j].status === 1) total++;
          }
          if(callback) callback(total);
          else return total;
        }
    }
  };

  function ReplaceIntegracaoVars(url,dados){
    url = url.replace(/X_AGENTE_NOME/gi, dados.nome);
    url = url.replace(/X_AGENTE_RAMAL/gi, dados.ramal);
    url = url.replace(/X_AGENTE_FILA/gi, dados.fila);
    url = url.replace(/X_AGENTE_STATUS_COD/gi, dados.status);
    url = url.replace(/X_AGENTE_STATUS_NOME/gi, dados.StatusNome);
    url = url.replace(/X_AGENTE_PAUSA_MOTIVO/gi, dados.pausa_motivo);
    url = url.replace(/X_AGENTE_DATA_STATUS/gi, dados.DataStatus);
    return url;
  }

  function NotificaIntegracaoStatus(dados){
    log.info('INTEGRACAO_STATUS_AGENTE: Iniciando chamada de integracao de status de agente.');
    let replaced = ReplaceIntegracaoVars(JSON.stringify(options.config.integracao_status_agente.request), dados);
    let integracao = JSON.parse(replaced);    
    
    request(integracao, (error, response, body) => {
      if(error){
        log.error('INTEGRACAO_STATUS_AGENTE: ERRO: '+JSON.stringify(error));
      }
      else {
        log.info('INTEGRACAO_STATUS_AGENTE: Retornou StatusCode: '+(response && response.statusCode));
        log.info('INTEGRACAO_STATUS_AGENTE: Body: '+body);
      }      
    });
  }

  function groupBy(ar, col) {
    var r = [], o = {}, array = JSON.parse(JSON.stringify(ar));
    array.forEach(function (a) {
        if (!o[a[col]]) {
            //o[a[col]] = {};
            //o[a[col]][col] = a[col];
            o[a[col]] = a;
            o[a[col]]['filas'] = [];
            r.push(o[a[col]]);
        }
        o[a[col]]['filas'].push(a['fila']);
    });
    return r;
  }

  exports.getAgentes = (filtros, callback) => {
    const agentes = [];
    var filtro = JSON.parse(filtros);
    for(let i=0;i<filas.length;++i){ 
      for(let x =0 ; x < filtro.length; x++){
        if(filas[i].nome == filtro[x].fila ){
          for(let j=0;j<filas[i].agentes.length;++j){        
            agentes.push(filas[i].agentes[j]);
            //  if(filas[i].agentes[j].status === 1) agentes_online.push(filas[i].agentes[j]);
          }      
        } 
      }       
      //agentes.push(filas[i].agentes);
              
    }
    if(filtros && filtros.agrupar){
      var agentes2 = groupBy(agentes, 'ramal');
      if(callback) callback(agentes2);
      else return agentes2;
    }
    else {
      if(callback) callback(agentes);
      else return agentes;
    }    
  };

  exports.queueShow = (fila) => {
    const qa = new namiLib.Actions.QueueStatus(fila);
    nami.send(qa);
  };

  exports.send = (dados,callback) => {
    if(typeof callback === "function"){
      nami.send(dados, callback);
    }
    else {
      nami.send(dados);
    }
  };

  exports.QueueAdd = function(dados,callback){
    //Não pode ser Array de filas pelo Retorno Que não é Sync
    const qa = new namiLib.Actions.QueueAdd(dados.ramal, dados.fila, dados.pausado, dados.nome, dados.penalty);
    nami.send(qa, (resposta) => {
      if(callback) callback(resposta);
      else return resposta;
    });
  };

  exports.QueueRemove = function(dados,callback){
    //Não pode ser Array de filas pelo Retorno Que não é Sync
    const qa = new namiLib.Actions.QueueRemove(dados.ramal, dados.fila);
    nami.send(qa, (resposta) => {
      if(callback) callback(resposta);
      else return resposta;
    });
  };

  exports.QueuePause = function(dados,callback){
    //Não pode ser Array de filas pelo Retorno Que não é Sync
    const qa = new namiLib.Actions.QueuePause(dados.ramal, dados.fila, dados.motivo);
    nami.send(qa, (resposta) => {
      if(callback) callback(resposta);
      else return resposta;
    });
  };

  exports.QueueUnpause = function(dados,callback){
    //Não pode ser Array de filas pelo Retorno Que não é Sync
    const qa = new namiLib.Actions.QueueUnpause(dados.ramal, dados.fila);
    nami.send(qa, (resposta) => {
      if(callback) callback(resposta);
      else return resposta;
    });
  };

  exports.Discar = function(dados,callback){
    const ac = new namiLib.Actions.Originate();
    ac.channel  = dados.origem;
    ac.exten    = dados.destino;
    ac.callerid = dados.destino_nome ? dados.destino_nome +" <"+dados.destino+">" : dados.destino;
    ac.priority = 1;
    ac.context  = dados.rota;
    ac.timeout  = dados.max_chamar ? dados.max_chamar * 1000 : 30 * 1000;
    ac.channelid = new Date().getTime();
    ac.async    = true;
    let ar = [];
    ar.push('__SIPADDHEADER51=X-UNIQUE-ID:'+ac.channelid);
    if(dados.campos && Object.keys(dados.campos).length > 0){
      let id = 1;
      Object.keys(dados.campos).forEach(function(key) {
        ar.push('__SIPADDHEADER5'+id+'=X-'+key+':'+encodeURIComponent(dados.campos[key]));
        ++id;
      });
    }
    ac.variable = ar.join(",");

    nami.send(ac, (resposta) => {
      resposta.callid = ac.channelid;
      if(callback) callback(resposta);
      else return resposta;
    });
  };

  exports.Desligar = function(dados,callback){
    const ac = new namiLib.Actions.Hangup();
    ac.channel  = dados.destino ? '/^'+dados.destino+'-.*$/' : dados.canal;
    ac.cause = dados.causa ? dados.causa : 16;
    nami.send(ac, (resposta) => {
      resposta.callid = ac.channelid;
      if(callback) callback(resposta);
      else return resposta;
    });
  };

  exports.nami = nami;

  myEmitter.on('FullyBooted', (event) => {
    log.info('AMI_FULLYBOOTED: #### Conectado no asterisk #####');
    log.info('AMI_FULLYBOOTED: Asterisk Uptime: '+event.uptime);
    log.info('AMI_FULLYBOOTED: Asterisk ultimo reload: '+event.lastreload);
    setTimeout(function(){
        self.queueShow();
    }, 10000);
  });

  // Acontece quando o comando queueShow é chamado pela API (Start do serviço)
  myEmitter.on('QueueParams', (ev) => {
    log.info('AMI_QUEUEPARAMS: Tratando QueueParams');
    //console.log('QueueParams');
    //console.log(ev);
    try {
      const Evento = {
         Queue: ev.queue,
         Max: ev.max,
         Strategy: ev.strategy,
         Calls: parseInt(ev.calls),
         Holdtime: parseInt(ev.holdtime),
         TalkTime: parseInt(ev.talktime),
         Completed: parseInt(ev.completed),
         Abandoned: parseInt(ev.abandoned),
         ServiceLevel: ev.servicelevel,
         ServicelevelPerf: ev.servicelevelperf,
         ServicelevelPerf2: ev.servicelevelperf2,
         Weight: parseInt(ev.weight)
      };
      let add = true;
      // Percorre fila
      for(let i=0;i<filas.length;++i){
          if(filas[i].nome === Evento.Queue){
              add = false;
              filas[i].aguardando = Evento.Calls;
              filas[i].maximo = Evento.Max;
              filas[i].estrategia = Evento.Strategy;
              filas[i].atendidas = Evento.Completed;
              filas[i].abandonadas = Evento.Abandoned;
              filas[i].efetuadas = 0;
              filas[i].tme = Evento.Holdtime ? Evento.Holdtime : 0;
              filas[i].tma = Evento.TalkTime ? Evento.TalkTime : 0;
              filas[i].tmab = 0;
              filas[i].total_espera = filas[i].tme;
              filas[i].total_falando = filas[i].tma;
              filas[i].total_abandono = 0;
              break;
          }
      }
      // Nova fila
      if(add){
          const fila = new Fila(Evento.Queue);
          fila.aguardando = Evento.Calls;
          fila.maximo = Evento.Max;
          fila.estrategia = Evento.Strategy;
          fila.atendidas = Evento.Completed;
          fila.abandonadas = Evento.Abandoned;
          fila.efetuadas = 0;
          fila.tme = Evento.Holdtime ? Evento.Holdtime : 0;
          fila.tma = Evento.TalkTime ? Evento.TalkTime : 0;
          fila.tmab = 0;
          fila.total_espera = fila.tme;
          fila.total_falando = fila.tma;
          fila.total_abandono = 0;
          filas.push(fila);
      }
    } catch (ex){
        log.error('AMI_QUEUEPARAMS: Falha ao atualizar as filas. ERRO: ' + ex.toString());
    }
  });

  // Também Acontece quando o comando queueShow é chamado pela API (Start do serviço)
  myEmitter.on('QueueMember', async (ev) => {
    //console.log('QueueMember');
    //console.log(ev);
    try {
      const Evento = {
        Queue: ev.queue,
        Name: ev.name,
        Location: ev.location,
        StateInterface: ev.stateinterface,
        Membership: ev.membership,
        Penalty: ev.penalty,
        CallsTaken: parseInt(ev.callstaken),
        LastCall: parseInt(ev.lastcall),
        LastPause: parseInt(ev.lastpause),
        InCall: parseInt(ev.incall),
        Status: parseInt(ev.status),
        Paused: parseInt(ev.paused),
        PausedReason: ev.pausedreason
      };
      for(let i=0;i<filas.length;++i){
          if(filas[i].nome === Evento.Queue){
             const agente = new Agente(Evento.Location);
             var chat = [];
             agente.nome = Evento.Name;
             agente.status = Evento.Paused ? 10 : Evento.Status;
             agente.StatusNome = agente.getStatusNome();
             agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
             agente.ligacoes = Evento.CallsTaken;
             agente.tipo = Evento.Membership;
             agente.ultimaligacao = Evento.LastCall;
             agente.ultimapausa = Evento.LastPause;
             agente.pausa = Evento.Paused;
             agente.pausa_motivo = Evento.PausedReason;
             agente.motivo = agente.pausa_motivo;
             agente.penalizacoes = Evento.Penalty;
             agente.fila = Evento.Queue;
             await agente.getMaxChatAtendeFromDb(Evento.Location);
             if(!auxbd){
              var chats = await agente.getChatFromDb(agente.nome); 
              if(!agentesChats[Evento.Location]){
                agentesChats[Evento.Location] = [];
              }
              if(chats){
                for(let i = 0; i < chats.length; i++){
                  this.addChatAgente(Evento.Queue,Evento.Location,chats[i].protocolo);                       
                }
                console.log('PESQUISANDO CHATS ?????????????????????????????????????????????????????',chats);    
              }      
                    
              console.log('PESQUISANDO CHATS ATIVOS DO BANCO',agentesChats);       
              
              auxbd = true;
             } 
             if(auxbd){                     
                agenteAux =  await filas[i].getAgente(QueueMemberAdded.Interface);
                if(agenteAux && agenteAux.chats_ativos && agenteAux.chats_ativos.length > 0 ){
                    agenteAux.chats_ativos.forEach(element => {      
                      chat.push(element)
                });         
                }    
              }          
             agente.chats_ativos = await chat; 
             filas[i].addOrUpdateAgente(agente);
          }
      }
    } catch (ex){
        log.error("'AMI_QUEUEMEMBER: Falha ao processar QueueMember. ERRO: "+ ex.toString());
    }
  });

  // Acontece no final do QueueStatus, uma ótima hora para reportar as filas!!
  myEmitter.on('QueueStatusComplete', (ev) => {
    log.info('AMI_QUEUESTATUSCOMPLETE: Tratando QueueStatusComplete');
    try {
      const ret = {event : 'FilasStatus'};
      ret.filas = filas;
      ws.broadcast(ret);
      myEmitter.emit('FilasStatus', ret);
    } catch (ex){
        log.error("AMI_QUEUESTATUSCOMPLETE: Falha ao processar QueueStatusComplete. ERRO: "+ ex.toString());
    }
  });

  myEmitter.on('QueueCallerJoin', (ev) => {
    //console.log('QueueCallerJoin');
    //console.log(ev);
    log.info('AMI_QUEUECALLERJOIN: Tratando Join');
    try {
      const l = new Ligacao(ev.linkedid);
      l.origem = ev.calleridnum;
      l.origem_nome = ev.calleridname;
      l.sentido = 'Entrante';
      l.fila = ev.queue;
      l.posicao = parseInt(ev.position);
      l.total = parseInt(ev.count);
      l.inicio = moment.tz(options.config.servidor.timezone);

      //const ret1 = {event : 'LigacaoEntrouNaFila'};
      //ret1.ligacao = l;
      //ws.broadcast(ret1);

      for(var i=0;i<filas.length;++i){
          if(filas[i].nome === l.fila){
             filas[i].addOrUpdateLigacao(l, (fila) => {
               //console.log('------>>>> Callback no JoinCaller <<<<------');
               const ret = {event : 'FilasStatus'};
               ret.filas = [];
               ret.filas.push(fila);
               ws.broadcast(ret);
               myEmitter.emit('FilasStatus', ret);
             });             
             break;
          }
      }
    } catch (ex){
        log.error("AMI_QUEUECALLERJOIN: Falha ao processar QueueCallerJoin. ERRO: " + ex.toString());
    }
  });

  myEmitter.on('QueueCallerAbandon', (ev) => {
    //console.log('QueueCallerAbandon');
    //console.log(ev);
    log.info('AMI_QUEUECALLERABANDON: Tratando QueueCallerAbandon');
    try {
      const l = new Ligacao(ev.linkedid);
      l.origem = ev.calleridnum;
      l.origem_nome = ev.calleridname;
      l.sentido = 'Entrante';
      l.fila = ev.queue;
      l.posicao = parseInt(ev.position);
      l.posicao_original = parseInt(ev.originalposition);
      l.total = parseInt(ev.count);
      l.espera = parseInt(ev.holdtime);
      l.fim = moment.tz(options.config.servidor.timezone);
      l.abandono = true;

      //const ret1 = {event : 'LigacaoAbandonou'};
      //ret1.ligacao = l;
      //ws.broadcast(ret1);

      for(let i=0;i<filas.length;++i){
          if(filas[i].nome === l.fila){
            // Discarta transbordos
            if(ev.reason && ev.reason === 'NORMAL'){
              filas[i].abandonadas = filas[i].abandonadas ? filas[i].abandonadas + 1 : 1;
              filas[i].total_abandono = filas[i].total_abandono ? filas[i].total_abandono + l.espera : l.espera;
              filas[i].tmab = Math.round(filas[i].total_abandono / filas[i].abandonadas);
            }

             filas[i].removeLigacao(l);

             // Ve se a ligacao ja nao foi para um agente.
             for(let j=0; j < filas[i].agentes.length; ++j){
               const agente = filas[i].agentes[j];
               const lig = new Ligacao(ev.linkedid);
               lig.fim = 'FilaAbandono';
               agente.removeLigacao(lig, () => {
                if(agente.ligacoes_ativas.length === 0){
                  agente.status = agente.pausa ? 10 : 1;
                  agente.StatusNome = agente.getStatusNome();
                  agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
                }
               });
             }

             // Processamento do discador
             try {
               discador.ProcessaAbandono(ev);
             } catch (ex){
                 log.error("AMI_QUEUECALLERABANDON: Falha ao processar discador.ProcessaAbandono. ERRO: "+ ex.toString());
             }

             const ret = {event : 'FilasStatus'};
             ret.filas = [];
             ret.filas.push(filas[i]);
             ws.broadcast(ret);
             myEmitter.emit('FilasStatus', ret);
             break;
          }
      }
    } catch (ex){
        log.error("AMI_QUEUECALLERABANDON: Falha ao processar QueueCallerAbandon. ERRO: "+ ex.toString());
    }
  });

  myEmitter.on('QueueCallerLeave', (ev) => {
    //console.log('QueueCallerLeave');
    //console.log(ev);
    log.info('AMI_QUEUECALLERLEAVE: Tratando QueueCallerLeave');
    try {
      const l = new Ligacao(ev.linkedid);
      l.origem = ev.calleridnum;
      l.origem_nome = ev.calleridname;
      l.sentido = 'Entrante';
      l.fila = ev.queue;
      l.posicao = parseInt(ev.position);
      l.total = parseInt(ev.count);
      l.fim = moment.tz(options.config.servidor.timezone);

      //const ret1 = {event : 'LigacaoSaiuDaFila'};
      //ret1.ligacao = l;
      //ws.broadcast(ret1);

      for(let i=0;i<filas.length;++i){
          if(filas[i].nome === l.fila){
             filas[i].removeLigacao(l);
             const ret = {event : 'FilasStatus'};
             ret.filas = [];
             ret.filas.push(filas[i]);
             ws.broadcast(ret);
             myEmitter.emit('FilasStatus', ret);
             break;
          }
      }
    } catch (ex){
        log.error("AMI_QUEUECALLERLEAVE: Falha ao processar QueueCallerLeave. ERRO: "+ex.toString());
    }
  });

  myEmitter.on('QueueMemberAdded', async (ev) => {
    //console.log('QueueMemberAdded');
    //console.log(ev);
    try {
      const QueueMemberAdded = {
        Queue: ev.queue,
        MemberName: ev.membername,
        Interface: ev.interface,
        StateInterface: ev.stateinterface,
        Membership: ev.membership,
        Penalty: ev.penalty,
        CallsTaken: parseInt(ev.callstaken),
        LastCall: parseInt(ev.lastcall),
        LastPause: parseInt(ev.lastpause),
        InCall: parseInt(ev.incall),
        Status: parseInt(ev.status),
        Paused: parseInt(ev.paused),
        PausedReason: ev.pausedreason,
        Ringinuse: parseInt(ev.ringinuse)
      };
      for(let i=0;i<filas.length;++i){
         
          if(filas[i].nome === QueueMemberAdded.Queue){  
             let chat = [];
             const agente = new Agente(QueueMemberAdded.Interface);
             agente.nome = QueueMemberAdded.MemberName;
             agente.status = QueueMemberAdded.Paused ? 10 : QueueMemberAdded.Status;
             agente.StatusNome = agente.getStatusNome();
             agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
             agente.ligacoes = QueueMemberAdded.CallsTaken;
             agente.tipo = QueueMemberAdded.Membership;
             agente.ultimaligacao = QueueMemberAdded.LastCall;
             agente.ultimapausa = QueueMemberAdded.LastPause;
             agente.pausa = QueueMemberAdded.Paused;
             agente.pausa_motivo = QueueMemberAdded.PausedReason;
             agente.motivo = agente.pausa_motivo;
             agente.penalizacoes = QueueMemberAdded.Penalty;
             agente.fila = QueueMemberAdded.Queue;
             agente.time = 0; 
             await  agente.getMaxChatAtendeFromDb(QueueMemberAdded.Interface);
            //  await agente.getChatFromDb(agente.nome);       
            for(let j=0; j < filas.length; j++){
              var agenteAux = await filas[j].getAgente(QueueMemberAdded.Interface);
                if(agenteAux && agenteAux.chats_ativos && agenteAux.chats_ativos.length > 0 ){
                  agenteAux.chats_ativos.forEach(element => {      
                    chat.push(element)
                   
                  });
                  return
                }
              }            
             agente.chats_ativos = await chat; 
             filas[i].addOrUpdateAgente(agente);
             console.log('AGENTE ADICIONADO', agente); 
             const ret = {event : 'AgenteAdicionado'};
             ret.agente = agente;
             ws.broadcast(ret);
             myEmitter.emit('AgenteAdicionado', ret);

             if(options.config.integracao_status_agente && options.config.integracao_status_agente.ativada){                
                // Processamento de integração de status de agente.
                try {
                  NotificaIntegracaoStatus(agente);
                } catch (ex){
                    log.error("INTEGRACAO_STATUS_AGENTE: Falha ao processar NotificaIntegracaoStatus(). ERRO: "+ ex.toString());
                }
             }             

             // Processamento do discador
             try {
               discador.ProcessaCampanhasByFila(filas[i].nome);
             } catch (ex){
                 log.error("AMI_QUEUEMEMBERADD: Falha ao processar discador.ProcessaCampanhasByFila. ERRO: "+ ex.toString());
             }

             break;
          }
      }
    } catch (ex){
        log.error("AMI_QUEUEMEMBERADD: Falha ao processar QueueMemberAdded. ERRO: " + ex.toString());
    }
  });

  myEmitter.on('QueueMemberPause', async (ev) => {
    //console.log('QueueMemberPause');
    //console.log(ev);
    
    try {
      const Evento = {
        Queue: ev.queue,
        MemberName: ev.membername,
        Interface: ev.interface,
        StateInterface: ev.stateinterface,
        Membership: ev.membership,
        Penalty: ev.penalty,
        CallsTaken: parseInt(ev.callstaken),
        LastCall: parseInt(ev.lastcall),
        LastPause: parseInt(ev.lastpause),
        InCall: parseInt(ev.incall),
        Status: parseInt(ev.status),
        Paused: parseInt(ev.paused),
        PausedReason: ev.pausedreason,
        Ringinuse: parseInt(ev.ringinuse),
        Reason: ev.reason,
      };
      for(let i=0;i<filas.length;++i){
          var chat = [];
          if(filas[i].nome === Evento.Queue){
             const agente = new Agente(Evento.Interface);
             agente.nome = Evento.MemberName;
             agente.status = Evento.Paused === 1 ? 10 : Evento.Status;
             agente.StatusNome = agente.getStatusNome();
             agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
             agente.ligacoes = Evento.CallsTaken;
             agente.tipo = Evento.Membership;
             agente.ultimaligacao = Evento.LastCall;
             agente.ultimapausa = Evento.LastPause;
             agente.pausa = Evento.Paused;
             agente.pausa_motivo = Evento.Reason;
             agente.motivo = Evento.Reason;
             agente.penalizacoes = Evento.Penalty;
             agente.fila = Evento.Queue;
             agente.time = 0;
             agenteAux =  await filas[i].getAgente(Evento.Interface);
              if(agenteAux && agenteAux.chats_ativos && agenteAux.chats_ativos.length > 0 ){
                  agenteAux.chats_ativos.forEach(element => {      
                    chat.push(element)
              }); 
              }
             agente.chats_ativos = await chat; 
             await agente.getMaxChatAtendeFromDb(Evento.Interface);
             filas[i].addOrUpdateAgente(agente);
             
             const ret = {event : 'AgentePausa'};
             ret.agente = agente;
             ws.broadcast(ret);
             myEmitter.emit('AgentePausa', ret);

            if(options.config.integracao_status_agente && options.config.integracao_status_agente.ativada){                
              // Processamento de integração de status de agente.
              try {
                NotificaIntegracaoStatus(agente);
              } catch (ex){
                  log.error("INTEGRACAO_STATUS_AGENTE: Falha ao processar NotificaIntegracaoStatus(). ERRO: "+ ex.toString());
              }
            }

             // Processamento do discador, só quando sai da pausa
             if(!Evento.Paused){
               try {
                 discador.ProcessaCampanhasByFila(filas[i].nome);
               } catch (ex){
                   log.error("AMI_QUEUEMEMBERPAUSE: Falha ao processar discador.ProcessaCampanhasByFila. ERRO: " + ex.toString());
               }
             }

             break;
          }
      }
    } catch (ex){
        log.error("AMI_QUEUEMEMBERPAUSE: Falha ao processar QueueMemberPause: ERRO: " + ex.toString());
    }
  });

  myEmitter.on('QueueMemberStatus', async (ev) => {
    //console.log('QueueMemberStatus');
    //console.log(ev);
    try {
      const Evento = {
        Queue: ev.queue,
        MemberName: ev.membername,
        Interface: ev.interface,
        StateInterface: ev.stateinterface,
        Membership: ev.membership,
        Penalty: ev.penalty,
        CallsTaken: parseInt(ev.callstaken),
        LastCall: parseInt(ev.lastcall),
        LastPause: parseInt(ev.lastpause),
        InCall: parseInt(ev.incall),
        Status: parseInt(ev.status),
        Paused: parseInt(ev.paused),
        PausedReason: ev.pausedreason,
        Ringinuse: parseInt(ev.ringinuse)
      };
      for(let i=0;i<filas.length;++i){
        let chat = [];
          if(filas[i].nome === Evento.Queue){
            const agente = filas[i].getAgente(ev.interface);
            // Agente encontrado
            if(agente){
              // Só atualiza se for problema, pois isso afeta o multifila
              let problemas = [0,4,5];
              if(problemas.indexOf(Evento.Status)){
                agente.update = 'QueueMemberStatus';
                agente.status = Evento.Paused ? 10 : Evento.Status;
                agente.StatusNome = agente.getStatusNome();
                agente.DataStatus = moment.tz(options.config.servidor.timezone).format();                
              }              
            }
            else {
              const agente = new Agente(Evento.Interface);
              agente.update = 'QueueMemberStatus';
              agente.nome = Evento.MemberName;
              agente.status = Evento.Paused ? 10 : Evento.Status;
              agente.StatusNome = agente.getStatusNome();
              agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
              agente.ligacoes = Evento.CallsTaken;
              agente.ligacoes_ativas = [];
              agente.atendeu = Evento.CallsTaken;
              agente.tipo = Evento.Membership;
              agente.ultimaligacao = Evento.LastCall;
              agente.ultimapausa = Evento.LastPause;
              agente.pausa = Evento.Paused;
              agente.pausa_motivo = Evento.PausedReason;
              agente.motivo = agente.pausa_motivo;
              agente.penalizacoes = Evento.Penalty;
              agente.fila = Evento.Queue;
              agente.time = 0; 
              agenteAux =  await filas[i].getAgente(Evento.Interface);
              if(agenteAux.chats_ativos && agenteAux.chats_ativos.length > 0 ){
                  agenteAux.chats_ativos.forEach(element => {      
                    chat.push(element)
              }); 
              }
             agente.chats_ativos = await chat; 
            
             filas[i].addOrUpdateAgente(agente);               
            }
            const ret = {event : 'AgenteStatus'};
            ret.agente = agente;
            ws.broadcast(ret);
            myEmitter.emit('AgenteStatus', ret);

            if(options.config.integracao_status_agente && options.config.integracao_status_agente.ativada){                
              // Processamento de integração de status de agente.
              try {
                NotificaIntegracaoStatus(agente);
              } catch (ex){
                  log.error("INTEGRACAO_STATUS_AGENTE: Falha ao processar NotificaIntegracaoStatus(). ERRO: "+ ex.toString());
              }
            }

            break;
          }
      }t
    } catch (ex){
        log.error("AMI_QUEUEMEMBERSTATUS: Falha ao processar QueueMemberStatus: "+ ex.toString());
    }
  });

  myEmitter.on('QueueMemberRemoved', (ev) => {
    //console.log('QueueMemberRemoved');
    //console.log(ev);
    try {
      const QueueMemberRemoved = {
        Queue: ev.queue,
        MemberName: ev.membername,
        Interface: ev.interface,
        StateInterface: ev.stateinterface,
        Membership: ev.membership,
        Penalty: ev.penalty,
        CallsTaken: parseInt(ev.callstaken),
        LastCall: ev.lastcall,
        LastPause: ev.lastpause,
        InCall: parseInt(ev.incall),
        Status: parseInt(ev.status),
        Paused: parseInt(ev.paused),
        PausedReason: ev.pausedreason,
        Ringinuse: parseInt(ev.ringinuse)
      };
      for(let i=0;i<filas.length;++i){
          if(filas[i].nome === QueueMemberRemoved.Queue){
             const agente = new Agente(QueueMemberRemoved.Interface);
             agente.nome = QueueMemberRemoved.MemberName;
             agente.status = 5;
             agente.StatusNome = agente.getStatusNome();
             agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
             agente.emligacao = QueueMemberRemoved.InCall;
             agente.Ringinuse = QueueMemberRemoved.Ringinuse;
             agente.ligacoes = QueueMemberRemoved.CallsTaken;
             agente.tipo = QueueMemberRemoved.Membership;
             agente.ultimaligacao = QueueMemberRemoved.LastCall;
             agente.ultimapausa = QueueMemberRemoved.LastPause;
             agente.pausa = QueueMemberRemoved.Paused;
             agente.pausa_motivo = QueueMemberRemoved.PausedReason;
             agente.motivo = agente.pausa_motivo;
             agente.penalizacoes = QueueMemberRemoved.Penalty;
             agente.fila = QueueMemberRemoved.Queue;
             filas[i].removeAgente(agente);

             const ret = {event : 'AgenteRemovido'};
             ret.agente = agente;
             ws.broadcast(ret);
             myEmitter.emit('AgenteRemovido', ret);

             if(options.config.integracao_status_agente && options.config.integracao_status_agente.ativada){                
              // Processamento de integração de status de agente.
              try {
                NotificaIntegracaoStatus(agente);
              } catch (ex){
                  log.error("INTEGRACAO_STATUS_AGENTE: Falha ao processar NotificaIntegracaoStatus(). ERRO: "+ ex.toString());
              }
             }

             break;
          }
      }
    } catch (ex){
        log.error("AMI_QUEUEMEMBERREMOVE: Falha ao processar QueueMemberRemoved. ERRO: " + ex.toString());
    }
  });

  // Acontece quando o agente é notificado de uma chamada. (Ringing)
  myEmitter.on('AgentCalled', (ev) => {
    //console.log('AgentCalled');
    //console.log(ev);
    try {
      // Ligação deverá ser alocada para o Agente em todas as filas que ele tiver logado.
      for(let i=0;i<filas.length;++i){
        const agente = filas[i].getAgente(ev.interface);
        // Agente encontrado
        if(agente){
          // Vamos ver se essa ligação já não está no agente...
          agente.getLigacao(ev.linkedid,(ligacao) => {
            // Se a ligação na está na sua lista de ligações add
            if(!ligacao){
              const l = new Ligacao(ev.linkedid);
              l.origem = ev.calleridnum;
              l.origem_nome = ev.calleridname;
              l.sentido = 'Entrante';
              l.status = 6;
              l.StatusNome = 'Tocando';
              l.fila = ev.queue;
              if(!agente.ligacoes) agente.ligacoes = 0 ;
              if(!agente.ligacoes_ativas) agente.ligacoes_ativas = [];
              agente.ligacoes++;
              agente.ligacoes_ativas.push(l);

              agente.status = 6;
              agente.update = 'AgentCalled';
              agente.StatusNome = agente.getStatusNome();
              agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
            }
            // Publica evento
            const ret = {event : 'AgenteStatus'};
            ret.agente = agente;
            ws.broadcast(ret);
            myEmitter.emit('AgenteStatus', ret);

            if(options.config.integracao_status_agente && options.config.integracao_status_agente.ativada){                
              // Processamento de integração de status de agente.
              try {
                NotificaIntegracaoStatus(agente);
              } catch (ex){
                  log.error("INTEGRACAO_STATUS_AGENTE: Falha ao processar NotificaIntegracaoStatus(). ERRO: "+ ex.toString());
              }
            }
          });          
        }
      }
    } catch (ex){
        log.error("AMI_AGENTCALLED: Falha ao processar AgentCalled, ERRO: "+ ex.toString());
    }
  });

  // Acontece quando o agente atende a chamada
  myEmitter.on('AgentConnect', (ev) => {
    //console.log('AgentConnect');
    //console.log(ev);
    try {
      // Status do agente deverá ser atualizado em todas as filas que ele tiver logado.
      for(let i=0;i<filas.length;++i){
        const agente = filas[i].getAgente(ev.interface);
        // Agente encontrado
        if(agente){
          agente.update = 'AgentConnect';
          agente.status = 3;
          agente.StatusNome = agente.getStatusNome();
          agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
          agente.AtendeuEm = ev.ringtime;
          //const ligacao = new Ligacao(ev.linkedid);
          //ligacao.StatusNome = 'Atendida';
          //agente.setStatusLigacao(ligacao);
          if(!agente.atendeu) agente.atendeu = 0 ;
          agente.atendeu++;
          agente.LigacaoAguardou = ev.holdtime;
          const ret = {event : 'AgenteStatus'};
          ret.agente = agente;
          ws.broadcast(ret);
          myEmitter.emit('AgenteStatus', ret);
          
          if(options.config.integracao_status_agente && options.config.integracao_status_agente.ativada){                
            // Processamento de integração de status de agente.
            try {
              NotificaIntegracaoStatus(agente);
            } catch (ex){
                log.error("INTEGRACAO_STATUS_AGENTE: Falha ao processar NotificaIntegracaoStatus(). ERRO: "+ ex.toString());
            }
          }
        }
      }
    } catch (ex){
        log.error("AMI_AGENTCONNECT: Falha ao processar AgentConnect. ERRO: " + ex.toString());
    }
  });

  // Acontece quando a chamada termina
  myEmitter.on('AgentComplete', (ev) => {
    //console.log('AgentComplete');
    //console.log(ev);
    try {
      const ReasonMap = new Map();
      ReasonMap.set('caller', 'Cliente');
      ReasonMap.set('agent', 'Agente');
      ReasonMap.set('transfer', 'Transferida');
      // Status do agente deverá ser atualizado em todas as filas que ele tiver logado.
      for(let i=0;i<filas.length;++i){
        // Atualiza informação de ligações só da fila que veio o evento
        if(filas[i].nome === ev.queue){
          filas[i].atendidas = filas[i].atendidas ? filas[i].atendidas + 1 : 1;
          filas[i].total_falando = filas[i].total_falando ? filas[i].total_falando + parseInt(ev.talktime) : parseInt(ev.talktime);
          filas[i].tma = Math.round(filas[i].total_falando / filas[i].atendidas);
          filas[i].total_espera = filas[i].total_espera ? filas[i].total_espera + parseInt(ev.holdtime) : parseInt(ev.holdtime);
          filas[i].tme = Math.round(filas[i].total_espera / filas[i].atendidas);
        }

        const agente = filas[i].getAgente(ev.interface);
        // Agente encontrado
        if(agente){
          const lig = new Ligacao(ev.linkedid);
          lig.fim = ReasonMap.get(ev.reason);

          agente.removeLigacao(lig, () => {
              if(agente.ligacoes_ativas.length === 0){
                  agente.status = agente.pausa ? 10 : 1;
                  agente.StatusNome = agente.getStatusNome();
                  agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
                  agente.DataUltimaLigacao = moment.tz(options.config.servidor.timezone).format();
                  agente.LigacaoDuracao = parseInt(ev.talktime);
                  agente.LigacaoAguardou = parseInt(ev.holdtime);
                  agente.LigacaoFim = ReasonMap.get(ev.reason);

                  const ret = {event : 'AgenteStatus'};
                  ret.agente = agente;
                  ws.broadcast(ret);
                  myEmitter.emit('AgenteStatus', ret);

                  if(options.config.integracao_status_agente && options.config.integracao_status_agente.ativada){                
                    // Processamento de integração de status de agente.
                    try {
                      NotificaIntegracaoStatus(agente);
                    } catch (ex){
                        log.error("INTEGRACAO_STATUS_AGENTE: Falha ao processar NotificaIntegracaoStatus(). ERRO: "+ ex.toString());
                    }
                  }
              }
            });       
        }
      }
      const ret = {event : 'FilasStatus'};
      ret.filas = filas;
      ws.broadcast(ret);
      myEmitter.emit('FilasStatus', ret);
    } catch (ex){
        log.error("AMI_AGENTCOMPLETE: Falha ao processar AgentComplete. ERRO: " + ex.toString());
    }
  });

  // Acontece quando um agente não atende uma ligação
  myEmitter.on('AgentRingNoAnswer', (ev) => {
    //console.log('AgentRingNoAnswer');
    //console.log(ev);
    try {
      const Evento = {
        Channel: ev.channel,
        ChannelState: ev.channelstate,
        ChannelStateDesc: ev.channelstatedesc,
        CallerIDNum: ev.calleridnum,
        CallerIDName: ev.calleridname,
        ConnectedLineNum: ev.connectedlinenum,
        ConnectedLineName: ev.connectedlinename,
        AccountCode: ev.accountcode,
        Context: ev.context,
        Exten: ev.exten,
        Priority: ev.priority,
        Uniqueid: ev.uniqueid,
        Linkedid: ev.linkedid,
        DestChannel: ev.destchannel,
        DestChannelState: ev.destchannelstate,
        DestChannelStateDesc: ev.destchannelstatedesc,
        DestCallerIDNum: ev.destcalleridnum,
        DestCallerIDName: ev.destcalleridname,
        DestConnectedLineNum: ev.destconnectedlinenum,
        DestConnectedLineName: ev.destconnectedlinename,
        DestAccountCode: ev.destaccountcode,
        DestContext: ev.destcontext,
        DestExten: ev.destexten,
        DestPriority: ev.destpriority,
        DestUniqueid: ev.destuniqueid,
        DestLinkedid: ev.destlinkedid,
        Queue: ev.queue,
        MemberName: ev.membername,
        Interface: ev.interface,
        RingTime: parseInt(ev.ringtime)
      };
      for(let i=0;i<filas.length;++i){
        if(filas[i].nome === Evento.Queue){
          const agente = filas[i].getAgente(Evento.Interface);
          if(agente){
            //////// MUDAR AQUI, INCLUIR REMOVER LIG COM CALLBACK E SO MUDAR STATUS SE FOR A ULTIMA LIGACAO
            const lig = new Ligacao(Evento.Linkedid);
            lig.fim = 'NaoAtendeu';            
            agente.removeLigacao(lig, () => {
              if(agente.ligacoes_ativas.length === 0) {
                  agente.status = agente.pausa ? 10 : 1;
                  agente.StatusNome = agente.getStatusNome();
                  agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
                  agente.DataUltimoNaoAtendimento = moment.tz(options.config.servidor.timezone).format();
                  agente.LigacaoTocou = Evento.RingTime;

                  const ret = {event : 'AgenteStatus'};
                  ret.agente = agente;
                  ws.broadcast(ret);
                  myEmitter.emit('AgenteStatus', ret);

                  if(options.config.integracao_status_agente && options.config.integracao_status_agente.ativada){                
                    // Processamento de integração de status de agente.
                    try {
                      NotificaIntegracaoStatus(agente);
                    } catch (ex){
                        log.error("INTEGRACAO_STATUS_AGENTE: Falha ao processar NotificaIntegracaoStatus(). ERRO: "+ ex.toString());
                    }
                  }
              }
            });            
          }
        }
      }
    } catch (ex){
        log.error("AMI_AGENTRINGNOANSWER: Falha ao processar AgentRingNoAnswer. ERRO: " + ex.toString());
    }
  });

  // Ocorre quando a ligacao é interrompida.
  myEmitter.on('AgentDump', (ev) => {
    //console.log('AgentDump');
    //console.log(ev);
    try {
      const Evento = {
        Channel: ev.channel,
        ChannelState: ev.channelstate,
        ChannelStateDesc: ev.channelstatedesc,
        CallerIDNum: ev.calleridnum,
        CallerIDName: ev.calleridname,
        ConnectedLineNum: ev.connectedlinenum,
        ConnectedLineName: ev.connectedlinename,
        AccountCode: ev.accountcode,
        Context: ev.context,
        Exten: ev.exten,
        Priority: ev.priority,
        Uniqueid: ev.uniqueid,
        Linkedid: ev.linkedid,
        DestChannel: ev.destchannel,
        DestChannelState: ev.destchannelstate,
        DestChannelStateDesc: ev.destchannelstatedesc,
        DestCallerIDNum: ev.destcalleridnum,
        DestCallerIDName: ev.destcalleridname,
        DestConnectedLineNum: ev.destconnectedlinenum,
        DestConnectedLineName: ev.destconnectedlinename,
        DestAccountCode: ev.destaccountcode,
        DestContext: ev.destcontext,
        DestExten: ev.destexten,
        DestPriority: ev.destpriority,
        DestUniqueid: ev.destuniqueid,
        DestLinkedid: ev.destlinkedid,
        Queue: ev.queue,
        MemberName: ev.membername,
        Interface: ev.interface
      };
      for(let i=0;i<filas.length;++i){
        if(filas[i].nome === Evento.Queue){
          const agente = filas[i].getAgente(Evento.Interface);
          if(agente){
            //////// MUDAR AQUI, INCLUIR REMOVER LIG COM CALLBACK E SO MUDAR STATUS SE FOR A ULTIMA LIGACAO
            const lig = new Ligacao(Evento.Linkedid);
            lig.fim = 'Dump';
            agente.removeLigacao(lig, () => {
              if(agente.ligacoes_ativas.length === 0){
                agente.status = agente.pausa ? 10 : 1;
                agente.StatusNome = agente.getStatusNome();
                agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
                agente.DataUltimoDump = moment.tz(options.config.servidor.timezone).format();
                    
                const ret = {event : 'AgenteStatus'};
                ret.agente = agente;
                ws.broadcast(ret);
                myEmitter.emit('AgenteStatus', ret);

                if(options.config.integracao_status_agente && options.config.integracao_status_agente.ativada){                
                  // Processamento de integração de status de agente.
                  try {
                    NotificaIntegracaoStatus(agente);
                  } catch (ex){
                      log.error("INTEGRACAO_STATUS_AGENTE: Falha ao processar NotificaIntegracaoStatus(). ERRO: "+ ex.toString());
                  }
                }
              }
            });
          }
        }
      }
    } catch (ex){
        log.error("AMI_AGENTDUMP: Falha ao processar AgentDump. ERRO: "+ ex.toString());
    }
  });

  myEmitter.on('Newchannel', (ev) => {
    // Publica esse evento para Apps não SIP identificarem o evento.
    ws.broadcast(ev);
    //console.log(ev);
    try {
      const Newchannel = {
        Channel: ev.channel,
        ChannelState: ev.channelctate,
        ChannelStateDesc: ev.channelctatedesc,
        CallerIDNum: ev.calleridnum,
        CallerIDName: ev.calleridname,
        ConnectedLineNum: ev.connectedlinenum,
        ConnectedLineName: ev.connectedlinename,
        AccountCode: ev.accountcode,
        Context: ev.context,
        Exten: ev.exten,
        Priority: ev.priority,
        Uniqueid: ev.uniqueid,
        Linkedid: ev.linkedid
      };
    } catch (ex){
        log.error("AMI_NEWCHANNEL: Falha ao processar Newchannel. ERRO: " + ex.toString());
    }
  });

  // Armazenar ligações efetuadas
  myEmitter.on('Newstate', (ev) => {
    // Publica esse evento para Apps não SIP identificarem o evento.
    ws.broadcast(ev);
    //console.log('Newstate');
    //console.log(ev);
    try {
      const ChannelStateMap = new Map();
      ChannelStateMap.set(0, 'Down');
      ChannelStateMap.set(1, 'Rsrvd');
      ChannelStateMap.set(2, 'OffHook');
      ChannelStateMap.set(3, 'Dialing');
      ChannelStateMap.set(4, 'Ring');
      ChannelStateMap.set(5, 'Ringing');
      ChannelStateMap.set(6, 'Up');
      ChannelStateMap.set(7, 'Busy');
      ChannelStateMap.set(8, 'Dialing Offhook');
      ChannelStateMap.set(9, 'Pre-ring');
      ChannelStateMap.set(10, 'Unknown');

      // Ligação deverá ser alocada para o Agente em todas as filas que ele tiver logado.
      for(let i=0;i<filas.length;++i){
        const agente = filas[i].getAgente(ev.channel.split('-')[0]);
        // Agente encontrado
        if(agente){
          // Vamos ver se essa ligação já não está no agente...
          agente.getLigacao(ev.linkedid,(ligacao) => {
            // Nova Ligação
            if(!ligacao){
              const l = new Ligacao(ev.linkedid);
              switch (parseInt(ev.channelstate)) {
                case 4:
                  l.sentido = 'Sainte'
                  l.destino = ev.exten;
                  agente.status = 6;
                  break;
                case 5:
                  l.origem = ev.connectedlinenum;
                  l.origem_nome = ev.connectedlinename;
                  l.sentido = 'Entrante';
                  agente.status = 6;
                  break;
                case 6:
                  agente.status = 2;
                  break;
              }
              agente.update = 'Newstate';
              agente.StatusNome = agente.getStatusNome();
              agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
              l.status = parseInt(ev.channelstate);
              l.StatusNome = ChannelStateMap.get(parseInt(ev.channelstate));
              l.fila = filas[i].nome;
              if(!agente.ligacoes) agente.ligacoes = 0 ;
              agente.ligacoes++;
              agente.addOrUpdateLigacao(l,filas[i]);
              const ret = {event : 'AgenteStatus'};
              ret.agente = agente;
              ws.broadcast(ret);
              myEmitter.emit('AgenteStatus', ret);

              if(options.config.integracao_status_agente && options.config.integracao_status_agente.ativada){                
                // Processamento de integração de status de agente.
                try {
                  NotificaIntegracaoStatus(agente);
                } catch (ex){
                    log.error("INTEGRACAO_STATUS_AGENTE: Falha ao processar NotificaIntegracaoStatus(). ERRO: "+ ex.toString());
                }
              }
            }
            else {              
              ligacao.status = parseInt(ev.channelstate);
              ligacao.StatusNome = ChannelStateMap.get(parseInt(ev.channelstate));
              // Ligação completada
              if(ligacao.status === 6) agente.status = 2;
              agente.StatusNome = agente.getStatusNome();
              agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
              agente.addOrUpdateLigacao(ligacao,filas[i]);
            }
            // Publica evento
            const ret = {event : 'AgenteStatus'};
            ret.agente = agente;
            ws.broadcast(ret);
            myEmitter.emit('AgenteStatus', ret);

            if(options.config.integracao_status_agente && options.config.integracao_status_agente.ativada){                
              // Processamento de integração de status de agente.
              try {
                NotificaIntegracaoStatus(agente);
              } catch (ex){
                  log.error("INTEGRACAO_STATUS_AGENTE: Falha ao processar NotificaIntegracaoStatus(). ERRO: "+ ex.toString());
              }
            }
          });          
        }
      }
    } catch (ex){
        log.error("AMI_NEWSTATE: Falha ao processar Newstate. ERRO: " + ex.toString());
    }
  });

  myEmitter.on('Hangup', (ev) => {
    // Publica esse evento para Apps não SIP identificarem o evento.
    ws.broadcast(ev);
    //console.log('Hangup');
    //console.log(ev);
    const Evento = {
      Channel: ev.channel,
      ChannelState: ev.channelctate,
      ChannelStateDesc: ev.channelstatedesc,
      CallerIDNum: ev.calleridnum,
      CallerIDName: ev.calleridname,
      ConnectedLineNum: ev.connectedlinenum,
      ConnectedLineName: ev.connectedlinename,
      AccountCode: ev.accountcode,
      Context: ev.context,
      Exten: ev.exten,
      Priority: ev.priority,
      Uniqueid: ev.uniqueid,
      Linkedid: ev.linkedid,
      Cause: parseInt(ev.cause)
    };


    // Desativado pois fica sem motivo? q sem motivo
    setTimeout(function(){
      try {
        for(let i=0;i<filas.length;++i){
          // Asterisk BUG, ligações transferidas continuam no aguardando.
          const l = new Ligacao(Evento.Linkedid);
          filas[i].removeLigacao(l);

          const agente = filas[i].getAgente(Evento.Channel.split('-')[0]);
          if(agente){
            filas[i].efetuada = filas[i].efetuada ? filas[i].efetuada + 1 : 1;

            for (let j = 0; j < agente.ligacoes_ativas.length; ++j) {
              if (agente.ligacoes_ativas[j].uniqueid === Evento.Linkedid) {
                agente.ligacoes_ativas.splice(j, 1);
                // Verifica se é a última ligação na lista
                if(agente.ligacoes_ativas.length === 0){
                  agente.status = agente.pausa ? 10 : 1;
                  agente.StatusNome = agente.getStatusNome();
                  agente.DataStatus = moment.tz(options.config.servidor.timezone).format();
                  const ret = {event : 'FilasStatus'};
                  ret.filas = [];
                  ret.filas.push(filas[i]);
                  ws.broadcast(ret);
                  myEmitter.emit('FilasStatus', ret);
                }                
              }
            }
          }
        }
      } catch (ex){
          log.error("AMI_HANGUP: Falha ao processar Hangup. ERRO: " + ex.toString());
      }
    }, 5000);

    // Processamento do discador, no caso de abandono o hangup vem antes do evento de abandono dai remove a ligacao como se tivese completado.
    // Para contornar isso chamo 5s depois
    setTimeout(function(){
        log.info('AMI_HANGUP: Executando processamento de Hangup do discador depois de 5s');
        try {
          discador.ProcessaHangup(Evento);
        } catch (ex){
            log.error("AMI_HANGUP: Falha ao processar discador.ProcessaHangup. ERRO: " + ex.toString());
        }
    }, 5000);
    //try {
    //  discador.ProcessaHangup(Evento);
    //} catch (ex){
    //    console.log("Falha ao processar discador.ProcessaHangup: %s",ex.toString());
    //}


    /**
    ligacao.fim = moment.tz(options.config.servidor.timezone);

    if(ligacao.atendida){
      const duration = moment.duration(ligacao.fim.diff(ligacao.atendida));
      const espera = moment.duration(ligacao.atendida.diff(ligacao.inicio));
      ligacao.atendidaf = ligacao.atendida.format('DD-MM-YYYY HH:mm:ss');
      ligacao.espera = espera.get("hours").toString().padStart(2, '0') +":"+ espera.get("minutes").toString().padStart(2, '0') +":"+ espera.get("seconds").toString().padStart(2, '0');
      ligacao.falando = duration.get("hours").toString().padStart(2, '0') +":"+ duration.get("minutes").toString().padStart(2, '0') +":"+ duration.get("seconds").toString().padStart(2, '0');
    }
    else {
      const espera = moment.duration(ligacao.fim.diff(ligacao.inicio));
      ligacao.espera = espera.get("hours").toString().padStart(2, '0') +":"+ espera.get("minutes").toString().padStart(2, '0') +":"+ espera.get("seconds").toString().padStart(2, '0');
    }

    ligacao.iniciof = ligacao.inicio.format('DD-MM-YYYY HH:mm:ss');
    ligacao.fimf = ligacao.fim.format('DD-MM-YYYY HH:mm:ss');
    //_s().lpad("1", 8, "0");

    console.log(ligacao);
    **/

  });

  myEmitter.on('OriginateResponse', (event) => {
    try {
      discador.ProcessaRetornoOriginate(event);
    } catch (ex){
        log.error("AMI_ORIGINATERESPONSE: Falha ao processar discador.ProcessaRetornoOriginate. ERRO: " + ex.toString());
    }
  }); 

  myEmitter.on('AgenteAddChat', async (dados) =>{
    console.log('ADD CHAT ATIVO',dados);
    this.addChatAgente(dados.fila,dados.ramal,dados.protocolo);

  })

  myEmitter.on('AgenteRemoveChat', async (dados)=>{
    console.log('REMOVE CHAT ATIVO ??????????????????????????????', dados);
    this.removeChatAgente(dados.ramal,dados.protocolo);
  })
  
  exports.ws = ws;

  next();
};

exports.register.attributes = {
    name: 'xcontact-ami'
};


exports.XCeventos = myEmitter;