'use strict'
require("./util/default_error_catcher");

const Hapi = require("hapi");
const Boom = require('boom');
var server = new Hapi.Server();
const plugins = require('./plugins/plugins');
const config = require('./config.json');
const auth = require('./util/auth');
const dotenv = require('dotenv');
const log = require('./util/Log');
const path = require("path");
const glob = require("glob");
const ApiLog = require('./models/ApiLog');

dotenv.config();
process.setMaxListeners(0);

// Vamos usar env pra tudo? é muito mais fácil que ficar importando o config onde a gente precisa.
if (!process.env.LOG) process.env.LOG = config.whatsApi.log;
if (!process.env.WHATSAPI_ATIVADO) process.env.WHATSAPI_ATIVADO = config.whatsApi.ativado;
if (!process.env.WHATSAPI_HOST) process.env.WHATSAPI_HOST = config.whatsApi.host;
if (!process.env.WHATSAPI_CLIENTID) process.env.WHATSAPI_CLIENTID = config.whatsApi.clientID;
if (!process.env.WHATSAPI_NOME) process.env.WHATSAPI_NOME = config.whatsApi.nome;
if (!process.env.LOG) process.env.LOG = config.whatsApi.log;
if (!process.env.ZENVIA_ATIVADO) process.env.ZENVIA_ATIVADO = config.zenvia.ativado;
if (!process.env.ZENVIA_HOST) process.env.ZENVIA_HOST = config.zenvia.host;
if (!process.env.ZENVIA_CLIENTID) process.env.ZENVIA_CLIENTID = config.zenvia.clientID;
if (!process.env.GUPSHUP_ATIVADO) process.env.GUPSHUP_ATIVADO = config.gupshup.ativado;
if (!process.env.GUPSHUP_HOST) process.env.GUPSHUP_HOST = config.gupshup.host;
if (!process.env.GUPSHUP_CLIENTID) process.env.GUPSHUP_CLIENTID = config.gupshup.clientID;
if (!process.env.GUPSHUP_NOME) process.env.GUPSHUP_NOME = config.gupshup.nome;
if (!process.env.SWAGGER_PROTOCOLO) process.env.SWAGGER_PROTOCOLO = 'https';
if (!process.env.PORTA_REST) process.env.PORTA_REST = config.servidor.porta_rest;
if (!process.env.PORTA_WS) process.env.PORTA_WS = config.servidor.porta_ws;

let noemail = process.env && process.env.NO_EMAIL ? JSON.parse(process.env.NO_EMAIL) : false;
let nodiscador = process.env && process.env.NO_DISCADOR ? JSON.parse(process.env.NO_DISCADOR) : false;
let nochat = process.env && process.env.NO_CHAT ? JSON.parse(process.env.NO_CHAT) : false;
let nowhats = process.env && process.env.NO_WHATS ? JSON.parse(process.env.NO_WHATS) : false;

if (config.ssl || process.env.SSL) {
    console.log(`RODANDO em modo TLS`);
    const fs = require('fs');
    // Gere os arquivos SSL no servidor com o comando: openssl req -x509 -newkey rsa:4096 -keyout key.pem -out cert.pem -days 3650 -subj "/C=BR/ST=Rio grande do Sul/L=Porto Alegre/O=X5 Solucoes LTDA/OU=com/CN=xcontact.local"
    // informe a senha 2004
    const tls = {
        key: fs.readFileSync('./ssl/key.pem'),
        cert: fs.readFileSync('./ssl/cert.pem'),
        passphrase: '2004'
    };

    try {
        server.connection({ "host": "0.0.0.0", "port": process.env.PORTA_REST, labels: ['api'], tls: tls });
    } catch (e) {
        log.error(`[App] Erro de conexão na porta [API] ${process.env.PORTA_REST}!\n` + e);
    }

    try {
        server.connection({ "host": "0.0.0.0", "port": process.env.PORTA_WS, labels: ['ws'], tls: tls });
    } catch (e) {
        log.error(`[App] Erro de conexão na porta [WS] ${process.env.PORTA_WS}!\n` + e);
    }
} else {
    try {
        server.connection({ "host": "0.0.0.0", "port": process.env.PORTA_REST, labels: ['api'] });
    } catch (e) {
        log.error(`[App] Erro de conexão na porta [API] ${process.env.PORTA_REST}!\n` + e);
    }

    try {
        server.connection({ "host": "0.0.0.0", "port": process.env.PORTA_WS, labels: ['ws'] });
    } catch (e) {
        log.error(`[App] Erro de conexão na porta [WS] ${process.env.PORTA_WS}!\n` + e);
    }
}

if (process.env && process.env.DEV_HOST) {
    console.log(`DEV Host informado no arquivo .env ${process.env.DEV_HOST}`);
    config.database.host = process.env.DEV_HOST;
    config.asterisk.host = process.env.DEV_HOST;
}
if (process.env && process.env.WHATS_API_KEY) {
    config.whatsApi.clientID = process.env.WHATS_API_KEY;
}

//Exporta a versão corrente para a api
global.xcontact_version = "3.5.6-4";

// https://github.com/Salesflare/hapi-auth-bearer-simple
// O cliente precisa enviar --header 'Authorization: Bearer abc456789'
// Temos a API Key do admin definida no arquivo que tem acesso full.
// Temos o login do supervisor que retorna o token e modulos que o usuario tem acesso.
// Temos o cache sendo feito dos tokens validados no banco.
// Atualmente o serviço reinicia diariamente entao isso limpa o cache dos tokens (Rever esse restart)
// ---- Falta os metodos na API nova para mudar as permições do usuário e quando alteradas remover o token do cache.
// ---- Falta definir o scopo nas todas...Ex: scope: ['app.filas','admin'] o scopo admin deve estar em todas as rotas.
var validateFunction = function (token, callback) {
    auth.ValidaToken(token, (ret, erro) => {
        if (erro) {
            callback(Boom.unauthorized('API KEY Inválida. Use o metodo POST /usuarios/supervisores/login para obter seu token de acesso.'), false, null);
        } else {
            callback(null, true, ret);
        }
    });
};

var apiServer = server.select('api');

apiServer.register(require('hapi-auth-bearer-simple'), function (err) {
    if (err) {
        log.error("[App] Erro ao registrar plugin! " + err);
        console.trace(err);
        throw err;
    }
});

apiServer.auth.strategy('bearer', 'bearerAuth', { validateFunction: validateFunction });

// Cache do banco
// const dbCache = require('./caches/db/index')
// dbCache()

// Xcontact plugins
let modulos = [
    { register: require('./ws'), options: { config: config } },
    { register: require('./xc_modules/omni'), options: { config: config } },
    { register: require('./ami'), options: { config: config } },
    { register: require('./ligacao'), options: { config: config } },
];

if (!nodiscador) {
    log.info(`[App] Discador Ativado`);
    modulos.push({ register: require('./discador'), options: { config: config } });
}


if (config.email_ativo) {
    log.info(`[App] E-mail Ativado`);
    //modulos.push({ register: require('./router/email'), options: { config: config } });
    modulos.push({ register: require('./mail_notifier/index'), options: { config: config } });
}

// if (!nochat) {
//     modulos.push({ register: require('./router/webChat'), options: { config: config } });
// }

// if (!nowhats) {
//     modulos.push({ register: require('./router/whatsApi'), options: { config: config } });
//     modulos.push({ register: require('./router/zenvia'), options: { config: config } });
// }

// modulos.push({ register: require('./router/zenvia'), options: { config: config } });

server.register(modulos, err => {
    if (err) {
        log.error("[App] Erro ao carregar módulos!" + err);
        console.trace(err);
        throw err;
    }
});

apiServer.register(plugins, err => {
    if (err) {
        log.error("[App] Erro ao registrar plugins! " + err);
        console.trace(err);
        throw err;
    }
});

// Importa as rotas manualmente, assim observando erros e reportando caso ocorram
// Substitui o plugin hapi-router
glob(`rest_api/**/routes.js`, { cwd: __dirname }, (error, files) => {
    if (error) {
        log.error(`[App] Erro ao carregar rotas api rest -> `, error);
    } else {
        files.forEach(file => {
            try {
                apiServer.route(require(path.join(__dirname, file)));
            } catch (error) {
                log.error(`[App] Erro ao definir '${file}' api rest -> `, error);
            }
        });
    }
});

// Asterisk ARI, Nao tem evento de fila...Mas pra status de ramal parece ser bom.
//server.register(require('./ari'), function (err) {
//    if (err) {
//        throw err;
//    }
//});

require('./bots/autopausa')

server.on('response', async (response) => {
    if (response.path.includes('api') && response.method != 'options') {
        //console.log(response);
        const log = {
            rota: response.path,
            RotaDescricao: response.route.settings.description,
            metodo: response.method,
            ip: response.headers['x-real-ip'] ? response.headers['x-real-ip'] : response.info.remoteAddress,
            pc: (response.id && response.id.split(':').length > 0) ? response.id.split(':')[1] : response.id,
            query: JSON.parse(JSON.stringify(response.query)),
            payload: (response.mime != 'multipart/form-data') ? JSON.parse(JSON.stringify(response.payload)) : null,
            parametros: JSON.parse(JSON.stringify(response.params)),
            status: response.response.statusCode
        }

        if (log.payload && log.payload.senha) log.payload.senha = '*******';

        if (response.auth && response.auth.credentials) {
            log.UserID = response.auth.credentials.id;
            log.UserLogin = response.auth.credentials.login;
        }

        // Pegar dados de retorno somente de post, put e delete para log
        if (['post', 'put', 'delete'].includes(response.method)) {
            if (response.mime != 'multipart/form-data' && response.response && response.response._payload) {
                log.retorno = JSON.parse(response.response._payload._data);
                if (log.retorno && log.retorno.senha) log.retorno.senha = '*******';
            }
        }

        try {
            const apilog = await ApiLog.insert(log);
        } catch (error) {
            console.log('Erro ao registrar o log da API: ', error);
        }
    }
});

try {
    server.start()
        .then(() => {
            log.info("Servidor rodando em " + apiServer.info.uri);
        })
        .catch(error => {
            log.error("[App] Erro ao iniciar servidor! " + error);
            console.trace(error);

            if (error && error.code === "EADDRINUSE") {
                process.exit(1);
            }
        });
} catch (error) {
    log.error(`[App] Erro ao iniciar o server!\n` + error);
}