
const axios = require('axios');
const https = require('https');
const vm = require('vm');
const flatten = require('flat');
const qs = require('qs');
const EventEmitter = require('events');
const SessoesUra = require('../../models/SessoesUra');
const fs = require('fs');
const db = require('../../util/db');
const Horarios = require('../../models/Horarios');
const ami = require("../../ami/index");
const log = require("../../util/Log");
const TextoUtils = require("../../util/texto");
const uraTextoBlocos = require('../../models/uraTextoBlocos');
const uraTexto = require('../../models/uraTexto');
const Agente = require('../../models/Agente');
let sessionMessages = {    
  };
  
class MyEmitter extends EventEmitter { }

module.exports = class UraProcessor {

    constructor() {
        this.emitter = new MyEmitter();
    }

    on(event, fn) {
        return this.emitter.addListener(event, fn);
    }

    loadFile(urafile) {
        try {
            let rawdata = fs.readFileSync(urafile);
            this.urafile = JSON.parse(rawdata);
        } catch (ex) {
            console.trace(ex);
            log.error(`Erro ao ler arquivo de ura -> ${ex}`);
        }
    }

    async loadFromDb(id) {
        try {
            const urapar = await uraTexto.fetchById(id);
            this.expire = urapar.expire;
            this.expire_msg = urapar.expireMsg;

            const blocos = await uraTextoBlocos.query
                .where({ uraTextoID: id })
                .require(true)
                .fetch();

            const jb = blocos.map(b => {
                const bloco = {};
                bloco.sequencia = b.sequencia;
                bloco.tipo = b.tipo;
                for (let [key, value] of Object.entries(b.bloco)) {
                    if (value) bloco[key] = value;
                }
                if (b.app) bloco[b.app] = b.appDados;
                if (b.falha_goto) bloco.falha_goto = b.falha_goto;
                return bloco;
            });
            this.urafile = jb;
        } catch (ex) {
            console.trace(ex);
            log.error(`Erro ao ler dados da URA do banco -> ${ex}`);
        }
    }

    setTimeExpire(minutes) {
        this.expire = minutes;
    }

    setTimeExpireMsg(msg) {
        this.expire_msg = msg;
    }
    setFilaTransbordo(fila) {
        this.filaTransbordo = fila;
    }
    async startContador(iduser, route_data, session) {
        setTimeout(async () => {
            let sessao = await SessoesUra.query.require(false).first()
                .where({ iduser: iduser })
                .where({ protocolo: route_data.atendimentoId })
                .fetch();
            if (!sessao) return;
            if (this.filaTransbordo) {
                this.emitter.emit('UraGotoFila', { idCliente: iduser, fila: this.filaTransbordo, msg: this.expire_msg, route_data: route_data });
                if (this.expire_msg) this.emitter.emit('UraMsg', { idCliente: iduser, msg: this.expire_msg, route_data: route_data });
                return await session.delete();
            }
            await session.delete();
            session = await this._getOrCreateUserSession(iduser, route_data);
            if (this.expire_msg) this.emitter.emit('UraMsg', { idCliente: iduser, msg: this.expire_msg, route_data: route_data });
            this.emitter.emit('UraFim', { idCliente: iduser, msg: this.expire_msg, route_data: route_data });
            await session.delete();
        }, this.expire * 60000);
    }

    /**
     * Informe os dados para simular o retorno da API em cada sequencia que a URA consultaria uma API.
     * Informe no padrão: [{sequencia: NUMERO_SEQUENCIA, data: OBJETO_DE_DADOS}]
     * @param {Array} moke
     */
    setMokeApi(moke) {
        this.moke = moke;
    }

    async _getOrCreateUserSession(iduser, route_data = null) {
        try {
            console.log('Bora pega a sessao', route_data.atendimentoId);
            let sessao = await SessoesUra.query.require(true).first()
                .where({ iduser: iduser })
                .where({ protocolo: route_data.atendimentoId })
                .fetch();
            console.log(`SESSAO DE URA`, sessao);
            if (typeof sessao.dados === 'string') sessao.dados = JSON.parse(sessao.dados);
            if (!sessao.dados) sessao.dados = {};

            sessao.dados["$CLIENTE_NUMERO"] = iduser;
            sessao.dados["$CLIENTE_NOME"] = route_data.cliente.nome;
            sessao.dados["$CLIENTE_ID"] =  route_data.cliente.id;
            if (route_data && route_data.atendimentoId) {
                sessao.dados["$PROTOCOLO"] = route_data.atendimentoId;
            }
            return sessao;
        } catch (error) {
            console.log(error);
            try {
                let sessao = await new SessoesUra({
                    iduser: iduser,
                    sequencia: 1,
                    interacoes: 1,
                    protocolo: route_data.atendimentoId,
                    dados: {
                        "creatAt": new Date()
                    }
                }).insert();
                sessao.dados["$CLIENTE_NUMERO"] = iduser;
                sessao.dados["$CLIENTE_NOME"] = route_data.cliente.nome;
                sessao.dados["$CLIENTE_ID"] =  route_data.cliente.id;
                if (route_data && route_data.atendimentoId) {
                    sessao.dados["$PROTOCOLO"] = route_data.atendimentoId;
                }
                if (route_data) {
                    this.startContador(iduser, route_data, sessao);
                }
                console.log('CRIAMOS A SESSAO', route_data.atendimentoId);
                return sessao;
            } catch (error) {
                console.error(error);
            }
        }
    }

    /**
     * Injects a variable in a text containing the following format {{$VAR_NAME}}
     *
     * @param {String} texto
     * @param {Array<any>} varlist
     */
    _injectVariablesInText(texto, varlist) {
        const regex = /\{\{(.*?)\}\}/g;

        let variables = Object.assign({}, varlist);
        variables["$DADOS"] = JSON.stringify(varlist);
        variables = flatten(variables);

        return texto.replace(regex, (_, group1) => variables[group1] ? variables[group1] : '');
    }

    async routeMsg(iduser, msg, route_data, tipoMensagem = "texto") {
        if (!this.urafile) {
            this.emitter.emit('UraErro', { idCliente: iduser, msg: "Arquivo de ura não definido ou inválido.", route_data: route_data });
        }

        var session = await this._getOrCreateUserSession(iduser, route_data);

        // Validar time da sessao se tiver definido
        if (this.expire) {
            let d1 = new Date(session.createAt);
            let d2 = new Date();
            let difference = d2.getTime() - d1.getTime();
            let minutos = Math.round(difference / 60000);
            if (minutos > this.expire) {
                if (this.filaTransbordo) {
                    this.emitter.emit('UraGotoFila', { idCliente: iduser, fila: this.filaTransbordo, msg: this.expire_msg, route_data: route_data });
                    if (this.expire_msg) this.emitter.emit('UraMsg', { idCliente: iduser, msg: this.expire_msg, route_data: route_data });
                    return await session.delete();
                }
                await session.delete();
                session = await this._getOrCreateUserSession(iduser, route_data);
                if (this.expire_msg) this.emitter.emit('UraMsg', { idCliente: iduser, msg: this.expire_msg, route_data: route_data });
            }
        }

        // Pega o bloco a ser executado
        var menu = this.urafile.find(ura => ura.sequencia == session.sequencia);
        if (!menu) {
            // this.emitter.emit('UraMsg', { idCliente: iduser, msg: "Menu não encontrado", route_data: route_data });
            console.log(`[MENU NAO ENCONTRADO] ${iduser} Atendimento: ${route_data.atendimentoId} bloco: ${session.sequencia} DATA: ${new Date()}`);
            session.sequencia = 1;
            await session.save({ require: false });
            // return;
            //Vamos botar bara a sequencia 1 ao inves de dar menu nao encontrado
            menu = this.urafile.find(ura => ura.sequencia == session.sequencia);
        }


        if (menu.retorno) {
            // Formata variaveis de retorno se tiver
            try {
                if (typeof menu.retorno !== 'object') {
                    //menu.retorno = JSON.parse(menu.retorno);
                    if (menu.retorno.includes(",")) {
                        var variaveis = menu.retorno.split(',');
                        menu.retorno = {};
                        variaveis.forEach(v => {
                            menu.retorno[v] = '';
                        });
                    } else {
                        var newretorno = {};
                        newretorno[menu.retorno] = '';
                        menu.retorno = newretorno;
                    }
                }
            } catch (er) {
                this.emitter.emit('UraErro', { idCliente: iduser, msg: "Erro ao converter menu.retorno: " + er.message, route_data: route_data });
                return;
            }
        }

        this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'routeMsg', msg: `Recebi ${msg} de ${session.iduser}, Next tipo: ${menu.tipo}, Sequencia: ${menu.sequencia}`, route_data: route_data });

        // Validação de dados de dados de entrada
        if (menu.validar && menu.validar.funcao) {
            let resultadoValidacao = await this._processValidations(session, menu, msg, route_data, tipoMensagem);

            if (!resultadoValidacao) return;
        }

        switch (menu.tipo.toLowerCase()) {

            case 'api':
                this._processApi(session, menu, msg, route_data);
                break;

            case 'menu':
                this._processMenu(session, menu, msg, route_data);
                break;

            case 'pergunta':
                this._processQuestion(session, menu, msg, route_data);
                break;

            case 'mensagem':
                this._processMessage(session, menu, msg, route_data);
                break;

            case 'sql':
                this._processSql(session, menu, msg, route_data);
                break;

            case 'gpt':
                this._processGpt(session, menu, msg, route_data);
                break;
        }
    }

    /**
     * Goto the specified sequence
     *
     * @param {Number} menu_sequence
     */
    async _gotoSequence(session, menu_sequence, route_data) {
        this.emitter.emit('UraDebug', {
            idCliente: session.iduser,
            debug: 'UraGotoSequence',
            msg: `GOTO PARA A SEQUÊNCIA -> ${menu_sequence}`,
            route_data: route_data
        });

        session.sequencia = menu_sequence;
        session.interacoes = 1;
        await session.save({ require: false });

        this.routeMsg(session.iduser, "Next", route_data);
    }

    /**
     * Process the URA validations, before proceeding further
     * Returns a boolean, indicating whether the validation returned valid
     *
     * @param {*} session
     * @param {*} menu
     * @param {*} route_data
     * @returns {Boolean} True when all validations are fulfilled
     */
    async _processValidations(session, menu, msg, route_data, tipoMensagem) {
        if (!menu.falha_goto) menu.falha_goto = menu.validar.falha_goto;
        switch (menu.validar.funcao) {

            case "TAMANHO":
                // Só executa na segunda+ iteração
                //if (session.interacoes <= 1) return true;

                this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'ValidacaoDeEntrada', msg: `Validar ${msg} de ${session.iduser} por TAMANHO >= ${menu.validar.dados}`, route_data: route_data });

                if (menu.coletar) {
                    this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'ValidacaoDeEntrada', msg: `Coletando o valor '${msg}' na variável '${menu.coletar}'` });
                    session.dados[menu.coletar] = msg;
                    await session.save({ require: false });
                }

                if (msg.length < parseInt(menu.validar.dados)) {
                    this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'ValidacaoDeEntrada', msg: `MSG ${msg} de ${session.iduser} INVALIDA por TAMANHO, Tamanho Msg: ${msg.length}, esperado ${menu.validar.dados}`, route_data: route_data });
                    // Verifica se existe um bloco a ser executado em caso de falhas
                    if (menu.falha_goto) {
                        await this._gotoSequence(session, menu.falha_goto, route_data);
                    }
                    return false;
                }

                break;

            case "CNPJ":
                // Só executa na segunda+ iteração
                if (session.interacoes <= 1) return true;

                this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'ValidacaoDeEntrada', msg: `Validar ${msg} de ${session.iduser} por CNPJ`, route_data: route_data });
                // Se o bloco atual precisa coletar algo, executa-o
                if (menu.coletar) {
                    this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'ValidacaoDeEntrada', msg: `Coletando o valor '${msg}' na variável '${menu.coletar}'` });
                    session.dados[menu.coletar] = msg;
                    await session.save({ require: false });
                }

                if (!TextoUtils.validarCNPJ(msg)) {
                    this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'ValidacaoDeEntrada', msg: `MSG ${msg} de ${session.iduser} INVALIDA por CNPJ INVÁLIDO`, route_data: route_data });
                    if (menu.falha_goto) {
                        await this._gotoSequence(session, menu.falha_goto, route_data);
                    }
                    return false;
                }

                break;

            case "CPF":
                // Só executa na segunda+ iteração
                //if (session.interacoes <= 1) return true;

                this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'ValidacaoDeEntrada', msg: `Validar ${msg} de ${session.iduser} por CPF`, route_data: route_data });

                if (menu.coletar) {
                    this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'ValidacaoDeEntrada', msg: `Coletando o valor '${msg}' na variável '${menu.coletar}'` });
                    session.dados[menu.coletar] = msg;
                    await session.save({ require: false });
                }

                if (!TextoUtils.validarCPF(msg)) {
                    this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'ValidacaoDeEntrada', msg: `MSG ${msg} de ${session.iduser} INVALIDA por CPF INVÁLIDO`, route_data: route_data });

                    // Verifica se existe um bloco a ser executado em caso de falhas
                    if (menu.falha_goto) {
                        await this._gotoSequence(session, menu.falha_goto, route_data);
                    }

                    return false;
                }

                break;

            case "CPF_CNPJ":
                // Só executa na segunda+ iteração
                if (session.interacoes <= 1) return true;

                this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'ValidacaoDeEntrada', msg: `Validar ${msg} de ${session.iduser} por CPF_CNPJ`, route_data: route_data });

                if (menu.coletar) {
                    this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'ValidacaoDeEntrada', msg: `Coletando o valor '${msg}' na variável '${menu.coletar}'` });
                    session.dados[menu.coletar] = msg;
                    await session.save({ require: false });
                }

                if (!TextoUtils.validarCPF(msg) && !TextoUtils.validarCNPJ(msg)) {
                    this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'ValidacaoDeEntrada', msg: `MSG ${msg} de ${session.iduser} INVALIDA por CPF_CNPJ INVÁLIDO`, route_data: route_data });
                    if (menu.falha_goto) {
                        await this._gotoSequence(session, menu.falha_goto, route_data);
                    }
                    return false;
                }
                break;

            case "TURNO":
                // Só executa na primeira iteração
                //if (session.interacoes > 1) return true;

                // Puxa os turnos da validação
                let turnos = menu.validar.dados;
                if (turnos && !Array.isArray(turnos)) turnos = [parseInt(turnos)];

                if (menu.coletar) {
                    this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'ValidacaoDeEntrada', msg: `Coletando o valor '${msg}' na variável '${menu.coletar}'` });
                    session.dados[menu.coletar] = msg;
                    await session.save({ require: false });
                }

                // Emite o log de debug
                this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'ValidacaoDeTurnos', msg: `Validar turnos ${turnos}`, route_data: route_data });

                // Valida os turnos
                let turnosValidos = await Horarios.verificarTurnos(turnos);

                if (turnosValidos) {
                    this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'ValidacaoDeTurnos', msg: `Turnos ${turnos} válidos!`, route_data: route_data });
                } else {
                    this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'ValidacaoDeTurnos', msg: `Turnos ${turnos} inválidos! Finalizando...`, route_data: route_data });

                    // Verifica se existe um bloco a ser executado em caso de falhas
                    if (!menu.falha_goto) menu.falha_goto = menu.validar.falha_goto;
                    if (menu.falha_goto) {
                        await this._gotoSequence(session, menu.falha_goto, route_data);
                    }
                }
                return turnosValidos;
            case "TIPO":
                try {
                    if (session.interacoes <= 1) return true;
                    let valor = this._injectVariablesInText(menu.validar.valor, session.dados);
                    console.log(tipoMensagem);
                    console.log(valor);
                    if (tipoMensagem && tipoMensagem.indexOf(`${valor}`) !== -1) {
                        if (menu.coletar) {
                            session.dados[menu.coletar] = msg;
                        }
                        this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'ValidacaoDeTipo', msg: `Tipo ${valor} válidos!`, route_data: route_data });

                        return true;
                        //await this._gotoSequence(session, menu.goto, route_data);

                    } else {
                        if (menu.falha_goto) {
                            this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'ValidacaoDeTipo', msg: `Tipo ${valor} inválidos!`, route_data: route_data });

                            await this._gotoSequence(session, menu.falha_goto, route_data);
                        }
                        return false;
                    }
                } catch (error) {
                    console.error(error);
                    this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'ValidacaoDeTipoERRO', msg: error, route_data: route_data });

                }

            case "BUSCAR_AGENTE":
                // Só executa na primeira iteração
                //if (session.interacoes > 1) return true;

                try {
                    // Injeta as variáveis nos valores
                    let coluna = this._injectVariablesInText(menu.validar.coluna, session.dados);
                    let valor = this._injectVariablesInText(menu.validar.valor, session.dados);
                    let fila = this._injectVariablesInText(menu.validar.fila, session.dados);

                    // Busca os agentes disponíveis na fila especificada
                    let agentes = ami.getAgentesOnline(fila);

                    // Busca um agente que possua o valor especificado em uma dada propriedade
                    if (agentes) {
                        let agente = agentes.find(x => x && x.data && x.data[coluna] == valor);

                        if (menu.retorno) {
                            session.dados[menu.retorno] = agente;
                            await session.save({ require: false });
                        }
                        return true;
                    } else {
                        this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'UraSearchAgent', msg: `Não foi possível pesquisar pelo agente '${coluna}': '${valor}' na fila '${fila}'` });
                        return false;
                    }
                } catch (error) {
                    console.error(error);
                    return false;
                }
        }

        return true;
    }

    //#region URA block processing
    async _processQuestion(session, menu, msg, route_data) {

        if (session.interacoes === 1) {
            session.interacoes++;

            if (menu.texto) {
                this.emitter.emit('UraMsg', {
                    idCliente: session.iduser,
                    msg: this._injectVariablesInText(menu.texto, session.dados),
                    route_data: route_data
                });
            }

            await session.save({ require: false });
            return;
        }

        if (menu.coletar) {
            this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'UraProcessQuestion', msg: `Coletando o valor '${msg}' na variável '${menu.coletar}'` });
            session.dados[menu.coletar] = msg;
            await session.save({ require: false });
        }

        this.next(session, menu, route_data);
    }

    async _processMenu(session, menu, msg, route_data) {

        var sub = menu.opcoes.find(el => {
            if (el.esperado.includes(",")) {
                var opcoes = el.esperado.split(',');
                return opcoes.includes(msg);
            } else {
                return el.esperado == msg;
            }
        });
        // Testar opções erradas..!!!!

        if (!sub && session.interacoes > 1) {
            // Resposta errada
            if (menu.falha) this.emitter.emit('UraMsg', { idCliente: session.iduser, msg: menu.falha, route_data: route_data });

            session.interacoes++;
            session.save({ require: false });
            return;
        }

        if (!sub && session.interacoes === 1) {
            session.interacoes++;
            // Primeira interacao do usuário com o nível
            // if (menu.falha) this.emitter.emit('UraMsg', { idCliente: session.iduser, msg: menu.falha, route_data: route_data });
            await session.save({ require: false });
            this.emitter.emit('UraMsg', {
                idCliente: session.iduser,
                msg: this._injectVariablesInText(menu.texto, session.dados),
                route_data: route_data,
                opcoes: menu.opcoes,
                tipo: menu.estilo,
                header: menu.header ? this._injectVariablesInText(menu.header, session.dados) : null,
                footer: menu.footer ? this._injectVariablesInText(menu.footer, session.dados) : null,
            });

            var subs = "";
            if (!menu.estilo || menu.estilo == 'texto') {
                menu.opcoes.forEach(sub => subs += this._injectVariablesInText(sub.texto, session.dados) + "\n");
                this.emitter.emit('UraMsg', { idCliente: session.iduser, msg: subs, route_data: route_data, message: menu.message });
            }
            return;
        }

        // Se foi informado para coletar o que o usuário digitou, coleta na sessão.
        if (menu.coletar) {
            session.dados[menu.coletar] = msg;
            await session.save({ require: false });
        }

        if (menu.resposta) {
            sub.resposta = menu.resposta;
        }

        this.next(session, sub, route_data);
    }

    async _processGpt(session, menu, msg, route_data) { 
           
        async function askGPT(sessionId, message, context) {
          if (!sessionMessages[sessionId]) {
            sessionMessages[sessionId] = [
                {
                    role: 'user',
                    content: msg
                },
                {
                    role:'system', content:context
                }
            ]
          }         
          sessionMessages[sessionId].push({
            role: 'user',
            content: msg
          });
          try {
            const apiKey = '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';
            const url = 'https://api.openai.com/v1/chat/completions';
            const data = {
                messages: sessionMessages[sessionId] ,
                model: 'gpt-3.5-turbo',
                temperature: 0.7,
                top_p: 1,
                frequency_penalty: 0,
                presence_penalty: 0                
              };
              
              const headers = {
                'Content-Type': 'application/json',
                'Authorization': `Bearer ${apiKey}`
              };
            const response = await axios.post(url,data, {headers});        
            const reply = response.data.choices[0].message.content.trim();        
            // Adicione a resposta do GPT à sessão
            sessionMessages[sessionId].push({
                role: 'assistant',
                content: response.data.choices[0].message.content.trim()
            });        
            const intent =  identifyIntent(response.data.choices[0].message.content.trim())      
            return { reply, intent };
          } catch (error) {
            console.error("Erro ao consultar a OpenAI:", error);
            return {
              reply: "Desculpe, não consegui processar sua solicitação.",
              intent: null,
            };
          }
        }
        
        function identifyIntent(message) {
          if (message == "Falar com atendente") {
            return menu.fila_transbordo;
          }
        }
        
        // Exemplo de uso
        (async () => {
            console.log('VAMOS CHAMAR AQUi')
          const sessionId = session.iduser; // Substitua por um identificador de sessão real
          const { reply, intent } = await askGPT(sessionId, msg, menu.contexto);
          
        if (session.interacoes > 4) {
            this.emitter.emit('UraMsg', {
                idCliente: session.iduser,
                msg:`*${menu.nome_bot}*:\n`+reply,
                route_data: route_data,
                tipo: 'botao',
                opcoes: [{texto	: 'Falar com atendente'}],
                header: '',
                footer: 'assistente virtual',
            });
        }else {
            this.emitter.emit('UraMsg', {
                idCliente: session.iduser,
                msg:`*${menu.nome_bot}*:\n`+reply,
                route_data: route_data,
            });
        }
        // this.emitter.emit('UraMsg', {
        //     msg: 'Falar com humano',
        //     idCliente: session.iduser,
        //     route_data: route_data,
        //     opcoes: [{texto	: 'Falar com humano'}],
        //     tipo: 'botao',
        //     header: 'O assistente te ajudou?',
        //     footer: 'assistente virtual',
        // });
        session.interacoes++;
        await session.save({ require: false });
          if (intent) this.emitter.emit('UraGotoFila', { idCliente: session.iduser, fila: intent, msg: reply, route_data: route_data });
          const temTrans = identifyIntent(msg)
          if (temTrans) this.emitter.emit('UraGotoFila', { idCliente: session.iduser, fila: temTrans, msg: reply, route_data: route_data });
          console.log(reply, intent);
        })();        
    }
    async _processApi(session, menu, msg, route_data) {
        // Deixei os blocos mais limpos executando apenas sua função, se quiser exibir uma MSG antes de consultar uma API use um bloco de Msg, mesma coisa pra falha.

        // Incrementa interracoes
        session.interacoes++;

        // Se foi informado para coletar o que o usuário digitou, coleta na sessão.
        if (menu.coletar) session.dados[menu.coletar] = msg;

        if (this.moke && this.moke.length > 0) {
            const moke = this.moke.find(mk => mk.sequencia === session.sequencia);
            menu.retorno.data = moke.data;
        } else {
            // Troca variaveis definidas na URL por valor da sessão se tiver.
            var url = this._injectVariablesInText(menu.url, session.dados);

            try {
                const apiConfig = {
                    method: menu.method,
                    url: url,
                    responseType: menu.responseType
                };

                if (menu.headers) {
                    menu.headers = JSON.parse(this._injectVariablesInText(JSON.stringify(menu.headers), session.dados));
                    apiConfig.headers = menu.headers;
                }

                if (menu.auth) apiConfig.auth = menu.auth;

                if (menu.data) {
                    if (menu.responseType == 'document') {
                        apiConfig.data = qs.stringify(apiConfig.data);
                    } else if (menu.responseType == 'text') {
                        apiConfig.data = this._injectVariablesInText(JSON.stringify(menu.data), session.dados);
                    } else {
                        apiConfig.data = JSON.parse(this._injectVariablesInText(JSON.stringify(menu.data), session.dados));
                        console.log('INJETEI',JSON.stringify(apiConfig.data))
                    }
                }

                this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'apiConfig', msg: apiConfig, route_data: route_data });
                this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'apiConfig', msg: session.dados, route_data: route_data });

                if (menu.ssl_self_signed) apiConfig.httpsAgent = new https.Agent({ rejectUnauthorized: false });

                var ret = await axios(apiConfig);

                if (!ret || !ret.data || (Array.isArray(ret.data) && ret.data.length <= 0)) {
                    this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'apiRetorno', msg: 'Api não retornou registros', route_data: route_data });

                    await session.save({ require: false });

                    // Erro da API retorna para sequencia definida.
                    if (menu.falha_goto) {
                        // Vai pra proxima
                        await this._gotoSequence(session, menu.falha_goto, route_data);
                    }
                    return;
                }

                if (menu.retorno) menu.retorno.data = ret.data;
                this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'apiRetorno', msg: ret.data, route_data: route_data });
            } catch (error) {
                //console.trace(error);
                //Api retornou erro, considera resposta errada.
                this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'apiErro', msg: error.message, route_data: route_data });

                await session.save({ require: false });

                // Erro da API retorna para sequencia definida.
                if (menu.falha_goto) {
                    // Vai pra proxima
                    await this._gotoSequence(session, menu.falha_goto, route_data);
                }
                return;
            }
        }
        // conversao do retorno usando código no script se definido
        if (menu.script) {
            try {
                const script = new vm.Script(menu.script);
                vm.createContext(menu.retorno);
                script.runInContext(menu.retorno);
            } catch (er) {
                //console.trace(er);
                //console.error(er);
                this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'scriptErro', msg: 'ERRO: Falha ao executar o script: ' + er.message, route_data: route_data });
            }
        }

        if (menu.retorno) {
            this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'uraRetorno', msg: menu.retorno, route_data: route_data });
            // Salva variaveis definidas no retorno na sessao
            Object.keys(menu.retorno).forEach(key => {
                if (key !== 'data') {
                    session.dados[key] = menu.retorno[key];
                }
            });
        }

        this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'sessao', msg: session, route_data: route_data });

        await session.save({ require: false });

        // Se não acabou vai pra proxima...
        this.next(session, menu, route_data);
    }

    async _processSql(session, menu, msg, route_data) {
        // Valida os argumentos do menu
        if (!menu.query) return this.emitter.emit('UraErro', { idCliente: session.iduser, debug: 'UraProcessSql', msg: `Argumentos em falta!\nEnvie: *query*` });

        // Incrementa a interação
        session.interacoes++;
        await session.save({ require: false });

        // Se especificado um texto a ser apresentado, mostra-o antes de realizar a query
        if (menu.texto) {
            this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'UraProcessSql', msg: `Enviando mensagem...` });

            this.emitter.emit('UraMsg', {
                idCliente: session.iduser,
                msg: this._injectVariablesInText(menu.texto, session.dados),
                route_data: route_data
            });
        }

        try {
            // Processa query
            let query = this._injectVariablesInText(menu.query, session.dados);
            this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'UraProcessSql', msg: `Processando query -> ${query}` });

            // Executa a query
            let result = await db.query(query);

            // Verifica se a query foi bem sucedida
            if (result && result.rows && result.rows.length > 0 && menu.coletar) {
                this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'UraProcessSql', msg: `Recebi da query -> ${JSON.stringify(result.rows)}` });

                // Salva o resultado na variável de coleta
                session.dados[menu.coletar] = result.rows.length === 1 ? result.rows[0] : result.rows;
            } else {
                this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'UraProcessSql', msg: `Não recebi nada da query` });
            }

            // Vai para o próximo bloco
            this.next(session, menu, route_data);
        } catch (error) {
            console.trace(error);
            this.emitter.emit('UraErro', { msg: `Erro ao executar bloco de SQL '${session.sequencia}' -> ${error}` });

            // Em caso de erro, se especificado, redireciona para uma rota
            if (menu.falha_goto) this._gotoSequence(session, menu.falha_goto, route_data);
        }
    }

    async _processMessage(session, menu, msg, route_data) {
        session.interacoes++;
        if (menu.texto) {
            this.emitter.emit('UraMsg', {
                idCliente: session.iduser,
                msg: this._injectVariablesInText(menu.texto, session.dados),
                route_data: route_data
            });
        }

        if (menu.coletar) {
            this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'UraProcessQuestion', msg: `Coletando o valor '${msg}' na variável '${menu.coletar}'` });
            session.dados[menu.coletar] = msg;
        }

        await session.save({ require: false });

        this.next(session, menu, route_data);
    }
    //#endregion

    async next(session, from, route_data) {
        try {
            // Espera antes de seguir
            if (from.sleep) await this._handleSleep(session, from, route_data);

            // Emite o evento para redirecionar para outra fila e finaliza a sessão.
            if (from.goto_fila) return await this._handleGotoQueue(session, from, route_data);

            // Emite o evento para redirecionar para um agente específico (Caso o mesmo esteja disponível) e finaliza a sessão.
            if (from.goto_agente) return await this._handleGotoAgent(session, from, route_data);

            // Emite o evento de fim e finaliza a sessão
            if (from.fim) return await this._handleEndSession(session, from, route_data);

            // Redireciona um bloco de acordo com uma condição
            if (from.goto_if) return await this._handleGotoIf(session, from, route_data);

            // Redireciona para um bloco
            if (from.goto) return await this._handleGoto(session, from, route_data);

        } catch (error) {
            console.trace(error);
            this.emitter.emit('UraErro', { msg: `Erro ao processar última iteração da sequência '${session.sequencia}' -> ${error}` });
        }
    }

    //#region URA sub-functions handling
    _handleSleep(session, from, route_data) {
        return new Promise((resolve, reject) => {
            let amount = (from.sleep || 0) * 1000;

            this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'UraSleep', msg: `Esperando por -> ${amount}`, route_data: route_data });

            setTimeout(() => {
                this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'UraSleep', msg: `Esperando por -> ${amount} [DONE]`, route_data: route_data });

                resolve();
            }, amount);
        });
    }

    async _handleGotoQueue(session, from, route_data) {
        let texto = from.texto ? this._injectVariablesInText(from.texto, session.dados) : null;
        let fila = this._injectVariablesInText(from.goto_fila, session.dados);

        this.emitter.emit('UraGotoFila', { idCliente: session.iduser, fila: fila, msg: texto, route_data: route_data });
        // Se vai pra fila finaliza a sessão
        await session.delete({ require: false });
    }

    async _handleGotoAgent(session, from, route_data) {
        // Valida os argumentos
        try {
            if (!from.goto_agente.agente || !from.goto_agente.fila) {
                this.emitter.emit('UraErro', { idCliente: iduser, msg: "Erro ao executar 'goto_agente'!\nArgumentos em falta, você precisa enviar 'agente' e 'fila'", route_data: route_data });
                return;
            }

            this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'UraGotoAgente', msg: `Recebi -> ${JSON.stringify(from.goto_agente)}`, route_data: route_data });

            // Puxa todos os agentes online na fila especificada
            let agentesOnline = ami.getAgentesOnline(from.goto_agente.fila) || [];

            this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'UraGotoAgente', msg: `Agentes online -> ${JSON.stringify(agentesOnline)}`, route_data: route_data });

            // Verifica se o agente especificado está disponível
            let nomeAgente = this._injectVariablesInText(from.goto_agente.agente, session.dados);
            let agente = agentesOnline.find(x => x.nome === nomeAgente);
            let ag = await Agente.fetchByFullname(nomeAgente);

            if (agente || (from.goto_agente.offline && ag)) {
                this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'UraGotoAgente', msg: `Agente disponível!`, route_data: route_data });
                if (!agente) {
                    agente = {};
                    agente.nome = nomeAgente;
                }
                // Ja envia a resposta de finalização, assim não precisa de timeout nos eventos.
                let resposta = from.resposta ? this._injectVariablesInText(from.resposta, session.dados) : null;

                this.emitter.emit('UraGotoAgente', {
                    idCliente: session.iduser,
                    fila: from.goto_agente.fila,
                    agente: agente,
                    msg: resposta,
                    route_data: route_data
                });
                await session.delete({ require: false });
            } else {
                this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'UraGotoAgente', msg: `Agente indisponível!`, route_data: route_data });

                // Em caso de erro, se especificado, redireciona para uma rota
                if (from.falha_goto) this._gotoSequence(session, from.falha_goto, route_data);
            }
        } catch (error) {
            this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'UraGotoAgente', msg: `Agente Nao existe!`, route_data: route_data });

            // Em caso de erro, se especificado, redireciona para uma rota
            if (from.falha_goto) this._gotoSequence(session, from.falha_goto, route_data);
        }

    }

    async _handleEndSession(session, from, route_data) {
        // Ja envia a resposta de finalização, assim não precisa de timeout nos eventos.
        let resposta = from.resposta ? this._injectVariablesInText(from.resposta, session.dados) : null;

        this.emitter.emit('UraFim', { idCliente: session.iduser, msg: resposta, route_data: route_data });
        await session.delete({ require: false });
    }

    async _handleGoto(session, from, route_data) {
        // Se tem respota gera evento UraMsg, usado para enviar uma msg pro cliente
        if (from.resposta) {
            let resposta = this._injectVariablesInText(from.resposta, session.dados);
            this.emitter.emit('UraMsg', { idCliente: session.iduser, msg: resposta, route_data: route_data });
        }

        if (typeof from.goto == "string") {
            from.goto = this._injectVariablesInText(from.goto, session.dados);
        }

        // Se chegou até até aqui é para continuar...
        await this._gotoSequence(session, parseInt(from.goto), route_data);
    }

    async _handleGotoIf(session, from, route_data) {
        try {
            this.emitter.emit('UraDebug', { idCliente: session.iduser, debug: 'UraGotoIf', msg: `Recebi -> ${from.goto_if}` });

            // Executa o operador ternário especificado na URA
            const script = new vm.Script(from.goto_if);
            let context = vm.createContext(session.dados);
            let result = script.runInContext(context);

            // Verifica o resultado da expressão
            result = result ? parseInt(result) : null;
            if (result) await this._gotoSequence(session, result, route_data);

        } catch (er) {
            this.emitter.emit('UraDebug', {
                idCliente: session.iduser,
                debug: 'scriptErro',
                msg: 'ERRO: Falha ao executar o script: ' + er.message,
                route_data: route_data
            });
        }
    }
    //#endregion

};