"use strict";
/**
 * Fluxo de trabalho deste código
 * enfileirarCampanha -> agendarTarefa -> processarCampanha -> enviarMensagemWhatsApp
 */

//#region Imports
const db = require('../util/db');
const socketIO = require("socket.io-client");
const DiscadorCommon = require("./DiscadorCommon");
const SimpleNodeLogger = require('simple-node-logger');
const moment = require('moment-timezone');
const Horarios = require("../models/Horarios");
const LayoutCampos = require('../models/LayoutsCampos');
const TextoUtil = require("../util/texto");
const UUID = require("uuid");
const fs = require("fs");
const mime = require("mime-to-extensions");
const path = require("path");
const Cliente = require('../models/Cliente');
const Fila = require('../models/Fila');
const ClienteCamposAdicionais = require('../models/ClienteCamposAdicionais');
const ClienteAtendimento = require('../models/ClienteAtendimento');
const AtendimentoMensagem = require('../models/AtendimentoMensagem');
const MensagemAnexos = require('../models/MensagemAnexos');
//#endregion

// Constantes
const TEMPO_TIMEOUT = 300000;

// Variáveis
let fila = [];// Fila de campanhas ativas
let whatsappSocket = null;// Usado para enviar mensagens
let zenviaSocket = null;// Usado para enviar mensagens

let options = null;// Arquivo de configurações
let log = null; // Carrega o logger

//#region Utilidade
/***
 * This method runs a chain of Promises in series
 * One after another, similar to Promise.all
 *
 * @author Leandro Quevedo
 * @see Promise.all
 * @param {Array} list An array of {@link Function}
 * @returns {Promise} A promise that will resolve in an array of the inner-methods resovled results
 ***/
Promise.sequential = function (list) {
    return new Promise(async (resolve, reject) => {
        try {
            let results = [];
            for (let method of list) {
                results.push(await method.apply(this, []));
            }

            resolve(results);
        } catch (e) {
            reject(e);
        }
    });
};

//#endregion

//#region Funções internas
function inicializarLogger() {
    log = SimpleNodeLogger.createSimpleLogger({
        logFilePath: '/var/log/xcontact.log',
        timestampFormat: 'DD-MM-YYYY HH:mm:ss.SSS'
    });

    log.setLevel(process.env.LOG ? process.env.LOG : 'error');
    log.info('[DISCADOR-WHATSAPP] Iniciando discador...');
}

function agendarTarefa(task, cooldown) {
    if (!cooldown || cooldown <= 0) cooldown = 10000;

    log.info(`[DISCADOR-WHATSAPP] Agendando tarefa da campanha ${task.campanha.id} para daqui ${cooldown / 1000} segundos`);

    /** Não entendi o motivo de fazer isso abaixo 
    let i = 0;
    let f = function () {
        i++;
        if (i * 1000 >= cooldown) {
            processarCampanha(task);
        } else {
            task.status = "Cooldown de " + (cooldown / 1000 - i) + "s";
            // log.info(`[DISCADOR-WHATSAPP] Agendando tarefa da campanha ${task.campanha.id} para daqui ${cooldown / 1000 - i} segundos`);
            setTimeout(f, 1000);
        }
    };

    f();
    **/

    setTimeout(() => {
        processarCampanha(task);
    }, cooldown);
}

//#region Relacionados ao WhatsApi
function conectarComSocketWhatsApp() {
    if (!process.env.WHATSAPI_ATIVADO) return log.info(`[DISCADOR-WHATSAPP] - API WhatsApp desativada por WHATSAPI_ATIVADO = ${process.env.WHATSAPI_ATIVADO}`);
    if (process.env.WHATSAPI_CLIENTID == 'SEM') return log.info(`[DISCADOR-WHATSAPP] - API WhatsApp desativada por WHATSAPI_CLIENTID = ${process.env.WHATSAPI_CLIENTID}`);

    log.info(`[DISCADOR-WHATSAPP] Inciando conexão em ${process.env.WHATSAPI_HOST} com ClientID = ${process.env.WHATSAPI_CLIENTID}`);
    let socket = socketIO(process.env.WHATSAPI_HOST);
    let handled = false;

    socket.on("error", (error) => {
        console.error(error);
        console.trace(error);
        log.error(`[Socket-Server-DiscadorWhatsApp-Error] Erro! ` + error);
    });

    socket.on('connect', () => {
        log.info(`[DISCADOR-WHATSAPP] Socket do whatsapp conectado em ${process.env.WHATSAPI_HOST} com ClientID = ${process.env.WHATSAPI_CLIENTID}`);

        socket.on("diconnect", () => {
            log.warn(`[DISCADOR-WHATSAPP] Socket do whatsapp desconectado!`);
            whatsappSocket = null;
        });

        socket.emit('Login', { clientID: process.env.WHATSAPI_CLIENTID }, (ret) => {
            // Notifica ao timeout que o evento já foi processado, então ignora o reject por tempo excedido
            handled = true;

            // Verifica se o login foi bem sucedido
            if (ret.error) {
                // Agenda uma outra tentativa
                log.error(`[DISCADOR-WHATSAPP] Erro ao autenticar-se com o socket do whatsapp!\n` + ret.error);
                setTimeout(conectarComSocketWhatsApp, 1000);
            } else {
                // Salva o socket
                log.info(`[DISCADOR-WHATSAPP] Login efetuado com sucesso ClientID = ${process.env.WHATSAPI_CLIENTID}`);
                whatsappSocket = socket;
            }
        });

        socket.on("WhatsStatus", (status) => {
            // 1 - Criar array para salvar o status das instancias e só enviar msg de o status tiver ok no array
            // 2 - Verificar a cada min o status das instancias que tiver campanha ativa.
            log.info(`[DISCADOR-WHATSAPP] WhatsStatus! ${JSON.stringify(status)}`);
            if (fila.length > 0) {
                let fi = fila.find(f => f.fila == status.instanceID);
                if (fi) {
                    fi.InstanciaStatus = status.status;
                }
            }
        });

        setInterval(() => {
            log.debug(`[DISCADOR-WHATSAPP] Rotina de checar status de instancias a cada 60s. Tamnho de fila = ${fila.length}`);
            if (fila.length > 0) {
                for (let i = 0; i < fila.length; i++) {
                    let campanha = fila[i].campanha;
                    log.debug(`[DISCADOR-WHATSAPP] getWhatsStatus Campanha ${campanha.nome} status = ${campanha.status}`);
                    log.debug(`[DISCADOR-WHATSAPP] getWhatsStatus para instancia ${campanha.fila}`);
                    socket.emit('getWhatsStatus', { instanceID: campanha.fila }, resposta => {
                        log.debug(`[DISCADOR-WHATSAPP] getWhatsStatus resposta: ${JSON.stringify(resposta)}`);
                        campanha.InstanciaStatus = resposta.WhatsStatus;
                    });
                }
            }
        }, 60000);
    });

    setTimeout(() => {
        // Se o evento foi processado, ignora o timeout
        if (handled) return;

        // Demorou demais para conectar e fazer a autenticação pelo whatsapp
        log.info(`[DISCADOR-WHATSAPP] Timeout na conexão e/ou login com o socket do whatsapp!`);
    }, TEMPO_TIMEOUT);
}
function conectarComSocketZenvia() {
    if (!process.env.ZENVIA_ATIVADO) return log.info(`[DISCADOR-WHATSAPP] - API ZENVIA desativada por WHATSAPI_ATIVADO = ${process.env.ZENVIA_ATIVADO}`);
    if (process.env.ZENVIA_CLIENTID == 'SEM') return log.info(`[DISCADOR-WHATSAPP] - API Zenvia desativada por WHATSAPI_CLIENTID = ${process.env.ZENVIA_CLIENTID}`);

    log.info(`[DISCADOR-WHATSAPP] Inciando conexão em ${process.env.ZENVIA_HOST} com ClientID = ${process.env.ZENVIA_CLIENTID}`);
    let socket = socketIO(process.env.ZENVIA_HOST);
    let handled = false;

    socket.on("error", (error) => {
        console.error(error);
        console.trace(error);
        log.error(`[Socket-Server-DiscadorWhatsApp-Error] Erro! ` + error);
    });

    socket.on('connect', () => {
        log.info(`[DISCADOR-WHATSAPP] Socket do zenvia conectado em ${process.env.ZENVIA_HOST} com ClientID = ${process.env.ZENVIA_CLIENTID}`);

        socket.on("diconnect", () => {
            log.warn(`[DISCADOR-WHATSAPP] Socket do zenvia desconectado!`);
            zenviaSocket = null;
        });

        socket.emit('Login', { clientID: process.env.ZENVIA_CLIENTID }, (ret) => {
            // Notifica ao timeout que o evento já foi processado, então ignora o reject por tempo excedido
            handled = true;

            // Verifica se o login foi bem sucedido
            if (ret.error) {
                // Agenda uma outra tentativa
                log.error(`[DISCADOR-WHATSAPP] Erro ao autenticar-se com o socket do zenvia!\n` + ret.error);
                setTimeout(conectarComSocketZenvia, 1000);
            } else {
                // Salva o socket
                log.info(`[DISCADOR-WHATSAPP] Login efetuado com sucesso ClientID = ${process.env.ZENVIA_CLIENTID}`);
                zenviaSocket = socket;
            }
        });
    });

    setTimeout(() => {
        // Se o evento foi processado, ignora o timeout
        if (handled) return;

        // Demorou demais para conectar e fazer a autenticação pelo whatsapp
        log.info(`[DISCADOR-WHATSAPP] Timeout na conexão e/ou login com o socket do zenvia!`);
    }, TEMPO_TIMEOUT);
}
function enviaTemplateZenvia(fila, atendimento, template, destino, campos) {
    return new Promise((resolve, reject) => {
        try {
            let config = fila.emailConfig;
            // let fields = {};
            // fields = Object.keys(campos).filter(k => template.fields.findIndex(k) !== -1);
            zenviaSocket.emit('newWhatsMsgZenvia', { zenviaID: config.zenviaID, from: config.from, to: destino, text: template.components.body.text, templateId: template.id, fields: campos }, async (retorno) => {

                if (retorno && !retorno.error) {
                    if (atendimento) {
                        let texto = template.components.body.text;
                        for (let key in campos) {
                            texto = texto.replace(`{{${key}}}`, campos[key]);
                        }                        // Cria uma mensagem e registra no atendimento
                        await new AtendimentoMensagem({
                            idAtendimento: atendimento.id,
                            dataHora: new Date(),
                            agente: "Xcontact",
                            texto: texto,
                            inOut: 3,
                            status: retorno.error ? `pending` : `seen`
                        }).save();
                    }

                    resolve();
                } else {
                    reject("Erro ao enviar mensagem de texto: " + JSON.stringify(retorno));
                }
            });
        } catch (error) {
            reject(error);
        }

    })

}
function enviarMensagemWhatsAppTexto(atendimento, texto, campanha, numero) {
    return new Promise((resolve, reject) => {
        try {
            // Define os argumentos para enviar ao CSA
            let options = {
                instanceID: campanha.fila,
                chatId: numero,
                msg: texto
            };
            console.log(`++++++++++++++++++++++++++++++++++++++++++++++++++++++ENTROU++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++`)
            log.info(`[DISCADOR-WHATSAPP] Dados do 'newWhatsMsg' -> ${JSON.stringify(options)}`);

            // Envia a mensagem
            whatsappSocket.emit("newWhatsMsg", options, async retorno => {
                log.info(`[DISCADOR-WHATSAPP] Retorno do 'newWhatsMsg' -> ${JSON.stringify(retorno)}`);
                // Se um atendimento foi provido
                if (atendimento) {
                    // Cria uma mensagem e registra no atendimento
                    await new AtendimentoMensagem({
                        idAtendimento: atendimento.id,
                        dataHora: new Date(),
                        agente: "Xcontact",
                        texto: texto,
                        inOut: 3,
                        status: retorno.error ? `pending` : `seen`
                    }).save();
                }

                if (retorno && !retorno.error) {
                    resolve();
                } else {
                    reject("Erro ao enviar mensagem de texto: " + JSON.stringify(retorno));
                }
            });
        } catch (error) {
            console.error(error);
            reject(error);
        }
    });
}

function enviarMensagemWhatsAppArquivo(atendimento, texto, caminho_arquivo, campanha, numero) {
    return new Promise((resolve, reject) => {
        try {
            // Cria uma stream para ler o arquivo
            let stream = fs.createReadStream(caminho_arquivo, { autoClose: true });

            // Define os argumentos para enviar ao CSA
            let options = {
                instanceID: campanha.fila,
                chatId: numero,
                dados: {
                    uid: UUID.v1(),
                    Upload: "start",
                    mimetype: mime.lookup(caminho_arquivo),
                    legenda: texto || "",
                }
            };

            log.info(`[DISCADOR-WHATSAPP] Dados do arquivo '${caminho_arquivo}'\n${JSON.stringify(options)}`);

            // Envia a mensagem
            whatsappSocket.emit("sendMsgFile", options);

            stream.on("error", error => {
                log.error(`[DISCADOR-WHATSAPP] Erro ao ler arquivo da campanha! Arquivo '${caminho_arquivo}'`);
                reject(error);
            });

            // Quando receber um pedaço do arquivo
            stream.on("data", chunk => {
                // log.info(`[DISCADOR-WHATSAPP] Recebido novo chunk do arquivo '${campanha.caminho_imagem_campanha}'`);

                // Atualiza o status do upload
                options.dados.Upload = "sending";
                // Envia para o CSA
                whatsappSocket.emit("sendMsgFile", options, null, chunk);
            });

            // Ao receber o arquivo inteiro
            stream.on("end", () => {
                // log.info(`[DISCADOR-WHATSAPP] Recebido fim arquivo '${caminho_arquivo}'`);

                // Atualiza o status do upload
                options.dados.Upload = "end";
                // Envia para o CSA, esperando uma resposta 'OK'
                whatsappSocket.emit("sendMsgFile", options, async retorno => {
                    if (retorno && !retorno.error) {
                        log.info(`[DISCADOR-WHATSAPP] Recebido retorno do 'sendMsgFile'`);

                        // Se um atendimento foi provido
                        if (atendimento) {
                            // Cria uma mensagem vazia (Ou como legenda) para anexar a imagem nela
                            let legenda = await new AtendimentoMensagem({
                                idAtendimento: atendimento.id,
                                dataHora: new Date(),
                                agente: "Xcontact",
                                texto: texto || "",
                                inOut: 3,
                                status: retorno.error ? `pending` : `seen`

                            }).save();

                            // Anexa o arquivo na mensagem criada acima
                            await new MensagemAnexos({
                                idMensagem: legenda.id,
                                caminho: caminho_arquivo,
                                mimeType: options.dados.mimetype
                            }).save();
                        }

                        resolve();
                    } else {
                        log.error("Erro ao enviar arquivo: " + JSON.stringify(retorno));
                        reject("Erro ao enviar arquivo: " + JSON.stringify(retorno));
                    }
                });
            });
        } catch (error) {
            reject(error);
        }
    });
}

function tratarNumeroDestinatario(numero, campanha) {
    // Se especificado, insere o DDI (Código do país) no início do número do destinatário
    if (campanha && campanha.ddi_padrao) numero = campanha.ddi_padrao + numero;
    // Não deveria ser necessário... porém apenas pra termos certeza, que não vai puxar números inválidos!
    numero = TextoUtil.limparTextoNumerico(numero);
    // Verifica se precisa do sufixo do WhatsApp
    // if (!numero.endsWith("@c.us")) numero += "@c.us";

    return numero;
}

/**
 * Cria um atendimento já encerrado, apenas para o agente poder ver as mensagens da campanha
 *
 * @param {String} fila Nome da fila
 * @param {string} nome Nome do cliente|contato
 * @param {String} numero Número do cliente|contato
 * @returns {ClienteAtendimento} Retorna o atendimento criado
 */
function criarAtendimentoEncerrado(campanha, nome, numero, dados) {
    return new Promise(async (resolve, reject) => {
        try {
            // Busca um cliente, dado o nome e o número
            let cliente = await Cliente.getClientFromMsg({
                idCliente: numero,
                nomeCliente: nome
            });
            if (cliente) {
                for (var name in dados) {
                    let verifica = await ClienteCamposAdicionais.query.require(false).first()
                        .where({ nome: name, clienteID: cliente.id }).fetch();
                    //vamos atualizar os que ja tem
                    if (verifica) {
                        verifica.valor = dados[name];
                        verifica.save();
                    }
                    //Vamos adicionar caso nao esteja no verifica
                    if (name !== "nome" && name !== "numero" && !verifica) {
                        //vamos verificar se e tipo telefone ou tipo texto
                        let guarda = await LayoutCampos.query.first().where({ id_layout: campanha.id_layout, nome: name }).fetch()
                        console.log(`GUARDA++++++++`, guarda)
                        if (guarda && !guarda.xcontact) {
                            let campo = {};
                            let valor = dados[name];
                            campo.nome = name;
                            campo.valor = valor;
                            campo.clienteID = cliente.id;
                            console.log(`CAMPO`, campo);
                            new ClienteCamposAdicionais(campo).save()

                        }

                    }



                }
                // Cria o atendimento
                let clienteAtendimento = new ClienteAtendimento({
                    queue: campanha.fila,
                    queueTypeID: 3,
                    sentido: "Sainte",
                    status: campanha.status_atendimento ? campanha.status_atendimento : "Encerrado",
                    idCliente: cliente.id,
                    fromID: cliente.numero,
                    dataFim: new Date()
                });

                //console.log('Criando Atendimento via Whats');
                //console.log(campanha);

                // Salva no banco
                resolve(await clienteAtendimento.save());
            }

            resolve(null);
        } catch (error) {
            console.trace(error);
            log.error(`[DISCADOR-WHATSAPP] Não foi possível criar um atendimento! ` + error);

            resolve(null);
        }
    });
}

function enviarMensagemWhatsApp(campanha, contato) {
    return new Promise(async (resolve, reject) => {
        log.info(`[DISCADOR-WHATSAPP] Enviando mensagem para ${contato.numero.numero} na campanha ${campanha.id}`);
        console.log(`================================================`, campanha.rota);
        // Define o objeto da ligação
        let registro = {
            CampanhaID: campanha.id,
            ContatoID: contato.id,
            NumeroID: contato.id_numero,
            Destino: contato.numero.numero,
            Linkedid: 0,
            ActionID: 0,
            Falha: false,
            Status: 9,
            StatusDescricao: "Iniciando",
            Campos: contato.campos
        };

        // Trata o número do destinatário
        let destinatario = tratarNumeroDestinatario(contato.numero.numero, campanha);

        // Variável para controlar o timeout do socket
        let handled = false;

        // Variável para guardar as promises de envio, porque o envio pode acontecer em múltiplas etapas
        let promiseList = [];

        let atendimento = await criarAtendimentoEncerrado(campanha, contato.campos["Nome"] || contato.campos["nome"], contato.numero.numero, contato.campos);

        // Se especificada uma imagem na campanha
        if (campanha.caminho_imagem_campanha) {
            let legenda = injetarVariaveisCoringa(campanha.legenda_imagem_campanha, contato.campos);
            log.info(`[DISCADOR-WHATSAPP] Enviando imagem para o número ${destinatario} ${legenda ? 'com legenda' : 'sem legenda'}`);
            promiseList.push(enviarMensagemWhatsAppArquivo.bind(this, atendimento, legenda, campanha.caminho_imagem_campanha, campanha, destinatario));
        }

        // Se especificado um arquivo na campanha
        if (campanha.caminho_arquivo_campanha) {
            log.info(`[DISCADOR-WHATSAPP] Enviando arquivo para o número ${destinatario}`);
            promiseList.push(enviarMensagemWhatsAppArquivo.bind(this, atendimento, path.basename(campanha.caminho_arquivo_campanha), campanha.caminho_arquivo_campanha, campanha, destinatario));
        }

        // Verifica se existe um texto para enviar, trata como prioridade o campo 'texto' do contato, tendo como fallback o texto_padrao_campanha
        //Vamos buscar o nome certo para o campo de texto
        let nomeCampo = await LayoutCampos.query.first().where({ id_layout: campanha.id_layout, xcontact: 'texto' }).fetch();
        console.log(`RETORNOU`, nomeCampo);
        //let text = contato.campos["Texto"] ? contato.campos["Texto"] : contato.campos["mensagem"];
        let textoPadrao = injetarVariaveisCoringa(campanha.texto_padrao_campanha, contato.campos);
        if (textoPadrao) {
            log.info(`[DISCADOR-WHATSAPP] Mensagem tipo texto para o número ${destinatario}`);
            promiseList.push(enviarMensagemWhatsAppTexto.bind(this, atendimento, textoPadrao, campanha, destinatario));
        }
        let texto = injetarVariaveisCoringa(contato.campos[`${nomeCampo.nome}`]);
        console.log(`==============================`, texto);
        if (texto && texto != "") {
            // Apenas para ter certeza que a mensagem foi enviada, espera dois segundos
            // Pois nos testes iniciais o CSA estava enviando o texto da mensagem antes da imagem, mesmo tudo sendo feito sequencialmente...
            if (campanha.caminho_imagem_campanha)
                promiseList.push(() => new Promise((resolve, _) => setTimeout(resolve, 5000)));

            log.info(`[DISCADOR-WHATSAPP] Mensagem tipo texto para o número ${destinatario}`);
            promiseList.push(enviarMensagemWhatsAppTexto.bind(this, atendimento, texto, campanha, destinatario));
        }
        //Vamos verificar se tem template
        if (campanha.rota && campanha.rota != 'Discador' && campanha.rota != 'discador') {
            let template = JSON.parse(campanha.rota);
            var fila = await Fila.query.require(true).first()
                .where({ name: atendimento.queue })
                .fetch();
            log.info(`[DISCADOR-WHATSAPP] Mensagem tipo texto para o número ${destinatario}`);
            promiseList.push(enviaTemplateZenvia.bind(this, fila, atendimento, template, destinatario, contato.campos));
        }
        console.log(`VAMOS VER AS PROMISSES AQUI MANE`, promiseList);
        // Atribui os handlers dos eventos de sucesso e falha
        // Que são iguais para quaisquer tipos de campanha!!!!!
        Promise.sequential(promiseList).then(
            async response => {
                log.info(`[DISCADOR-WHATSAPP] Mensagem enviada para ${destinatario} com sucesso!`, response);

                // Notifica ao timeout que o evento já foi processado, então ignora o reject por tempo excedido
                handled = true;

                // Altera o status da ligação, para inserir no banco
                registro.Status = 16;
                registro.StatusDescricao = "Completou";

                // Registra no banco o sucesso do contato
                await DiscadorCommon.notificarConclusaoDaCampanha(registro);
                await DiscadorCommon.inserirRegistroDaCampanha(registro);

                resolve(response);
            },
            async error => {
                log.error(`[DISCADOR-WHATSAPP] Erro ao enviar mensagem para ${destinatario}!\n${error}`);

                // Notifica ao timeout que o evento já foi processado, então ignora o reject por tempo excedido
                handled = true;

                // Altera o status da ligação, para inserir no banco
                registro.Status = 4;
                registro.StatusDescricao = "Falha";
                registro.Falha = true;

                // Registra no banco a falha do contato
                await DiscadorCommon.inserirRegistroDaCampanha(registro);
                await DiscadorCommon.notificarRetentativaDeContato(contato.id_numero);

                reject(error);
            }
        );

        // Adiciona um 'timeout' no envio da mensagem
        setTimeout(() => {
            // Se o evento foi processado, ignora o timeout
            if (handled) return;

            log.error(`[DISCADOR-WHATSAPP] Timeout ao enviar mensagem para ${destinatario}`);

            // Demorou demais para conectar e fazer a autenticação pelo whatsapp
            reject("Timeout no envio da mensagem");
        }, TEMPO_TIMEOUT);
    });
}
//#endregion

async function buscarNumerosDaCampanha(campanha) {
    // Define a ordenação por prioridades
    // Assim, apenas prioridades acima de 0 vão alterar a ordem da ordenação
    let casePrioridade = /*sql*/`CASE
                                    WHEN prioridade <= 0 THEN 0
                                    WHEN prioridade >= 1 THEN prioridade
                                END DESC`;

    // Busca os contatos no banco
    let selectQuery = /*sql*/`SELECT id FROM public.campanhas_contato_numeros WHERE
                                    id_campanha=${campanha.id} AND
                                    sucesso=false AND
                                    tentativas < ${campanha.tentativas} AND
                                    retentativas < ${campanha.tentativas}
                                    ORDER BY ${casePrioridade},
                                            fone_sequencia ASC,
                                            tentativas ASC,
                                            retentativas ASC,
                                            id_contato ASC
                                    LIMIT 1`;// Apenas um número por vez

    // Atualiza os contatos selecionados
    let updateQuery = /*sql*/`UPDATE public.campanhas_contato_numeros
                                SET
                                    tentativas=tentativas+1,
                                    prioridade=prioridade-1
                                WHERE
                                    id_campanha=${campanha.id} AND
                                    id IN (${selectQuery})
                                    returning *`;

    let result = await db.query(updateQuery);

    // Verifica se o retorno da query é válido
    if (!result || !result.rows || result.rows.length <= 0) {
        return [];
    } else {
        return result.rows;
    }
}

async function buscarNumerosAdicionaisDoContato(contato) {
    let query = /*sql*/`SELECT nome, numero, fone_sequencia FROM public.campanhas_contato_numeros WHERE id_contato = ${contato.id}`;

    let result = await db.query(query);

    // Verifica se o retorno da query é válido
    if (!result || !result.rows || result.rows.length <= 0) {
        return [];
    } else {
        return result.rows;
    }
}

async function buscarCamposAdicionais(contato) {
    // Campos adicionais
    var camposAdicionais = await db.query(/*sql*/`SELECT * from public.campanhas_contato_campos WHERE id_contato = $1`, [contato.id]);

    for (let i = 0; i < camposAdicionais.rows.length; i++) {
        contato.campos[camposAdicionais.rows[i].nome] = camposAdicionais.rows[i].valor;
    }

    log.info("[DISCADOR-WHATSAPP] Campos adicionais -> " + JSON.stringify(contato.campos));
}

async function processarCampanha(task) {
    try {
        //log.info(`[DISCADOR-WHATSAPP] EXECUTANDO A CAMPANHA ${task.campanha.id}`);
        log.debug(`[DISCADOR-WHATSAPP] Dados da Task ${JSON.stringify(task)}`);
        // Verifica se a campanha saiu da fila
        if (fila.indexOf(task) === -1) {
            return log.info(`[DISCADOR-WHATSAPP] A campanha ${task.campanha.id} foi removida da fila!`);

        }
        // Calcula o intervalo da tarefa
        const cooldown = task.campanha.intervalo_entre_mensagens * 1000;
        // Verifica se a campanha não está ativa
        if (task.campanha.status !== "A") {
            task.status = `Aguardando ser ativada`;
            return log.info(`[DISCADOR-WHATSAPP] A campanha ${task.campanha.id} Não está ativa! Aguardando ser ativada`);
            // Agenda a próxima execução da tarefa
            // Agenda tarefa se a campanha não está ativa? Legal!!! Dai quando ativar já envia tudo sem respeitar o limite de tempo e bani o número do cliente!!!
            //return agendarTarefa(task, cooldown);
        }



        // Atualiza o status da tarefa
        task.status = "Processando número " + task.progresso;
        log.info(`[DISCADOR-WHATSAPP] Processando campanha ${task.campanha.id}. Progresso: ${task.progresso}`);


        // Verifica se o turno é valido
        if (await verificarTurnosDaCampanha(task.campanha)) {
            log.info("[DISCADOR-WHATSAPP] TURNO VÁLIDO");
        } else {
            task.status = "Aguardando turno válido";
            //return log.info("[DISCADOR-WHATSAPP] TURNO INVÁLIDO");
            // Agenda a próxima execução da tarefa
            log.info("[DISCADOR-WHATSAPP] TURNO INVÁLIDO");
            return agendarTarefa(task, cooldown);
        }

        // Verifica se a campanha possui agendamento e se o mesmo está ativo
        if (DiscadorCommon.verificarAgendamento(task.campanha)) {
            log.info("[DISCADOR-WHATSAPP] AGENDAMENTO VÁLIDO");
        } else {
            task.status = "Aguardando agendamento...";
            return log.info("[DISCADOR-WHATSAPP] AGENDAMENTO INVÁLIDO");

            // Agenda a próxima execução da tarefa
            //return agendarTarefa(task, cooldown);
        }

        // Verifica se o socket do whatsapp está valido e conectado
        if (!whatsappSocket && !zenviaSocket) {
            task.status = "Aguardando socket ficar online";
            return log.error("[DISCADOR-WHATSAPP] Socket não conectado com o servidor CSA! Aguardando ficar online...");
            // Atualiza o status da tarefa                    

            // Agenda a próxima execução da tarefa
            //return agendarTarefa(task, cooldown);
        }

        // Busca no banco os números referentes a campanha fornecida
        let contatos = await buscarNumerosDaCampanha(task.campanha);

        // Se não existe nenhum contato a ser enviado
        if (contatos.length <= 0) {
            log.info(`[DISCADOR-WHATSAPP] Nenhum contato para enviar mensagens na campanha ${task.campanha.id}!`);

            // Se o discador pode finalizar a campanha
            if (task.campanha.autofim) {
                log.info(`[DISCADOR-WHATSAPP] Finalizando campanha ${task.campanha.id}...`);
                // Atualiza o status da tarefa
                task.status = "Campanha finalizada";

                // Finaliza a campanha
                await DiscadorCommon.finalizarCampanha(task.campanha);

                // Remove a campanha da fila
                removerDaFila(task.campanha);
            } else {
                // Atualiza o status da tarefa
                task.status = "Standby";
            }
        } else {
            log.info(`[DISCADOR-WHATSAPP] ${contatos.length} contatos para enviar mensagens na campanha ${task.campanha.id}!`);

            // Pega o primeiro número válido
            let numero = contatos.find(x => x);

            // Monta um contato equivalente ao número
            const contato = {
                numero: {},
                campos: {},
                id: numero.id_contato,
                id_numero: numero.id
            };

            // Adiciona todos os campos do número para o contato
            for (const numeroContato in numero) {
                contato.numero[numeroContato] = numero[numeroContato];
            }

            // Extrai todos os números do contato, para poder acessar nas variáveis coringa
            try {
                // Busca todos os números que tem o mesmo id do contato selecionado
                let numerosDoContato = await buscarNumerosAdicionaisDoContato(contato);//contatos.find(x => contatos.id_contato === numero.id_contato);
                numerosDoContato.forEach(x => {
                    contato.campos[x.nome] = x.numero;
                });
            } catch (e) {
                log.warn("[DISCADOR-WHATSAPP] Falha ao extrair o número para variável coringa!");
                console.trace(e);
            }
            // Popula os campos adicionais no contato
            await buscarCamposAdicionais(contato);
            // Envia a mensagem
            enviarMensagemWhatsApp(task.campanha, contato)
                .then(async x => {
                    // Incrementa o progresso da tarefa
                    task.progresso++;
                    // Atualiza o status da tarefa
                    task.status = `Cooldown de ${cooldown}s`;

                    // Agenda a próxima execução turno da tarefa
                    agendarTarefa(task, cooldown);
                })
                .catch(async error => {
                    log.info(`[DISCADOR-WHATSAPP] Erro ao enviar mensagem para ${contato.numero.numero} na campanha ${task.campanha.id}.\nNotificando re-tentativa`);

                    // Incrementa o progresso da tarefa
                    task.progresso++;
                    // Atualiza o status da tarefa
                    task.status = 'Retentativa -> ' + error.toString();

                    // Notifica que vai ser feita uma re-tentativa
                    await DiscadorCommon.notificarRetentativaDeContato();

                    // Agenda a próxima execução turno da tarefa
                    agendarTarefa(task, cooldown * 2);
                });


        }

    } catch (e) {
        console.trace(e);
        log.info(`[DISCADOR-WHATSAPP] Erro ao processar campanha!\n` + e);

        // Incrementa o progresso da tarefa
        task.progresso++;
        // Atualiza o status da tarefa
        task.status = 'Retentativa -> ' + e.toString();

        agendarTarefa(task, cooldown * 10);
    }
}
exports.processarCampanha = processarCampanha;

function verificarTurnosDaCampanha(campanha) {
    return Horarios.verificarTurnos(campanha.turnos);
}

function injetarVariaveisCoringa(texto, camposAdicionais) {
    log.info(`[DISCADOR-WHATSAPP] Injetando variáveis coringa no texto!`);

    if (!texto || texto === "") return null;

    try {
        // Itera todos os campos adicionais provenientes do banco
        for (let key in camposAdicionais) {
            // Ignora propriedades nativas do objeto
            if (!camposAdicionais.hasOwnProperty(key)) continue;

            // Substitui o $ seguido por uma key pelo seu respectivo valor nos campos adicionais inseridos
            let regex = new RegExp(`\\$(${key})`, `gi`);
            texto = texto.replace(regex, camposAdicionais[key]);
        }

        // Retorna o texto processado
        return texto;
    } catch (e) {
        console.trace(e);
        log.info(`[DISCADOR-WHATSAPP] Erro ao injetar variáveis coringa na campanha!\n` + e);
    }
}
//#endregion

//#region Funções exportadas
function removerDaFila(campanha) {
    log.info(`[DISCADOR-WHATSAPP] Campanha ${campanha.id} removida da fila!`);

    // Remove a campanha da fila
    let index = fila.findIndex(x => x.campanha.id === campanha.id);
    if (index >= 0) fila.splice(index, 1);
}
exports.removerDaFila = removerDaFila;


function enfileirarCampanha(campanha) {
    // Ignora as campanhas que não são do tipo whatsapp
    if (campanha.tipo_campanha !== 1) return;

    // Ignora campanhas que já estão enfileiradas
    let oldCampanha = fila.find(x => x.campanha.id === campanha.id);
    if (oldCampanha) {
        log.warn(`[DISCADOR-WHATSAPP] Ignorando enfileiramento da campanha ${campanha.id} porque ela já está na fila!`);
        // Atualiza as propriedades da campanha
        for (let property in campanha) {
            if (campanha.hasOwnProperty(property)) oldCampanha.campanha[property] = campanha[property];
        }
        // agendarTarefa(oldCampanha, oldCampanha.campanha.intervalo_entre_mensagens * 1000);

        return;
    }

    log.info(`[DISCADOR-WHATSAPP] Adicionando campanha ${campanha.id} na fila...`);

    // Cria uma tarefa
    let task = {
        campanha: campanha,
        progresso: 0,
        status: 'Adicionada a fila'
    };

    // Adiciona a campanha na fila
    fila.push(task);

    // Roda no próximo tick
    agendarTarefa(task, 0);
}
exports.enfileirarCampanha = enfileirarCampanha;


function recarregarCampanhas(campanhasAtivas) {
    // Filtra apenas as campanhas do tipo whatsapp
    let campanhasWhatsAppAtivas = campanhasAtivas.filter(x => x && x.tipo_campanha === 1);

    log.info(`[DISCADOR-WHATSAPP] Recarregando ${campanhasWhatsAppAtivas.length} campanhas...`);

    for (let i = 0; i < fila.length; i++) {
        // Verifica se a campanha da fila continua ativa
        let index = campanhasWhatsAppAtivas.findIndex(x => x.id === fila.id);

        // Se não estiver ativa, remove a campanha da fila
        if (index === -1) fila.splice(index, 1);
    }

    // Filtra apenas pelas campanhas ainda não enfileiradas
    let campanhasNovas = campanhasWhatsAppAtivas.filter(x => !fila.find(y => y.id === x.id));
    // Adiciona na fila todas as campanhas novas
    campanhasNovas.forEach(enfileirarCampanha);
}
exports.recarregarCampanhas = recarregarCampanhas;


function iniciar(config) {
    options = config;

    // Configura a timezone
    moment.tz.setDefault(config["config"].servidor.timezone);

    inicializarLogger();

    // Tenta se conectar com o socket do whatsapp
    conectarComSocketWhatsApp();
    conectarComSocketZenvia();

    return this;
}
exports.iniciar = iniciar;
//#endregion