'use strict'
exports.register = (server, options, next) => {
    const db = require('../util/db');
    const Campanha = require('./Campanha');
    const ws = require('../ws');
    const namiLib = require("nami");
    const Ami = require('../ami');
    const uuid = require('uuid/v1');
    const moment = require('moment-timezone');
    const fs = require("fs");
    const Horarios = require("../models/Horarios");
    const Depurador = require('./Depurador');
    const DiscadorCommon = require('./DiscadorCommon');
    const log = require("../util/Log");
    const ClienteCamposAdicionais = require('../models/ClienteCamposAdicionais');
    const ClienteAtendimento = require('../models/ClienteAtendimento');
    const AtendimentoMensagem = require('../models/AtendimentoMensagem');
    const MensagemAnexos = require('../models/MensagemAnexos');
    const Cliente = require('../models/Cliente');
    const Fila = require('../models/Fila');
    const LayoutCampos = require('../models/LayoutsCampos');
    const UUID = require("uuid");
    const TextoUtil = require("../util/texto");
    const mime = require("mime-to-extensions");
    const path = require("path");
    const socketIO = require("socket.io-client");
    let whatsappSocket = null;// Usado para enviar mensagens x5api
    let zenviaSocket = null;// Usado para enviar mensagens zenvia
    let gupshupSocket = null;//usado para enviar mensagens gup
    moment.tz.setDefault(options.config.servidor.timezone);
    const TEMPO_TIMEOUT = 300000;

    log.info('DISCADORSTART: Discador iniciado.');
    Promise.sequential = function (list) {
        return new Promise(async (resolve, reject) => {
            try {
                let results = [];
                for (let method of list) {
                    results.push(await method.apply(this, []));
                }

                resolve(results);
            } catch (e) {
                reject(e);
            }
        });
    };
    let fila = [];// Fila de campanhas ativas
    let campanhas = [];
    const ClaTimers = [];
    const discador = this;
    const EMITIR_EVENTOS = false;

    let discadorWhatsApp = require("./DiscadorWhatsApp").iniciar(options);

    exports.iniciarDepuracao = (dados) => {
        // Emite o evento de status por WS
        Depurador.emitirCampanha(dados.id);
    };
    function conectarComSocketWhatsApp() {
        if (!process.env.WHATSAPI_ATIVADO) return log.info(`[DISCADOR-WHATSAPP] - API WhatsApp desativada por WHATSAPI_ATIVADO = ${process.env.WHATSAPI_ATIVADO}`);
        if (process.env.WHATSAPI_CLIENTID == 'SEM') return log.info(`[DISCADOR-WHATSAPP] - API WhatsApp desativada por WHATSAPI_CLIENTID = ${process.env.WHATSAPI_CLIENTID}`);

        log.info(`[DISCADOR-WHATSAPP] Inciando conexão em ${process.env.WHATSAPI_HOST} com ClientID = ${process.env.WHATSAPI_CLIENTID}`);
        let socket = socketIO(process.env.WHATSAPI_HOST);
        let handled = false;

        socket.on("error", (error) => {
            console.error(error);
            console.trace(error);
            log.error(`[Socket-Server-DiscadorWhatsApp-Error] Erro! ` + error);
        });

        socket.on('connect', () => {
            log.info(`[DISCADOR-WHATSAPP] Socket do whatsapp conectado em ${process.env.WHATSAPI_HOST} com ClientID = ${process.env.WHATSAPI_CLIENTID}`);

            socket.on("diconnect", () => {
                log.warn(`[DISCADOR-WHATSAPP] Socket do whatsapp desconectado!`);
                whatsappSocket = null;
            });

            socket.emit('Login', { clientID: process.env.WHATSAPI_CLIENTID }, (ret) => {
                // Notifica ao timeout que o evento já foi processado, então ignora o reject por tempo excedido
                handled = true;

                // Verifica se o login foi bem sucedido
                if (ret.error) {
                    // Agenda uma outra tentativa
                    log.error(`[DISCADOR-WHATSAPP] Erro ao autenticar-se com o socket do whatsapp!\n` + ret.error);
                    setTimeout(conectarComSocketWhatsApp, 1000);
                } else {
                    // Salva o socket
                    log.info(`[DISCADOR-WHATSAPP] Login efetuado com sucesso ClientID = ${process.env.WHATSAPI_CLIENTID}`);
                    whatsappSocket = socket;
                }
            });

            socket.on("WhatsStatus", (status) => {
                // 1 - Criar array para salvar o status das instancias e só enviar msg de o status tiver ok no array
                // 2 - Verificar a cada min o status das instancias que tiver campanha ativa.
                log.info(`[DISCADOR-WHATSAPP] WhatsStatus! ${JSON.stringify(status)}`);
                if (fila.length > 0) {
                    let fi = fila.find(f => f.fila == status.instanceID);
                    if (fi) {
                        fi.InstanciaStatus = status.status;
                    }
                }
            });

            setInterval(() => {
                log.debug(`[DISCADOR-WHATSAPP] Rotina de checar status de instancias a cada 60s. Tamnho de fila = ${fila.length}`);
                if (fila.length > 0) {
                    for (let i = 0; i < fila.length; i++) {
                        let campanha = fila[i].campanha;
                        log.debug(`[DISCADOR-WHATSAPP] getWhatsStatus Campanha ${campanha.nome} status = ${campanha.status}`);
                        log.debug(`[DISCADOR-WHATSAPP] getWhatsStatus para instancia ${campanha.fila}`);
                        socket.emit('getWhatsStatus', { instanceID: campanha.fila }, resposta => {
                            log.debug(`[DISCADOR-WHATSAPP] getWhatsStatus resposta: ${JSON.stringify(resposta)}`);
                            campanha.InstanciaStatus = resposta.WhatsStatus;
                        });
                    }
                }
            }, 60000);
        });

        setTimeout(() => {
            // Se o evento foi processado, ignora o timeout
            if (handled) return;

            // Demorou demais para conectar e fazer a autenticação pelo whatsapp
            log.info(`[DISCADOR-WHATSAPP] Timeout na conexão e/ou login com o socket do whatsapp!`);
        }, TEMPO_TIMEOUT);
    }
    function conectarComSocketZenvia() {
        if (!process.env.ZENVIA_ATIVADO) return log.info(`[DISCADOR-WHATSAPP] - API ZENVIA desativada por WHATSAPI_ATIVADO = ${process.env.ZENVIA_ATIVADO}`);
        if (process.env.ZENVIA_CLIENTID == 'SEM') return log.info(`[DISCADOR-WHATSAPP] - API Zenvia desativada por WHATSAPI_CLIENTID = ${process.env.ZENVIA_CLIENTID}`);

        log.info(`[DISCADOR-WHATSAPP] Inciando conexão em ${process.env.ZENVIA_HOST} com ClientID = ${process.env.ZENVIA_CLIENTID}`);
        let socket = socketIO(process.env.ZENVIA_HOST);
        let handled = false;

        socket.on("error", (error) => {
            console.error(error);
            console.trace(error);
            log.error(`[Socket-Server-DiscadorWhatsApp-Error] Erro! ` + error);
        });

        socket.on('connect', () => {
            log.info(`[DISCADOR-WHATSAPP] Socket do zenvia conectado em ${process.env.ZENVIA_HOST} com ClientID = ${process.env.ZENVIA_CLIENTID}`);

            socket.on("diconnect", () => {
                log.warn(`[DISCADOR-WHATSAPP] Socket do zenvia desconectado!`);
                zenviaSocket = null;
            });

            socket.emit('Login', { clientID: process.env.ZENVIA_CLIENTID }, (ret) => {
                // Notifica ao timeout que o evento já foi processado, então ignora o reject por tempo excedido
                handled = true;

                // Verifica se o login foi bem sucedido
                if (ret.error) {
                    // Agenda uma outra tentativa
                    log.error(`[DISCADOR-WHATSAPP] Erro ao autenticar-se com o socket do zenvia!\n` + ret.error);
                    setTimeout(conectarComSocketZenvia, 1000);
                } else {
                    // Salva o socket
                    log.info(`[DISCADOR-WHATSAPP] Login efetuado com sucesso ClientID = ${process.env.ZENVIA_CLIENTID}`);
                    zenviaSocket = socket;
                }
            });
        });

        setTimeout(() => {
            // Se o evento foi processado, ignora o timeout
            if (handled) return;

            // Demorou demais para conectar e fazer a autenticação pelo whatsapp
            log.info(`[DISCADOR-WHATSAPP] Timeout na conexão e/ou login com o socket do zenvia!`);
        }, TEMPO_TIMEOUT);
    }
    function conectarComSocketGupshup() {
        if (!process.env.GUPSHUP_ATIVADO) return log.info(`[DISCADOR-WHATSAPP] - API GUP desativada por WHATSAPI_ATIVADO = ${process.env.GUPSHUP_ATIVADO}`);
        if (process.env.GUPSHUP_CLIENTID == 'SEM') return log.info(`[DISCADOR-WHATSAPP] - API Gup desativada por WHATSAPI_CLIENTID = ${process.env.GUPSHUP_CLIENTID}`);

        log.info(`[DISCADOR-WHATSAPP] Inciando conexão em ${process.env.GUPSHUP_HOST} com ClientID = ${process.env.GUPSHUP_CLIENTID}`);
        let socket = socketIO(process.env.GUPSHUP_HOST);
        let handled = false;

        socket.on("error", (error) => {
            console.error(error);
            console.trace(error);
            log.error(`[Socket-Server-DiscadorWhatsApp-Error] Erro! ` + error);
        });

        socket.on('connect', () => {
            log.info(`[DISCADOR-WHATSAPP] Socket do gup conectado em ${process.env.GUPSHUP_HOST} com ClientID = ${process.env.GUPSHUP_CLIENTID}`);

            socket.on("diconnect", () => {
                log.warn(`[DISCADOR-WHATSAPP] Socket do gup desconectado!`);
                gupshupSocket = null;
            });

            socket.emit('Login', { clientID: process.env.GUPSHUP_CLIENTID }, (ret) => {
                // Notifica ao timeout que o evento já foi processado, então ignora o reject por tempo excedido
                handled = true;

                // Verifica se o login foi bem sucedido
                if (ret.error) {
                    // Agenda uma outra tentativa
                    log.error(`[DISCADOR-WHATSAPP] Erro ao autenticar-se com o socket do gup!\n` + ret.error);
                    setTimeout(conectarComSocketGupshup, 1000);
                } else {
                    // Salva o socket
                    log.info(`[DISCADOR-WHATSAPP] Login efetuado com sucesso ClientID = ${process.env.GUPSHUP_CLIENTID}`);
                    gupshupSocket = socket;
                }
            });
        });

        setTimeout(() => {
            // Se o evento foi processado, ignora o timeout
            if (handled) return;

            // Demorou demais para conectar e fazer a autenticação pelo whatsapp
            log.info(`[DISCADOR-WHATSAPP] Timeout na conexão e/ou login com o socket do GUP!`);
        }, TEMPO_TIMEOUT);
    }
    // conectarComSocketWhatsApp();
    // conectarComSocketZenvia();
    async function enviaTemplateZenvia(fila, atendimento, template, destino, campos, campanha = null) {
        return new Promise(async (resolve, reject) => {
            try {
                let config = fila.emailConfig;
                // let fields = {};
                // fields = Object.keys(campos).filter(k => template.fields.findIndex(k) !== -1);
                if (atendimento.queueTypeID == 5) {
                    zenviaSocket.emit('newWhatsMsgZenvia', { zenviaID: config.zenviaID, from: config.from, to: destino, text: template.components.body.text, templateId: template.id, fields: campos }, async (retorno) => {

                        if (retorno && !retorno.error) {
                            if (atendimento) {
                                let texto = template.components.body.text;
                                for (let key in campos) {
                                    texto = texto.replace(`{{${key}}}`, campos[key]);
                                }                        // Cria uma mensagem e registra no atendimento
                                await new AtendimentoMensagem({
                                    idAtendimento: atendimento.id,
                                    dataHora: new Date(),
                                    agente: "Xcontact",
                                    texto: texto,
                                    inOut: 3,
                                    status: retorno.error ? `pending` : `seen`
                                }).save();
                            }

                            resolve();
                        } else {
                            reject("Erro ao enviar mensagem de texto: " + JSON.stringify(retorno));
                        }
                    });
                }
                if (atendimento.queueTypeID == 7) {
                    // if (template.templateType !== 'TEXT') {                        
                    //    return enviarMensagemWhatsAppArquivoGup(atendimento,template,campanha.caminho_arquivo_campanha,campanha,destino)
                    // }
                    console.log(campos)
                    const verificaVariaveisTemplate = await LayoutCampos.query.where({id_layout:campanha.id_layout}).require(false).fetch()
                    let camposAux = []
                    verificaVariaveisTemplate.map(v=>{
                        if(campos.hasOwnProperty(v.nome) && v.xcontact === 'variavelTemplate') camposAux.push(campos[`${v.nome}`])
                    } )
                    console.log(camposAux)
                    gupshupSocket.emit('newWhatsMsgGupshup', { apikey: config.apikey, source: config.source, appName: config.appName, to: destino, text: template.data, template: { id: template.id, imageUrl:campos.imageUrl, fields: camposAux } }, async (retorno) => {

                        if (retorno && !retorno.error) {
                            if (atendimento) {
                                let texto = template.data;
                                for (let key in campos) {
                                    texto = texto.replace(`{{${key}}}`, campos[key]);
                                }                        // Cria uma mensagem e registra no atendimento
                                await new AtendimentoMensagem({
                                    idAtendimento: atendimento.id,
                                    dataHora: new Date(),
                                    agente: "Xcontact",
                                    texto: texto,
                                    inOut: 3,
                                    status: retorno.error ? `pending` : `seen`
                                }).save();
                            }

                            resolve();
                        } else {
                            reject("Erro ao enviar mensagem de texto: " + JSON.stringify(retorno));
                        }
                    });
                }

            } catch (error) {
                console.error(error)
                reject(error);
            }

        })

    }

    function esperarParaLigar() {
        return new Promise((resolve, reject) => {
            try {
                const maximo = 3500;// 3,5 segundos no máximo
                const minimo = 1000;// Mínimo 1 segundo

                // Gera o tempo aleatório entre as contantes acima
                let time = Math.random() * (maximo - minimo) + minimo;
                log.debug("PROCESSACAMPANHA: Esperando " + time + "ms ...");

                // Espera o tempo definido e da resolve na promise
                setTimeout(() => {
                    resolve();
                }, time);
            } catch (e) {
                log.error(e);
                console.error(e);
                reject(e);
            }
        });
    }

    async function ProcessaCampanha(campanha) {
        try {
            // if (campanha.tipo_campanha !== 0) return;
            console.log('PROCESSACAMPANHA: Processando campanha "' + campanha.nome + '" com o tipo ' + campanha.tipo_campanha);
            console.log(campanha)

            // DEBUG: Alimenta a campanha no depurador, e busca informações do banco
            Depurador.notificarProcessamento(campanha);

            // Verifica se a campanha não está ativa
            if (campanha.status !== "A") {
                console.log(`PROCESSACAMPANHA: A campanha ${campanha.nome} não está ativa no array de campanhas do index.`)
                // DEBUG: Informa que a campanha está parada
                Depurador.atualizarCampanha(campanha.id, {
                    status_descricao: "Campanha não ativa",
                    status: campanha.status,
                    rodando: false
                });
                return;
            }


            // Verifica se a campanha possui turnos definidos, e se possuir, se algum desses turnos é válido
            if (await DiscadorCommon.verificarTurnos(campanha)) {
                console.log(`PROCESSACAMPANHA: A campanha ${campanha.nome} está com turno ok.`)
                Depurador.atualizarCampanha(campanha.id, { turnoValido: true });
            } else {
                console.log(`PROCESSACAMPANHA: A campanha ${campanha.nome} abortada por tunos inválidos.`)
                // DEBUG: Informa que a campanha parou
                Depurador.atualizarCampanha(campanha.id, {
                    status_descricao: "Processamento abortado por turnos inválidos",
                    rodando: false,
                    turnoValido: false
                });
                return;
            }

            // Verifica se a campanha possui agendamento e se o mesmo está ativo
            if (!DiscadorCommon.verificarAgendamento(campanha)) {
                console.log(`PROCESSACAMPANHA: A campanha ${campanha.nome} foi abortada por agendamento.`)
                // DEBUG: Informa que a campanha parou
                Depurador.atualizarCampanha(campanha.id, {
                    status_descricao: "Processamento abortado por agendamento",
                    rodando: false,
                    agendamentoValido: false
                });
                return;
            }

            if (campanha.ligacoes_ativas) {
                try {
                    if (campanha.ligacoes_ativas.length >= campanha.max_simultaneas_campanha) {
                        log.info(`PROCESSACAMPANHA: ABORTANDO. A campanha ${campanha.nome} já tem ${campanha.ligacoes_ativas.length} ligações ativas. O máximo para essa campanha é ${campanha.max_simultaneas_campanha}`);

                        // DEBUG: Informa que a campanha parou
                        Depurador.atualizarCampanha(campanha.id, {
                            status_descricao: "Processamento abortado por excesso de ligações ativas!",
                            rodando: false
                        });
                        return;
                    } else {
                        log.debug(`PROCESSACAMPANHA: A campanha '${campanha.nome}' não atingiu o limite máximo para ligações! seguindo o fluxo...`);
                    }
                } catch (e) {
                    log.error(e);
                    console.error(e);
                    console.trace(e);
                }
            } else {
                campanha.ligacoes_ativas = [];
            }

            // Pega os agentes disponíveis
            var agentes = await Ami.getAgentesOnline(campanha.fila);

            // Bugfix - Caso o AMI retorne uma resposta inválida, mostra nos logs para investigação futura.
            if (!agentes) {
                log.error(`PROCESSACAMPANHA: Campanha ${campanha.nome}. Ocorreu algum erro ao puxar os agentes online do AMI!`);
                agentes = [];
            }

            // DEBUG: Atualiza o array de agentes online disponíveis para a campanha.
            Depurador.atualizarCampanha(campanha.id, { agentes, ligacoes_ativas: campanha.ligacoes_ativas });

            // Verifica se existem agentes disponíveis (Apenas se a flag 'requer_agente_online' estiver setada!)
            if (agentes.length <= 0 && campanha.requer_agente_online) {
                log.info(`PROCESSACAMPANHA: A campanha ${campanha.nome} associada a fila ${campanha.fila} não possui agentes online, nada para fazer agora.`);

                // DEBUG: Informa que a campanha parou
                Depurador.atualizarCampanha(campanha.id, {
                    status_descricao: "Processamento abortado por falta de agentes online",
                    rodando: false
                });
                return;
            }

            log.debug(`LIGACOESATIVAS: Campanha '${campanha.nome}' -> ${JSON.stringify(campanha.ligacoes_ativas)}`);

            // Calcula quantas ligações pode fazer agora, sem exceder o limite da campanha
            let simultaneas = 0;
            let quantidadeDeLigacoesPorAgente = 0;
            // Segura o vínculo entre as ligações da campanha, usado para linkar ligações simultâneas para posteriormente desconecta-las
            let vinculo = null;

            if (campanha.requer_agente_online) {
                simultaneas = agentes.length * campanha.max_por_agente - campanha.ligacoes_ativas.length;

                // Se a flag de desconectar simultâneas estiver ativas, e podermos ter mais que uma ligação por agente
                // Vincula as ligações e calcula quantas ligações por agente
                if (campanha.max_por_agente > 1 && campanha.desconecta_multiplas) {
                    quantidadeDeLigacoesPorAgente = agentes.length - campanha.ligacoes_ativas.length;
                    vinculo = new Date().getTime();
                }
            } else {
                simultaneas = campanha.max_simultaneas_campanha - campanha.ligacoes_ativas.length;
            }

            if (simultaneas > campanha.max_simultaneas_campanha) {
                log.debug('PROCESSACAMPANHA: Iria inciar ' + simultaneas + ' ligações, mas esse número é maior que o máximo de ' + campanha.max_simultaneas_campanha + ' ligaçãoes definidas na campanha.');
                simultaneas = campanha.max_simultaneas_campanha - campanha.ligacoes_ativas.length;
            }

            Depurador.atualizarCampanha(campanha.id, { simultaneas, quantidadeDeLigacoesPorAgente });


            if (simultaneas <= 0) {
                if (campanha.max_por_agente) {
                    log.info(`PROCESSACAMPANHA: Abortando. A campanha ${campanha.nome} possui ${campanha.ligacoes_ativas.length}/${campanha.max_por_agente * agentes.length} ligações ativas | Simultâneas: ${simultaneas}`);

                    // DEBUG: Informa que a campanha parou
                    Depurador.atualizarCampanha(campanha.id, {
                        status_descricao: `Processamento abortado por excesso de ligações simultâneas ativas! ${campanha.ligacoes_ativas.length}/${campanha.max_por_agente * agentes.length}`,
                        rodando: false
                    });
                } else {
                    log.info(`PROCESSACAMPANHA: Abortando. A campanha ${campanha.nome} possui ${campanha.ligacoes_ativas.length}/${campanha.max_simultaneas_campanha} ligações ativas | Simultâneas: ${simultaneas}`);

                    // DEBUG: Informa que a campanha parou
                    Depurador.atualizarCampanha(campanha.id, {
                        status_descricao: `Processamento abortado por excesso de ligações simultâneas ativas! ${campanha.ligacoes_ativas.length}/${campanha.max_simultaneas_campanha}`,
                        rodando: false
                    });
                }

                return;
            }

            // Em caso de campanhas muito pequenas precisamos garantir que o discador não execute a segunda tentativa enquanto ainda esta fazendo a primeira
            // Para isso vamos montar um array com os IDs dos números ativos e ramos remove-los da busca.
            var ativas_array = [];
            for (let la = 0; la < campanha.ligacoes_ativas.length; ++la) {
                ativas_array.push(campanha.ligacoes_ativas[la].NumeroID);
            }
            // Se não tiver nenhuma ativa coloca -1 no array para não ter que montar outra query...
            if (ativas_array.length == 0) {
                ativas_array.push(-1);
            }

            log.debug(`PROCESSACAMPANHA: Array de ligações ativas: ` + JSON.stringify(ativas_array));
            log.debug('PROCESSACAMPANHA: Buscando ' + simultaneas + ' novo(s) número(s) para discar');

            // Define a ordenação por prioridades
            // Assim, apenas prioridades acima de 0 vão alterar a ordem da ordenação
            let casePrioridade = /*sql*/`CASE
                                    WHEN prioridade <= 0 THEN 0
                                    WHEN prioridade >= 1 THEN prioridade
                                END DESC`;

            // Busca os contatos no banco


            // Atualiza os contatos selecionados
            var c = null;
            if (campanha.tipo_campanha == 0) {
                let selectQuery = /*sql*/`SELECT id FROM public.campanhas_contato_numeros WHERE
                                    id != ALL(ARRAY[${ativas_array}]) AND
                                    id_campanha=${campanha.id} AND
                                    sucesso=false AND
                                    tentativas < ${campanha.tentativas} AND
                                    retentativas < ${campanha.tentativas}
                                    ORDER BY ${casePrioridade},
                                            fone_sequencia ASC,
                                            tentativas ASC,
                                            retentativas ASC,
                                            id_contato ASC
                                    LIMIT ${simultaneas}`;
                let updateQuery = /*sql*/`UPDATE public.campanhas_contato_numeros
                                SET
                                    tentativas=tentativas+1,
                                    prioridade=prioridade-1
                                WHERE
                                    id_campanha=${campanha.id} AND
                                    id IN (${selectQuery})
                                    returning *`;
                "UPDATE public.campanhas_contato_numeros set tentativas = tentativas + 1 WHERE id_campanha = $1 AND id IN(SELECT id FROM public.campanhas_contato_numeros WHERE id != ALL(ARRAY[" + ativas_array + "]) AND id_campanha = $1 AND sucesso = false AND tentativas < $2 AND retentativas < $2 order by fone_sequencia, tentativas, retentativas, id_contato LIMIT $3) returning *";

                c = await db.query(updateQuery);

                // Verifica se não acabou os contatos
                if (!c || !c.rows || c.rows.length <= 0) {
                    log.info(`PROCESSACAMPANHA: Campanha '${campanha.nome}'. Fim, nada para discar`);

                    if (campanha.autofim) {
                        log.info(`PROCESSACAMPANHA: Campanha '${campanha.nome}' com AUTOFIM ativo, fechando campanha ${campanha.nome}`);
                        campanha.status = 'F';
                        campanha.data_status = moment.tz(options.config.servidor.timezone).format('YYYY-MM-DD HH:mm:ss');
                        FechaCampanha(campanha);

                        // DEBUG: Informa que a campanha parou
                        Depurador.atualizarCampanha(campanha.id, {
                            status_descricao: "Processamento abortado por falta de contatos! Campanha finalizada.",
                            rodando: false,
                            status: "F"
                        });
                    } else {
                        // DEBUG: Informa que a campanha parou
                        Depurador.atualizarCampanha(campanha.id, {
                            status_descricao: "Processamento abortado por falta de contatos!",
                            rodando: false
                        });
                    }

                    return;
                }
            } else {
                let selectQuery1 = /*sql*/`SELECT id FROM public.campanhas_contato_numeros WHERE                                    
                                    id_campanha=${campanha.id} AND
                                    sucesso=false AND
                                    retentativas < ${campanha.tentativas}
                                    ORDER BY  ${casePrioridade}, id_contato ASC
                                    LIMIT ${simultaneas}`;
                let updateQuery = /*sql*/`UPDATE public.campanhas_contato_numeros
                                SET
                                    tentativas=tentativas,
                                    prioridade=prioridade-1
                                WHERE
                                    id_campanha=${campanha.id} AND
                                    id IN (${selectQuery1})
                                    returning *`;
                //  "UPDATE public.campanhas_contato_numeros set tentativas = 1 WHERE id_campanha = $1 AND id IN(SELECT id FROM public.campanhas_contato_numeros WHERE id != ALL(ARRAY[" + ativas_array + "]) AND id_campanha = $1 AND sucesso = false AND tentativas < $2 AND retentativas < $2 order by fone_sequencia, tentativas, retentativas, id_contato LIMIT $3) returning *";
                c = await db.query(updateQuery);
                c.rows = c.rows.sort((a, b) => parseInt(a.id_contato) - parseInt(b.id_contato));
            }

            var numeros = c.rows;
            Depurador.atualizarCampanha(campanha.id, { status_descricao: `Discando para ${numeros.length} números...` });
            log.debug('PROCESSACAMPANHA: Retornou: ' + numeros.length + ' contatos para discar.');

            // Monta objeto para discagem           
            for (let n = 0; n < numeros.length; ++n) {
                const numero = numeros[n];
                const contato = {};

                if (vinculo) {
                    contato.vinculo = vinculo;
                    contato.vinculo_qtd = quantidadeDeLigacoesPorAgente;
                    contato.vinculo_total = simultaneas;
                    log.debug(`PROCESSACAMPANHA: Com ${numero.numero} vínculo!`);
                }

                contato.numero = {};
                contato.campos = {};
                contato.id = numero.id_contato;
                contato.id_numero = numero.id;
                for (const co in numero) {
                    contato.numero[co] = numero[co];
                }

                // Campos adicionais
                var campos_q = await db.query("SELECT * from public.campanhas_contato_campos WHERE id_contato = $1", [numero.id_contato]);
                for (let cq = 0; cq < campos_q.rows.length; ++cq) {
                    contato.campos[campos_q.rows[cq].nome] = campos_q.rows[cq].valor;
                }

                // Hora de discar, antes preciso colocar o numero nas ativas...
                var Ligacao = { CampanhaID: campanha.id, Status: 99, StatusDescricao: 'Load', ContatoID: contato.id, NumeroID: contato.numero.id, Vinculo: contato.vinculo, VinculoQTD: contato.vinculo_qtd, VinculoTOTAL: contato.vinculo_total, Destino: contato.numero.numero, Campos: contato.campos };
                AddOrUpdateLigacaoAtiva(campanha, Ligacao);

                // Se existe mais que um contato pra ligar, espera antes de fazer a próxima ligação
                if (campanha.tipo_campanha == 0 && numeros.length > 1 && n > 0) {
                    await esperarParaLigar();
                }


                // Verifica se é Power ou Preview
                if (campanha.tipo_campanha == 0) {
                    if (campanha.modo === 'A') {
                        log.debug("PROCESSACAMPANHA: Fim, enviado para Discagem Power");
                        DiscaPower(campanha, contato);
                    } else {
                        log.debug("PROCESSACAMPANHA: Fim, enviado para Discagem preview");
                        Disca(campanha, contato, agentes);
                    }
                }
                if (campanha.tipo_campanha == 1) {
                    await enviarMensagemWhatsApp(campanha, contato, Ligacao);
                    return;
                }

            }


        } catch (e) {
            console.error(e);
            console.trace(e);
            log.error("DISCADOR: ERRO! " + e);
        }
    }
    function criarAtendimentoEncerrado(campanha, nome, numero, dados) {
        return new Promise(async (resolve, reject) => {
            try {
                // Busca um cliente, dado o nome e o número
                let cliente = await Cliente.getClientFromMsg({
                    idCliente: numero,
                    nomeCliente: nome
                });
                let filaAmi = await Fila.query.require(true).first()
                    .where({ name: campanha.fila })
                    .fetch();
                if (cliente) {
                    for (var name in dados) {
                        let verifica = await ClienteCamposAdicionais.query.require(false).first()
                            .where({ nome: name, clienteID: cliente.id }).fetch();
                        //vamos atualizar os que ja tem
                        if (verifica) {
                            verifica.valor = dados[name];
                            verifica.save();
                        }
                        //Vamos adicionar caso nao esteja no verifica
                        if (name !== "nome" && name !== "numero" && !verifica) {
                            //vamos verificar se e tipo telefone ou tipo texto
                            let guarda = await LayoutCampos.query.first().where({ id_layout: campanha.id_layout, nome: name }).fetch()
                            console.log(`GUARDA++++++++`, guarda)
                            if (guarda && !guarda.xcontact) {
                                let campo = {};
                                let valor = dados[name];
                                campo.nome = name;
                                campo.valor = valor;
                                campo.clienteID = cliente.id;
                                console.log(`CAMPO`, campo);
                                new ClienteCamposAdicionais(campo).save()

                            }

                        }



                    }
                    // Cria o atendimento
                    let clienteAtendimento = new ClienteAtendimento({
                        queue: campanha.fila,
                        queueTypeID: filaAmi.queue_type,
                        sentido: "Sainte",
                        status: campanha.status_atendimento ? campanha.status_atendimento : "Encerrado",
                        idCliente: cliente.id,
                        fromID: cliente.numero,
                        dataFim: new Date()
                    });

                    //console.log('Criando Atendimento via Whats');
                    //console.log(campanha);

                    // Salva no banco
                    resolve(await clienteAtendimento.save());
                }

                resolve(null);
            } catch (error) {
                console.trace(error);
                log.error(`[DISCADOR-WHATSAPP] Não foi possível criar um atendimento! ` + error);

                resolve(null);
            }
        });
    }
    function tratarNumeroDestinatario(numero, campanha) {
        // Se especificado, insere o DDI (Código do país) no início do número do destinatário
        if (campanha && campanha.ddi_padrao) numero = campanha.ddi_padrao + numero;
        // Não deveria ser necessário... porém apenas pra termos certeza, que não vai puxar números inválidos!
        numero = TextoUtil.limparTextoNumerico(numero);
        // Verifica se precisa do sufixo do WhatsApp
        // if (!numero.endsWith("@c.us")) numero += "@c.us";

        return numero;
    }
    function injetarVariaveisCoringa(texto, camposAdicionais) {
        log.info(`[DISCADOR-WHATSAPP] Injetando variáveis coringa no texto!`);

        if (!texto || texto === "") return null;

        try {
            // Itera todos os campos adicionais provenientes do banco
            for (let key in camposAdicionais) {
                // Ignora propriedades nativas do objeto
                if (!camposAdicionais.hasOwnProperty(key)) continue;

                // Substitui o $ seguido por uma key pelo seu respectivo valor nos campos adicionais inseridos
                let regex = new RegExp(`\\$(${key})`, `gi`);
                texto = texto.replace(regex, camposAdicionais[key]);
            }

            // Retorna o texto processado
            return texto;
        } catch (e) {
            console.trace(e);
            log.info(`[DISCADOR-WHATSAPP] Erro ao injetar variáveis coringa na campanha!\n` + e);
        }
    }
    function enviarMensagemWhatsAppTexto(atendimento, texto, campanha, numero) {
        return new Promise((resolve, reject) => {
            try {
                // Define os argumentos para enviar ao CSA
                let options = {
                    instanceID: campanha.fila,
                    chatId: numero,
                    msg: texto
                };
                console.log(`++++++++++++++++++++++++++++++++++++++++++++++++++++++ENTROU++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++`)
                log.info(`[DISCADOR-WHATSAPP] Dados do 'newWhatsMsg' -> ${JSON.stringify(options)}`);

                // Envia a mensagem
                whatsappSocket.emit("newWhatsMsg", options, async retorno => {
                    log.info(`[DISCADOR-WHATSAPP] Retorno do 'newWhatsMsg' -> ${JSON.stringify(retorno)}`);
                    // Se um atendimento foi provido
                    if (atendimento) {
                        // Cria uma mensagem e registra no atendimento
                        await new AtendimentoMensagem({
                            idAtendimento: atendimento.id,
                            dataHora: new Date(),
                            agente: "Xcontact",
                            texto: texto,
                            inOut: 3,
                            status: retorno.error ? `pending` : `seen`
                        }).save();
                    }

                    if (retorno && !retorno.error) {
                        resolve();
                    } else {
                        reject("Erro ao enviar mensagem de texto: " + JSON.stringify(retorno));
                    }
                });
            } catch (error) {
                console.error(error);
                reject(error);
            }
        });
    }

    function enviarMensagemWhatsAppArquivo(atendimento, texto, caminho_arquivo, campanha, numero) {
        return new Promise((resolve, reject) => {
            try {
                // Cria uma stream para ler o arquivo
                let stream = fs.createReadStream(caminho_arquivo, { autoClose: true });

                // Define os argumentos para enviar ao CSA
                let options = {
                    instanceID: campanha.fila,
                    chatId: numero,
                    dados: {
                        uid: UUID.v1(),
                        Upload: "start",
                        mimetype: mime.lookup(caminho_arquivo),
                        legenda: texto || "",
                    }
                };

                log.info(`[DISCADOR-WHATSAPP] Dados do arquivo '${caminho_arquivo}'\n${JSON.stringify(options)}`);

                // Envia a mensagem
                whatsappSocket.emit("sendMsgFile", options);

                stream.on("error", error => {
                    log.error(`[DISCADOR-WHATSAPP] Erro ao ler arquivo da campanha! Arquivo '${caminho_arquivo}'`);
                    reject(error);
                });

                // Quando receber um pedaço do arquivo
                stream.on("data", chunk => {
                    // log.info(`[DISCADOR-WHATSAPP] Recebido novo chunk do arquivo '${campanha.caminho_imagem_campanha}'`);

                    // Atualiza o status do upload
                    options.dados.Upload = "sending";
                    // Envia para o CSA
                    whatsappSocket.emit("sendMsgFile", options, null, chunk);
                });

                // Ao receber o arquivo inteiro
                stream.on("end", () => {
                    // log.info(`[DISCADOR-WHATSAPP] Recebido fim arquivo '${caminho_arquivo}'`);

                    // Atualiza o status do upload
                    options.dados.Upload = "end";
                    // Envia para o CSA, esperando uma resposta 'OK'
                    setTimeout(() => {
                        reject("Erro ao enviar arquivo: NUMERO NAO EXISTE ");
                    }, 20000);
                    whatsappSocket.emit("sendMsgFile", options, async retorno => {
                        if (retorno && !retorno.error) {
                            log.info(`[DISCADOR-WHATSAPP] Recebido retorno do 'sendMsgFile'`);

                            // Se um atendimento foi provido
                            if (atendimento) {
                                // Cria uma mensagem vazia (Ou como legenda) para anexar a imagem nela
                                let legenda = await new AtendimentoMensagem({
                                    idAtendimento: atendimento.id,
                                    dataHora: new Date(),
                                    agente: "Xcontact",
                                    texto: texto || "",
                                    inOut: 3,
                                    status: retorno.error ? `pending` : `seen`

                                }).save();

                                // Anexa o arquivo na mensagem criada acima
                                await new MensagemAnexos({
                                    idMensagem: legenda.id,
                                    caminho: caminho_arquivo,
                                    mimeType: options.dados.mimetype
                                }).save();
                            }

                            resolve();
                        } else {
                            log.error("Erro ao enviar arquivo: " + JSON.stringify(retorno));
                            reject("Erro ao enviar arquivo: " + JSON.stringify(retorno));
                        }
                    });
                });
            } catch (error) {
                reject(error);
            }
        });
    }

        function enviarMensagemWhatsAppArquivoGup(atendimento, template, caminho_arquivo, campanha, numero) {
        return new Promise((resolve, reject) => {
            try {
                // Cria uma stream para ler o arquivo
                let stream = fs.createReadStream(caminho_arquivo, { autoClose: true });

                // Define os argumentos para enviar ao CSA
                let options = {
                    instanceID: campanha.fila,
                    chatId: numero,
                    dados: {
                        uid: UUID.v1(),
                        Upload: "start",
                        mimetype: mime.lookup(caminho_arquivo),
                        legenda: "",
                    }
                };

                log.info(`[DISCADOR-WHATSAPP] Dados do arquivo '${caminho_arquivo}'\n${JSON.stringify(options)}`);

                // Envia a mensagem
                gupshupSocket.emit("sendMsgFile", options);

                stream.on("error", error => {
                    log.error(`[DISCADOR-WHATSAPP] Erro ao ler arquivo da campanha! Arquivo '${caminho_arquivo}'`);
                    reject(error);
                });

                // Quando receber um pedaço do arquivo
                stream.on("data", chunk => {
                    // log.info(`[DISCADOR-WHATSAPP] Recebido novo chunk do arquivo '${campanha.caminho_imagem_campanha}'`);

                    // Atualiza o status do upload
                    options.dados.Upload = "sending";
                    // Envia para o CSA
                    gupshupSocket.emit("sendMsgFile", options, null, chunk);
                });

                // Ao receber o arquivo inteiro
                stream.on("end", () => {
                    // log.info(`[DISCADOR-WHATSAPP] Recebido fim arquivo '${caminho_arquivo}'`);

                    // Atualiza o status do upload
                    options.dados.Upload = "end";
                    options.dados.template = { id: template.templateId, fields: template.fields } 
                    // Envia para o CSA, esperando uma resposta 'OK'
                    setTimeout(() => {
                        reject("Erro ao enviar arquivo: NUMERO NAO EXISTE ");
                    }, 20000);
                    gupshupSocket.emit("sendMsgFile", options, async retorno => {
                        if (retorno && !retorno.error) {
                            log.info(`[DISCADOR-WHATSAPP] Recebido retorno do 'sendMsgFile'`);

                            // Se um atendimento foi provido
                            if (atendimento) {
                                // Cria uma mensagem vazia (Ou como legenda) para anexar a imagem nela
                                let legenda = await new AtendimentoMensagem({
                                    idAtendimento: atendimento.id,
                                    dataHora: new Date(),
                                    agente: "Xcontact",
                                    texto: template.message,
                                    inOut: 7,
                                    status: retorno.error ? `pending` : `seen`

                                }).save();

                                // Anexa o arquivo na mensagem criada acima
                                await new MensagemAnexos({
                                    idMensagem: legenda.id,
                                    caminho: caminho_arquivo,
                                    mimeType: options.dados.mimetype
                                }).save();
                            }

                            resolve();
                        } else {
                            log.error("Erro ao enviar arquivo: " + JSON.stringify(retorno));
                            reject("Erro ao enviar arquivo: " + JSON.stringify(retorno));
                        }
                    });
                });
            } catch (error) {
                reject(error);
            }
        });
    }
    function enviarMensagemWhatsApp(campanha, contato, Ligacao) {
        return new Promise(async (resolve, reject) => {
            log.info(`[DISCADOR-WHATSAPP] Enviando mensagem para ${contato.numero.numero} na campanha ${campanha.id}`);
            console.log(`================================================`, campanha.rota);
            // Define o objeto da ligação
            let registro = {
                CampanhaID: campanha.id,
                ContatoID: contato.id,
                NumeroID: contato.id_numero,
                Destino: contato.numero.numero,
                Linkedid: 0,
                ActionID: 0,
                Falha: false,
                Status: 9,
                StatusDescricao: "Iniciando",
                Campos: contato.campos
            };

            // Trata o número do destinatário
            let destinatario = tratarNumeroDestinatario(contato.numero.numero, campanha);

            // Variável para controlar o timeout do socket
            let handled = false;

            // Variável para guardar as promises de envio, porque o envio pode acontecer em múltiplas etapas
            let promiseList = [];

            let atendimento = await criarAtendimentoEncerrado(campanha, contato.campos["Nome"] || contato.campos["nome"], contato.numero.numero, contato.campos);

            // Se especificada uma imagem na campanha
            if (campanha.caminho_imagem_campanha) {
                let legenda = injetarVariaveisCoringa(campanha.legenda_imagem_campanha, contato.campos);
                log.info(`[DISCADOR-WHATSAPP] Enviando imagem para o número ${destinatario} ${legenda ? 'com legenda' : 'sem legenda'}`);
                promiseList.push(enviarMensagemWhatsAppArquivo.bind(this, atendimento, legenda, campanha.caminho_imagem_campanha, campanha, destinatario));
            }

            // Se especificado um arquivo na campanha
            if (campanha.caminho_arquivo_campanha) {
                log.info(`[DISCADOR-WHATSAPP] Enviando arquivo para o número ${destinatario}`);
                promiseList.push(enviarMensagemWhatsAppArquivo.bind(this, atendimento, path.basename(campanha.caminho_arquivo_campanha), campanha.caminho_arquivo_campanha, campanha, destinatario));
            }

            // Verifica se existe um texto para enviar, trata como prioridade o campo 'texto' do contato, tendo como fallback o texto_padrao_campanha
            //Vamos buscar o nome certo para o campo de texto
            let nomeCampo = await LayoutCampos.query.first().where({ id_layout: campanha.id_layout, xcontact: 'texto' }).fetch();
            console.log(`RETORNOU`, nomeCampo);
            //let text = contato.campos["Texto"] ? contato.campos["Texto"] : contato.campos["mensagem"];
            let textoPadrao = injetarVariaveisCoringa(campanha.texto_padrao_campanha, contato.campos);
            if (textoPadrao) {
                log.info(`[DISCADOR-WHATSAPP] Mensagem tipo texto para o número ${destinatario}`);
                promiseList.push(enviarMensagemWhatsAppTexto.bind(this, atendimento, textoPadrao, campanha, destinatario));
            }
            let texto = injetarVariaveisCoringa(contato.campos[`${nomeCampo.nome}`]);
            console.log(`==============================`, texto);
            if (texto && texto != "") {
                // Apenas para ter certeza que a mensagem foi enviada, espera dois segundos
                // Pois nos testes iniciais o CSA estava enviando o texto da mensagem antes da imagem, mesmo tudo sendo feito sequencialmente...
                if (campanha.caminho_imagem_campanha)
                    promiseList.push(() => new Promise((resolve, _) => setTimeout(resolve, 5000)));

                log.info(`[DISCADOR-WHATSAPP] Mensagem tipo texto para o número ${destinatario}`);
                promiseList.push(enviarMensagemWhatsAppTexto.bind(this, atendimento, texto, campanha, destinatario));
            }
            //Vamos verificar se tem template
            if (campanha.rota && campanha.rota != 'Discador' && campanha.rota != 'discador') {
                let template = JSON.parse(campanha.rota);
                var fila = await Fila.query.require(true).first()
                    .where({ name: atendimento.queue })
                    .fetch();
                log.info(`[DISCADOR-WHATSAPP] Mensagem tipo texto para o número ${destinatario}`);
                promiseList.push(enviaTemplateZenvia.bind(this, fila, atendimento, template, destinatario, contato.campos, campanha));
            }
            console.log(`VAMOS VER AS PROMISSES AQUI MANE`, promiseList);
            // Atribui os handlers dos eventos de sucesso e falha
            // Que são iguais para quaisquer tipos de campanha!!!!!
            Promise.sequential(promiseList).then(
                async response => {
                    log.info(`[DISCADOR-WHATSAPP] Mensagem enviada para ${destinatario} com sucesso!`, response);

                    // Notifica ao timeout que o evento já foi processado, então ignora o reject por tempo excedido
                    handled = true;

                    // Altera o status da ligação, para inserir no banco
                    registro.Status = 16;
                    registro.StatusDescricao = "Completou";
                    console.log(`VAMOS SALVAR _________________________________`, registro);
                    // Registra no banco o sucesso do contato
                    await DiscadorCommon.notificarConclusaoDaCampanha(registro);
                    await DiscadorCommon.inserirRegistroDaCampanha(registro);
                    RemoveLigacaoAtiva(Ligacao);
                    resolve(response);
                },
                async error => {
                    log.error(`[DISCADOR-WHATSAPP] Erro ao enviar mensagem para ${destinatario}!\n${error}`);

                    // Notifica ao timeout que o evento já foi processado, então ignora o reject por tempo excedido
                    handled = true;

                    // Altera o status da ligação, para inserir no banco
                    registro.Status = 4;
                    registro.StatusDescricao = "Falha";
                    registro.Falha = true;
                    if (atendimento) {
                        atendimento.status = 'Encerrado';
                        atendimento.save();
                    }
                    console.log(`VAMOS SALVAR _________________________________`, registro);
                    // Registra no banco a falha do contato
                    RemoveLigacaoAtiva(Ligacao);
                    await DiscadorCommon.inserirRegistroDaCampanha(registro);
                    await DiscadorCommon.notificarRetentativaDeContato(contato.id_numero);

                    reject(error);
                }
            );

            // Adiciona um 'timeout' no envio da mensagem
            setTimeout(async () => {
                // Se o evento foi processado, ignora o timeout
                if (handled) return;

                log.error(`[DISCADOR-WHATSAPP] Timeout ao enviar mensagem para ${destinatario}`);
                registro.Status = 4;
                registro.StatusDescricao = "Falha";
                registro.Falha = true;
                if (atendimento) {
                    atendimento.status = 'Encerrado';
                    atendimento.save();
                }
                console.log(`VAMOS SALVAR _________________________________`, registro);
                // Registra no banco a falha do contato
                RemoveLigacaoAtiva(Ligacao);
                await DiscadorCommon.inserirRegistroDaCampanha(registro);
                await DiscadorCommon.notificarRetentativaDeContato(contato.id_numero);

                // Demorou demais para conectar e fazer a autenticação pelo whatsapp
                reject("Timeout no envio da mensagem");
            }, 300000);
        });
    }

    function Disca(campanha, contato, agentes) {
        log.info("DISCA: Inicio da Discagem preview");
        if (agentes.length > 0) {
            const sorteado = agentes[Math.floor(Math.random() * agentes.length)];
            sorteado.status = 6;
            sorteado.DataStatus = moment.tz(options.config.servidor.timezone).format();

            log.info("DISCA: Agente sorteado para discar: Nome: " + sorteado.nome + ", Ramal: " + sorteado.ramal);
            log.info("DISCA: Discagem preview para Numero: " + contato.numero.numero + ", ContatoID: " + contato.numero.id_contato + ", Fila: " + campanha.fila + ", Tentativa: " + contato.numero.tentativas);
            try {
                const ac = new namiLib.Actions.Originate();
                //ac.channel  = sorteado.ramal;
                //ac.exten    = contato.numero.numero;
                const spt = sorteado.ramal.split('/');
                const tipo = spt[0];
                const ramal = spt[1];

                ac.channel = 'Local/s@discadorPreview';
                ac.exten = contato.numero.numero;
                ac.callerid = "Discador <" + contato.numero.numero + ">";
                ac.account = "Discador";
                ac.priority = 1;
                ac.context = campanha.rota;
                ac.timeout = campanha.max_chamar ? campanha.max_chamar * 1000 : 30 * 1000;
                ac.async = true;
                ac.ActionID = uuid();

                let ar = [];
                ar.push('ActionID=' + ac.ActionID);
                ar.push('TIPO=' + tipo);
                ar.push('RAMAL=' + ramal);
                ar.push('FILA=' + campanha.fila);
                //ar.push('XCONTACT_REGISTRA_GRAVACAO()='+ac.ActionID+','+monitor_filename+'.mp3');
                ar.push('__SIPADDHEADER51=X-UNIQUE-ID:' + ac.ActionID);

                if (contato.campos && Object.keys(contato.campos).length > 0) {
                    let id = 2;
                    Object.keys(contato.campos).forEach(function (key) {
                        //ar.push('__SIPADDHEADER5' + id + '=X-' + key + ':' + encodeURIComponent(contato.campos[key]));
                        ++id;
                    });
                }
                ac.variable = ar.join(",");

                var Ligacao = { CampanhaID: campanha.id, ActionID: ac.ActionID, Status: 10, StatusDescricao: 'Iniciando', ContatoID: contato.id, NumeroID: contato.id_numero, Vinculo: contato.vinculo, VinculoQTD: contato.vinculo_qtd, VinculoTOTAL: contato.vinculo_total, Destino: contato.numero.numero, Campos: contato.campos };
                Ligacao.inicio = moment.tz(options.config.servidor.timezone);

                Ami.nami.send(ac, function (resposta) {
                    if (resposta.response === 'Error') {
                        // Tenta notificar
                        try {
                            Ligacao.Falha = true;
                            Ligacao.Status = 9;
                            Ligacao.StatusDescricao = resposta.message;
                            Ligacao.fim = moment.tz(options.config.servidor.timezone);
                            const espera = moment.duration(Ligacao.fim.diff(Ligacao.inicio));
                            Ligacao.espera = espera.get("hours").toString().padStart(2, '0') + ":" + espera.get("minutes").toString().padStart(2, '0') + ":" + espera.get("seconds").toString().padStart(2, '0');
                            log.info("PROCESSARETORNOORIGINATE: A ligação para " + Ligacao.Destino + " falhou. Causa: " + Ligacao.StatusDescricao);

                            if (EMITIR_EVENTOS) {
                                ws.broadcast({ event: 'Discador', acao: 'FimLigacao', Falha: true, Ligacao: Ligacao });
                            }
                        } catch (ex) {
                            log.error("PROCESSARETORNOORIGINATE: Falha notificar evento Discador, acao: FimLigacao, ERRO: " + ex.toString());
                        }
                        // Remove ligação das ativas
                        RemoveLigacaoAtiva(Ligacao);

                        // Registra LOG
                        RegistraLog(Ligacao);

                        // Processa campanha novamente.
                        discador.ProcessaCampanha(campanha);
                    } else {
                        // Atualiza dados da ligação ativa
                        AddOrUpdateLigacaoAtiva(campanha, Ligacao);

                        if (EMITIR_EVENTOS) {
                            try {
                                ws.broadcast({ event: 'Discador', acao: 'NovaLigacao', Ligacao: Ligacao });
                            } catch (ex) {
                                log.error("DISCA: Falha notificar evento Discador, acao: NovaLigacao, ERRO: " + ex.toString());
                            }
                        }
                    }
                });
            } catch (ex) {
                log.error("DISCA: Falha ao criar Originate: " + ex.toString());
            }
        }
        else {
            log.info("DISCA: PROBLEMA, Tinha agente online mas entre selecionar o numero e discar não tem mais...:)");
        }
    }

    function DiscaPower(campanha, contato) {
        log.info("DISCAPOWER: Discagem power para Numero: " + contato.numero.numero + ", ContatoID: " + contato.numero.id_contato + ", Fila: " + campanha.fila + ", Tentativa: " + contato.numero.tentativas);
        try {
            const ac = new namiLib.Actions.Originate();
            ac.channel = "Local/" + contato.numero.numero + "@" + campanha.rota;
            if (campanha.exec_contexto) {
                // Manda pra rota
                ac.exten = campanha.macro;
                ac.priority = 1;
                ac.context = campanha.exec_contexto;
            }
            else {
                // Executa Macro
                ac.application = "Macro";
                ac.data = campanha.macro + "," + campanha.fila;
            }
            ac.callerid = contato.campos.Nome ? contato.campos.Nome + " <" + contato.numero.numero + ">" : contato.numero.numero;
            ac.timeout = campanha.max_chamar ? campanha.max_chamar * 1000 : 30 * 1000;
            ac.async = true;
            ac.ActionID = uuid();
            if (contato.campos && Object.keys(contato.campos).length > 0) {
                let ar = [];
                let id = 1;

                Object.keys(contato.campos).forEach(function (key) {
                    ar.push('__SIPADDHEADER5' + id + '=X-' + key + ':' + encodeURIComponent(contato.campos[key]));
                    ar.push(key + '=' + encodeURIComponent(contato.campos[key]));
                    ++id;
                });
                ac.variable = ar.join(",");
            }

            var Ligacao = { CampanhaID: campanha.id, ActionID: ac.ActionID, ChannelId: ac.channel, Status: 10, StatusDescricao: 'Iniciando', ContatoID: contato.id, NumeroID: contato.id_numero, Vinculo: contato.vinculo, VinculoQTD: contato.vinculo_qtd, VinculoTOTAL: contato.vinculo_total, Destino: contato.numero.numero, Campos: contato.campos };
            Ligacao.inicio = moment.tz(options.config.servidor.timezone);

            // Atualiza ligacao ativa.
            AddOrUpdateLigacaoAtiva(campanha, Ligacao);

            // Envia comando
            Ami.nami.send(ac);

            // Notifica
            if (EMITIR_EVENTOS) {
                try {
                    ws.broadcast({ event: 'Discador', acao: 'NovaLigacao', Ligacao: Ligacao });
                } catch (ex) {
                    log.error("DISCAPOWER: Falha notificar evento Discador, acao: NovaLigacao, ERRO: " + ex.toString());
                }
            }
        } catch (ex) {
            log.error("DISCAPOWER: Falha ao criar Originate: " + ex.toString());
        }
    }

    exports.reloadCampanhas = () => {
        log.info('RELOADCAMPANHAS: Recarregando ativas campanhas do banco');
        db.query("SELECT * FROM public.campanhas WHERE status = 'A' ORDER by id")
            .then(res => {
                campanhas = [];
                for (let i = 0; i < res.rows.length; ++i) {
                    const c = new Campanha();
                    const row = res.rows[i];
                    for (const co in row) {
                        c[co] = row[co];
                    }
                    campanhas.push(c);
                }

                // discadorWhatsApp.recarregarCampanhas(campanhas);
                conectarComSocketWhatsApp();
                conectarComSocketZenvia();
                conectarComSocketGupshup();
                log.info('RELOADCAMPANHAS: Campanhas recarregadas com sucesso.');
                ws.broadcast({ event: 'campanhas_reload', sucess: true, msg: 'Campanhas recarregadas com sucesso.' });
            })
            .catch(e => {
                log.error(e);
                ws.broadcast({ event: 'campanhas_reload', sucess: false, msg: e });
            });
    };

    const addCampanhaByID = (id) => {
        log.info('ADDCAMPANHABYID: Adicionando campanha id = ' + id);
        const knex = db.knex('public.campanhas');
        knex.where('id', id);
        knex.select('*')
            .then(rows => {
                discador.addCampanha(rows.find(x => x));
            })
            .catch(e => {
                log.error(e);
                ws.broadcast({ event: 'campanha_add', sucess: false, msg: e });
            });
    };

    exports.reloadCampanha = (id) => {
        log.info('RELOADCAMPANHA: Recarregando campanha id = ' + id);
        const knex = db.knex('public.campanhas');
        let campanha = {};
        knex.where('id', id);
        knex.select('*')
            .then(function (rows) {
                let tem = false;
                for (let i = 0; i < campanhas.length; ++i) {
                    if (campanhas[i].id === rows[0].id) {
                        campanhas[i] = rows[0];
                        campanha = campanhas[i];
                        if (campanha.tipo_campanha == 0)
                            discador.ProcessaCampanha(campanha);
                        tem = true;
                    }
                }
                if (!tem) {
                    log.info('RELOADCAMPANHA: Essa campanha não tá na lista, vou chamar add');
                    addCampanhaByID(id);
                } else {
                    // discadorWhatsApp.enfileirarCampanha(campanha);

                    log.info('RELOADCAMPANHA: Campanha id = ' + id + ' recarregada com sucesso.');
                    ws.broadcast({ event: 'campanha_reload', sucess: true, msg: 'Campanhas id = ' + id + ' recarregadas com sucesso.', campanha: campanha });
                }
            })
            .catch(e => {
                log.error(e);

                ws.broadcast({ event: 'campanha_reload', sucess: false, msg: e });
            });
    };

    exports.reloadCampanhasUpdate = (camps) => {
        console.log(`RELOADCAMPANHA: Recarregando ${camps.length} campanhas`);

        try {
            let ids = campanhas.map((c) => c.id);
            let existentes = camps.filter(c => ids.includes(c.id));
            let inexistentes = camps.filter(c => !ids.includes(c.id));

            if (existentes && existentes.length > 0) {
                for (let i = 0; i < existentes.length; i++) {
                    //atualiza a campanha dentro do array
                    let indexCampanha = campanhas.findIndex(c => c.id == existentes[i].id)
                    if (indexCampanha != -1) campanhas.splice(indexCampanha, 1, existentes[i]);

                    if (existentes[i].tipo_campanha == 0) discador.ProcessaCampanha(existentes[i])
                    console.log('RELOADCAMPANHA: Campanha id = ' + existentes[i].id + ' recarregada com sucesso.');
                    ws.broadcast({ event: 'campanha_reload', sucess: true, msg: 'Campanhas id = ' + existentes[i].id + ' recarregadas com sucesso.', campanha: existentes[i] });
                }
            }

            if (inexistentes && inexistentes.length > 0) {
                for (let i = 0; i < inexistentes.length; i++) {
                    console.log('RELOADCAMPANHA: Essa campanha não tá na lista, vou chamar add');
                    discador.addCampanha(inexistentes[i]);
                }
            }
        } catch (e) {
            console.log(e);
            ws.broadcast({ event: 'campanha_reload', sucess: false, msg: e });
        }
    };

    exports.addCampanha = (campanha) => {
        Depurador.adicionarCampanha(campanha);

        // De acordo com o tipo da campanha, manda para o handler certo
        // switch (campanha.tipo_campanha) {
        //     case 0:
        //         campanhas.push(campanha);
        //         discador.ProcessaCampanha(campanha);
        //         break;
        //     case 1:
        //         discadorWhatsApp.enfileirarCampanha(campanha);
        //         break;
        // }
        campanhas.push(campanha);
        discador.ProcessaCampanha(campanha);

        log.info('[DISCADOR] Campanha id = ' + campanha.id + ' adicionada com sucesso.');

        if (EMITIR_EVENTOS) {
            ws.broadcast({ event: 'campanha_add', sucess: true, msg: 'Nova campanha adicionada.', campanha });
        }
    };

    exports.delCampanha = (id) => {
        log.info(`[DISCADOR] Removendo a campanha '${id}'`);

        try {
            // Remove todas as campanhas com o id da listagem
            campanhas = campanhas.filter(x => x.id !== id);

            // Remove as campanhas do discador whatsapp
            // discadorWhatsApp.removerDaFila({ id });

            // DEBUG: Remove a campanha do depurador
            Depurador.removerCampanha(id);

            // Da broadcast no evento
            ws.broadcast({ event: 'campanha_del', sucess: true, msg: 'Campanha apagada.', campanhaID: id });
        } catch (error) {
            log.error("[DISCADOR] Erro ao remover campanha -> " + error);
            console.trace(error);
        }
    };

    exports.delCampanhas = (ids) => {
        console.log(`[DISCADOR] Removendo as campanha '${ids}'`);

        try {
            // Remove todas as campanhas com o id da listagem
            campanhas = campanhas.filter(x => !ids.includes(x.id));

            // DEBUG: Remove a campanha do depurador
            for (let id = 0; id < ids.length; id++) {
                Depurador.removerCampanha(id);
                ws.broadcast({ event: 'campanha_del', sucess: true, msg: 'Campanha apagada.', campanhaID: id });
            }
        } catch (error) {
            console.log("[DISCADOR] Erro ao remover as campanhas -> " + error);
            console.trace(error);
        }
    };

    exports.getCampanhas = () => {
        return campanhas;
    };

    exports.getCampanhaByID = (id) => {
        for (let i = 0; i < campanhas.length; ++i) {
            if (campanhas[i].id === id) {
                return campanhas[i];
            }
        }
    };

    exports.ProcessaCampanhas = () => {
        for (let i = 0; i < campanhas.length; ++i) {
            // discador.ProcessaCampanha(campanhas[i]);
            if (campanhas[i].tipo_campanha === 0)
                discador.ProcessaCampanha(campanhas[i]);

        }
    };

    exports.ProcessaCampanhasNew = () => {
        for (let i = 0; i < campanhas.length; ++i) {
            discador.ProcessaCampanha(campanhas[i]);
        }
    };

    exports.AdicionaCampanhasAgendadas = () => {
        let hoje = moment().format('YYYY-MM-DD')
        console.log('ADICIONACAMPANHASAGENDADAS: Recarregando ativas campanhas do banco');
        db.query(`SELECT * FROM public.campanhas WHERE data_ini::date = '${hoje}' ORDER by id`)
            .then(res => {
                for (let i = 0; i < res.rows.length; ++i) {
                    const c = new Campanha();
                    const row = res.rows[i];

                    for (const co in row) {
                        c[co] = row[co];
                    }

                    //Verifica se a campanha já está no array
                    let existe = campanhas.find(camp => camp.id == c.id)
                    if (!existe) {
                        campanhas.push(c);
                        console.log(`ADICIONACAMPANHASAGENDADAS: Campanha com o id ${c.id} adicionada ao array de campanha`)
                    } else { console.log(`ADICIONACAMPANHASAGENDADAS: Campanha com o id ${c.id} já estava no array de campanha`) }
                }
                console.log('ADICIONACAMPANHASAGENDADAS: Campanhas adicionadas com sucesso.');
            })
            .catch(e => {
                console.log('ADICIONACAMPANHASAGENDADAS: ' + e);
            });
    };

    exports.ProcessaCampanhasByFila = (fila) => {
        for (let i = 0; i < campanhas.length; ++i) {
            if (campanhas[i].fila === fila) {
                if (campanhas[i].tipo_campanha == 0)
                    discador.ProcessaCampanha(campanhas[i]);
            }
        }
    };

    function getLigacaoAtivaByActionID(campanha, ActionID) {
        if (!campanha.ligacoes_ativas || campanha.ligacoes_ativas.length === 0) return undefined;

        for (let i = 0; i < campanha.ligacoes_ativas.length; ++i) {
            if (campanha.ligacoes_ativas[i].ActionID === ActionID) {
                return campanha.ligacoes_ativas[i];
            }
        }

        return undefined;
    }

    function getLigacaoAtivaByContatoID(campanha, ContatoID) {
        if (!campanha.ligacoes_ativas || campanha.ligacoes_ativas.length === 0) return undefined;

        for (let i = 0; i < campanha.ligacoes_ativas.length; ++i) {
            if (campanha.ligacoes_ativas[i].ContatoID === ContatoID) {
                return campanha.ligacoes_ativas[i];
            }
        }

        return undefined;
    }

    function getLigacaoAtivaByNumeroID(campanha, NumeroID) {
        if (!campanha.ligacoes_ativas || campanha.ligacoes_ativas.length === 0) return undefined;

        for (let i = 0; i < campanha.ligacoes_ativas.length; ++i) {
            if (campanha.ligacoes_ativas[i].NumeroID === NumeroID) {
                return campanha.ligacoes_ativas[i];
            }
        }

        return undefined;
    }

    function getLigacaoByLinkedid(campanha, Linkedid) {
        if (!campanha.ligacoes_ativas || campanha.ligacoes_ativas.length === 0) return undefined;

        for (let i = 0; i < campanha.ligacoes_ativas.length; ++i) {
            if (campanha.ligacoes_ativas[i].Linkedid === Linkedid) {
                return campanha.ligacoes_ativas[i];
            }
        }

        return undefined;
    }

    function AddOrUpdateLigacaoAtiva(campanha, ligacao) {
        try {
            log.info('ADDORUPDATELIGACAOATIVA: Solicitação de add/update ligação ActionID: ' + ligacao.ActionID + ', NumeroID: ' + ligacao.NumeroID + ', ContatoID: ' + ligacao.ContatoID + ', na lista de ativas.');

            if (ligacao.fim) {
                log.info('ADDORUPDATELIGACAOATIVA: A ligação ActionID: ' + ligacao.ActionID + ', NumeroID: ' + ligacao.NumeroID + ', ContatoID: ' + ligacao.ContatoID + ', já finalizou, deveria ser removida não atualizada, ignorando...');
                return;
            }

            for (let i = 0; i < campanha.ligacoes_ativas.length; i++) {
                if (ligacao.ActionID && campanha.ligacoes_ativas[i].ActionID == ligacao.ActionID) {
                    log.info('ADDORUPDATELIGACAOATIVA: Ligacao encontrada por ActionID na posição ' + i + ', atualizando.');
                    var lig = campanha.ligacoes_ativas[i];
                    for (const key in ligacao) {
                        lig[key] = ligacao[key];
                    }

                    return;
                } else if (ligacao.NumeroID && campanha.ligacoes_ativas[i].NumeroID == ligacao.NumeroID) {
                    log.info('ADDORUPDATELIGACAOATIVA: Ligacao encontrada por NumeroID na posição ' + i + ', atualizando.');
                    var lig = campanha.ligacoes_ativas[i];
                    for (const key in ligacao) {
                        lig[key] = ligacao[key];
                    }

                    return;
                } else if (ligacao.ContatoID && campanha.ligacoes_ativas[i].ContatoID == ligacao.ContatoID) {
                    log.info('ADDORUPDATELIGACAOATIVA: Ligacao encontrada por ContatoID na posição ' + i + ', atualizando.');
                    var lig = campanha.ligacoes_ativas[i];
                    for (const key in ligacao) {
                        lig[key] = ligacao[key];
                    }

                    return;
                }
            }

            log.info('ADDORUPDATELIGACAOATIVA: NOVA: A Ligacao para ' + ligacao.Destino + ' NÃO foi encontrada nas ligações ativas, adicionando ela.');
            if (!campanha.ligacoes_ativas) campanha.ligacoes_ativas = [];
            campanha.ligacoes_ativas.push(ligacao);

            // Atualiza as ligações ativas no depurador
            Depurador.atualizarCampanha(campanha.id, {
                ligacoes_ativas: campanha.ligacoes_ativas
            });
        } catch (e) {
            console.error(e);
            console.trace(e);
            log.error("DISCADOR: ERRO! " + e);
        }
    }

    function RemoveLigacaoAtiva(ligacao) {
        // log.info('REMOVELIGACAOATIVA: Solicitação de remover ligação ActionID: ' + ligacao.ActionID + ', NumeroID: ' + ligacao.NumeroID + ', ContatoID: ' + ligacao.ContatoID + ', da lista de ativas.');

        for (let j = 0; j < campanhas.length; ++j) {
            var campanha = campanhas[j];
            if (campanha.ligacoes_ativas) {
                for (let i = 0; i < campanha.ligacoes_ativas.length; i++) {
                    if (ligacao.ActionID && campanha.ligacoes_ativas[i].ActionID == ligacao.ActionID) {
                        // log.info('REMOVELIGACAOATIVA: Ligacao encontrada por ActionID na posição ' + i + ', removendo.');
                        campanha.ligacoes_ativas.splice(i, 1);
                        Depurador.registrarLigacaoCompletada(campanha, ligacao, "Removido por ActionId");
                    } else if (ligacao.NumeroID && campanha.ligacoes_ativas[i].NumeroID == ligacao.NumeroID) {
                        // log.info('REMOVELIGACAOATIVA: Ligacao encontrada por NumeroID na posição ' + i + ', removendo.');
                        campanha.ligacoes_ativas.splice(i, 1);
                        Depurador.registrarLigacaoCompletada(campanha, ligacao, "Removido por NumeroId");
                    } else if (ligacao.ContatoID && campanha.ligacoes_ativas[i].ContatoID == ligacao.ContatoID) {
                        // log.info('REMOVELIGACAOATIVA: Ligacao encontrada por ContatoID na posição ' + i + ', removendo.');
                        campanha.ligacoes_ativas.splice(i, 1);
                        Depurador.registrarLigacaoCompletada(campanha, ligacao, "Removido por ContatoId");
                    } else if (campanha.ligacoes_ativas[i].fim) {
                        // log.info('REMOVELIGACAOATIVA_FIM: Ligacao para ' + campanha.ligacoes_ativas[i].Destino + ' FINALIZADA e ainda na lista, removendo.');
                        campanha.ligacoes_ativas.splice(i, 1);
                        Depurador.registrarLigacaoCompletada(campanha, ligacao, "Removido por 'fim'");
                    }
                }

                // Atualiza as ligações ativas no depurador
                Depurador.atualizarCampanha(campanha.id, {
                    ligacoes_ativas: campanha.ligacoes_ativas
                });
            }
        }

        // log.info('REMOVELIGACAOATIVA_FALHA: A Ligacao para ' + ligacao.Destino + ' NÃO foi encontrada nas ligações ativas.');
    }

    function getLigacoesVinculadas(campanha, VinculoID) {
        if (!VinculoID || VinculoID === 0) return [];

        return campanha.ligacoes_ativas.filter(lig => lig.Vinculo == VinculoID);
    }

    exports.ProcessaRetornoOriginate = (originate) => {
        log.info('PROCESSARETORNOORIGINATE: Processando retorno do Originate, Canal = ' + originate.channel);
        const ReasonMap = new Map();
        ReasonMap.set('0', 'Sem rota para o destino');
        ReasonMap.set('1', 'Nao atende');
        ReasonMap.set('2', 'Local nao atende');
        ReasonMap.set('3', 'Chamou ate cair');
        ReasonMap.set('4', 'Atendeu e caiu');
        ReasonMap.set('5', 'Ocupado');
        ReasonMap.set('8', 'Congestionado ou nao disponivel');
        ReasonMap.set('52', 'Desconexão ligação simultânea');

        for (let i = 0; i < campanhas.length; ++i) {
            var Ligacao = getLigacaoAtivaByActionID(campanhas[i], originate.actionid);
            if (Ligacao) {
                if (originate.response === 'Failure') {
                    Ligacao.Falha = true;
                    Ligacao.Status = Ligacao.derrubada ? 52 : parseInt(originate.reason);
                    Ligacao.StatusDescricao = ReasonMap.get(originate.reason);
                    Ligacao.fim = moment.tz(options.config.servidor.timezone);
                    const espera = moment.duration(Ligacao.fim.diff(Ligacao.inicio));
                    Ligacao.espera = espera.get("hours").toString().padStart(2, '0') + ":" + espera.get("minutes").toString().padStart(2, '0') + ":" + espera.get("seconds").toString().padStart(2, '0');

                    // Registra LOG
                    RegistraLog(Ligacao);

                    // Remove ligação das ativas
                    RemoveLigacaoAtiva(Ligacao);

                    // Verifica se foi uma ligação derrubada não precisa reiniciar o processamento da campanha agora.
                    if (Ligacao.derrubada) {
                        log.info('PROCESSARETORNOORIGINATE: DERRUBADA: Ligação para ' + Ligacao.Destino + ' foi derrubada vamos reabri-la');
                        // Reabre numero
                        DecTentativaIncRetentativa(Ligacao.NumeroID);
                    } else {
                        // Processa campanha novamente.
                        if (campanhas[i].tipo_campanha == 0)
                            discador.ProcessaCampanha(campanhas[i]);
                    }

                    try {
                        log.info("PROCESSARETORNOORIGINATE: A ligação para " + Ligacao.Destino + " falhou. Causa: " + Ligacao.StatusDescricao);
                        if (EMITIR_EVENTOS) {
                            ws.broadcast({ event: 'Discador', acao: 'FimLigacao', Falha: true, Ligacao: Ligacao });
                        }
                    } catch (ex) {
                        log.error("PROCESSARETORNOORIGINATE: Falha notificar evento Discador, acao: FimLigacao, ERRO: " + ex.toString());
                    }
                } else {
                    Ligacao.Falha = false;
                    Ligacao.Status = 9;
                    Ligacao.StatusDescricao = 'Ativa';
                    Ligacao.Linkedid = originate.uniqueid;
                    Ligacao.atendida = moment.tz(options.config.servidor.timezone);
                    log.info("PROCESSARETORNOORIGINATE: A ligação para " + Ligacao.Destino + " completou e está sendo transferida para seu destino.");

                    // Registra completada
                    RegistraCompletada(Ligacao);

                    // Atualiza ligacao ativa.
                    AddOrUpdateLigacaoAtiva(campanhas[i], Ligacao);

                    // Registra LOG? Registrar ao termino da ligação, pois aqui é o retorno do Originate, não significa que a ligação completou de fato, pode chamar até cair...
                    //RegistraLog(Ligacao);

                    // Remove ligação das ativas? Nao posso remover agora, pois a ligação esta em curso
                    //var n = RemoveLigacaoAtiva(campanhas[i], Ligacao);

                    // Processamento de vinculadas, esse processo pode estar falho aqui, deveria processar se a ligação realmente completou..
                    var Vinculadas = getLigacoesVinculadas(campanhas[i], Ligacao.Vinculo);

                    if (Vinculadas) {
                        for (let v = 0; v < Vinculadas.length; ++v) {
                            var vinculada = Vinculadas[v];
                            // Nao derrubar a ligação atual!!!!
                            if (vinculada.ActionID != Ligacao.ActionID) {
                                log.info("PROCESSARETORNOORIGINATE: DERRUBAR: Ligação para " + vinculada.Destino + ' vinculada a ' + Ligacao.Destino + ', será derrubada.');
                                vinculada.derrubada = true;

                                // Atualiza ligacao
                                AddOrUpdateLigacaoAtiva(campanhas[i], vinculada);

                                // Pedi pra derrubar...
                                try {
                                    const ac = new namiLib.Actions.Hangup();
                                    ac.channel = '/^' + vinculada.ChannelId + '-.*$/';
                                    ac.cause = 52;
                                    ac.ActionID = uuid();
                                    Ami.nami.send(ac);
                                } catch (ex) {
                                    log.error("PROCESSARETORNOORIGINATE: DERRUBAR: Falha ao criar Hangup: " + ex.toString());
                                }
                            }
                        }
                    }

                    // Notificacao
                    if (EMITIR_EVENTOS) {
                        try {
                            ws.broadcast({ event: 'Discador', acao: 'LigacaoAtiva', Falha: false, Ligacao: Ligacao });
                        } catch (ex) {
                            log.error("PROCESSARETORNOORIGINATE: Falha notificar evento Discador, acao: LigacaoAtiva, ERRO: " + ex.toString());
                        }
                    }
                }
            }
        }
    };

    exports.ProcessaHangup = (hangup) => {
        log.info('PROCESSAHANGUP: Ligacao Callid = ' + hangup.Linkedid + ', finalizou com causa = ' + hangup.Cause);
        for (let i = 0; i < campanhas.length; ++i) {
            var Ligacao = getLigacaoByLinkedid(campanhas[i], hangup.Linkedid);
            if (Ligacao) {
                Ligacao.Falha = Ligacao.derrubada ? true : false;
                Ligacao.Status = Ligacao.derrubada ? 52 : hangup.Cause;
                Ligacao.StatusDescricao = Ligacao.derrubada ? 'Derrubada' : 'Fim';
                Ligacao.fim = moment.tz(options.config.servidor.timezone);

                if (Ligacao.atendida) {
                    const duration = moment.duration(Ligacao.fim.diff(Ligacao.atendida));
                    const espera = moment.duration(Ligacao.atendida.diff(Ligacao.inicio));
                    Ligacao.espera = espera.get("hours").toString().padStart(2, '0') + ":" + espera.get("minutes").toString().padStart(2, '0') + ":" + espera.get("seconds").toString().padStart(2, '0');
                    Ligacao.falando = duration.get("hours").toString().padStart(2, '0') + ":" + duration.get("minutes").toString().padStart(2, '0') + ":" + duration.get("seconds").toString().padStart(2, '0');

                    // Verifica se tera que reabrir numero pelo tempo minimo
                    if (campanhas[i].tempo_minimo_completar) {
                        log.info('PROCESSAHANGUP: A capanha possui duração minima de chamada = ' + campanhas[i].tempo_minimo_completar);
                        log.info('PROCESSAHANGUP: A ligação teve uma duração de: ' + Ligacao.falando);

                        if (duration <= (campanhas[i].tempo_minimo_completar * 1000)) {
                            log.info('PROCESSAHANGUP: A ligação durou menos que o mínimo, reabrindo ela.');
                            log.info('PROCESSAHANGUP: Duration = ' + duration + ', Ligação = ' + campanhas[i].tempo_minimo_completar * 1000);
                            // Reabre Contato
                            ReabreContato(Ligacao);
                        }
                    }
                } else {
                    const espera = moment.duration(Ligacao.fim.diff(Ligacao.inicio));
                    Ligacao.espera = espera.get("hours").toString().padStart(2, '0') + ":" + espera.get("minutes").toString().padStart(2, '0') + ":" + espera.get("seconds").toString().padStart(2, '0');
                }

                // Registra LOG
                RegistraLog(Ligacao);

                // Verifica se foi uma ligação derrubada
                if (Ligacao.derrubada) {
                    log.info('PROCESSAHANGUP: DERRUBADA: Ligação para ' + Ligacao.Destino + ' foi derrubada vamos reabri-la');
                    // Reabre Contato
                    ReabreContato(Ligacao);
                }

                // Remove da lista
                RemoveLigacaoAtiva(Ligacao);

                // Notifica
                if (EMITIR_EVENTOS) {
                    try {
                        ws.broadcast({ event: 'Discador', acao: 'FimLigacao', Ligacao: Ligacao });
                    } catch (ex) {
                        log.error("PROCESSAHANGUP: Falha notificar evento Discador, acao: FimLigacao, ERRO: " + ex.toString());
                    }
                }

                // Processa campanha novamente.
                if (campanhas[i].tipo_campanha == 0)
                    discador.ProcessaCampanha(campanhas[i]);

            }
        }
    };

    exports.ProcessaAbandono = (evento) => {
        log.info('PROCESSAABANDONO: Processando Abandono');

        for (let i = 0; i < campanhas.length; ++i) {
            var Ligacao = getLigacaoByLinkedid(campanhas[i], evento.linkedid);
            if (Ligacao) {
                Ligacao.Falha = true;
                Ligacao.Status = 995;
                Ligacao.StatusDescricao = 'Abandono';
                Ligacao.fim = moment.tz(options.config.servidor.timezone);
                const espera = moment.duration(Ligacao.fim.diff(Ligacao.inicio));
                Ligacao.espera = espera.get("hours").toString().padStart(2, '0') + ":" + espera.get("minutes").toString().padStart(2, '0') + ":" + espera.get("seconds").toString().padStart(2, '0');

                // Registra LOG
                RegistraLog(Ligacao);
                // Reabre Contato
                ReabreContato(Ligacao);
                // Remove da lista
                RemoveLigacaoAtiva(Ligacao);

                // Notifica
                if (EMITIR_EVENTOS) {
                    try {
                        ws.broadcast({ event: 'Discador', acao: 'FimLigacao', Falha: true, Ligacao: Ligacao });
                    } catch (ex) {
                        log.error("PROCESSAABANDONO: Falha notificar evento Discador, acao: FimLigacao, ERRO: " + ex.toString());
                    }
                }

                // Processa campanha novamente.
                if (campanhas[i].tipo_campanha == 0)
                    discador.ProcessaCampanha(campanhas[i]);
            }
        }
    };

    exports.ProcessaChannelUP = (evento) => {
        // Preciso analizar se é melhor derrubar as ligações vinculadas no retorno do originate ou quando a ligação realmente completa..
        // Se for derrubar quando completa preciso fazer aqui e chamar isso no NewStat do AMI
        log.info('PROCESSACHANNELUP: Processamento de channelUP');
    };

    function RegistraLog(ligacao) {
        const duracao = ligacao.falando ? ligacao.falando : '00:00:00';
        db.query("INSERT INTO campanhas_registro(id_campanha,id_contato,id_numero,numero,callid,actionid,falha,id_status,campos,duracao) VALUES($1,$2,$3,$4,$5,$6,$7,$8,$9,$10)", [ligacao.CampanhaID, ligacao.ContatoID, ligacao.NumeroID, ligacao.Destino, ligacao.Linkedid, ligacao.ActionID, ligacao.Falha, ligacao.Status, ligacao.Campos, duracao]).then(res => {
            log.info("REGISTRALOG: Log da campanha ID = " + ligacao.CampanhaID + " registrado.");
        }).catch(e => {
            log.error('REGISTRALOG: Falha ao registrar o log para camapanhaID = ' + ligacao.CampanhaID);
            log.error(e);
        });
    };

    function RegistraCompletada(ligacao) {
        // Fecha Numero
        db.query("UPDATE campanhas_contato_numeros SET sucesso = true WHERE id_contato = $1", [ligacao.ContatoID]).then(res => {
            log.info('REGISTRACOMPLETADA: Numero Finalizado para contatoID = ' + ligacao.ContatoID);
        }).catch(e => {
            log.error('REGISTRACOMPLETADA: Falha ao registrar chamada completada.');
            log.error(e);
        });

        // Fecha contato
        db.query("UPDATE campanhas_contato SET sucesso = true WHERE id = $1", [ligacao.ContatoID]).then(res => {
            log.info("REGISTRACOMPLETADA: Contato ID = " + ligacao.ContatoID + " Finalizado.");
        }).catch(e => {
            log.error('REGISTRACOMPLETADA: Falha ao registrar contato finalizado.');
            log.error(e);
        });
    };

    function ReabreContato(ligacao) {
        // Reabre Numero
        // 28-05-2018 Alterado para reabrir todos os numeros do contato pois a rotina de completar fecha todos e sem isso o discador não disca os demais números do contato.
        db.query("UPDATE campanhas_contato_numeros SET sucesso = false WHERE id_contato = $1", [ligacao.ContatoID]).then(res => {
            log.info('REABRECONTATO: Numeros reabertos para contato ID = ' + ligacao.ContatoID);
        }).catch(e => {
            log.error('REABRECONTATO: Falha ao reabrir numero.');
            log.error(e);
        });

        // Reabre contato
        db.query("UPDATE campanhas_contato SET sucesso = false WHERE id = $1", [ligacao.ContatoID]).then(res => {
            log.info('REABRECONTATO: Contato Reaberto id = ' + ligacao.ContatoID);
        }).catch(e => {
            log.error('REABRECONTATO: Falha ao reabrir contato.');
            log.error(e);
        });
    };

    function DecTentativaIncRetentativa(id) {
        db.query("UPDATE campanhas_contato_numeros SET tentativas = tentativas - 1, retentativas = retentativas + 1 WHERE id = $1", [id]).then(res => {
            log.info('DEC_TENTATIVA_INC_RETENTATIVA: Numero ID = ' + id + ' teve tentatativas decrementada e retentativas aumentada.');
        }).catch(e => {
            log.error('DEC_TENTATIVA_INC_RETENTATIVA: Falha ao incrementar retentativas.');
            log.error(e);
        });
    }

    function FechaCampanha(campanha) {
        const fim = moment.tz(options.config.servidor.timezone).format('YYYY-MM-DD HH:mm:ss');

        if (EMITIR_EVENTOS) {
            ws.broadcast({ event: 'CampanhaFim', campanha: campanha });
        }

        // Atualiza todos os campos distintos
        Depurador.atualizarCampanha(campanha.id, campanha);

        if (campanha.notificar_email && campanha.notificar_email.length > 0) {
            let transporter = nodemailer.createTransport({
                host: options.config.email.host,
                port: options.config.email.port,
                secure: options.config.email.secure,
                auth: {
                    user: options.config.email.user,
                    pass: options.config.email.pass
                }
            });

            let mailOptions = {
                from: '"' + options.config.email.from_name + '" <' + options.config.email.from_email + '>',
                to: campanha.notificar_email,
                subject: 'Xcontact - A campanha ' + campanha.nome + ' foi finalizada',
                text: 'A campanha ' + campanha.nome + ' foi finalizada.',
            };

            transporter.sendMail(mailOptions, (error, info) => {
                if (error) {
                    log.error('NOTIFICA_EMAIL: Falha ao enviar email.');
                    return log.error(error);
                }
                log.info('Message sent: ' + info.messageId);
            });
        }

        db.query("UPDATE campanhas set status = 'F',data_status = $2,descricao = 'Campanha finalizada pelo discador.' WHERE id = $1", [campanha.id, fim]).then(res => {
            log.info('FECHACAMPANHA: Campanha finalizada ID = ' + campanha.id);
        }).catch(e => {
            log.error('FECHACAMPANHA: Falha finalizar campanha.');
            log.error(e);
        });
    };

    function RemoveNumero(id_numero) {
        // Remove Numero
        db.query("DELETE FROM campanhas_contato_numeros WHERE id = $1", [id_numero]).then(res => {
            log.info('REMOVENUMERO: Numero de id ' + id_numero + ' removido.');
        }).catch(e => {
            log.error('REMOVENUMERO: Falha ao remover numero.');
            log.error(e);
        });
    };

    function RemoveContato(id_contato) {
        // Remove Contato
        db.query("DELETE FROM campanhas_contato WHERE id = $1", [id_contato]).then(res => {
            log.info('REMOVECONTATO: Contato de id ' + id_contato + ' removido.');
        }).catch(e => {
            log.error('REMOVECONTATO: Falha ao remover contato.');
            log.error(e);
        });
    };

    function cancelaClaTimers(ob) {
        for (let i = 0; i < ClaTimers.length; ++i) {
            if (ClaTimers[i].ob.id === ob.id) {
                log.info('CANCELACLATIMERS: TimerID = ' + ob.id);
                clearInterval(ClaTimers[i].timer);
                ClaTimers.splice(i, 1);
                return;
            }
        }
    };

    exports.ProcessaClassificacao = (callid, classificacao) => {
        var element = { id: new Date().getTime(), tentativas: 1, callid: callid, classificacao: classificacao };
        log.info('PROCESSACLASSIFICACAO: Add agenda ID = ' + element.id);
        var job = {
            ob: element,
            timer: setInterval(function (Obj) {
                // Busca os dados da ligação no log pelo callid
                db.query("SELECT id_campanha,id_numero,id_contato FROM campanhas_registro WHERE callid = $1 OR actionid = $1 LIMIT 1", [Obj.callid]).then(res => {
                    if (res.rows.length > 0) {
                        log.info('PROCESSACLASSIFICACAO: Callid ' + Obj.callid + ' encontrado cancelando agendamento de classificação e seguindo com o processo.');
                        cancelaClaTimers(Obj);
                        // Busca a ação da classificação
                        db.query("SELECT acao FROM campanhas_classificacoes WHERE id_campanha = $1 AND id_classificacao = $2 LIMIT 1", [res.rows[0].id_campanha, Obj.classificacao]).then(res2 => {
                            if (res2.rows.length > 0) {
                                log.info('PROCESSACLASSIFICACAO: Ação ' + res2.rows[0].acao + ' encontrada para o callid ' + Obj.callid + ' e classificação ' + Obj.classificacao);
                                switch (res2.rows[0].acao) {
                                    case 'AN':
                                        const ligacao = { NumeroID: res.rows[0].id_numero, ContatoID: res.rows[0].id_contato };
                                        ReabreContato(ligacao);
                                        break;
                                    case 'RN':
                                        RemoveNumero(res.rows[0].id_numero);
                                        break;
                                    case 'RC':
                                        RemoveContato(res.rows[0].id_contato);
                                        break;
                                }
                            }
                            else {
                                log.info('PROCESSACLASSIFICACAO: Nenhuma ação encontrada para campanha ' + res.rows[0].id_campanha + ' e classificação ' + Obj.classificacao);
                            }
                        }).catch(e => {
                            log.error('PROCESSACLASSIFICACAO: Falha ao processar a classificação: ');
                            log.error(e);
                        });
                    }
                    else {
                        log.info('PROCESSACLASSIFICACAO: Callid ' + callid + ' não encontrado nos registros da campanha, pode não ser uma ligação realizada pelo discador. Vou tentar novamente daqui 30s');
                    }
                }).catch(e => {
                    log.error('PROCESSACLASSIFICACAO: Falha ao obter dados da tabela campanhas_registro:');
                    log.error(e);
                });

                ++Obj.tentativas;
                if (Obj.tentativas > 5) {
                    log.info('PROCESSACLASSIFICACAO: Callid ' + callid + ' não encontrado depois de 5 tentativas nos registros da campanha, não deve ser uma ligação realizada pelo discador. Cancelando procura.');
                    cancelaClaTimers(Obj);
                }
            }, 30000, element)
        }
        ClaTimers.push(job);
    };

    exports.addCampanhaByID = addCampanhaByID;
    exports.ProcessaCampanha = ProcessaCampanha;

    discador.reloadCampanhas();
    discador.AdicionaCampanhasAgendadas();

    setInterval(() => {
        console.log('DISCADOR: Executando rotina de adicionar campanhas agendadas a cada 1h');
        discador.AdicionaCampanhasAgendadas();
    }, 3600000)

    // Executando uma checagem nas campanhas a cada 30s
    setInterval(() => {
        log.info('DISCADOR: Executando rotina a cada 30s');
        discador.ProcessaCampanhasNew();
        // Tenta remover ligações presas...
        var Ligacao = { ActionID: 'uidRotinaDeLimpeza', Destino: 'RotinaDeLimpeza' };
        RemoveLigacaoAtiva(Ligacao);
    }, 30000);

    next();
};

exports.register.attributes = {
    name: 'xcontact-discador'
};