'use strict'
exports.register = (server, options, next) => {
	const db = require('../util/db');
	const Campanha = require('./Campanha');
	const ws = require('../ws');
	const namiLib = require("nami");
	const Ami = require('../ami');
	const uuid = require('uuid/v1');
	const opts = {
		logFilePath:'/var/log/xcontact.log',
		timestampFormat:'DD-MM-YYYY HH:mm:ss.SSS'
	};
  const log = require('simple-node-logger').createSimpleLogger( opts );
	const log_level = options.config.log ? options.config.log : 'error';
	log.setLevel(log_level);
	log.debug('DISCADORSTART: LOG_LEVEL = '+log_level);
	const ClaTimers = [];
	const Vinculadas = [];

	const {SequentialTaskQueue} = require('sequential-task-queue');
  var queue = new SequentialTaskQueue();
  const nodemailer = require('nodemailer');


    log.info('DISCADORSTART: Discador iniciado.');
    const moment = require('moment-timezone');
    moment.tz.setDefault(options.config.servidor.timezone);
    let campanhas = [];
    const discador = this;

    exports.reloadCampanhas = () => {
      log.info('RELOADCAMPANHAS: Recarregando ativas campanhas do banco');
      db.query("SELECT * FROM public.campanhas WHERE status = 'A' ORDER by id")
      .then(res => {
          campanhas = [];
          for(let i=0; i < res.rows.length; ++i){
            const c = new Campanha();
            const row = res.rows[i];
            for(const co in row) {
              c[co] = row[co];
            }
            campanhas.push(c);
          }
          log.info('RELOADCAMPANHAS: Campanhas recarregadas com sucesso.');
          ws.broadcast({event:'campanhas_reload',sucess: true,msg:'Campanhas recarregadas com sucesso.'});
      })
      .catch(e => {
          log.error(e);
          ws.broadcast({event:'campanhas_reload',sucess: false,msg:e});
      });
    };

    const addCampanhaByID = (id) => {
      log.info('ADDCAMPANHABYID: Adicionando campanha id = '+id);
      const knex = db.knex('public.campanhas');
      knex.where('id', id);
      knex.select('*')
      .then(function(rows) {
          campanhas.push(rows[0]);
          discador.ProcessaCampanha(rows[0]);
          log.info('ADDCAMPANHABYID: Campanha id = '+id+' adicionada com sucesso.');
          ws.broadcast({event:'campanha_add',sucess: true,msg:'Nova campanha adicionada.',campanha: rows[0]});
      })
      .catch(e => {
          log.error(e);
          ws.broadcast({event:'campanha_add',sucess: false,msg:e});
      });
    };

    exports.reloadCampanha = (id) => {
      log.info('RELOADCAMPANHA: Recarregando campanha id = '+id);
      const knex = db.knex('public.campanhas');
      let campanha = {};
      knex.where('id', id);
      knex.select('*')
      .then(function(rows) {
        let tem = false;
          for(let i=0; i < campanhas.length; ++i){
            if(campanhas[i].id === rows[0].id){
              campanhas[i] =  rows[0];
              campanha = campanhas[i];
              discador.ProcessaCampanha(campanha);
              tem = true;
            }
          }
          if(!tem){
            log.info('RELOADCAMPANHA: Essa campanha não tá na lista, vou chamar add');
            addCampanhaByID(id);
          }
          else {
            log.info('RELOADCAMPANHA: Campanha id = '+id+' recarregada com sucesso.');
            ws.broadcast({event:'campanha_reload',sucess: true,msg:'Campanhas id = '+id+' recarregadas com sucesso.',campanha: campanha});
          }
      })
      .catch(e => {
          log.error(e);
          ws.broadcast({event:'campanha_reload',sucess: false,msg:e});
      });
    };

    exports.addCampanha = (campanha) => {
      log.info('ADDCAMPANHA: Adicionando nova campanha');
      campanhas.push(campanha);
      discador.ProcessaCampanha(campanha);
      ws.broadcast({event:'campanha_add',sucess: true,msg:'Nova campanha adicionada.',campanha: campanha});
    };

    exports.addCampanhaByID = addCampanhaByID;

    exports.delCampanha = (id) => {
      log.info('DELCAMPANHA: Removendo campanha de id = '+id);
      for(let i=0; i < campanhas.length; ++i){
        if(campanhas[i].id === id){
          campanhas.splice(i, 1);
        }
      }
      ws.broadcast({event:'campanha_del',sucess: true,msg:'Campanha apagada.',campanhaID: id});
    };

    discador.reloadCampanhas();

    exports.getCampanhas = () => {
      return campanhas;
    };

    exports.getCampanhaByID = (id) => {
      for(let i=0; i < campanhas.length; ++i){
        if(campanhas[i].id === id){
          return campanhas[i];
        }
      }
    };

    exports.ProcessaCampanhas = () => {
      for(let i=0; i < campanhas.length; ++i){
          discador.ProcessaCampanha(campanhas[i]);
      }
    };

    exports.ProcessaCampanhasByFila = (fila) => {
      for(let i=0; i < campanhas.length; ++i){
        if(campanhas[i].fila === fila){
          discador.ProcessaCampanha(campanhas[i]);
        }
      }
    };

    function getQtdLigacoesDiscando(campanha,callback){
      let total = 0;
      try {
        for(let i=0; i < campanha.ligacoes_ativas.length; ++i){
          if(campanha.ligacoes_ativas[i].Status === 10){
            total++;
          }
        }
      } catch (ex){
        log.warn('GETQTDLIGACOESDISCANDO: Erro ao percorrer as ligações ativas da campanha '+campanha.nome+', possivelmente não tem ligações ativas! :)');
      }
      if(callback) callback(total);
      else return total;
    }

		// Processo extenso por causa da Task Queue
    exports.ProcessaCampanha = (campanha) => {
      log.info('PROCESSACAMPANHA: Processando campanha '+campanha.nome);
      //console.log(queue);
      if(campanha.ligacoes_ativas && campanha.ligacoes_ativas.length >= campanha.max_simultaneas_campanha){
        log.info('PROCESSACAMPANHA: A campanha '+campanha.nome+' já tem '+campanha.ligacoes_ativas.length+' ligações ativas. O máximo para essa campanha é '+campanha.max_simultaneas_campanha);
        return;
      }
      // Processa somente ativas
      if(campanha.status === 'A'){
				queue.push(() => {
				    return new Promise(done => {
			        Ami.getAgentesOnline(campanha.fila,(agentes) => {
			          if(agentes.length > 0){
									log.info('PROCESSACAMPANHA: A campanha '+campanha.nome+' associada a fila '+campanha.fila+' possui '+agentes.length+' agentes online.');
			            getQtdLigacoesDiscando(campanha,(discando) => {
			              let qtd = agentes.length - discando;
			              if(qtd > 0){
			                log.info('PROCESSACAMPANHA: A campanha '+campanha.nome+' com simultâneas por agente = '+campanha.max_por_agente+' possui '+discando+' ligações discando, vou tentar iniciar '+qtd+' ligação(es)');
											var vinculo = 0;
											var simultaneas = qtd;
											if(campanha.max_por_agente > 1){
												if(campanha.desconecta_multiplas){
													vinculo = new Date().getTime();
													simultaneas = qtd * campanha.max_por_agente;
													log.info('GETNEXNUMERO: Campanha com '+campanha.max_por_agente+' ligações por agente, e com opção de desconectar multiplas ativa, criando vinculo de ligacoes = ' + vinculo);
												}
												else {
													simultaneas = qtd * campanha.max_por_agente;
													log.info('GETNEXNUMERO: Campanha com '+campanha.max_por_agente+' ligações por agente');
												}
											}                      
                      if(campanha.ligacoes_ativas && campanha.ligacoes_ativas.length > 0) {
                        log.info('GETNEXNUMERO: Temos ligações ativas na campanha, verificando se já não está discando para algum número');
                        db.query("SELECT id FROM public.campanhas_contato_numeros WHERE id_campanha = $1 AND sucesso = false AND tentativas < $2 AND retentativas < $2 LIMIT $3",[campanha.id,campanha.tentativas,simultaneas]).then(res => {
                          if(res.rows.length > 0){
                            log.info('GETNEXNUMERO: Teriamos '+res.rows.length+' novo(s) número(s) para discar, verificando se já não está discando...');
                            var discando = false;
                            for(let n=0;n<res.rows.length;++n){
                              for(let la=0;la<campanha.ligacoes_ativas.length;++la){
                                if(res.rows[n].id == campanha.ligacoes_ativas[la].NumeroID){
                                  log.warn('GETNEXNUMERO: Já estamos ligando para '+campanha.ligacoes_ativas[la].Destino+', ABORTANDO.');
                                  discando = true;
                                  break;
                                }
                              }
                            }
                            if(!discando){
                              // Busca mais numeros...
                              BuscaMaisNumeros(campanha,agentes,simultaneas,vinculo,done);
                            }
                            else {
                              done();                              
                            }                          
                          }
                          else {
                            log.info("GETNEXNUMERO: Fim, nada para discar");
                            done();                   
                          }
                        });
                      }
                      else {
                        // Busca mais numeros
                        BuscaMaisNumeros(campanha,agentes,simultaneas,vinculo,done);
                      }                      
			              }
			              else {
			                log.info('PROCESSACAMPANHA: A campanha '+campanha.nome+' possui '+discando+' ligações discando e '+agentes.length+' agentes online, não vou iniciar mais ligações agora.');
											done();
			              }
			            });
			          }
								else {
									log.info('PROCESSACAMPANHA: A campanha '+campanha.nome+' associada a fila '+campanha.fila+' não possui agentes online, nada para fazer agora.');
									done();
								}
			        });
					});
				});
      }
      else {
        log.info('PROCESSACAMPANHA: A campanha '+campanha.nome+' não está ativa, status = '+campanha.status);        
      }
    };

    function BuscaMaisNumeros(campanha,agentes,simultaneas,vinculo,done){
      log.info('GETNEXNUMERO: Buscando '+simultaneas+' novo(s) número(s) para discar');
      db.query("UPDATE public.campanhas_contato_numeros set tentativas = tentativas + 1 WHERE id_campanha = $1 AND id IN (SELECT id FROM public.campanhas_contato_numeros WHERE id_campanha = $1 AND sucesso = false AND tentativas < $2 AND retentativas < $2 order by fone_sequencia,tentativas,retentativas,id_contato LIMIT $3) returning *",[campanha.id,campanha.tentativas,simultaneas]).then(res => {
        log.info('GETNEXNUMERO: Retornou '+res.rows.length+' número(s) para discar');
        if(res.rows.length > 0){
          for(let n=0;n<res.rows.length;++n){
            const numero = res.rows[n];
            db.query("SELECT * from public.campanhas_contato_campos WHERE id_contato = $1",[numero.id_contato]).then(res2 => {
              const contato = {};
              if(vinculo) {
                contato.vinculo = vinculo;
                contato.vinculo_qtd = qtd;
                contato.vinculo_total = simultaneas;
              }
              contato.numero = {};
              contato.campos = {};
              contato.id = numero.id_contato;
              contato.id_numero = numero.id;
              for(const co in numero) {
                contato.numero[co] = numero[co];
              }

              if(res2.rows.length > 0){
                //console.log(res2.rows);
                for(let i=0; i < res2.rows.length; ++i){
                  contato.campos[res2.rows[i].nome] = res2.rows[i].valor;
                }
              }
              // Verifica se é Power ou Preview
              if(campanha.modo === 'A'){
                DiscaPower(campanha,contato);
                log.info("GETNEXNUMERO: Fim, enviado para Discagem Power");
                done();
              }
              else {
                Disca(campanha,contato,agentes,done);
                log.info("GETNEXNUMERO: Fim, enviado para Discagem preview");
                //done();
              }
            }).catch(e => {
                log.info("GETNEXNUMERO: Fim com erro");
                log.error(e);
                done();
            });
            log.info('GETNEXNUMERO: Contato carregado.');
          }
        }
        else {
            log.info("GETNEXNUMERO: Fim, nada para discar");
            if(campanha.autofim){
              log.info("GETNEXNUMERO: Campanha com AUTOFIM ativo, fechando campanha "+campanha.nome);
              campanha.status = 'F';
              campanha.data_status = moment.tz(options.config.servidor.timezone).format('YYYY-MM-DD HH:mm:ss');
              FechaCampanha(campanha);
            }
            done();
        }
      }).catch(e => {
          //console.log("GETNEXNUMERO: Fim com erro");
          log.error(e);
          done();
      });
    }

    function Disca(campanha,contato,agentes22,done){
      log.info("DISCA: Inicio da Discagem preview");
      //console.log(Ami.getAgentes());

      Ami.getAgentesOnline(campanha.fila,(agentes) => {
        if(agentes.length > 0){
          const sorteado = agentes[Math.floor(Math.random() * agentes.length)];
          sorteado.status = 6;

          log.info("DISCA: Agente sorteado para discar: Nome: "+sorteado.nome+", Ramal: "+sorteado.ramal);
          log.info("DISCA: Discagem preview para Numero: " + contato.numero.numero +", ContatoID: "+ contato.numero.id_contato +", Fila: "+ campanha.fila +", Tentativa: "+ contato.numero.tentativas);
          try {
            const ac = new namiLib.Actions.Originate();
            //ac.channel  = sorteado.ramal;
            //ac.exten    = contato.numero.numero;
            const spt = sorteado.ramal.split('/');
            const tipo = spt[0];
            const ramal = spt[1];

            ac.channel  = 'Local/s@discadorPreview';
            ac.exten    = contato.numero.numero;
            ac.callerid = "Discador <"+contato.numero.numero+">";
            ac.account  = "Discador";
            ac.priority = 1;
            ac.context  = campanha.rota;
            ac.timeout  = campanha.max_chamar ? campanha.max_chamar * 1000 : 30 * 1000;
            ac.async    = true;
            ac.ActionID = uuid();

            let ar = [];
            ar.push('ActionID='+ac.ActionID);
            ar.push('TIPO='+tipo);
            ar.push('RAMAL='+ramal);
            ar.push('FILA='+campanha.fila);
            //ar.push('XCONTACT_REGISTRA_GRAVACAO()='+ac.ActionID+','+monitor_filename+'.mp3');
            ar.push('__SIPADDHEADER51=X-UNIQUE-ID:'+ac.ActionID);

            if(contato.campos && Object.keys(contato.campos).length > 0){
              let id = 2;
              Object.keys(contato.campos).forEach(function(key) {
                ar.push('__SIPADDHEADER5'+id+'=X-'+key+':'+encodeURIComponent(contato.campos[key]));
                ++id;
              });
            }
            ac.variable = ar.join(",");

            var Ligacao = {CampanhaID:campanha.id,ActionID:ac.ActionID,Status:10,StatusDescricao:'Iniciando',ContatoID:contato.id,NumeroID:contato.id_numero,Vinculo:contato.vinculo,VinculoQTD:contato.vinculo_qtd,VinculoTOTAL:contato.vinculo_total,Destino:contato.numero.numero,Campos:contato.campos};
            Ligacao.inicio = moment.tz(options.config.servidor.timezone);

            Ami.nami.send(ac, function(resposta) {
              //console.log(resposta);
              if(resposta.response === 'Error'){
                // Tenta notificar
                try {
                  Ligacao.Falha = true;
                  Ligacao.Status = 9;
                  Ligacao.StatusDescricao = resposta.message;
                  Ligacao.fim = moment.tz(options.config.servidor.timezone);
                  const espera = moment.duration(Ligacao.fim.diff(Ligacao.inicio));
                  Ligacao.espera = espera.get("hours").toString().padStart(2, '0') +":"+ espera.get("minutes").toString().padStart(2, '0') +":"+ espera.get("seconds").toString().padStart(2, '0');
                  log.info("PROCESSARETORNOORIGINATE: A ligação para "+Ligacao.Destino+" falhou. Causa: "+Ligacao.StatusDescricao);
                  //ws.broadcast({event:'Discador',acao:'FimLigacao',Falha:true,Ligacao:campanhas[i].ligacoes_ativas[j]});
                } catch (ex){
                    log.error("PROCESSARETORNOORIGINATE: Falha notificar evento Discador, acao: FimLigacao, ERRO: " + ex.toString());
                }
                // Registra LOG
                RegistraLog(Ligacao);
                // Finaliza tarefa
                done();
                // Processa campanha novamente.
                discador.ProcessaCampanha(campanha);
              }
              else {
                if(!campanha.ligacoes_ativas) campanha.ligacoes_ativas = [];
                campanha.ligacoes_ativas.push(Ligacao);
                // Finaliza tarefa
                done();
                //console.log('Depois do DONE...');
                try {
                  ws.broadcast({event:'Discador',acao:'NovaLigacao',Ligacao:Ligacao});
                } catch (ex){
                    log.error("DISCA: Falha notificar evento Discador, acao: NovaLigacao, ERRO: " + ex.toString());
                }
              }
            });
          } catch (ex){
              // Finaliza tarefa
              done();
              log.error("DISCA: Falha ao criar Originate: " + ex.toString());
          }
        }
        else {
          log.info("DISCA: PROBLEMA, Tinha agente online mas entre selecionar o numero e discar não tem mais...:)");          
        }
      });
    };

    function DiscaPower(campanha,contato){
      log.info("DISCAPOWER: Discagem power para Numero: " + contato.numero.numero +", ContatoID: "+ contato.numero.id_contato +", Fila: "+ campanha.fila +", Tentativa: "+ contato.numero.tentativas);
      try {
        const ac = new namiLib.Actions.Originate();
        ac.channel  = "Local/"+contato.numero.numero+"@"+campanha.rota;
        if(campanha.exec_contexto){
          // Manda pra rota
          ac.exten    = campanha.macro;			
          ac.priority = 1;
          ac.context  = campanha.exec_contexto;
        }        
        else {
          // Executa Macro
          ac.application = "Macro";
          ac.data = campanha.macro+","+campanha.fila;        
        }        
        ac.callerid = contato.campos.Nome ? contato.campos.Nome + " <"+contato.numero.numero+">" : contato.numero.numero;
        ac.timeout  = campanha.max_chamar ? campanha.max_chamar * 1000 : 30 * 1000;
        ac.async    = true;
        ac.ActionID = uuid();
				if(contato.campos && Object.keys(contato.campos).length > 0){
          let ar = [];
					let id = 1;
          Object.keys(contato.campos).forEach(function(key) {
            ar.push('__SIPADDHEADER5'+id+'=X-'+key+':'+encodeURIComponent(contato.campos[key]));
            ar.push(key+'='+encodeURIComponent(contato.campos[key]));
						++id;
          });
          ac.variable = ar.join(",");
        }
        var Ligacao = {CampanhaID:campanha.id,ActionID:ac.ActionID,ChannelId:ac.channel,Status:10,StatusDescricao:'Iniciando',ContatoID:contato.id,NumeroID:contato.id_numero,Vinculo:contato.vinculo,VinculoQTD:contato.vinculo_qtd,VinculoTOTAL:contato.vinculo_total,Destino:contato.numero.numero,Campos:contato.campos};
        Ligacao.inicio = moment.tz(options.config.servidor.timezone);

        if(!campanha.ligacoes_ativas) campanha.ligacoes_ativas = [];
        campanha.ligacoes_ativas.push(Ligacao);

        Ami.nami.send(ac);
      } catch (ex){
          log.error("DISCAPOWER: Falha ao criar Originate: " + ex.toString());
      }

      try {
        ws.broadcast({event:'Discador',acao:'NovaLigacao',Ligacao:Ligacao});
      } catch (ex){
          log.error("DISCAPOWER: Falha notificar evento Discador, acao: NovaLigacao, ERRO: " + ex.toString());
      }
    };

    exports.ProcessaRetornoOriginate = (originate) => {
      log.info('PROCESSARETORNOORIGINATE: Processando retorno do Originate, Canal = '+originate.channel);
			//console.log(originate);
      const ReasonMap = new Map();
      ReasonMap.set('0', 'Sem rota para o destino');
      ReasonMap.set('1', 'Nao atende');
      ReasonMap.set('2', 'Local nao atende');
      ReasonMap.set('3', 'Chamou ate cair');
      ReasonMap.set('4', 'Atendeu e caiu');
      ReasonMap.set('5', 'Ocupado');
      ReasonMap.set('8', 'Congestionado ou nao disponivel');
			ReasonMap.set('52', 'Desconexão ligação simultânea');

      for(let i=0; i < campanhas.length; ++i){
        if(campanhas[i].ligacoes_ativas) {
          for(let j=0; j < campanhas[i].ligacoes_ativas.length; ++j){
            if(campanhas[i].ligacoes_ativas[j].ActionID === originate.actionid){
              // Em caso de falha remove ligacao
              if(originate.response === 'Failure'){
                // Tenta notificar
                try {
                  campanhas[i].ligacoes_ativas[j].Falha = campanhas[i].ligacoes_ativas[j].derrubada ? true : false;;
                  campanhas[i].ligacoes_ativas[j].Status = campanhas[i].ligacoes_ativas[j].derrubada ? 52 : parseInt(originate.reason);
                  campanhas[i].ligacoes_ativas[j].StatusDescricao = ReasonMap.get(originate.reason);
                  campanhas[i].ligacoes_ativas[j].fim = moment.tz(options.config.servidor.timezone);
                  const espera = moment.duration(campanhas[i].ligacoes_ativas[j].fim.diff(campanhas[i].ligacoes_ativas[j].inicio));
                  campanhas[i].ligacoes_ativas[j].espera = espera.get("hours").toString().padStart(2, '0') +":"+ espera.get("minutes").toString().padStart(2, '0') +":"+ espera.get("seconds").toString().padStart(2, '0');
									log.info("PROCESSARETORNOORIGINATE: A ligação para "+campanhas[i].ligacoes_ativas[j].Destino+" falhou. Causa: "+campanhas[i].ligacoes_ativas[j].StatusDescricao);
                  ws.broadcast({event:'Discador',acao:'FimLigacao',Falha:true,Ligacao:campanhas[i].ligacoes_ativas[j]});
                } catch (ex){
                    log.error("PROCESSARETORNOORIGINATE: Falha notificar evento Discador, acao: FimLigacao, ERRO: " + ex.toString());
                }
								const Lig = campanhas[i].ligacoes_ativas[j];
                // Registra LOG
                RegistraLog(Lig);

								// Remove ligacao da fila
								campanhas[i].ligacoes_ativas.splice(j, 1);

								// Verifica se foi uma ligação derrubada não precisa reiniciar o processamento da campanha agora.
								if(Lig.derrubada){
									log.info('PROCESSARETORNOORIGINATE: DERRUBADA: Ligação ActionID = '+originate.actionid+' foi derrubada vamos reabri-la');
									// Reabre numero
									DecTentativaIncRetentativa(Lig.NumeroID);
								}
								else {
									// Processa campanha novamente.
	                discador.ProcessaCampanha(campanhas[i]);
								}
              }
              // Sucesso
              else {
                // Registra completada
                RegistraCompletada(campanhas[i].ligacoes_ativas[j]);
                // Notifica ligação ativa
                try {
                  campanhas[i].ligacoes_ativas[j].Falha = false;
                  campanhas[i].ligacoes_ativas[j].Status = 9;
                  campanhas[i].ligacoes_ativas[j].StatusDescricao = 'Ativa';
                  campanhas[i].ligacoes_ativas[j].Linkedid = originate.uniqueid;
                  campanhas[i].ligacoes_ativas[j].atendida = moment.tz(options.config.servidor.timezone);
									log.info("PROCESSARETORNOORIGINATE: A ligação "+ campanhas[i].ligacoes_ativas[j].ActionID +" para "+campanhas[i].ligacoes_ativas[j].Destino+" completou e está sendo transferida para seu destino.");

								  // Completou?? Entao vamos derrubar vinculadas antes de completar..
									for(let jd=0; jd < campanhas[i].ligacoes_ativas.length; ++jd){
										// Pega ligacoes vinculadas, menos a que esta completando agora
										if(campanhas[i].ligacoes_ativas[jd].Vinculo && campanhas[i].ligacoes_ativas[jd].ActionID !== campanhas[i].ligacoes_ativas[j].ActionID){
											// log
											log.info("PROCESSARETORNOORIGINATE: DERRUBAR: Ligação "+campanhas[i].ligacoes_ativas[jd].ActionID+' vinculada a '+campanhas[i].ligacoes_ativas[j].ActionID+' com canal = '+ campanhas[i].ligacoes_ativas[jd].ChannelId +', será derrubada.');
											// Marca como derrubada
											campanhas[i].ligacoes_ativas[jd].derrubada = true;
											try {
												const ac = new namiLib.Actions.Hangup();
												ac.channel  = '/^'+campanhas[i].ligacoes_ativas[jd].ChannelId+'-.*$/';
												ac.cause = 52;
												ac.ActionID = uuid();
												Ami.nami.send(ac);
											} catch (ex){
													log.error("PROCESSARETORNOORIGINATE: DERRUBAR: Falha ao criar Hangup: " + ex.toString());
											}
										}
									}

                  ws.broadcast({event:'Discador',acao:'LigacaoAtiva',Falha:false,Ligacao:campanhas[i].ligacoes_ativas[j]});
                } catch (ex){
                    log.error("PROCESSARETORNOORIGINATE: Falha notificar evento Discador, acao: LigacaoAtiva, ERRO: " + ex.toString());
                }
              }

              return;
            }
          }
        }
      }
    };

    exports.ProcessaHangup = (hangup) => {
      log.info('PROCESSAHANGUP: Processando Hangup');
			log.info('PROCESSAHANGUP: Ligacao Callid = '+ hangup.Linkedid +', finalizou com causa = '+ hangup.Cause);
      for(let i=0; i < campanhas.length; ++i){
        if(campanhas[i].ligacoes_ativas) {
          for(let j=0; j < campanhas[i].ligacoes_ativas.length; ++j){
            if(campanhas[i].ligacoes_ativas[j].Linkedid === hangup.Linkedid){
              // Tenta notificar
              try {
                campanhas[i].ligacoes_ativas[j].Falha = campanhas[i].ligacoes_ativas[j].derrubada ? true : false;
                campanhas[i].ligacoes_ativas[j].Status = campanhas[i].ligacoes_ativas[j].derrubada ? 52 : hangup.Cause;
                campanhas[i].ligacoes_ativas[j].StatusDescricao = campanhas[i].ligacoes_ativas[j].derrubada ? 'Derrubada' : 'Fim';
                campanhas[i].ligacoes_ativas[j].fim = moment.tz(options.config.servidor.timezone);
                if(campanhas[i].ligacoes_ativas[j].atendida){
                  const duration = moment.duration(campanhas[i].ligacoes_ativas[j].fim.diff(campanhas[i].ligacoes_ativas[j].atendida));
                  const espera = moment.duration(campanhas[i].ligacoes_ativas[j].atendida.diff(campanhas[i].ligacoes_ativas[j].inicio));
                  //ligacao.atendidaf = ligacao.atendida.format('DD-MM-YYYY HH:mm:ss');
                  campanhas[i].ligacoes_ativas[j].espera = espera.get("hours").toString().padStart(2, '0') +":"+ espera.get("minutes").toString().padStart(2, '0') +":"+ espera.get("seconds").toString().padStart(2, '0');
                  campanhas[i].ligacoes_ativas[j].falando = duration.get("hours").toString().padStart(2, '0') +":"+ duration.get("minutes").toString().padStart(2, '0') +":"+ duration.get("seconds").toString().padStart(2, '0');

									// Verifica se tera que reabrir numero pelo tempo minimo
                  if(campanhas[i].tempo_minimo_completar){
                    log.info('PROCESSAHANGUP: A capanha possui duração minima de chamada = ' + campanhas[i].tempo_minimo_completar);
                    log.info('PROCESSAHANGUP: A ligação teve uma duração de: '+ campanhas[i].ligacoes_ativas[j].falando);
                    if(duration <= (campanhas[i].tempo_minimo_completar * 1000)){
                      log.info('PROCESSAHANGUP: A ligação durou menos que o mínimo, reabrindo ela.');
                      log.info('PROCESSAHANGUP: Duration = '+ duration + ', Ligação = ' + campanhas[i].tempo_minimo_completar * 1000);
                      // Reabre Contato
                      ReabreContato(campanhas[i].ligacoes_ativas[j]);
                    }
                  }
                }
                else {
                  const espera = moment.duration(campanhas[i].ligacoes_ativas[j].fim.diff(campanhas[i].ligacoes_ativas[j].inicio));
                  campanhas[i].ligacoes_ativas[j].espera = espera.get("hours").toString().padStart(2, '0') +":"+ espera.get("minutes").toString().padStart(2, '0') +":"+ espera.get("seconds").toString().padStart(2, '0');
                }
                ws.broadcast({event:'Discador',acao:'FimLigacao',Ligacao:campanhas[i].ligacoes_ativas[j]});
              } catch (ex){
                  log.error("PROCESSAHANGUP: Falha notificar evento Discador, acao: FimLigacao, ERRO: " + ex.toString());
              }
              // Registra LOG
              RegistraLog(campanhas[i].ligacoes_ativas[j]);

							// Verifica se foi uma ligação derrubada
							if(campanhas[i].ligacoes_ativas[j].derrubada){
								log.info('PROCESSAHANGUP: DERRUBADA: Ligação callid = '+hangup.Linkedid+' foi derrubada vamos reabri-la');
								// Reabre Contato
								ReabreContato(campanhas[i].ligacoes_ativas[j]);
							}

              // Remove da lista
              campanhas[i].ligacoes_ativas.splice(j, 1);

              // Processa campanha novamente.
              discador.ProcessaCampanha(campanhas[i]);

              return;
            }
          }
        }
      }
    };

    exports.ProcessaAbandono = (evento) => {
      log.info('PROCESSAABANDONO: Processando Abandono');

      for(let i=0; i < campanhas.length; ++i){
        if(campanhas[i].ligacoes_ativas) {
          for(let j=0; j < campanhas[i].ligacoes_ativas.length; ++j){
            if(campanhas[i].ligacoes_ativas[j].Linkedid === evento.linkedid){
              // Tenta notificar
              try {
                campanhas[i].ligacoes_ativas[j].Falha = true;
                campanhas[i].ligacoes_ativas[j].Status = 995;
                campanhas[i].ligacoes_ativas[j].StatusDescricao = 'Abandono';
                campanhas[i].ligacoes_ativas[j].fim = moment.tz(options.config.servidor.timezone);
                const espera = moment.duration(campanhas[i].ligacoes_ativas[j].fim.diff(campanhas[i].ligacoes_ativas[j].inicio));
                campanhas[i].ligacoes_ativas[j].espera = espera.get("hours").toString().padStart(2, '0') +":"+ espera.get("minutes").toString().padStart(2, '0') +":"+ espera.get("seconds").toString().padStart(2, '0');
                ws.broadcast({event:'Discador',acao:'FimLigacao',Falha:true,Ligacao:campanhas[i].ligacoes_ativas[j]});
              } catch (ex){
                  log.error("PROCESSAABANDONO: Falha notificar evento Discador, acao: FimLigacao, ERRO: " + ex.toString());
              }
              // Registra LOG
              RegistraLog(campanhas[i].ligacoes_ativas[j]);
              // Reabre Contato
              ReabreContato(campanhas[i].ligacoes_ativas[j]);
              // Remove da lista
              campanhas[i].ligacoes_ativas.splice(j, 1);

              // Processa campanha novamente.
              discador.ProcessaCampanha(campanhas[i]);

              return;
            }
          }
        }
      }
    };

    function RegistraLog(ligacao) {
      const duracao = ligacao.falando ? ligacao.falando : '00:00:00';
      db.query("INSERT INTO campanhas_registro(id_campanha,id_contato,id_numero,numero,callid,actionid,falha,id_status,campos,duracao) VALUES($1,$2,$3,$4,$5,$6,$7,$8,$9,$10)",[ligacao.CampanhaID,ligacao.ContatoID,ligacao.NumeroID,ligacao.Destino,ligacao.Linkedid,ligacao.ActionID,ligacao.Falha,ligacao.Status,ligacao.Campos,duracao]).then(res => {
        log.info("REGISTRALOG: Log da campanha ID = "+ ligacao.CampanhaID +" registrado.");
      }).catch(e => {
        log.error('REGISTRALOG: Falha ao registrar o log para camapanhaID = '+ligacao.CampanhaID);
        log.error(e);
      });
    };

    function RegistraCompletada(ligacao) {
      // Fecha Numero
      db.query("UPDATE campanhas_contato_numeros SET sucesso = true WHERE id_contato = $1",[ligacao.ContatoID]).then(res => {
        log.info('REGISTRACOMPLETADA: Numero Finalizado para contatoID = '+ligacao.ContatoID);
      }).catch(e => {
        log.error('REGISTRACOMPLETADA: Falha ao registrar chamada completada.');
        log.error(e);
      });

      // Fecha contato
      db.query("UPDATE campanhas_contato SET sucesso = true WHERE id = $1",[ligacao.ContatoID]).then(res => {
        log.info("REGISTRACOMPLETADA: Contato ID = "+ ligacao.ContatoID +" Finalizado.");
      }).catch(e => {
        log.error('REGISTRACOMPLETADA: Falha ao registrar contato finalizado.');
        log.error(e);
      });
    };

    function ReabreContato(ligacao) {
      // Reabre Numero
			// 28-05-2018 Alterado para reabrir todos os numeros do contato pois a rotina de completar fecha todos e sem isso o discador não disca os demais números do contato.
      db.query("UPDATE campanhas_contato_numeros SET sucesso = false WHERE id_contato = $1",[ligacao.ContatoID]).then(res => {
        log.info('REABRECONTATO: Numeros reabertos para contato ID = '+ligacao.ContatoID);
      }).catch(e => {
        log.error('REABRECONTATO: Falha ao reabrir numero.');
        log.error(e);
      });

      // Reabre contato
      db.query("UPDATE campanhas_contato SET sucesso = false WHERE id = $1",[ligacao.ContatoID]).then(res => {
        log.info('REABRECONTATO: Contato Reaberto id = '+ligacao.ContatoID);
      }).catch(e => {
        log.error('REABRECONTATO: Falha ao reabrir contato.');
        log.error(e);
      });
    };

		function DecTentativaIncRetentativa(id) {
      db.query("UPDATE campanhas_contato_numeros SET tentativas = tentativas - 1, retentativas = retentativas + 1 WHERE id = $1",[id]).then(res => {
        log.info('DEC_TENTATIVA_INC_RETENTATIVA: Numero ID = '+id+' teve tentatativas decrementada e retentativas aumentada.');
      }).catch(e => {
        log.error('DEC_TENTATIVA_INC_RETENTATIVA: Falha ao incrementar retentativas.');
        log.error(e);
      });
		}

    function FechaCampanha(campanha) {
      const fim = moment.tz(options.config.servidor.timezone).format('YYYY-MM-DD HH:mm:ss');      
      ws.broadcast({event:'CampanhaFim',campanha: campanha});

      if(campanha.notificar_email && campanha.notificar_email.length > 0){
        let transporter = nodemailer.createTransport({
            host: options.config.email.host,
            port: options.config.email.port,
            secure: options.config.email.secure,
            auth: {
                user: options.config.email.user,
                pass: options.config.email.pass
            }
        });
            
        let mailOptions = {
            from: '"'+options.config.email.from_name +'" <'+options.config.email.from_email+'>',
            to: campanha.notificar_email,
            subject: 'Xcontact - A campanha '+campanha.nome+' foi finalizada',
            text: 'A campanha '+campanha.nome+' foi finalizada.',
        };

        transporter.sendMail(mailOptions, (error, info) => {
            if (error) {
              log.error('NOTIFICA_EMAIL: Falha ao enviar email.');
              return log.error(error);
            }
            log.info('Message sent: ' + info.messageId);
        });
      }

      db.query("UPDATE campanhas set status = 'F',data_status = $2,descricao = 'Campanha finalizada pelo discador.' WHERE id = $1",[campanha.id,fim]).then(res => {
        log.info('FECHACAMPANHA: Campanha finalizada ID = '+campanha.id);
      }).catch(e => {
        log.error('FECHACAMPANHA: Falha finalizar campanha.');
        log.error(e);
      });
    };

    function RemoveNumero(id_numero) {
      // Remove Numero
      db.query("DELETE FROM campanhas_contato_numeros WHERE id = $1",[id_numero]).then(res => {
        log.info('REMOVENUMERO: Numero de id '+ id_numero +' removido.');
      }).catch(e => {
        log.error('REMOVENUMERO: Falha ao remover numero.');
        log.error(e);
      });
    };

    function RemoveContato(id_contato) {
      // Remove Contato
      db.query("DELETE FROM campanhas_contato WHERE id = $1",[id_contato]).then(res => {
        log.info('REMOVECONTATO: Contato de id '+ id_contato +' removido.');
      }).catch(e => {
        log.error('REMOVECONTATO: Falha ao remover contato.');
        log.error(e);
      });
    };

		function cancelaClaTimers(ob) {
		  for(let i=0;i<ClaTimers.length;++i){
		  	if(ClaTimers[i].ob.id === ob.id) {
		      log.info('CANCELACLATIMERS: TimerID = '+ob.id);
		      clearInterval(ClaTimers[i].timer);
		      ClaTimers.splice(i,1);
		      return;
		    }
		  }
		};

    exports.ProcessaClassificacao = (callid,classificacao) => {
			var element = {id:new Date().getTime(), tentativas: 1, callid: callid, classificacao: classificacao};
	    log.info('PROCESSACLASSIFICACAO: Add agenda ID = '+element.id);
	    var job = {
	    	ob: element,
	        timer: setInterval(function(Obj) {
						// Busca os dados da ligação no log pelo callid
			      db.query("SELECT id_campanha,id_numero,id_contato FROM campanhas_registro WHERE callid = $1 OR actionid = $1 LIMIT 1",[Obj.callid]).then(res => {
							if(res.rows.length > 0){
								log.info('PROCESSACLASSIFICACAO: Callid '+ Obj.callid +' encontrado cancelando agendamento de classificação e seguindo com o processo.');
								cancelaClaTimers(Obj);
								// Busca a ação da classificação
			          db.query("SELECT acao FROM campanhas_classificacoes WHERE id_campanha = $1 AND id_classificacao = $2 LIMIT 1",[res.rows[0].id_campanha,Obj.classificacao]).then(res2 => {
			            if(res2.rows.length > 0){
			              log.info('PROCESSACLASSIFICACAO: Ação '+ res2.rows[0].acao +' encontrada para o callid '+ Obj.callid +' e classificação '+ Obj.classificacao);
			              switch (res2.rows[0].acao) {
			                case 'AN':
			                  const ligacao = {NumeroID:res.rows[0].id_numero,ContatoID:res.rows[0].id_contato};
			                  ReabreContato(ligacao);
			                  break;
			                case 'RN':
			                  RemoveNumero(res.rows[0].id_numero);
			                  break;
			                case 'RC':
			                  RemoveContato(res.rows[0].id_contato);
			                  break;
			              }
			            }
			            else {
			              log.info('PROCESSACLASSIFICACAO: Nenhuma ação encontrada para campanha '+ res.rows[0].id_campanha +' e classificação ' + Obj.classificacao);
			            }
			          }).catch(e => {
			            log.error('PROCESSACLASSIFICACAO: Falha ao processar a classificação: ');
			            log.error(e);
			          });
							}
							else {
			          log.info('PROCESSACLASSIFICACAO: Callid '+ callid +' não encontrado nos registros da campanha, pode não ser uma ligação realizada pelo discador. Vou tentar novamente daqui 30s');
			        }
						}).catch(e => {
			        log.error('PROCESSACLASSIFICACAO: Falha ao obter dados da tabela campanhas_registro:');
			        log.error(e);
			      });

	          ++Obj.tentativas;
	          if(Obj.tentativas > 5) {
							log.info('PROCESSACLASSIFICACAO: Callid '+ callid +' não encontrado depois de 5 tentativas nos registros da campanha, não deve ser uma ligação realizada pelo discador. Cancelando procura.');
	            cancelaClaTimers(Obj);
	          }
	        }, 30000, element)
	    }
	    ClaTimers.push(job);
    };

		// Executando uma checagem nas campanhas a cada 30s
		setInterval(() => {
			log.info('DISCADOR: Executando rotina a cada 30s');
			discador.ProcessaCampanhas();
		}, 30000);

    next();
};

exports.register.attributes = {
    name: 'xcontact-discador'
};
