var Watson = require('../bots/watson/index');
var InstanciasWatson = [];
const log = require('simple-node-logger').createSimpleLogger({ logFilePath: "/var/log/xcontact.log", timestampFormat: "DD-MM-YYYY HH:mm:ss.SSS" });
log.setLevel('info');
var Integracao = require('../integracao/index');
Integracao = new Integracao();
const moment = require('moment');
moment.locale('pt-br');
const ClienteAtendimento = require('../models/ClienteAtendimento');

module.exports = class InterconexaoWatson {

    constructor() { }

    enviaMensagemWatson(configwatson, atendimento, msg) {
        return new Promise(async (resolve, reject) => {
            try {
                var watsondata = await this.verificaSessao(atendimento.id, configwatson);
                if (!watsondata) {
                    watsondata = await this.criarSessao(atendimento, configwatson);
                }
                //Aqui verificamos se o status da instancia watson esta pendente para o watson ou para o cliente
                if (watsondata && InstanciasWatson[configwatson.parametros.apikey].status == 'watson') { //Caso esteja pendente para o watson
                    var retornoWatson = await InstanciasWatson[configwatson.parametros.apikey].enviarMensagem(msg.texto, watsondata.session_id, configwatson.parametros);
                    if (retornoWatson) {
                        log.info(retornoWatson);
                        let retorno = await this.processaRetorno(retornoWatson, configwatson);
                        if (retorno.error) {
                            this.processaErro(configwatson, atendimento, msg, retorno);
                        }
                        if (!retorno.error) {
                            let verifica = this.verificaParametros(InstanciasWatson[configwatson.parametros.apikey]);
                            if (verifica) {
                                InstanciasWatson[configwatson.parametros.apikey].status = 'wait';
                                let resp = await Integracao.fazRequisicao(`/opt/xcontact/xcontact-server/scripts/agenda.js`, verifica.Inte, (dados) => {
                                    log.info(resp);
                                    resolve(dados);
                                });
                            } else {
                                resolve(retorno.response);

                            }
                        }

                    }

                } else {
                    if (InstanciasWatson[configwatson.parametros.apikey].status == 'wait') {// Caso esteja pendente para o cliente
                        this.processaAcao(atendimento.id);
                    } else {
                        resolve();
                    }
                }
            } catch (error) {
                log.error('ERRO AO ENVIAR MENSAGEM NO WATSON', error);
            }

        })


    }
    async processaErro(configwatson, atendimento, msg, retorno) {
        try {
            if (retorno.code === 404) {
                let deletou = await InstanciasWatson[configwatson.parametros.apikey].deleteSession(atendimento.id);
                if (deletou) {
                    await InstanciasWatson[configwatson.parametros.apikey].criarSessao(atendimento.id, configwatson.parametros);
                    watsondata = await InstanciasWatson[configwatson.parametros.apikey].getSessionIdFromProtocolo(atendimento.id);
                    var retornoNew = await InstanciasWatson[configwatson.parametros.apikey].enviarMensagem(msg.texto, watsondata.session_id, configwatson.parametros);
                    resolve(retornoNew.response);
                }
            }
        } catch (error) {
            log.error(error);
        }

    }
    async processaAcao(protocolo) {
        await new ClienteAtendimento({ id: protocolo, ativo: true }).save({ require: false });

    }

    async verificaSessao(protocolo, configwatson) {
        var watsondata = await InstanciasWatson[configwatson.parametros.apikey];
        if (watsondata) {
            return watsondata.getSessionIdFromProtocolo(protocolo);
        }
        return undefined;
    }

    async criarSessao(atendimento, configwatson) {
        try {
            InstanciasWatson[configwatson.parametros.apikey] = new Watson(configwatson.parametros);
            InstanciasWatson[configwatson.parametros.apikey].status = 'watson';
            await InstanciasWatson[configwatson.parametros.apikey].criarSessao(atendimento.id, configwatson.parametros);
            let watsondata = await InstanciasWatson[configwatson.parametros.apikey].getSessionIdFromProtocolo(atendimento.id);
            return watsondata;
        } catch (error) {
            log.error('Erro ao criar sessao', error);
        }

    }
    async processaRetorno(retorno, configwatson) {
        try {
            let dados = retorno.data;
            if (dados.result.output.entities.length > 0) {
                if (!InstanciasWatson[configwatson.parametros.apikey].entities) InstanciasWatson[configwatson.parametros.apikey].entities = [];
                let entity = this.processaEntities(dados.result.output.entities);
                InstanciasWatson[configwatson.parametros.apikey].entities.push(entity);
            }
            if (dados.result.output.intents.length > 0) {
                let intent = this.processaIntents(dados.result.output.intents);
                InstanciasWatson[configwatson.parametros.apikey].lastIntent = intent;
            }
            if (dados.result.output.generic.length > 1) {
                let ret;
                for (let i = 0; i < dados.result.output.generic.length; i++) {
                    if (dados.result.output.generic[i].text) {
                        ret += dados.result.output.generic[i].text + "\n"
                    }
                }
                log.info(ret);
                retorno.response = ret;
            }

            if (retorno.error) return retorno;
            return retorno;
        } catch (error) {
            log.error('Erro ao processar retorno ', error);
        }

    }
    processaIntents(dados) {
        for (let i of dados) {
            if (i.confidence > 0.7) {
                return i.intent;
            }
        }
    }
    processaEntities(dados) {
        let aux = {};
        for (let i of dados) {
            if (i.confidence > 0.7) {
                aux.nome = i.entity;
                aux.value = i.value;
            }
        }
        log.info('PROCESSA ENTITY');
        return aux;

    }
    verificaParametros(dados) {
        try {
            if (!dados.entities) dados.entities = [];
            if (dados.lastIntent === 'Agendamento') {
                let aux = {};
                for (let entity of dados.entities) {
                    log.info(entity);
                    aux[`${entity.nome}`] = entity.value;
                }
                // caso tenha os dados que precisamos bora retornar true
                if (aux.localidades && aux.Inte) {
                    log.info('Temos tudo!!!!!');
                    return aux;
                }
                return false;
            }
        } catch (error) {

        }

    }







}