const config = require('../config.json');
const log = require('simple-node-logger').createSimpleLogger({ logFilePath: "/var/log/xcontact.log", timestampFormat: "DD-MM-YYYY HH:mm:ss.SSS" });
log.setLevel(config.log ? config.log : 'error');
const Cliente = require('../models/Cliente');
const NumerosAdicionais = require(`../models/ClienteNumerosAdicionais`);

const ClienteAtendimento = require('../models/ClienteAtendimento');
const ClienteGrupo = require('../models/ClienteGrupo');
const Message = require('../models/AtendimentoMensagem');
const Anexos = require('../models/MensagemAnexos');
const MensagemAnexos = require('../models/MensagemAnexos');

const Ami = require('../ami/index');
const ws = require('../ws/index');
const HandlersWS = require('../ws/handlers');
const namiLib = require("nami");
const moment = require('moment');
const { date } = require('joi');

module.exports = class Controller {
    constructor() {
    }

    async encerrarAtendimento(data) {

        let atendimento = await new ClienteAtendimento({ id: data.protocolo, status: 'Encerrado', dataFim: new Date(moment.tz(config.servidor.timezone)) }).save({ require: false });
        let ra = await ws.getSocketAgenteByFullName(atendimento.ultimoAgente);
        if (ra) ra.emit('message', JSON.stringify({ id: atendimento.id, event: 'RemoveAtendimento' }));

    }
    async getCliente(msg) {
        msg.idCliente = msg.remetente;
        try {

            var num = msg.idCliente.substr(-8);
            let numeroAdicional = await NumerosAdicionais.query.first()
                .where(NumerosAdicionais.where.like({ numero: `%${num}` })).fetch();

            //Caso nao ache na tabela de numeros adicionais
            if (!numeroAdicional) {
                console.log(`Nao tinha numeroAdicional`, num);
                var cliente = await Cliente.query.require(true).first()
                    .where(Cliente.where.like({ numero: `%${num}` }))
                    .fetch();
                return cliente;
            }
            var cliente = await Cliente.query.require(true).first()
                .where({ id: numeroAdicional.clienteID })
                .fetch();
            console.log(`Achamos o medonho`, cliente);
            return cliente;
            // if (msg.idCliente.length === 11) {
            //     var num = msg.idCliente.substring(3);
            //     var cliente = await Cliente.query.require(true).first()
            //         .where(Cliente.where.like({ numero: `%${num}` }))
            //         .fetch();
            //     return cliente;
            // } else if (msg.idCliente.length === 13) {
            //     var num = msg.idCliente.substring(5);
            //     var cliente = await Cliente.query.require(true).first()
            //         .where(Cliente.where.like({ numero: `%${num}` }))
            //         .fetch();
            //     return cliente;
            // } else {
            //     var cliente = await Cliente.query.require(true).first()
            //         .where(Cliente.where.like({ numero: `%${msg.idCliente}` }))
            //         .fetch();
            //     return cliente;
            // }
        } catch (error) {
            var cliente = await Cliente.fetchById(9999999);
            return cliente;
        }
    }
    async Desliga(dados) {
        try {
            ws.broadcastEvent(`Hangup`, dados);
        } catch (error) {
            console.error(error);
        }
    }

    async NewCall(socket, dados) {
        log.info('ENTROU NO NEW CALL', dados);
        // log.info(dados);
        dados.remetente = dados.destino;
        if (dados.fila) {
            const cliente = await this.getCliente(dados);
            console.log(`CLIENTE`, cliente);
            let atendimento = await ClienteAtendimento.query.require(false).first()
                .where({ idCliente: cliente.id })
                .where({ queue: dados.fila })
                .where({ status: 'Entrou' })
                .where({ linkedid: dados.uniqueid })
                .fetch();
            //Tinha um atendimento ativo
            if (atendimento && cliente.id != 9999999) {
                atendimento.ultimoAgente = dados.agente;
                atendimento.save();
                await new Message({
                    idAtendimento: atendimento.id,
                    dataHora: new Date(moment.tz(config.servidor.timezone)),
                    inOut: 3,
                    agente: atendimento.ultimoAgente,
                    mailMessageID: null,
                    texto: `O Agente ${dados.agente} atendeu`
                }).save();
                return

            }
            log.info('CRIAMOS UM NOVO ATENDIMENTO')
            var at = await new ClienteAtendimento({
                fromID: dados.remetente,
                fromName: cliente.nome,
                sentido: dados.sentido,
                queue: dados.fila,
                idCliente: cliente.id,
                linkedid: null,
                ativo: true,
                status: 'Entrou',
                queueTypeID: 1,
                newstate: true,
                action: 'NewCall',
                linkedid: dados.uniqueid,
                ultimoAgente: dados.agente,
                dadosAdicionais: { "nome": cliente.nome, "remetente": dados.remetente, "tipo": "Sainte" }
            }).save();


            await new Message({
                idAtendimento: at.id,
                dataHora: new Date(moment.tz(config.servidor.timezone)),
                inOut: 3,
                agente: at.ultimoAgente,
                mailMessageID: null,
                texto: dados.sentido === 'Sainte' ? `O Agente ${at.ultimoAgente} inicou uma chamada` : `O Agente ${at.ultimoAgente} recebeu`
            }).save();
        }
        ws.broadcastEvent(`AddAtendimento`, at);


        // var d = {
        //     origem: dados.origem,
        //     destino: dados.destino,
        //     contexto: dados.contexto,
        //     action: 'Disca'
        // }
        // HandlersWS[d.action](socket, dados);
        //  this.Discar(dados);
    }


}