const Agente = require("../models/Agente");
const Cliente = require("../models/Cliente");
const ListaTransmissao = require("../models/ListaTransmissao");
const ListaTransmissaoArquivos = require("../models/ListaTransmissaoArquivos");
const ListaTransmissaoNumeros = require("../models/ListaTransmissaoNumeros");
const Atendimento = require("../xc_modules/omni/class/Atendimento");
const cron = require("node-cron");
let scheduledJobs = [];
let socket = null;
const path = require("path");
const fileType = require("file-type");
const fs = require("fs");
var queue = require("queue");
const Fila = require("../models/Fila");
var processaQueue = queue({
  autostart: true,
  concurrency: 1,
});
const setQueue = async (idLista) => {
  let that = this;
  let job = async function () {
    try {
      await processaListaQueue(idLista);
    } catch (error) {
      console.log(error);
    }
  };
  processaQueue.push(job);
};
async function sendFiles(files, contato, atendimento, lista) {
  for (const arquivo of files) {
    const buffer = fs.readFileSync(arquivo.caminho);
    const file = buffer;
    file.name = path.basename(arquivo.caminho);
    file.type = fileType(buffer).mime;
    file.path = arquivo.caminho;
    sendMsgFile(file, contato, atendimento, arquivo.legenda, lista);
  }
}
function isToday(date) {
  const today = new Date();
  return (
    date.getDate() === today.getDate() &&
    date.getMonth() === today.getMonth() &&
    date.getFullYear() === today.getFullYear()
  );
}

exports.initListaTransmissao = async () => {
  try {
    socket = require("socket.io-client")("ws://localhost:8000/omni");
    socket.on("connect_error", (response) => console.log(response));
    socket.on("connect", () => {
      console.log("CONECTOU");
      socket.emit(
        "Login",
        {
          xcontactID: "teste",
          id: socket.id,
          token: "teste",
          agente: "modulo_xc",
          ativo: 1,
          ramal: "xc",
          web: true,
          versao: "4.0",
        },
        async (retorno) => {
          console.log(retorno);
          const listas = await ListaTransmissao.query.fetch();
          if (listas && listas.length > 0) {
            for (let lista of listas) {
              if (lista.agendaData) {
                this.setarCron(lista, new Date(lista.agendaData));
              }
            }
          }
        }
      );
    });
  } catch (error) {
    console.error("ERRO AO INICIAR MODULO LISTA DE TRANSMISSAO", error);
  }
};
exports.processaLista = async (idLista) => {
  const lista = await ListaTransmissao.fetchById(idLista);
  lista.status = "Enfileirada";
  ListaTransmissao.save({
    id: idLista,
    status: "Enfileirada",
    ultimaExecucao: new Date(),
  });
  setQueue(idLista);
};
const processaListaQueue = async (idLista) => {
  try {
    console.log("VAMOS PROCESSAR", idLista);
    const lista = await ListaTransmissao.fetchById(idLista);
    if (!lista.ativa) return;
    lista.status = "Processando";
    ListaTransmissao.save({ id: idLista, status: "Processando" });
    const numeros = await ListaTransmissaoNumeros.query
      .where({ idLista: idLista })
      .fetch();
    const files = await ListaTransmissaoArquivos.query
      .where({ idLista: idLista })
      .fetch();
    if (numeros && numeros.length > 0) {
      console.log("LISTA", lista);
      for (const item of numeros) {
        const atendimento = await criarAtendimento(lista, item);
        if (!lista.idTemplate) {
          sendMsg(lista.texto, item, atendimento);
        }
        if (files && files.length > 0) {
          await sendFiles(files, item, atendimento, lista)
            .then(() =>
              console.log("Todos os arquivos foram enviados com sucesso!")
            )
            .catch((error) => console.error(error));
        } else {
          if (lista.idTemplate) sendTemplate(lista, item, atendimento);
        }
        // Aguarde 300 ms antes de prosseguir para o próximo item
        await new Promise((resolve) => setTimeout(resolve, 300));
      }
    }
    lista.ultimaExecucao = new Date();
    if (!lista.agendaHora) delete lista.agendaHora;
    if (!lista.agendaDias) delete lista.agendaDias;
    if (!lista.agendaMesses) delete lista.agendaMesses;
    if (!lista.agendaDiasSemana) delete lista.agendaDiasSemana;
    lista.status = "Concluida";
    lista.save();
    return lista;
  } catch (error) {
    console.error("ERRO AO PROCESSAR LISTA DE TRANSMISSAO", error);
  }
};

function dateToCronExpression(date) {
  const year = date.getFullYear();
  const month = date.getMonth() + 1;
  const day = date.getDate();
  const hour = date.getHours();
  const minute = date.getMinutes();

  return `${minute} ${hour} ${day} ${month} *`;
}

// Função para agendar um novo job
const scheduleJob = (jobId, scheduleTime) => {
  // Verifica se já existe um job com o mesmo ID
  const existingJob = scheduledJobs.find((job) => job.id === jobId);

  // Se existir, cancela o job
  if (existingJob) {
    existingJob.task.stop();
    scheduledJobs = scheduledJobs.filter((job) => job.id !== jobId);
  }
  console.log("DATA", dateToCronExpression(scheduleTime));
  // Agendando o novo job
  const newJob = {
    id: jobId,
    task: cron.schedule(dateToCronExpression(scheduleTime), () =>
      setQueue(jobId)
    ),
  };

  // Adicionando o novo job ao array
  scheduledJobs.push(newJob);
};
exports.setarCron = (lista, date) => {
  try {
    // lista.ativo = true
    scheduleJob(lista.id, date);
  } catch (error) {
    console.error(error);
  }
};
async function criarAtendimento(lista, contato) {
  try {
    let fila = await Fila.query.require(true).first()
    .where({ name: lista.instancia })
    .fetch();
    let cliente = await Cliente.BuscaClientePorNumero(contato.numero);
    if (cliente) cliente = cliente[0];
    console.log("CLIENTE", cliente);
    if (!cliente) {
      cliente = await Cliente.save({
        nome: contato.nome ? contato.nome : "",
        numero: contato.numero,
      });
      cliente = cliente[0];
    }
    const agente = await Agente.fetchById(lista.idAgente);
    let atendimento =await Atendimento.save({
        ultimoAgente: agente.fullname,
        queueTypeID: fila.queue_type,
        queue: lista.instancia,
        queueFrom: lista.instancia,
        fromID: contato.numero,
        fromName: contato.nome,
        sentido: 'Sainte',
        status: lista.status_atendimento || "Encerrado",
        idCliente: cliente.id,
      });
    console.log("TEMOS UM ATENDIMENTO PARA TRANSMISSAO", atendimento);
    if (Array.isArray(atendimento)) atendimento = atendimento[0];
    return atendimento;
  } catch (error) {
    console.error("ERRO AO criar atendimento lista de transmissao", error);
  }
}

function sendMsg(texto, contato, atendimento) {
  console.log("ENVIANDO MENSAGEM", texto);
  const message = {};
  message.protocolo = atendimento.id;
  message.message = texto;
  message.resposta = texto;
  message.texto = texto;
  message.queue = atendimento.queueFrom
    ? atendimento.queueFrom
    : atendimento.queue;
  message.fromID = contato.numero;
  message.action = "ClientChatMessage";
  //Omni.sendMsg(message)
  socket.emit("sendMsg", message, (retorno) => {
    console.log("sendmsg", retorno);
  });
}
function getVariaveisString(str, contato) {
  const regex = /{{([^}]*)}}/g;
  const matches = [];
  let match;
  while ((match = regex.exec(str))) {
    if (match[1].trim() == "Nome") {
      matches.push(contato.nome);
    } else if (match[1].trim() == "Numero") {
      matches.push(contato.numero);
    } else {
      matches.push(match[1].trim());
    }
  }
  return matches;
}
function sendTemplate(lista, contato, atendimento) {
  console.log("SENDTEMPLATELISTA");
  const data = {};
  data.agente = atendimento.ultimoAgente;
  data.protocolo = atendimento.id;
  data.queue = atendimento.queue;
  data.templateId = lista.idTemplate;
  data.message = lista.texto;
  data.fields = getVariaveisString(lista.texto, contato);
  socket.emit("sendTemplate", data, (retorno) =>
    console.log("RETORNO SENDTEMPLATE", retorno)
  );
}

function sendMsgFile(file, contato, atendimento, legenda, lista) {
  console.log("ENVIANDO ARQUIVO", lista);
  const message = {};
  message.file = file;
  message.protocolo = atendimento.id;
  message.queue = atendimento.queueFrom
    ? atendimento.queueFrom
    : atendimento.queue;
  (message.queueFrom = atendimento.queueFrom
    ? atendimento.queueFrom
    : atendimento.queue),
    (message.fromID = contato.numero);
  message.action = "ClientChatMessage";
  message.queueAnexos = [];
  message.file_name = message.file.name;
  message.filename = message.file.name;
  message.queueAnexos.push({
    mimeType: message.file.type,
    caminho: message.file.path,
    filename: message.file.name,
  });
  message.type = message.file.type;
  message.client = `newClient`;
  message.mimetype = message.file.type;
  message.caption = legenda;
  message.Msg = legenda;
  if (lista && lista.idTemplate) {
    message.template = {};
    message.template.templateId = lista.idTemplate;
    message.template.fields = getVariaveisString(lista.texto, contato);
  }

  let data = {
    file: true,
    protocolo: atendimento.id,
    queue: atendimento.queue,
    queueFrom: atendimento.queueFrom,
    queueAnexos: [
      {
        caminho: message.file.path,
        mimeType: message.file.type,
        filename: message.file.name,
      },
    ],
    type: message.file.type,
    filename: message.file.name,
    file_name: message.file.name,
    Msg: "",
    client: `newClient`,
    caminhoArquivoLista: file.path,
  };
  if (message.template) data.template = message.template
  try {
    const { Omni } = require("../ws");

    Omni.sendFileMsg(data);
  } catch (error) {
    console.error("DEU PAu-===================", error);
  }
  // socket.emit('sendFileMsg', message, (retorno) => {
  //     console.log('sendmsg', retorno)
  // })
}
