const { setTimeout } = require("timers");

exports.register = (server, options, next) => {
    //#region imports
    const nodemailer = require("nodemailer");
    const notifier = require('mail-notifier');
    const Fila = require('../models/Fila');
    const Ws = require('../ws/index');
    const fs = require('fs');
    const msal = require('@azure/msal-node');    


    async function getTokenMicrosoft(data) {  
        const msalConfig = {  
            auth: {  
            clientId: data.clientId,   
            authority: `https://login.microsoftonline.com/${data.tenantId}`,  
            clientSecret: data.clientSecret
           } 
        }
        const tokenRequest = {  
            scopes:["https://outlook.office365.com/.default"]
        };  
        const cca = new msal.ConfidentialClientApplication(msalConfig);  
     return await cca.acquireTokenByClientCredential(tokenRequest);  
    }  
    const log = require('simple-node-logger').createSimpleLogger({ logFilePath: "/var/log/xcontact.log", timestampFormat: "DD-MM-YYYY HH:mm:ss.SSS" });
    log.setLevel(options.config.log ? options.config.log : 'error');
    const transporters = [];
    const Guarda = [];
    //#endregion
    //#region main 
    exports.main = async () => {
        try {
            let filas = await Fila.query.require(true)
                .where({ queue_type: 4 })
                .fetch();
            filas = filas.filter(fila => fila.emailConfig !== null);
            //VAMOS INSTANCIAR O IMAP PARA TODAS AS FILAS DE EMAIL
            for (let fila of filas) {

                let config = fila.emailConfig;
                //CONFIGURACOES NODEMAILER 
                let objTransport = {};
                console.log(fila.name + ' - ' + config.smtp.host)
                if (config.smtp.host.indexOf('gmail') !== -1) {
                    objTransport.service = 'gmail';
                    objTransport.auth = {
                        user: config.email,
                        pass: config.password
                    };
                } else {
                    objTransport = {
                        host: config.smtp.host,
                        port: config.smtp.port,
                        secure: config.smtp.port === '465' ? true : false, // true for 465, false for other ports
                        auth: {
                            user: config.email,
                            pass: config.password
                        },
                        tls: config.imap.tls,
                        tls: {
                            rejectUnauthorized: false
                        }
                    };

                    if(config.oauth2 && config.microsoft) {
                        const authResponse = await getTokenMicrosoft(config.microsoft_data);  
                        const token =  authResponse.accessToken
                        objTransport.auth = {  type: "OAuth2", user: config.email, accessToken: token  }
                    }
                }

                let transporter = nodemailer.createTransport(objTransport)
                transporters[fila.name] = transporter;

                //CONFIGURACOES IMAP
                let imap = {
                    user: config.email,
                    password: config.password,
                    host: config.imap.host,
                    port: config.imap.port,
                    tls: config.imap.tls,
                    search: config.imap.search,
                    box: config.imap.box,
                    tlsOptions: { rejectUnauthorized: false }
                };
                if(config.oauth2 && config.microsoft) {
                    const authResponse = await getTokenMicrosoft(config.microsoft_data);  
                    const token =  authResponse.accessToken
                    let base64Encoded = Buffer.from([`user=${config.email}`, `auth=Bearer ${token}`, '', ''].join('\x01'), 'utf-8').toString('base64'); 
                    imap.xoauth2 = base64Encoded
                }        
                let email = notifier(imap);
                email.on('mail', mail => {
                    if (Guarda.indexOf(mail.messageId) === -1) {
                        Guarda.push(mail.messageId);
                        ProcessaEmail(mail, fila)
                    }
                });
                email.on('end', () => {
                    email.stop();
                });
                email.on('error', (err) => {
                    console.error('ERRO NO IMAP DO EMAIL ' + fila.name, err)
                    email.stop();
                    // email.start();

                });
                email.on('connected', () => {
                   console.log(`${config.email} conectado!`)
                });
                email.start();
            }
        } catch (error) {
           console.error('ERRRO NO MAIL NOTIFIER', error);
        }

    }
    //#endregion

    //#region processa 
    async function criarInstanciaNodeMailer(fila, data) {
        try {
            let config = fila.emailConfig;
            let objTransport = {};
           
            if (config.smtp.host.indexOf('gmail') !== -1) {
                objTransport.service = 'gmail';
                objTransport.auth = {
                    user: config.email,
                    pass: config.password
                };
            } else {
                objTransport = {
                    host: config.smtp.host,
                    port: config.smtp.port,
                    secure: config.smtp.port === '465' ? true : false,
                    auth: {
                        user: config.email,
                        pass: config.password
                    },
                    tls: config.imap.tls,
                    tls: {
                        rejectUnauthorized: false
                    }
                };

                if(config.oauth2 && config.microsoft) {
                    const authResponse = await getTokenMicrosoft(config.microsoft_data);  
                    const token =  authResponse.accessToken
                    objTransport.auth = {  type: "OAuth2", user: config.email, accessToken: token  }
                }
            }

            const transporter = nodemailer.createTransport(objTransport)
            return transporter
        } catch (error) {
            console.error(error)
        }
    }
    async function ProcessaEmail(mail, fila) {
        try {
            let msg = {};
            //Regex para remover fwd e re do assunto do email....
            var regex = /([\[\(] *)?(RE|FWD?) *([-:;)\]][ :;\])-]*|$)|\]+ *$/igm

            msg.texto = mail.html ? mail.html : mail.text;
            msg.assunto = mail.subject.replace(regex, "");
            msg.mailMessageID = mail.messageId;
            msg.from = mail.from[0].address;
            msg.name = mail.from[0].address;
            if (mail.headers.references) msg.references = mail.headers.references.split(' ').map(ref => ref.replace(/<|>/g, ""));
            console.log('REFERENCES_________', msg.references)
            msg.queueTypeID = 4;
            let aux = {};
            if (mail.cc) {
                aux.emails = mail.to.map(m => m.address) && mail.cc.map(m => m.address);
            } else {
                aux.emails = mail.to.map(m => m.address);
            }
            msg.protocolo = mail.headers['x-xcontactid'];
            console.log('HEADERS', mail.headers)
            console.log('PROTOCOLO DO EMAIL', msg.protocolo)
            aux.from = msg.from;
            msg.dados_adicionais = JSON.stringify(aux);
            msg.inOut = 1;
            msg.instanceID = fila.name;
            msg.dataHora = mail.date;
            if (mail.attachments) {
                msg.queueAnexos = [];
                for (let i = 0; i < mail.attachments.length; i++) {
                    let attachment = mail.attachments[i];
                    var today = new Date();
                    var dir = `/gravacoes/${fila.name}/${today.getFullYear()}/${today.getMonth() + 1}/${today.getDate()}`;
                    fs.mkdirSync(dir, { recursive: true });
                    msg.file_name = dir + "/" + attachment.generatedFileName;
                    msg.mimeType = attachment.contentType;
                    msg.queueAnexos.push({ caminho: dir + "/" + attachment.generatedFileName, mimeType: attachment.contentType });
                    //msg.anexo.push(dir_link +"/"+ attachment.generatedFileName);
                    fs.writeFile(dir + "/" + attachment.generatedFileName, attachment.content, 'base64', function (err) {

                    });
                }
                Ws.wsEvento.emit('mailMessage', msg);

            } else {
                Ws.wsEvento.emit('mailMessage', msg);
            }

        } catch (error) {
            log.error('ERRO NO PROCESAEMAIL', error);
        }


    }
    this.main();
    let that = this
    setTimeout(() => {
        that.main()
    }, 1800000);
    //#endregion

    //#region Responde Email
    exports.respondeEmail = async data => {
        try {
            let fila = await Fila.query.require(true).first().where({ name: data.queue }).fetch();
            
            const nodeMailer = await criarInstanciaNodeMailer(fila, data)
            console.log(`RESPONDE EMAIL`, data);
            var regex = /([\[\(] *)?(RE|FWD?) *([-:;)\]][ :;\])-]*|$)|\]+ *$/igm
            const { v4: uuidv4 } = require('uuid'); // UUID para gerar um identificador único
            const uid = uuidv4();
            var reply = {
                from: nodeMailer.options.auth.user,
                name: nodeMailer.options.auth.user,
                to: data.remetente,
                xMailer: 'Atmail 7.8.0.2',
                subject: data.assunto.replace(regex, ""),
                html: data.message,
                inReplyTo: data.mailMessageID,
                headers: {
                    'X-Xcontactid': data.protocolo,
                    'Message-ID': `<${uid}@x5s.com.br>`
                } 
            }
            if (data.cc) {
                reply.cc = data.cc
            }
            if (data.cco) reply.bcc = data.cco

            let auxEnv = {};
            auxEnv.name = nodeMailer.options.auth.user;
            auxEnv.from = nodeMailer.options.auth.user;
            auxEnv.headers = reply.headers
            auxEnv.to = data.remetente;
            if (data.cc !== null) {
                auxEnv.cc = data.cc
            }
            if (data.cco !== null) {
                auxEnv.bcc = data.cco
            }
            auxEnv.inReplyTo = data.mailMessageID
            auxEnv.references = data.mailMessageID
            auxEnv.xMailer = 'Atmail 7.8.0.2'
            auxEnv.html = data.message
            auxEnv.subject = data.assunto.replace(regex, "");
            var replyEnvelope = {
                envelope: auxEnv,
                subject: data.assunto.replace(regex, ""),
                inReplyTo: data.mailMessageID,
                xMailer: 'Atmail 7.8.0.2',
                html: data.message
            }

            if (data.anexos) {
               const aux =  data.anexos.map( a => {
                    a.cid = a.filename
                    return a
                })
                reply.attachments = aux

                if (data.envelope) {
                    auxEnv.attachments = aux
                }
                console.log('ANEXOS==================================', aux)
            }
        

            if (data.envelope) {
                let mail = nodeMailer.sendMail(auxEnv, function (error, responde) {
                    if (error) {
                        log.error('ERRO NO IMAP => ', error);
                    } else {
                        log.info("Email enviado com sucesso", success);
                    }
                });
            } else {
                let mail = nodeMailer.sendMail(reply, function (error, success) {
                    if (error) {
                        log.error('ERRO NO IMAP => ', error);
                    } else {
                        console.log("Email enviado com sucesso sem envelope", success);
                    }
                });
            }
            return {id: `${uid}@x5s.com.br`, status: 'success'};
        } catch (error) {
            console.error('Erro ao enviar email', error);
        }

    }
    //#endregion
    next();
}

exports.register.attributes = {
    name: 'xcontact-mailnotifier'
};
