
'use strict';

const pkg = require('../package.json');
require('dotenv/config');

module.exports = {
	register: require('hapi-swagger'),
	options: {
		info: {
			title: 'Xcontact REST API',
			description: `API REST v2 e v3 do Xcontact.

Use o campo de api_key no topo da página para informar seu <mark>token de acesso</mark>, seguido da tecla ENTER e ter acesso aos testes dos métodos. 
<ul class="operations">
	<li class="put operation">
		Use o metodo POST /usuario/login para obter seu <mark>token de acesso</mark>.
	</li>
</ul>

Esta API apresenta a seguinte estrutura:
Os métodos são agrupados por Operação, que equivale ao nível do caminho. 
<table>
	<thead>
		<th><b>Operação</b></th>
		<th><b>Método</b></th>
		<th><b>URL</b></th>
	</thead>
	<tr>
		<td colspan="2">Usuario</td>
		<td><mark><i>apiRoot</i></mark>/api/v<mark>X</mark>/usuario</td>
	</tr>
	<tr>
		<td>&nbsp;</td>
		<td>GET</td>
		<td><mark><i>apiRoot</i></mark>/api/v<mark>X</mark>/usuario</td>
	</tr>
	<tr>
		<td>&nbsp;</td>
		<td>*</td>
		<td><mark><i>apiRoot</i></mark>/api/v<mark>X</mark>/usuario/*</td>
	</tr>
</table>
Executando o método GET de cada operação, sem parâmetros será retornada a consulta padrão, limitada a uma página. Através desta consulta você saber se existem mis pãginas de dados e refinar sua pesquisa.

<script type="text/javascript">
	$("#api_selector").on("keyup", function(event) {
		var acv = $("#input_apiKey").val();
		if(acv && !acv.match("^[B,b]earer"))
		{
			$("#input_apiKey").val("Bearer " + acv);
		}
	});
</script>
`,
			// Get the version from package.json
			version: pkg.version,
			contact: {
				name: 'X5 Soluções',
				url: 'http://xcontactcenter.com.br/'
			}
		},
		securityDefinitions: {
			Bearer: {
				type: 'apiKey',
				name: 'Authorization',
				in: 'header'
			}
		},
		schemes: [process.env.SWAGGER_PROTOCOLO ? process.env.SWAGGER_PROTOCOLO : 'https'],
		// Adicionado o agrupamento por tags, funciona melhor que por caminho
		grouping: 'tags',
		// Esconde o conteúdo das operações
		uiCompleteScript: `$("[id$='endpoint_list']").attr("style","display: none");`,
		// Setup the documentation to be the root of this host
		documentationPath: '/',
		jsonEditor: true,
		// This is for use of grouping together paths.  Since each of our paths begin with `/api/v{1,2,3,...}`, we want to ignore those first to arguments in the path, since they won't help us group together resources
		pathPrefixSize: 8,
		// This is also used for grouping, though because of the line above, I don't believe that this line may be needed.  Seems to work with/without it.
		basePath: '/api',
	}
};
