/**
 * Changelog
 * 07/11/2023 <Rossato> - Adicionado terceiro parametro no handler passando o emiter
 */
const path = require('path');
const glob = require('glob');

module.exports = class EventoRoute {
    constructor() {
        this.rotas = [];
    }
    start(io, emiter) {
        this.rotas.forEach(rota => {
            emiter.on(`${rota.method}`, async (ev) => {
                await rota.handler(io, ev, emiter);
            });
        });
    }
    register(dir) {
        return new Promise((resolve, reject) => {
            glob(`${dir}/**/routes.js`, { cwd: process.cwd() }, (error, files) => {
                if (error) {
                    console.error(`[App] Erro ao carregar rotas api WS -> `, error);
                    reject(error);
                } else {
                    files.forEach(file => {
                        try {
                            let route = require(path.join(process.cwd(), file));
                            this._add(route);
                        } catch (error) {
                            console.error(`[App] Erro ao definir '${file}' api WS -> `, error);
                            reject(error);
                        }
                    });
                    resolve('Modulos add');
                }
            });
        });
    }
    _add(rotas) {
        let metodos = rotas.map(m => m.method);
        let metodo = this.rotas.find(r => metodos.includes(r.method));
        if (metodo) {
            throw new Error(`O Metodo ${metodo.method} já está definido.`);
        }
        else {
            rotas.forEach(rota => {
                this[rota.method] = rota;
            });
            this.rotas = this.rotas.concat(rotas);
        }
    }
    method(method) {
        let metodo = this.rotas.find(r => r.method == method);
        if (!metodo) {
            throw new Error(`O Metodo ${method} Não Existe`);
        }
        else {
            return metodo;
        }
    }
}