const redis = require("redis");
const AtendimentoMensagem = require("../models/AtendimentoMensagem");
const db = require("../util/db");
const Atendimento = require("../xc_modules/omni/class/Atendimento");
let client = null
exports.register = async (server, options, next) => {
    const REDIS_PORT = process.env.REDIS_PORT || 6379;
    client = redis.createClient(REDIS_PORT);
    exports.client = client;
    next()

    client.on("connect", function() {
        console.log("Redis connected");
      });
      
      exports.set = (key, value) => {
        client.set(key, value, function(err, reply) {
            if (err) {
              console.error(err);
            } else {
              console.log(reply);
            }
          });
      }


      exports.get = (key) => {
        return new Promise((resolve,reject) => {
            client.get(key, (err, data) => {
                if (err) {
                  console.error(err);
                  reject(err)
                } else {
                  resolve(data)
                }
              });
        })       
    };
    const addAtendimento = async (atendimento, atendimentos) => {
      try {
        let msgs = await getMensagensAnexos(atendimento)
        if(msgs && msgs.rows) {
          msgs = msgs.rows          
          atendimento.mensagens = msgs
        } 
        atendimentos.push(atendimento)
      } catch (error) {
        console.error(error)
      }
    }
    const getMensagensAnexos = async (atendimento) => {
      const sql = `  SELECT m.*, anexos
      FROM "queueChatMessages" m
      LEFT JOIN (
        SELECT "id_message", json_agg(an) AS anexos
        FROM "queueAnexos" an
        GROUP BY "id_message"
      ) anexos ON m.id = anexos."id_message"     
      WHERE m."idQueueChat" = $1 
      ORDER BY m."dataHora" ASC`
      return await db.query(sql, [atendimento.id])
    }
    const atualizaAtendimento = async (old, novoAtendimento) => {
      try {
        old.status = novoAtendimento.status
        old.ultimoAgente = novoAtendimento.ultimoAgente
        let msgs = await getMensagensAnexos(old)
        // console.log('MENSAGENS', old)
        if(msgs && msgs.rows) {
          msgs = msgs.rows
          old.status = novoAtendimento.status
          old.ultimoAgente = novoAtendimento.ultimoAgente
          old.mensagens = msgs
          return old
        } 
      } catch (error) {
        console.error(error)
      }
    }
    exports.verificaAtendimentoAgente = async (agente, atendimentoId) => {
      try {
        const atendimento = await Atendimento.fetchById(atendimentoId)
        let contatos = await this.get(`contatos-${agente}`)
        if(contatos) contatos = JSON.parse(contatos)
        const contato = contatos.find(c=>c.id === atendimento.idCliente)
        if(contato){
          let atendimentos = contato.atendimentos.find(a=>a.tipo == atendimento.queueTypeID)
          if(atendimentos) {
            atendimentos = atendimentos.atendimentos
            const index = atendimentos.findIndex(a=>a.id == atendimento.id)
            if(index !== -1) {
              const jatem = atendimentos[index]   
              if(atendimento.ultimoAgente !== agente) {
                atendimentos.splice(index,1)
              }else{
                await atualizaAtendimento(jatem, atendimento)
              }                
              this.set(`contatos-${agente}`, JSON.stringify(contatos))
            } else{
              await addAtendimento(atendimento, atendimentos)
              this.set(`contatos-${agente}`, JSON.stringify(contatos))
            }
            
          } 
        }
      } catch (error) {
        console.error(error)
      }
    }
     
      // Setando um valor na chave "minhaChave"
     
};

exports.register.attributes = {
    name: 'redis'
};
  

