'use strict'

const Boom = require('boom');
const db = require('../../util/db');
const config = require('../../config.json');
const moment = require('moment-timezone');
moment.tz.setDefault(config.servidor.timezone);
const Paginacao = require('../../util/Paginacao');


/////////////
////////////    Apenas Exemplo, no caso da agenda faz tudo na rota com o CRUD
////////////




exports.getall = (request, reply) => {
  const max = request.query.max ? request.query.max : 15;
  const page = request.query.page ? request.query.page : 1;
  const knex = db.knex('public.agenda');

  if(request.query.nome) knex.whereRaw('LOWER(nome) LIKE ?', '%'+decodeURIComponent(request.query.nome).toLowerCase()+'%');
  if(request.query.numero) knex.whereRaw('numero LIKE ?', '%'+decodeURIComponent(request.query.numero)+'%');

  const pag = new Paginacao(max,page);
  pag.knex = knex;
  pag.getRet((ret, err) => {
      if(err){
          console.error(err);
          reply(Boom.badData(err));
      }
      else {
        reply(ret);
      }
  });
};

exports.getOne = (request, reply) => {
  const knex = db.knex('public.agenda');
  knex.where('id', request.params.ID);
  knex.select('*')
  .then(function(rows) {    
    const ret = {};
    if(rows.length > 0){
      ret.error = false;
      ret.full = true;
      ret.total = rows.length;
      ret.message = 'Dados da agenda id = '+request.params.ID;
      ret.dados = rows[0];
    }
    else {
      ret.error = true;
      ret.total = rows.length;
      ret.message = 'Nenhum registro encontrado para id = '+request.params.ID;
      ret.dados = rows[0];
    }
    reply(ret);
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};

exports.create = (request, reply) => {
  const knex = db.knex('public.agenda');
  knex.insert(request.payload);
  knex.returning('id')
  .then(function(id) {
    const ret = {};
    ret.error = false;
    ret.id = id[0];
    ret.message = 'Agenda id = '+id[0]+' criada com sucesso.';
    reply(ret);
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};

exports.update = (request, reply) => {
  const knex = db.knex('public.agenda');
  knex.update(request.payload);

  knex.where('id', request.params.ID)
  .then(function(rows) {
    const ret = {};
    ret.error = false;
    ret.total = rows;
    ret.message = 'Agenda id = '+request.params.ID+' atualizada com sucesso.';
    reply(ret);
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};

exports.delete = (request, reply) => {
  const knex = db.knex('public.agenda');
  knex.where('id', request.params.ID)
  .del()
  .then(function(rows) {
    if(rows > 0){
      const ret = {};
      ret.error = false;
      ret.total = rows;
      ret.message = 'Agenda id = '+request.params.ID+' apagada com sucesso.';
      reply(ret);
    }
    else {
      reply(Boom.badData("Nenhum resgistro encontrado para ID = "+request.params.ID));
    }
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};

exports.getGrupos = (request, reply) => {
  const max = request.query.max ? request.query.max : 15;
  const page = request.query.page ? request.query.page : 1;
  const knex = db.knex('public.agenda_grupos');

  if(request.query.nome) knex.whereRaw('LOWER(nome) LIKE ?', '%'+decodeURIComponent(request.query.nome).toLowerCase()+'%');  

  const pag = new Paginacao(max,page);
  pag.knex = knex;
  pag.getRet((ret, err) => {
      if(err){
          console.error(err);
          reply(Boom.badData(err));
      }
      else {
        reply(ret);
      }
  });
};

exports.CreateGrupo = (request, reply) => {
  const knex = db.knex('public.agenda_grupos');
  knex.insert(request.payload);
  knex.returning('id')
  .then(function(id) {
    const ret = {};
    ret.error = false;
    ret.id = id[0];
    ret.message = 'Grupo de agenda id = '+id[0]+' criado com sucesso.';
    reply(ret);
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};

exports.UpdateGrupo = (request, reply) => {
  const knex = db.knex('public.agenda_grupos');
  knex.update(request.payload);

  knex.where('id', request.params.ID)
  .then(function(rows) {
    const ret = {};
    ret.error = false;
    ret.total = rows;
    ret.message = 'Grupo de agenda id = '+request.params.ID+' atualizado com sucesso.';
    reply(ret);
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};

exports.DeleteGrupo = (request, reply) => {
  const knex = db.knex('public.agenda_grupos');
  knex.where('id', request.params.ID)
  .del()
  .then(function(rows) {
    if(rows > 0){
      const ret = {};
      ret.error = false;
      ret.total = rows;
      ret.message = 'Grupo de agenda id = '+request.params.ID+' apagado com sucesso.';
      reply(ret);
    }
    else {
      reply(Boom.badData("Nenhum resgistro encontrado para ID = "+request.params.ID));
    }
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};