'use strict'

const joi = require('joi');
const Boom = require('boom');
const CRUD = require('../../util/CRUD');
const db = require('../../util/db');
const ws = require('../../ws');
const Ami = require('../../ami');
const moment = require('moment-timezone');
const config = require('../../config');
const Agente = require('../../models/Agente');
const AgenteFilas = require('../../models/AgenteFilas');

exports.create = (request, reply) => {
  const { username, password } = request.payload;
  if (username === 'juca') {
    reply(Boom.badData('Juca não entra aqui'));
  }
  else {
    reply({ success: true });
  }
};

exports.getall = (request, reply) => {
  const max = request.query.max ? request.query.max : 15;
  const page = request.query.page ? request.query.page : 1;
  const knex = db.knex('public.v_agentes_full');
  if (request.query.nome) knex.whereRaw('LOWER(fullname) LIKE ?', '%' + decodeURIComponent(request.query.nome).toLowerCase() + '%');
  if (request.query.ramal && request.query.ramal.length > 0) knex.where('ramal', request.query.ramal);
  if (max === -1) {
    //console.log('SQL = '+ knex.toString());
    knex.clearSelect();
    // Parametros de ordenação, fora do count
    if (request.query.order) {
      for (let p = 0; p < request.query.order.length; ++p) {
        if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
          knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
        }
        else if (request.query.order[p] && request.query.order[p].campo) {
          knex.orderBy(request.query.order[p].campo, 'ASC');
        }
      }
    }
    knex.select('id', 'nome as login,', 'fullname as nome', 'ramal', 'filas')
      .then(function (rows) {
        for (let l = 0; l < rows.length; ++l) {
          for (let i = 0; i < rows[l].filas.length; ++i) {
            const agente = Ami.getAgenteLogadoByFilaRamal(rows[l].filas[i], rows[l].ramal);
            const fila = {
              fila: rows[l].filas[i],
              status: {
                codigo: agente && agente.status ? agente.status : 5,
                StatusNome: agente && agente.StatusNome ? agente.StatusNome : 'Indiponivel',
                DataStatus: agente && agente.DataStatus ? agente.DataStatus : moment.tz(config.servidor.timezone).format(),
                pausa: agente && agente.pausa ? agente.pausa : 0,
                pausa_motivo: agente && agente.pausa_motivo ? agente.pausa_motivo : undefined
              }
            };
            rows[l].filas[i] = fila;
            //rows.filas[i].status = agente;
          }
        }

        const ret = {};
        ret.error = false;
        ret.full = true;
        ret.total = rows.length;
        ret.message = 'Lista de todos os agentes';
        ret.dados = rows;
        reply(ret);
      })
      .catch(function (error) {
        console.error(error);
        reply(Boom.badData(error));
      });
  }
  else {
    const offset = max * (page - 1);
    //console.log('SQL = '+knex.count('* as total').toString());
    knex.count('* as total')
      .then(function (total) {
        if (total[0].total > 0) {
          knex.clearSelect();
          // Parametros de ordenação, fora do count
          if (request.query.order) {
            for (let p = 0; p < request.query.order.length; ++p) {
              if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
              }
              else if (request.query.order[p] && request.query.order[p].campo) {
                knex.orderBy(request.query.order[p].campo, 'ASC');
              }
            }
          }
          //console.log('SQL = '+knex.select('*').limit(max).offset(offset).toString());
          knex.select('id', 'nome as login,', 'fullname as nome', 'ramal', 'filas').limit(max).offset(offset).then(function (rows) {
            const pages = Math.ceil(parseInt(total[0].total) / max);
            for (let l = 0; l < rows.length; ++l) {
              for (let i = 0; i < rows[l].filas.length; ++i) {
                const agente = Ami.getAgenteLogadoByFilaRamal(rows[l].filas[i], rows[l].ramal);
                const fila = {
                  fila: rows[l].filas[i],
                  status: {
                    codigo: agente && agente.status ? agente.status : 5,
                    StatusNome: agente && agente.StatusNome ? agente.StatusNome : 'Indiponivel',
                    DataStatus: agente && agente.DataStatus ? agente.DataStatus : moment.tz(config.servidor.timezone).format(),
                    pausa: agente && agente.pausa ? agente.pausa : 0,
                    pausa_motivo: agente && agente.pausa_motivo ? agente.pausa_motivo : undefined
                  }
                };
                rows[l].filas[i] = fila;
              }
            }

            const ret = {};
            ret.error = false;
            ret.full = pages === 1 ? true : false;
            ret.page = page;
            ret.max = max;
            ret.offset = offset;
            ret.total = parseInt(total[0].total);
            ret.pages = pages;
            ret.message = 'Lista de agentes paginada';
            ret.dados = rows;
            if (request.query.return === 'array') reply(rows);
            else reply(ret);
          })
            .catch(function (error) {
              console.error(error);
              reply(Boom.badData(error));
            });
        }
        else {
          const ret = {};
          ret.error = true;
          ret.total = 0;
          ret.message = 'Nenhum registro encontrado';
          reply(ret);
        }
      })
      .catch(function (error) {
        console.error(error);
        reply(Boom.badData(error));
      });
  }
};

exports.getAgente = async (request, reply) => {
  try {
    const agente = await Agente.query
      .leftJoin(AgenteFilas)
      .require(true)
      .where({ id: request.params.AgenteID })
      .fetch();
    reply(agente[0]);
  } catch (error) {
    reply(Boom.badData(error));
  }
};

exports.getAgenteByRamal = (request, reply) => {
  const knex = db.knex('public.v_agentes_full');
  knex.where('ramal', request.params.AgenteRamal);
  knex.select('id', 'nome as login,', 'fullname as nome', 'ramal', 'filas')
    .then(function (rows) {
      for (let l = 0; l < rows.length; ++l) {
        for (let i = 0; i < rows[l].filas.length; ++i) {
          const agente = Ami.getAgenteLogadoByFilaRamal(rows[l].filas[i], rows[l].ramal);
          const fila = {
            fila: rows[l].filas[i],
            status: {
              codigo: agente && agente.status ? agente.status : 5,
              StatusNome: agente && agente.StatusNome ? agente.StatusNome : 'Indiponivel',
              DataStatus: agente && agente.DataStatus ? agente.DataStatus : moment.tz(config.servidor.timezone).format(),
              pausa: agente && agente.pausa ? agente.pausa : 0,
              pausa_motivo: agente && agente.pausa_motivo ? agente.pausa_motivo : undefined
            }
          };
          rows[l].filas[i] = fila;
        }
      }

      const ret = {};
      ret.error = false;
      ret.full = true;
      ret.total = rows.length;
      ret.message = 'Dados do agente pelo ramal = ' + request.params.AgenteRamal;
      ret.dados = rows;
      reply(ret);
    })
    .catch(function (error) {
      console.error(error);
      reply(Boom.badData(error));
    });
};

exports.AgenteLogin = (request, reply) => {
  const knex = db.knex('public.v_agentes_full');
  knex.where('id', request.params.AgenteID);
  knex.select('id', 'nome as login,', 'fullname as nome', 'ramal', 'filas')
    .then(function (rows) {
      const agente = {
        ramal: rows[0].ramal,
        fila: request.payload.fila,
        pausado: request.payload.pausado ? request.payload.pausado : false,
        nome: rows[0].nome,
      };
      Ami.QueueAdd(agente, (resposta) => {
        const ret = {};
        ret.error = resposta.response === 'Error' ? true : false;
        ret.message = resposta.message;
        reply(ret);
      });
    })
    .catch(function (error) {
      console.error(error);
      reply(Boom.badData(error));
    });
};

exports.AgenteLogoff = (request, reply) => {
  const knex = db.knex('public.v_agentes_full');
  knex.where('id', request.params.AgenteID);
  knex.select('id', 'nome as login,', 'fullname as nome', 'ramal', 'filas')
    .then(function (rows) {
      const agente = {
        ramal: rows[0].ramal,
        fila: request.payload.fila,
      };
      Ami.QueueRemove(agente, (resposta) => {
        const ret = {};
        ret.error = resposta.response === 'Error' ? true : false;
        ret.message = resposta.message;
        reply(ret);
      });
    })
    .catch(function (error) {
      console.error(error);
      reply(Boom.badData(error));
    });
};

exports.AgentePausa = (request, reply) => {
  const knex = db.knex('public.v_agentes_full');
  knex.where('id', request.params.AgenteID);
  knex.select('id', 'nome as login,', 'fullname as nome', 'ramal', 'filas')
    .then(function (rows) {
      const agente = {
        ramal: rows[0].ramal,
        fila: request.payload.fila,
        motivo: request.payload.motivo
      };
      Ami.QueuePause(agente, (resposta) => {
        const ret = {};
        ret.error = resposta.response === 'Error' ? true : false;
        ret.message = resposta.message;
        reply(ret);
      });
    })
    .catch(function (error) {
      console.error(error);
      reply(Boom.badData(error));
    });
};

exports.AgenteTiraPausa = (request, reply) => {
  const knex = db.knex('public.v_agentes_full');
  knex.where('id', request.params.AgenteID);
  knex.select('id', 'nome as login,', 'fullname as nome', 'ramal', 'filas')
    .then(function (rows) {
      const agente = {
        ramal: rows[0].ramal,
        fila: request.payload.fila
      };
      Ami.QueueUnpause(agente, (resposta) => {
        const ret = {};
        ret.error = resposta.response === 'Error' ? true : false;
        ret.message = resposta.message;
        reply(ret);
      });
    })
    .catch(function (error) {
      console.error(error);
      reply(Boom.badData(error));
    });
};

exports.AgenteLoginRamal = (request, reply) => {
  const agente = {
    ramal: request.params.AgenteRamal,
    fila: request.payload.fila,
    pausado: request.payload.pausado ? request.payload.pausado : false,
    nome: request.payload.nome,
  };
  Ami.QueueAdd(agente, (resposta) => {
    const ret = {};
    ret.error = resposta.response === 'Error' ? true : false;
    ret.message = resposta.message;
    reply(ret);
  });
};

exports.AgenteLogoffRamal = (request, reply) => {
  const agente = {
    ramal: request.params.AgenteRamal,
    fila: request.payload.fila,
  };
  Ami.QueueRemove(agente, (resposta) => {
    const ret = {};
    ret.error = resposta.response === 'Error' ? true : false;
    ret.message = resposta.message;
    reply(ret);
  });
};

exports.AgentePausaRamal = (request, reply) => {
  const agente = {
    ramal: request.params.AgenteRamal,
    fila: request.payload.fila,
    motivo: request.payload.motivo
  };
  Ami.QueuePause(agente, (resposta) => {
    const ret = {};
    ret.error = resposta.response === 'Error' ? true : false;
    ret.message = resposta.message;
    reply(ret);
  });
};

exports.AgenteTiraPausaRamal = (request, reply) => {
  const agente = {
    ramal: request.params.AgenteRamal,
    fila: request.payload.fila
  };
  Ami.QueueUnpause(agente, (resposta) => {
    const ret = {};
    ret.error = resposta.response === 'Error' ? true : false;
    ret.message = resposta.message;
    reply(ret);
  });
};

exports.getEvents = (request, reply) => {
  const crud = new CRUD('public.queue_events AS eventos');
  crud.filtrosraw = [];
  crud.filtros = [];
  if (request.query.eventos) crud.filtrosraw.push('eventos.event = ANY(ARRAY ' + JSON.stringify(request.query.eventos).replace(/"/g, "'") + ')');
  if (request.query.filas) crud.filtrosraw.push('eventos.queue = ANY(ARRAY ' + JSON.stringify(request.query.filas).replace(/"/g, "'") + ')');
  if (request.query.agentes) crud.filtrosraw.push('eventos.agent = ANY(ARRAY ' + JSON.stringify(request.query.agentes).replace(/"/g, "'") + ')');
  if (request.query.grupos) crud.filtrosraw.push('eventos.envent_group = ANY(ARRAY[' + request.query.grupos + '])');
  if (request.query.data_ini) crud.filtros.push({ campo: 'eventos.event_start', operador: '>=', valor: moment(request.query.data_ini).format('YYYY-MM-DDTHH:mm:ss') + 'Z' });
  if (request.query.data_fim) crud.filtros.push({ campo: 'eventos.event_start', operador: '<=', valor: moment(request.query.data_fim).format('YYYY-MM-DDTHH:mm:ss') + 'Z' });
  if (request.query.order) crud.order = request.query.order;

  crud.leftjoin = [
    { tabela: "event_group AS grupo", campo1: "eventos.envent_group", campo2: "grupo.id" }
  ];

  crud.select = [
    'eventos.*',
    'grupo.name as grupo_nome',
    'grupo.icon as grupo_icone'
  ];

  crud.getAll(request, reply);
};

exports.returnEvents = async (request) => {
  const crud = new CRUD('public.queue_events AS eventos');
  crud.filtrosraw = [];
  crud.filtros = [];
  if (request.query.eventos) crud.filtrosraw.push('eventos.event = ANY(ARRAY ' + JSON.stringify(request.query.eventos).replace(/"/g, "'") + ')');
  if (request.query.filas) crud.filtrosraw.push('eventos.queue = ANY(ARRAY ' + JSON.stringify(request.query.filas).replace(/"/g, "'") + ')');
  if (request.query.agentes) crud.filtrosraw.push('eventos.agent = ANY(ARRAY ' + JSON.stringify(request.query.agentes).replace(/"/g, "'") + ')');
  if (request.query.grupos) crud.filtrosraw.push('eventos.envent_group = ANY(ARRAY[' + request.query.grupos + '])');
  if (request.query.data_ini) crud.filtros.push({ campo: 'eventos.event_start', operador: '>=', valor: moment(request.query.data_ini).format('YYYY-MM-DDTHH:mm:ss') + 'Z' });
  if (request.query.data_fim) crud.filtros.push({ campo: 'eventos.event_start', operador: '<=', valor: moment(request.query.data_fim).format('YYYY-MM-DDTHH:mm:ss') + 'Z' });
  if (request.query.order) crud.order = request.query.order;

  crud.leftjoin = [
    { tabela: "event_group AS grupo", campo1: "eventos.envent_group", campo2: "grupo.id" }
  ];

  crud.select = [
    'eventos.*',
    'grupo.name as grupo_nome',
    'grupo.icon as grupo_icone'
  ];

  const result = await crud.returnAll(request);
  return result;
};

exports.returnAll = async (request) => {
  const max = request.max ? request.max : 15;
  const page = request.page ? request.page : 1;
  const knex = db.knex('public.v_agentes_full');
  if (request.nome) knex.whereRaw('LOWER(fullname) LIKE ?', '%' + decodeURIComponent(request.nome).toLowerCase() + '%');
  if (request.ramal && request.ramal.length > 0) knex.where('ramal', request.ramal);

  knex.clearSelect();
  if (request.order) {
    for (let p = 0; p < request.order.length; ++p) {
      if (request.order[p] && request.order[p].campo && request.order[p].ordem) {
        knex.orderBy(request.order[p].campo, request.order[p].ordem);
      }
      else if (request.order[p] && request.order[p].campo) {
        knex.orderBy(request.order[p].campo, 'ASC');
      }
    }
  }
  const ret = {};
  await knex.select('id', 'nome as login,', 'fullname as nome', 'ramal', 'filas')
    .then(function (rows) {
      for (let l = 0; l < rows.length; ++l) {
        for (let i = 0; i < rows[l].filas.length; ++i) {
          const agente = Ami.getAgenteLogadoByFilaRamal(rows[l].filas[i], rows[l].ramal);
          const fila = {
            fila: rows[l].filas[i],
            status: {
              codigo: agente && agente.status ? agente.status : 5,
              StatusNome: agente && agente.StatusNome ? agente.StatusNome : 'Indiponivel',
              DataStatus: agente && agente.DataStatus ? agente.DataStatus : moment.tz(config.servidor.timezone).format(),
              pausa: agente && agente.pausa ? agente.pausa : 0,
              pausa_motivo: agente && agente.pausa_motivo ? agente.pausa_motivo : undefined
            }
          };
          rows[l].filas[i] = fila;
        }
      }

      ret.error = false;
      ret.full = true;
      ret.total = rows.length;
      ret.message = 'Lista de todos os agentes';
      ret.dados = rows;
    })
    .catch(function (error) {
      console.error(error);
      return (Boom.badData(error));
    });
  return ret;
};

exports.schema = {
  agente: joi.object().keys({
    id: joi
      .number()
      .required()
      .integer()
      .min(0)
      .description('id do agente')
      .example(1),
    nome: joi
      .string()
      .required()
      .description('nome para login')
      .example('teste'),
    senha: joi
      .string()
      .required()
      .description('senha para login')
      .example('teste'),
    ramal: joi
      .any()
      .description('ramal do agente')
      .example('SIP/1000'),
    token: joi
      .any()
      .description('token de acesso')
      .example('asddsfdsgffdgfdghsdferf'),
    fila: joi
      .any()
      .description('fila (Não é mais usado)')
      .example('nda'),
    token_create: joi
      .any()
      .description('data de criacção do token')
      .example('nda'),
    datastatus: joi
      .any()
      .description('data de criacção do token')
      .example('nda'),
    status: joi
      .any()
      .description('status')
      .example('nda'),
    fullname: joi
      .string()
      .required()
      .description('nome completo do agente')
      .example('Teste da silva'),
    modo: joi
      .string()
      .required()
      .valid('F', 'T')
      .description('Modo de operação: F = Fila Unica, T = Multi fila.')
      .example('F'),
    filas: joi
      .array()
      .items(
        joi
          .any()
          .description('nome da fila')
          .example('Atendimento')
      )
  }).label('agente'),
  // Using a getter here so that I can refer to `this`, in reference to the character node of the schema object
  get agentes () {
    return joi
      .array()
      .items(this.agente)
      .label('lista_de_agentes')
  }
};
