'use strict'
const joi = require('joi');
const Boom = require('boom');
const db = require('../../util/db');
const Ami = require('../../ami');
const config = require('../../config.json');
const moment = require('moment-timezone');
const Agentes = require('../../models/Agente');
moment.tz.setDefault(config.servidor.timezone);

exports.getOne = (request, reply) => {
  const knex = db.knex('public.ligacoes');
  knex.where('callid', request.params.CallID);
  knex.select('*')
    .then(function (rows) {
      if (rows && rows.length > 0) {
        const ret = {};
        ret.error = false;
        ret.full = true;
        ret.total = rows.length;
        ret.message = 'Dados da ligação Callid = ' + request.params.CallID;
        ret.dados = rows[0];
        reply(ret);
      }
      else {
        const ret = {};
        ret.error = true;
        ret.total = 0;
        ret.message = 'A ligação Callid = ' + request.params.CallID + ' não foi encontrada na base, talvez ainda esteja em curso ou não existe.';
        reply(ret);
      }
    })
    .catch(function (error) {
      console.error(error);
      reply(Boom.badData(error));
    });
};

exports.discar = (request, reply) => {
  Ami.Discar(request.payload, (resposta) => {
    const ret = {};
    ret.error = resposta.response === 'Error' ? true : false;
    ret.callid = resposta.response === 'Error' ? undefined : resposta.callid;
    ret.message = resposta.message;
    reply(ret);
  });
};

exports.desligar = (request, reply) => {
  try {
    const knex = db.knex('public.ligacoes');
    knex.where('callid', request.params.CallID);
    knex.select('*')
      .then(async function (rows) {
        if (rows && rows.length > 0) {
          const ret = {};
          ret.error = false;
          ret.full = true;
          ret.total = rows.length;
          ret.message = 'Dados da ligação Callid = ' + request.payload.callId;
          ret.dados = rows[0];
          //reply(ret);
          let agente = await Agentes.query.require(true).first()
            .where({ fullname: ret.dados.agent })
            .fetch();
          Ami.Desligar({ destino: agente.ramal }, (resposta) => {
            reply(resposta);
          });

        }
        else {
          const ret = {};
          ret.error = true;
          ret.total = 0;
          ret.message = 'A ligação Callid = ' + request.params.callId + ' não foi encontrada na base, talvez ainda esteja em curso ou não existe.';
          reply(ret);
        }
      })
      .catch(function (error) {
        console.error(error);
        reply(Boom.badData(error));
      });
  } catch (error) {
    reply(Boom.badData(error));
  }

};
