'use strict'

const Joi = require('joi');
const handlers = require('./handlers');

module.exports = [

  {
    method: 'GET',
    path: '/api/v2/apiary/{CallID}',
    handler: handlers.getOne,
    config: {
      description: 'Obtem os dados de uma ligação pelo callid',
      tags: ['api', 'v2', 'Apiary'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          CallID: Joi.string().required().description('Callid da ligação')
        }
      }
    }
  },

  {
    method: 'POST',
    path: '/api/v2/apiary/efetuar',
    handler: handlers.discar,
    config: {
      description: 'Efetua uma ligação',
      tags: ['api', 'v2', 'Apiary'],
      validate: {
        payload: Joi.object({
          caller: Joi.string().required().example('SIP/1000').description('Ramal de origem.'),
          called: Joi.string().required().description('Destino').example('05130305500'),
          rota: Joi.string().required().description('Rota de saída. Grupo de rota informado do cadastro do Ramal.').example('Ramais'),
        })
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v2/apiary/desligar',
    handler: handlers.desligar,
    config: {
      description: 'Desliga uma ligação pelo ramal ou pelo destino',
      tags: ['api', 'v2', 'Apiary'],
      validate: {
        payload: Joi.object({
          callId: Joi.string().required().description('call id gerado pela ligacao').example('123tesste'),
          causa: Joi.number().integer().description('Causa da desconexão').default(16).example(16)
        })
      }
    }
  }
];
