'use strict'

const joi = require('joi');
const Boom = require('boom');
const fs = require('fs');
const { exec } = require('child_process');
const db = require('../../util/db');
const config = require('../../config.json');
const moment = require('moment-timezone');
moment.tz.setDefault(config.servidor.timezone);
var namiLib = require("nami");
const Ami = require('../../ami');

const deleteFolderRecursive = function (path) {
  fs.readdirSync(path).forEach(function (file, index) {
    var curPath = path + "/" + file;
    if (fs.lstatSync(curPath).isDirectory()) { // recurse
      deleteFolderRecursive(curPath);
    } else { // delete file
      fs.unlinkSync(curPath);
    }
  });
  fs.rmdirSync(path);
};

exports.delete_dir = (request, reply) => {
  db.knex('public.musicas_espera')
    .where('directory', request.query.dir_name)
    .del()
    .then(function (rows) {
      if (rows > 0) {
        if (fs.existsSync(request.query.dir_name)) {
          try {
            deleteFolderRecursive(request.query.dir_name);
            reply({ error: false, message: 'Diretório apagado.' });
          }
          catch (e) {
            reply(Boom.badData(e));
          }
        }
        else {
          reply(Boom.badData('Diretório não existe'));
        }
      }
      else {
        reply(Boom.badData("Nenhum resgistro encontrado para diretório = " + request.query.dir_name));
      }
    })
    .catch(function (error) {
      console.error(error);
      reply(Boom.badData(error));
    });
};

exports.delete_audio = (request, reply) => {
  if (fs.existsSync(request.query.file_name)) {
    try {
      fs.unlinkSync(request.query.file_name);
      reply({ error: false, message: 'Arquivo apagado.' });
    }
    catch (e) {
      reply(Boom.badData(e));
    }
  }
  else {
    reply(Boom.badData('Arquivo não existe'));
  }
};

exports.create_dir = (request, reply) => {
  const nome = request.payload.name.replace(/[^0-9a-zA-Z\xC0-\xFF \-]/g, '').replace(/\s/g, "_");
  const dir = '/opt/xcontact/audios/' + nome;
  const campos = {
    name: nome,
    sort: request.payload.sort,
    mode: 'files',
    directory: dir
  };
  if (!fs.existsSync(dir)) {
    fs.mkdirSync(dir);
  }

  db.knex('public.musicas_espera')
    .insert(campos)
    .then(function (rows) {
      const ret = {
        error: false,
        name: nome,
        directory: dir,
        message: 'Diretório ' + nome + ' criado com sucesso.'
      };
      reply(ret);
    })
    .catch(function (error) {
      console.error(error);
      reply(Boom.badData(error));
    });
};

exports.getall = (request, reply) => {
  const max = request.query.max ? request.query.max : 15;
  const page = request.query.page ? request.query.page : 1;
  const knex = db.knex('public.musicas_espera');
  if (request.query.nome) knex.whereRaw('LOWER(name) LIKE ?', '%' + decodeURIComponent(request.query.nome).toLowerCase() + '%');
  if (max === -1) {
    //console.log('SQL = '+ knex.toString());
    knex.clearSelect();
    // Parametros de ordenação, fora do count
    if (request.query.order) {
      for (let p = 0; p < request.query.order.length; ++p) {
        if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
          knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
        }
        else if (request.query.order[p] && request.query.order[p].campo) {
          knex.orderBy(request.query.order[p].campo, 'ASC');
        }
      }
    }
    knex.select('*')
      .then(function (rows) {
        const ret = {};
        ret.error = false;
        ret.full = true;
        ret.total = rows.length;
        ret.message = 'Lista de todos os diretórios';
        for (let i = 0; i < rows.length; ++i) {
          rows[i].arquivos = [];
          fs.readdirSync(rows[i].directory).forEach(file => {
            rows[i].arquivos.push(rows[i].directory + '/' + file);
          });
        }
        ret.dados = rows;
        reply(ret);
      })
      .catch(function (error) {
        console.error(error);
        reply(Boom.badData(error));
      });
  }
  else {
    const offset = max * (page - 1);
    //console.log('SQL = '+knex.count('* as total').toString());
    knex.count('* as total')
      .then(function (total) {
        if (total[0].total > 0) {
          knex.clearSelect();
          // Parametros de ordenação, fora do count
          if (request.query.order) {
            for (let p = 0; p < request.query.order.length; ++p) {
              if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
              }
              else if (request.query.order[p] && request.query.order[p].campo) {
                knex.orderBy(request.query.order[p].campo, 'ASC');
              }
            }
          }
          //console.log('SQL = '+knex.select('*').limit(max).offset(offset).toString());
          knex.select('*').limit(max).offset(offset).then(function (rows) {
            const pages = Math.ceil(parseInt(total[0].total) / max);
            const ret = {};
            ret.error = false;
            ret.full = pages === 1 ? true : false;
            ret.page = page;
            ret.max = max;
            ret.offset = offset;
            ret.total = parseInt(total[0].total);
            ret.pages = pages;
            ret.message = 'Lista de diretórios paginados';
            for (let i = 0; i < rows.length; ++i) {
              rows[i].arquivos = [];
              fs.readdirSync(rows[i].directory).forEach(file => {
                rows[i].arquivos.push(rows[i].directory + '/' + file);
              });
            }
            ret.dados = rows;
            reply(ret);
          })
            .catch(function (error) {
              console.error(error);
              reply(Boom.badData(error));
            });
        }
        else {
          const ret = {};
          ret.error = true;
          ret.total = 0;
          ret.message = 'Nenhum registro encontrado';
          reply(ret);
        }
      })
      .catch(function (error) {
        console.error(error);
        reply(Boom.badData(error));
      });
  }
};

exports.getalldir = (request, reply) => {
  const max = request.query.max ? request.query.max : 15;
  const page = request.query.page ? request.query.page : 1;
  const knex = db.knex('public.musicas_espera');

  if (request.query.nome) knex.whereRaw('LOWER(name) LIKE ?', '%' + decodeURIComponent(request.query.nome).toLowerCase() + '%');

  if (max === -1) {
    //console.log('SQL = '+ knex.toString());
    knex.clearSelect();
    // Parametros de ordenação, fora do count
    if (request.query.order) {
      for (let p = 0; p < request.query.order.length; ++p) {
        if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
          knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
        }
        else if (request.query.order[p] && request.query.order[p].campo) {
          knex.orderBy(request.query.order[p].campo, 'ASC');
        }
      }
    }
    knex.select('*')
      .then(function (rows) {
        const ret = {};
        ret.error = false;
        ret.full = true;
        ret.total = rows.length;
        ret.message = 'Lista de todos os diretórios';
        ret.dados = rows;
        reply(ret);
      })
      .catch(function (error) {
        console.error(error);
        reply(Boom.badData(error));
      });
  }
  else {
    const offset = max * (page - 1);
    //console.log('SQL = '+knex.count('* as total').toString());
    knex.count('* as total')
      .then(function (total) {
        if (total[0].total > 0) {
          knex.clearSelect();
          // Parametros de ordenação, fora do count
          if (request.query.order) {
            for (let p = 0; p < request.query.order.length; ++p) {
              if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
              }
              else if (request.query.order[p] && request.query.order[p].campo) {
                knex.orderBy(request.query.order[p].campo, 'ASC');
              }
            }
          }
          //console.log('SQL = '+knex.select('*').limit(max).offset(offset).toString());
          knex.select('*').limit(max).offset(offset).then(function (rows) {
            const pages = Math.ceil(parseInt(total[0].total) / max);
            const ret = {};
            ret.error = false;
            ret.full = pages === 1 ? true : false;
            ret.page = page;
            ret.max = max;
            ret.offset = offset;
            ret.total = parseInt(total[0].total);
            ret.pages = pages;
            ret.message = 'Lista de diretórios paginados';
            ret.dados = rows;
            reply(ret);
          })
            .catch(function (error) {
              console.error(error);
              reply(Boom.badData(error));
            });
        }
        else {
          const ret = {};
          ret.error = true;
          ret.total = 0;
          ret.message = 'Nenhum registro encontrado';
          reply(ret);
        }
      })
      .catch(function (error) {
        console.error(error);
        reply(Boom.badData(error));
      });
  }
};

exports.getfiles = (request, reply) => {
  db.knex('public.musicas_espera')
    .where('name', request.params.DiretorioNome)
    .select('*')
    .then(function (rows) {
      if (rows.length > 0) {
        const arquivos = [];
        fs.readdirSync(rows[0].directory).forEach(file => {
          arquivos.push(rows[0].directory + '/' + file);
        });
        reply(arquivos);
      }
      else {
        reply(Boom.badData("Diretório não encontrado"));
      }
    })
    .catch(function (error) {
      fs.unlinkSync(path);
      console.error(error);
      reply(Boom.badData(error));
    });
};

exports.upload = (request, reply) => {
  const data = request.payload;
  if (data.file) {
    db.knex('public.musicas_espera')
      .where('name', request.params.DiretorioNome)
      .select('*')
      .then(function (rows) {
        if (rows.length > 0) {
          const name = data.file.hapi.filename.replace(/[^0-9a-zA-Z\xC0-\xFF \-]/g, '').replace(/\s/g, "_");
          const path = rows[0].directory + "/" + name;
          const file = fs.createWriteStream(path);

          file.on('error', function (err) {
            console.error(err);
            fs.unlinkSync(path);
            reply(Boom.badData(err));
          });
          data.file.pipe(file);

          data.file.on('end', function (err) {
            // Converter
            if (data.file.hapi.headers['content-type'] === 'audio/mp3' || data.file.hapi.headers['content-type'] === 'audio/mpeg') {
              exec('/opt/xcontact/mp32wav ' + path, (err, stdout, stderr) => {
                if (err) {
                  fs.unlinkSync(path);
                  reply(Boom.badData(err));
                  return;
                }
                // Log da conversão
                console.log(`stdout: ${stdout}`);
                console.log(`stderr: ${stderr}`);
                const n = path.lastIndexOf(".");
                const caminho = n > 0 ? path.substr(0, n) + '_xc.wav' : path + '_xc.wav';

                const audio = {
                  nome: name,
                  caminho: caminho,
                  diretorio: request.params.DiretorioNome,
                  msg: 'Arquivo salvo com sucesso.'
                };
                reply(audio);
              });
            }
            else if (data.file.hapi.headers['content-type'] === 'audio/wav') {
              const audio = {
                error: false,
                tipo: data.file.hapi.headers['content-type'],
                msg: 'Arquivo salvo com sucesso.'
              };
              reply(audio);
            }
            else {
              fs.unlinkSync(path);
              reply(Boom.badData("Desculpe, só aceito arquivos de áudio mp3 ou wav"));
            }
          });
        }
        else {
          fs.unlinkSync(path);
          reply(Boom.badData("Diretório não encontrado para Nome = " + request.params.DiretorioNome));
        }
      })
      .catch(function (error) {
        fs.unlinkSync(path);
        console.error(error);
        reply(Boom.badData(error));
      });
  }
  else {
    reply(Boom.badData("Nenhum arquivo enviado"));
  }
};
