'use strict'

const Joi = require('joi');
const handlers = require('./handlers');

module.exports = [
  {
    method: 'GET',
    path: '/api/v2/audios',
    handler: handlers.getall,
    config: {
      description: 'Obtem os áudios',
      tags: ['api','v2','Áudios'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
                campo: Joi.string().description('Nome da coluna'),
                ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          nome: Joi.string().description('Filtra por nome')
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/audios/diretorios',
    handler: handlers.getalldir,
    config: {
      description: 'Obtem os diretórios de áudios.',
      tags: ['api','v2','Áudios'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
                campo: Joi.string().description('Nome da coluna'),
                ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          nome: Joi.string().description('Filtra por nome')
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/audios/{DiretorioNome}',
    handler: handlers.getfiles,
    config: {
      description: 'Obtem os arquivos de áudio de um diretório.',
      tags: ['api','v2','Áudios'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            DiretorioNome: Joi.string().required().description('Nome do diretório.')
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v2/audio/upload/{DiretorioNome}',
    handler: handlers.upload,
    config: {
      description: 'Envia um arquivo de aúdio para um diretório',
      tags: ['api','v2','Áudios'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      payload: {
          output: 'stream',
          allow: 'multipart/form-data',
          maxBytes: 104857600, // 100mb
          parse: true,
      },
      validate: {
        params: {
            DiretorioNome: Joi.string().required().description('Nome do diretório')
        },
        payload: {
          file: Joi.object().unknown().description('Arquivo')
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v2/audio/diretorio',
    handler: handlers.create_dir,
    config: {
      description: 'Cria um diretório de áudio.',
      tags: ['api','v2','Áudios'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        payload: Joi.object({
          name: Joi.string().required().example('esperas').description('Nome do diretório'),
          sort: Joi.string().required().valid('random','alpha').description('Ordem de execução dos aúdios: random = Aleatório, alpha = Ordem alfabetica.')
        })
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v2/audio',
    handler: handlers.delete_audio,
    config: {
      description: 'Apaga um arquivo de áudio',
      tags: ['api','v2','Áudios'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          file_name: Joi.string().required().description('Nome completo do arquivo')
        }
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v2/audio/diretorio',
    handler: handlers.delete_dir,
    config: {
      description: 'Apaga um arquivo de áudio',
      tags: ['api','v2','Áudios'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          dir_name: Joi.string().required().description('Nome completo do diretorio')
        }
      }
    }
  },
];
