'use strict'
const Joi = require('joi');
const CRUD = require('../../util/CRUD');
const md5 = require('md5');
const db = require('../../util/db');
const Boom = require('boom');
const handlers = require('./handlers');

module.exports = [
  {
    method: 'GET',
    path: '/api/v2/autocomplete',
    handler: (request, reply) => {
      const crud = new CRUD('queueAutoComplete AS c');
      // Ativa/Desativa Knexnest
      crud.knexnest_op = request.query.FullMsg;
      crud.filtros = [];
      crud.filtrosraw = [];
      if (request.query.order) crud.order = request.query.order;
      // if (request.query.queue) crud.filtros.push({ campo: '"queue"', operador: 'like', valor: '%' + request.query.queue + '%' })

      if (request.query.nome) crud.filtrosraw.push("LOWER(nome) LIKE '%" + decodeURIComponent(request.query.nome).toLowerCase() + "%'");
      if (request.query.texto) crud.filtrosraw.push("LOWER(texto) LIKE '%" + decodeURIComponent(request.query.texto).toLowerCase() + "%'");
      crud.leftjoin = [{ tabela: "queue_table AS qt", campo1: "qt.name", campo2: "c.queue" }];
      if (request.query.queue) crud.filtrosraw.push("(c.queue is null OR c.queue ='" + request.query.queue + "')");

      crud.getAll(request, reply);
    },
    config: {
      description: 'Pesquisa uma mensagem automática do banco através da string enviada',
      tags: ['api', 'v2', 'AutoComplete'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
              campo: Joi.string().description('Nome da coluna'),
              ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          nome: Joi.string().allow(null).description('Filtra pelo nome do autocomplete'),
          texto: Joi.string().allow(null).description('Filtra por texto'),
          queue: Joi.string().allow(null).description('queue'),
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v2/autocomplete',
    handler: handlers.create,
    config: {
      description: 'Cria uma mensagem pré definida',
      tags: ['api', 'v2', 'AutoComplete'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        payload: Joi.object({
          nome: Joi.string().required().example('teste').description('Nome da mensagem pré definida'),
          texto: Joi.string().description('Texto da mensagem').example('Alguma coisa'),
          queue: Joi.string().allow(null).description('queue'),
        })
      }
    }
  },

  {
    method: 'PUT',
    path: '/api/v2/autocomplete/{ID}',
    handler: handlers.update,
    config: {
      description: 'Atualiza uma mensagem pré definida',
      tags: ['api', 'v2', 'AutoComplete'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        },
        payload: Joi.object({
          nome: Joi.string().required().example('teste').description('Nome da mensagem pré definida'),
          texto: Joi.string().required().description('Texto da mensagem').example('Alguma coisa'),
          queue: Joi.string().allow(null).description('queue'),
        })
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v2/autocomplete/{ID}',
    handler: handlers.delete,
    config: {
      description: 'Deleta uma mensagem pré definida',
      tags: ['api', 'v2', 'AutoComplete'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        }
      }
    }
  }
];
