const Boom = require('boom');
const CRUD = require('../../util/CRUD');
const db = require('../../util/db');
const Paginacao = require('../../util/Paginacao');
const config = require('../../config.json');
const restbase = require('../../util/restbase');
const moment = require('moment-timezone');
moment.tz.setDefault(config.servidor.timezone);
const Atendimento = require('../../xc_modules/omni/class/Atendimento');
const AtendimentoMensagem = require('../../models/AtendimentoMensagem');
const { saveMsg } = require('../../xc_modules/omni');
const log = require('../../util/Log');
const Handlers = require('../../ws/handlers');

const MensagemAnexos = require('../../models/MensagemAnexos');
const RepliedMsg = require('../../models/RepliedMsg');
const RepliedAnexos = require('../../models/RepliedAnexos');
const ClienteAtendimento = require('../../models/ClienteAtendimento');
const AtendimentoClassificacao = require('../../models/AtendimentoClassificacao');
const Classificacao = require('../../models/Classificacao');
const Subclassificacao = require('../../models/Subclassificacao');
const AtendimentoEventosAgentes = require('../../models/AtendimentoEventosAgentes');


exports.getMensagens = async (request, reply) => {
	try {
		const query = AtendimentoMensagem.query;
		query.where({ '"queueChatMessages"."idQueueChat"': request.params.ID });

		if (request.query.agentes) query.where(query.sql('agente = ANY(ARRAY$1)', [JSON.stringify(request.query.agentes).replace(/"/g, "'")]));
		if (request.query.texto) query.where(query.sql("LOWER(texto) LIKE '%$1%'", [decodeURIComponent(request.query.texto).toLowerCase()]));
		if (request.query.inOut) query.where({ inOut: request.query.inOut });
		if (request.query.dataIni) query.where.gte({ dataHora: request.query.dataIni });
		if (request.query.dataFim) query.where.lte({ dataHora: request.query.dataFim });

		query.orderBy('id');

		query.leftJoin(MensagemAnexos);
		query.leftJoin(
			RepliedMsg.query
				.leftJoin(RepliedAnexos)
		);

		var registros = await query.require(false).fetch();
		reply({
			dados: registros,
			error: registros.length <= 0 ? true : false,
			full: true,
			message: "Listagem full",
			total: registros.length
		});
	} catch (e) {
		reply(Boom.badData(e));
	}
};

exports.returnAllChats = async (request) => {
	const crud = new CRUD('queueChat AS c');
	// Ativa/Desativa Knexnest
	crud.knexnest_op = request.FullMsg;

	crud.filtrosraw = [];
	crud.filtros = [];

	if (request.id) crud.filtros.push({ campo: 'c.id', operador: '=', valor: request.id });
	if (request.fromName) crud.filtrosraw.push("LOWER(\"c\".\"fromName\") LIKE '%" + decodeURIComponent(request.fromName).toLowerCase() + "%'");
	if (request.assunto) crud.filtrosraw.push("LOWER(c.assunto) LIKE LIKE '%" + decodeURIComponent(request.assunto).toLowerCase() + "%'");
	if (request.action) crud.filtros.push({ campo: 'c.action', operador: '=', valor: request.action });
	if (request.ativo !== undefined) crud.filtros.push({ campo: 'c.ativo', operador: '=', valor: request.ativo });
	if (request.filas) crud.filtrosraw.push('c.queue = ANY(ARRAY' + JSON.stringify(request.filas).replace(/"/g, "'") + ')');
	if (request.agentes) crud.filtrosraw.push('"c"."ultimoAgente" = ANY(ARRAY' + JSON.stringify(request.agentes).replace(/"/g, "'") + ')');
	if (request.dataIni) crud.filtros.push({ campo: '"c"."dataIni"', operador: '>=', valor: request.dataIni });
	if (request.dataFim) crud.filtros.push({ campo: '"c"."dataIni"', operador: '<=', valor: request.dataFim });
	if (request.sentido) crud.filtrosraw.push('c.sentido = ANY(ARRAY' + JSON.stringify(request.sentido).replace(/"/g, "'") + ')');
	if (request.queueTypeID) crud.filtrosraw.push('"c"."queueTypeID" = ANY(ARRAY' + JSON.stringify(request.queueTypeID).replace(/"/g, "'") + ')');
	if (request.idCliente) crud.filtrosraw.push('"c"."idCliente" = ANY(ARRAY[' + request.idCliente + '])');

	// Filtro Aguardando não existe, é o status = Entrou com ultimoAgente null      
	if (request.status && request.status.includes('Aguardando')) {
		request.status = request.status.map(x => x.replace('Aguardando', 'Entrou'));

		crud.filtrosraw.push('c.status = ANY(ARRAY' + JSON.stringify(request.status).replace(/"/g, "'") + ')');
		crud.filtrosraw.push('"c"."ultimoAgente" is null');
	}
	else if (request.status && request.status.includes('Entrou') && !request.agentes) {
		crud.filtrosraw.push('c.status = ANY(ARRAY' + JSON.stringify(request.status).replace(/"/g, "'") + ')');
		crud.filtrosraw.push('"c"."ultimoAgente" is not null');
	}
	else if (request.status) crud.filtrosraw.push('c.status = ANY(ARRAY' + JSON.stringify(request.status).replace(/"/g, "'") + ')');

	// if (request.classificacoes) crud.filtrosraw.push(/*sql*/`"c"."classificacoes" = ANY(ARRAY[${request.classificacoes.map(x => "'" + x + "'").join(", ")}])`);
	if (request.classificacoes) crud.filtrosraw.push(/*sql*/`"qcc"."classificacaoID" = ANY(ARRAY[${request.classificacoes.join(", ")}])`);

	if (request.order) crud.order = request.query.order;


	crud.leftjoin = [
		{ tabela: "queueChatClassificacao AS qcc", campos: ["qcc.queueChatID", "c.id", "qcc.agente", "c.ultimoAgente"] },
		{ tabela: "classificacoes AS cla", campo1: "qcc.classificacaoID", campo2: "cla.id" },
		{ tabela: "subclassificacao AS subcla", campo1: "qcc.subClassificacaoID", campo2: "subcla.id" },
		{ tabela: "cliente AS cli", campo1: "cli.id", campo2: "c.idCliente" },
	];

	crud.select = [
		'c.*',
		'cla.nome as classificacao',
		'subcla.nome as subclassificacao',
		'qcc.obs as obs',
		'cli.profileImage',
		'cli.nome as clienteNome',
		'cli.id as idCliente',
		'cli.numero as numero',

	];

	if (crud.knexnest_op) {
		crud.innerjoin = [
			{ tabela: "queueChatMessages AS m", campo1: "m.idQueueChat", campo2: "c.id" }
		];
		crud.leftjoin = [
			{ tabela: "queueChatClassificacao AS qcc", campos: ["qcc.queueChatID", "c.id", "qcc.agente", "c.ultimoAgente"] },
			{ tabela: "classificacoes AS cla", campo1: "qcc.classificacaoID", campo2: "cla.id" },
			{ tabela: "subclassificacao AS subcla", campo1: "qcc.subClassificacaoID", campo2: "subcla.id" },
			{ tabela: "cliente AS cli", campo1: "cli.id", campo2: "c.idCliente" },
			{ tabela: "queueAnexos AS anexo", campo1: "m.id", campo2: "anexo.id_message" }
		];
		crud.select = [
			'c.id as _id',
			'c.sentido as _sentido',
			'c.fromID as _fromID',
			'c.fromName as _fromName',
			'c.assunto as _assunto',
			'c.action as _action',
			'c.ativo as _ativo',
			'c.queue as _queue',
			'c.ultimoAgente as _ultimoAgente',
			'c.dataIni as _dataIni',
			'c.dataFim as _dataFim',
			'c.status as _status',
			'c.queueTypeID as _queuetype',
			'c.linkedid as _linkedid',
			'c.idCliente as _idCliente',
			'cli.numero as _numero',
			'cla.nome as _classificacao',
			'subcla.nome as _subclassificacao',
			'qcc.obs as _obs',
			'm.id as _queueChatMessages__id',
			'm.dataHora as _queueChatMessages__dataHora',
			'm.inOut as _queueChatMessages__inOut',
			'm.agente as _queueChatMessages__agente',
			'm.texto as _queueChatMessages__texto',
			'm.status as _queueChatMessages__status ',
			'm.repliedIdMsg as _queueChatMessages__repliedIdMsg',
			'cli.profileImage as _image'

		];
	}
	const ret = await crud.returnAll({ query: { ...request } });
	return ret
}

exports.classifica = (request, reply) => {
	const knex = db.knex;
	request.payload.queueChatID = request.params.ID;
	knex.transaction(function (t) {
		return knex('queueChatClassificacao')
			.transacting(t)
			.where('queueChatID', request.params.ID)
			.where('agente', request.payload.agente)
			.del()
			.then(function () {
				return db.knex('queueChatClassificacao')
					.transacting(t)
					.insert(request.payload)
					.then(() => {
					})
					.catch(function (e) {
						t.rollback;
						throw e;
					}
					)
			})
			.then(t.commit)
			.catch(function (e) {
				throw e;
			})
			.then(function () {
				const ret = {};
				ret.error = false;
				ret.message = 'Chat Classificado';
				reply(ret);
			})
			.catch(function (e) {
				t.rollback();
				reply(Boom.badData(e));
			});
	});
};

exports.fluxo = (request, reply) => {
	const knex = db.knex('public.chat_fluxo');

	if (request.query.data_ini) knex.where('data', '>=', moment(request.query.data_ini).format('YYYY-MM-DDTHH:mm:ss') + 'Z');
	if (request.query.data_fim) knex.where('data', '<=', moment(request.query.data_fim).format('YYYY-MM-DDTHH:mm:ss') + 'Z');
	if (request.query.id_client) knex.whereRaw(restbase.buildObjectSearchExpression("id_client", request.query.id_client));
	if (request.query.menu) knex.whereRaw(restbase.buildObjectSearchExpression('menu', request.query.menu));
	if (request.query.opcao) knex.whereRaw(restbase.buildObjectSearchExpression('opcao', request.query.opcao));
	if (request.query.resposta) knex.whereRaw(restbase.buildObjectSearchExpression('resposta', request.query.resposta));

	knex.orderBy("data", "ASC");

	knex.select('*')
		.then(function (rows) {
			const ret = {};
			ret.error = false;
			ret.full = true;
			ret.total = !rows ? 0 : rows.length;
			ret.message = 'Fluxo de Chat';
			ret.dados = rows;
			reply(ret);
		})
		.catch(function (error) {
			console.error(error);
			reply(Boom.badData(error));
		});
};
exports.transferirParaFila = async (request, reply) => {
	try {
		const filaDestino = request.payload.fila;
		for (let protocolo of request.payload.atendimentosId) {
			let atendimento = await Atendimento.query.first().where({ id: protocolo }).fetch();
			const dados = {
				protocolo: protocolo,
				nome: null,
				queue: filaDestino ? filaDestino : atendimento.queue,
				queueFrom: atendimento.queueFrom ? atendimento.queueFrom : atendimento.queue
			}
			Handlers.TransfereAtendimentoParaFila(null, dados);

		}
		return reply({
			message: "OK",
			error: false
		});

	} catch (error) {
		reply(Boom.badData(error));
		console.error(error)
	}
}
exports.transferir = async (request, reply) => {
	try {
		const agenteDestinatario = request.params.agente;
		const filaDestino = request.payload.fila;
		for (let protocolo of request.payload.atendimentosId) {
			let atendimento = await Atendimento.query.first().where({ id: protocolo }).fetch();
			const dados = {
				protocolo: protocolo,
				nome: agenteDestinatario,
				queue: filaDestino ? filaDestino : atendimento.queue,
				queueFrom: atendimento.queueFrom ? atendimento.queueFrom : atendimento.queue
			}
			Handlers.TransferenciaAtendimento(null, dados);
		}
		return reply({
			message: "OK",
			error: false
		});

	} catch (error) {
		reply(Boom.badData(error));
		console.error(error)
	}
}
exports.transferirParaAgente = async (request, reply) => {
	try {
		log.debug(`[API][CHAT][TRANSFERIR_PARA_AGENTE] Início da rota!`);
		const agenteDestinatario = request.params.agente;
		const filaDestino = request.payload.fila;

		// Para cada protocolo provido, tranfere para a fila e agente especificados
		for (let protocolo of request.payload.atendimentosId) {
			let atendimento = await Atendimento.query.first().where({ id: protocolo }).fetch();

			if (atendimento) {
				atendimento.id = protocolo;

				// Salva o agente remetente
				let agenteRemetente = atendimento.ultimoAgente;
				// Altera o agente para o destinatário
				atendimento.ultimoAgente = agenteDestinatario;

				log.debug(`[API][CHAT][TRANSFERIR_PARA_AGENTE] Transferindo de ${agenteRemetente} para ${agenteDestinatario}`);

				// Salva o atendimento
				log.debug(`[API][CHAT][TRANSFERIR_PARA_AGENTE] Salvando o atendimento`);
				await new Atendimento(atendimento).save();

				// Caso a fila possua respostas automáticas
				log.debug(`[API][CHAT][TRANSFERIR_PARA_AGENTE] Buscando por respostas automáticas`);
				let query = await db.query(/* sql */`SELECT texto FROM public."queueRespostasAutomaticas" where "fila"=$1 AND "idRespostaTipo"=$2`, [filaDestino, 9]);
				if (query && query.rowCount > 0) {
					log.debug(`[API][CHAT][TRANSFERIR_PARA_AGENTE] Respostas automáticas -> `, query.rows);

					let texto = query.rows[0].texto;

					// Substitui as variáveis do texto
					let trocas = { "$AGENTEREMETENTE": agenteRemetente, "$AGENTEDESTINATARIO": agenteDestinatario };
					Object.entries(trocas).forEach(([key, value]) => texto = texto.split(key).join(value));

					// Busca a última mensagem
					let ultimaMensagem = await AtendimentoMensagem.query.require(true)
						.first()
						.where({ idAtendimento: atendimento.id })
						.orderBy(`id DESC`)
						.fetch();

					// Envia a mensagem
					log.debug(`[API][CHAT][TRANSFERIR_PARA_AGENTE] Salvando mensagem`);
					await saveMsg({
						idAtendimento: atendimento.id,
						texto,
						inOut: 3,
						agente: "Xcontact",
						mailMessageID: atendimento.queueTypeID === 4 && ultimaMensagem
							? ultimaMensagem.mailMessageID
							: 'Xcontact'
					}, null);
				}
			}
		}

		return reply({
			message: "OK",
			error: false
		});
	} catch (error) {
		console.error(error);
		reply(Boom.badData(error));
	}

};

exports.getAllClassificacoes = async (request, reply) => {
	try {
		const query = AtendimentoClassificacao.query;
		const dataini = request.query.data_ini ? request.query.data_ini : new Date().toDateString();
		const datafim = request.query.data_fim ? request.query.data_fim : new Date().toDateString();
		query.where(query.sql('"dataRegistro"::date BETWEEN $1 AND $2', [dataini, datafim]));
		if (request.query.hora_ini) query.where(query.sql('"dataRegistro"::time >= $1', [request.query.hora_ini]));
		if (request.query.hora_fim) query.where(query.sql('"dataRegistro"::time <= $1', [request.query.hora_fim]));

		if (request.query.filas) {
			query.innerJoin(ClienteAtendimento.query.where(ClienteAtendimento.where.in('queue', request.query.filas)))
		}
		else {
			query.innerJoin(ClienteAtendimento)
		}

		//if (request.query.data_ini) query.where(AtendimentoClassificacao.where.gte({ dataRegistro: request.query.data_ini }));
		//if (request.query.data_fim) query.where(AtendimentoClassificacao.where.lte({ dataRegistro: request.query.data_fim }));

		query.leftJoin(Classificacao);
		query.leftJoin(Subclassificacao);

		var registros = await query.require(false).fetch();
		reply({
			dados: registros,
			error: registros.length <= 0 ? true : false,
			full: true,
			message: "Listagem full",
			total: registros.length
		});
	} catch (e) {
		reply(Boom.badData(e));
	}
};

exports.getChat = async (request, reply) => {
	try {
		reply(await ClienteAtendimento.fetchById(request.params.ID));
	} catch (e) {
		reply(Boom.badData(e));
	}
};

exports.AgentesEventos = async (request, reply) => {
	try {
		const query = AtendimentoEventosAgentes.query;

		if (request.query.agentes) query.where(AtendimentoEventosAgentes.where.in('agente', request.query.agentes));
		if (request.query.filas) query.where(AtendimentoEventosAgentes.where.in('fila', request.query.filas));
		if (request.query.data_ini) query.where(AtendimentoEventosAgentes.where.gte({ dataIni: request.query.data_ini }));
		if (request.query.data_fim) query.where(AtendimentoEventosAgentes.where.lte({ dataIni: request.query.data_fim }));
		if (request.query.queueTypeID) {
			query.innerJoin(ClienteAtendimento.query.where(ClienteAtendimento.where.in('queueTypeID', request.query.queueTypeID)));
		}
		else {
			query.innerJoin(ClienteAtendimento);
		}

		reply(await query.require(false).fetch());
	} catch (e) {
		reply(Boom.badData(e));
	}
};