'use strict'

const joi = require('joi');
const Boom = require('boom');
const db = require('../../util/db');
//const squel = require("squel");
const ws = require('../../ws');
const config = require('../../config.json');
const moment = require('moment-timezone');
moment.tz.setDefault(config.servidor.timezone);
var knexnest = require('knexnest');
const Paginacao = require('../../util/Paginacao');
const Classificacao = require('../../models/Classificacao');
const Subclassificacao = require('../../models/Subclassificacao');

exports.create = async (request, reply) => {
  request.log(['classificacao', 'create'], 'Usuario criou nova classificação');
  try {
    const classificacao = await new Classificacao(request.payload).save();
    reply(classificacao);
  } catch (error) {
    reply(Boom.badData(error));
  }
};

exports.update = async (request, reply) => {
  try {
    const dados = request.payload;
    dados.id = request.params.ID;
    const classificacao = await new Classificacao(dados).save();
    reply(classificacao);
  } catch (error) {
    reply(Boom.badData(error));
  }
};

exports.delete = async (request, reply) => {
  try {
    const classificacao = await Classificacao.query
      .require(true)
      .where({ id: request.params.ID })
      .delete();
    reply(classificacao);
  } catch (error) {
    reply(Boom.badData(error));
  }
};

exports.getall = (request, reply) => {
  request.log(['classificacao', 'getAll'], 'Usuario listou todas as classificações');

  const max = request.query.max ? request.query.max : 15;
  const page = request.query.page ? request.query.page : 1;
  const pag = new Paginacao(max, page);
  const knex = db.knex('public.classificacoes AS a')
  // Isso estava aqui, mas aparentemente não consegui encontrar fundamento para o join.
  // Todas as combos de classificação na UI estavam falhando pois aquery tem id e nome em ambas as tabelas 
  // Além de replicar o retorno que gera um Invalid DUPES no Angular.
  //.leftJoin('public.subclassificacao AS b', 'b.id_classificacao', 'a.id');

  if (request.query.nome) knex.whereRaw('LOWER(a.nome) LIKE ?', '%' + decodeURIComponent(request.query.nome).toLowerCase() + '%');
  if (request.query.filas) knex.whereRaw('a.fila =  any (ARRAY' + JSON.stringify(request.query.filas).replace(/"/g, "'") + ') OR a.fila IS NULL');

  pag.order = request.query.order;
  pag.knex = knex;
  pag.getRet((ret, err) => {
    if (err) {
      reply(Boom.badData(err));
    }
    else {
      reply(ret);
    }
  });
};

exports.getOne = async (request, reply) => {
  try {
    const classificacao = await Classificacao.query
      .require(true)
      .where({ id: request.params.ID })
      .first()
      .fetch();
    reply(classificacao);
  } catch (error) {
    reply(Boom.badData(error));
  }
};

exports.getSubcla = async (request, reply) => {
  try {
    const subs = await Subclassificacao.query
      .require(false)
      .where({ idClassificacao: request.params.ID })
      .fetch();
    reply(subs);
  } catch (error) {
    reply(Boom.badData(error));
  }
};

exports.updateSubcla = async (request, reply) => {
  try {
    if (request.payload.apagar && request.payload.apagar.length > 0) {
      await Subclassificacao.query
        .require(false)
        .where(Subclassificacao.where.in('id', request.payload.apagar))
        .delete();
    }

    const sub = await Subclassificacao.save(request.payload.update);

    reply(sub);
  } catch (error) {
    reply(Boom.badData(error));
  }
};