'use strict'
const Boom = require('boom');
const Cliente = require('../../models/Cliente');
const ClienteGrupo = require('../../models/ClienteGrupo');
const ClienteNumerosAdicionais = require('../../models/ClienteNumerosAdicionais');
const ClienteCamposAdicionais = require('../../models/ClienteCamposAdicionais');
const db = require('../../util/db');
const Layout = require('../../models/Layouts');
const LayoutCampos = require('../../models/LayoutsCampos');
const csv = require("fast-csv");
const uuid = require('uuid');
const TextoUtil = require('../../util/texto');

exports.getClientes = async (request, reply) => {
  try {
    var pag = (request.query.max && request.query.max > 0) ? { page: request.query.page, perPage: request.query.max } : undefined;

    const query = Cliente.query;

    if (request.query.order && request.query.order.length > 0) {
      var order = [];
      for (let i = 0; i < request.query.order.length; i++) {
        order.push(`${request.query.order[i].campo} ${request.query.order[i].ordem}`);
      }
      query.orderBy(order);
    }

    if (request.query.searchText && typeof request.query.searchText === 'string' && request.query.searchText.length > 0) {
      query.where(
        Cliente.where.or(
          Cliente.where.ilike({ nome: `%${request.query.searchText}%` }),
          Cliente.where.ilike({ numero: `%${request.query.searchText}%` }),
          Cliente.where.ilike({ documento: `%${request.query.searchText}%` }),
          Cliente.where.ilike({ email: `%${request.query.searchText}%` }),
        )
      )
    }

    if (request.query.nome) query.where(Cliente.where.like({ nome: `%${request.query.nome}%` }));
    if (request.query.numero) query.where(Cliente.where.like({ numero: `%${request.query.numero}%` }));
    if (request.query.documento) query.where(Cliente.where.like({ documento: `%${request.query.documento}%` }));
    if (request.query.idGrupo) query.where({ idGrupo: request.query.idGrupo });

    query.leftJoin(ClienteGrupo);
    var clientes = await query.require(false).fetch(pag);
    reply(clientes);

  } catch (error) {
    reply(Boom.badData(error));
  }
}

exports.getClientesFull = async (request, reply) => {
  try {
    var pag = (request.query.max && request.query.max > 0) ? { page: request.query.page, perPage: request.query.max } : undefined;

    const query = Cliente.query;

    if (request.query.order && request.query.order.length > 0) {
      var order = [];
      for (let i = 0; i < request.query.order.length; i++) {
        order.push(`${request.query.order[i].campo} ${request.query.order[i].ordem}`);
      }
      query.orderBy(order);
    }
    if (request.query.nome) query.where(Cliente.where.like({ nome: `%${request.query.nome}%` }));
    if (request.query.numero) query.where(Cliente.where.like({ numero: `%${request.query.numero}%` }));
    if (request.query.documento) query.where(Cliente.where.like({ documento: `%${request.query.documento}%` }));
    if (request.query.idGrupo) query.where({ idGrupo: request.query.idGrupo });

    query.leftJoin(ClienteGrupo);
    query.leftJoin(ClienteCamposAdicionais);
    query.leftJoin(ClienteNumerosAdicionais);
    var clientes = await query.require(false).fetch(pag);
    reply(clientes);

  } catch (error) {
    reply(Boom.badData(error));
  }
}

exports.FindClient = async (request, reply) => {
  try {
    const q = request.query.q.toLowerCase()
    var clientes = await db.query(`SELECT DISTINCT ON(a."id") a."id",a."numero", a."isBusiness", a."nome", a."profileImage", a."idGrupo", a."email", a."documento", a."url", a."facebookID" 
    FROM "cliente" AS a 
    LEFT JOIN "clienteNumerosAdicionais" AS c ON c."clienteID" = a."id" 
    LEFT JOIN "clienteCamposAdicionais" AS cc ON cc."clienteID" = a."id"
    WHERE LOWER(a."email") LIKE '%' || $1 || '%' OR LOWER(a."nome") LIKE '%' || $1 || '%' OR a."numero" LIKE '%' || $1 || '%' OR a."documento" LIKE '%' || $1 || '%' OR c."numero" LIKE '%' || $1 || '%' OR LOWER(cc."valor") LIKE '%' || $1 || '%' LIMIT 50`, [q])
    reply(clientes.rows);

  } catch (error) {
    reply(Boom.badData(error));
  }
}

exports.getClientesGrupos = async (request, reply) => {
  try {
    var pag = (request.query.max && request.query.max > 0) ? { page: request.query.page, perPage: request.query.max } : undefined;
    const query = ClienteGrupo.query;

    if (request.query.order && request.query.order.length > 0) {
      var order = [];
      for (let i = 0; i < request.query.order.length; i++) {
        order.push(`${request.query.order[i].campo} ${request.query.order[i].ordem}`);
      }
      query.orderBy(order);
    }
    var clientesGrupos = await query.require(false).fetch(pag);
    reply(clientesGrupos);
  } catch (error) {
    reply(Boom.badData(error));
  }
}

exports.getCliente = async (request, reply) => {
  try {
    var cliente = await Cliente.query
      .leftJoin(ClienteGrupo)
      .first()
      .require(true)
      .where({ id: request.params.ID })
      .fetch();
    cliente.NumerosAdicionais = await ClienteNumerosAdicionais.query.require(false).where({ clienteID: request.params.ID }).fetch();
    cliente.CamposAdicionais = await ClienteCamposAdicionais.query.require(false).where({ clienteID: request.params.ID }).fetch();
    reply(cliente);
  } catch (error) {
    reply(Boom.badData(error));
  }

}

exports.getClienteByNumero = async (request, reply) => {
  try {
    let cliente = await Cliente.BuscaClientePorNumero(request.params.NUMERO);

    if (cliente && cliente.length > 0) {
      cliente[0].NumerosAdicionais = await ClienteNumerosAdicionais.query.require(false).where({ clienteID: cliente[0].id }).fetch();
      cliente[0].CamposAdicionais = await ClienteCamposAdicionais.query.require(false).where({ clienteID: cliente[0].id }).fetch();
      reply(cliente[0]);
    }
    else {
      reply(Boom.badData('Cliente não encontrado'));
    }
  } catch (error) {
    console.error(error);
    reply(Boom.badData(error));
  }

}

exports.getClienteByDocumento = async (request, reply) => {
  try {
    var cliente = await Cliente.query
      .leftJoin(ClienteGrupo)
      .first()
      .require(true)
      .where({ documento: request.params.DOCUMENTO })
      .fetch();
    cliente.NumerosAdicionais = await ClienteNumerosAdicionais.query.require(false).where({ clienteID: cliente.id }).fetch();
    cliente.CamposAdicionais = await ClienteCamposAdicionais.query.require(false).where({ clienteID: cliente.id }).fetch();
    reply(cliente);
  } catch (error) {
    console.error(error);
    reply(Boom.badData(error));
  }

}


exports.createOrUpdateCliente = async (request, reply) => {
  try {
    let cliente = await Cliente.getClienteCall({ remetente: request.payload.numero });
    if (cliente.id !== 9999999) {
      request.params.ID = cliente.id;
      this.updateCliente(request, reply);
    } else {
      this.createCliente(request, reply);
    }
  } catch (error) {
    console.error('ERRO NO CREATE OR UPDATECLIENTE', error)
  }


}
exports.createCliente = async (request, reply) => {
  const transaction = new db.orm.Transaction();
  const ClienteT = transaction.models.Cliente;
  const ClienteNumerosAdicionaisT = transaction.models.ClienteNumerosAdicionais;
  const ClienteCamposAdicionaisT = transaction.models.ClienteCamposAdicionais;

  try {
    var { NumerosAdicionais, CamposAdicionais, ...clienteUp } = request.payload;
    let t = await ClienteT.insert(clienteUp, { require: true });
    var cliente = t[0];

    if (NumerosAdicionais) {
      NumerosAdicionais = NumerosAdicionais.map((element) => {
        element.clienteID = cliente.id;
        return element;
      });
      cliente.NumerosAdicionais = await ClienteNumerosAdicionaisT.insert(NumerosAdicionais, { require: true });
    }
    if (CamposAdicionais) {
      CamposAdicionais = CamposAdicionais.map((element) => {
        element.clienteID = cliente.id;
        return element;
      });
      cliente.CamposAdicionais = await ClienteCamposAdicionaisT.insert(CamposAdicionais, { require: true });
    }
    await transaction.commit();

    reply(cliente);
  } catch (error) {
    await transaction.rollback();
    reply(Boom.badData(error));
  }
}
exports.updateCliente = async (request, reply) => {
  const transaction = new db.orm.Transaction();
  const ClienteT = transaction.models.Cliente;
  const ClienteNumerosAdicionaisT = transaction.models.ClienteNumerosAdicionais;
  const ClienteCamposAdicionaisT = transaction.models.ClienteCamposAdicionais;

  try {
    var { NumerosAdicionais, CamposAdicionais, ...clienteUp } = request.payload;
    var cliente;

    if (Object.keys(clienteUp).length > 0) {
      clienteUp.id = request.params.ID;
      clienteUp.idGrupo ? clienteUp.idGrupo : null
      var t = await ClienteT.update(clienteUp, { require: true });
      cliente = t[0];
    }
    else {
      cliente = await Cliente.fetchById(request.params.ID);
    }

    if (NumerosAdicionais) {
      await ClienteNumerosAdicionaisT.query
        .require(false)
        .where({ clienteID: request.params.ID })
        .delete();
      NumerosAdicionais = NumerosAdicionais.map((element) => {
        element.clienteID = request.params.ID;
        return element;
      });
      cliente.NumerosAdicionais = await ClienteNumerosAdicionaisT.insert(NumerosAdicionais, { require: true });
    }

    if (CamposAdicionais) {
      await ClienteCamposAdicionaisT.query
        .require(false)
        .where({ clienteID: request.params.ID })
        .delete();
      CamposAdicionais = CamposAdicionais.map((element) => {
        element.clienteID = request.params.ID;
        return element;
      });

      cliente.CamposAdicionais = await ClienteCamposAdicionaisT.insert(CamposAdicionais, { require: true });
    }
    await transaction.commit();
    reply(cliente);

  } catch (error) {
    await transaction.rollback();
    reply(Boom.badData(error));
  }
}
exports.deleteCliente = async (request, reply) => {
  try {
    var cliente = await Cliente.fetchById(request.params.ID);
    cliente.delete();
    reply(cliente);
  } catch (error) {
    reply(Boom.badData(error));
  }
}
exports.deleteAll = async (request, reply) => {
  try {
    await Cliente.query
      .require(false)
      .where(Cliente.where.notEqual({ id: 9999999 }))
      .delete();

    reply(true);
  } catch (error) {
    reply(Boom.badData(error));
  }
}

exports.deleteGroupAll = async (request, reply) => {
  try {
    await Cliente.query
      .require(false)
      .update({ idGrupo: null });

    reply(true);
  } catch (error) {
    reply(Boom.badData(error));
  }
}

exports.getClienteGrupo = async (request, reply) => {
  try {
    var ret = await ClienteGrupo.fetchById(request.params.ID);
    reply(ret);
  } catch (error) {
    reply(Boom.badData(error));
  }
}
exports.createClientesGrupos = async (request, reply) => {
  try {
    var ret = await new ClienteGrupo(request.payload).save()
    reply(ret);
  } catch (error) {
    reply(Boom.badData(error));
  }
}
exports.updateClientesGrupos = async (request, reply) => {
  try {
    const dados = request.payload;
    dados.id = request.params.ID;
    var ret = await new ClienteGrupo(dados).save()
    reply(ret);
  } catch (error) {
    reply(Boom.badData(error));
  }
}
exports.deleteClientesGrupos = async (request, reply) => {
  try {
    const ob = await ClienteGrupo.fetchById(request.params.ID);
    ob.delete();
    reply(ob);
  } catch (error) {
    reply(Boom.badData(error));
  }
}

let processarCSV = function (dadosDoArquivo, delimitador) {
  return new Promise((resolve, reject) => {
    let registros = [];
    csv.fromString(dadosDoArquivo, { headers: false, delimiter: delimitador })
      .on("data", data => registros.push(data))
      .on("error", error => reject(`Erro ao processar o CSV: ${error}`))
      .on("end", () => resolve(registros));
  });
};

exports.importClientes = async (request, reply) => {
  var file = request.payload.file;
  const id_layout = request.payload.layout;
  const idGrupo = request.payload.idGrupo ? request.payload.idGrupo : null;
  const aceitos = ['application/vnd.ms-excel', 'text/plain', 'text/csv', 'text/tsv'];

  var ret = {
    status: 'Importado com sucesso.',
    nome_arquivo: file.hapi.filename,
    type: file.hapi.headers['content-type']
  }

  if (aceitos.includes(file.hapi.headers['content-type'])) {
    let body = '';
    file.on('data', (chunk) => {
      body += chunk;
    })
    file.on('end', async () => {
      // Pega layout
      const layout = await Layout.query
        .innerJoin(LayoutCampos)
        .where({ id: id_layout })
        .fetch();

      // Se o layout possuir cabeçalho, deve-se considerar apenas da linha 1 pra baixo
      let indiceInicialRegistros = layout[0].cabecalho ? 1 : 0;
      let campos = layout[0].layoutsCampos.filter(x => !["telefone_adicional", "campo_adicional"].includes(x.xcontact));
      let numeros_adicionais = layout[0].layoutsCampos.filter(x => x.xcontact === "telefone_adicional");
      let campos_adicionais = layout[0].layoutsCampos.filter(x => x.xcontact === "campo_adicional");

      // Na rota
      let dadosDoArquivo = body;

      // Da parse no conteúdo do arquivo
      let registros = await processarCSV(dadosDoArquivo, layout[0].delimitador);

      // Monta retorno
      let result = {
        total: 0,
        adicionados: 0,
        atualizados: 0,
        falha: 0
      };
      result.total = registros.length - indiceInicialRegistros;

      const uid = uuid.v4();

      // Popula registros para insert.
      let clientesList = [];
      let clientesCamposAdicionaisList = [];
      let clientesNumerosAdicionaisList = [];
      for (let i = indiceInicialRegistros; i < registros.length; i++) {
        let cliente = {};
        cliente.uid = uid;
        cliente.idGrupo = idGrupo;
        for (let campo of campos) {
          let registro = registros[i][campo.posicao_csv];
          if (registro && registro.trim().length > 0) {
            cliente[campo.xcontact] = campo.xcontact == 'numero' ? TextoUtil.limparTextoNumerico(registro) : registro;
          }
          if (campo.xcontact === "isBusiness") {
            const isEmpresa = ["cnpj", "pessoa juridica", "pessoa jurídica", "juridica", "jurídica", "empresa"].includes(registros[i][campo.posicao_csv].toLowerCase());
            cliente[campo.xcontact] = isEmpresa ? true : false;
          }
        }
        clientesList.push(cliente);

        // Registra Numeros adicionais
        for (let campo of numeros_adicionais) {
          let registro = registros[i][campo.posicao_csv];
          if (registro && registro.trim().length > 0) {
            let newReg = {
              uid: uid,
              clienteNumero: cliente.numero,
              clienteEmail: cliente.email,
              nome: campo.nome,
              numero: TextoUtil.limparTextoNumerico(registro)
            };
            clientesNumerosAdicionaisList.push(newReg);
          }
        }

        // Registra campos adicionais
        for (let campo of campos_adicionais) {
          let registro = registros[i][campo.posicao_csv];
          if (registro && registro.trim().length > 0) {
            let newReg = {
              uid: uid,
              clienteNumero: cliente.numero,
              clienteEmail: cliente.email,
              nome: campo.nome,
              valor: registro
            };
            clientesCamposAdicionaisList.push(newReg);
          }
        }
      }

      // Insere na tabela temp
      try {
        await db.knex.batchInsert('clientes_import', clientesList);
        if (clientesNumerosAdicionaisList.length > 0) {
          await db.knex.batchInsert('clientes_import_numeros', clientesNumerosAdicionaisList);
        }
        if (clientesCamposAdicionaisList.length > 0) {
          await db.knex.batchInsert('clientes_import_campos', clientesCamposAdicionaisList);
        }

        // Agora vem os pesados...
        await db.knex.raw(`UPDATE cliente b
    SET numero = COALESCE(a.numero, b.numero), nome = COALESCE(a.nome, b.nome), "profileImage" = COALESCE(a."profileImage", b."profileImage"), "idGrupo" = COALESCE(a."idGrupo", b."idGrupo"), email = COALESCE(a.email, b.email), documento = COALESCE(a.documento, b.documento), url = COALESCE(a.url, b.url), "facebookID" = COALESCE(a."facebookID", b."facebookID")
    FROM clientes_import a
    WHERE ((a.email is not null AND a.email = b.email) OR (a.numero is not null AND a.numero = b.numero)) AND a.uid = '${uid}'`);

        await db.knex.raw(`INSERT INTO cliente("numero", "isBusiness", "nome", "profileImage", "idGrupo", "email", "documento", "url", "facebookID")
    select "numero", "isBusiness", "nome", "profileImage", "idGrupo", "email", "documento", "url", "facebookID" from clientes_import WHERE uid = '${uid}' ON CONFLICT DO NOTHING`);

        await db.knex.raw(`UPDATE "clienteCamposAdicionais" up
    SET valor = imp."valor"
    FROM "clientes_import_campos" imp, "cliente" j
    WHERE (j.email = imp."clienteEmail" OR j.numero = imp."clienteNumero") AND j.id = up."clienteID" AND up.nome = imp.nome AND imp.uid = '${uid}'`);

        await db.knex.raw(`INSERT INTO "clienteCamposAdicionais"("clienteID", nome, valor)
    SELECT j.id,imp.nome,imp.valor
    FROM "clientes_import_campos" imp, "cliente" j
    WHERE (j.email = imp."clienteEmail" OR j.numero = imp."clienteNumero") AND imp.uid = '${uid}' ON CONFLICT DO NOTHING`);

        await db.knex.raw(`UPDATE "clienteNumerosAdicionais" up
    SET numero = imp."numero"
    FROM "clientes_import_numeros" imp, "cliente" j
    WHERE (j.email = imp."clienteEmail" OR j.numero = imp."clienteNumero") AND j.id = up."clienteID" AND up.nome = imp.nome AND imp.uid = '${uid}'`);

        await db.knex.raw(`INSERT INTO "clienteNumerosAdicionais"("clienteID", nome, numero)
    SELECT j.id,imp.nome,imp.numero
    FROM "clientes_import_numeros" imp, "cliente" j
    WHERE (j.email = imp."clienteEmail" OR j.numero = imp."clienteNumero") AND imp.uid = '${uid}' ON CONFLICT DO NOTHING`);

        // Limpar tabelas de import
        await db.knex.raw(`DELETE FROM clientes_import WHERE uid = '${uid}'`);
        await db.knex.raw(`DELETE FROM clientes_import_campos WHERE uid = '${uid}'`);
        await db.knex.raw(`DELETE FROM clientes_import_numeros WHERE uid = '${uid}'`);

        reply(ret);
      } catch (error) {
        reply(Boom.badData(error));
      }
    });
    file.on('error', (err) => {
      reply(Boom.badData('Erro ao ler o arquivo'));
    });
  } else {
    reply(Boom.badData('Formato de arquivo não aceito.'));
  }
}