'use strict'

const Boom = require('boom');
const db = require('../../util/db');
const Paginacao = require('../../util/Paginacao');
const config = require('../../config.json');
const moment = require('moment-timezone');
const mail = require('../../mail_notifier/index');
const restbase = require('../../util/restbase');
moment.tz.setDefault(config.servidor.timezone);
const Ami = require('../../ami/index');

const Horarios = require('../../models/Horarios');
const Fila = require('../../models/Fila');
const FilaTurnos = require('../../models/FilaTurnos');
const FilaRespostasAutomaticas = require('../../models/FilaRespostasAutomaticas');
const FilaPreAtendimento = require('../../models/FilaPreAtendimento');
const FilaPreAtendimentoOpcoes = require('../../models/FilaPreAtendimentoOpcoes');
// const Campanha = require('../../models/Campanha');

exports.create = async (request, reply) => {
    const transaction = new db.orm.Transaction();

    const FilaT = transaction.models.Fila;
    const FilaTurnosT = transaction.models.FilaTurnos;
    const FilaRespostasAutomaticasT = transaction.models.FilaRespostasAutomaticas;
    const FilaPreAtendimentoT = transaction.models.FilaPreAtendimento;
    const FilaPreAtendimentoOpcoesT = transaction.models.FilaPreAtendimentoOpcoes;

    try {
        const { turnos, RespostasAutomaticas, preatendimento, ...FilaUp } = request.payload;
        FilaUp.name = FilaUp.name ? FilaUp.name : request.params.FILA_NOME;

        var fila = await FilaT.insert(FilaUp, { require: true });

        if (turnos) {
            FilaTurnosT.insert(turnos);
        }

        if (RespostasAutomaticas) {
            FilaRespostasAutomaticasT.insert(RespostasAutomaticas);
        }

        if (preatendimento) {
            const { menu, ...pre } = preatendimento;
            FilaPreAtendimentoT.insert(pre);
            if (menu) {
                FilaPreAtendimentoOpcoesT.insert(menu);
            }
        }

        await transaction.commit();

        if (FilaUp.emailConfig && FilaUp.queue_type == 4 && config.email_ativo) {
            mail.main();
        }
        // Atualiza a fila no AMI
        Ami.loadFilaByName(FilaUp.name);

        reply(fila);
    } catch (e) {
        await transaction.rollback();
        reply(Boom.badData(e));
    }
}

exports.update = async (request, reply) => {
    const transaction = new db.orm.Transaction();

    const FilaT = transaction.models.Fila;
    const FilaTurnosT = transaction.models.FilaTurnos;
    const FilaRespostasAutomaticasT = transaction.models.FilaRespostasAutomaticas;
    const FilaPreAtendimentoT = transaction.models.FilaPreAtendimento;
    const FilaPreAtendimentoOpcoesT = transaction.models.FilaPreAtendimentoOpcoes;
    const CampanhaT = transaction.models.Campanha;

    try {
        const { turnos, RespostasAutomaticas, preatendimento, ...FilaUp } = request.payload
        FilaUp.name = FilaUp.name ? FilaUp.name : request.params.FILA_NOME;

        var fila = await FilaT.save(FilaUp, { require: true });

        // Check if the queue's name has changed
        /* if (FilaUp.name !== request.params.FILA_NOME) {
            // Change all campaigns
            let campanhas = CampanhaT.query
                                .require(false)
                                .where({ fila: request.params.FILA_NOME })
                                .fetch();

            for (let c of campanhas) {
                c.fila = FilaUp.name;
                c.save();
            }
        } */

        if (turnos) {
            await FilaTurnosT.query
                .require(false)
                .where({ FilaID: FilaUp.name })
                .delete();

            FilaTurnosT.insert(turnos);
        }

        if (RespostasAutomaticas) {
            await FilaRespostasAutomaticasT.query
                .require(false)
                .where({ fila: FilaUp.name })
                .delete();

            FilaRespostasAutomaticasT.insert(RespostasAutomaticas);
        }

        if (preatendimento) {
            const { menu, ...pre } = preatendimento;

            await FilaPreAtendimentoT.query
                .require(false)
                .where({ queueName: FilaUp.name })
                .delete();

            FilaPreAtendimentoT.insert(pre);
            if (menu) {
                FilaPreAtendimentoOpcoesT.insert(menu);
            }
        }

        await transaction.commit();

        if (FilaUp.emailConfig && FilaUp.queue_type == 4 && config.email_ativo) {
            console.log('Caiu no mail.main')
            mail.main();
        }
        // Atualiza a fila no AMI
        Ami.loadFilaByName(FilaUp.name);

        reply(fila);
    } catch (e) {
        await transaction.rollback();
        reply(Boom.badData(e));
    }
}

exports.delete = (request, reply) => {
    const knex = db.knex('public.queue_table');
    knex.where('name', request.params.FilaID)
        .del()
        .then(function (rows) {
            if (rows > 0) {
                const ret = {};
                ret.error = false;
                ret.total = rows;
                ret.message = 'Fila name = ' + request.params.FilaID + ' apagada com sucesso.';
                reply(ret);
            }
            else {
                reply(Boom.badData("Nenhum resgistro encontrado para name = " + request.params.FilaID));
            }
        })
        .catch(function (error) {
            console.error(error);
            reply(Boom.badData(error));
        });
};

exports.getall = (request, reply) => {
    const max = request.query.max ? request.query.max : 15;
    const page = request.query.page ? request.query.page : 1;
    const knex = db.knex('public.queue_table');
    const pag = new Paginacao(max, page);

    if (request.query.status) knex.where('status', request.query.status);
    if (request.query.nome) knex.whereRaw('LOWER(name) LIKE ?', '%' + decodeURIComponent(request.query.nome).toLowerCase() + '%');

    pag.order = request.query.order;
    pag.knex = knex;

    pag.getRet((ret, err) => {
        if (err) {
            reply(Boom.badData(err));
        }
        else {
            reply(ret);
        }
    });
};

exports.getLigacoes = (request, reply) => {
    const max = request.query.max ? request.query.max : 15;
    const page = request.query.page ? request.query.page : 1;
    const knex = db.knex('public.queue_calls');
    const pag = new Paginacao(max, page);

    if (request.query.data_ini) knex.where('queue_calls.calldate', '>=', moment(request.query.data_ini).format('YYYY-MM-DDTHH:mm:ss') + 'Z');
    if (request.query.data_fim) knex.where('queue_calls.calldate', '<=', moment(request.query.data_fim).format('YYYY-MM-DDTHH:mm:ss') + 'Z');
    if (request.query.status && request.query.status.length > 0) knex.whereRaw("queue_calls.status = ANY (ARRAY " + JSON.stringify(request.query.status).replace(/"/g, "'") + ")");
    if (request.query.agentes && request.query.agentes.length > 0) knex.whereRaw("LOWER(queue_calls.agent) = ANY (ARRAY " + JSON.stringify(request.query.agentes.map((x) => { return x.toLowerCase(); })).replace(/"/g, "'") + ")");
    if (request.query.filas && request.query.filas.length > 0) knex.whereRaw("queue_calls.queuename = ANY (ARRAY " + JSON.stringify(request.query.filas).replace(/"/g, "'") + ")");
    if (request.query.sentido) knex.where('queue_calls.inout', request.query.sentido);
    if (request.query.classificacao && request.query.classificacao.length > 0) knex.whereRaw("classificacoes.nome = ANY (ARRAY " + JSON.stringify(request.query.classificacao).replace(/"/g, "'") + ") OR cla2.nome = ANY (ARRAY " + JSON.stringify(request.query.classificacao).replace(/"/g, "'") + ")");
    if (request.query.subclassificacao && request.query.subclassificacao.length > 0) knex.whereRaw("subclassificacao.nome = ANY (ARRAY " + JSON.stringify(request.query.subclassificacao).replace(/"/g, "'") + ") OR sub2.nome = ANY (ARRAY " + JSON.stringify(request.query.subclassificacao).replace(/"/g, "'") + ")");
    if (request.query.protocolo) knex.whereRaw("cdr_classificacoes.protocolo = '" + request.query.protocolo + "' OR c2.protocolo = '" + request.query.protocolo + "'");
    //if(request.query.classificacao && request.query.classificacao.length > 0) knex.whereRaw("classificacoes.nome = ANY (ARRAY "+JSON.stringify(request.query.classificacao).replace(/"/g, "'")+")");
    //if(request.query.subclassificacao && request.query.subclassificacao.length > 0) knex.whereRaw("subclassificacao.nome = ANY (ARRAY "+JSON.stringify(request.query.subclassificacao).replace(/"/g, "'")+")");
    //if(request.query.protocolo) knex.where('cdr_classificacoes.protocolo', request.query.protocolo);
    if (request.query.finalizadapor) knex.where('queue_calls.terminatedby', request.query.finalizadapor);
    if (request.query.numero) knex.whereRaw('queue_calls.numero LIKE ?', '%' + decodeURIComponent(request.query.numero) + '%');
    if (request.query.identclient) knex.whereRaw(restbase.buildSearchExpression('identclient', request.query.identclientc, request.query.identclient));
    if (request.query.cliente_nome) knex.whereRaw('queue_calls.cliente_nome LIKE ?', '%' + decodeURIComponent(request.query.cliente_nome) + '%');

    knex.leftJoin('cdr_gravacao', function () {
        this.on('queue_calls.callid', '=', 'cdr_gravacao.uniqueid').andOn('queue_calls.queuename', '=', 'cdr_gravacao.queuename')
    });
    knex.leftJoin('cdr_classificacoes', 'queue_calls.callid', 'cdr_classificacoes.uniqueid');
    knex.leftJoin('classificacoes', 'cdr_classificacoes.id_classificacao', 'classificacoes.id');
    knex.leftJoin('subclassificacao', 'cdr_classificacoes.id_subclassificacao', 'subclassificacao.id');
    knex.leftJoin('cdr_classificacoes as c2', 'queue_calls.callid2', 'c2.uniqueid');
    knex.leftJoin('classificacoes as cla2', 'c2.id_classificacao', 'cla2.id');
    knex.leftJoin('subclassificacao as sub2', 'c2.id_subclassificacao', 'sub2.id');

    pag.select = ['queue_calls.callid',
        'queue_calls.calldate',
        'queue_calls.queuename as fila',
        'queue_calls.agent as agente',
        'queue_calls.numero',
        'queue_calls.cliente_nome',
        'queue_calls.status',
        db.knex.raw("TO_CHAR((queue_calls.holdtime || ' second'::text)::interval,'HH24:MI:SS') as espera"),
        db.knex.raw("TO_CHAR((queue_calls.calltime || ' second'::text)::interval,'HH24:MI:SS') as duracao"),
        'queue_calls.terminatedby as finalizadapor',
        'queue_calls.position as posicao_final',
        'queue_calls.origposition as posicao_entrada',
        'queue_calls.inout as sentido',
        'queue_calls.transferredto as transferida_para',
        'queue_calls.identclient',
        'queue_calls.idCliente',
        db.knex.raw("COALESCE ( classificacoes.id, cla2.id) as id_classificacao"),
        db.knex.raw("COALESCE ( classificacoes.nome, cla2.nome) as classificacao"),
        db.knex.raw("COALESCE ( cdr_classificacoes.obs, c2.obs) as observacao"),
        db.knex.raw("COALESCE ( cdr_classificacoes.protocolo, c2.protocolo) as protocolo"),
        db.knex.raw("COALESCE ( cdr_classificacoes.id_subclassificacao, c2.id_subclassificacao) as id_subclassificacao"),
        db.knex.raw("COALESCE ( subclassificacao.nome, sub2.nome) as subclassificacao")];

    if (request.auth && request.auth.credentials && request.auth.credentials.extra) {
        const permi = request.auth.credentials.extra.find(item => item.modulo === 'app.cdrfilas');
        if (permi && permi.extra) {
            if (permi.extra.includes('G')) pag.select.push('cdr_gravacao.link as gravacao');
        }
    }


    pag.order = request.query.order;
    pag.knex = knex;

    pag.getRet((ret, err) => {
        if (err) {
            reply(Boom.badData(err));
        }
        else {
            reply(ret);
        }
    });
};

exports.returnLigacoes = async (request) => {
    const max = request.max ? request.max : 15;
    const page = request.page ? request.page : 1;
    const knex = db.knex('public.queue_calls');
    const pag = new Paginacao(max, page);

    if (request.data_ini) knex.where('queue_calls.calldate', '>=', moment(request.data_ini).format('YYYY-MM-DDTHH:mm:ss') + 'Z');
    if (request.data_fim) knex.where('queue_calls.calldate', '<=', moment(request.data_fim).format('YYYY-MM-DDTHH:mm:ss') + 'Z');
    if (request.status && request.status.length > 0) knex.whereRaw("queue_calls.status = ANY (ARRAY " + JSON.stringify(request.status).replace(/"/g, "'") + ")");
    if (request.agentes && request.agentes.length > 0) knex.whereRaw("LOWER(queue_calls.agent) = ANY (ARRAY " + JSON.stringify(request.agentes.map((x) => { return x.toLowerCase(); })).replace(/"/g, "'") + ")");
    if (request.filas && request.filas.length > 0) knex.whereRaw("queue_calls.queuename = ANY (ARRAY " + JSON.stringify(request.filas).replace(/"/g, "'") + ")");
    if (request.sentido) knex.where('queue_calls.inout', request.sentido);
    if (request.classificacao && request.classificacao.length > 0) knex.whereRaw("classificacoes.nome = ANY (ARRAY " + JSON.stringify(request.classificacao).replace(/"/g, "'") + ") OR cla2.nome = ANY (ARRAY " + JSON.stringify(request.classificacao).replace(/"/g, "'") + ")");
    if (request.subclassificacao && request.subclassificacao.length > 0) knex.whereRaw("subclassificacao.nome = ANY (ARRAY " + JSON.stringify(request.subclassificacao).replace(/"/g, "'") + ") OR sub2.nome = ANY (ARRAY " + JSON.stringify(request.subclassificacao).replace(/"/g, "'") + ")");
    if (request.protocolo) knex.whereRaw("cdr_classificacoes.protocolo = '" + request.protocolo + "' OR c2.protocolo = '" + request.protocolo + "'");
    //if(request.classificacao && request.classificacao.length > 0) knex.whereRaw("classificacoes.nome = ANY (ARRAY "+JSON.stringify(request.classificacao).replace(/"/g, "'")+")");
    //if(request.subclassificacao && request.subclassificacao.length > 0) knex.whereRaw("subclassificacao.nome = ANY (ARRAY "+JSON.stringify(request.subclassificacao).replace(/"/g, "'")+")");
    //if(request.protocolo) knex.where('cdr_classificacoes.protocolo', request.protocolo);
    if (request.finalizadapor) knex.where('queue_calls.terminatedby', request.finalizadapor);
    if (request.numero) knex.whereRaw('queue_calls.numero LIKE ?', '%' + decodeURIComponent(request.numero) + '%');
    if (request.identclient) knex.whereRaw(restbase.buildSearchExpression('identclient', request.identclientc, request.identclient));
    if (request.cliente_nome) knex.whereRaw('queue_calls.cliente_nome LIKE ?', '%' + decodeURIComponent(request.cliente_nome) + '%');

    knex.leftJoin('cdr_gravacao', function () {
        this.on('queue_calls.callid', '=', 'cdr_gravacao.uniqueid').andOn('queue_calls.queuename', '=', 'cdr_gravacao.queuename')
    });
    knex.leftJoin('cdr_classificacoes', 'queue_calls.callid', 'cdr_classificacoes.uniqueid');
    knex.leftJoin('classificacoes', 'cdr_classificacoes.id_classificacao', 'classificacoes.id');
    knex.leftJoin('subclassificacao', 'cdr_classificacoes.id_subclassificacao', 'subclassificacao.id');
    knex.leftJoin('cdr_classificacoes as c2', 'queue_calls.callid2', 'c2.uniqueid');
    knex.leftJoin('classificacoes as cla2', 'c2.id_classificacao', 'cla2.id');
    knex.leftJoin('subclassificacao as sub2', 'c2.id_subclassificacao', 'sub2.id');

    pag.select = ['queue_calls.callid',
        'queue_calls.calldate',
        'queue_calls.queuename as fila',
        'queue_calls.agent as agente',
        'queue_calls.numero',
        'queue_calls.cliente_nome',
        'queue_calls.status',
        db.knex.raw("TO_CHAR((queue_calls.holdtime || ' second'::text)::interval,'HH24:MI:SS') as espera"),
        db.knex.raw("TO_CHAR((queue_calls.calltime || ' second'::text)::interval,'HH24:MI:SS') as duracao"),
        'queue_calls.terminatedby as finalizadapor',
        'queue_calls.position as posicao_final',
        'queue_calls.origposition as posicao_entrada',
        'queue_calls.inout as sentido',
        'queue_calls.transferredto as transferida_para',
        'queue_calls.identclient',
        'queue_calls.idCliente',
        db.knex.raw("COALESCE ( classificacoes.id, cla2.id) as id_classificacao"),
        db.knex.raw("COALESCE ( classificacoes.nome, cla2.nome) as classificacao"),
        db.knex.raw("COALESCE ( cdr_classificacoes.obs, c2.obs) as observacao"),
        db.knex.raw("COALESCE ( cdr_classificacoes.protocolo, c2.protocolo) as protocolo"),
        db.knex.raw("COALESCE ( cdr_classificacoes.id_subclassificacao, c2.id_subclassificacao) as id_subclassificacao"),
        db.knex.raw("COALESCE ( subclassificacao.nome, sub2.nome) as subclassificacao")];

    if (request.auth && request.auth.credentials && request.auth.credentials.extra) {
        const permi = request.auth.credentials.extra.find(item => item.modulo === 'app.cdrfilas');
        if (permi && permi.extra) {
            if (permi.extra.includes('G')) pag.select.push('cdr_gravacao.link as gravacao');
        }
    }


    pag.order = request.order;
    pag.knex = knex;

    try {
        const ret = await pag.returnAll();
        return ret;
    } catch (error) {
        console.error(error);
        return Boom.badData(error);
    }
};

exports.getOne = async (request, reply) => {
    try {
        var fila = await Fila.fetchByName(request.params.FILA_NOME);
        var respostas = await FilaRespostasAutomaticas.query.require(false).where({ fila: request.params.FILA_NOME }).fetch();
        fila.RespostasAutomaticas = respostas;

        var turnos = await Horarios.query.require(false).fields(['id', 'nome']).innerJoin(FilaTurnos.query.where({ FilaID: request.params.FILA_NOME })).fetch();
        fila.turnos = turnos;

        var preatendimento = await FilaPreAtendimento.query.require(false).first().where({ queueName: request.params.FILA_NOME }).fetch();
        if (preatendimento) {
            preatendimento.menu = await FilaPreAtendimentoOpcoes.query.require(false).where({ queueName: request.params.FILA_NOME }).fetch();
        }
        fila.preatendimento = preatendimento;

        reply(fila);

    } catch (error) {
        console.error(error);
        reply(Boom.badData(error));
    }
};