'use strict'

const Joi = require('joi');
const handlers = require('./handlers');
const CRUD = require('../../util/CRUD');
const Ami = require('../../ami/index');
const XCconfig = require('../../config');
const HTTPr = require('request');
const fs = require('fs');

const CreateReturnSchema = {
  error: Joi.boolean().example(false).description('Indica se o processo gerou algum erro.'),
  name: Joi.string().example('fila1').description('ID da fila'),
  message: Joi.string().example('Registro criado com aucesso.')
};

const ListaReturnSchema = {
  error: Joi.boolean().example(false).description('Indica se o processo gerou algum erro.'),
  full: Joi.boolean().example(false).description('Indica se o retorno é completo, caso seja false será paginado.'),
  total: Joi.number().integer().example('10').description('Quantidade de registros retornados'),
  page: Joi.number().integer().example('1').description('Pagina atual dos registros'),
  offset: Joi.number().integer().example('1').description('Offset'),
  pages: Joi.number().integer().example('1').description('total de página'),
  max: Joi.number().integer().example('15').description('Máximo de registros por página'),
  pages: Joi.number().integer().example('1').description('total de páginas'),
  message: Joi.string().example('Lista de registros').description('Descrição da lista'),
  dados: Joi.array().items().example([{ id: 1, nome: "Registro1" }, { id: 2, nome: "Registro2" }]).description('Lista dos registros')
};

const operationData = {
  name: Joi.string().required(),
  musiconhold: Joi.string().optional().allow(null).default(null),
  announce: Joi.string().optional().allow(null).default(null),
  context: Joi.string().optional().allow(null).default(null),
  timeout: Joi.number().integer().default(30),
  monitor_join: Joi.bool().optional().allow(null).default(null),
  monitor_format: Joi.string().optional().allow(null).default(null),
  queue_youarenext: Joi.string().optional().allow(null).default(null),
  queue_thereare: Joi.string().optional().allow(null).default(null),
  queue_callswaiting: Joi.string().optional().allow(null).default(null),
  queue_holdtime: Joi.string().optional().allow(null).default(null),
  queue_minutes: Joi.string().optional().allow(null).default(null),
  queue_seconds: Joi.string().optional().allow(null).default(null),
  queue_lessthan: Joi.string().optional().allow(null).default(null),
  queue_thankyou: Joi.string().optional().allow(null).default(null),
  queue_reporthold: Joi.string().optional().allow(null).default(null),
  announce_frequency: Joi.number().integer().allow(null).default(null),
  announce_round_seconds: Joi.number().integer().allow(null).default(null),
  announce_holdtime: Joi.string().optional().allow(null).default(null),
  retry: Joi.number().integer().default(0).optional().allow(null),
  wrapuptime: Joi.number().integer().allow(null).default(null),
  maxlen: Joi.number().integer().default(0),
  servicelevel: Joi.number().integer().allow(null).default(null),
  strategy: Joi.string().optional().allow(null).default(null),
  joinempty: Joi.string().optional().allow(null).default(null),
  leavewhenempty: Joi.string().optional().allow(null).default(null),
  eventmemberstatus: Joi.bool().optional().allow(null).default(null),
  eventwhencalled: Joi.bool().optional().allow(null).default(null),
  reportholdtime: Joi.bool().optional().allow(null).default(null),
  memberdelay: Joi.number().integer().allow(null).default(null),
  weight: Joi.number().integer().allow(null).default(null),
  timeoutrestart: Joi.bool().optional().allow(null).default(null),
  setinterfacevar: Joi.bool().optional().allow(null).default(true),
  ringinuse: Joi.number().integer().default(0),
  nivel_percentual: Joi.number().integer().default(80),
  nivel_tempo: Joi.number().integer().default(20),
  numero: Joi.number().integer().allow(null).default(null),
  nivel_abandono_tempo: Joi.number().integer().default(20),
  nivel_abandono_percentual: Joi.number().integer().default(80),
  id_calculo_nivel: Joi.number().integer().allow(null).default(null),
  queue_type: Joi.number().integer().allow(null).default(1),
  autofill: Joi.string().optional().allow(null).default('yes'),
  autopause: Joi.string().optional().allow(null).default('yes'),
  id_calculo_performance: Joi.number().integer().allow(null).default(null),
  setqueuevar: Joi.bool().default(true),
  verificaFeriado: Joi.bool().allow(null).default(true),
  emailConfig: Joi.object().allow(null).description('Configurações de email'),
  obrigaClassificar: Joi.bool().default(false),
  PesquisaID: Joi.number().integer().allow(null).default(null),
  instance: Joi.bool().optional().allow(null).allow(''),
  desabilitar_hangup: Joi.bool().optional().allow(null).allow(''),
  RespostasAutomaticas: Joi.array().allow(null).items(
    Joi.object().keys({
      idRespostaTipo: Joi.number().integer().description('id do tipo da Resposta'),
      fila: Joi.string().description('Nome da Fila'),
      texto: Joi.string().description('Texto da Resposta'),
    })
  ).description('Array de respostas automaricas'),
  turnos: Joi.array().allow(null).items(
    Joi.object().keys({
      TurnoID: Joi.number().integer().description('id do turno'),
      FilaID: Joi.string().description('Nome da Fila')
    })
  ).description('Array de turnos'),
  preatendimento: Joi.object().keys({
    queueName: Joi.string().description('Nome da fila'),
    msgInicial: Joi.string().description('Mensagem de menu'),
    msgOpcaoErrada: Joi.string().allow('').allow(null).description('Mensagem de opção inválida'),
    repetirMsgInicial: Joi.bool().description('Define se a mensagem inicial será repetida quando uma opção errada for digitada.'),
    menu: Joi.array().allow(null).items(
      Joi.object().keys({
        queueName: Joi.string().description('Nome da Fila'),
        opcao: Joi.string().description('Opção'),
        opcaoDescricao: Joi.string().allow('').allow(null).description('Descrição da Opção'),
        app: Joi.string().description('Ação'),
        pars: Joi.string().allow('').allow(null).description('Parametros para a ação'),
      })
    ).description('Array de opções')
  }).allow(null).description('Configurações de pré-atendimentos'),
}

const scopes = ['admin', 'app.filas', 'app.filasperformance', 'app.filasperformance_aesc', 'app.agenteperformance', 'app.webchat']

module.exports = [
  {
    method: 'GET',
    path: '/api/v2/filas',
    handler: handlers.getall,
    config: {
      description: 'Obtem as filas',
      tags: ['api', 'v2', 'Filas'],
      auth: {
        strategy: 'bearer',
        scope: scopes
      },
      response: {
        schema: ListaReturnSchema
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
              campo: Joi.string().description('Nome da coluna'),
              ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          status: Joi.string().valid('A', 'P', 'F').description('Status da campanha, A=Ativa,P=Pausa e F=Finalizada'),
          nome: Joi.string().description('Filtra por nome')
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/filas/ligacoes',
    handler: handlers.getLigacoes,
    config: {
      description: 'Obtem as ligações das filas',
      tags: ['api', 'v2', 'Filas'],
      auth: {
        strategy: 'bearer',
        scope: ['app.cdrfilas', 'admin']
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
              campo: Joi.string().description('Nome da coluna'),
              ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          data_ini: Joi.date().description('Filtro por data inicial'),
          data_fim: Joi.date().description('Filtro por data Final'),
          numero: Joi.string().description('Filtro por numero (pode ser origem ou destino dependendo do sentido)'),
          filas: Joi.array().items(Joi.string()).example(['FilaA', 'FilaB']).description('Filtra por uma ou mais filas'),
          agentes: Joi.array().items(Joi.string()).example(['AgenteA', 'AgenteB']).description('Filtra por um ou mais agentes'),
          status: Joi.array().items(Joi.string().valid('Atendida', 'Abandonada', 'Transbordou', 'Não atendida', 'Ocupado', 'Falha')).description('Filtra por um ou mais status'),
          protocolo: Joi.string().description('Filtro por protocolo'),
          classificacao: Joi.array().items(Joi.string()).example(['ClassificacaoA', 'ClassificacaoB']).description('Filtra por uma ou mais classificações.'),
          subclassificacao: Joi.array().items(Joi.string()).example(['SubclassificacaoA', 'SubclassificacaoB']).description('Filtra por uma ou mais subclassificações.'),
          sentido: Joi.number().integer().valid(1, 2).description('Filtro por sentido, 1 = Entrante, 2 = Sainte'),
          finalizadapor: Joi.number().integer().valid(1, 2).description('Filtro por quem finalizou a ligação (Valida apenas para entrantes), 1 = Agente, 2 = Cliente'),
          identclient: Joi.string().description('Filtra por Identificação do cliente, campo que pode ser alimentado por integrações'),
          identclientc: Joi.string().allow("").description('Condição do filtro de identificação do cliente'),
          cliente_nome: Joi.string().allow("").allow(null).description('Filtro por nome do cliente'),
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/fila/{FILA_NOME}',
    handler: handlers.getOne,
    config: {
      description: 'Obtem os dados da fila',
      tags: ['api', 'v2', 'Filas'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          FILA_NOME: Joi.string().required()
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v2/fila',
    handler: handlers.create,
    config: {
      description: 'Cria uma nova fila',
      tags: ['api', 'v2', 'Filas'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        payload: operationData
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v2/fila/{FILA_NOME}',
    handler: handlers.update,
    config: {
      description: 'Altera os dados da fila',
      tags: ['api', 'v2', 'Filas'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          FILA_NOME: Joi.string().required()
        },
        payload: operationData
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v2/fila/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.queue_table');
      crud.id = "name";
      crud.delete(request, (ret) => {
        // Depois de apagar do banco remove do server.
        Ami.removeFila(request.params.ID);
        reply(ret);
      });
    },
    config: {
      description: 'Remove a fila por nome',
      tags: ['api', 'v2', 'Filas'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.string().required()
        }
      }
    }
  }
];
