'use strict'

const joi = require('joi');
const Boom = require('boom');
const db = require('../../util/db');
const ws = require('../../ws');
const Ami = require('../../ami');
const discador = require('../../discador');
const config = require('../../config.json');
const moment = require('moment-timezone');
moment.tz.setDefault(config.servidor.timezone);

function rollback(client) {
  client.query('ROLLBACK', function () {
  });
};

function getPar(item, index) {
  return '$' + (parseInt(index) + 1);
};

function getReq(item, index) {
  return 'request.payload.' + item;
}

exports.classifica = (request, reply) => {
  // Ajuste de UNIQUE ID repetido, ocorre em transbordo
  if (request.payload.uniqueid.toString().indexOf(',')) {
    request.payload.uniqueid = request.payload.uniqueid.toString().split(',')[0];
  }

  // Açoes com o discador
  if (request.payload.id_classificacao) {
    try {
      discador.ProcessaClassificacao(request.payload.uniqueid, request.payload.id_classificacao);
    } catch (ex) {
    }
  }
  // Açoes com banco
  db.knex.transaction(function (t) {
    return db.knex('cdr_classificacoes')
      .transacting(t)
      .where('uniqueid', request.payload.uniqueid)
      .del()
      .then(function () {
        return db.knex('cdr_classificacoes').insert(request.payload)
          .transacting(t)
          .then()
          .catch(function (error) {
            t.rollback();
            throw error;
          });
      })
      .then(t.commit)
      .catch(function (e) {
        t.rollback();
        throw e;
      })
  })
    .then(function () {
      const ret = {};
      ret.error = false;
      ret.message = 'Classificação salva com sucesso!';
      reply(ret);
    })
    .catch(function (e) {
      reply(Boom.badData(e));
    });
};

exports.create = (request, reply) => {
  const { username, password } = request.payload;
  if (username === 'juca') {
    reply(Boom.badData('Juca não entra aqui'));
  }
  else {
    reply({ success: true });
  }
};
exports.getStatus = (request, reply) => {
  const max = request.query.max ? request.query.max : 15;
  const page = request.query.page ? request.query.page : 1;
  const knex = db.knex('public.status_ligacao');

  if (request.query.nome) knex.whereRaw('LOWER(nome) LIKE ?', '%' + decodeURIComponent(request.query.nome).toLowerCase() + '%');

  if (max === -1) {
    knex.clearSelect();
    // Parametros de ordenação, fora do count
    if (request.query.order) {
      for (let p = 0; p < request.query.order.length; ++p) {
        if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
          knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
        }
        else if (request.query.order[p] && request.query.order[p].campo) {
          knex.orderBy(request.query.order[p].campo, 'ASC');
        }
      }
    }
    knex.select('*')
      .then(function (rows) {
        const ret = {};
        ret.error = false;
        ret.full = true;
        ret.total = rows.length;
        ret.message = 'Lista de todos os status de ligação';
        ret.dados = rows;
        if (request.query.return === 'array') reply(rows);
        else reply(ret);
      })
      .catch(function (error) {
        console.error(error);
        reply(Boom.badData(error));
      });
  }
  else {
    const offset = max * (page - 1);
    knex.count('* as total')
      .then(function (total) {
        if (total[0].total > 0) {
          knex.clearSelect();
          // Parametros de ordenação, fora do count
          if (request.query.order) {
            for (let p = 0; p < request.query.order.length; ++p) {
              if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
              }
              else if (request.query.order[p] && request.query.order[p].campo) {
                knex.orderBy(request.query.order[p].campo, 'ASC');
              }
            }
          }
          knex.select('*').limit(max).offset(offset).then(function (rows) {
            const pages = Math.ceil(parseInt(total[0].total) / max);
            const ret = {};
            ret.error = false;
            ret.full = pages === 1 ? true : false;
            ret.page = page;
            ret.max = max;
            ret.offset = offset;
            ret.total = parseInt(total[0].total);
            ret.pages = pages;
            ret.message = 'Lista de status de ligação paginada';
            ret.dados = rows;
            if (request.query.return === 'array') reply(rows);
            else reply(ret);
          })
            .catch(function (error) {
              console.error(error);
              reply(Boom.badData(error));
            });
        }
        else {
          const ret = {};
          ret.error = true;
          ret.total = 0;
          ret.message = 'Nenhum registro encontrado';
          reply(ret);
        }
      })
      .catch(function (error) {
        console.error(error);
        reply(Boom.badData(error));
      });
  }
};
exports.getall = (request, reply) => {
  const max = request.query.max ? request.query.max : 15;
  const page = request.query.page ? request.query.page : 1;
  const knex = db.knex('public.ligacoes');
  let origem = { operador: 'like', valor: `%${request.query.origem}%` };
  switch (request.query.origem_operador) {
    case 'igual':
      origem = { operador: '=', valor: `${request.query.origem}` };
      break;
    case 'iniciando':
      origem = { operador: 'like', valor: `${request.query.origem}%` };
      break;
    case 'finalizando':
      origem = { operador: 'like', valor: `%${request.query.origem}` };
      break;
    case 'nao_contendo':
      origem = { operador: 'not like', valor: `%${request.query.origem}%` };
      break;
    case 'diferente':
      origem = { operador: '!=', valor: `${request.query.origem}` };
      break;
    default:
      origem = { operador: 'like', valor: `%${request.query.origem}%` };
      break;
  }

  let destino = { operador: 'like', valor: `%${request.query.destino}%` };
  switch (request.query.destino_operador) {
    case 'igual':
      destino = { operador: '=', valor: `${request.query.destino}` };
      break;
    case 'iniciando':
      destino = { operador: 'like', valor: `${request.query.destino}%` };
      break;
    case 'finalizando':
      destino = { operador: 'like', valor: `%${request.query.destino}` };
      break;
    case 'nao_contendo':
      destino = { operador: 'not like', valor: `%${request.query.destino}%` };
      break;
    case 'diferente':
      destino = { operador: '!=', valor: `${request.query.destino}` };
      break;
    default:
      destino = { operador: 'like', valor: `%${request.query.destino}%` };
      break;
  }

  if (request.query.data_ini) knex.where('calldate', '>=', request.query.data_ini);
  if (request.query.data_fim) knex.where('calldate', '<=', request.query.data_fim);
  if (request.query.origem) knex.where('src', origem.operador, origem.valor);
  if (request.query.destino) knex.where('dst', destino.operador, destino.valor);
  if (request.query.callid) knex.where('callid', request.query.callid);
  if (request.query.status && request.query.status.length > 0) knex.whereRaw("status = ANY (ARRAY " + JSON.stringify(request.query.status).replace(/"/g, "'") + ")");
  if (request.query.sentido && request.query.sentido.length > 0) knex.whereRaw("sentido = ANY (ARRAY " + JSON.stringify(request.query.sentido).replace(/"/g, "'") + ")");
  if (request.query.canal) knex.where('canal', 'like', '%' + request.query.canal + '%');
  if (request.query.pin_id) knex.where('pin_id', request.query.pin_id);
  if (request.query.pin_usuario) knex.where('pin_usuario', request.query.pin_usuario);

  if (max === -1) {
    knex.clearSelect();
    // Parametros de ordenação, fora do count
    if (request.query.order) {
      for (let p = 0; p < request.query.order.length; ++p) {
        if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
          knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
        }
        else if (request.query.order[p] && request.query.order[p].campo) {
          knex.orderBy(request.query.order[p].campo, 'ASC');
        }
      }
    }
    knex.select('*')
      .then(function (rows) {
        const ret = {};
        ret.error = false;
        ret.full = true;
        ret.total = rows.length;
        ret.message = 'Lista de todas as ligações';
        ret.dados = rows;
        reply(ret);
      })
      .catch(function (error) {
        console.error(error);
        reply(Boom.badData(error));
      });
  }
  else {
    const offset = max * (page - 1);
    knex.count('* as total')
      .then(function (total) {
        if (total[0].total > 0) {
          knex.clearSelect();
          // Parametros de ordenação, fora do count
          if (request.query.order) {
            for (let p = 0; p < request.query.order.length; ++p) {
              if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
              }
              else if (request.query.order[p] && request.query.order[p].campo) {
                knex.orderBy(request.query.order[p].campo, 'ASC');
              }
            }
          }
          knex.select('*').limit(max).offset(offset).then(function (rows) {
            const pages = Math.ceil(parseInt(total[0].total) / max);
            const ret = {};
            ret.error = false;
            ret.full = pages === 1 ? true : false;
            ret.page = page;
            ret.max = max;
            ret.offset = offset;
            ret.total = parseInt(total[0].total);
            ret.pages = pages;
            ret.message = 'Lista de ligações paginada';
            ret.dados = rows;
            reply(ret);
          })
            .catch(function (error) {
              console.error(error);
              reply(Boom.badData(error));
            });
        }
        else {
          const ret = {};
          ret.error = true;
          ret.total = 0;
          ret.message = 'Nenhum registro encontrado';
          reply(ret);
        }
      })
      .catch(function (error) {
        console.error(error);
        reply(Boom.badData(error));
      });
  }
};

exports.getOne = (request, reply) => {
  const knex = db.knex('public.ligacoes');
  knex.where('callid', request.params.CallID);
  knex.select('*')
    .then(function (rows) {
      if (rows && rows.length > 0) {
        const ret = {};
        ret.error = false;
        ret.full = true;
        ret.total = rows.length;
        ret.message = 'Dados da ligação Callid = ' + request.params.CallID;
        ret.dados = rows[0];
        reply(ret);
      }
      else {
        const ret = {};
        ret.error = true;
        ret.total = 0;
        ret.message = 'A ligação Callid = ' + request.params.CallID + ' não foi encontrada na base, talvez ainda esteja em curso ou não existe.';
        reply(ret);
      }
    })
    .catch(function (error) {
      console.error(error);
      reply(Boom.badData(error));
    });
};

exports.discar = (request, reply) => {
  Ami.Discar(request.payload, (resposta) => {
    const ret = {};
    ret.error = resposta.response === 'Error' ? true : false;
    ret.callid = resposta.response === 'Error' ? undefined : resposta.callid;
    ret.message = resposta.message;
    reply(ret);
  });
};
exports.addConferencia = (request, reply) => {
  Ami.Conferencia(request.payload, (resposta) => {
    const ret = {};
    ret.error = resposta.response === 'Error' ? true : false;
    ret.callid = resposta.response === 'Error' ? undefined : resposta.callid;
    ret.message = resposta.message;
    reply(ret);
  });
};

exports.desligar = (request, reply) => {
  Ami.Desligar(request.payload, (resposta) => {
    const ret = {};
    ret.error = resposta.response === 'Error' ? true : false;
    ret.message = resposta.message;
    reply(ret);
  });
};

exports.getTotais = async (request, reply) => {
  const dataIni = request.query.dataIni ? new Date(request.query.dataIni) :  new Date(new Date().setDate(new Date().getDate() - 30))
  const dataFim = request.query.dataFim ? new Date(request.query.dataFim) : new Date()
  const sqlRamais = request.query.ramais ? '= ANY($3::character varying[])' : 'IN (SELECT name from ramais)';
  
  const pars = [dataIni, dataFim]
  if(request.query.ramais) pars.push(request.query.ramais)

  const sql = `SELECT ramal,data,SUM(qtd) as quantidate FROM
  ((select src as ramal,to_char(calldate,'DD-MM-YYYY') as data,count(*) as qtd
  FROM cdr
  where calldate between $1::timestamp AND $2::timestamp AND src ${sqlRamais}
  group by src,data)
  UNION
  (select dst as ramal,to_char(calldate,'DD-MM-YYYY') as data,count(*) as qtd
  FROM cdr
  where calldate between $1::timestamp AND $2::timestamp AND dst ${sqlRamais}
  group by dst,data)) as a
  GROUP by ramal,data
  ORDER by data`

  try {
    const dados = await db.query(sql, pars)
    reply(dados.rows);
  } catch (error) {
    console.log(error)
      reply(Boom.badData(error));
  }

}

exports.schema = {
  ligacao: joi.object().keys({
    id: joi
      .number()
      .required()
      .integer()
      .min(0)
      .description('id do agente')
      .example(1),
    nome: joi
      .string()
      .required()
      .description('nome para login')
      .example('teste'),
    senha: joi
      .string()
      .required()
      .description('senha para login')
      .example('teste'),
    ramal: joi
      .any()
      .description('ramal do agente')
      .example('SIP/1000'),
    token: joi
      .any()
      .description('token de acesso')
      .example('asddsfdsgffdgfdghsdferf'),
    fila: joi
      .any()
      .description('fila (Não é mais usado)')
      .example('nda'),
    token_create: joi
      .any()
      .description('data de criacção do token')
      .example('nda'),
    datastatus: joi
      .any()
      .description('data de criacção do token')
      .example('nda'),
    status: joi
      .any()
      .description('status')
      .example('nda'),
    fullname: joi
      .string()
      .required()
      .description('nome completo do agente')
      .example('Teste da silva'),
    filas: joi
      .array()
      .items(
        joi
          .any()
          .description('nome da fila')
          .example('Atendimento')
      )
  }).label('ligacao'),
  // Using a getter here so that I can refer to `this`, in reference to the character node of the schema object
  get ligacoes() {
    return joi
      .array()
      .items(this.ligacao)
      .label('lista_de_ligacoes')
  }
};
