const Boom = require('boom');
const Ramal = require('../../models/Ramal');
const RamalMulti = require('../../models/RamalMulti');
const db = require('../../util/db');
const namiLib = require('../../plugins/nami');
const nami = require('../../ami/index');

exports.create = async (request, reply) => {
    // Crie a transação
    const transaction = new db.orm.Transaction();
    const RamalMultiT = transaction.models.RamalMulti;
    const RamalT = transaction.models.Ramal;

    try {
        var { ramalMulti, ...ramal } = request.payload;

        // Salva o ramal
        let novo = await RamalT.save(ramal);

        // Se tiver Multi
        if (ramalMulti && ramalMulti.length > 0) {

            // Gera objeto dos ramais multi
            let newMultis = ramalMulti.map(m => {
                return {
                    numero: m.numero,
                    nome: m.nome,
                    senha: ramal.senha,
                    grupo: ramal.grupo,
                    callerid: `${ramal.nome} <${ramal.numero}>`,
                    allow: ramal.allow,
                    insecure: ramal.insecure,
                    subscribecontext: ramal.subscribecontext,
                    pickupgroup: ramal.pickupgroup,
                    callgroup: ramal.callgroup,
                    transport: ramal.transport,
                    call_limit: ramal.call_limit,
                    nat: ramal.nat,
                    mac: ramal.mac,
                    accountcode: ramal.accountcode,
                    hassip: ramal.hassip,
                    dtmfmode: ramal.dtmfmode,
                    language: ramal.language,
                    musiconhold: ramal.musiconhold,
                    encryption: ramal.encryption,
                    avpf: ramal.avpf,
                    icesupport: ramal.icesupport,
                    dtlsenable: ramal.dtlsenable,
                    dtlsverify: ramal.dtlsverify,
                    dtlscertfile: ramal.dtlscertfile,
                    dtlssetup: ramal.dtlssetup,
                    rtcp_mux: ramal.rtcp_mux,
                    xc_tipo: 'multi',
                    tem_multi: false
                }
            });
            // Cria os multi novamente
            await RamalT.save(newMultis);


            // Salva a ordem dos multi
            let om = ramalMulti.map(o => {
                o.ramalID = novo[0].id
                return o;
            });
            await RamalMultiT.save(om);
        }
        await transaction.commit();

        reply(novo[0]);
    } catch (error) {
        await transaction.rollback();
        reply(Boom.badData(error));
    }
};


exports.update = async (request, reply) => {
    //Definição do comando
    let action = new namiLib.Actions.Command();

    // Crie a transação
    const transaction = new db.orm.Transaction();
    const RamalMultiT = transaction.models.RamalMulti;
    const RamalT = transaction.models.Ramal;

    try {
        var { ramalMulti, ...ramal } = request.payload;
        ramal.id = request.params.ID ? request.params.ID : undefined;

        // Apaga ramais multi device        
        await RamalT.query
            .require(false)
            .where(Ramal.where.like({ numero: `${ramal.numero}_%` }))
            .delete();

        // Se tiver Multi
        if (ramal.tem_multi && ramalMulti && ramalMulti.length > 0) {

            // Gera objeto dos ramais multi
            let newMultis = ramalMulti.map(m => {
                return {
                    numero: m.numero,
                    nome: m.nome,
                    senha: ramal.senha,
                    grupo: ramal.grupo,
                    callerid: `${ramal.nome} <${ramal.numero}>`,
                    allow: ramal.allow,
                    insecure: ramal.insecure,
                    subscribecontext: ramal.subscribecontext,
                    pickupgroup: ramal.pickupgroup,
                    callgroup: ramal.callgroup,
                    transport: ramal.transport,
                    call_limit: ramal.call_limit,
                    nat: ramal.nat,
                    mac: ramal.mac,
                    accountcode: ramal.accountcode,
                    hassip: ramal.hassip,
                    dtmfmode: ramal.dtmfmode,
                    language: ramal.language,
                    musiconhold: ramal.musiconhold,
                    encryption: ramal.encryption,
                    avpf: ramal.avpf,
                    icesupport: ramal.icesupport,
                    dtlsenable: ramal.dtlsenable,
                    dtlsverify: ramal.dtlsverify,
                    dtlscertfile: ramal.dtlscertfile,
                    dtlssetup: ramal.dtlssetup,
                    rtcp_mux: ramal.rtcp_mux,
                    xc_tipo: 'multi',
                    tem_multi: false
                }
            });
            // Cria os multi novamente
            await RamalT.save(newMultis);

            // Apaga a ordem dos multi
            await RamalMultiT.query
                .require(false)
                .where({ ramalID: request.params.ID })
                .delete();

            // Salva a ordem dos multi
            await RamalMultiT.save(ramalMulti);
        }

        // Salva o ramal
        let novo = await RamalT.save(ramal);
        await transaction.commit();

        try {
            //AMI Commands
            // Isso desloga o ramal e ele só voltaria se fechar e abrir lovamente...
            //action.command = "sip prune realtime peer " + ramal.numero;
            //nami.send(action);

            action.command = "sip show peer " + ramal.numero + ' load';
            nami.send(action);
        } catch (error) {
            console.log(`Erro no AMI`)
        }

        reply(novo[0]);
    } catch (error) {
        await transaction.rollback();
        reply(Boom.badData(error));
    }
};

exports.getAll = async (request, reply) => {
    try {
        const ramais = await Ramal.query
            .fields(['id', 'numero', 'nome', 'grupo', 'transport', 'useragent', 'lastms', 'ipaddr', 'xc_tipo', 'icesupport'])
            .require(false)
            .fetch();
        reply(ramais);
    } catch (error) {
        reply(Boom.badData(error));
    }
};

exports.getOne = async (request, reply) => {
    try {
        const ramal = await Ramal.query
            .leftJoin(RamalMulti)
            .require(true)
            .where({ id: request.params.ID })
            .fetch();
        reply(ramal[0]);
    } catch (error) {
        reply(Boom.badData(error));
    }
};

exports.delete = async (request, reply) => {
    const transaction = new db.orm.Transaction();
    const RamalMultiT = transaction.models.RamalMulti;
    const RamalT = transaction.models.Ramal;

    try {
        // Ve se tem multi
        const mult = await RamalMultiT.query
            .require(false)
            .where({ ramalID: request.params.ID })
            .fetch();
        if (mult && mult.length > 0) {
            let array_del = mult.map(d => d.numero);
            await RamalT.query
                .require(false)
                .where(Ramal.where.in('name', array_del))
                .delete();
        }


        const ramal = await RamalT.query
            .require(true)
            .where({ id: request.params.ID })
            .delete();

        await transaction.commit();
        reply(ramal);
    } catch (error) {
        await transaction.rollback();
        reply(Boom.badData(error));
    }
};
