const Joi = require('joi');
const handlers = require('./handlers');

const payload_validade = Joi.object({
    numero: Joi.string().required(),
    nome: Joi.string().required(),
    senha: Joi.string().required(),
    grupo: Joi.string().required().description('Grupo de rota do ramal'),
    allow: Joi.string().optional().allow(null).allow(''),
    insecure: Joi.string().optional().allow(null).allow(''),
    subscribecontext: Joi.string().optional().allow(null).allow(''),
    pickupgroup: Joi.string().optional().allow(null).allow(''),
    callgroup: Joi.number().integer().optional().allow(null).allow(''),
    transport: Joi.string().required().valid('udp', 'wss'),
    call_limit: Joi.number().integer().optional().allow(null).allow(''),
    nat: Joi.string().optional().allow(null).allow(''),
    mac: Joi.string().optional().allow(null).allow(''),
    accountcode: Joi.string().optional().allow(null).allow(''),
    dtmfmode: Joi.string().optional().allow(null).allow(''),
    language: Joi.string().optional().allow(null).allow(''),
    musiconhold: Joi.string().optional().allow(null).allow(''),
    callerid: Joi.string().optional().allow(null).allow(''),
    hassip: Joi.string().optional().allow(null).allow('').valid('yes', 'no'),
    encryption: Joi.string().optional().allow(null).allow('').valid('yes', 'no'),
    avpf: Joi.string().optional().allow(null).allow('').valid('yes', 'no'),
    icesupport: Joi.string().optional().allow(null).allow('').valid('yes', 'no'),
    dtlsenable: Joi.string().optional().allow(null).allow('').valid('yes', 'no'),
    dtlsverify: Joi.string().optional().allow(null).allow(''),
    dtlscertfile: Joi.string().optional().allow(null).allow(''),
    dtlssetup: Joi.string().optional().allow(null).allow(''),
    rtcp_mux: Joi.string().optional().allow(null).allow(''),
    xc_tipo: Joi.string().valid('ramal', 'multi').description('Tipo de ramal. Aceita: ramal ou multi'),
    tem_multi: Joi.boolean().optional().allow(null).allow(''),
    multi_regra: Joi.string().optional().allow(null).allow('').description('Regra para distruicao para multi device'),
    ramalMulti: Joi.array().optional().allow(null)
});

module.exports = [
    {
        method: 'GET',
        path: '/api/v2/ramais',
        handler: handlers.getAll,
        config: {
            description: 'Obtem os ramais',
            tags: ['api', 'Ramais'],
            auth: {
                strategy: 'bearer',
                scope: ['app.ramais', 'admin']
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v2/ramal/{ID}',
        handler: handlers.getOne,
        config: {
            description: 'Obtem os dados de um ramal',
            tags: ['api', 'Ramais'],
            auth: {
                strategy: 'bearer',
                scope: ['app.ramais', 'admin']
            },
            validate: {
                params: {
                    ID: Joi.number().integer().required().description('ID do ramal')
                }
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v2/ramal/{ID}',
        handler: handlers.update,
        config: {
            description: 'Atualiza um ramal',
            tags: ['api', 'Ramais'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    ID: Joi.number().integer().required().description('ID do ramal')
                },
                payload: payload_validade
            }
        }
    },
    {
        method: 'POST',
        path: '/api/v2/ramal',
        handler: handlers.create,
        config: {
            description: 'Cria um ramal',
            tags: ['api', 'Ramais'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                payload: payload_validade
            }
        }
    },
    {
        method: 'DELETE',
        path: '/api/v2/ramal/{ID}',
        handler: handlers.delete,
        config: {
            description: 'exclui um ramal',
            tags: ['api', 'Ramais'],
            auth: {
                strategy: 'bearer',
                scope: ['app.ramais', 'admin']
            },
            validate: {
                params: {
                    ID: Joi.number().integer().required()
                }
            }
        }
    },
]