'use strict'

const joi = require('joi');
const Boom = require('boom');
const db = require('../../util/db');
//const squel = require("squel");
const ws = require('../../ws');
const config = require('../../config.json');
const moment = require('moment-timezone');
moment.tz.setDefault(config.servidor.timezone);

exports.create = (request, reply) => {
  const knex = db.knex('public.rotas');
  knex.insert(request.payload)
  .then(function(id) {
    const ret = {};
    ret.error = false;
    ret.name = request.payload.name;
    ret.message = 'Rota id = '+id[0]+' criada com sucesso.';
    reply(ret);
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};

exports.update = (request, reply) => {
  const knex = db.knex('public.rotas');
  knex.update(request.payload);

  knex.where('name', request.params.FilaID)
  .then(function(rows) {
    const ret = {};
    ret.error = false;
    ret.total = rows;
    ret.message = 'Fila name = '+request.params.FilaID+' atualizada com sucesso.';
    reply(ret);
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};

exports.delete = (request, reply) => {
  const knex = db.knex('public.rotas');
  knex.where('name', request.params.FilaID)
  .del()
  .then(function(rows) {
    if(rows > 0){
      const ret = {};
      ret.error = false;
      ret.total = rows;
      ret.message = 'Fila name = '+request.params.FilaID+' apagada com sucesso.';
      reply(ret);
    }
    else {
      reply(Boom.badData("Nenhum resgistro encontrado para name = "+request.params.FilaID));
    }
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};

exports.getall = (request, reply) => {
  const max = request.query.max ? request.query.max : 15;
  const page = request.query.page ? request.query.page : 1;
  const knex = db.knex('public.rotas');

  if(request.query.context) knex.where('context', request.query.context);
  if(request.query.exten) knex.where('exten', request.query.exten);

  if(max === -1){
    //console.log('SQL = '+ knex.toString());
    knex.clearSelect();
    // Parametros de ordenação, fora do count
    if(request.query.order){
      for(let p=0;p<request.query.order.length;++p){
        if(request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
            knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
        }
        else if(request.query.order[p] && request.query.order[p].campo) {
          knex.orderBy(request.query.order[p].campo, 'ASC');
        }
      }
    }
    knex.select('*')
    .then(function(rows) {
      const ret = {};
      ret.error = false;
      ret.full = true;
      ret.total = rows.length;
      ret.message = 'Lista de todas as rotas';
      ret.dados = rows;
      reply(ret);
    })
    .catch(function(error) {
      console.error(error);
      reply(Boom.badData(error));
    });
  }
  else {
    const offset = max * (page - 1);
    //console.log('SQL = '+knex.count('* as total').toString());
    knex.count('* as total')
    .then(function(total) {
      if(total[0].total > 0){
        knex.clearSelect();
        // Parametros de ordenação, fora do count
        if(request.query.order){
          for(let p=0;p<request.query.order.length;++p){
            if(request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
            }
            else if(request.query.order[p] && request.query.order[p].campo) {
              knex.orderBy(request.query.order[p].campo, 'ASC');
            }
          }
        }
        //console.log('SQL = '+knex.select('*').limit(max).offset(offset).toString());
        knex.select('*').limit(max).offset(offset).then(function(rows) {
          const pages = Math.ceil(parseInt(total[0].total) / max);
          const ret = {};
          ret.error = false;
          ret.full = pages === 1 ? true : false;
          ret.page = page;
          ret.max = max;
          ret.offset = offset;
          ret.total = parseInt(total[0].total);
          ret.pages = pages;
          ret.message = 'Lista de rotas paginada';
          ret.dados = rows;
          reply(ret);
        })
        .catch(function(error) {
          console.error(error);
          reply(Boom.badData(error));
        });
      }
      else {
        const ret = {};
        ret.error = true;
        ret.total = 0;
        ret.message = 'Nenhum registro encontrado';
        reply(ret);
      }
    })
    .catch(function(error) {
      console.error(error);
      reply(Boom.badData(error));
    });
  }
};

exports.getOne = (request, reply) => {
  const knex = db.knex('public.rotas');
  knex.where('id', request.params.RotaID);
  knex.select('*')
  .then(function(rows) {
    const ret = {};
    ret.error = false;
    ret.full = true;
    ret.total = rows.length;
    ret.message = 'Dados da rota id = '+request.params.FilaID;
    ret.dados = rows[0];
    reply(ret);
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};

exports.getGrupos = (request, reply) => {
  const max = request.query.max ? request.query.max : 15;
  const page = request.query.page ? request.query.page : 1;
  const knex = db.knex('public.contexto');
  if(request.query.sentido) knex.whereIn('sentido', request.query.sentido);
  if(request.query.nome) knex.whereRaw('LOWER(context) LIKE ?', '%'+decodeURIComponent(request.query.context).toLowerCase()+'%');

  if(max === -1){
    //console.log('SQL = '+ knex.toString());
    knex.clearSelect();
    // Parametros de ordenação, fora do count
    if(request.query.order){
      for(let p=0;p<request.query.order.length;++p){
        if(request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
            knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
        }
        else if(request.query.order[p] && request.query.order[p].campo) {
          knex.orderBy(request.query.order[p].campo, 'ASC');
        }
      }
    }
    knex.select('*')
    .then(function(rows) {
      const ret = {};
      ret.error = false;
      ret.full = true;
      ret.total = rows.length;
      ret.message = 'Lista de todos os grupos de rotas';
      ret.dados = rows;
      reply(ret);
    })
    .catch(function(error) {
      console.error(error);
      reply(Boom.badData(error));
    });
  }
  else {
    const offset = max * (page - 1);
    //console.log('SQL = '+knex.count('* as total').toString());
    knex.count('* as total')
    .then(function(total) {
      if(total[0].total > 0){
        knex.clearSelect();
        // Parametros de ordenação, fora do count
        if(request.query.order){
          for(let p=0;p<request.query.order.length;++p){
            if(request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
            }
            else if(request.query.order[p] && request.query.order[p].campo) {
              knex.orderBy(request.query.order[p].campo, 'ASC');
            }
          }
        }
        //console.log('SQL = '+knex.select('*').limit(max).offset(offset).toString());
        knex.select('*').limit(max).offset(offset).then(function(rows) {
          const pages = Math.ceil(parseInt(total[0].total) / max);
          const ret = {};
          ret.error = false;
          ret.full = pages === 1 ? true : false;
          ret.page = page;
          ret.max = max;
          ret.offset = offset;
          ret.total = parseInt(total[0].total);
          ret.pages = pages;
          ret.message = 'Lista dos grupos de rotas paginada';
          ret.dados = rows;
          reply(ret);
        })
        .catch(function(error) {
          console.error(error);
          reply(Boom.badData(error));
        });
      }
      else {
        const ret = {};
        ret.error = true;
        ret.total = 0;
        ret.message = 'Nenhum registro encontrado';
        reply(ret);
      }
    })
    .catch(function(error) {
      console.error(error);
      reply(Boom.badData(error));
    });
  }
};