'use strict'

const Joi = require('joi');
const handlers = require('./handlers');

const CreateReturnSchema = {
    error: Joi.boolean().example(false).description('Indica se o processo gerou algum erro.'),
    name: Joi.string().example('fila1').description('ID da fila'),
    message: Joi.string().example('Registro criado com aucesso.')
};

const ListaReturnSchema = {
    error: Joi.boolean().example(false).description('Indica se o processo gerou algum erro.'),
    full: Joi.boolean().example(false).description('Indica se o retorno é completo, caso seja false será paginado.'),
    total: Joi.number().integer().example('10').description('Quantidade de registros retornados'),
    page: Joi.number().integer().example('1').description('Pagina atual dos registros'),
    offset: Joi.number().integer().example('1').description('Offset'),
    pages: Joi.number().integer().example('1').description('total de página'),
    max: Joi.number().integer().example('15').description('Máximo de registros por página'),
    pages: Joi.number().integer().example('1').description('total de páginas'),
    message: Joi.string().example('Lista de registros').description('Descrição da lista'),
    dados: Joi.array().items().example([{id: 1, nome: "Registro1"},{id: 2, nome: "Registro2"}]).description('Lista dos registros')
};

module.exports = [
  {
    method: 'GET',
    path: '/api/v2/rotas',
    handler: handlers.getall,
    config: {
      description: 'Obtem as rotas',
      tags: ['api','v2','Rotas'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      response: {
        schema: ListaReturnSchema
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
                campo: Joi.string().description('Nome da coluna'),
                ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          context: Joi.string().description('Filtra por contexto (grupo)'),
          exten: Joi.string().description('Filtra por exten (rota)')
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/rotas/grupos',
    handler: handlers.getGrupos,
    config: {
      description: 'Obtem os grupos de rotas',
      tags: ['api','v2','Rotas'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      response: {
        schema: ListaReturnSchema
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
                campo: Joi.string().description('Nome da coluna'),
                ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          sentido: Joi.array().items(
            Joi.string().valid('Entrada', 'Saida', 'URA', "URA_MENSAGEM").description('Sentido do Grupo de rotas, Entrada=Entrada,Saida=Saída, URA=Ura e URA_MENSAGEM=Ura de mensagem'),
          ),
          nome: Joi.string().description('Filtra por nome')
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/rota/{RotaID}',
    handler: handlers.getOne,
    config: {
      description: 'Obtem os dados da fila',
      tags: ['api','v2','Rotas'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            RotaID: Joi.number().integer().required()
        }
      }
    }
  }, 
];
