const Boom = require('boom');
const AutenticacaoFontes = require('../../models/AutenticacaoFontes');
const Agentes = require('../../models/Agente');
const AgentesFilas = require('../../models/AgenteFilas');
const Filas = require('../../models/Fila');
const Ramal = require('../../models/Ramal');
const timerExpediente = require('../../timer_expediente');
const uuid = require('uuid/v4');
const md5 = require('md5');
const twoFactor = require('node-2fa');
const ldap = require('../../util/ldap');

exports.AgenteLogin = async (request, reply) => {
    try {
        // Busca o agente pelo nome para verificar qual a fonte de autenticação
        let agente = await Agentes.query
            .require(true)
            .first()
            .where({ nome: request.payload.nome })
            .fetch();

        // Se tem tempo maximo de sessão ativo
        if (agente.tempo_maximo_sessao) {
            let terminou = await timerExpediente.verficaTimerAgente(agente, `login`);
            if (terminou) return reply({ error: true, msg: "O seu expediente ja acabou!" });
        }

        // Autenticação local
        if (!agente.autenticacao_fonte) {
            if (agente.senha !== md5(request.payload.senha)) {
                return reply({ error: true, msg: "Dados para login inválidos" });
            }
        }
        else {
            const fonte = await AutenticacaoFontes.query
                .where({ id: agente.autenticacao_fonte })
                .where({ ativo: true })
                .first()
                .require(true)
                .fetch();

            switch (fonte.tipo) {
                case 1: // Xcontact
                    if (agente.senha !== md5(request.payload.senha)) {
                        return reply({ error: true, msg: "Dados para login inválidos" });
                    }
                    break;
                case 2: // LDAP/AD
                    try {
                        agente.ldap_user = await ldap.ldapLogin(request.payload.nome, request.payload.senha, fonte.config);
                    } catch (error) {
                        return reply({ error: true, msg: "Dados para login inválidos" });
                    }
                    break;
                default:
                    return reply({ error: true, msg: "Metodo de autenticação não implementado" });
            }
        }

        if (agente.ramal) {
            let auxLocal = agente.ramal.split(`@`);
            let ramal = auxLocal[0].split('/');
            agente.ramal_numero = ramal[1];
            if (ramal[0] === `SIP`) {
                let ramaldb = await Ramal.query
                    .require(false)
                    .first()
                    .where({ numero: agente.ramal_numero })
                    .fetch();
                if (ramaldb) {
                    agente.ramal_senha = ramaldb.senha;
                    agente.contexto = ramaldb.grupo;
                }
            }
        }

        // Gera o token
        agente.token = uuid();
        agente.token_create = new Date();
        await new Agentes({ id: agente.id, token: agente.token, token_create: agente.token_create }).update()

        let agentes_filas = await AgentesFilas.query
            .require(false)
            .innerJoin(Filas)
            .where({ id_agente: agente.id })
            .fetch();

        agente.filas = agentes_filas.map(f => {
            const desHang = f.fila[0].desabilitar_hangup
            f.instance = f.fila[0].instance
            f.queue_type = f.fila[0].queue_type
            f.fila = f.fila[0].name
            f.desabilitar_hangup = desHang
            return f
        })

        delete agente.senha
        reply(agente);

    } catch (error) {
        console.log('Erro no login do agente: ', error)
        reply({ error: true, msg: "Dados para login inválidos" });
    }
}

exports.logOff = async (request, reply) => {
    try {
        const user = await Supervisores.fetchById(request.params.id);
        user.session_id = null;
        await user.save();
        reply(user);
    } catch (error) {
        reply(Boom.badData(error));
    }
}

exports.getSupervisor = async (request, reply) => {
    try {
        // Se não passar session_id verifica se está nula, se não tiver não deixa logar
        if (!request.payload.session_id) {
            let sup1 = await Supervisores.query
                .where({ nome: request.payload.nome })
                .where(Supervisores.where.isNotNull('session_id'))
                .require(false)
                .first()
                .fetch();
            if (sup1) {
                reply(Boom.badData(`[Erro1] Usuário já logado na máquina de nome ${sup1.pc_login} de IP ${sup1.ip_login}`));
                return;
            }
        }
        // Busca usário pelo nome para verificar qual método de autenticação.
        const query = Supervisores.query;
        query.where({ nome: request.payload.nome });

        let sup = await query.require(true).first().fetch();

        if (request.payload.session_id && sup.session_id && request.payload.session_id !== sup.session_id) {
            reply(Boom.badData(`[Erro2] Usuário já logado na máquina de nome ${sup.pc_login} de IP ${sup.ip_login}`));
            return;
        }

        if (!request.payload.duplo_fator && sup.duplo_fator_secret) {
            const error = Boom.forbidden("Seu usuário precisa de duplo fator");
            error.output.statusCode = 499;
            error.output.payload.duplo_fator = true;
            error.reformat();
            return reply(error);
        }

        if (request.payload.duplo_fator) {
            if (sup.duplo_fator_secret) {
                //console.log(`2fa secret ${sup.duplo_fator_secret} autenticando `)
                const tf = twoFactor.verifyToken(sup.duplo_fator_secret, request.payload.duplo_fator);
                if (tf) {
                    if (tf.delta != 0) {
                        const error = Boom.forbidden("Código de duplo fator inválido");
                        error.output.statusCode = 498;
                        error.output.payload.duplo_fator = true;
                        error.reformat();
                        return reply(error);
                    }
                }
                else {
                    const error = Boom.forbidden("Código de duplo fator inválido");
                    error.output.statusCode = 498;
                    error.output.payload.duplo_fator = true;
                    error.reformat();
                    return reply(error);
                }
            }
            else {
                const error = Boom.forbidden("Código de duplo fator inválido");
                error.output.statusCode = 498;
                error.output.payload.duplo_fator = true;
                error.reformat();
                return reply(error);
            }
        }


        // Autenticação local
        if (!sup.autenticacao_fonte) {
            if (sup.senha !== md5(request.payload.senha)) {
                reply(Boom.badData('[Erro3] Dados para login inválidos'));
                return;
            }
        }
        else {
            const fonte = await AutenticacaoFontes.query
                .where({ id: sup.autenticacao_fonte })
                .where({ ativo: true })
                .first()
                .require(true)
                .fetch();

            switch (fonte.tipo) {
                case 1: // Xcontact
                    if (sup.senha !== md5(request.payload.senha)) {
                        reply(Boom.badData('[Erro4] Dados para login inválidos'));
                        return;
                    }
                    break;
                case 2: // LDAP/AD
                    // Seta para aceitar certificado auto assinado no caso de TLS
                    fonte.config.tlsOptions = { rejectUnauthorized: false };
                    try {
                        sup.ldap_user = await ldapLogin(request.payload.nome, request.payload.senha, fonte.config);
                    } catch (error) {
                        reply(Boom.badData(error));
                        return;
                    }
                    break;
                default:
                    reply(Boom.badData('[Erro5] Metodo de autenticação não implementado'));
                    return;
            }
        }

        // Gera o token
        sup.token = uuid();
        sup.session_id = uuid();
        sup.token_create = new Date();
        sup.pc_login = (request.id && request.id.split(':').length > 0) ? request.id.split(':')[1] : request.id;
        sup.ip_login = request.info.remoteAddress;
        await sup.save();


        const supfilas = await SupervisoresFilas.query
            .where({ id_supervisor: sup.id })
            .require(false)
            .fetch();

        if (supfilas) {
            sup.filas = supfilas.map(f => {
                return f.id_fila;
            })
        }


        const supModulos = await SupervisoresModulos.query
            .innerJoin(Modulos)
            .require(false)
            .where({ id_supervisor: sup.id })
            .fetch();

        if (supModulos) {

            const somodulos = supModulos.map(modulo => {
                let mod = modulo.modulos[0];
                mod.extra = modulo.extra_usuario;
                return mod;
            })

            const parents = somodulos.filter(pa => pa.parent == null).slice(0)
                .sort((a, b) => {
                    return a.ordem > b.ordem ? 1 : -1
                });;
            const childrens = somodulos.filter(pa => pa.parent != null).slice(0)
                .sort((a, b) => {
                    return a.ordem > b.ordem ? 1 : -1
                });;

            const ordemm = parents.concat(childrens);

            const nest = (items, id = null, link = 'parent') =>
                items
                    .filter(item => item[link] === id)
                    .map(item => ({ ...item, modulos: nest(items, item.id) }));

            sup.modulos = nest(ordemm);
        }
        reply(sup);
    } catch (error) {
        reply(Boom.badData('[Erro6] Dados para login inválidos'));
    }
};