'use strict'

const joi = require('joi');
const Boom = require('boom');
const _ = require('lodash');
const CRUD = require('../../../util/CRUD');
const db = require('../../../util/db');
const md5 = require('md5');
const Ami = require('../../../ami');
const config = require('../../../config');
const moment = require('moment-timezone');
moment.tz.setDefault(config.servidor.timezone);
const util = require('../../../util/restbase');
const timerExpediente = require(`../../../timer_expediente`);
//Processa os resultados para obter os dados no formato esperado na interface
function processResult(rows) {

    return _.chain(rows).groupBy("id").map(function (v, i) {
        //Dados necessários para a consulta à Asterisk
        let r = _.get(_.find(v, 'ramal'), 'ramal');
        let f = _.map(v, function (af) {
            return {
                id: _.get(af, 'bid'),
                id_agente: i,
                fila: _.get(af, 'fila'),
                deletar: false,
                status: 0,
            }
        });
        //Dados da Asterisk
        let agente = Ami.getAgenteLogadoByFilaRamal(f, r);
        //Objeto Agente
        return {
            id: i,
            nome: _.get(_.find(v, 'nome'), 'nome'),
            fullname: _.get(_.find(v, 'fullname'), 'fullname'),
            senha: _.get(_.find(v, 'senha'), 'senha'),
            ramal: r,
            token: _.get(_.find(v, 'token'), 'token'),
            token_create: _.get(_.find(v, 'token_create'), 'token_create'),
            status: _.get(_.find(v, 'status'), 'status'),
            datastatus: _.get(_.find(v, 'datastatus'), 'datastatus'),
            modo: _.get(_.find(v, 'modo'), 'modo'),
            filas: f,
            status: {
                codigo: agente && agente.status ? agente.status : 5,
                StatusNome: agente && agente.StatusNome ? agente.StatusNome : 'Indiponivel',
                DataStatus: agente && agente.DataStatus ? agente.DataStatus : moment.tz(config.servidor.timezone).format(),
                pausa: agente && agente.pausa ? agente.pausa : 0,
                pausa_motivo: agente && agente.pausa_motivo ? agente.pausa_motivo : undefined
            }
        }
    }).value();
}

exports.getall = (request, reply) => {
    const max = request.query.max ? request.query.max : 15;
    const page = request.query.page ? request.query.page : 1;
    const knex = db.knex('public.agentes as a')
        .innerJoin('public.agentes_filas as b', 'b.id_agente', 'a.id');

    if (request.query.nome) knex.whereRaw('LOWER(a.fullname) LIKE ?', '%' + decodeURIComponent(request.query.nome).toLowerCase() + '%');
    if (request.query.ramal && request.query.ramal.length > 0) knex.where('a.ramal', request.query.ramal);
    if (request.query.fila) knex.where('b.fila', request.query.fila);


    if (max === -1) {
        knex.clearSelect();
        // Parametros de ordenação, fora do count
        if (request.query.order) {
            for (let p = 0; p < request.query.order.length; ++p) {
                if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                    knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
                }
                else if (request.query.order[p] && request.query.order[p].campo) {
                    knex.orderBy(request.query.order[p].campo, 'ASC');
                }
            }
        }

        knex.select('a.id', 'a.nome', 'a.senha', 'a.ramal', 'a.fullname', 'a.token', 'a.token_create', 'a.status', 'a.datastatus', 'b.id as bid', 'b.fila')
            .then(function (rows) {
                let prs = processResult(rows);
                const ret = {};
                ret.error = false;
                ret.full = true;
                ret.total = prs.length;
                ret.message = 'Lista de todos os agentes';
                ret.dados = prs
                reply(ret);
            })
            .catch(function (error) {
                console.error(error);
                reply(Boom.badData(error));
            });
    }
    else {
        const offset = max * (page - 1);
        knex.countDistinct({ total: 'a.id' })
            .then(function (total) {
                if (total[0].total > 0) {
                    knex.clearSelect();
                    // Parametros de ordenação, fora do count
                    if (request.query.order) {
                        for (let p = 0; p < request.query.order.length; ++p) {
                            if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                                knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
                            }
                            else if (request.query.order[p] && request.query.order[p].campo) {
                                knex.orderBy(request.query.order[p].campo, 'ASC');
                            }
                        }
                    }
                    knex.select('a.id', 'a.nome', 'a.senha', 'a.ramal', 'a.fullname', 'a.token', 'a.token_create', 'a.status', 'a.datastatus', 'b.id as bid', 'b.fila').limit(max).offset(offset).then(function (rows) {
                        const pages = Math.ceil(parseInt(total[0].total) / max);
                        let prs = processResult(rows);
                        const ret = {};
                        ret.error = false;
                        ret.full = pages === 1 ? true : false;
                        ret.page = page;
                        ret.max = max;
                        ret.offset = offset;
                        ret.total = parseInt(total[0].total);
                        ret.pages = pages;
                        ret.message = 'Lista de agentes paginada';
                        ret.dados = prs;
                        if (request.query.return === 'array') reply(rows);
                        else reply(ret);
                    })
                        .catch(function (error) {
                            console.error(error);
                            reply(Boom.badData(error));
                        });
                }
                else {
                    const ret = {};
                    ret.error = true;
                    ret.total = 0;
                    ret.message = 'Nenhum registro encontrado';
                    reply(ret);
                }
            })
            .catch(function (error) {
                console.error(error);
                reply(Boom.badData(error));
            });
    }
};

exports.getAgente = (request, reply) => {
    const knex = db.knex('public.agentes as a')
        .innerJoin('public.agentes_filas as b', 'b.id_agente', 'a.id');
    knex.where('a.id', request.params.ID);
    knex.select('a.id', 'a.nome', 'a.senha', 'a.ramal', 'a.fullname', 'a.token', 'a.token_create', 'a.status', 'a.datastatus', 'a.modo', 'b.id as bid', 'b.fila')
        .then(function (rows) {
            let prs = processResult(rows);
            const ret = {};
            ret.error = false;
            ret.full = true;
            ret.total = prs.length;
            ret.message = 'Dados do agente de ID = ' + request.params.ID;
            ret.dados = prs[0];
            reply(ret);
        })
        .catch(function (error) {
            console.error(error);
            reply(Boom.badData(error));
        });
};

exports.create = (request, reply) => {
    let idRet = undefined;
    const knex = db.knex;
    knex.transaction(function (t) {
        return knex('agentes')
            .transacting(t)
            .returning('id')
            .insert({
                nome: request.payload.nome,
                fullname: request.payload.fullname.trim(),
                ramal: request.payload.ramal,
                status: request.payload.status ? request.payload.status.codigo : null,
                datastatus: request.payload.status ? moment.format() : null,
                tempo_maximo_sessao: request.payload.tempo_maximo_sessao,
                tempo_auto_pausa: request.payload.tempo_auto_pausa,
                senha: md5(request.payload.senha),
                token: md5(util.GenerateUID()),
                modo: request.payload.modo,
                abrir_url: request.payload.abrir_url,
                atalho_url: request.payload.atalho_url
            })
            .then(async function (id) {
                idRet = id;
                //Assync não se dá bem com forEach
                for (let i = 0; i < request.payload.filas.length; i++) {
                    await knex('agentes_filas')
                        .transacting(t)
                        .insert({ id_agente: parseInt(id), fila: request.payload.filas[i].fila });
                }
            })
            .then(t.commit)
            .catch(function (e) {
                throw e;
            })
            .then(function () {
                const ret = {};
                ret.error = false;
                ret.message = `Agente id:${idRet}, criado com sucesso!`;
                reply(ret);
            })
            .catch(function (e) {
                t.rollback();
                reply(Boom.badData(e));
            });
    });
};

exports.update = (request, reply) => {
    const knex = db.knex;
    const upa = {
        nome: request.payload.nome,
        fullname: request.payload.fullname.trim(),
        ramal: request.payload.ramal,
        modo: request.payload.modo,
        max_chat_atende: request.payload.max_chat_atende,
        visualiza_todos_clientes: request.payload.visualiza_todos_clientes,
        edita_rota: request.payload.edita_rota,
        assinatura: request.payload.assinatura,
        extra1: request.payload.extra1,
        extra2: request.payload.extra2,
        tempo_maximo_sessao: request.payload.tempo_maximo_sessao,
        tempo_auto_pausa: request.payload.tempo_auto_pausa,
        abrir_url: request.payload.abrir_url,
        atalho_url: request.payload.atalho_url
    };

    if (request.payload.status) {
        upa.status = request.payload.status.codigo;
        upa.datastatus = moment(request.payload.status.DataStatus).format();
    }

    if (request.payload.mudasenha) {
        upa.senha = md5(request.payload.senha);
        upa.token = md5(util.GenerateUID());
    }

    knex.transaction(function (t) {
        return knex('agentes')
            .transacting(t)
            .update(upa)
            .where('id', request.params.ID)
            .then(function () {
                return knex('agentes_filas')
                    .transacting(t)
                    .where('id_agente', request.params.ID)
                    .del()
                    .then(async function () {
                        //Assync não se dá bem com forEach
                        for (let i = 0; i < request.payload.filas.length; i++) {
                            if (!request.payload.filas[i].deletar) {
                                await knex('agentes_filas')
                                    .transacting(t)
                                    .insert({ id_agente: request.params.ID, fila: request.payload.filas[i].fila })
                                    .catch(function (e) {
                                        throw e;
                                    });
                            }
                        }
                    })
                    .then(t.commit)
                    .catch(function (error) {
                        t.rollback();
                        throw error;
                    });
            })
            .catch(function (e) {
                t.rollback();
                throw e;
            })
    })
        .then(function () {
            const ret = {};
            ret.error = false;
            ret.message = `Agente id:${request.payload.id}, atualizado com sucesso!`;
            reply(ret);
            if (upa.tempo_maximo_sessao) {
                timerExpediente.changeTimerProd({ fullname: upa.fullname });
            }

        })
        .catch(function (e) {
            reply(Boom.badData(e));
        });
};

exports.delete = (request, reply) => {
    const knex = db.knex;
    knex.transaction(function (t) {
        return knex('agentes_filas')
            .transacting(t)
            .where('id_agente', request.params.ID)
            .del()
            .then(function () {
                return knex('agentes')
                    .transacting(t)
                    .where('id', request.params.ID)
                    .del()
                    .then(t.commit)
                    .catch(function (error) {
                        throw error;
                    });
            })
            .catch(function (e) {
                t.rollback();
                throw e;
            })
    })
        .then(function () {
            const ret = {};
            ret.error = false;
            ret.message = `Agente id:${request.params.ID}, atualizado com sucesso!`;
            reply(ret);
        })
        .catch(function (e) {
            reply(Boom.badData(e));
        });
};

exports.getAgenteByRamal = (request, reply) => {
    const knex = db.knex('public.agentes as a')
        .innerJoin('public.agentes_filas as b', 'b.id_agente', 'a.id');
    knex.where('ramal', request.params.AgenteRamal);
    knex.select('a.id', 'a.nome', 'a.senha', 'a.ramal', 'a.fullname', 'a.token', 'a.token_create', 'a.status', 'a.datastatus', 'b.fila')
        .then(function (rows) {
            let prs = processResult(rows);
            const ret = {};
            ret.error = false;
            ret.full = true;
            ret.total = prs.length;
            ret.message = 'Dados do agente pelo ramal = ' + request.params.AgenteRamal;
            ret.dados = prs;
            reply(ret);
        })
        .catch(function (error) {
            console.error(error);
            reply(Boom.badData(error));
        });
};

exports.AgenteLogin = (request, reply) => {
    const knex = db.knex('public.agentes');
    knex.where('id', request.params.AgenteID);
    knex.select('id', 'nome as login,', 'fullname as nome', 'ramal')
        .then(function (rows) {
            const agente = {
                ramal: rows[0].ramal,
                fila: request.payload.fila,
                pausado: request.payload.pausado ? request.payload.pausado : false,
                nome: rows[0].nome,
            };
            Ami.QueueAdd(agente, (resposta) => {
                const ret = {};
                ret.error = resposta.response === 'Error' ? true : false;
                ret.message = resposta.message;
                reply(ret);
            });
        })
        .catch(function (error) {
            console.error(error);
            reply(Boom.badData(error));
        });
};

exports.AgenteLogoff = (request, reply) => {
    const knex = db.knex('public.agentes');
    knex.where('id', request.params.ID);
    knex.select('id', 'nome as login,', 'fullname as nome', 'ramal')
        .then(async function (rows) {
            const ret = {};
            ret.error = false;
            ret.message = "";
            let fa = [];
            if (request.payload && request.payload.filas)
                fa = request.payload.filas;
            else {
                //Busca as filas atuais
                let filas = Ami.getFilas();
                //Filtra as do agente em questão
                fa = filas.filter(function (x) { return x.agentes.filter(function (y) { return y.ramal == rows[0].ramal }) });
                //prepara a saída para um formato comum
                if (fa && fa.length > 0) fa = fa.map((x) => x.nome);
            }

            for (let i = 0; i < fa.length; i++) {
                let agente = {
                    ramal: rows[0].ramal,
                    fila: fa[i],
                };
                await Ami.QueueRemove(agente, (resposta) => {
                    //mantém o status até encontrar um erro
                    ret.error = resposta.response === 'Error' ? true : ret.error;
                    ret.message += resposta.message + ' ';
                });
            }
            //Retorna o resultado
            reply(ret);
        })
        .catch(function (error) {
            console.error(error);
            reply(Boom.badData(error));
        });
};

exports.AgenteAlteraSenha = (request, reply) => {
    const knex = db.knex;

    const upa = {
        senha: md5(request.payload.newPassword),
        token: md5(util.GenerateUID())
    }

    knex.transaction(function (t) {
        if (request.newPassword != request.newConfirm) throw "A nova Senha não é igual a sua Confirmação!";
        return knex('agentes')
            .transacting(t)
            .select("*")
            .where('id', request.params.ID)
            .then(function (rows) {
                //Verificamos se a senha atual é a senha atual
                if (rows[0].senha != md5(request.payload.oldPassword)) throw "A senha atual não confere. Verifique.";
                return knex('agentes')
                    .transacting(t)
                    .where('id', request.params.ID)
                    .update(upa)
                    .then(t.commit)
                    .catch(function (error) {
                        throw error;
                    });
            })
            .catch(function (e) {
                throw e;
            })
    })
        .then(function () {
            const ret = {};
            ret.error = false;
            ret.message = `Agente id:${request.payload.id}, atualizado com sucesso!`;
            reply(ret);
        })
        .catch(function (e) {
            reply(Boom.badData(e));
        });
};

exports.AgentePausa = (request, reply) => {
    const knex = db.knex('public.agentes');
    knex.where('id', request.params.AgenteID);
    knex.select('id', 'nome as login,', 'fullname as nome', 'ramal')
        .then(function (rows) {
            const agente = {
                ramal: rows[0].ramal,
                fila: request.payload.fila,
                motivo: request.payload.motivo
            };
            Ami.QueuePause(agente, (resposta) => {
                const ret = {};
                ret.error = resposta.response === 'Error' ? true : false;
                ret.message = resposta.message;
                reply(ret);
            });
        })
        .catch(function (error) {
            console.error(error);
            reply(Boom.badData(error));
        });
};

exports.AgenteTiraPausa = (request, reply) => {
    const knex = db.knex('public.agentes');
    knex.where('id', request.params.AgenteID);
    knex.select('id', 'nome as login,', 'fullname as nome', 'ramal')
        .then(function (rows) {
            const agente = {
                ramal: rows[0].ramal,
                fila: request.payload.fila
            };
            Ami.QueueUnpause(agente, (resposta) => {
                const ret = {};
                ret.error = resposta.response === 'Error' ? true : false;
                ret.message = resposta.message;
                reply(ret);
            });
        })
        .catch(function (error) {
            console.error(error);
            reply(Boom.badData(error));
        });
};

exports.AgenteLoginRamal = (request, reply) => {
    const agente = {
        ramal: request.params.AgenteRamal,
        fila: request.payload.fila,
        pausado: request.payload.pausado ? request.payload.pausado : false,
        nome: request.payload.nome,
    };
    Ami.QueueAdd(agente, (resposta) => {
        const ret = {};
        ret.error = resposta.response === 'Error' ? true : false;
        ret.message = resposta.message;
        reply(ret);
    });
};

exports.AgenteLogoffRamal = (request, reply) => {
    const agente = {
        ramal: request.params.AgenteRamal,
        fila: request.payload.fila,
    };
    Ami.QueueRemove(agente, (resposta) => {
        const ret = {};
        ret.error = resposta.response === 'Error' ? true : false;
        ret.message = resposta.message;
        reply(ret);
    });
};

exports.AgentePausaRamal = (request, reply) => {
    const agente = {
        ramal: request.params.AgenteRamal,
        fila: request.payload.fila,
        motivo: request.payload.motivo
    };
    Ami.QueuePause(agente, (resposta) => {
        const ret = {};
        ret.error = resposta.response === 'Error' ? true : false;
        ret.message = resposta.message;
        reply(ret);
    });
};

exports.AgenteTiraPausaRamal = (request, reply) => {
    const agente = {
        ramal: request.params.AgenteRamal,
        fila: request.payload.fila
    };
    Ami.QueueUnpause(agente, (resposta) => {
        const ret = {};
        ret.error = resposta.response === 'Error' ? true : false;
        ret.message = resposta.message;
        reply(ret);
    });
};

exports.getEvents = (request, reply) => {
    const crud = new CRUD('public.queue_events AS eventos');
    crud.filtrosraw = [];
    crud.filtros = [];
    if (request.query.eventos) crud.filtrosraw.push('eventos.event = ANY(ARRAY ' + JSON.stringify(request.query.eventos).replace(/"/g, "'") + ')');
    if (request.query.filas) crud.filtrosraw.push('eventos.queue = ANY(ARRAY ' + JSON.stringify(request.query.filas).replace(/"/g, "'") + ')');
    if (request.query.agentes) crud.filtrosraw.push('eventos.agent = ANY(ARRAY ' + JSON.stringify(request.query.agentes).replace(/"/g, "'") + ')');
    if (request.query.grupos) crud.filtrosraw.push('eventos.envent_group = ANY(ARRAY[' + request.query.grupos + '])');
    if (request.query.data_ini) crud.filtros.push({ campo: 'eventos.event_start', operador: '>=', valor: moment(request.query.data_ini).format('YYYY-MM-DDTHH:mm:ss') + 'Z' });
    if (request.query.data_fim) crud.filtros.push({ campo: 'eventos.event_start', operador: '<=', valor: moment(request.query.data_fim).format('YYYY-MM-DDTHH:mm:ss') + 'Z' });
    if (request.query.order) crud.order = request.query.order;

    crud.leftjoin = [
        { tabela: "event_group AS grupo", campo1: "eventos.envent_group", campo2: "grupo.id" }
    ];

    crud.select = [
        'eventos.*',
        'grupo.name as grupo_nome',
        'grupo.icon as grupo_icone'
    ];

    crud.getAll(request, reply);
};

exports.all = (request, reply) => {
    const knex = db.knex('public.agentes');

    knex.distinct('a.id', db.knex.raw("'A' || a.id as key"), 'a.fullname',
        db.knex.raw('ARRAY(SELECT fila FROM public.agentes_filas WHERE id_agente=a.id) as filas'))
        .from('public.agentes as a');
    if (request.query.filas) {
        knex.innerJoin('public.agentes_filas as af', 'af.id_agente', 'a.id')
            .whereRaw('af.fila = ANY(ARRAY ' + JSON.stringify(request.query.filas).replace(/"/g, "'") + ')');
    }
    knex.union(function () {
        this.select('uniqueid as id', db.knex.raw("'M' || uniqueid as key"), 'membername as fullname',
            db.knex.raw('ARRAY[queue_name] as filas'))
            .from('public.queue_member_table');
        if (request.query.filas) this.whereRaw('queue_name = ANY(ARRAY ' + JSON.stringify(request.query.filas).replace(/"/g, "'") + ')');
    })
        .then(function (rows) {
            const ret = {};
            ret.error = false;
            ret.full = true;
            if (rows && rows.length > 0) {
                ret.total = rows.length;
                ret.dados = rows;
            }
            else {
                ret.total = 0;
                ret.message = 'Não foram encontrados dados de agentes ou agentes estáticos.';
                ret.dados = [];
            }
            reply(ret);
        })
        .catch(function (error) {
            console.error(error);
            reply(Boom.badData(error));
        });
};

exports.schema = {
    agente: joi.object().keys({
        id: joi
            .number()
            .required()
            .integer()
            .min(0)
            .description('id do agente')
            .example(1),
        nome: joi
            .string()
            .required()
            .description('nome para login')
            .example('teste'),
        senha: joi
            .string()
            .required()
            .description('senha para login')
            .example('teste'),
        ramal: joi
            .any()
            .description('ramal do agente')
            .example('SIP/1000'),
        token: joi
            .any()
            .description('token de acesso')
            .example('asddsfdsgffdgfdghsdferf'),
        fila: joi
            .any()
            .description('fila (Não é mais usado)')
            .example('nda'),
        token_create: joi
            .any()
            .description('data de criacção do token')
            .example('nda'),
        datastatus: joi
            .any()
            .description('data de criacção do token')
            .example('nda'),
        status: joi
            .any()
            .description('status')
            .example('nda'),
        fullname: joi
            .string()
            .required()
            .description('nome completo do agente')
            .example('Teste da silva'),
        modo: joi
            .string()
            .required()
            .valid('F', 'T')
            .description('Modo de operação: F = Fila Unica, T = Multi fila.')
            .example('F'),
        filas: joi
            .array()
            .items(
                joi
                    .any()
                    .description('nome da fila')
                    .example('Atendimento')
            )
    }).label('agente'),
    // Using a getter here so that I can refer to `this`, in reference to the character node of the schema object
    get agentes() {
        return joi
            .array()
            .items(this.agente)
            .label('lista_de_agentes')
    }
};
