'use strict'

const Joi = require('joi');
const CRUD = require('../../../util/CRUD');
const util = require('../../../util/restbase');

//Conjunto de dados padrão + específico
const ListaReturnSchema = util.CreateReturnSchema({dados: Joi.array().items().example([{id: 1, nome: "Registro1"},{id: 2, nome: "Registro2"}]).description('Lista dos registros')});
const operationData = {
      membername: Joi.string().required().label("Nome").description("Nome do Agente"),
      interface: Joi.string().required().label("Ramal").description("Ramal do agente"),
      queue_name: Joi.string().optional().allow(null).label("Fila").description("Fila na qual o agente estará lotado"),
};

module.exports = [
    {
      method: 'GET',
      path: '/api/v3/agente_estatico',
      handler: (request, reply) => {
        const crud = new CRUD('public.queue_member_table');      
        crud.filtros = [];
        if(request.query.nome) crud.filtros.push({campo: 'LOWER(membername)', operador: 'LIKE', valor: '%'+decodeURIComponent(request.query.nome).toLowerCase()+'%'});
        if(request.query.ramal) crud.filtros.push({campo: 'LOWER(interface)', operador: '=', valor: decodeURIComponent(request.query.ramal).toLowerCase()});
        if(request.query.fila) crud.filtros.push({campo: 'LOWER(queue_name)', operador: '=', valor: decodeURIComponent(request.query.fila).toLowerCase()});
        crud.getAll(request, reply);
      },
      config: {
        description: 'Pesqusia os agentes estáticos cadastrados. Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
        tags: ['api','Agente Estático'],
        auth: {
            strategy: 'bearer',
            scope: 'user' // or [ 'user', 'admin' ]
        },
        response: {
          schema: ListaReturnSchema
        },
        validate: {
          query: {
            max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
            page: Joi.number().integer().default(1).description('Página'),
            order: Joi.array().items(
              Joi.object().keys({
                  campo: Joi.string().description('Nome da coluna'),
                  ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
              })
            ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
            nome: Joi.string().description('Filtra por nome'),
            ramal: Joi.string().description('Filtra por ramal'),
            fila: Joi.string().description('Filtra por fila'),
          }
        }
      }
    },
    {
      method: 'GET',
      path: '/api/v3/agente_estatico/{ID}',
      handler: (request, reply) => {
        const crud = new CRUD('public.queue_member_table');
        crud.id = 'uniqueid';
        crud.getOne(request, reply);
      },
      config: {
        description: 'Obtém os dados do agente estático',
        tags: ['api','Agente Estático'],
        auth: {
            strategy: 'bearer',
            scope: 'user' // or [ 'user', 'admin' ]
        },
        validate: {
          params: {
            ID: Joi.number().integer().required()
          }
        }
      }
    },
    {
        method: 'POST',
        path: '/api/v3/agente_estatico',
        handler: (request, reply) => {
            const crud = new CRUD('public.queue_member_table');
            crud.id = 'uniqueid';
            crud.create(request, reply);
        },
        config: {
          description: 'Cria um novo agente estático',
          tags: ['api','Agente Estático'],
          auth: {
              strategy: 'bearer',
              scope: 'user' // or [ 'user', 'admin' ]
          },
          validate: {
            payload: operationData
          }
        }
    },
    {
        method: 'PUT',
        path: '/api/v3/agente_estatico/{ID}',
        handler: (request, reply) => {
            const crud = new CRUD('public.queue_member_table');
            crud.id = 'uniqueid';
            crud.update(request, reply);
          },
        config: {
          description: 'Altera os dados do agente estático',
          tags: ['api','Agente Estático'],
          auth: {
              strategy: 'bearer',
              scope: 'user' // or [ 'user', 'admin' ]
          },
          validate: {
            params: {
              ID: Joi.number().integer().required()
            },
            payload: operationData
          }
        }
    },
    {
        method: 'DELETE',
        path: '/api/v3/agente_estatico/{ID}',
        handler: (request, reply) => {
            const crud = new CRUD('public.queue_member_table');
            crud.id = 'uniqueid';
            crud.delete(request, reply);
          },
        config: {
          description: 'Remove o agente estático indicado',
          tags: ['api','Agente Estático'],
          auth: {
              strategy: 'bearer',
              scope: 'user' // or [ 'user', 'admin' ]
          },
          validate: {
            params: {
              ID: Joi.number().integer().required()
            }
          }
        }
    },
];