'use strict'

const joi = require('joi');
const Boom = require('boom');
const db = require('../../../util/db');
//const squel = require("squel");
const ws = require('../../../ws');
const config = require('../../../config.json');
const moment = require('moment-timezone');
moment.tz.setDefault(config.servidor.timezone);

exports.getall = (request, reply) => {
  const max = request.query.max ? request.query.max : 15;
  const page = request.query.page ? request.query.page : 1;
  const knex = db.knex('public.canais_gsm as c')
  .innerJoin('public.operadoras as o','c.id_operadora','o.id')
  .innerJoin('public.canais_gsm_grupos as g','c.id_grupo','g.id')
  .select('c.*','g.nome as grupo', 'o.nome as operadora');
  //TODO: Aqui se utiliza uma View para reduzir e aninhar a tablea de filhos (n-n) pin_grupos. Talvez possamos usar Objection ou outro ORM no futuro.
  if(request.query.canal) knex.whereRaw('LOWER(c.canal) LIKE ?', '%'+decodeURIComponent(request.query.canal).toLowerCase()+'%');
  if(request.query.descricao) knex.whereRaw('LOWER(c.descricao) LIKE ?', '%'+decodeURIComponent(request.query.descricao).toLowerCase()+'%');
  
  if(max === -1){
    //console.log('SQL = '+ knex.toString());
    knex.clearSelect();
    // Parametros de ordenação, fora do count
    if(request.query.order){
      for(let p=0;p<request.query.order.length;++p){
        if(request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
            knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
        }
        else if(request.query.order[p] && request.query.order[p].campo) {
          knex.orderBy(request.query.order[p].campo, 'ASC');
        }
      }
    }
    knex.select('*')
    .then(function(rows) {
      const ret = {};
      ret.error = false;
      ret.full = true;
      ret.total = rows.length;
      ret.message = 'Lista de todos os Canais GSM';
      ret.dados = rows;
      reply(ret);
    })
    .catch(function(error) {
      console.error(error);
      reply(Boom.badData(error));
    });
  }
  else {
    const offset = max * (page - 1);
    //console.log('SQL = '+knex.count('* as total').toString());
    knex.clearSelect();
    knex.count('* as total')
    .then(function(total) {
      if(total[0].total > 0){
        knex.clearSelect();
        // Parametros de ordenação, fora do count
        if(request.query.order){
          for(let p=0;p<request.query.order.length;++p){
            if(request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
            }
            else if(request.query.order[p] && request.query.order[p].campo) {
              knex.orderBy(request.query.order[p].campo, 'ASC');
            }
          }
        }
        //console.log('SQL = '+knex.select('*').limit(max).offset(offset).toString());
        knex.select('c.*','g.nome as grupo', 'o.nome as operadora').limit(max).offset(offset).then(function(rows) {
          const pages = Math.ceil(parseInt(total[0].total) / max);
          const ret = {};
          ret.error = false;
          ret.full = pages === 1 ? true : false;
          ret.page = page;
          ret.max = max;
          ret.offset = offset;
          ret.total = parseInt(total[0].total);
          ret.pages = pages;
          ret.dados = rows;
          ret.message = 'Lista de Canais, paginada';
          reply(ret);
        })
        .catch(function(error) {
          console.error(error);
          reply(Boom.badData(error));
        });
      }
      else {
        const ret = {};
        ret.error = true;
        ret.total = 0;
        ret.message = 'Nenhum registro encontrado';
        reply(ret);
      }
    })
    .catch(function(error) {
      console.error(error);
      reply(Boom.badData(error));
    });
  }
};

exports.zeraContadores = (request, reply) => {
  const knex = db.knex('public.canais_gsm');
  knex.where('id', request.params.ID);
  knex.update({
    enviados: 0,
    recebidos: 0
  })
  .then(function(rows) {
    const ret = {};
    ret.error = false;
    ret.full = true;
    ret.total = rows.length;
    ret.message = 'Contadores reiniciados para o canal de id = '+request.params.ID;
    ret.dados = rows[0];
    reply(ret);
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};
