'use strict'

const Boom = require('boom');
const db = require('../../../util/db');
const restbase = require('../../../util/restbase');
const Paginacao = require('../../../util/Paginacao');
const config = require('../../../config.json');
const moment = require('moment-timezone');
moment.tz.setDefault(config.servidor.timezone);

exports.create = (request, reply) => {
  const knex = db.knex('public.queue_table');
  knex.insert(request.payload)
  .then(function(id) {
    const ret = {};
    ret.error = false;
    ret.name = request.payload.name;
    ret.message = 'Campanha id = '+id[0]+' criada com sucesso.';
    reply(ret);
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};

exports.update = (request, reply) => {
  const knex = db.knex('public.queue_table');
  knex.update(request.payload);

  knex.where('name', request.params.FilaID)
  .then(function(rows) {
    const ret = {};
    ret.error = false;
    ret.total = rows;
    ret.message = 'Fila name = '+request.params.FilaID+' atualizada com sucesso.';
    reply(ret);
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};

exports.delete = (request, reply) => {
  const knex = db.knex('public.queue_table');
  knex.where('name', request.params.FilaID)
  .del()
  .then(function(rows) {
    if(rows > 0){
      const ret = {};
      ret.error = false;
      ret.total = rows;
      ret.message = 'Fila name = '+request.params.FilaID+' apagada com sucesso.';
      reply(ret);
    }
    else {
      reply(Boom.badData("Nenhum registro encontrado para name = "+request.params.FilaID));
    }
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};

exports.getall = (request, reply) => {
  const max = request.query.max ? request.query.max : 15;
  const page = request.query.page ? request.query.page : 1;
  const knex = db.knex('public.queue_table');
  const pag = new Paginacao(max,page);

  if(request.query.numero) knex.where('numero', request.query.numero);
  if(request.query.name) knex.whereRaw('LOWER(name) LIKE ?', '%'+decodeURIComponent(request.query.name).toLowerCase()+'%');
  if(request.query.context) knex.whereRaw('LOWER(context) LIKE ?', '%'+decodeURIComponent(request.query.context).toLowerCase()+'%');

  pag.order = request.query.order;
  pag.knex = knex;

  pag.getRet((ret, err) => {
      if(err){
        console.log('Opa, tivemos um erro. Erro = '+err);
        reply(Boom.badData(err));
      }
      else {
        reply(ret);
      }      
  });
};

exports.getLigacoes = (request, reply) => {
  const max = request.query.max ? request.query.max : 15;
  const page = request.query.page ? request.query.page : 1;
  const knex = db.knex('public.queue_calls');
  const pag = new Paginacao(max,page);

  if(request.query.data_ini) knex.where('queue_calls.calldate', '>=', moment(request.query.data_ini).format('YYYY-MM-DDTHH:mm:ss')+'Z');
  if(request.query.data_fim) knex.where('queue_calls.calldate', '<=', moment(request.query.data_fim).format('YYYY-MM-DDTHH:mm:ss')+'Z');
  if(request.query.status && request.query.status.length > 0) knex.whereRaw("queue_calls.status = ANY (ARRAY "+JSON.stringify(request.query.status).replace(/"/g, "'")+")");
  if(request.query.agentes && request.query.agentes.length > 0) knex.whereRaw("queue_calls.agent = ANY (ARRAY "+JSON.stringify(request.query.agentes).replace(/"/g, "'")+")");
  if(request.query.filas && request.query.filas.length > 0) knex.whereRaw("queue_calls.queuename = ANY (ARRAY "+JSON.stringify(request.query.filas).replace(/"/g, "'")+")");
  if(request.query.classificacao && request.query.classificacao.length > 0) knex.whereRaw("classificacoes.nome = ANY (ARRAY "+JSON.stringify(request.query.classificacao).replace(/"/g, "'")+") OR cla2.nome = ANY (ARRAY "+JSON.stringify(request.query.classificacao).replace(/"/g, "'")+")");
  if(request.query.subclassificacao && request.query.subclassificacao.length > 0) knex.whereRaw("subclassificacao.nome = ANY (ARRAY "+JSON.stringify(request.query.subclassificacao).replace(/"/g, "'")+") OR sub2.nome = ANY (ARRAY "+JSON.stringify(request.query.subclassificacao).replace(/"/g, "'")+")");
  if(request.query.sentido) knex.where('queue_calls.inout', request.query.sentido);
  if(request.query.protocolo) knex.where('cdr_classificacoes.protocolo', request.query.protocolo);
  if(request.query.finalizadapor) knex.where('queue_calls.terminatedby', request.query.finalizadapor);
  if(request.query.numero) knex.whereRaw('queue_calls.numero LIKE ?', '%'+decodeURIComponent(request.query.numero)+'%');
  if(request.query.identclient) knex.whereRaw(restbase.buildSearchExpression('identclient', request.query.identclientc, request.query.identclient));

  knex.leftJoin('cdr_gravacao', function() {
    this.on('queue_calls.callid', '=', 'cdr_gravacao.uniqueid').andOn('queue_calls.queuename', '=', 'cdr_gravacao.queuename')
  });
  knex.leftJoin('cdr_classificacoes', 'queue_calls.callid', 'cdr_classificacoes.uniqueid');
  knex.leftJoin('classificacoes', 'cdr_classificacoes.id_classificacao', 'classificacoes.id');
  knex.leftJoin('subclassificacao', 'cdr_classificacoes.id_subclassificacao', 'subclassificacao.id');

  //knex.leftJoin('cdr_classificacoes as c2', 'queue_calls.callid2', 'c2.uniqueid');
  //knex.leftJoin('classificacoes as cla2', 'c2.id_classificacao', 'cla2.id');
  //knex.leftJoin('subclassificacao as sub2', 'c2.id_subclassificacao', 'sub2.id');

  pag.select = ['queue_calls.callid',
  'queue_calls.calldate',
  'queue_calls.queuename as fila',
  'queue_calls.agent as agente',
  'queue_calls.numero',
  'queue_calls.status',
  db.knex.raw("TO_CHAR((queue_calls.holdtime || ' second'::text)::interval,'HH24:MI:SS') as espera"),
  db.knex.raw("TO_CHAR((queue_calls.calltime || ' second'::text)::interval,'HH24:MI:SS') as duracao"),
  'queue_calls.terminatedby as finalizadapor', 
  'queue_calls.position as posicao_final', 
  'queue_calls.origposition as posicao_entrada', 
  'queue_calls.inout as sentido', 
  'queue_calls.transferredto as transferida_para',
  'queue_calls.idcliente',
  'classificacoes.id as id_classificacao',
  'classificacoes.nome as classificacao',
  'cdr_classificacoes.obs as observacao',
  'cdr_classificacoes.protocolo',
  'cdr_classificacoes.id_subclassificacao',
  'subclassificacao.nome as subclassificacao',
  //'cla2.id as id_classificacao2',
  //'cla2.nome as classificacao2',
  //'c2.obs as observacao2',
  //'c2.protocolo as protocolo2',
  //'c2.id_subclassificacao as id_subclassificacao2',
  //'sub2.nome as subclassificacao2',
  'cdr_gravacao.link as gravacao'];
  
  pag.order = request.query.order;
  pag.knex = knex;

  pag.getRet((ret, err) => {
      if(err){
        console.log('Opa, tivemos um erro. Erro = '+err);
        reply(Boom.badData(err));
      }
      else {
        reply(ret);
      }      
  });
};

exports.getCliente = (request, reply) => {
  const knex = db.knex('public.queue_calls')
  .where('callid', request.query.callid)
  .select('identclient as Cliente')
  .then(function(rows) {
    const ret = {};
    ret.error = false;
    ret.total = rows.length;
    if(rows.length == 0)
    {
      ret.data = null;
      ret.message = `Não foi possível localizar a ligação ${request.query.callid}.`;
      ret.error = true;
    }
    else
    {
      ret.data = rows[0];
      ret.message = `Cliente da ligação ${request.query.callid}, localizado com sucesso.`;
    }
    reply(ret);
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};

exports.setCliente = (request, reply) => {
  const upd = {
    identclient: request.payload.identclient,
  };
  const knex = db.knex('public.queue_calls');
  knex.update(upd);
  knex.where('callid', request.payload.callid)
  .then(function(rows) {
    const ret = {};
    ret.error = false;
    ret.total = rows;
    ret.message = `Cliente ${request.payload.identclient}, atualizado com sucesso na ligação ${request.payload.callid}.`;
    reply(ret);
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};