'use strict'

const Joi = require('joi');
const handlers = require('./handlers');
const CRUD = require('../../../util/CRUD');
const util = require('../../../util/restbase');

//Conjunto de dados padrão + específico
const ListaReturnSchema = util.CreateReturnSchema(null);
const operationData ={
  name: Joi.string().required(),
  musiconhold: Joi.string().optional().allow(null).default(null),
  announce: Joi.string().optional().allow(null).default(null),
  context: Joi.string().optional().allow(null).default(null),
  timeout: Joi.number().integer().default(30),
  monitor_join: Joi.bool().optional().allow(null).default(null),
  monitor_format: Joi.string().optional().allow(null).default(null),
  queue_youarenext: Joi.string().optional().allow(null).default(null),
  queue_thereare: Joi.string().optional().allow(null).default(null),
  queue_callswaiting: Joi.string().optional().allow(null).default(null),
  queue_holdtime: Joi.string().optional().allow(null).default(null),
  queue_minutes: Joi.string().optional().allow(null).default(null),
  queue_seconds: Joi.string().optional().allow(null).default(null),
  queue_lessthan: Joi.string().optional().allow(null).default(null),
  queue_thankyou: Joi.string().optional().allow(null).default(null),
  queue_reporthold: Joi.string().optional().allow(null).default(null),
  announce_frequency: Joi.number().integer().allow(null).default(null),
  announce_round_seconds: Joi.number().integer().allow(null).default(null),
  announce_holdtime: Joi.string().optional().allow(null).default(null),
  retry: Joi.number().integer().default(0).optional().allow(null),
  wrapuptime: Joi.number().integer().allow(null).default(null),
  maxlen: Joi.number().integer().default(0),
  servicelevel: Joi.number().integer().allow(null).default(null),
  strategy: Joi.string().optional().allow(null).default(null),
  joinempty: Joi.string().optional().allow(null).default(null),
  leavewhenempty: Joi.string().optional().allow(null).default(null),
  eventmemberstatus: Joi.bool().optional().allow(null).default(null),
  eventwhencalled: Joi.bool().optional().allow(null).default(null),
  reportholdtime: Joi.bool().optional().allow(null).default(null),
  memberdelay: Joi.number().integer().allow(null).default(null),
  weight: Joi.number().integer().allow(null).default(null),
  timeoutrestart: Joi.bool().optional().allow(null).default(null),
  setinterfacevar: Joi.bool().optional().allow(null).default(true),
  ringinuse: Joi.number().integer().default(0),
  nivel_percentual: Joi.number().integer().default(80),
  nivel_tempo: Joi.number().integer().default(20),
  numero: Joi.number().integer().allow(null).default(null),
  nivel_abandono_tempo: Joi.number().integer().default(20),
  nivel_abandono_percentual: Joi.number().integer().default(80),
  id_calculo_nivel: Joi.number().integer().allow(null).default(null),
  autofill: Joi.string().optional().allow(null).default('yes'),
  autopause: Joi.string().optional().allow(null).default('yes'),
  id_calculo_performance: Joi.number().integer().allow(null).default(null),
  id_lista_callback: Joi.number().integer().allow(null).default(null),
  callback_tolerancia: Joi.number().integer().allow(null).default(null),
  setqueuevar: Joi.bool().default(true),
}

module.exports = [
  {
    method: 'GET',
    path: '/api/v3/fila',
    handler: handlers.getall,
    config: {
      description: 'Pesquisa as filas. Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
      tags: ['api','Fila'],
      auth: {
          strategy: 'bearer',
          scope: ['app.filas','admin']
      },
      response: {
        schema: ListaReturnSchema
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
                campo: Joi.string().description('Nome da coluna'),
                ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          context: Joi.string().optional().description('Filtra por nome'),
          name: Joi.string().optional().description('Filtra por nome'),
          numero: Joi.number().integer().optional().description('Filtra por número da fila')
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/fila/ligacao',
    handler: handlers.getLigacoes,
    config: {
      description: 'Pesquisa as ligações das filas. Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
      tags: ['api','Fila'],
      auth: {
          strategy: 'bearer',
          scope: [ 'app.cdrfilas', 'admin' ]
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
                campo: Joi.string().description('Nome da coluna'),
                ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          data_ini: Joi.date().description('Filtro por data inicial'),
          data_fim: Joi.date().description('Filtro por data Final'),
          numero: Joi.string().description('Filtro por numero (pode ser origem ou destino dependendo do sentido)'),
          filas: Joi.array().items(Joi.string()).example(['FilaA','FilaB']).description('Filtra por uma ou mais filas'),
          agentes: Joi.array().items(Joi.string()).example(['AgenteA','AgenteB']).description('Filtra por um ou mais agentes'),
          status: Joi.array().items(Joi.string().valid('Atendida', 'Abandonada', 'Transbordou', 'Não atendida', 'Ocupado', 'Falha')).allow("",null).description('Filtra por um ou mais status'),          
          protocolo: Joi.string().description('Filtro por protocolo'),
          classificacao: Joi.array().items(Joi.string()).example(['ClassificacaoA','ClassificacaoB']).description('Filtra por uma ou mais classificações.'),
          subclassificacao: Joi.array().items(Joi.string()).example(['SubclassificacaoA','SubclassificacaoB']).description('Filtra por uma ou mais subclassificações.'),
          sentido: Joi.number().integer().valid(1, 2).description('Filtro por sentido, 1 = Entrante, 2 = Sainte'),
          finalizadapor: Joi.number().integer().valid(1, 2).description('Filtro por quem finalizou a ligação (Valida apenas para entrantes), 1 = Agente, 2 = Cliente'),
          identclient: Joi.string().description('Filtro pela Identificação do Cliente (recurso disponível mediante solcitiação).'),
          identclientc: Joi.string().allow("").description('Condição do filtro de identificação do cliente'),
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/fila/ligacao/cliente',
    handler: handlers.getCliente,
    config: {
      description: 'Obtém o cliente de uma determinada ligação de fila.',
      tags: ['api','Fila'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          callid: Joi.string().required()
        }
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v3/fila/ligacao/cliente',
    handler: handlers.setCliente,
    config: {
      description: 'Altera o cliente de uma determinada ligação de fila.',
      tags: ['api','Fila'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        payload: {
          callid: Joi.string().required(),
          identclient: Joi.string().required()
        }
      }
    }
  }, 
  {
    method: 'GET',
    path: '/api/v3/fila/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.queue_table');
      crud.id = 'id';
      crud.getOne(request, reply);
    },
    config: {
      description: 'Obtém os dados da fila',
      tags: ['api','Fila'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v3/fila',
    handler: (request, reply) => {
        const crud = new CRUD('public.queue_table');
        crud.id = 'id';
        crud.create(request, reply);
    },
    config: {
      description: 'Cria uma nova fila',
      tags: ['api','Fila'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        payload: operationData
      }
    }
},
{
    method: 'PUT',
    path: '/api/v3/fila/{ID}',
    handler: (request, reply) => {
        const crud = new CRUD('public.queue_table');
        crud.id = "id";
        crud.update(request, reply);
      },
    config: {
      description: 'Altera os dados da fila',
      tags: ['api','Fila'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        },
        payload: operationData
      }
    }
},
{
    method: 'DELETE',
    path: '/api/v3/fila/{ID}',
    handler: (request, reply) => {
        const crud = new CRUD('public.queue_table');
        crud.id = "id";
        crud.delete(request, reply);
      },
    config: {
      description: 'Remove a fila indicada',
      tags: ['api','Fila'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        }
      }
    }
  }
];
