'use strict'
const Joi = require('joi');
const Boom = require('boom');
const si = require('systeminformation');

module.exports = [
  {
    method: 'GET',
    path: '/api/v3/hardware/system/info',
    handler: (request, reply) => {
      si.system()
        .then(data => {
          reply(data);
        })
        .catch(error => {
          reply(Boom.badData(error));
        });
    },
    config: {
      description: 'Obtém as informações do sistema',
      tags: ['api','Hardware'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/hardware/system/bios',
    handler: (request, reply) => {
      si.bios()
        .then(data => {
          reply(data);
        })
        .catch(error => {
          reply(Boom.badData(error));
        });
    },
    config: {
      description: 'Obtém as informações da BIOS',
      tags: ['api','Hardware'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/hardware/cpu/info',
    handler: (request, reply) => {
      si.cpu()
        .then(data => {
          reply(data);
        })
        .catch(error => {
          reply(Boom.badData(error));
        });
    },
    config: {
      description: 'Obtém as informações da CPU do servidor',
      tags: ['api','Hardware'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/hardware/cpu/temps',
    handler: (request, reply) => {
      si.cpuTemperature()
        .then(data => {
          reply(data);
        })
        .catch(error => {
          reply(Boom.badData(error));
        });
    },
    config: {
      description: 'Obtém as informações de Temperatura da CPU',
      tags: ['api','Hardware'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/hardware/men/info',
    handler: (request, reply) => {
      si.mem()
        .then(data => {
          reply(data);
        })
        .catch(error => {
          reply(Boom.badData(error));
        });
    },
    config: {
      description: 'Obtém as informações de memória',
      tags: ['api','Hardware'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/hardware/disk/info',
    handler: (request, reply) => {
      si.diskLayout()
        .then(data => {
          reply(data);
        })
        .catch(error => {
          reply(Boom.badData(error));
        });
    },
    config: {
      description: 'Obtém as informações de disco',
      tags: ['api','Hardware'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/hardware/disk/parts',
    handler: (request, reply) => {
      si.blockDevices()
        .then(data => {
          reply(data);
        })
        .catch(error => {
          reply(Boom.badData(error));
        });
    },
    config: {
      description: 'Obtém as informações de partições dos discos',
      tags: ['api','Hardware'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/hardware/disk/mount',
    handler: (request, reply) => {
      si.fsSize()
        .then(data => {
          reply(data);
        })
        .catch(error => {
          reply(Boom.badData(error));
        });
    },
    config: {
      description: 'Obtém as informações de partições em uso pelo sistema',
      tags: ['api','Hardware'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/hardware/net/info',
    handler: (request, reply) => {
      si.networkInterfaces()
        .then(data => {
          reply(data);
        })
        .catch(error => {
          reply(Boom.badData(error));
        });
    },
    config: {
      description: 'Obtém as informações das placas de rede',
      tags: ['api','Hardware'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/hardware/net/status',
    handler: (request, reply) => {
      const iface = request.query.iface ? request.query.iface : undefined;
      si.networkStats(iface)
        .then(data => {
          reply(data);
        })
        .catch(error => {
          reply(Boom.badData(error));
        });
    },
    config: {
      description: 'Obtém os status de uma conexões de rede.',
      tags: ['api','Hardware'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          iface: Joi.string().description('Filtra pelo nome da interface, se não for informado o status da primeira conexão é retornado.')             
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/hardware/net/connections',
    handler: (request, reply) => {
      si.networkConnections()
        .then(data => {
          reply(data);
        })
        .catch(error => {
          reply(Boom.badData(error));
        });
    },
    config: {
      description: 'Obtém as conexões de rede ativas',
      tags: ['api','Hardware'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/hardware/load',
    handler: (request, reply) => {
      si.currentLoad()
        .then(data => {
          reply(data);
        })
        .catch(error => {
          reply(Boom.badData(error));
        });
    },
    config: {
      description: 'Obtém as informções de carga do sistema.',
      tags: ['api','Hardware'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/os/info',
    handler: (request, reply) => {
      si.osInfo()
        .then(data => {
          reply(data);
        })
        .catch(error => {
          reply(Boom.badData(error));
        });
    },
    config: {
      description: 'Obtém as informções sobre o sistema operacional.',
      tags: ['api','Sistema Operacional'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/os/processos',
    handler: (request, reply) => {
      si.processes()
        .then(data => {
          reply(data);
        })
        .catch(error => {
          reply(Boom.badData(error));
        });
    },
    config: {
      description: 'Obtém as informções sobre os processos em execução',
      tags: ['api','Sistema Operacional'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/os/processo/info',
    handler: (request, reply) => {
      si.processLoad(request.query.nome_processo)
        .then(data => {
          reply(data);
        })
        .catch(error => {
          reply(Boom.badData(error));
        });
    },
    config: {
      description: 'Obtém as informções detalhadas sobre um processo',
      tags: ['api','Sistema Operacional'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          nome_processo: Joi.string().required().description('Informe o nome do processo')             
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/os/servicos',
    handler: (request, reply) => {
      si.services(request.query.servicos.join(','))
        .then(data => {
          reply(data);
        })
        .catch(error => {
          reply(Boom.badData(error));
        });
    },
    config: {
      description: 'Obtém as informções sobre os serviços informados',
      tags: ['api','Sistema Operacional'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          servicos: Joi.array().required().items(Joi.string()).description('Informe um ou vários nomes de serviços'),           
        }
      }
    }
  },
];
