'use strict'

const joi = require('joi');
const Boom = require('boom');
const db = require('../../../util/db');
const config = require('../../../config.json');

exports.getall = (request, reply) => {
  const max = request.query.max ? request.query.max : 15;
  const page = request.query.page ? request.query.page : 1;
  const knex = db.knex('public.horarios');

  if(request.query.dias_semana) knex.whereRaw('dias_semana::text[] && ARRAY'+JSON.stringify(request.query.dias_semana).replace(/"/g, "'"));
  if(request.query.messes) knex.whereRaw('messes::text[] && ARRAY'+JSON.stringify(request.query.messes).replace(/"/g, "'"));
  if(request.query.dias) knex.whereRaw('dias && ARRAY['+request.query.dias+']');
  if(request.query.nome) knex.whereRaw('LOWER(nome) LIKE ?', '%'+decodeURIComponent(request.query.nome).toLowerCase()+'%');
  if(request.query.tipo) knex.where('tipo', request.query.tipo);

  if(max === -1){
    //console.log('SQL = '+ knex.toString());
    knex.clearSelect();
    // Parametros de ordenação, fora do count
    if(request.query.order){
      for(let p=0;p<request.query.order.length;++p){
        if(request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
            knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
        }
        else if(request.query.order[p] && request.query.order[p].campo) {
          knex.orderBy(request.query.order[p].campo, 'ASC');
        }
      }
    }
    knex.select('*')
    .then(function(rows) {
      const ret = {};
      ret.error = false;
      ret.full = true;
      ret.total = rows.length;
      ret.message = 'Lista de todos os horarios';
      ret.dados = rows;
      reply(ret);
    })
    .catch(function(error) {
      console.error(error);
      reply(Boom.badData(error));
    });
  }
  else {
    const offset = max * (page - 1);
    //console.log('SQL = '+knex.count('* as total').toString());
    knex.count('* as total')
    .then(function(total) {
      if(total[0].total > 0){
        knex.clearSelect();
        // Parametros de ordenação, fora do count
        if(request.query.order){
          for(let p=0;p<request.query.order.length;++p){
            if(request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
            }
            else if(request.query.order[p] && request.query.order[p].campo) {
              knex.orderBy(request.query.order[p].campo, 'ASC');
            }
          }
        }
        //console.log('SQL = '+knex.select('*').limit(max).offset(offset).toString());
        knex.select('*').limit(max).offset(offset).then(function(rows) {
          const pages = Math.ceil(parseInt(total[0].total) / max);
          const ret = {};
          ret.error = false;
          ret.full = pages === 1 ? true : false;
          ret.page = page;
          ret.max = max;
          ret.offset = offset;
          ret.total = parseInt(total[0].total);
          ret.pages = pages;
          ret.message = 'Lista de horarios paginados';
          ret.dados = rows;
          reply(ret);
        })
        .catch(function(error) {
          console.error(error);
          reply(Boom.badData(error));
        });
      }
      else {
        const ret = {};
        ret.error = true;
        ret.total = 0;
        ret.message = 'Nenhum registro encontrado';
        reply(ret);
      }
    })
    .catch(function(error) {
      console.error(error);
      reply(Boom.badData(error));
    });
  }
};

exports.create = (request, reply) => {
  const knex = db.knex('public.horarios');
  if(request.payload.hora_ini === '') request.payload.hora_ini = null;
  if(request.payload.hora_fim === '') request.payload.hora_fim = null;
  knex.insert(request.payload);
  knex.returning('id')
  .then(function(id) {
    const ret = {};
    ret.error = false;
    ret.id = id[0];
    ret.message = 'Horario id = '+id[0]+' criado com sucesso.';
    reply(ret);
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};

exports.update = (request, reply) => {
  const knex = db.knex('public.horarios');
  if(request.payload.hora_ini === '') request.payload.hora_ini = null;
  if(request.payload.hora_fim === '') request.payload.hora_fim = null;
  knex.update(request.payload);

  knex.where('id', request.params.HorarioID)
  .then(function(rows) {
    const ret = {};
    ret.error = false;
    ret.total = rows;
    ret.message = 'Horario id = '+request.params.HorarioID+' atualizado com sucesso.';
    reply(ret);
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};

exports.delete = (request, reply) => {
  const knex = db.knex('public.horarios');
  knex.where('id', request.params.HorarioID)
  .del()
  .then(function(rows) {
    if(rows > 0){
      const ret = {};
      ret.error = false;
      ret.total = rows;
      ret.message = 'Horario id = '+request.params.HorarioID+' apagado com sucesso.';
      reply(ret);
    }
    else {
      reply(Boom.badData("Nenhum registro encontrado para ID = "+request.params.HorarioID));
    }
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};
