'use strict'

const joi = require('joi');
const Boom = require('boom');
const db = require('../../../util/db');
//const squel = require("squel");
const ws = require('../../../ws');
const discador = require('../../../discador');
const config = require('../../../config.json');
const moment = require('moment-timezone');
moment.tz.setDefault(config.servidor.timezone);

exports.getall = (request, reply) => {
  const max = request.query.max ? request.query.max : 15;
  const page = request.query.page ? request.query.page : 1;
  const knex = db.knex('public.layouts');

  if(request.query.nome) knex.whereRaw('LOWER(nome) LIKE ?', '%'+decodeURIComponent(request.query.nome).toLowerCase()+'%');

  if(max === -1){
    //console.log('SQL = '+ knex.toString());
    knex.clearSelect();
    // Parametros de ordenação, fora do count
    if(request.query.order){
      for(let p=0;p<request.query.order.length;++p){
        if(request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
            knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
        }
        else if(request.query.order[p] && request.query.order[p].campo) {
          knex.orderBy(request.query.order[p].campo, 'ASC');
        }
      }
    }
    knex.select('*')
    .then(function(rows) {
      const ret = {};
      ret.error = false;
      ret.full = true;
      ret.total = rows.length;
      ret.message = 'Lista de todos os layouts';
      ret.dados = rows;
      reply(ret);
    })
    .catch(function(error) {
      console.error(error);
      reply(Boom.badData(error));
    });
  }
  else {
    const offset = max * (page - 1);
    //console.log('SQL = '+knex.count('* as total').toString());
    knex.count('* as total')
    .then(function(total) {
      if(total[0].total > 0){
        knex.clearSelect();
        // Parametros de ordenação, fora do count
        if(request.query.order){
          for(let p=0;p<request.query.order.length;++p){
            if(request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
            }
            else if(request.query.order[p] && request.query.order[p].campo) {
              knex.orderBy(request.query.order[p].campo, 'ASC');
            }
          }
        }
        //console.log('SQL = '+knex.select('*').limit(max).offset(offset).toString());
        knex.select('*').limit(max).offset(offset).then(function(rows) {
          const pages = Math.ceil(parseInt(total[0].total) / max);
          const ret = {};
          ret.error = false;
          ret.full = pages === 1 ? true : false;
          ret.page = page;
          ret.max = max;
          ret.offset = offset;
          ret.total = parseInt(total[0].total);
          ret.pages = pages;
          ret.message = 'Lista de layouts paginados';
          ret.dados = rows;
          reply(ret);
        })
        .catch(function(error) {
          console.error(error);
          reply(Boom.badData(error));
        });
      }
      else {
        const ret = {};
        ret.error = true;
        ret.total = 0;
        ret.message = 'Nenhum registro encontrado';
        reply(ret);
      }
    })
    .catch(function(error) {
      console.error(error);
      reply(Boom.badData(error));
    });
  }
};

exports.getLayout = (request, reply) => {
  const knex = db.knex('public.layouts');
  knex.where('id', request.params.LayoutID);
  knex.select('*')
  .then(function(rows) {
    const ret = {};
    ret.error = false;
    ret.full = true;
    ret.total = rows.length;
    ret.message = 'Dados do layout id = '+request.params.LayoutID;
    ret.layout = rows[0];
    reply(ret);
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};

exports.create = (request, reply) => {
  const knex = db.knex('public.layouts');
  knex.insert(request.payload);
  knex.returning('id')
  .then(function(id) {
    const ret = {};
    ret.error = false;
    ret.id = id[0];
    ret.message = 'layouts id = '+id[0]+' criado com sucesso.';
    reply(ret);
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};

exports.update = (request, reply) => {
  const knex = db.knex('public.layouts');
  knex.update(request.payload);

  knex.where('id', request.params.LayoutID)
  .then(function(rows) {
    const ret = {};
    ret.error = false;
    ret.total = rows;
    ret.message = 'Layout id = '+request.params.LayoutID+' atualizado com sucesso.';
    reply(ret);
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};

exports.delete = (request, reply) => {
  const knex = db.knex('public.layouts');
  knex.where('id', request.params.LayoutID)
  .del()
  .then(function(rows) {
    if(rows > 0){
      const ret = {};
      ret.error = false;
      ret.total = rows;
      ret.message = 'Layout id = '+request.params.LayoutID+' apagado com sucesso.';
      reply(ret);
    }
    else {
      reply(Boom.badData("Nenhum registro encontrado para ID = "+request.params.LayoutID));
    }
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};

exports.getLayoutCampos = (request, reply) => {
  const max = request.query.max ? request.query.max : 15;
  const page = request.query.page ? request.query.page : 1;
  const knex = db.knex('public.layouts_campos');
  knex.where('id_layout', request.params.LayoutID);
  if(request.query.nome) knex.whereRaw('LOWER(nome) LIKE ?', '%'+decodeURIComponent(request.query.nome).toLowerCase()+'%');

  if(max === -1){
    //console.log('SQL = '+ knex.toString());
    knex.clearSelect();
    // Parametros de ordenação, fora do count
    if(request.query.order){
      for(let p=0;p<request.query.order.length;++p){
        if(request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
            knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
        }
        else if(request.query.order[p] && request.query.order[p].campo) {
          knex.orderBy(request.query.order[p].campo, 'ASC');
        }
      }
    }
    knex.select('*')
    .then(function(rows) {
      const ret = {};
      ret.error = false;
      ret.full = true;
      ret.total = rows.length;
      ret.message = 'Lista de todos os campos do layout';
      ret.dados = rows;
      reply(ret);
    })
    .catch(function(error) {
      console.error(error);
      reply(Boom.badData(error));
    });
  }
  else {
    const offset = max * (page - 1);
    //console.log('SQL = '+knex.count('* as total').toString());
    knex.count('* as total')
    .then(function(total) {
      if(total[0].total > 0){
        knex.clearSelect();
        // Parametros de ordenação, fora do count
        if(request.query.order){
          for(let p=0;p<request.query.order.length;++p){
            if(request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
            }
            else if(request.query.order[p] && request.query.order[p].campo) {
              knex.orderBy(request.query.order[p].campo, 'ASC');
            }
          }
        }
        //console.log('SQL = '+knex.select('*').limit(max).offset(offset).toString());
        knex.select('*').limit(max).offset(offset).then(function(rows) {
          const pages = Math.ceil(parseInt(total[0].total) / max);
          const ret = {};
          ret.error = false;
          ret.full = pages === 1 ? true : false;
          ret.page = page;
          ret.max = max;
          ret.offset = offset;
          ret.total = parseInt(total[0].total);
          ret.pages = pages;
          ret.message = 'Lista dos campos do layout paginados';
          ret.dados = rows;
          reply(ret);
        })
        .catch(function(error) {
          console.error(error);
          reply(Boom.badData(error));
        });
      }
      else {
        const ret = {};
        ret.error = true;
        ret.total = 0;
        ret.message = 'Nenhum registro encontrado';
        reply(ret);
      }
    })
    .catch(function(error) {
      console.error(error);
      reply(Boom.badData(error));
    });
  }
};

exports.ReplaceCampos = (request, reply) => {
  const knex = db.knex;
  const classificacoes = [];

  for(let i=0;i<request.payload.length;++i){
    const cla = {
      id_layout: request.params.LayoutID,
      nome: request.payload[i].nome,
      posicao_csv: request.payload[i].posicao_csv,
      xcontact: request.payload[i].xcontact,
    };
    classificacoes.push(cla);
  }

  knex.transaction(function(t) {
    return knex('layouts_campos')
     .transacting(t)
     .where('id_layout', request.params.LayoutID)
     .del()
     .then(function() {
       return knex.batchInsert('layouts_campos', classificacoes, 100)
        .transacting(t)
        .then()
        .catch(function(error) {
          console.log('Falha ao inserir os campos do layout. Erro:');
          console.log(error);
          t.rollback();
          throw error;
        });
     })
     .then(t.commit)
     .catch(function(e) {
          t.rollback();
          console.log('No rollback. Erro:');
          console.log(e);
          throw e;
     })
  })
  .then(function() {
    console.log('Campos do layout atualizados com sucesso!');
    const ret = {};
    ret.error = false;
    ret.message = 'Campos do layout atualizados com sucesso!';
    reply(ret);
  })
  .catch(function(e) {
    console.log('Falha ao atualizar os campos do layout. Erro:');
    console.log(e);
    reply(Boom.badData(e));
  });
};
