'use strict'

const joi = require('joi');
const Boom = require('boom');
const db = require('../../../util/db');
const restbase = require('../../../util/restbase');
const ws = require('../../../ws');
const Ami = require('../../../ami');
const discador = require('../../../discador');
const config = require('../../../config.json');
const moment = require('moment-timezone');
moment.tz.setDefault(config.servidor.timezone);

function rollback(client) {
  client.query('ROLLBACK', function () {
    console.log('ROLLBACK acionado.');
  });
};

function getPar(item, index) {
  return '$' + (parseInt(index) + 1);
};

function getReq(item, index) {
  return 'request.payload.' + item;
}

exports.classifica = (request, reply) => {
  // Ajuste de UNIQUE ID repetido, ocorre em transbordo
  if (request.payload.uniqueid.toString().indexOf(',')) {
    request.payload.uniqueid = request.payload.uniqueid.toString().split(',')[0];
  }

  // Açoes com o discador
  if (request.payload.id_classificacao) {
    try {
      discador.ProcessaClassificacao(request.payload.uniqueid, request.payload.id_classificacao);
    } catch (ex) {
      console.log("Falha ao chamar discador.ProcessaClassificacao(%s), ERRO: ", request.payload.id_classificacao, ex.toString());
    }
  }
  // Açoes com banco
  db.knex.transaction(function (t) {
    return db.knex('cdr_classificacoes')
      .transacting(t)
      .where('uniqueid', request.payload.uniqueid)
      .del()
      .then(function () {
        return db.knex('cdr_classificacoes').insert(request.payload)
          .transacting(t)
          .then()
          .catch(function (error) {
            console.log('Falha ao inserir em cdr_classificacoes. Erro:');
            console.log(error);
            t.rollback();
            throw error;
          });
      })
      .then(t.commit)
      .catch(function (e) {
        t.rollback();
        console.log('No rollback. Erro:');
        console.log(e);
        throw e;
      })
  })
    .then(function () {
      console.log('Classificação salva com sucesso!');
      const ret = {};
      ret.error = false;
      ret.message = 'Classificação salva com sucesso!';
      reply(ret);
    })
    .catch(function (e) {
      console.log('Falha ao salvar a classificação. Erro:');
      console.log(e);
      reply(Boom.badData(e));
    });
};

exports.tag = (request, reply) => {
  // Ajuste de UNIQUE ID repetido, ocorre em transbordo
  if (request.payload.uniqueid.toString().indexOf(',')) {
    request.payload.uniqueid = request.payload.uniqueid.toString().split(',')[0];
  }

  //Prepara os registros para inserção
  var tags = request.payload.tags.map((x) => {
    return {
      uniqueid: request.payload.uniqueid,
      agente: request.payload.agente,
      id_tag: x
    };
  });

  // Açoes com banco
  db.knex.transaction(function (t) {
    return db.knex.batchInsert('cdr_tags', tags, 500)
      .transacting(t)
      .then(function () {
        const ret = {};
        ret.error = false;
        ret.message = 'Ligação etiquetada com sucesso!';
        reply(ret);
      })
      .catch(function (e) {
        console.log('Erro ao etiquetar ligação. Erro:');
        console.log(e);
      });
  });
};

exports.unTag = (request, reply) => {
  // Ajuste de UNIQUE ID repetido, ocorre em transbordo
  if (request.payload.uniqueid.toString().indexOf(',')) {
    request.payload.uniqueid = request.payload.uniqueid.toString().split(',')[0];
  }
  // Açoes com banco
  db.knex('cdr_tags')
    .where('uniqueid', request.payload.uniqueid)
    .where('id_tag', request.payload.id_tag)
    .del()
    .then(function () {
      const ret = {};
      ret.error = false;
      ret.message = 'Etiqueta removida com sucesso!';
      reply(ret);
    })
    .catch(function (e) {
      console.log('Erro ao remover etiqueta da ligação. Erro:');
      console.log(e);
    });
};

exports.create = (request, reply) => {
  const { username, password } = request.payload;
  if (username === 'juca') {
    reply(Boom.badData('Juca não entra aqui'));
  }
  else {
    reply({ success: true });
  }
};
exports.getStatus = (request, reply) => {
  const max = request.query.max ? request.query.max : 15;
  const page = request.query.page ? request.query.page : 1;
  const knex = db.knex('public.status_ligacao');

  if (request.query.nome) knex.whereRaw('LOWER(nome) LIKE ?', '%' + decodeURIComponent(request.query.nome).toLowerCase() + '%');

  if (max === -1) {
    //console.log('SQL = '+ knex.toString());
    knex.clearSelect();
    // Parametros de ordenação, fora do count
    if (request.query.order) {
      for (let p = 0; p < request.query.order.length; ++p) {
        if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
          knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
        }
        else if (request.query.order[p] && request.query.order[p].campo) {
          knex.orderBy(request.query.order[p].campo, 'ASC');
        }
      }
    }
    knex.select('*')
      .then(function (rows) {
        const ret = {};
        ret.error = false;
        ret.full = true;
        ret.total = rows.length;
        ret.message = 'Lista de todos os status de ligação';
        ret.dados = rows;
        if (request.query.return === 'array') reply(rows);
        else reply(ret);
      })
      .catch(function (error) {
        console.error(error);
        reply(Boom.badData(error));
      });
  }
  else {
    const offset = max * (page - 1);
    //console.log('SQL = '+knex.count('* as total').toString());
    knex.count('* as total')
      .then(function (total) {
        if (total[0].total > 0) {
          knex.clearSelect();
          // Parametros de ordenação, fora do count
          if (request.query.order) {
            for (let p = 0; p < request.query.order.length; ++p) {
              if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
              }
              else if (request.query.order[p] && request.query.order[p].campo) {
                knex.orderBy(request.query.order[p].campo, 'ASC');
              }
            }
          }
          //console.log('SQL = '+knex.select('*').limit(max).offset(offset).toString());
          knex.select('*').limit(max).offset(offset).then(function (rows) {
            const pages = Math.ceil(parseInt(total[0].total) / max);
            const ret = {};
            ret.error = false;
            ret.full = pages === 1 ? true : false;
            ret.page = page;
            ret.max = max;
            ret.offset = offset;
            ret.total = parseInt(total[0].total);
            ret.pages = pages;
            ret.message = 'Lista de status de ligação paginada';
            ret.dados = rows;
            if (request.query.return === 'array') reply(rows);
            else reply(ret);
          })
            .catch(function (error) {
              console.error(error);
              reply(Boom.badData(error));
            });
        }
        else {
          const ret = {};
          ret.error = true;
          ret.total = 0;
          ret.message = 'Nenhum registro encontrado';
          reply(ret);
        }
      })
      .catch(function (error) {
        console.error(error);
        reply(Boom.badData(error));
      });
  }
};

exports.getall = (request, reply) => {
  const max = request.query.max ? request.query.max : 15;
  const page = request.query.page ? request.query.page : 1;
  const knex = db.knex('public.ligacoes');

  if (request.query.data_ini) knex.whereRaw('calldate::TIMESTAMP without TIME ZONE >= ?', request.query.data_ini);
  if (request.query.data_fim) knex.whereRaw('calldate::TIMESTAMP without TIME ZONE <= ?', request.query.data_fim);
  if (request.query.origem) knex.whereRaw(restbase.buildSearchExpression('src', request.query.origemc, request.query.origem));
  if (request.query.destino) knex.whereRaw(restbase.buildSearchExpression('dst', request.query.destinoc, request.query.destino));
  if (request.query.callid) knex.where('callid', request.query.callid);
  if (request.query.status) knex.where('status', request.query.status);
  if (request.query.sentido) knex.where('sentido', request.query.sentido);
  if (request.query.canal) knex.where('canal', 'like', '%' + request.query.canal + '%');
  if (request.query.pin_id) knex.where('pin_id', request.query.pin_id);
  if (request.query.pin_usuario) knex.where('pin_usuario', request.query.pin_usuario);
  if (request.query.idcliente) knex.whereRaw(restbase.buildSearchExpression('idcliente', request.query.idclientec, request.query.idcliente));

  if (max === -1) {
    //console.log('SQL = '+ knex.toString());
    knex.clearSelect();
    // Parametros de ordenação, fora do count
    if (request.query.order) {
      for (let p = 0; p < request.query.order.length; ++p) {
        if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
          knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
        }
        else if (request.query.order[p] && request.query.order[p].campo) {
          knex.orderBy(request.query.order[p].campo, 'ASC');
        }
      }
    }
    knex.select('*')
      .then(function (rows) {
        const ret = {};
        ret.error = false;
        ret.full = true;
        ret.total = rows.length;
        ret.message = 'Lista de todas as ligações';
        ret.dados = rows;
        reply(ret);
      })
      .catch(function (error) {
        console.error(error);
        reply(Boom.badData(error));
      });
  }
  else {
    const offset = max * (page - 1);
    //console.log('SQL = '+knex.count('* as total').toString());
    knex.count('* as total')
      .then(function (total) {
        if (total[0].total > 0) {
          knex.clearSelect();
          // Parametros de ordenação, fora do count
          if (request.query.order) {
            for (let p = 0; p < request.query.order.length; ++p) {
              if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
              }
              else if (request.query.order[p] && request.query.order[p].campo) {
                knex.orderBy(request.query.order[p].campo, 'ASC');
              }
            }
          }
          //console.log('SQL = '+knex.select('*').limit(max).offset(offset).toString());
          knex.select('*').limit(max).offset(offset).then(function (rows) {
            const pages = Math.ceil(parseInt(total[0].total) / max);
            const ret = {};
            ret.error = false;
            ret.full = pages === 1 ? true : false;
            ret.page = page;
            ret.max = max;
            ret.offset = offset;
            ret.total = parseInt(total[0].total);
            ret.pages = pages;
            ret.message = 'Lista de ligações paginada';
            ret.dados = rows;
            reply(ret);
          })
            .catch(function (error) {
              console.error(error);
              reply(Boom.badData(error));
            });
        }
        else {
          const ret = {};
          ret.error = true;
          ret.total = 0;
          ret.message = 'Nenhum registro encontrado';
          reply(ret);
        }
      })
      .catch(function (error) {
        console.error(error);
        reply(Boom.badData(error));
      });
  }
};

exports.getOne = (request, reply) => {
  const knex = db.knex('public.ligacoes');
  knex.where('callid', request.params.CallID);
  knex.select('*')
    .then(function (rows) {
      if (rows && rows.length > 0) {
        const ret = {};
        ret.error = false;
        ret.full = true;
        ret.total = rows.length;
        ret.message = 'Dados da ligação Callid = ' + request.params.CallID;
        ret.dados = rows[0];
        reply(ret);
      }
      else {
        const ret = {};
        ret.error = true;
        ret.total = 0;
        ret.message = 'A ligação Callid = ' + request.params.CallID + ' não foi encontrada na base, talvez ainda esteja em curso ou não existe.';
        reply(ret);
      }
    })
    .catch(function (error) {
      console.error(error);
      reply(Boom.badData(error));
    });
};

exports.discar = (request, reply) => {
  Ami.Discar(request.payload, (resposta) => {
    const ret = {};
    ret.error = resposta.response === 'Error' ? true : false;
    ret.callid = resposta.response === 'Error' ? undefined : resposta.callid;
    ret.message = resposta.message;
    reply(ret);
  });
};

exports.desligar = (request, reply) => {
  Ami.Desligar(request.payload, (resposta) => {
    const ret = {};
    ret.error = resposta.response === 'Error' ? true : false;
    ret.message = resposta.message;
    reply(ret);
  });
};

exports.schema = {
  ligacao: joi.object().keys({
    id: joi
      .number()
      .required()
      .integer()
      .min(0)
      .description('id do agente')
      .example(1),
    nome: joi
      .string()
      .required()
      .description('nome para login')
      .example('teste'),
    senha: joi
      .string()
      .required()
      .description('senha para login')
      .example('teste'),
    ramal: joi
      .any()
      .description('ramal do agente')
      .example('SIP/1000'),
    token: joi
      .any()
      .description('token de acesso')
      .example('asddsfdsgffdgfdghsdferf'),
    fila: joi
      .any()
      .description('fila (Não é mais usado)')
      .example('nda'),
    token_create: joi
      .any()
      .description('data de criacção do token')
      .example('nda'),
    datastatus: joi
      .any()
      .description('data de criacção do token')
      .example('nda'),
    status: joi
      .any()
      .description('status')
      .example('nda'),
    fullname: joi
      .string()
      .required()
      .description('nome completo do agente')
      .example('Teste da silva'),
    filas: joi
      .array()
      .items(
        joi
          .any()
          .description('nome da fila')
          .example('Atendimento')
      )
  }).label('ligacao'),
  // Using a getter here so that I can refer to `this`, in reference to the character node of the schema object
  get ligacoes() {
    return joi
      .array()
      .items(this.ligacao)
      .label('lista_de_ligacoes')
  }
};
