'use strict'

const Boom = require('boom');
const md5 = require('md5');
const db = require('../../../util/db');
const config = require('../../../config.json');
const util = require('../../../util/restbase');
const moment = require('moment-timezone');
moment.tz.setDefault(config.servidor.timezone);

function ordered_menu(array,parent_id){
  const temp_array = [];
  //console.log('Resultados='+array.length);
  for(let i=0;i<array.length;++i){
    //console.log('id='+array[i].id+' parent='+parent_id);
    if(array[i].parent==parent_id){
      const modulo = {
        id: array[i].id, 
        parent: array[i].parent, 
        nome: array[i].nome, 
        icon: array[i].icon, 
        descricao: array[i].descricao, 
        order:array[i].ordem, 
        extra:array[i].extra, 
        tem:array[i].tem,
        extra_usuario:array[i].extra_usuario, 
        modulos:[]
      };
      modulo.modulos = ordered_menu(array,array[i].id);
      temp_array.push(modulo);
    }
  }
  return temp_array;
};

exports.getall = (request, reply) => {
  const max = request.query.max ? request.query.max : 15;
  const page = request.query.page ? request.query.page : 1;
  const knex = db.knex('public.modulos as m');

  if(request.query.nome) knex.whereRaw('LOWER(nome) LIKE ?', '%'+decodeURIComponent(request.query.nome).toLowerCase()+'%');
  if(request.query.descricao) knex.whereRaw('LOWER(descricao) LIKE ?', '%'+decodeURIComponent(request.query.descricao).toLowerCase()+'%');
  if(request.query.menu) 
  {
    knex.innerJoin('public.modulos as p','p.id','m.parent')
    knex.whereRaw('p.parent IS NULL AND p.descricao = ?', request.query.menu);
  }

  if(max === -1){
    //console.log('SQL = '+ knex.toString());
    knex.clearSelect();
    // Parametros de ordenação, fora do count
    if(request.query.order){
      for(let p=0;p<request.query.order.length;++p){
        if(request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
            knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
        }
        else if(request.query.order[p] && request.query.order[p].campo) {
          knex.orderBy(request.query.order[p].campo, 'ASC');
        }
      }
    }
    knex.select('*')
    .then(function(rows) {
      const ret = {};
      ret.error = false;
      ret.full = true;
      ret.total = rows.length;
      ret.message = 'Lista de todos os módulos';
      ret.dados = rows;
      reply(ret);
    })
    .catch(function(error) {
      console.error(error);
      reply(Boom.badData(error));
    });
  }
  else {
    const offset = max * (page - 1);
    //console.log('SQL = '+knex.count('* as total').toString());
    knex.count('* as total')
    .then(function(total) {
      if(total[0].total > 0){
        knex.clearSelect();
        // Parametros de ordenação, fora do count
        if(request.query.order){
          for(let p=0;p<request.query.order.length;++p){
            if(request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
            }
            else if(request.query.order[p] && request.query.order[p].campo) {
              knex.orderBy(request.query.order[p].campo, 'ASC');
            }
          }
        }
        //console.log('SQL = '+knex.select('*').limit(max).offset(offset).toString());
        knex.select('*').limit(max).offset(offset).then(function(rows) {
          const pages = Math.ceil(parseInt(total[0].total) / max);
          const ret = {};
          ret.error = false;
          ret.full = pages === 1 ? true : false;
          ret.page = page;
          ret.max = max;
          ret.offset = offset;
          ret.total = parseInt(total[0].total);
          ret.pages = pages;
          ret.message = 'Lista de módulos, paginada';
          ret.dados = rows;
          reply(ret);
        })
        .catch(function(error) {
          console.error(error);
          reply(Boom.badData(error));
        });
      }
      else {
        const ret = {};
        ret.error = true;
        ret.total = 0;
        ret.message = 'Nenhum registro encontrado';
        reply(ret);
      }
    })
    .catch(function(error) {
      console.error(error);
      reply(Boom.badData(error));
    });
  }
};

exports.getModulosUsuario = (request, reply) => {
    db.query('BEGIN')
    .then(()=>{
      db.query(`select a.*,
      case when b.id_modulo is null then false else true end as tem,
      case when b.id_modulo is null then false else true end as tem_old,
      case when b.extra_usuario is null then false else true end as extra_usuario,
      case when b.extra_usuario is null then false else true end as extra_old 
      FROM modulos a
      left join supervisores_modulos b on a.id = b.id_modulo AND b.id_supervisor = $1 
      ORDER BY a.ordem`, [request.params.ID])
    .then(async r => {
        //Só loga quando está na UI
        if(request.query.userreq)
        {
          await db.query('INSERT INTO public.supervisores_log(id_supervisor,acao,id_user,descricao) VALUES ($1,$2,$3,$4)',
          [request.query.userreq, 'V', request.params.ID,'Usuário visualizou permissões. IP: '+ util.GetRequestIP(request)]);
        }
        //console.log('Resultados intra='+res.rows.length);
        await db.query('COMMIT')
        .then(function(){
          const ret = {};
          ret.error = false;
          ret.full = 1;
          ret.total = parseInt(r.rows.length);
          ret.pages = 1;
          ret.message = `Lista de Módulos do Usuário ${request.params.ID}`;
          ret.dados = ordered_menu(r.rows);
          reply(ret);
        });
    })
    .catch(e => {
        throw e;
    });
  })
  .catch(e=>{
    db.query('ROLLBACK');
    console.error(e);
    reply(Boom.badData(e));
  })
};

exports.getFilasUsuario = (request, reply) => {
   db.query(`select a.id, a.name,
    case when b.id_fila is null then false else true end as tem,
    case when b.id_fila is null then false else true end as tem_old 
    from queue_table a
    left join supervisores_filas b on a.name = b.id_fila AND b.id_supervisor = $1 
    order by a.name`, [request.params.ID])
   .then(r => {
        const ret = {};
        ret.error = false;
        ret.full = 1;
        ret.total = parseInt(r.rows.length);
        ret.pages = 1;
        ret.message = `Lista de Filas do Usuário ${request.params.ID}`;
        ret.dados = r.rows;
        reply(ret);
   });
};

exports.setAcessosk = (request, reply) =>{
  db.query('BEGIN')
  .then(()=>{
    db.query('DELETE FROM public.supervisores_modulos WHERE id_supervisor = $1', [request.params.ID])
    .then(()=>{
      db.query('DELETE FROM public.supervisores_filas WHERE id_supervisor = $1', [request.params.ID])
      .then(async r => {
         //Não é possível utilizar o forEach aqui pois ele é baseado em promises
         for(let i=0;i<request.payload.length;i++)
         {
             //Simula a praticidade do forEach
             let modulo = request.payload[i];
             if(modulo.modulos.count() > 0)
             {
               console.log("1");
               //Registra o Parent
               if(modulo.modulos.some(function(x){ return x.tem;}))
                 await regModulo(x);

               //Registra os modulos do módulo
               for(let j=0; j<modulo.modulos.length; j++) await regModulo(modulo.modulos[j]);  
             }  
             if(modulo.filas.count() > 0)
             {
              console.log("2");
               //Registra as filas do usuário
               for(let l=0; l<modulo.filas.length; l++) await regfila(modulo.fila[l]);  
             }
             //Nested Function para tratar os vários casos e eliminar a recursividade ou repetição
             async function regModulo(m)
             {
               console.log("3");
               //Permissão atribuída ou não alterada
               if(m.tem===true)
               {
                console.log("4");
                 await db.query('INSERT INTO public.supervisores_modulos (id_supervisor, id_modulo, extra_usuario) VALUES($1,$2,$3)', 
                 [request.params.ID, m.id, m.extra_usuario===true ? 'G' : null]);
               }
               
               //Só loga quando está na UI e é um módulo children (que tem parent != null)
               if(request.query.userreq && m.parent)
               {
                console.log("5");
                 //Loga alteração de gravação
                 if(m.extra == 'G')
                 {
                  console.log("6");
                   await db.query('INSERT INTO public.supervisores_log (id_supervisor, acao, gravacao, id_user, descricao) VALUES($1,$2,$3,$4,$5)',
                   [
                     request.query.userreq,
                     'U', 
                     m.extra_usuario===true ? 'A' : 'R', 
                     request.params.ID,
                     `${m.extra_usuario===true ? 'Adicionou' : 'Removeu'} acesso a gravação de ${m.descricao}. IP: ${util.GetRequestIP(request)}`
                    ]
                   );
                 }

                 console.log("7");
                 //Loga alteração de permissão
                 await db.query('INSERT INTO public.supervisores_log (id_supervisor, acao, modulo, id_user, descricao)',
                    [
                     request.query.userreq, 
                     m.tem===true ? 'A' : 'R',
                     m.id,
                     request.params.ID,
                     `${m.tem===true ? 'Adicionou' : 'Removeu'} acesso ao módulo ${m.descricao}. IP: ${util.GetRequestIP(request)}`
                    ]);
               }
               return;
             }

             async function regfila(fila)
             {
              console.log("8");
               //Somente se estiver marcada
               if(fila.tem===true)
               {
                console.log("9");
                 await db.query('INSERT INTO public.supervisores_filas (id_supervisor, id_fila) VALUES($1,$2)',
                 [
                    request.params.ID,
                    fila.name
                 ]);
               }
               return;
             }
         } 

        await db.query('COMMIT')
        .then(function(){
          const ret = {};
          ret.error = false;
          ret.message = `Usuário id = ${request.params.ID} teve suas permissões alteradas com sucesso.`;
          reply(ret);
        });
      })
      .catch(e => {
        throw e;
      });
    })
    .catch(e => {
      throw e;
    });
  })
  .catch(e=>{
    db.query('ROLLBACK');
    console.error(e);
    reply(Boom.badData(e));
  })
};


exports.setAcessos = (request, reply) =>{
  //Operações com banco de dados
  const knex = db.knex;
  knex.transaction(function(t) {
    //Limpa tudo
    return knex('public.supervisores_modulos')
     .transacting(t)
     .where('id_supervisor', request.params.ID)
     .del()
     .then(function(){ 
        return knex('public.supervisores_filas')
        .transacting(t)
        .where('id_supervisor', request.params.ID)
        .del()
        .then(async function() {
            //Processa os módulos
            if(request.payload.modulos && request.payload.modulos.length > 0)
            {
              //Registra os modulos do módulo
              for(let j=0; j<request.payload.modulos.length; j++){
                 //Regista os módulos
                 await regModulo(request.payload.modulos[j]);  
              }
            }  
            //Processa as filas
            if(request.payload.filas && request.payload.filas.length > 0)
            {
              //Registra as filas do usuário
              for(let l=0; l<request.payload.filas.length; l++) await regfila(request.payload.filas[l]);  
            }

            //Nested Function para tratar os vários casos e eliminar a recursividade ou repetição
            async function regModulo(m)
            {
              //Permissão atribuída ou não alterada
              if(m.tem===true)
              {
                await knex('public.supervisores_modulos')
                .transacting(t)
                .insert({
                  id_supervisor: request.params.ID,
                  id_modulo: m.id,
                  extra_usuario: m.extra_usuario===true ? 'G' : null 
                });
              }
              
              //Só loga quando está na UI e é um módulo children (que tem parent != null)
              if(request.query.userreq && m.parent)
              {
                //Loga alteração de gravação
                if(m.extra == 'G')
                {
                  await knex('public.supervisores_log')
                  .transacting(t)
                  .insert({
                      id_supervisor: request.query.userreq, 
                      acao: 'U',
                      gravacao: m.extra_usuario===true ? 'A' : 'R',
                      id_user: request.params.ID,
                      descricao: `${m.extra_usuario===true ? 'Adicionou' : 'Removeu'} acesso a gravação de ${m.descricao}. IP: ${util.GetRequestIP(request)}`,
                  });
                }

                //Loga alteração de permissão
                await knex('public.supervisores_log')
                .transacting(t)
                .insert({
                    id_supervisor: request.query.userreq, 
                    acao: m.tem===true ? 'A' : 'R',
                    modulo: m.id,
                    id_user: request.params.ID,
                    descricao: `${m.tem===true ? 'Adicionou' : 'Removeu'} acesso ao módulo ${m.descricao}. IP: ${util.GetRequestIP(request)}`,
                });
              }
              return;
            }

            async function regfila(fila)
            {
              //Somente se estiver marcada
              if(fila.tem===true)
              {
                await knex('public.supervisores_filas')
                .transacting(t)
                .insert({
                  id_supervisor: request.params.ID,
                  id_fila: fila.name
                });
              }
            }
            //Commit final da transação é automáico na ausência de erros?
        })
        .catch(function(e){
          throw e;
        })
    })
    .catch(t.rollback)
  })
  .then(function() {
    console.log('Permissões alteradas com sucesso!');
    const ret = {};
    ret.error = false;
    ret.message = `Usuário id = ${request.params.ID} teve suas permissões alteradas com sucesso.`;
    reply(ret);
  })
  .catch(function(e) {
    console.log(`Falha ao alterar permissões do usuário ${request.params.ID}. Erro = ${e}`);
    reply(Boom.badData(e));
  });
};

exports.delete = (request, reply) => {
  const knex = db.knex;
  knex.transaction(function(t) {
    return knex('public.supervisores_modulos')
     .transacting(t)
     .where('id_supervisor', request.params.ID)
     .del()
     .then(function() {
          return knex('public.supervisores_filas')
             .where('id_supervisor', request.params.ID)
             .del()
             .then(function() {
                return knex('public.supervisores')
                  .where('id', request.params.ID)
                  .del()
                  .then(function() {
                    //Só loga quando está UI
                    if(request.query.userreq)
                    {
                      const log = {
                        id_supervisor: request.query.userreq, 
                        acao: 'R',
                        id_user: request.params.ID,
                        descricao: 'Usuário de id = '+ request.params.ID +' removido. IP: '+ ip,
                      }; 
                      return knex('public.supervisores_log')
                        .insert(log)
                        .then(t.commit)
                        .catch(t.rollback);
                    }
                  })
                  .catch(t.rollback);
              })
             .catch(t.rollback);
     })
     .catch(t.rollback)
  })
  .then(function() {
    console.log('Usuário apagado com sucesso!');
    const ret = {};
    ret.error = false;
    ret.message = 'Usuário id = '+request.params.ID+' apagado com sucesso.';
    reply(ret);
  })
  .catch(function(e) {
    console.log('Falha ao apagar usuário. Erro = '+e);
    reply(Boom.badData(e));
  });
};