'use strict'

const joi = require('joi');
const Boom = require('boom');
const db = require('../../../util/db');
//const squel = require("squel");
const ws = require('../../../ws');
const config = require('../../../config.json');
const moment = require('moment-timezone');
moment.tz.setDefault(config.servidor.timezone);

exports.getall = (request, reply) => {
  const max = request.query.max ? request.query.max : 15;
  const page = request.query.page ? request.query.page : 1;
  const knex = db.knex('public.agentes_status');

  if(request.query.nome) knex.whereRaw('LOWER(nome) LIKE ?', '%'+decodeURIComponent(request.query.nome).toLowerCase()+'%');
  if(request.query.fila) knex.where("fila", request.query.fila);
  if(request.query.produtiva) knex.where("produtiva", request.query.produtiva);

  if(max === -1){
    //console.log('SQL = '+ knex.toString());
    knex.clearSelect();
    // Parametros de ordenação, fora do count
    if(request.query.order){
      for(let p=0;p<request.query.order.length;++p){
        if(request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
            knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
        }
        else if(request.query.order[p] && request.query.order[p].campo) {
          knex.orderBy(request.query.order[p].campo, 'ASC');
        }
      }
    }
    knex.select('*')
    .then(function(rows) {
      const ret = {};
      ret.error = false;
      ret.full = true;
      ret.total = rows.length;
      ret.message = 'Lista de todas as pausas';
      ret.dados = rows;
      reply(ret);
    })
    .catch(function(error) {
      console.error(error);
      reply(Boom.badData(error));
    });
  }
  else {
    const offset = max * (page - 1);
    //console.log('SQL = '+knex.count('* as total').toString());
    knex.count('* as total')
    .then(function(total) {
      if(total[0].total > 0){
        knex.clearSelect();
        // Parametros de ordenação, fora do count
        if(request.query.order){
          for(let p=0;p<request.query.order.length;++p){
            if(request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
            }
            else if(request.query.order[p] && request.query.order[p].campo) {
              knex.orderBy(request.query.order[p].campo, 'ASC');
            }
          }
        }
        //console.log('SQL = '+knex.select('*').limit(max).offset(offset).toString());
        knex.select('*').limit(max).offset(offset).then(function(rows) {
          const pages = Math.ceil(parseInt(total[0].total) / max);
          const ret = {};
          ret.error = false;
          ret.full = pages === 1 ? true : false;
          ret.page = page;
          ret.max = max;
          ret.offset = offset;
          ret.total = parseInt(total[0].total);
          ret.pages = pages;
          ret.message = 'Lista de pausas, paginada';
          ret.dados = rows;
          reply(ret);
        })
        .catch(function(error) {
          console.error(error);
          reply(Boom.badData(error));
        });
      }
      else {
        const ret = {};
        ret.error = true;
        ret.total = 0;
        ret.message = 'Nenhum registro encontrado';
        reply(ret);
      }
    })
    .catch(function(error) {
      console.error(error);
      reply(Boom.badData(error));
    });
  }
};

exports.getOne = (request, reply) => {
  const knex = db.knex('public.agentes_status');
  knex.where('id', request.params.ID);
  knex.select('*')
  .then(function(rows) {
    const ret = {};
    ret.error = false;
    ret.full = true;
    ret.total = rows.length;
    ret.message = 'Dados da pausa id = '+request.params.ID;
    ret.dados = rows[0];
    reply(ret);
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};
