'use strict'

//const joi = require('joi');
const Boom = require('boom');
const db = require('../../../util/db');
const _ = require('lodash');
const restbase = require('../../../util/restbase');
const config = require('../../../config.json');
const moment = require('moment-timezone');
moment.tz.setDefault(config.servidor.timezone);

function ordenContexto(a)
{
    const rt = _.chain(a)
               .sortBy('context') 
               .groupBy('context')
               .map(function(values, key)
               {
                  let c= {
                        contexto: values[0].context,
                        sentido: values[0].sentido,
                        descricao: values[0].descricao,
                  };
                  
                  c.rotas = _.chain(values)
                  .sortBy('exten') 
                  .groupBy('exten')
                  .map(function(v)
                  {
                      return v[0];
                  }).value();

                  return c;
               }).value();

    return rt;
}

function ordenContextoURA(a)
{
  const rt = [];
  for (let i = 0; i < a.length; i++){
    //Emula o foreach e remove o elemento da lista
    let e = a[i];
    //Monta o retorno
    let r= {
      contexto: e.context,
      sentido: e.sentido,
      descricao: e.descricao,
      rotas: a.filter(function(row){
        return row.context == e.context && row.sentido == e.sentido && row.descricao == e.descricao;
      })
    };
    if(!Array.isArray(r.rotas)) r.rotas = [e]; 
    rt.push(r);
    _.remove(a,function(row){
      return row.context == e.context && row.sentido == e.sentido && row.descricao == e.descricao;
    });
  }
  return rt;
}

// //Processa os resultados para obter os dados no formato esperado na interface
// function ordenContexto(rows){
//     if(!rows || !Array.isArray(rows)) return [];

//     let x = util.nestElements(rows,['context','sentido','descricao']);
//     console.log(JSON.stringify(x));
//     return x;
// }


exports.getall = (request, reply) => {
  const max = request.query.max ? request.query.max : 15;
  const page = request.query.page ? request.query.page : 1;
  const knex = db.knex('public.rotas as a')
  .leftOuterJoin('public.contexto as b', 'a.context', 'b.context');
  
  if(request.query.nome) knex.where('a.nome', request.query.nome);
  if(request.query.context) knex.where('b.context', request.query.context);
  if(request.query.exten) knex.where('a.exten', request.query.exten);
  if(request.query.priority) knex.where('a.priority', request.query.priority);
  if(request.query.app) knex.where('a.app', request.query.app);
  if(request.query.sentido) knex.where('b.sentido', request.query.sentido);
  
  if(max === -1){
    //console.log('SQL = '+ knex.toString());
    knex.clearSelect();
    knex.orderBy("a.context", "ASC").orderBy("a.exten", "ASC");
    // Parametros de ordenação, fora do count
    if(request.query.order){
      for(let p=0;p<request.query.order.length;++p){
        if(request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
            knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
        }
        else if(request.query.order[p] && request.query.order[p].campo) {
          knex.orderBy(request.query.order[p].campo, 'ASC');
        }
      }
    }
    knex.select('a.context', 'a.exten', 'a.priority', 'a.app', 'a.appdata', 'a.nome', 'b.sentido', 'b.descricao')
    .then(function(rows) {
      const ret = {};
      ret.error = false;
      ret.full = true;
      ret.total = rows.length;
      ret.message = 'Lista de todas as rotas';
      ret.dados = ordenContexto(rows);
      reply(ret);
    })
    .catch(function(error) {
      console.error(error);
      reply(Boom.badData(error));
    });
  }
  else {
    const offset = max * (page - 1);
    console.log('SQL = '+knex.count('* as total').toString());
    knex.count('* as total')
    .groupBy('a.context', 'a.exten')
    .then(function(total) {
      if(total[0].total > 0){
        knex.clearSelect();
        knex._clearGrouping('group');
        // Parametros de ordenação, fora do count
        if(request.query.order){
          for(let p=0;p<request.query.order.length;++p){
            if(request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
            }
            else if(request.query.order[p] && request.query.order[p].campo) {
              knex.orderBy(request.query.order[p].campo, 'ASC');
            }
          }
        }
        //console.log('SQL = '+knex.select('*').limit(max).offset(offset).toString());
        //knex.select(db.knex.raw('DISTINCT on (a.context, a.exten) a.id'), 'a.context', 'a.exten', 'a.priority', 'a.app', 'a.appdata', 'a.nome', 'b.sentido', 'b.descricao')
        knex.select('a.context', 'a.exten', 'a.priority', 'a.app', 'a.appdata', 'a.nome', 'b.sentido', 'b.descricao')
        .limit(max).offset(offset).then(function(rows) {
          const pages = Math.ceil(parseInt(total[0].total) / max);
          const ret = {};
          ret.error = false;
          ret.full = pages === 1 ? true : false;
          ret.page = page;
          ret.max = max;
          ret.offset = offset;
          ret.total = parseInt(total[0].total);
          ret.pages = pages;
          ret.message = 'Lista de rotas paginada';
          ret.dados = ordenContexto(rows);
          reply(ret);
        })
        .catch(function(error) {
          console.error(error);
          reply(Boom.badData(error));
        });
      }
      else {
        const ret = {};
        ret.error = true;
        ret.total = 0;
        ret.message = 'Nenhum registro encontrado';
        reply(ret);
      }
    })
    .catch(function(error) {
      console.error(error);
      reply(Boom.badData(error));
    });
  }
};

exports.getUra = (request, reply) => {
  const knex = db.knex('public.contexto as b')
  .leftJoin('public.rotas as a', 'a.context', 'b.context')
  .where("b.context", request.query.context)
  .orderBy("b.context","ASC") 
  .orderBy("a.exten","ASC") 
  .orderBy("a.priority","ASC") 
  .select('b.context', 'a.exten', 'a.priority', 'a.app', 'a.appdata', 'a.nome', 'b.sentido', 'b.descricao')
  .then(function(rows) {
    const ret = {};
    ret.error = false;
    ret.full = true;
    ret.total = rows.length;
    ret.message = 'Dados da rota de URA com Contexto = '+request.query.context;
    ret.dados = ordenContextoURA(rows);
    reply(ret);
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};

exports.getByExtenContext = (request, reply) => {
  const knex = db.knex('public.rotas');
  knex.where({
    exten: request.params.exten,
    context: request.params.context});
  knex.orderBy("priority","ASC"); 
  knex.select('*')
  .then(function(rows) {
    const ret = {};
    ret.error = false;
    ret.full = true;
    ret.total = rows.length;
    ret.message = 'Dados da rota com Prefixo = "'+request.params.exten+'" e Grupo = "'+request.params.context+'"';
    ret.dados = rows;
    reply(ret);
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};

exports.limpaGrupo = (request, reply) => {
  const knex = db.knex('public.rotas');
  knex.where('context', request.params.context)
  .del()
  .then(function(rows) {
    if(rows > 0){
      const ret = {};
      ret.error = false;
      ret.total = rows;
      ret.message = 'Rotas do grupo = '+request.params.context+' apagadas com sucesso.';
      reply(ret);
    }
    else {
      reply(Boom.badData("Nenhum registro encontrado para grupo = "+request.params.context));
    }
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};

exports.deleteByExtenContext = (request, reply) => {
  const knex = db.knex('public.rotas');
  knex.where({
    exten: request.params.exten,
    context: request.params.context})
  .del()
  .then(function(rows) {
    if(rows > 0){
      const ret = {};
      ret.error = false;
      ret.total = rows;
      ret.message = 'Rotas do prefixo = "'+request.params.exten+'" e grupo = "'+request.params.context+'" apagadas com sucesso.';
      reply(ret);
    }
    else {
      reply(Boom.badData("Nenhum registro encontrado para prefixo = '"+request.params.exten+"' e grupo = '"+request.params.context+"'"));
    }
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};

exports.clonaRota = (request, reply) => {
  //Limpa o id do array, se existir
  request.payload.forEach(function(v){ 
    if(v.id)
      delete v.id 
    else
     return;  
  });
  //Prepara o retorno
  const ret = {};
  //Insere o bunch
  const knex = db.knex;
  knex.transaction(function(t) {
     return db.knex('public.rotas')
    .transacting(t)
    .insert(request.payload)
    .returning('id')
    .then(t.commit)
    .catch(t.rollback)
 })
 .then(function(inserts) {
    // it worked
    const ret = {};
    ret.error = false;
    ret.ids = inserts;
    ret.message = 'IDs = '+inserts.join(',')+' criados com sucesso.';
    reply(ret);
 })
 .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
 });
};

exports.clonaGrupo = (request, reply) => {
  (async () => {
    try {
      await db.query('BEGIN')
      if(!request.payload || !Array.isArray(request.payload)) throw 'É necessário informar um ou mais destinos.';
      //forEach não funciona bem em métodos assíncronos
      for(let i=0;i<request.payload.length;i++)
      {
        console.log('1');
        await db.query(`INSERT INTO public.rotas(context,exten,priority,app,appdata,nome) 
                        (SELECT $1,exten,priority,app,appdata,nome 
                           FROM public.rotas 
                          WHERE context = $2 ORDER by exten,priority)`, 
                          [request.payload[i], request.params.origem]);
      }
      await db.query('COMMIT');
      //Tudo certo
      const ret = {
        error: false,
        message: 'Clonagem bem sucedida',
      };
      reply(ret);
    } catch (e) {
      await db.query('ROLLBACK')
      throw e
    } 
  })().catch(function(e){
    console.error(e);
    reply(Boom.badData(e));
  });
};

exports.updateRotaFull = (request, reply) => {
  //Limpa o id do array, se existir
  request.payload.forEach(function(v){ 
    if(v.id) delete v.id; 
    if(v.rota_original) delete v.rota_original; 
    return;  
  });
  //Prepara o retorno
  const ret = {};
  //Insere o bunch
  const knex = db.knex;
  knex.transaction(function(t) {
     return db.knex('public.rotas')
    .transacting(t)
    .where('context',request.payload[0].context)
    .del()
    .then(function(){
      return db.knex
      .batchInsert('public.rotas', request.payload, request.payload.length)
      .transacting(t)
      .returning('id')
      .then(function(inserts){
        ret.error = false;
        ret.ids = inserts;
        ret.message = 'IDs = '+inserts.join(',')+' criados com sucesso.';
      })
      .catch(t.rollback)
    })
    .catch(t.rollback);
 })
 .then(function() {
    // it worked
    reply(ret);
 })
 .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
 });
};

exports.updateRota = (request, reply) => {
  let rota_original = request.payload[0].rota_original;
  let bunch = [];
  //Limpa o id do array, se existir
  request.payload.forEach(function(v){ 
    if(v.id) delete v.id; 
    delete v.rota_original; 
    bunch.push(v);
    return;  
  });
  //Prepara o retorno
  const ret = {};
  //Insere o bunch
  const knex = db.knex;
  knex.transaction(function(t) {
     return db.knex('public.rotas')
    .transacting(t)
    .where('context', request.payload[0].context)
    .where('exten',rota_original)
    .del()
    .then(function(){
      return db.knex
      .batchInsert('public.rotas', bunch, bunch.length)
      .transacting(t)
      .returning('id')
      .then(function(inserts){
        ret.error = false;
        ret.ids = inserts;
        ret.message = 'IDs = '+inserts.join(',')+' criados com sucesso.';
      })
      .catch(t.rollback)
    })
    .catch(t.rollback);
 })
 .then(function() {
    // it worked
    reply(ret);
 })
 .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
 });
};

exports.uraFluxo = (request, reply) => {
  const knex = db.knex('public.ura_fluxo');
  if(request.query.contexts && Array.isArray(request.query.contexts) && request.query.contexts.length > 0) knex.whereIn('menu', request.query.contexts);
  if(request.query.extens && Array.isArray(request.query.extens) && request.query.extens.length > 0) knex.whereIn('src', request.query.extens);
  if(request.query.data_ini) knex.where('calldate', '>=', moment(request.query.data_ini).format('YYYY-MM-DDTHH:mm:ss')+'Z');
  if(request.query.data_fim) knex.where('calldate', '<=', moment(request.query.data_fim).format('YYYY-MM-DDTHH:mm:ss')+'Z');
  if(request.query.id_client) knex.whereRaw(restbase.buildObjectSearchExpression("id_client", request.query.id_client));
  if(request.query.data_a) knex.whereRaw(restbase.buildObjectSearchExpression('data_a', request.query.data_a));
  if(request.query.data_b) knex.whereRaw(restbase.buildObjectSearchExpression('data_b', request.query.data_b));
  if(request.query.src) knex.whereRaw(restbase.buildObjectSearchExpression('src', request.query.src));
  if(request.query.menu) knex.whereRaw(restbase.buildObjectSearchExpression('menu', request.query.menu));
  if(request.query.src_menu) knex.whereRaw(restbase.buildObjectSearchExpression('src_menu', request.query.src_menu));
  if(request.query.opcao) knex.whereRaw(restbase.buildObjectSearchExpression('opcao', request.query.opcao));
  
  knex.orderBy("calldate","ASC"); 
  knex.select('*')
  .then(function(rows) {
    const ret = {};
    ret.error = false;
    ret.full = true;
    ret.total = rows.length;
    ret.message = 'Fluxo de URAs';
    ret.dados = rows;
    reply(ret);
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};
