'use strict'

const fs = require('fs');
const Boom = require('boom');
const db = require('../../../util/db');
const ws = require('../../../ws');
const config = require('../../../config.json');
const moment = require('moment-timezone');
const namiLib = require("nami");
const Ami = require('../../../ami');

moment.tz.setDefault(config.servidor.timezone);
//FileSystem Promisses (Node v10.16.0 - Experimental)
const fsPromises = fs.promises;
const config_path = '/etc/asterisk/sip_trunk.conf';

//Função para capturar a situação dos troncos no asterisk
async function sipControllerState(rows) {

  //Registros de resposta
  for (let i in rows) {
    // //Consulta Asterisk
    // let action = new namiLib.Actions.SipShowPeer();
    // action.Peer = rows[i].nome;
    // await Ami.nami.send(action, function(result){
    //   if(result.response === 'Success')
    //   {
    //     rows[i].status = result.status;
    //     rows[i].ipregistro = result.address_ip+":"+result.addres_port;
    //     //console.log(`Tronco ${rows[i].nome}, AMI Result: ${Object.getOwnPropertyNames(result)}.`);
    //     console.log(`Tronco ${rows[i].nome}, AMI Result: ${result.status} ${result.address_ip}.`);
    //   }
    //   else
    //   {
    //     rows[i].status = result.message;
    //     rows[i].ipregistro = "";
    //     //console.log(`Tronco ${rows[i].nome}, AMI Result: ${result.message}.`);
    //   }
    // });

    let action = new namiLib.Actions.SipShowPeer();
    action.Peer = rows[i].nome;
    await callNami(action)
      .then((result) => {
        rows[i].status = result.status;
        rows[i].ipregistro = result.address_ip + ":" + result.address_port;
        //console.log(`Tronco ${rows[i].nome}, AMI Result: ${result.status} ${result.address_ip}.`);
      })
      .catch((e) => {
        rows[i].status = e;
        rows[i].ipregistro = "";
        console.log("Asterisk Fails: " + e);
      });

  }
  return rows;
};

const callNami = (action) => {
  return new Promise((resolve, reject) => {
    Ami.nami.send(action, function (result) {
      setTimeout(() => {
        //console.log("AMI RAW Result:" + result);
        if (result.response == "Success") {
          //Retorna os dados
          resolve(result);
        }
        else {
          reject(result.message);
        }
      }, 500);
    });
  });
};

exports.getall = async (request, reply) => {
  const max = request.query.max ? request.query.max : 15;
  const page = request.query.page ? request.query.page : 1;
  const knex = db.knex('public.troncos');

  if (request.query.context) knex.where('context', request.query.context);
  if (request.query.exten) knex.where('exten', request.query.exten);

  //console.log('Chamando sip show peer na rota');
  //Ami.SipShowPeer('tip');

  if (max === -1) {
    //console.log('SQL = '+ knex.toString());
    knex.clearSelect();
    // Parametros de ordenação, fora do count
    if (request.query.order) {
      for (let p = 0; p < request.query.order.length; ++p) {
        if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
          knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
        }
        else if (request.query.order[p] && request.query.order[p].campo) {
          knex.orderBy(request.query.order[p].campo, 'ASC');
        }
      }
    }
    knex.select('*')
      .then(async function (rows) {

        for (let tronco of rows) {
          let peer = await Ami.SipShowPeer(tronco.nome);
          tronco.status = peer.status;
          tronco.ip = peer.address_ip;
        }

        const ret = {};
        ret.error = false;
        ret.full = true;
        ret.total = rows.length;
        ret.message = 'Lista de todos os troncos';
        ret.dados = rows;

        reply(ret);
      })
      .catch(function (error) {
        console.error(error);
        reply(Boom.badData(error));
      });
  }
  else {
    const offset = max * (page - 1);
    //console.log('SQL = '+knex.count('* as total').toString());
    knex.count('* as total')
      .then(function (total) {
        if (total[0].total > 0) {
          knex.clearSelect();
          // Parametros de ordenação, fora do count
          if (request.query.order) {
            for (let p = 0; p < request.query.order.length; ++p) {
              if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
              }
              else if (request.query.order[p] && request.query.order[p].campo) {
                knex.orderBy(request.query.order[p].campo, 'ASC');
              }
            }
          }
          //console.log('SQL = '+knex.select('*').limit(max).offset(offset).toString());
          knex.select('*').limit(max).offset(offset).then(async function (rows) {
            const pages = Math.ceil(parseInt(total[0].total) / max);
            const ret = {};
            ret.error = false;
            ret.full = pages === 1 ? true : false;
            ret.page = page;
            ret.max = max;
            ret.offset = offset;
            ret.total = parseInt(total[0].total);
            ret.pages = pages;
            ret.message = 'Lista de troncos paginada';
            ret.dados = rows;
            reply(ret);
          })
            .catch(function (error) {
              console.error(error);
              reply(Boom.badData(error));
            });
        }
        else {
          const ret = {};
          ret.error = true;
          ret.total = 0;
          ret.message = 'Nenhum registro encontrado';
          reply(ret);
        }
      })
      .catch(function (error) {
        console.error(error);
        reply(Boom.badData(error));
      });
  }
};

exports.getOne = (request, reply) => {
  const knex = db.knex('public.troncos');
  knex.where('id', request.params.ID);
  knex.select('*')
    .then(async function (rows) {
      const ret = {};
      ret.error = false;
      ret.full = true;
      ret.total = rows.length;
      ret.message = 'Dados do Tronco id = ' + request.params.ID;
      ret.dados = await cfgFromFile(config_path, rows[0], false);
      reply(ret);
    })
    .catch(function (error) {
      console.error(error);
      reply(Boom.badData(error));
    });
};

exports.getState = async (request, reply) => {
  const troncos = request.payload.troncos.map((x) => {
    return { nome: x };
  });

  const ret = {};
  ret.error = false;
  ret.full = true;
  ret.total = 1;
  ret.message = 'Dados de Registro dos Troncos = ' + request.payload.troncos.join(', ');
  sipControllerState(troncos)
    .then((t) => {
      ret.dados = t;
      reply(ret);
    })
    .catch((e) => {
      console.error(e);
      reply(Boom.badData(e));
    });
};

exports.create = (request, reply) => {
  var cfg = request.payload.conf;
  delete request.payload.conf;
  let trunkId = 0;
  const knex = db.knex;
  knex.transaction(function (t) {
    return db.knex('public.contexto')
      .select('sentido')
      .where('context', request.payload.rota)
      .transacting(t)
      .then(function (rows) {
        //Validação
        if (rows[0].sentido != 'Entrada') throw `Troncos devem ter rotas de Entrada. A rota ${request.payload.rota}, com sentido ${rows[0].sentido}, não é válida para uso.`;
        request.payload
        //Insere o tronco
        return db.knex('public.troncos')
          .insert(request.payload)
          .transacting(t)
          .returning('id')
          .then(async function (id) {
            trunkId = id[0];
            if (cfg && Array.isArray(cfg))
              cfg.push(`context=${request.payload.rota}`);

            request.payload.conf = cfg;
            return await registerTrunk(request.payload)
              .then(async () => {
                let action = new namiLib.Actions.Reload();
                return await callNami(action)
                  .then(t.commit)
                  .catch((e) => { throw e; });
              })
              .catch((e) => { throw e; });
          })
          .catch((e) => { throw e; });
      })
      .catch(e => { throw e; });
  })
    .then(function () {
      const ret = {};
      ret.error = false;
      ret.id = trunkId;
      ret.message = 'Tronco id = ' + trunkId + ', criado com sucesso.';
      reply(ret);
    })
    .catch(function (error) {
      console.error(error);
      reply(Boom.badData(error));
    });
};

exports.update = (request, reply) => {
  var cfg = request.payload.conf;
  delete request.payload.conf;
  const knex = db.knex;
  knex.transaction(function (t) {
    return db.knex('public.contexto')
      .select('sentido')
      .where('context', request.payload.rota)
      .then(function (rows) {
        //Validação
        if (rows[0].sentido != 'Entrada') throw `Troncos devem ter rotas de Entrada. A rota ${request.payload.rota}, com sentido ${rows[0].sentido}, não é válida para uso.`;
        //Lê os dados do tronco persistidos
        return db.knex('public.troncos')
          .select('registro')
          .where('id', request.params.ID)
          .transacting(t)
          .then(function (rows) {
            //Atualiza o tronco
            return db.knex('public.troncos')
              .update(request.payload)
              .where('id', request.params.ID)
              .then(async function () {
                request.payload.oldRegistro = rows[0].registro;
                if (cfg && Array.isArray(cfg))
                  cfg.push(`context=${request.payload.rota}`);
                request.payload.conf = cfg;
                return await registerTrunk(request.payload)
                  .then(async () => {
                    let action = new namiLib.Actions.Reload();
                    return await callNami(action)
                      .then(t.commit)
                      .catch((e) => { throw e; });
                  })
                  .catch((e) => { throw e; });
              })
              .catch((e) => { throw e; });
          })
          .catch((e) => { throw e; });
      })
      .catch(e => { throw e; });
  })
    .then(function () {
      const ret = {};
      ret.error = false;
      ret.id = request.params.ID;
      ret.message = 'Tronco id = ' + request.params.ID + ', alterado com sucesso.';
      reply(ret);
    })
    .catch(function (error) {
      console.error(error);
      reply(Boom.badData(error));
    });
};

exports.delete = (request, reply) => {
  const knex = db.knex;
  knex.transaction(function (t) {
    return knex('public.troncos')
      .transacting(t)
      .where('id', request.params.ID)
      .select('*')
      .then(function (rows) {
        return knex('public.troncos')
          .transacting(t)
          .where('id', request.params.ID)
          .del()
          .then(async function () {
            var data = await cfgFromFile(config_path, rows[0], true);
            return await deleteFromFile(config_path, data)
              .then(async () => {
                let action = new namiLib.Actions.Reload();
                return await callNami(action)
                  .then(t.commit)
                  .catch((e) => { throw e; });
              })
              .catch((e) => { throw e; });
          })
          .catch(e => { throw e; });
      })
      .catch(e => { throw e; });
  })
    .then(function () {
      console.log('Tronco apagado com sucesso!');
      const ret = {};
      ret.error = false;
      ret.message = 'Tronco id = ' + request.params.ID + ' apagado com sucesso.';
      reply(ret);
    })
    .catch(function (e) {
      console.log('Falha ao apagar tronco. Erro = ' + e);
      reply(Boom.badData(e));
    });
};

//Efetua o registro de um tronco
async function registerTrunk(tdata) {
  const directive = 'register = ';
  let reg = `${directive}${tdata.registro}`;
  var filehandle = null;
  var content = "";
  var changed = false;
  var newFile = false;
  try {
    try {
      //Leitura e escrita
      filehandle = await fsPromises.open(config_path, 'r+');
    }
    catch (err) {
      //Não existindo
      console.log('sip_trunk.conf:' + err);
      filehandle = await fsPromises.open(config_path, 'a+');
      newFile = true;
    }
    content = await filehandle.readFile({ encoding: 'utf8' });

    //Config  
    if (tdata.conf && Array.isArray(tdata.conf) && tdata.conf.length > 0) {
      const config = stringfyConfig(tdata.nome, tdata.conf); //`[${tdata.nome}]\n${tdata.conf.join('\n').replace('\n\n','')}${tdata.conf.indexOf('\n')==tdata.conf.length?'\n':'\n\n'}`; 
      const oldConfig = stringfyConfig(tdata.nome, extractConfig(content, tdata.nome));
      //console.log("oldConfig: " +oldConfig);    
      //console.log("Config: " +config);   
      if (oldConfig) {
        content = content.replace(oldConfig, config);
        //console.log('Efetuado replace');
      }
      else
        content = content + '\n' + config;

      //console.log(content);

      changed = true;
    }

    if (tdata.oldRegistro) content = content.replace(directive + tdata.oldRegistro, '');

    //Sip registrado
    if (tdata.registro) {
      content = content + '\n' + reg;
      changed = true;
    }

    //Grava tudo
    if (changed) {
      if (!newFile) {
        await filehandle.truncate(0);
        await filehandle.sync();
      }
      await filehandle.write(content, 0, 'utf8');
    }

  } finally {
    if (filehandle) {
      //Fecha o arquivo
      await filehandle.close();
    }
  }
};

function stringfyConfig(name, data) {
  if (data && Array.isArray(data) && data.length > 0)
    return `[${name}]\n${data.join('\n').replace('\n\n', '')}${data.indexOf('\n') == data.length ? '\n' : '\n\n'}`;
  else
    return "";
}

async function deleteFromFile(file, data) {
  let filehandle = null;
  var content = "";
  try {
    //Leitura e escrita
    filehandle = await fsPromises.open(file, 'r+');
    content = await filehandle.readFile({ encoding: 'utf8', flag: 'r' });
    if (data.registro) content = content.replace('register = ' + data.registro, '');
    if (data.conf) content = content.replace(stringfyConfig(data.nome, data.conf), '');
    await filehandle.truncate(0);
    await filehandle.sync();
    await filehandle.write(content, 0, 'utf8');
  } finally {
    if (filehandle) {
      //Fecha o arquivo
      await filehandle.close();
    }
  }
};

async function cfgFromFile(file, data, full) {
  let filehandle = null;
  if (full == null || full == undefined) full = true;
  var content = "";
  try {
    if (data && data.nome) {
      //Leitura e escrita
      filehandle = await fsPromises.open(file, 'r+');
      content = await filehandle.readFile({ encoding: 'utf8', flag: 'r' });
      await filehandle.close();
      filehandle = null;
      data.conf = extractConfig(content, data.nome);
      if (!full && data.conf) {
        for (let index = 0; index < data.conf.length; index++) {
          const element = data.conf[index];
          if (element.startsWith('context=')) {
            data.conf.splice(index, 1);
            index = data.conf.length;
          }
        }
      }
    }
    return data;
  } finally {
    if (filehandle) {
      //Fecha o arquivo
      await filehandle.close();
    }
  }
};

function extractConfig(text, cfgName) {
  if (text && cfgName) {
    var start = text.indexOf("[" + cfgName + "]")
    var cfgRaw = text.substr(start, text.indexOf("\n\n", start) - start);
    var cfg = null
    if (cfgRaw && cfgRaw[0] == '[') {
      cfg = cfgRaw.split('\n');
      //O primeiro elemento é o identificador, e ele não existe na vida real
      cfg.splice(0, 1);
    }

    return cfg;
  }
  else
    return [];
}