const Boom = require('boom');
const Supervisores = require('../../../models/Supervisores');
const twoFactor = require('node-2fa');

exports.create = async (request, reply) => {
    try {
        const user = request.auth.credentials.login;

        const newSecret = twoFactor.generateSecret({ name: `${request.payload.nome} - ${user}`, account: "X5 Soluções LTDA" });

        await Supervisores.query
            .require(false)
            .where({ nome: user })
            .update({ duplo_fator_secret: newSecret.secret, duplo_fator_uri: newSecret.uri, duplo_fator_qr: newSecret.qr });

        reply({ duplo_fator_qr: newSecret.qr, duplo_fator_uri: newSecret.uri });
    } catch (error) {
        console.error(error);
        reply(Boom.badData(error));
    }

}

exports.getAll = async (request, reply) => {
    try {
        const user = request.auth.credentials.login;

        let ret = await Supervisores.query
            .require(false)
            .fields(['id', 'nome', 'fullname', 'duplo_fator_uri', 'duplo_fator_qr'])
            .where({ nome: user })
            .first()
            .fetch();

        reply(ret);
    } catch (e) {
        reply(Boom.badData(e));
    }
};

exports.delete = async (request, reply) => {
    try {
        const user = request.auth.credentials.login;

        await Supervisores.query
            .require(false)
            .where({ nome: user })
            .update({ duplo_fator_secret: null, duplo_fator_uri: null, duplo_fator_qr: null });

        reply(true);
    } catch (error) {
        console.error(error);
        reply(Boom.badData(error));
    }

}