const Boom = require('boom');
const db = require('../../../util/db');
const md5 = require('md5');
const Agente = require('../../../models/Agente');
const AgenteFilas = require('../../../models/AgenteFilas');
const AgenteGruposFila = require('../../../models/AgenteGruposFila')
const Ami = require('../../../ami');
const AmiEvent = require('../../../ami/Events');
const AutenticacaoFontes = require('../../../models/AutenticacaoFontes')
const path = require('path')
const fs = require('fs')
const _ = require('underscore')
const util = require("util");
const { getAtendimento } = require('../../../xc_modules/omni/class/Atendimento');


exports.create = async (request, reply) => {
    const transaction = new db.orm.Transaction();
    const AgenteFilasT = transaction.models.AgenteFilas;
    const AgenteGruposFilaT = transaction.models.AgenteGruposFila
    const AgenteT = transaction.models.Agente;
    try {
        var { filas, grupo_filas, ...agData } = request.payload;
        agData.senha = md5(agData.senha);
        const agente = await new AgenteT(agData).save();

        const gruposFilasInsert = [];
        if (grupo_filas && grupo_filas.length > 0) {
            for (const grupo of grupo_filas) {
                const agGrupo = {
                    id_agente: agente.id,
                    fila_grupo: grupo
                }
                gruposFilasInsert.push(agGrupo);
            }
        }

        const filasInsert = [];
        if (filas && filas.length > 0) {
            for (const fila of filas) {
                const agFila = {
                    id_agente: agente.id,
                    fila: fila
                }
                filasInsert.push(agFila);
            }
        }

        await AgenteGruposFilaT.insert(gruposFilasInsert)
        await AgenteFilasT.insert(filasInsert);
        await transaction.commit()
        reply(agente);
    } catch (error) {
        await transaction.rollback();
        reply(Boom.badData(error));
    }
};

exports.update = async (request, reply) => {
    // Crie a transação
    const transaction = new db.orm.Transaction();
    const AgenteFilasT = transaction.models.AgenteFilas;
    const AgenteGruposFilaT = transaction.models.AgenteGruposFila
    const AgenteT = transaction.models.Agente;

    try {
        var { filas, grupo_filas, ...agData } = request.payload;
        const agg = await AgenteGruposFilaT.query
            .require(false)
            .where({ id_agente: request.params.AgenteID })
            .delete()

        const gruposFilasInsert = [];
        if (grupo_filas && grupo_filas.length > 0) {
            for (const grupo of grupo_filas) {
                const agGrupo = {
                    id_agente: request.params.AgenteID,
                    fila_grupo: grupo
                }
                gruposFilasInsert.push(agGrupo);
            }
        }

        await AgenteGruposFilaT.insert(gruposFilasInsert);

        // Pega as filas que o agente estava
        const agf = await AgenteFilasT.query
            .require(false)
            .where({ id_agente: request.params.AgenteID })
            .delete();
        let filasantigas = [];
        if (agf && agf.length > 0) {
            filasantigas = agf.map(f => f.fila);
        }

        agData.id = request.params.AgenteID;
        if (agData.senha) agData.senha = md5(agData.senha);

        const teup = await AgenteT.update(agData, { require: true });
        const agenteup = teup[0];

        const filasInsert = [];
        if (filas && filas.length > 0) {
            for (const fila of filas) {
                const agFila = {
                    id_agente: request.params.AgenteID,
                    fila: fila
                }
                filasInsert.push(agFila);
            }
        }
        await AgenteFilasT.insert(filasInsert);
        await transaction.commit();

        // Pega a diferença entre as filas que o agente estava e em quais deve ficar e desloga o agente delas
        let difference = filasantigas.filter(x => !filas.includes(x));
        if (difference && difference.length > 0) {
            agenteup.filas_deslogado = difference;
            agenteup.filas = filas;
            for (let fila of difference) {
                agenteup.fila = fila;
                Ami.QueueRemove(agenteup);
            }
        }
        delete agenteup.fila;
        AmiEvent.AgenteCadastroUpdate(agenteup);

        reply(agenteup);
    } catch (error) {
        await transaction.rollback();
        reply(Boom.badData(error));
    }
};

exports.getAll = async (request, reply) => {
    try {
        const agentes = await Agente.query
            .leftJoin(AgenteFilas)
            .leftJoin(AutenticacaoFontes)
            .require(false)
            .fetch();

        if (agentes && agentes.length > 0) {
            const grupo_filas = await AgenteGruposFila.query
                .require(false)
                .fetch()

            if (grupo_filas && grupo_filas.length > 0) {
                for (let agente of agentes) {
                    agente.agenteGruposFila = grupo_filas.filter(grupo => grupo.id_agente == agente.id)
                }
            }
        }

        reply(agentes);

    } catch (error) {
        reply(Boom.badData(error));
    }
};

exports.getOne = async (request, reply) => {
    try {
        const agente = await Agente.query
            .leftJoin(AgenteFilas)
            .leftJoin(AutenticacaoFontes)
            .require(true)
            .where({ id: request.params.AgenteID })
            .fetch()

        agente[0].agenteGruposFila = await AgenteGruposFila.query
            .where({ id_agente: request.params.AgenteID })
            .require(false)
            .fetch()

        reply(agente[0])
    } catch (error) {
        reply(Boom.badData(error))
    }
};

exports.delete = async (request, reply) => {
    const transaction = new db.orm.Transaction()
    const AgenteFilasT = transaction.models.AgenteFilas
    const AgenteGruposFilaT = transaction.models.AgenteGruposFila
    const AgenteT = transaction.models.Agente
    try {
        const agenteGrupos = await AgenteGruposFilaT.query
            .require(false)
            .where({ id_agente: request.params.AgenteID })
            .delete()

        const agenteFilas = await AgenteFilasT.query
            .require(false)
            .where({ id_agente: request.params.AgenteID })
            .delete()


        const agente = await AgenteT.query
            .require(true)
            .where({ id: request.params.AgenteID })
            .delete()


        await transaction.commit()
        reply(agente)
    } catch (error) {
        await transaction.rollback()
        reply(Boom.badData(error))
    }
};

exports.logoffall = async (request, reply) => {
    try {
        const ag = await Agente.query
            .leftJoin(AgenteFilas)
            .require(true)
            .where({ id: request.params.AgenteID })
            .fetch();
        const agente = ag[0];


        if (agente) {
            let filas = agente.agenteFilas.map(f => f.fila);
            for (let fila of filas) {
                agente.fila = fila;
                Ami.QueueRemove(agente);
            }
        }

        reply(agente);
    } catch (error) {
        reply(Boom.badData(error));
    }
};

getContatosAgente = async (request) => {
    try {
        const sqlChats = `SELECT json_agg(chats) as chats
        FROM (
          SELECT a.*, json_agg(cliente) AS cliente, json_agg(mensagens) AS mensagens
          FROM "queueChat" a
          LEFT JOIN cliente ON a."idCliente" = cliente.id
          LEFT JOIN (
            SELECT m.*, anexos
            FROM "queueChatMessages" m
            LEFT JOIN (
              SELECT "id_message", json_agg(an) AS anexos
              FROM "queueAnexos" an
              GROUP BY "id_message"
            ) anexos ON m.id = anexos."id_message"
            JOIN "queueChat" a2 ON m."idQueueChat" = a2.id
            WHERE a2."queueTypeID" != 1 AND a2."ultimoAgente" = $1 AND a2."dataIni" >= $2
            ORDER BY m."dataHora" ASC
          ) mensagens ON a.id = mensagens."idQueueChat"
          WHERE a."queueTypeID" != 1 AND a."ultimoAgente" = $1 AND a."dataIni" >= $2
          GROUP BY a.id
          ORDER BY a.id ASC
        ) AS chats;`

        const sqlCalls = `select json_agg(calls) as calls FROM
        (SELECT 
        a.*,a."calldate" as "data_hora_last_msg", 1 as "queueTypeID",
        ( 
            SELECT json_agg(item)
            FROM (
            SELECT b.*
            FROM cliente b where b.id = a."idCliente"
            ) item
        ) AS cliente,
        ( 
            SELECT json_agg(item)
            FROM (
            SELECT b."data",b.obs,b.protocolo,c.*
            FROM "cdr_classificacoes" b 
            INNER JOIN "classificacoes" c on b."id_classificacao" = c.id
            where b."uniqueid" = a."callid"	  
            ) item
        ) AS classificacao
        FROM "queue_calls" a 
        WHERE a."agent" = $1 AND a."calldate" >= $2
         ORDER BY calldate
        ) as calls`

        const today = new Date();
        today.setDate(today.getDate() - 10);
        const yesterday = today.toISOString().substring(0, 10);
        let d = yesterday;
        if (request.query.dataStart) d = new Date(request.query.dataStart)
        console.log('DATE', d)
        const retChats = await db.query(sqlChats, [request.params.AgenteFullName, d])
        const retCall = await db.query(sqlCalls, [request.params.AgenteFullName, d])

        const ret = _.union(retChats.rows[0].chats, retCall.rows[0].calls)
        const ret2 = _.chain(ret)
            .groupBy('idCliente')
            .sortBy('data_hora_last_msg')
            .map((clientes) => {
                let ob = clientes[0].cliente ? clientes[0].cliente[0] : { naocliente: true }
                let atendimentos = clientes.map(at => {
                    const { cliente, ...dados } = at
                    return dados
                })
                if (!ob) ob = {}
                ob.data_hora_last_msg = atendimentos.reduce((a, b) => {
                    if (a.data_hora_last_msg && b.data_hora_last_msg) {
                        return new Date(a.data_hora_last_msg).getTime() > new Date(b.data_hora_last_msg).getTime() ? a : b;
                    } else {
                        return a.data_hora_last_msg ? a : b;
                    }
                }).data_hora_last_msg;

                const allAtendimentos = _.groupBy(atendimentos, 'queueTypeID')
                ob.atendimentos = Object.entries(allAtendimentos).map(tipo => {
                    const at = {}
                    at.tipo = tipo[0]
                    at.atendimentos = tipo[1]
                    return at
                })
                return ob
            })
            .reverse()
            .value()

        return (ret2);
    } catch (error) {
        console.error(error)
        return (Boom.badData(error));
    }
}

getCallsAgente = async (request) => {
    try {
        const sqlCalls = `select json_agg(calls) as calls FROM
        (SELECT 
        a.*,a."calldate" as "data_hora_last_msg", 1 as "queueTypeID",
        ( 
            SELECT json_agg(item)
            FROM (
            SELECT b.*
            FROM cliente b where b.id = a."idCliente"
            ) item
        ) AS cliente,
        ( 
            SELECT json_agg(item)
            FROM (
            SELECT b."data",b.obs,b.protocolo,c.*
            FROM "cdr_classificacoes" b 
            INNER JOIN "classificacoes" c on b."id_classificacao" = c.id
            where b."uniqueid" = a."callid"	  
            ) item
        ) AS classificacao
        FROM "queue_calls" a 
        WHERE a."agent" = $1 AND a."calldate" >= $2
         ORDER BY calldate
        ) as calls`

        const today = new Date();
        today.setDate(today.getDate() - 7);
        const yesterday = today.toISOString().substring(0, 10);
        let d = yesterday;
        if (request.query.dataStart) d = new Date(request.query.dataStart)
        console.log('DATE', d)
        const retCall = await db.query(sqlCalls, [request.params.AgenteFullName, d])
        const ret = retCall.rows[0].calls
        const ret2 = _.chain(ret)
            .groupBy('idCliente')
            .sortBy('data_hora_last_msg')
            .map((clientes) => {
                let ob = clientes[0].cliente ? clientes[0].cliente[0] : { naocliente: true }
                let atendimentos = clientes.map(at => {
                    const { cliente, ...dados } = at
                    return dados
                })
                if (!ob) ob = {}
                ob.data_hora_last_msg = atendimentos.reduce((a, b) => {
                    if (a.data_hora_last_msg && b.data_hora_last_msg) {
                        return new Date(a.data_hora_last_msg).getTime() > new Date(b.data_hora_last_msg).getTime() ? a : b;
                    } else {
                        return a.data_hora_last_msg ? a : b;
                    }
                }).data_hora_last_msg;

                const allAtendimentos = _.groupBy(atendimentos, 'queueTypeID')
                ob.atendimentos = Object.entries(allAtendimentos).map(tipo => {
                    const at = {}
                    at.tipo = tipo[0]
                    at.atendimentos = tipo[1]
                    return at
                })
                return ob
            })
            .reverse()
            .value()

        return (ret2);
    } catch (error) {
        console.error(error)
        return (Boom.badData(error));
    }
}

exports.uploadImage = async (request, reply) => {
    try {
        const dir = '/gravacoes/agentes/'
        const file_path = path.resolve(dir, `${request.params.agenteID.toString()}.jpg`)
        if (!fs.existsSync(path.resolve(dir))) {
            fs.mkdirSync(path.resolve(dir), { recursive: true })
        }
        const file = fs.createWriteStream(file_path)
        file.on('error', function (err) {
            console.error(err);
            fs.unlinkSync(file_path);
            reply(Boom.badData(err));
        });
        request.payload.file.pipe(file);
        request.payload.file.on('end', async function (err) {
            if (err) {
                fs.unlinkSync(file_path);
                reply(Boom.badData(err));
            }
            else {
                let dados = {
                    id: request.params.agenteID,
                    profileImage: file_path
                }
                let ret = await Agente.update(dados);
                reply(ret);
            }
        })
    } catch (error) {
        console.error(error);
        reply(Boom.badData(error));
    }
}

exports.getProfileImage = async (request, reply) => {
    try {
        const agente = await Agente.fetchByFullname(request.params.AgenteFullName);
        if (!agente || !agente.profileImage || !fs.existsSync(agente.profileImage)) {
            return reply(Boom.badData(`Arquivo não encontrado`));
        }
        return reply.file(agente.profileImage, {
            name: path.basename(agente.profileImage),
            confine: false,
        });
    } catch (error) {
        console.log(error);
        reply(Boom.badData(error));
    }
}

exports.getClientesAtendidos = async (request, reply) => {
    try {
        if (request.query.apenasabertos) {
            const sqlChats = `SELECT a."idCliente" as id, SUM(a.qtd_msg_nao_lidas) as qtd_msg_nao_lidas, MAX(a."data_hora_last_msg") as data_hora_last_msg, cliente.numero, cliente.nome, cliente."profileImage", cliente."isBusiness", cliente.documento, cliente.email, a.assunto
            FROM "queueChat" a
            LEFT JOIN cliente ON a."idCliente" = cliente.id
            WHERE a."idCliente" IS NOT NULL AND a."queueTypeID" != 1 AND a."data_hora_last_msg" IS NOT NULL AND a."ultimoAgente" = $1 AND a."status" = 'Entrou'
            GROUP BY a."idCliente", cliente.numero, cliente.nome, cliente."profileImage", cliente."isBusiness", cliente.documento, cliente.email, a.assunto
            ORDER BY MAX(a."data_hora_last_msg") DESC LIMIT 250;`
            const retChats = await db.query(sqlChats, [request.params.AgenteFullName])
            reply(retChats.rows)
        }
        else {
            const sqlChats = `SELECT a."idCliente" as id, SUM(a.qtd_msg_nao_lidas) as qtd_msg_nao_lidas, MAX(a."data_hora_last_msg") as data_hora_last_msg, cliente.numero, cliente.nome, cliente."profileImage", cliente."isBusiness", cliente.documento, cliente.email, a.assunto
            FROM "queueChat" a
            LEFT JOIN cliente ON a."idCliente" = cliente.id
            WHERE a."idCliente" IS NOT NULL AND a."queueTypeID" != 1 AND a."data_hora_last_msg" IS NOT NULL AND a."ultimoAgente" = $1
            GROUP BY a."idCliente", cliente.numero, cliente.nome, cliente."profileImage", cliente."isBusiness", cliente.documento, cliente.email, a.assunto
            ORDER BY MAX(a."data_hora_last_msg") DESC LIMIT 250;`
            const retChats = await db.query(sqlChats, [request.params.AgenteFullName])

            reply(retChats.rows)
        }

    } catch (error) {
        reply(Boom.badData(error));
    }
}
exports.getClienteAtendimentosByProtocolo = async (request, reply) => {
    // Removi os dados de paginação para otimizar os retorno da rota pois o infiniscrool usado no cliente não precisa deles.
    // Para usar em relatório com tabelas paginadas crie uma outra rota.   

    try {
        const knex = db.knex('public.mensagens_omni')
        knex.where('agentes_filas.id_agente', request.params.AgenteID)
        knex.where('idQueueChat', request.params.idQueueChat)
        if (request.query.dataHora) knex.where('dataHora', '>', request.query.dataHora)

        const ret = await knex.select('mensagens_omni.*')
            .innerJoin('agentes_filas', 'mensagens_omni.queue', 'agentes_filas.fila')
            .orderBy('dataHora', 'DESC')
        reply(ret)
    } catch (error) {
        reply(Boom.badData(error))
    }
}
exports.getCallsByCliente = async (request, reply) => {
    try {
        const knex = db.knex('public.mensagens_omni')
        knex.where('agentes_filas.id_agente', request.params.AgenteID)
        knex.where('idCliente', request.params.idCliente)
        knex.where('queueTypeID', 1)
        if (request.query.dataHora) knex.where('dataHora', '>', request.query.dataHora)

        const ret = await knex.select('mensagens_omni.*')
            .innerJoin('agentes_filas', 'mensagens_omni.queue', 'agentes_filas.fila')
            .orderBy('dataHora', 'DESC')
        reply(ret)
    } catch (error) {
        reply(Boom.badData(error))
    }
}

exports.getCallsByAgente = async (request, reply) => {
    try {
        const knex = db.knex('public.mensagens_omni')
        knex.where('agentes_filas.id_agente', request.params.AgenteID)
        knex.where('ultimoAgente', request.params.agente)
        knex.where('queueTypeID', 1)
        if (request.query.dataHora) knex.where('dataHora', '>', request.query.dataHora)

        const ret = await knex.select('mensagens_omni.*')
            .innerJoin('agentes_filas', 'mensagens_omni.queue', 'agentes_filas.fila')
            .orderBy('dataHora', 'DESC')
        reply(ret)
    } catch (error) {
        reply(Boom.badData(error))
    }
}
exports.getClienteAtendimentos = async (request, reply) => {
    // Removi os dados de paginação para otimizar os retorno da rota pois o infiniscrool usado no cliente não precisa deles.
    // Para usar em relatório com tabelas paginadas crie uma outra rota.
    const max = request.query.rowsPerPage ? request.query.rowsPerPage : 100
    const page = request.query.page ? request.query.page : 1
    const offset = max * (page - 1)

    try {
        const knex = db.knex('public.mensagens_omni')
        knex.where('agentes_filas.id_agente', request.params.AgenteID)
        knex.where('idCliente', request.params.ClienteID)
        if (request.query.dataHora) knex.where('dataHora', '>', request.query.dataHora)

        const ret = await knex.select('mensagens_omni.*')
            .innerJoin('agentes_filas', 'mensagens_omni.queue', 'agentes_filas.fila')
            .orderBy('dataHora', 'DESC')
            .limit(max)
            .offset(offset)
        reply(ret)
    } catch (error) {
        reply(Boom.badData(error))
    }
}

exports.getClienteEmails = async (request, reply) => {
    const max = request.query.rowsPerPage ? request.query.rowsPerPage : 0
    const page = request.query.page ? request.query.page : 1
    const offset = max * (page - 1)

    try {
        const knex = db.knex('public.mensagens_omni')
        knex.where('agentes_filas.id_agente', request.params.AgenteID)
        knex.where('idCliente', request.params.ClienteID)
        knex.where('queueTypeID', 4)
        if (request.query.dataHora) knex.where('dataHora', '>', request.query.dataHora)

        const ret = await knex.select('mensagens_omni.*')
            .innerJoin('agentes_filas', 'mensagens_omni.queue', 'agentes_filas.fila')
            .orderBy('dataHora', 'DESC')
            .limit(max)
            .offset(offset)
        reply(ret)
    } catch (error) {
        reply(Boom.badData(error))
    }
}