const Joi = require('joi');
const handlers = require('./handlers');

module.exports = [  
    {
        method: 'GET',
        path: '/api/v4/atendimento_campos_adicionais/{idAtendimento}',
        handler: handlers.getCamposAdicionaisByIdAtendimento,
        config: {
            description: 'Obtem todas as tags',
            tags: ['api', 'atendimento'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate:{
                params: {
                        idAtendimento: Joi.string().required()
                }
            }            
        }
    },
      {
        method: 'POST',
        path: '/api/v4/atendimento_campos_adicionais',
        handler: handlers.addCampoAdicionalAtendimento,
        config: {
            description: 'Add uma tag em um atendimento',
            tags: ['api', 'atendimento'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                payload: Joi.object({
                    id_atendimento: Joi.string().required().description('Id do atendimento que foi tageado'),
                    nome: Joi.string().required().description('Nome do campo adicional'),
                    valor: Joi.string().allow('').allow(null).description('Valor do campo adicional'),
                    agente: Joi.string().required().description('Fullname do agente que adicionou o campo adicional'),
                    privado: Joi.boolean().required().description('Se for privado so o agente que criou podera ver')
                })
            }
        }
    },
     {
        method: 'PUT',
        path: '/api/v4/atendimento_campos_adicionais/{idCampo}',
        handler: handlers.updateCampoAdicionalAtendimento,
        config: {
            description: 'Add uma tag em um atendimento',
            tags: ['api', 'atendimento'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    idCampo: Joi.number().integer().required()
                },
                payload: Joi.object({
                    id_atendimento: Joi.string().required().description('Id do atendimento que foi tageado'),
                    nome: Joi.string().required().description('Nome do campo adicional'),
                    valor: Joi.string().allow('').allow(null).description('Valor do campo adicional'),
                    agente: Joi.string().required().description('Fullname do agente que adicionou o campo adicional'),
                    privado: Joi.boolean().required().description('Se for privado so o agente que criou podera ver')
                })
            }
        }
    },
     {
        method: 'DELETE',
        path: '/api/v4/atendimento_campos_adicionais/{idCampo}',
        handler: handlers.removeCampoAdicionalById,
        config: {
            description: 'Add uma tag em um atendimento',
            tags: ['api', 'atendimento'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    idCampo: Joi.number().integer().required()
                }
            }
        }
    },    
]