const Boom = require('boom');
const AutenticacaoFontes = require('../../../models/AutenticacaoFontes');
const AutenticacaoTipos = require('../../../models/AutenticacaoTipos');
const ldap = require('../../../util/ldap');

exports.create = async (request, reply) => {
    try {
        let fonte = await AutenticacaoFontes.save(request.payload);
        reply(fonte);
    } catch (error) {
        console.error(error);
        reply(Boom.badData(error));
    }

}

exports.update = async (request, reply) => {
    try {
        let dados = request.payload;
        dados.id = request.params.id;
        let fonte = await AutenticacaoFontes.save(dados);
        reply(fonte);
    } catch (error) {
        console.error(error);
        reply(Boom.badData(error));
    }

}

exports.getOne = async (request, reply) => {
    try {
        const query = AutenticacaoFontes.query;
        query.where({ id: request.params.id });
        query.innerJoin(AutenticacaoTipos);
        var registros = await query.require(true).fetch();
        reply(registros[0]);
    } catch (e) {
        reply(Boom.badData(e));
    }
};

exports.getAll = async (request, reply) => {
    try {
        const query = AutenticacaoFontes.query;
        query.innerJoin(AutenticacaoTipos)
        var registros = await query.require(false).fetch();
        reply(registros);
    } catch (e) {
        reply(Boom.badData(e));
    }
};

exports.getTipos = async (request, reply) => {
    try {
        const query = AutenticacaoTipos.query;
        var registros = await query.require(false).fetch();
        reply(registros);
    } catch (e) {
        reply(Boom.badData(e));
    }
};

exports.delete = async (request, reply) => {
    try {
        const query = AutenticacaoFontes.query;
        query.where({ id: request.params.id });
        var registros = await query.require(true).delete();
        reply(registros);
    } catch (e) {
        reply(Boom.badData(e));
    }
};


exports.ldapteste = async (request, reply) => {
    // Seta para aceitar certificado auto assinado no caso de TLS
    const config = request.payload.config;
    config.tlsOptions = { rejectUnauthorized: false };
    try {
        let ret = await ldap.ldapLogin(request.payload.nome, request.payload.senha, config);
        reply(ret);
    } catch (error) {
        reply(Boom.badData(error));
    }
};