'use strict'
const Joi = require('joi');
const handlers = require('./handlers');

module.exports = [
    {
        method: 'POST',
        path: '/api/v4/autenticacao_fonte',
        handler: handlers.create,
        config: {
            description: 'Cria uma fonte de autenticação',
            tags: ['api', 'v4', 'Fontes de autenticação'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                payload: Joi.object({
                    nome: Joi.string().required(),
                    tipo: Joi.number().integer().required(),
                    config: Joi.object().required(),
                    ativo: Joi.boolean().required()
                })
            }
        }
    },
    {
        method: 'POST',
        path: '/api/v4/autenticacao_fonte/ldapteste',
        handler: handlers.ldapteste,
        config: {
            description: 'Testa login de um usuário no LDAP/AD',
            tags: ['api', 'v4', 'Fontes de autenticação'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                payload: Joi.object({
                    nome: Joi.string().required(),
                    senha: Joi.string().required(),
                    config: Joi.object().required(),
                })
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v4/autenticacao_fonte/{id}',
        handler: handlers.update,
        config: {
            description: 'Atualiza uma fonte de autenticação',
            tags: ['api', 'v4', 'Fontes de autenticação'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                },
                payload: Joi.object({
                    nome: Joi.string().optional(),
                    tipo: Joi.number().integer().optional(),
                    config: Joi.object().optional(),
                    ativo: Joi.boolean().optional()
                })
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/autenticacao_fonte/{id}',
        handler: handlers.getOne,
        config: {
            description: 'Obtem uma fonte de autenticação',
            tags: ['api', 'v4', 'Fontes de autenticação'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/autenticacao_fontes',
        handler: handlers.getAll,
        config: {
            description: 'Obtem as fontes de autenticação',
            tags: ['api', 'v4', 'Fontes de autenticação'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/autenticacao_fontes/tipos',
        handler: handlers.getTipos,
        config: {
            description: 'Obtem os tipos de fontes de autenticação',
            tags: ['api', 'v4', 'Fontes de autenticação'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            }
        }
    },
    {
        method: 'DELETE',
        path: '/api/v4/autenticacao_fonte/{id}',
        handler: handlers.delete,
        config: {
            description: 'Apaga uma fonte de autenticação',
            tags: ['api', 'v4', 'Fontes de autenticação'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                }
            }
        }
    },
]