const Joi = require('joi');
const handlers = require('./handlers');

module.exports = [
    {
        method: 'GET',
        path: '/api/v4/autocompletes',
        handler: handlers.getAll,
        config: {
            description: 'Retorna os Autocomplete cadastrados',
            tags: ['api', 'Autocomplete'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.autocomplete']
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/autocomplete/{id}',
        handler: handlers.getOne,
        config: {
            description: 'Retorna o Autocomplete cadastrado',
            tags: ['api', 'Autocomplete'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.autocomplete']
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v4/autocomplete/{id}',
        handler: handlers.update,
        config: {
            description: 'Atualiza um Autocomplete',
            tags: ['api', 'Autocomplete'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.autocomplete']
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                },
                payload: Joi.object({
                    nome: Joi.string().optional().allow([null, '']),
                    fila: Joi.string().optional().allow([null, '']),
                    texto: Joi.string().optional().allow([null, '']),
                })
            }
        }
    },
    {
        method: 'POST',
        path: '/api/v4/autocomplete',
        handler: handlers.create,
        config: {
            description: 'Cria um Autocomplete',
            tags: ['api', 'Autocomplete'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.autocomplete']
            },
            validate: {
                payload: Joi.object({
                    nome: Joi.string().required(),
                    texto: Joi.string().required(),
                    fila: Joi.string().optional().allow([null, '']),
                })
            }
        }
    },
    {
        method: 'DELETE',
        path: '/api/v4/autocomplete/{id}',
        handler: handlers.delete,
        config: {
            description: 'Remove um Autocomplete',
            tags: ['api', 'Autocomplete'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.autocomplete']
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                }
            }
        }
    },
];
