'use strict'

const Paginacao = require('../../../util/Paginacao');
const Boom = require('boom');
const db = require('../../../util/db');
//const squel = require("squel");
const ws = require('../../../ws');
const discador = require('../../../discador');
const config = require('../../../config.json');
const TextoUtil = require("../../../util/texto");
const moment = require('moment-timezone');
require('moment-duration-format');
moment.tz.setDefault(config.servidor.timezone);
const csv = require("fast-csv");
const uuid = require('uuid/v1');
const uuid2 = require('uuid');
const fs = require("fs");
const MimeType = require("mime");
const _ = require('lodash');

const handlersAgentesV2 = require('../../agentes/handlers');
const handlersCampanhasV2 = require('../../campanhas/handlers');
const handlersFilasV2 = require('../../filas/handlers');
const handlersChatsV2 = require('../../chat/handlers');

const Campanha = require("../../../models/Campanha");
const CampanhaContato = require("../../../models/CampanhaContato");
const CampanhaContatoNumeros = require("../../../models/CampanhaContatoNumeros");
const CampanhaContatoCampo = require("../../../models/CampanhaContatoCampo");

const log = require('simple-node-logger').createSimpleLogger({
    logFilePath: '/var/log/xcontact.log',
    timestampFormat: 'DD-MM-YYYY HH:mm:ss.SSS'
});
const log_level = config.log ? config.log : 'error';
log.setLevel(log_level);

function removeDiscador (filasArr) {
    if (filasArr && filasArr.length > 0) {
        let fila = filasArr.filter(f => !f.match('DISCADOR'))
        return fila
    }
}

const getPercentual = (feito, total) => {
    if (feito == total) return 100;
    if (!feito || feito == 0) return 0;
    let perc = (feito * 100) / total;
    return perc < 100 ? perc.toFixed(2) : 100;
}

function getDataHoraMedia (datas_array) {
    let milsec = datas_array.map(l => { return l ? l.getTime() : 0 }).reduce((a, b) => a + b, 0);
    let tan = datas_array.map(l => { return l ? 1 : 0 }).reduce((a, b) => a + b, 0);
    return new Date(milsec / tan);
}

function diffDateInSeconds (inicio, fim) {
    let a = moment(inicio);
    let b = moment(fim);
    return b.diff(a, 'seconds');
}

function padTime (time) {
    time = parseInt(time);
    return (time < 10 ? "0" : "") + time;
}

function SecToTime (seconds) {
    if (!seconds) return '00:00:00';
    let duration = moment.duration(seconds, 'seconds');
    return duration.format('h:mm:ss', { trim: false, useGrouping: false })
};

function TimeTosec (time) {
    if (!time) return 0;
    if (typeof time == 'object') time = `${time.getHours()}:${time.getMinutes()}:${time.getSeconds()}`
    var tt = time.split(":");
    return parseInt(tt[0]) * 3600 + parseInt(tt[1]) * 60 + parseInt(tt[2]) * 1;
};

function MilSecToTime (mili) {
    if (!mili) return '00:00:00';
    var date = moment().toDate();
    date.setMilliseconds(mili);
    return padTime(date.getHours()) + ":" + padTime(date.getMinutes()) + ":" + padTime(date.getSeconds());
};

function getTimeInEvent (eventos) {
    if (eventos.length == 0) return;

    let timeline = JSON.parse(JSON.stringify(eventos[0]));
    timeline.event_start = moment(timeline.event_start).toDate();
    timeline.event_end = timeline.event_end ? moment(timeline.event_end).toDate() : atual;
    timeline.tempo = diffDateInSeconds(timeline.event_start, timeline.event_end);

    for (let i = 1; i < eventos.length; i++) {
        if (eventos[i].event_start > timeline.event_end) {
            //console.log(`Caiu no if pro agente ${timeline.agent} com ${MilSecToTime(timeline.tempo)}`);
            timeline.tempo += diffDateInSeconds(eventos[i].event_start, eventos[i].event_end);
            timeline.event_end = eventos[i].event_end;
        }
        else if (eventos[i].event_end > timeline.event_end) {
            //console.log(`Caiu no else pro agente ${timeline.agent}, evento ${eventos[i].event_start} até ${eventos[i].event_end}, timeline ${timeline.event_start} até ${timeline.event_end}`);
            timeline.tempo += diffDateInSeconds(timeline.event_end, eventos[i].event_end);
            timeline.event_end = eventos[i].event_end;
        }
    }
    return timeline.tempo;
}

function filtraLigacoes (lig) {
    const classificacoesEfetivas = []
    if (classificacoesEfetivas && classificacoesEfetivas.length > 0) {
        let nomesClassificacao = classificacoesEfetivas.map(c => c.nome)
        let r = lig.filter(l => nomesClassificacao.includes(l.classificacao))
        return r
    } else { return lig }
}

function getAgentesDasFilas (agentes) {
    return agentes.forEach(ag => {
        ag.filas_nome = ag.filas.map(f => f.fila);
    });
}

exports.getTemposEMovimentos = async (request, reply) => {
    try {
        const atual = moment().toDate();
        const hojeInicio = moment().startOf('day').format('YYYY-MM-DDTHH:mm:ss') + 'Z';
        const hojeFim = moment().endOf('day').format('YYYY-MM-DDTHH:mm:ss') + 'Z';
        const data_ini = request.query.data_ini ? request.query.data_ini : hojeInicio;
        const data_fim = request.query.data_fim ? request.query.data_fim : hojeFim;
        const todas_pausas_nr17 = ["ALMOCO", "LANCHE_MANHA", "LANCHE_TARDE"];

        const getagentes = await handlersAgentesV2.returnAll({ max: -1 });
        let agentes = getAgentesDasFilas(getagentes.dados);
        agentes = _.sortBy(agentes, 'nome');

        const geteventos = await handlersAgentesV2.returnEvents({
            query: {
                max: -1,
                data_ini,
                data_fim,
            }
        })
        const eventos = geteventos.dados.map(item => {
            item.event_start = moment(item.event_start).toDate();
            item.event_end = item.event_end ? moment(item.event_end).toDate() : atual;
            return item;
        });


        const getcampanhareg = await handlersCampanhasV2.returnCampanhasRegistros({
            max: -1,
            data_ini,
            data_fim,
        })
        const campanhareg = getcampanhareg.dados.map(item => {
            item.data_status = moment(item.data_status).toDate();
            item.duracao_sec = TimeTosec(item.duracao);
            return item;
        });

        const getcampanhas = await handlersCampanhasV2.returnAll({
            max: -1,
            data_agendamento_ini: data_ini,
            data_agendamento_fim: data_fim,
        })
        const campanhas = getcampanhas.dados;

        const getfilaligacoes = await handlersFilasV2.returnLigacoes({
            max: -1,
            data_ini,
            data_fim,
            status: ["Atendida"]
        })
        const filaligacoes = getfilaligacoes.dados.map(item => {
            item.calldate = moment(item.calldate).toDate();
            item.duracao_sec = TimeTosec(item.duracao);
            return item;
        });

        const getChats = await handlersChatsV2.returnAllChats({
            max: -1,
            dataIni: data_ini,
            dataFim: data_fim,
            status: ["Entrou", "Pesquisa", "Encerrado"]
        })
        const chats = getChats.dados.map(item => {
            item.dataIni = moment(item.dataIni).toDate();
            item.dataFim = item.dataFim ? moment(item.dataFim).toDate() : atual;
            item.duracao = MilSecToTime(item.dataFim - item.dataIni);
            item.duracao_sec = TimeTosec(item.duracao);
            return item;
        });

        let registros = [];
        let totais = {
            qtd_pausa: 0,
            total_logado: 0,
            total_pausa: 0,
            jornada_trabalho: 0,
            total_agentes: 0,
            total_agentes_discador: 0,
            total_agentes_fora: 0,
            total_agentes_outras_pausas: 0,
            total_logado_discador: 0,
            total_pausa_discador: 0,
            total_ligacoes: 0,
            total_tempo_ligacoes: 0,
            total_tempo_ligacoes_fora: 0,
            jornada_trabalho_discador: 0,
            presenca_discador: 0,
            meta_mailing: 0,
            real_mailing: 0,
            total_ligacoes_efetivas: 0,
            aderencia_discador: 0,
            tempo_medio_falado: 0,
            total_ligacoes_fora: 0,
            total_ligacoes_efetivas_fora: 0,
            presenca_fora: 0,
            total_logado_fora: 0,
            total_pausa_fora: 0,
            tempo_medio_falado_fora: 0,
            jornada_fora: 0,
            total_chats: 0,
            total_agentes_chats: 0,
            total_chats_entrantes: 0,
            total_chats_entrantes_encerrados: 0,
            total_chats_entrantes_abertos: 0,
            total_chats_saintes: 0,
            total_chats_saintes_encerrados: 0,
            total_chats_saintes_abertos: 0,
            perc_chats_entrantes_encerrados: 0,
            perc_chats_saintes_encerrados: 0,
            total_pausas_nr17: [],
            total_outras_pausas: 0,
            total_primeiro_login: 0,
            total_ultimo_logoff: 0,
            media_outras_pausas () {
                return SecToTime(this.total_outras_pausas / this.total_agentes_outras_pausas);
            },
            media_primeiro_login () {
                return SecToTime(this.total_primeiro_login / this.total_agentes);
            },
            media_ultimo_logoff () {
                return SecToTime(this.total_ultimo_logoff / this.total_agentes);
            },
            total_primeira_ligacao: 0,
            total_ultima_ligacao: 0,
            media_primeira_ligacao () {
                return SecToTime(this.total_primeira_ligacao / this.total_agentes_discador);
            },
            media_ultima_ligacao () {
                return SecToTime(this.total_ultima_ligacao / this.total_agentes_discador);
            },
            total_primeira_ligacao_fora: 0,
            total_ultima_ligacao_fora: 0,
            media_primeira_ligacao_fora () {
                return SecToTime(this.total_primeira_ligacao_fora / this.total_agentes_fora);
            },
            media_ultima_ligacao_fora () {
                return SecToTime(this.total_ultima_ligacao_fora / this.total_agentes_fora);
            },
        };

        let ordenado = _.chain(eventos)
            .sortBy('event_start')
            .groupBy('agent')
            .map(function (value1) {
                let ob = {};
                ob.login = Object.assign([], value1.filter(item => item.event == 'Login'));
                if (!ob.login || ob.login.length <= 0) return;
                ob.agente = value1[0].agent;
                ob.agenteDados = agentes.filter(a => a.nome == ob.agente);
                ob.login_geral = Object.assign([], value1.filter(item => item.event == 'Login'));
                ob.pausas = Object.assign([], value1.filter(item => item.event == 'Pause' && item.event_data1 != 'Classificando'));
                ob.pausas_geral = Object.assign([], value1.filter(item => item.event == 'Pause' && item.event_data1 != 'Classificando'));

                totais.total_agentes++;

                ob.pausas_nr17 = Object.assign([], ob.pausas.filter(item => todas_pausas_nr17.includes(item.event_data1)));
                ob.outras_pausas = Object.assign([], ob.pausas.filter(item => !todas_pausas_nr17.includes(item.event_data1)));

                ob.pausas_nr17_group = _.chain(ob.pausas_nr17).groupBy('event_data1').map(p => {
                    return {
                        nome: p[0].event_data1,
                        tempo: getTimeInEvent(p)
                    };
                }).value();

                let pausas_nr17 = 0;
                ob.pausas_desgroup = [];
                for (let i = 0; i < ob.pausas_nr17_group.length; i++) {
                    if (!totais.total_pausas_nr17[ob.pausas_nr17_group[i].nome]) {
                        totais.total_pausas_nr17[ob.pausas_nr17_group[i].nome] = {
                            tempo: 0,
                            total: 0,
                            media () {
                                return SecToTime(this.tempo / this.total);
                            }
                        };
                    }
                    totais.total_pausas_nr17[ob.pausas_nr17_group[i].nome].total++;
                    totais.total_pausas_nr17[ob.pausas_nr17_group[i].nome].tempo += ob.pausas_nr17_group[i].tempo;
                    ob.pausas_desgroup[ob.pausas_nr17_group[i].nome] = SecToTime(ob.pausas_nr17_group[i].tempo);
                    pausas_nr17 += ob.pausas_nr17_group[i].tempo;
                }

                ob.pausas_nr17_ob = Object.assign({}, ob.pausas_desgroup);



                let outras_pausas = (ob.outras_pausas && ob.outras_pausas.length > 0) ? getTimeInEvent(ob.outras_pausas) : 0;
                ob.total_outras_pausas = SecToTime(outras_pausas);
                if (outras_pausas > 0) totais.total_agentes_outras_pausas++;
                totais.total_outras_pausas += outras_pausas;

                if (ob.login && ob.login.length > 0) {
                    ob.primeiro_login_sf = ob.login.reduce((a, b) => (a.event_start < b.event_start ? a : b)).event_start;
                    ob.primeiro_login = moment(ob.primeiro_login_sf).format('HH:mm:ss')
                    totais.total_primeiro_login += TimeTosec(ob.primeiro_login_sf);

                    ob.ultimo_logoff_sf = ob.login.reduce((a, b) => ((a.event_end ? a.event_end : atual) > (b.event_end ? b.event_end : atual) ? a : b)).event_end;
                    ob.ultimo_logoff = ob.ultimo_logoff_sf && ob.ultimo_logoff_sf != atual ? moment(ob.ultimo_logoff_sf).format('HH:mm:ss') : '';
                    totais.total_ultimo_logoff += TimeTosec(ob.ultimo_logoff);
                }

                let logado = getTimeInEvent(ob.login_geral);
                ob.total_logado = SecToTime(logado);
                totais.total_logado += logado;

                let pausado = outras_pausas + pausas_nr17;
                ob.total_pausa = SecToTime(pausado);
                ob.qtd_pausa = ob.pausas.length;
                totais.total_pausa += pausado;
                totais.qtd_pausa += ob.qtd_pausa;

                ob.jornada_trabalho = pausado ? SecToTime(logado - pausado) : SecToTime(logado);
                totais.jornada_trabalho += (logado - pausado);

                ob.filas = []
                if (ob.agenteDados && ob.agenteDados.length > 0) {
                    ob.filas = ob.agenteDados[0].filas_nome
                }

                // Discador
                ob.ligacoes_efetivas = 0;
                ob.real_mailing = 0;

                if (campanhas) {
                    ob.campanhas = campanhas.filter(c => ob.filas.includes(c.fila));
                    if (ob.campanhas) {
                        ob.campanhas_nome = [...new Set(ob.campanhas.map(_.property("nome")))];
                        ob.filas_discador = [...new Set(ob.campanhas.map(_.property("fila")))];
                        ob.meta_mailing = ob.campanhas.map(l => { return parseInt(l.total_contatos) }).reduce((a, b) => a + b, 0);
                        totais.meta_mailing += ob.meta_mailing;

                        if (campanhareg && campanhareg.length > 0) {
                            ob.campanha_registros = campanhareg.filter(c => ob.campanhas_nome.includes(c.campanha));
                            ob.ligacoes_completadas = ob.campanha_registros.filter(item => item.status == 'Completou');
                            ob.ligacoes_efetivas = filtraLigacoes(ob.ligacoes_completadas);
                            ob.total_ligacoes_efetivas = ob.ligacoes_efetivas.length;
                            totais.total_ligacoes_efetivas += ob.total_ligacoes_efetivas;

                            ob.real_mailing = ob.campanhas.map(l => { return parseInt(l.total_sucessos) }).reduce((a, b) => a + b, 0);
                            totais.real_mailing += ob.real_mailing;

                            ob.aderencia_discador = ob.meta_mailing ? ((ob.real_mailing * 100) / ob.meta_mailing).toFixed(2) : 0;
                            totais.aderencia_discador += parseFloat(ob.aderencia_discador);

                            ob.primeira_ligacao = ob.campanha_registros.reduce((a, b) => (a.data_status && a.data_status < b.data_status ? a : b), 0).data_status;
                            ob.primeira_ligacao = ob.primeira_ligacao ? moment(ob.primeira_ligacao).format('HH:mm:ss') : '';
                            totais.total_primeira_ligacao += TimeTosec(ob.primeira_ligacao);

                            ob.ultima_ligacao = ob.campanha_registros.reduce((a, b) => (a.data_status && a.data_status > b.data_status ? a : b), 0).data_status;
                            ob.ultima_ligacao = ob.ultima_ligacao ? moment(ob.ultima_ligacao).format('HH:mm:ss') : '';
                            totais.total_ultima_ligacao += TimeTosec(ob.ultima_ligacao);


                            ob.tempo_ligacoes = ob.ligacoes_efetivas.map(l => { return parseInt(l.duracao_sec) }).reduce((a, b) => a + b, 0);
                            ob.tempo_medio_falado = ob.ligacoes_efetivas.length > 0 ? SecToTime(ob.tempo_ligacoes / ob.ligacoes_efetivas.length) : 0;
                            totais.total_ligacoes += ob.ligacoes_completadas.length;
                            totais.total_tempo_ligacoes += ob.tempo_ligacoes;

                            ob.logins_discador = ob.login.filter(item => ob.filas_discador.includes(item.queue));
                            ob.pausas_discador = ob.pausas.filter(item => ob.filas_discador.includes(item.queue));

                            let logado_discador = ob.logins_discador.length > 0 ? getTimeInEvent(ob.logins_discador) : 0;
                            ob.total_logado_discador = SecToTime(logado_discador);
                            totais.total_logado_discador += logado_discador;

                            if (logado_discador) totais.total_agentes_discador++;

                            let pausado_discador = ob.pausas_discador.length > 0 ? getTimeInEvent(ob.pausas_discador) : 0;
                            ob.total_pausa_discador = SecToTime(pausado_discador);
                            totais.total_pausa_discador += pausado_discador;

                            ob.jornada_trabalho_discador = logado_discador > 0 ? SecToTime(logado_discador - pausado_discador) : 0;
                            totais.jornada_trabalho_discador += (logado_discador - pausado_discador);

                            ob.presenca_discador = logado_discador ? getPercentual((logado_discador - pausado_discador), (logado - pausado)) : 0;
                        }
                    }
                }

                // Nao discador
                ob.total_ligacoes_fora = 0;
                ob.total_ligacoes_efetivas_fora = 0;
                ob.presenca_fora = 0;
                ob.total_logado_fora = 0;
                ob.total_pausa_fora = 0;
                ob.tempo_medio_falado_fora = 0;
                ob.jornada_fora = 0;

                ob.filas_fora = removeDiscador(ob.filas)

                if (ob.filas_fora && ob.filas_fora.length > 0) {
                    ob.ligacoes_fora_discador = filaligacoes.filter(e => e.agente == ob.agente && !ob.filas_fora.includes(e.fila));
                    ob.total_ligacoes_fora = ob.ligacoes_fora_discador.length;
                    ob.ligacoes_efetivas_fora = filtraLigacoes(ob.ligacoes_fora_discador);
                    ob.total_ligacoes_efetivas_fora = ob.ligacoes_efetivas_fora.length;
                    totais.total_ligacoes_efetivas_fora += ob.total_ligacoes_efetivas_fora;
                    totais.total_ligacoes_fora += ob.total_ligacoes_fora;

                    if (ob.ligacoes_fora_discador && ob.ligacoes_fora_discador.length > 0) {
                        totais.total_agentes_fora++;

                        ob.primeira_ligacao_fora = ob.ligacoes_fora_discador.reduce((a, b) => (a.calldate && a.calldate < b.calldate ? a : b)).calldate;
                        ob.primeira_ligacao_fora = ob.primeira_ligacao_fora ? moment(ob.primeira_ligacao_fora).format('HH:mm:ss') : '';
                        totais.total_primeira_ligacao_fora += TimeTosec(ob.primeira_ligacao_fora);

                        ob.ultima_ligacao_fora = ob.ligacoes_fora_discador.reduce((a, b) => (a.calldate && a.calldate > b.calldate ? a : b)).calldate;
                        ob.ultima_ligacao_fora = ob.ultima_ligacao_fora ? moment(ob.ultima_ligacao_fora).format('HH:mm:ss') : '';
                        totais.total_ultima_ligacao_fora += TimeTosec(ob.ultima_ligacao_fora);

                        ob.logins_fora = ob.login.filter(item => !ob.filas_fora.includes(item.queue));
                        ob.pausas_fora = ob.pausas.filter(item => !ob.filas_fora.includes(item.queue));

                        let logado_fora = ob.logins_fora.length > 0 ? getTimeInEvent(ob.logins_fora) : 0;
                        ob.total_logado_fora = SecToTime(logado_fora);
                        totais.total_logado_fora += logado_fora;

                        let pausado_fora = ob.pausas_fora.length > 0 ? getTimeInEvent(ob.pausas_fora) : 0;
                        ob.total_pausa_fora = SecToTime(pausado_fora);
                        totais.total_pausa_fora += pausado_fora;

                        ob.jornada_fora = logado_fora > 0 ? SecToTime(logado_fora - pausado_fora) : 0;
                        totais.jornada_fora += logado_fora > 0 ? (logado_fora - pausado_fora) : 0;

                        ob.tempo_ligacoes_fora = ob.ligacoes_efetivas_fora.map(l => { return parseInt(l.duracao_sec) }).reduce((a, b) => a + b, 0);
                        totais.total_tempo_ligacoes_fora += ob.tempo_ligacoes_fora;
                        ob.tempo_medio_falado_fora = ob.ligacoes_efetivas_fora.length > 0 ? SecToTime(ob.tempo_ligacoes_fora / ob.ligacoes_efetivas_fora.length) : 0;
                        ob.presenca_fora = logado_fora ? ((logado_fora - pausado_fora) * 100) / (logado - pausado) : 0;
                        totais.presenca_fora += parseFloat(ob.presenca_fora);
                        ob.presenca_fora = ob.presenca_fora.toFixed(2);
                    }
                }

                // WhatsApp
                ob.total_chats = 0;
                ob.total_chats_entrantes = 0;
                ob.total_chats_entrantes_encerrados = 0;
                ob.total_chats_entrantes_abertos = 0;

                ob.total_chats_saintes = 0;
                ob.total_chats_saintes_encerrados = 0;
                ob.total_chats_saintes_abertos = 0;

                ob.perc_chats_entrantes_encerrados = 0;
                ob.perc_chats_saintes_encerrados = 0;

                ob.chats = chats.filter(e => e.ultimoAgente == ob.agente);
                if (ob.chats && ob.chats.length > 0) {
                    ob.total_chats = ob.chats.length;
                    if (ob.total_chats > 0) totais.total_agentes_chats++;

                    totais.total_chats += ob.total_chats;

                    ob.chats_entrantes = ob.chats.filter(item => item.sentido == 'Entrante');
                    ob.chats_saintes = ob.chats.filter(item => item.sentido == 'Sainte');

                    ob.chats_entrantes_encerrados = (ob.chats_entrantes && ob.chats_entrantes.length > 0) ? ob.chats_entrantes.filter(item => item.status == 'Encerrado') : [];
                    ob.chats_saintes_encerrados = (ob.chats_saintes && ob.chats_saintes.length > 0) ? ob.chats_saintes.filter(item => item.status == 'Encerrado') : [];

                    ob.total_chats_entrantes = ob.chats_entrantes.length;
                    totais.total_chats_entrantes += ob.total_chats_entrantes;

                    ob.total_chats_entrantes_encerrados = ob.chats_entrantes_encerrados.length;
                    totais.total_chats_entrantes_encerrados += ob.total_chats_entrantes_encerrados;

                    ob.total_chats_entrantes_abertos = ob.total_chats_entrantes > 0 ? ob.total_chats_entrantes - ob.total_chats_entrantes_encerrados : 0;
                    totais.total_chats_entrantes_abertos += ob.total_chats_entrantes_abertos;

                    ob.perc_chats_entrantes_encerrados = ob.total_chats_entrantes_encerrados > 0 ? (ob.total_chats_entrantes_encerrados * 100) / ob.total_chats_entrantes : 0;
                    totais.perc_chats_entrantes_encerrados += ob.perc_chats_entrantes_encerrados;
                    ob.perc_chats_entrantes_encerrados = ob.perc_chats_entrantes_encerrados.toFixed(2);

                    ob.total_chats_saintes = ob.chats_saintes.length;
                    totais.total_chats_saintes += ob.total_chats_saintes;

                    ob.total_chats_saintes_encerrados = ob.chats_saintes_encerrados.length;
                    totais.total_chats_saintes_encerrados += ob.total_chats_saintes_encerrados;

                    ob.total_chats_saintes_abertos = ob.total_chats_saintes > 0 ? ob.total_chats_saintes - ob.total_chats_saintes_encerrados : 0;
                    totais.total_chats_saintes_abertos += ob.total_chats_saintes_abertos;

                    ob.perc_chats_saintes_encerrados = ob.total_chats_saintes_encerrados > 0 ? (ob.total_chats_saintes_encerrados * 100) / ob.total_chats_saintes : 0;
                    totais.perc_chats_saintes_encerrados += ob.perc_chats_saintes_encerrados;
                    ob.perc_chats_saintes_encerrados = ob.perc_chats_saintes_encerrados.toFixed(2);
                }

                return ob;
            })
            .value();

        // Calcula totais
        totais.total_logado_f = SecToTime(totais.total_logado / totais.total_agentes);
        totais.total_pausa_f = SecToTime(totais.total_pausa / totais.total_agentes);
        totais.jornada_trabalho = SecToTime(totais.jornada_trabalho / totais.total_agentes);
        totais.total_logado_discador_f = SecToTime(totais.total_logado_discador / totais.total_agentes_discador);
        totais.total_pausa_discador_f = SecToTime(totais.total_pausa_discador / totais.total_agentes_discador);
        totais.jornada_trabalho_discador_f = SecToTime(totais.jornada_trabalho_discador / totais.total_agentes_discador);
        totais.presenca_discador_f = getPercentual((totais.total_logado_discador - totais.total_pausa_discador), (totais.total_logado - totais.total_pausa));
        totais.tempo_medio_falado = SecToTime(totais.total_tempo_ligacoes / totais.total_ligacoes_efetivas);
        totais.total_logado_fora_f = SecToTime(totais.total_logado_fora / totais.total_agentes_fora);
        totais.total_pausa_fora_f = SecToTime(totais.total_pausa_fora / totais.total_agentes_fora);
        totais.jornada_fora = SecToTime(totais.jornada_fora / totais.total_agentes_fora);
        totais.presenca_fora = (totais.presenca_fora / totais.total_agentes_fora).toFixed(2);
        totais.tempo_medio_falado_fora = SecToTime(totais.total_tempo_ligacoes_fora / totais.total_ligacoes_efetivas_fora);
        totais.perc_chats_entrantes_encerrados = (totais.perc_chats_entrantes_encerrados / totais.total_agentes_chats).toFixed(2);

        totais.perc_chats_saintes_encerrados = (totais.perc_chats_saintes_encerrados / totais.total_agentes_chats).toFixed(2);
        registros = ordenado.map(reg => ({
            agente: reg.agente,
            primeiro_login: reg.primeiro_login,
            ultimo_logoff: reg.ultimo_logoff,
            total_logado: reg.total_logado,
            qtd_pausa: reg.qtd_pausa,
            total_pausa: reg.total_pausa,
            jornada_trabalho: reg.jornada_trabalho,
            meta_mailing: reg.meta_mailing,
            real_mailing: reg.real_mailing,
            total_ligacoes_efetivas: reg.total_ligacoes_efetivas,
            aderencia_discador: reg.aderencia_discador,
            primeira_ligacao: reg.primeira_ligacao,
            ultima_ligacao: reg.ultima_ligacao,
            total_logado_discador: reg.total_logado_discador,
            total_pausa_discador: reg.total_pausa_discador,
            jornada_trabalho_discador: reg.jornada_trabalho_discador,
            presenca_discador: reg.presenca_discador,
            tempo_medio_falado: reg.tempo_medio_falado,
            total_ligacoes_fora: reg.total_ligacoes_fora,
            total_ligacoes_efetivas_fora: reg.total_ligacoes_efetivas_fora,
            primeira_ligacao_fora: reg.primeira_ligacao_fora,
            ultima_ligacao_fora: reg.ultima_ligacao_fora,
            total_logado_fora: reg.total_logado_fora,
            total_pausa_fora: reg.total_pausa_fora,
            jornada_fora: reg.jornada_fora,
            presenca_fora: reg.presenca_fora,
            tempo_medio_falado_fora: reg.tempo_medio_falado_fora,
            total_chats: reg.total_chats,
            total_chats_entrantes: reg.total_chats_entrantes,
            total_chats_entrantes_encerrados: reg.total_chats_entrantes_encerrados,
            total_chats_entrantes_abertos: reg.total_chats_entrantes_abertos,
            perc_chats_entrantes_encerrados: reg.perc_chats_entrantes_encerrados,
            total_chats_saintes: reg.total_chats_saintes,
            total_chats_saintes_encerrados: reg.total_chats_saintes_encerrados,
            total_chats_saintes_abertos: reg.total_chats_saintes_abertos,
            perc_chats_saintes_encerrados: reg.perc_chats_saintes_encerrados,
            pausas_nr17_ob: reg.pausas_nr17_ob,
            total_outras_pausas: reg.total_outras_pausas
        }));

        reply({ registros, totais });
    } catch (error) {
        console.log(error)
        reply(Boom.badData(error));
    }
}


exports.getCampanhasRegistrosTags = async (request, reply) => {
    try {
        const max = request.query.max ? request.query.max : 15;
        const page = request.query.page ? request.query.page : 1;
        const knex = db.knex.select('campanhas_registro.*',
            'status_ligacao.nome as status',
            'campanhas.nome as campanha',
            'campanhas.fila as fila',);

        if (request.query.campanhas) knex.whereIn('campanhas_registro.id_campanha', request.query.campanhas);
        if (request.query.status && request.query.status.length > 0) knex.whereIn('campanhas_registro.id_status', request.query.status);
        if (request.query.data_ini) knex.where('campanhas_registro.data_status', '>=', request.query.data_ini);
        if (request.query.data_fim) knex.where('campanhas_registro.data_status', '<=', request.query.data_fim);
        if (request.query.numero_operador && request.query.numero) {
            switch (request.query.numero_operador) {
                case 1:
                    knex.where('campanhas_registro.numero', '=', request.query.numero);
                    break;
                case 2:
                    knex.where('campanhas_registro.numero', 'like', request.query.numero + '%');
                    break;
                case 3:
                    knex.where('campanhas_registro.numero', 'like', '%' + request.query.numero);
                    break;
                case 4:
                    knex.where('campanhas_registro.numero', 'not like', '%' + request.query.numero + '%');
                    break;
                case 5:
                    knex.where('campanhas_registro.numero', 'not like', request.query.numero);
                    break;
            }
        }
        else if (request.query.numero) {
            knex.where('campanhas_registro.numero', 'like', '%' + request.query.numero + '%');
        }

        if (max === -1) {
            // Parametros de ordenação, fora do count
            if (request.query.order) {
                for (let p = 0; p < request.query.order.length; ++p) {
                    if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                        knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
                    }
                    else if (request.query.order[p] && request.query.order[p].campo) {
                        knex.orderBy(request.query.order[p].campo, 'ASC');
                    }
                }
            }

            const rows = await knex.from('campanhas_registro')
                .leftJoin('status_ligacao', 'campanhas_registro.id_status', 'status_ligacao.id')
                .leftJoin('campanhas', 'campanhas_registro.id_campanha', 'campanhas.id')

            const callids = rows.map((row) => {
                if (row.callid) return row.callid
                if (row.actionid) return row.actionid
                return
            })

            const adicionais = await db.knex.select('*')
                .whereIn('id_atendimento', callids)
                .from('atendimento_campos_adicionais')

            const tags = await db.knex.select('tags_atendimentos.*',
                'tags.nome as nome_tag')
                .whereIn('id_atendimento', callids)
                .from('tags_atendimentos')
                .leftJoin('tags', 'tags_atendimentos.id_tag', 'tags.id')

            for (const row of rows) {
                row.tags = tags.filter((tag) => tag.id_atendimento === row.callid || tag.id_atendimento === row.actionid)
                row.adicionais = adicionais.filter((ad) => ad.id_atendimento === row.callid || ad.id_atendimento === row.actionid)
                console.log(row.callid, row.tags)
            }

            const ret = {};
            ret.error = false;
            ret.full = true;
            ret.total = rows.length;
            ret.message = 'Registros das campanhas';
            ret.dados = rows;

            reply(ret);
        }
        else {
            const offset = max * (page - 1);
            knex.clearSelect();
            knex.count('* as total')
                .from('campanhas_registro')
                .then(async function (total) {
                    if (total[0].total > 0) {
                        knex.clearSelect();
                        // Parametros de ordenação, fora do count
                        if (request.query.order) {
                            for (let p = 0; p < request.query.order.length; ++p) {
                                if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                                    knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
                                }
                                else if (request.query.order[p] && request.query.order[p].campo) {
                                    knex.orderBy(request.query.order[p].campo, 'ASC');
                                }
                            }
                        }
                        const rows = await knex.select('campanhas_registro.*', 'status_ligacao.nome as status', 'campanhas.nome as campanha', 'campanhas.fila as fila', 'classificacoes.nome as classificacao', 'cdr_classificacoes.obs as observacao')
                            .from('campanhas_registro')
                            .leftJoin('status_ligacao', 'campanhas_registro.id_status', 'status_ligacao.id')
                            .leftJoin('campanhas', 'campanhas_registro.id_campanha', 'campanhas.id')
                            .leftJoin('cdr_classificacoes', function () {
                                this.on('campanhas_registro.callid', '=', 'cdr_classificacoes.uniqueid').orOn('campanhas_registro.actionid', '=', 'cdr_classificacoes.uniqueid')
                            })
                            .leftJoin('classificacoes', 'cdr_classificacoes.id_classificacao', 'classificacoes.id')
                            .limit(max)
                            .offset(offset)

                        const callids = rows.map((row) => {
                            if (row.callid) return row.callid
                            if (row.actionid) return row.actionid
                            return
                        })

                        const adicionais = await db.knex.select('*')
                            .whereIn('id_atendimento', callids)
                            .from('atendimento_campos_adicionais')

                        const tags = await db.knex.select('tags_atendimentos.*',
                            'tags.nome as nome_tag')
                            .whereIn('id_atendimento', callids)
                            .from('tags_atendimentos')
                            .leftJoin('tags', 'tags_atendimentos.id_tag', 'tags.id')

                        for (const row of rows) {
                            row.tags = tags.filter((tag) => tag.id_atendimento === row.callid || tag.id_atendimento === row.actionid)
                            row.adicionais = adicionais.filter((ad) => ad.id_atendimento === row.callid || ad.id_atendimento === row.actionid)
                            console.log(row.callid, row.tags)
                        }

                        const pages = Math.ceil(parseInt(total[0].total) / max);
                        const ret = {};
                        ret.error = false;
                        ret.full = pages === 1 ? true : false;
                        ret.page = page;
                        ret.max = max;
                        ret.offset = offset;
                        ret.total = parseInt(total[0].total);
                        ret.pages = pages;
                        ret.message = 'Lista de campanhas paginada';
                        ret.dados = rows;
                        reply(ret);

                    }
                    else {
                        const ret = {};
                        ret.error = true;
                        ret.total = 0;
                        ret.message = 'Nenhum registro encontrado';
                        reply(ret);
                    }
                })
                .catch(function (error) {
                    console.error(error);
                    reply(Boom.badData(error));
                });
        }
    } catch (error) {
        console.error(error);
        reply(Boom.badData(error))
    }
};
exports.delete = async (request, reply) => {

    const transaction = new db.orm.Transaction();
    const CampanhaT = transaction.models.Campanha;
    const CampanhaContatoT = transaction.models.CampanhaContato;
    const CampanhaContatoNumerosT = transaction.models.CampanhaContatoNumeros;
    const CampanhaContatoCampoT = transaction.models.CampanhaContatoCampo;

    try {
        await CampanhaContatoNumerosT.query
            .require(false)
            .where(CampanhaContatoNumeros.where.in('id_campanha', request.params.CampanhaID))
            .delete()

        await CampanhaContatoCampoT.query
            .require(false)
            .where(CampanhaContatoCampo.where.in('id_campanha', request.params.CampanhaID))
            .delete()

        await CampanhaContatoT.query
            .require(false)
            .where(CampanhaContato.where.in('id_campanha', request.params.CampanhaID))
            .delete()

        await CampanhaT.query
            .require(false)
            .where(Campanha.where.in('id', request.params.CampanhaID))
            .delete()

        await transaction.commit();

        discador.delCampanhas(request.params.CampanhaID)
        reply('Campanhas deletadas')
    } catch (e) {
        transaction.rollback();
        reply(Boom.badData(e));
    }
};

exports.remove_todos_contatos = async (request, reply) => {

    const transaction = new db.orm.Transaction();
    const CampanhaContatoT = transaction.models.CampanhaContato;
    const CampanhaContatoNumerosT = transaction.models.CampanhaContatoNumeros;
    const CampanhaContatoCampoT = transaction.models.CampanhaContatoCampo;

    try {
        await CampanhaContatoCampoT.query
            .require(false)
            .where(CampanhaContatoCampo.where.in('id_campanha', request.params.CampanhaID))
            .delete()

        await CampanhaContatoNumerosT.query
            .require(false)
            .where(CampanhaContatoNumeros.where.in('id_campanha', request.params.CampanhaID))
            .delete()

        await CampanhaContatoT.query
            .require(false)
            .where(CampanhaContato.where.in('id_campanha', request.params.CampanhaID))
            .delete()

        await transaction.commit();
        reply('Contatos removidos')
    } catch (e) {
        transaction.rollback();
        reply(Boom.badData(e));
    }
};

exports.atualiza_status = async (request, reply) => {
    const transaction = new db.orm.Transaction();
    const CampanhaT = transaction.models.Campanha;

    try {
        let atualiza = {
            status: request.payload.status,
            data_status: request.payload.data_status,
        }

        await CampanhaT.query
            .require(false)
            .where(Campanha.where.in('id', request.payload.CampanhaID))
            .update(atualiza)

        await transaction.commit();

        let campanhas = await Campanha.query
            .require(false)
            .where(Campanha.where.in('id', request.payload.CampanhaID))
            .fetch()
        console.log(campanhas)
        discador.reloadCampanhasUpdate(campanhas)

        reply(atualiza);
    } catch (e) {
        transaction.rollback();
        reply(Boom.badData(e));
    }
}

exports.sucesso_true = async (request, reply) => {
    const transaction = new db.orm.Transaction();
    const CampanhaContatoT = transaction.models.CampanhaContato;
    const CampanhaContatoNumerosT = transaction.models.CampanhaContatoNumeros;

    try {
        const numeros = await CampanhaContatoNumeros.query
            .require(false)
            .where({ numero: request.params.Telefone })
            .fetch()
        let ContatosID = numeros.map(n => n.id_contato)

        await CampanhaContatoNumerosT.query
            .require(false)
            .where(CampanhaContatoNumeros.where.in('id_contato', ContatosID))
            .update({ sucesso: true })

        await CampanhaContatoT.query
            .require(false)
            .where(CampanhaContato.where.in('id', ContatosID))
            .update({ sucesso: true })

        await transaction.commit();
        reply(`Contato da campanha atualizado com sucesso!`);
    } catch (e) {
        transaction.rollback();
        reply(Boom.badData(e));
    }
}

async function upsertClientes (trx, layouts_campos, fones, registros) {
    try {
        const camposXcontact = layouts_campos.filter(c => c.xcontact && !c.xcontact.startsWith('telefone') && !c.xcontact.startsWith('variavelTemplate') && !c.xcontact.startsWith('texto'))
        const camposExtras = layouts_campos.filter(c => !c.xcontact || c.xcontact == 'variavelTemplate')
        const numerosExtras = fones.filter(f => f.xcontact !== 'telefoneA')
        const numeroPrincipal = fones.find(f => f.xcontact === 'telefoneA')
        const uid = uuid2.v4();
        camposXcontact.push({ xcontact: "idGrupo", nome: "idGrupo" })

        const contatos = []
        const contatosNumeros = []
        const contatosCampos = []

        registros.forEach(registro => {
            const ctt = {}
            ctt.uid = uid
            ctt.numero = String(registro[numeroPrincipal.nome]).replace(/\s+/g, '');

            //Vamos organizar os contatos
            camposXcontact.forEach(campo => {
                const nomeCampo = campo.xcontact;
                let valorCampo = registro[campo.nome];
                if (nomeCampo === "isBusiness") {
                    const isEmpresa = ["cnpj", "pessoa juridica", "pessoa jurídica", "juridica", "jurídica", "empresa"].includes(valorCampo.toLowerCase());
                    valorCampo = isEmpresa ? true : false;
                }
                // Renomeia a propriedade do registro de acordo com xcontact
                ctt[nomeCampo] = valorCampo === '' ? null : valorCampo
            })
            contatos.push(ctt)

            // Adiciona os campos adicionais
            camposExtras.forEach(campoExtra => {
                const valorCampo = registro[campoExtra.nome]
                if (valorCampo != null) {
                    const campoAdicional = {
                        uid: uid,
                        clienteNumero: ctt.numero,
                        clienteEmail: ctt.email,
                        nome: campoExtra.nome,
                        valor: valorCampo,
                    }
                    contatosCampos.push(campoAdicional)
                }
            })

            //Adiciona os numero adicionais
            numerosExtras.forEach(numeroExtra => {
                const valorNumeros = registro[numeroExtra.nome]
                // if (valorNumeros != null) {
                if (valorNumeros && valorNumeros !== '') {
                    const numeroAdicional = {
                        uid: uid,
                        clienteNumero: ctt.numero,
                        clienteEmail: ctt.email,
                        nome: numeroExtra.nome,
                        numero: valorNumeros.replace(/\s+/g, ''),
                    }
                    contatosNumeros.push(numeroAdicional)
                }
            })
        })

        await trx.batchInsert('clientes_import', contatos);
        if (contatosNumeros.length > 0) {
            await trx.batchInsert('clientes_import_numeros', contatosNumeros);
        }
        if (contatosCampos.length > 0) {
            await trx.batchInsert('clientes_import_campos', contatosCampos);
        }


        // Agora vem os pesados...
        await trx.raw(`UPDATE cliente b
        SET numero = COALESCE(a.numero, b.numero), nome = COALESCE(a.nome, b.nome), "profileImage" = COALESCE(a."profileImage", b."profileImage"), "idGrupo" = COALESCE(a."idGrupo", b."idGrupo"), email = COALESCE(a.email, b.email), documento = COALESCE(a.documento, b.documento), url = COALESCE(a.url, b.url), "facebookID" = COALESCE(a."facebookID", b."facebookID")
        FROM clientes_import a
        WHERE ((a.email is not null AND a.email = b.email) OR (a.numero is not null AND a.numero = b.numero)) AND a.uid = '${uid}'`);

        await trx.raw(`INSERT INTO cliente("numero", "isBusiness", "nome", "profileImage", "idGrupo", "email", "documento", "url", "facebookID")
        select "numero", "isBusiness", "nome", "profileImage", "idGrupo", "email", "documento", "url", "facebookID" from clientes_import WHERE uid = '${uid}' ON CONFLICT DO NOTHING`);

        await trx.raw(`UPDATE "clienteCamposAdicionais" up
       SET valor = imp."valor"
        FROM "clientes_import_campos" imp, "cliente" j
        WHERE (j.email = imp."clienteEmail" OR j.numero = imp."clienteNumero") AND j.id = up."clienteID" AND up.nome = imp.nome AND imp.uid = '${uid}'`);

        await trx.raw(`INSERT INTO "clienteCamposAdicionais"("clienteID", nome, valor)
        SELECT j.id,imp.nome,imp.valor
        FROM "clientes_import_campos" imp, "cliente" j
        WHERE (j.email = imp."clienteEmail" OR j.numero = imp."clienteNumero") AND imp.uid = '${uid}' ON CONFLICT DO NOTHING`);

        await trx.raw(`
        UPDATE "clienteNumerosAdicionais" up
        SET numero = imp."numero"
        FROM "clientes_import_numeros" imp
        JOIN "cliente" j ON (j.email = imp."clienteEmail" OR j.numero = imp."clienteNumero")
        WHERE j.id = up."clienteID" AND up.nome = imp.nome AND imp.uid = '${uid}'
        AND NOT EXISTS (
                SELECT 1 FROM "clienteNumerosAdicionais"
                WHERE numero = imp."numero" AND "clienteID" != j.id
        ) `);

        await trx.raw(`INSERT INTO "clienteNumerosAdicionais"("clienteID", nome, numero)
        SELECT j.id,imp.nome,imp.numero
        FROM "clientes_import_numeros" imp, "cliente" j
        WHERE (j.email = imp."clienteEmail" OR j.numero = imp."clienteNumero") AND imp.uid = '${uid}' ON CONFLICT DO NOTHING`);

        // Limpar tabelas de import
        await trx.raw(`DELETE FROM clientes_import WHERE uid = '${uid}'`);
        await trx.raw(`DELETE FROM clientes_import_campos WHERE uid = '${uid}'`);
        await trx.raw(`DELETE FROM clientes_import_numeros WHERE uid = '${uid}'`);

    } catch (error) {
        console.log(error)
        throw error;
    }
}

async function cadastraClientes (trx, layouts_campos, fones, registros) {
    try {
        const camposXcontact = layouts_campos.filter(c => c.xcontact && !c.xcontact.startsWith('telefone') && !c.xcontact.startsWith('variavelTemplate') && !c.xcontact.startsWith('texto'))
        const camposExtras = layouts_campos.filter(c => !c.xcontact || c.xcontact == 'variavelTemplate')
        const numerosExtras = fones.filter(f => f.xcontact !== 'telefoneA')
        const numeroPrincipal = fones.find(f => f.xcontact === 'telefoneA')
        const uid = uuid2.v4();
        camposXcontact.push({ xcontact: "idGrupo", nome: "idGrupo" })

        const contatos = []
        const contatosNumeros = []
        const contatosCampos = []

        registros.forEach(registro => {
            const ctt = {}
            ctt.uid = uid
            ctt.numero = String(registro[numeroPrincipal.nome]).replace(/\s+/g, '');

            //Vamos organizar os contatos
            camposXcontact.forEach(campo => {
                const nomeCampo = campo.xcontact;
                let valorCampo = registro[campo.nome];
                if (nomeCampo === "isBusiness") {
                    const isEmpresa = ["cnpj", "pessoa juridica", "pessoa jurídica", "juridica", "jurídica", "empresa"].includes(valorCampo.toLowerCase());
                    valorCampo = isEmpresa ? true : false;
                }
                // Renomeia a propriedade do registro de acordo com xcontact
                ctt[nomeCampo] = valorCampo === '' ? null : valorCampo
            })
            contatos.push(ctt)

            // Adiciona os campos adicionais
            camposExtras.forEach(campoExtra => {
                const valorCampo = registro[campoExtra.nome]
                if (valorCampo != null) {
                    const campoAdicional = {
                        uid: uid,
                        clienteNumero: ctt.numero,
                        clienteEmail: ctt.email,
                        nome: campoExtra.nome,
                        valor: valorCampo,
                    }
                    contatosCampos.push(campoAdicional)
                }
            })

            //Adiciona os numero adicionais
            numerosExtras.forEach(numeroExtra => {
                const valorNumeros = registro[numeroExtra.nome]
                // if (valorNumeros != null) {
                if (valorNumeros && valorNumeros !== '') {
                    const numeroAdicional = {
                        uid: uid,
                        clienteNumero: ctt.numero,
                        clienteEmail: ctt.email,
                        nome: numeroExtra.nome,
                        numero: valorNumeros.replace(/\s+/g, ''),
                    }
                    contatosNumeros.push(numeroAdicional)
                }
            })
        })

        await trx.batchInsert('clientes_import', contatos);
        if (contatosNumeros.length > 0) {
            await trx.batchInsert('clientes_import_numeros', contatosNumeros);
        }
        if (contatosCampos.length > 0) {
            await trx.batchInsert('clientes_import_campos', contatosCampos);
        }

        // Agora vem os pesados...
        await trx.raw(`INSERT INTO cliente("numero", "isBusiness", "nome", "profileImage", "idGrupo", "email", "documento", "url", "facebookID")
        select "numero", "isBusiness", "nome", "profileImage", "idGrupo", "email", "documento", "url", "facebookID" from clientes_import WHERE uid = '${uid}' ON CONFLICT DO NOTHING`);

        await trx.raw(`INSERT INTO "clienteCamposAdicionais"("clienteID", nome, valor)
        SELECT j.id,imp.nome,imp.valor
        FROM "clientes_import_campos" imp, "cliente" j
        WHERE (j.email = imp."clienteEmail" OR j.numero = imp."clienteNumero") AND imp.uid = '${uid}' ON CONFLICT DO NOTHING`);

        await trx.raw(`INSERT INTO "clienteNumerosAdicionais"("clienteID", nome, numero)
        SELECT j.id,imp.nome,imp.numero
        FROM "clientes_import_numeros" imp, "cliente" j
        WHERE (j.email = imp."clienteEmail" OR j.numero = imp."clienteNumero") AND imp.uid = '${uid}' ON CONFLICT DO NOTHING`);

        // Limpar tabelas de import
        await trx.raw(`DELETE FROM clientes_import WHERE uid = '${uid}'`);
        await trx.raw(`DELETE FROM clientes_import_campos WHERE uid = '${uid}'`);
        await trx.raw(`DELETE FROM clientes_import_numeros WHERE uid = '${uid}'`);

    } catch (error) {
        console.log(error)
        throw error;
    }
}

exports.uploadContatosJSON = async (request, reply) => {
    try {
        await db.knex.transaction(async trx => {
            const { CampanhaID } = request.params;

            const rows = await db.knex('public.campanhas')
                .where('id', CampanhaID)
                .select('id_layout');

            if (rows.length === 0) {
                return reply(Boom.badRequest(`Nenhuma campanha encontrada para CampanhaID = ${CampanhaID}`));
            }

            const { id_layout } = rows[0];
            if (!id_layout) {
                return reply(Boom.badRequest(`A CampanhaID = ${CampanhaID} não possui layout cadastrado.`));
            }

            const [layouts, layouts_campos] = await Promise.all([
                db.knex('public.layouts').where('id', id_layout).select('*'),
                db.knex('public.layouts_campos').where('id_layout', id_layout).select('*'),
            ]);

            if (layouts_campos.length === 0) {
                return reply(Boom.badRequest(`O layout associado a CampanhaID = ${CampanhaID} não possui campos definidos.`));
            }

            const fones = layouts_campos.filter(campo => campo.xcontact && campo.xcontact.startsWith('telefone'));
            if (fones.length === 0) {
                return reply(Boom.badRequest(`O layout associado a CampanhaID = ${CampanhaID} não possui campos de telefone definidos.`));
            }

            const registros = request.payload;
            const id_campanha = CampanhaID;
            const contatos = registros.map(registro => ({ id_campanha }));
            const insertedContatos = await trx.batchInsert('campanhas_contato', contatos, registros.length).returning('id');
            const sample = await db.knex('public.campanhas').where('id', CampanhaID).first();
            if (sample.cadastra_clientes) await upsertClientes(trx, layouts_campos, fones, registros)
            if (sample.cadastra_clientes_novos && !sample.cadastra_clientes) await cadastraClientes(trx, layouts_campos, fones, registros)

            const array_campos = [];
            const array_numeros = [];

            registros.forEach((registro, r) => {
                fones.forEach((fone) => {
                    if (registro[fone.nome] && registro[fone.nome].trim().length > 0) {
                        const id_contato = insertedContatos[r];
                        const cn = {
                            id_contato,
                            id_campanha,
                            numero: registro[fone.nome],
                            fone_sequencia: fone.xcontact,
                            nome: fone.nome
                        };
                        array_numeros.push(cn);
                    }
                });
                layouts_campos.forEach((campo) => {
                    if (typeof registro[campo.nome] === 'boolean' || (registro[campo.nome] && registro[campo.nome].trim().length > 0)) {
                        const id_contato = insertedContatos[r];
                        const cc = {
                            id_contato,
                            id_campanha,
                            nome: campo.nome,
                            valor: registro[campo.nome],
                        };
                        array_campos.push(cc);
                    }
                });
            });

            await trx.batchInsert('campanhas_contato_numeros', array_numeros);
            if (!sample.cadastra_clientes_novos && !sample.cadastra_clientes) await trx.batchInsert('campanhas_contato_campos', array_campos);
            reply({ error: false, rows: registros.length, message: `Registros importados com sucesso. ${insertedContatos.length}/${registros.length}` });
        })

    } catch (error) {
        console.error(error);
        reply(Boom.badData(error));
    }
};

exports.uploadContatos = async (request, reply) => {
    try {
        //Transformar para um array de objetos no processaCSV

        await db.knex.transaction(async trx => {
            let processarCSV = function (dadosDoArquivo, delimitador, layoutCampos) {
                return new Promise((resolve, reject) => {
                    let registros = [];
                    csv
                        .fromString(dadosDoArquivo, { headers: false, delimiter: delimitador })
                        .on("data", data => registros.push(data))
                        .on("error", error => reject(`Erro ao processar o CSV: ${error}`))
                        .on("end", () => {
                            let registros2 = registros.map(registro => {
                                let obj = {};
                                layoutCampos.forEach(campo => {
                                    const { posicao_csv, nome } = campo;
                                    obj[nome] = registro[posicao_csv];
                                });
                                return obj;
                            });
                            resolve(registros2);
                        });
                });
            };

            const CampanhaID = request.params.CampanhaID;
            const importacaoPrioritaria = request.query.prioridade;

            const rows = await db.knex('public.campanhas')
                .where('id', CampanhaID)
                .select('id_layout');

            if (rows.length === 0) {
                return reply(Boom.badRequest(`Nenhuma campanha encontrada para CampanhaID = ${CampanhaID}`));
            }

            const { id_layout } = rows[0];
            if (!id_layout) {
                return reply(Boom.badRequest(`A CampanhaID = ${CampanhaID} não possui layout cadastrado.`));
            }

            const [layouts, layouts_campos] = await Promise.all([
                db.knex('public.layouts').where('id', id_layout).select('*'),
                db.knex('public.layouts_campos').where('id_layout', id_layout).select('*'),
            ]);

            if (layouts_campos.length === 0) {
                return reply(Boom.badRequest(`O layout associado a CampanhaID = ${CampanhaID} não possui campos definidos.`));
            }

            const fones = layouts_campos.filter(campo => campo.xcontact && campo.xcontact.startsWith('telefone'));
            if (fones.length === 0) {
                return reply(Boom.badRequest(`O layout associado a CampanhaID = ${CampanhaID} não possui campos de telefone definidos.`));
            }

            const dadosMontados = request.payload.file.toString("latin1", 0, request.payload.file.length);
            const dadosDoArquivo = layouts[0].cabecalho ? dadosMontados.split('\n').slice(1).join('\n') : dadosMontados

            const registros = await processarCSV(dadosDoArquivo, layouts[0].delimitador, layouts_campos);
            const id_campanha = CampanhaID;
            const contatos = registros.map(registro => ({ id_campanha }));
            const insertedContatos = await trx.batchInsert('campanhas_contato', contatos, registros.length).returning('id');
            const sample = await db.knex('public.campanhas').where('id', CampanhaID).first();
            if (sample.cadastra_clientes) await upsertClientes(trx, layouts_campos, fones, registros)
            if (sample.cadastra_clientes_novos && !sample.cadastra_clientes) await cadastraClientes(trx, layouts_campos, fones, registros)

            let prioridade = 0;
            if (importacaoPrioritaria) {
                // Busca a prioridade atual
                let prioridadeAtual = await db.knex("public.campanhas_contato_numeros")
                    .where("id_campanha", CampanhaID)
                    .max("prioridade");
                if (prioridadeAtual && prioridadeAtual.length > 0 && "max" in prioridadeAtual[0]) {
                    // Incrementa a prioridade
                    prioridadeAtual = parseInt(prioridadeAtual[0].max);
                    prioridade += (prioridadeAtual || 0) + 1;
                }
            }

            const array_campos = [];
            const array_numeros = [];

            registros.forEach((registro, r) => {
                fones.forEach((fone) => {
                    if (registro[fone.nome] && registro[fone.nome].trim().length > 0) {
                        const id_contato = insertedContatos[r];
                        const cn = {
                            id_contato,
                            id_campanha,
                            numero: registro[fone.nome],
                            fone_sequencia: fone.xcontact,
                            nome: fone.nome,
                            prioridade
                        };
                        array_numeros.push(cn);
                    }
                });
                layouts_campos.forEach((campo) => {
                    if (typeof registro[campo.nome] === 'boolean' || (registro[campo.nome] && registro[campo.nome].trim().length > 0)) {
                        const id_contato = insertedContatos[r];
                        const cc = {
                            id_contato,
                            id_campanha,
                            nome: campo.nome,
                            valor: registro[campo.nome],
                        };
                        array_campos.push(cc);
                    }
                });
            });

            await trx.batchInsert('campanhas_contato_numeros', array_numeros);
            if (!sample.cadastra_clientes_novos && !sample.cadastra_clientes) await trx.batchInsert('campanhas_contato_campos', array_campos);
            reply({ error: false, rows: registros.length, message: `Registros importados com sucesso. ${insertedContatos.length}/${registros.length}` });
        })

    } catch (error) {
        console.error(error);
        reply(Boom.badData(error));
    }
};

exports.importLotesJSON = async (request, reply) => {
    if (request.payload && request.payload.dados) {
        const CampanhaID = request.query.sampleCampainID
        const errorLog = [];
        const dados = request.payload.dados;

        try {
            await db.knex.transaction(async trx => {
                const sample = await db.knex('public.campanhas').where('id', CampanhaID).first();

                if (sample.length === 0) {
                    return reply(Boom.badRequest(`Nenhuma campanha encontrada para CampanhaID = ${CampanhaID}`));
                }

                const { id_layout } = sample;
                if (!id_layout) {
                    return reply(Boom.badRequest(`A CampanhaID = ${CampanhaID} não possui layout cadastrado.`));
                }

                const [layouts, layouts_campos] = await Promise.all([
                    db.knex('public.layouts').where('id', id_layout).select('*'),
                    db.knex('public.layouts_campos').where('id_layout', id_layout).select('*'),
                ]);

                const campanhasClassificacoes = await db.knex('public.campanhas_classificacoes').where('id_campanha', CampanhaID).select('id_classificacao', 'acao')

                if (layouts_campos.length === 0) {
                    return reply(Boom.badRequest(`O layout associado a CampanhaID = ${CampanhaID} não possui campos definidos.`));
                }

                const fones = layouts_campos.filter(campo => campo.xcontact && campo.xcontact.startsWith('telefone'));
                if (fones.length === 0) {
                    return reply(Boom.badRequest(`O layout associado a CampanhaID = ${CampanhaID} não possui campos de telefone definidos.`));
                }

                sample.fones = fones
                sample.campos = layouts_campos

                const campanhas = await agruparCampanhas(dados, sample);

                // Vou inserir as campanhas
                const campanhasInseridas = await createOrLoadCampanhas(campanhas.campanhasCriadas, trx)
                const campanhasInseridasIds = campanhasInseridas.map(c => c.id)

                await trx('public.campanhas_classificacoes').whereIn('id_campanha', campanhasInseridasIds).delete()
                if (campanhasClassificacoes && campanhasClassificacoes.length > 0) {
                    const insertClassificacoes = campanhasInseridasIds.flatMap(id => {
                        return campanhasClassificacoes.map(cla => {
                            return {
                                ...cla,
                                id_campanha: id
                            }
                        })
                    })
                    await trx.batchInsert('campanhas_classificacoes', insertClassificacoes)
                }

                //Vou pegar os campos do CSV que estão no layout
                const indicesCampos = layouts_campos.map((c) => c.posicao_csv);
                const primeiroReg = campanhas.agrupado[0].datas[0].registros[0];
                const camposFiltrados = Object.keys(primeiroReg).filter((campo, indice) => {
                    return indicesCampos.includes(indice);
                });
                const telefonePrincipal = Object.keys(primeiroReg)[layouts_campos.find(c => c.xcontact === 'telefoneA').posicao_csv]


                // Agora vou adicionar o id para os contatos da campanha e limpar os campos que não estão no layout
                let allContacts = []
                let contactsInsert = []
                for (let i = 0; i < campanhas.agrupado.length; i++) {
                    const vendedor = campanhas.agrupado[i];
                    vendedor.datas.forEach((data) => {
                        const campanha = campanhasInseridas.find(c => c.nome.startsWith(vendedor.agente.nome) && c.data_ini.startsWith(data.data));
                        data.registros.forEach(registro => {
                            const registroFiltrado = {};
                            camposFiltrados.forEach((campo) => {
                                registroFiltrado[campo] = registro[campo];
                            });
                            const grupoCliente = campanhas.gruposClientes.find(grupo => grupo.nome === vendedor.agente.fullname)
                            if (grupoCliente) registroFiltrado.idGrupo = grupoCliente.id
                            registroFiltrado.id_campanha = campanha.id;
                            allContacts.push(registroFiltrado);
                        });
                        const contatos = data.registros.map(registro => ({ id_campanha: campanha.id }));
                        contactsInsert.push(...contatos);
                    });
                }

                // Faz um array sem clientes repetidos para poder efetuar o cadastro
                const dadosCadastro = _.uniqBy(allContacts, telefonePrincipal);

                // Inserindo os contatos
                const insertedContatos = await trx.batchInsert('campanhas_contato', contactsInsert).returning('id');
                const array_campos = [];
                const array_numeros = [];

                allContacts.forEach((registro, r) => {
                    const id_campanha = registro.id_campanha
                    fones.forEach((fone) => {
                        if (registro[fone.nome] && String(registro[fone.nome]).trim().length > 0) {
                            const id_contato = insertedContatos[r];
                            const cn = {
                                id_contato,
                                id_campanha,
                                numero: String(registro[fone.nome]),
                                fone_sequencia: fone.xcontact,
                                nome: String(fone.nome)
                            };
                            array_numeros.push(cn);
                        }
                    });
                    layouts_campos.forEach((campo) => {
                        if (typeof registro[campo.nome] === 'boolean' || (registro[campo.nome] && String(registro[campo.nome]).trim().length > 0)) {
                            const id_contato = insertedContatos[r];
                            const cc = {
                                id_contato,
                                id_campanha,
                                nome: String(campo.nome),
                                valor: String(registro[campo.nome]),
                            };
                            array_campos.push(cc);
                        }
                    });
                });

                await trx.batchInsert('campanhas_contato_numeros', array_numeros);
                if (!sample.cadastra_clientes_novos && !sample.cadastra_clientes) await trx.batchInsert('campanhas_contato_campos', array_campos);

                // Cadastrando os clientes
                if (sample.cadastra_clientes) await upsertClientes(trx, layouts_campos, fones, dadosCadastro)
                if (sample.cadastra_clientes_novos && !sample.cadastra_clientes) await cadastraClientes(trx, layouts_campos, fones, dadosCadastro)

                console.log(`[Rest/v4/campanhas/importLotes] Fim!`)
                if (errorLog.length > 0) reply(Boom.badData(errorLog.join('\l\n')));
                else reply(dados);
            })
        } catch (error) {
            log.error(`[Rest/v4/campanhas/importLotes] Erro ao processar arquivo -> ` + error);
            reply(Boom.badData(`Erro ao processar arquivo!\n` + error));
        }
    }
};

exports.importLotes = async (request, reply) => {
    if (request.payload && request.payload.file) {
        const CampanhaID = request.query.sampleCampainID
        const errorLog = [];
        const dados = [];

        //Processando o CSV (Não foi refatorado)
        let buffer = "";
        request.payload.file.on("data", chunk => {
            console.log(`[Rest/v4/campanhas/importLotes] Recebendo pedaços do arquivo!`);
            buffer += chunk;
        });
        request.payload.file.on("end", async () => {
            try {
                await db.knex.transaction(async trx => {
                    console.log(`[Rest/v4/campanhas/importLotes] Arquivo recebido! Processando...`);

                    buffer = buffer.toString("latin1", 0, buffer.length);

                    let parse = new Promise((resolve, reject) => {
                        csv.fromString(buffer, { headers: true, delimiter: ';' })
                            .on('data', row => dados.push(row))
                            .on('end', () => resolve(true));
                    });
                    await parse;

                    console.log(`[Rest/v4/campanhas/importLotes] CSV processado! Inserindo dados...`);

                    const sample = await db.knex('public.campanhas').where('id', CampanhaID).first();

                    if (sample.length === 0) {
                        return reply(Boom.badRequest(`Nenhuma campanha encontrada para CampanhaID = ${CampanhaID}`));
                    }

                    const { id_layout } = sample;
                    if (!id_layout) {
                        return reply(Boom.badRequest(`A CampanhaID = ${CampanhaID} não possui layout cadastrado.`));
                    }

                    const [layouts, layouts_campos] = await Promise.all([
                        db.knex('public.layouts').where('id', id_layout).select('*'),
                        db.knex('public.layouts_campos').where('id_layout', id_layout).select('*'),
                    ]);

                    const campanhasClassificacoes = await db.knex('public.campanhas_classificacoes').where('id_campanha', CampanhaID).select('id_classificacao', 'acao')

                    if (layouts_campos.length === 0) {
                        return reply(Boom.badRequest(`O layout associado a CampanhaID = ${CampanhaID} não possui campos definidos.`));
                    }

                    const fones = layouts_campos.filter(campo => campo.xcontact && campo.xcontact.startsWith('telefone'));
                    if (fones.length === 0) {
                        return reply(Boom.badRequest(`O layout associado a CampanhaID = ${CampanhaID} não possui campos de telefone definidos.`));
                    }

                    sample.fones = fones
                    sample.campos = layouts_campos

                    const campanhas = await agruparCampanhas(dados, sample);

                    // Vou inserir as campanhas
                    const campanhasInseridas = await createOrLoadCampanhas(campanhas.campanhasCriadas, trx)
                    const campanhasInseridasIds = campanhasInseridas.map(c => c.id)

                    await trx('public.campanhas_classificacoes').whereIn('id_campanha', campanhasInseridasIds).delete()
                    if (campanhasClassificacoes && campanhasClassificacoes.length > 0) {
                        const insertClassificacoes = campanhasInseridasIds.flatMap(id => {
                            return campanhasClassificacoes.map(cla => {
                                return {
                                    ...cla,
                                    id_campanha: id
                                }
                            })
                        })
                        await trx.batchInsert('campanhas_classificacoes', insertClassificacoes)
                    }

                    //Vou pegar os campos do CSV que estão no layout
                    const indicesCampos = layouts_campos.map((c) => c.posicao_csv);
                    const primeiroReg = campanhas.agrupado[0].datas[0].registros[0];
                    const camposFiltrados = Object.keys(primeiroReg).filter((campo, indice) => {
                        return indicesCampos.includes(indice);
                    });
                    const telefonePrincipal = Object.keys(primeiroReg)[layouts_campos.find(c => c.xcontact === 'telefoneA').posicao_csv]


                    // Agora vou adicionar o id para os contatos da campanha e limpar os campos que não estão no layout
                    let allContacts = []
                    let contactsInsert = []
                    for (let i = 0; i < campanhas.agrupado.length; i++) {
                        const vendedor = campanhas.agrupado[i];
                        vendedor.datas.forEach((data) => {
                            const campanha = campanhasInseridas.find(c => c.nome.startsWith(vendedor.agente.nome) && c.data_ini.startsWith(data.data));
                            data.registros.forEach(registro => {
                                const registroFiltrado = {};
                                camposFiltrados.forEach((campo) => {
                                    registroFiltrado[campo] = registro[campo];
                                });
                                const grupoCliente = campanhas.gruposClientes.find(grupo => grupo.nome === vendedor.agente.fullname)
                                if (grupoCliente) registroFiltrado.idGrupo = grupoCliente.id
                                registroFiltrado.id_campanha = campanha.id;
                                allContacts.push(registroFiltrado);
                            });
                            const contatos = data.registros.map(registro => ({ id_campanha: campanha.id }));
                            contactsInsert.push(...contatos);
                        });
                    }

                    // Faz um array sem clientes repetidos para poder efetuar o cadastro
                    const dadosCadastro = _.uniqBy(allContacts, telefonePrincipal);

                    // Inserindo os contatos
                    const insertedContatos = await trx.batchInsert('campanhas_contato', contactsInsert).returning('id');
                    const array_campos = [];
                    const array_numeros = [];

                    allContacts.forEach((registro, r) => {
                        const id_campanha = registro.id_campanha
                        fones.forEach((fone) => {
                            if (registro[fone.nome] && registro[fone.nome].trim().length > 0) {
                                const id_contato = insertedContatos[r];
                                const cn = {
                                    id_contato,
                                    id_campanha,
                                    numero: registro[fone.nome],
                                    fone_sequencia: fone.xcontact,
                                    nome: fone.nome
                                };
                                array_numeros.push(cn);
                            }
                        });
                        layouts_campos.forEach((campo) => {
                            if (typeof registro[campo.nome] === 'boolean' || (registro[campo.nome] && registro[campo.nome].trim().length > 0)) {
                                const id_contato = insertedContatos[r];
                                const cc = {
                                    id_contato,
                                    id_campanha,
                                    nome: campo.nome,
                                    valor: registro[campo.nome],
                                };
                                array_campos.push(cc);
                            }
                        });
                    });

                    await trx.batchInsert('campanhas_contato_numeros', array_numeros);
                    if (!sample.cadastra_clientes_novos && !sample.cadastra_clientes) await trx.batchInsert('campanhas_contato_campos', array_campos);

                    // Cadastrando os clientes
                    if (sample.cadastra_clientes) await upsertClientes(trx, layouts_campos, fones, dadosCadastro)
                    if (sample.cadastra_clientes_novos && !sample.cadastra_clientes) await cadastraClientes(trx, layouts_campos, fones, dadosCadastro)

                    console.log(`[Rest/v4/campanhas/importLotes] Fim!`)
                    if (errorLog.length > 0) reply(Boom.badData(errorLog.join('\l\n')));
                    else reply(dados);
                })
            } catch (error) {
                log.error(`[Rest/v4/campanhas/importLotes] Erro ao processar arquivo -> ` + error);
                reply(Boom.badData(`Erro ao processar arquivo!\n` + error));
            }
        });

        request.payload.file.on("error", error => {
            log.error(`[Rest/v4/campanhas/importLotes] Erro ao receber arquivo -> ` + error);
            reply(Boom.badData(`Erro ao receber arquivo!\n` + error));
        });
    }
};

async function createOrLoadCampanhas (campanhas, trx) {
    const retorno = { existentes: [], novas: [] };
    const nomesCampanhas = campanhas.map((c) => c.nome);
    let novasCampanhas = [];

    // Verifico quais campanhas já existem e separo as criadas das novas
    await db.knex("public.campanhas")
        .whereIn("nome", nomesCampanhas)
        .select("id", "nome")
        .then((campanhasExistentes) => {
            const existentesNomes = campanhasExistentes.map((c) => c.nome);
            const existentes = campanhas
                .filter((c) => existentesNomes.includes(c.nome))
                .map((campanha) => {
                    const campanhaExistente = campanhasExistentes.find(
                        (c) => c.nome === campanha.nome
                    );
                    campanha.id = campanhaExistente.id;
                    campanha.nova = false;
                    return campanha;
                });
            novasCampanhas = campanhas
                .filter((c) => !existentesNomes.includes(c.nome))
                .map(({ fones, campos, ...resto }) => resto);
            retorno.existentes = existentes;
        });

    // Agora crio as novas
    await trx
        .batchInsert("public.campanhas", novasCampanhas)
        .returning(["id", "nome"])
        .then((result) => {
            novasCampanhas = novasCampanhas.map((campanha) => {
                const nova = result.find((c) => c.nome === campanha.nome);
                campanha.id = nova.id;
                campanha.nova = false;
                return campanha;
            });
            retorno.novas = novasCampanhas;
        })
        .catch((error) => {
            console.error(error);
            throw error;
        });

    // Executar a operação em cada campanha
    for (let i = 0; i < campanhas.length; i++) {
        const campanha = campanhas[i];
        try {
            discador.addCampanha(campanha);
        } catch (ex) {
            console.log("Erro interno ao adicionar a campanha no discador, erro: " + ex);
        }
    }
    const todasCampanhas = retorno.existentes.concat(retorno.novas);
    return todasCampanhas;
}

//Agrupa os dados do arquivo em campanhas a serem inseridas
async function agruparCampanhas (data, sample) {

    const rt = _.chain(data)
        .sortBy('Vendedor')
        .groupBy('Vendedor')
        .map(function (values, key) {
            let c = {
                vendedor: key,
            };

            c.datas = _.chain(values)
                .sortBy('Data')
                .groupBy('Data')
                .map(function (v) {
                    return {
                        data: v[0].Data,
                        registros: v
                    };
                }).value();

            return c;
        }).value();

    const vendedores = _.uniq(_.map(data, 'Vendedor'));
    const agentes = await getAgentes(vendedores)
    const gruposClientes = await db.knex('public.clienteGrupos as cg').select('cg.*').whereIn('cg.nome', vendedores)
    const campanhasCriadas = [];
    for (const c of rt) {
        //Localiza o Agente
        const agente = agentes.find(ag => ag.nome === c.vendedor || ag.fullname === c.vendedor);
        c.agente = agente;
        for (const d of c.datas) {
            //Prepara os demais dados da campanha (lidos de uma campanha exemplo)
            let campanha = Object.assign({}, sample);
            //Ajustes do objeto
            delete campanha.id;
            delete campanha.data_create;
            campanha.nome = `${c.agente.nome}-${d.data.replace(/\//g, '')}`;
            campanha.fila = c.agente.filas[0];
            campanha.data_ini = `${d.data} 08:00:00`;
            campanha.data_fim = `${d.data} 18:00:00`;
            campanhasCriadas.push(campanha);
        }
    }

    const retorno = { agrupado: rt, campanhasCriadas: campanhasCriadas, gruposClientes }
    return retorno;
}

// Obtém todos os agentes do arquivo com suas filas
async function getAgentes (nomes) {
    const agentes = [];
    try {
        const rows = await db.knex('public.agentes as a')
            .innerJoin('public.agentes_filas as f', 'a.id', 'f.id_agente')
            .whereIn('a.fullname', nomes)
            .select('a.nome', 'a.fullname', 'f.fila')
            .orderBy('f.id');

        for (const nome of nomes) {
            const rowsIn = rows.filter((r) => r.fullname === nome);
            if (rowsIn && rowsIn.length > 0) {
                const agente = {
                    nome: rowsIn[0].nome,
                    fullname: rowsIn[0].fullname,
                    filas: rowsIn.map(row => row.fila)
                };
                agentes.push(agente);
            } else {
                throw `Nenhum agente encontrado com o Nome Completo: ${nome}`;
            }
        }

        return agentes;
    } catch (error) {
        console.error(error);
        throw error;
    }
}