'use strict'
const Joi = require('joi');
const handlers = require('./handlers');

module.exports = [

  {
    method: 'GET',
    path: '/api/v4/canaisDeTexto',
    handler: handlers.getCanaisDeTexto,
    config: {
      description: 'Obtem os canais',
      tags: ['api', 'CanaisDeTexto'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {       
          agente: Joi.string().optional().allow(null).description('Filtra por agente')        
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v4/getUsuariosCanalDeTexto/{idCanal}',
    handler: handlers.getUsuarioCanalDeTexto,
    config: {
      description: 'Obtem os usuarios canal de texto',
      tags: ['api', 'CanaisDeTexto'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {       
          idCanal: Joi.string().required()     
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v4/mensagensCanaisDeTexto/{idCanal}',
    handler: handlers.getMensagensCanalDeTexto,
    config: {
      description: 'Obtem as menasgens de um  canal',
      tags: ['api', 'CanaisDeTexto'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params:{
          idCanal:Joi.string().required()
        },
        query:{
          max: Joi.number().max(1000).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página')
        }      
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v4/canaisDeTexto/{agente}/{idCanal}/addUserCanal',
    handler: handlers.addUserCanalDeTexto,
    config: {
      description: 'Adiciona um usuario a um canal de texto',
      tags: ['api', 'CanaisDeTexto'],
      auth: {
        strategy: 'bearer',
        scope: ['user']
      },
      validate: {
        params: {
          agente: Joi.string().required(),
          idCanal: Joi.string().required()
        }
      }
    }
  },  
  {
    method: 'POST',
    path: '/api/v4/canaisDeTexto/{idCanal}/enviarMensagem',
    handler: handlers.enviaMensagemCanalDeTexto,
    config: {
        description: 'Envia mensagem de texto em um canal',
        tags: ['api', 'CanaisDeTexto'],
        auth: {
            strategy: 'bearer',
            scope: ['user', 'admin']
        },     
        validate: {   
            params:{
                idCanal:Joi.string().required()
            }, 
            payload: {
                agente: Joi.string().required().description('Agente que esta enviando a mensagem'),
                texto: Joi.string().required('mensagem a ser enviada!'),
            }
        }
        }
    },
   {
    method: 'POST',
    path: '/api/v4/canaisDeTexto',
    handler: handlers.criarCanalDeTexto,
    config: {
      description: 'Envia um ou mais anexos',
      tags: ['api', 'CanaisDeTexto'],
      auth: {
        strategy: 'bearer',
        scope: ['user', 'admin']
      },     
      validate: {       
        payload: {
            nome: Joi.string().required().description('Nome do canal de texto'),
            tipo: Joi.string().required('Tipo do canal de texto, Publico ou Privado'),
        }
      }
    }
  },
];
