const Boom = require("boom");
const vCliente = require("../../../models/vCliente");
const vClienteAtendimento = require("../../../models/vClienteAtendimento");
const vAtendimentosOmnichannel = require("../../../models/vAtendimentosOmnichannel");
const ClienteAtendimento = require("../../../models/ClienteAtendimento");
const AtendimentoMensagem = require("../../../models/AtendimentoMensagem");
const MensagemAnexos = require("../../../models/MensagemAnexos");
const RepliedMsg = require("../../../models/RepliedMsg");
const RepliedAnexos = require("../../..//models/RepliedAnexos");
const QueueChat = require("../../../models/QueueChat");
const QueueChatTags = require("../../../models/QueueChatTags");
const QueueChatCamposAdicionais = require("../../../models/QueueChatCamposAdicionais");
const Tags = require("../../../models/Tags");
const moment = require("moment");
const lodash = require("lodash");
const uuid = require("uuid/v1");
const path = require("path");
moment.locale("pt-br");
const fs = require("fs");
const { Console } = require("console");
const db = require("../../../util/db");

async function getMensagensAtendimento (contato) {
	return new Promise(async (resolve, reject) => {
		try {
			let idAtendimentos = contato.atendimentos
				.filter((a) => a.canal_tipo != "1")
				.map((a) => a.id);
			const q = AtendimentoMensagem.query;
			q.leftJoin(MensagemAnexos);
			q.leftJoin(RepliedMsg.query.leftJoin(RepliedAnexos));
			q.where(AtendimentoMensagem.where.in("idAtendimento", idAtendimentos));
			q.orderBy('"queueChatMessages".id ASC');
			contato.mensagens = await q.require(false).fetch();
			resolve(contato);
		} catch (error) {
			console.error(error);
			reject(error);
		}
	});
}

exports.getAtendimentos = async (request, reply) => {
	try {
		const query = vAtendimentosOmnichannel.query;
		query.leftJoin(vCliente);
		const dataini = request.query.data_ini
			? request.query.data_ini
			: new Date().toDateString();
		const datafim = request.query.data_fim
			? request.query.data_fim
			: new Date().toDateString();
		if (request.query.data_ini && request.query.data_fim)
			query.where(
				query.sql('"dataIni"::date BETWEEN $1 AND $2', [dataini, datafim])
			);
		if (request.query.protocolo)
			query.where(
				vAtendimentosOmnichannel.where.in("id", request.query.protocolo)
			);
		if (request.query.cliente)
			query.where(
				vAtendimentosOmnichannel.where.in("cliente_id", request.query.cliente)
			);
		if (request.query.hora_ini)
			query.where(query.sql('"dataIni"::time >= $1', [request.query.hora_ini]));
		if (request.query.hora_fim)
			query.where(query.sql('"dataIni"::time <= $1', [request.query.hora_fim]));
		if (request.query.agentes)
			query.where(
				vAtendimentosOmnichannel.where.in("agente", request.query.agentes)
			);
		if (request.query.filas)
			query.where(
				vAtendimentosOmnichannel.where.in("fila", request.query.filas)
			);
		if (request.query.status)
			query.where(
				vAtendimentosOmnichannel.where.in("status", request.query.status)
			);
		var registros = await query.require(false).fetch();
		let contatos = [];
		let contatos_ids = lodash.groupBy(registros, "cliente_id");
		for (const [key, value] of Object.entries(contatos_ids)) {
			let cliente = { ...value[0].vCliente[0] };
			delete vCliente;
			if (cliente) {
				console.log("tinha cliente", cliente);
				cliente.atendimentos = value;
				await getMensagensAtendimento(cliente);
				contatos.push(cliente);
			}
		}
		console.log("LISTA DE CONTATOS", contatos);
		reply(contatos);
	} catch (error) {
		console.error(error);
		reply(Boom.badData(error));
	}
};

exports.getChats = async (request, reply) => {
	try {
		const query = vClienteAtendimento.query;
		query.leftJoin(vCliente);

		const dataini = request.query.data_ini
			? request.query.data_ini
			: new Date().toDateString();
		const datafim = request.query.data_fim
			? request.query.data_fim
			: new Date().toDateString();
		query.where(
			query.sql('"dataIni"::date BETWEEN $1 AND $2', [dataini, datafim])
		);
		if (request.query.protocolo)
			query.where(vClienteAtendimento.where.in("id", request.query.protocolo));
		if (request.query.cliente)
			query.where(
				vClienteAtendimento.where.in("idCliente", request.query.cliente)
			);
		if (request.query.hora_ini)
			query.where(query.sql('"dataIni"::time >= $1', [request.query.hora_ini]));
		if (request.query.hora_fim)
			query.where(query.sql('"dataIni"::time <= $1', [request.query.hora_fim]));
		if (request.query.agentes)
			query.where(
				vClienteAtendimento.where.in("ultimoAgente", request.query.agentes)
			);
		if (request.query.filas)
			query.where(vClienteAtendimento.where.in("queue", request.query.filas));
		var registros = await query.require(false).fetch();
		reply(registros);
	} catch (e) {
		reply(Boom.badData(e));
	}
};

exports.getAllTags = async (request, reply) => {
	try {
		const query = QueueChatTags.query;
		query.leftJoin(Tags);
		const dataini = request.query.data_ini
			? request.query.data_ini
			: new Date().toDateString();
		const datafim = request.query.data_fim
			? request.query.data_fim
			: new Date().toDateString();
		query.where(
			query.sql('"data_registro"::date BETWEEN $1 AND $2', [dataini, datafim])
		);
		if (request.query.hora_ini)
			query.where(
				query.sql('"data_registro"::time >= $1', [request.query.hora_ini])
			);
		if (request.query.hora_fim)
			query.where(
				query.sql('"data_registro"::time <= $1', [request.query.hora_fim])
			);
		if (request.query.agentes) query.where(QueueChatTags.where.in("agente", request.query.agentes));
		if (request.query.tags_id) query.where(QueueChatTags.where.in("id_tag", request.query.tags_id));

		const ret = await query.require(false).fetch();

		// Prevenção de bug, alguns registros vem quebrado com o id_atendimento 'undefined' ou um UID com letras
		const registros = ret.filter((r) => /^\d+$/.test(r.id_atendimento));
		const idsAtendimentos = registros.map((reg) => reg.id_atendimento);
		let queueChat;

		if (request.query.filas) {
			const q = QueueChat.query;
			q.where(QueueChat.where.in("queue", request.query.filas));
			q.where(QueueChat.where.in("id", idsAtendimentos));
			queueChat = await q.require(false).fetch();
		} else {
			queueChat = await QueueChat.query
				.innerJoin(QueueChat.query.where(QueueChat.where.in("id", idsAtendimentos)))
				.fetch();
		}

		for (const reg of registros) {
			reg.clienteAtendimento = queueChat.filter(
				(chat) => chat.id == reg.id_atendimento
			);
		}

		reply({
			dados: registros.filter(reg => reg.clienteAtendimento.length > 0),
			error: registros.length <= 0 ? true : false,
			full: true,
			message: "Listagem full",
			total: registros.length,
		});
	} catch (e) {
		reply(Boom.badData(e));
	}
};

exports.getChatsTags = async function (request, reply) {
	try {
		const {
			max,
			page,
			filas,
			agentes,
			protocolo,
			cliente,
			data_ini,
			data_fim,
			hora_ini,
			hora_fim,
			status,
			tags_id,
			order
		} = request.query;

		console.log(agentes)

		const offset = (page - 1) * (max === -1 ? 0 : max);

		let query = db.knex('queueChat').select('*');

		// Aplicar filtros
		if (filas) {
			query.whereIn('queue', filas);
		}
		if (agentes) {
			query.whereIn('ultimoAgente', agentes);
		}
		if (protocolo) {
			query.whereIn('id', protocolo);
		}
		if (cliente) {
			query.whereIn('idCliente', cliente);
		}
		if (data_ini) {
			const startTime = hora_ini ? hora_ini : '00:00:00';
			const startTimestamp = `${data_ini}T${startTime}`;
			query.where('dataIni', '>=', startTimestamp);
		}
		if (data_fim) {
			const endTime = hora_fim ? hora_fim : '23:59:59';
			const endTimestamp = `${data_fim}T${endTime}`;
			query.where('dataIni', '<=', endTimestamp);
		}
		if (status) {
			// Verifica se 'EmAtendimento' está entre os status desejados
			const hasEmAtendimento = status.includes('EmAtendimento');

			// Remove 'EmAtendimento' do array para evitar confusão na consulta
			const filteredStatusArray = status.filter(status => status !== 'EmAtendimento');

			if (hasEmAtendimento) {
				// Aplica uma lógica condicional para quando 'EmAtendimento' é um dos status desejados
				query.where((builder) => {
					builder
						.whereIn('status', filteredStatusArray) // Aplica os outros filtros de status, se houver
						.orWhere(function () {
							this.where('status', 'Entrou')
								.andWhereNotNull('ultimoAgente');
						});
				});
			} else {
				// Caso 'EmAtendimento' não esteja entre os status, aplica o filtro normalmente
				query.whereIn('status', status);
			}
		}
		if (tags_id && tags_id.length > 0) {
			const tagsSubquery = db.knex('tags_atendimentos')
				.select('id_atendimento')
				.whereIn('id_tag', tags_id)
				.groupBy('id_atendimento')
			query.whereIn('queueChat.id', tagsSubquery);
		}
		if (order && order.length > 0) {
			order.forEach(orderItem => {
				query.orderBy(`queueChat.${orderItem.campo}`, orderItem.ordem);
			});
		}

		// Paginação e contagem total
		let result;
		let total;
		if (max !== 0) {
			result = await query.clone().limit(max).offset(offset);
			total = await query.clone().clearSelect().clearOrder().count('* as total').first();
		} else {
			result = await query.clone();
			total = { total: result.length };
		}

		const resultIds = result.map(row => row.id.toString())

		const tagsAtendimento = await db.knex('tags_atendimentos').select('*').whereIn('id_atendimento', resultIds)
		const tags = await db.knex('tags').select('*').whereIn('id', tagsAtendimento.map(tagAt => tagAt.id_tag))
		tagsAtendimento.forEach(tagAt => { tagAt.tags = tags.filter(tag => tag.id === tagAt.id_tag) })

		const camposAdicionais = await db.knex('atendimento_campos_adicionais').select('*').whereIn('id_atendimento', resultIds)

		const clientes = await db.knex('v_cliente').select('*').whereIn('id', result.map(row => row.idCliente))

		result.forEach(row => {
			row.queueChatCamposAdicionais = camposAdicionais.filter(ca => ca.id_atendimento == row.id)
			row.queueChatTags = tagsAtendimento.filter(ta => ta.id_atendimento == row.id)
			row.vCliente = clientes.filter(cli => cli.id == row.idCliente)
		})

		reply({
			page,
			max: max === -1 ? total.total : max,
			rows: result,
			pages: max === -1 ? 1 : Math.ceil(total.total / max),
			total: total.total
		});
	} catch (error) {
		reply(Boom.badImplementation('Erro ao buscar chats com tags', error));
	}
};

exports.getChatsTagsIds = async (request, reply) => {
	try {
		const q = QueueChat.query;
		q.leftJoin(vCliente);
		// q.leftJoin(
		// 	QueueChatTags.query
		// 		.leftJoin(Tags)
		// );
		// q.leftJoin(QueueChatCamposAdicionais);

		q.where(QueueChat.where.in("id", request.payload.protocolos));
		const registros = await q.require(false).fetch();

		const chatsID = registros.map((chat) => chat.id);
		const tags = await QueueChatTags.query
			.where(QueueChatTags.where.in("id_atendimento", chatsID))
			.leftJoin(Tags)
			.fetch();
		const adicionais = await QueueChatCamposAdicionais.query
			.where(QueueChatCamposAdicionais.where.in("id_atendimento", chatsID))
			.fetch();
		for (const chat of registros) {
			chat.queueChatCamposAdicionais = adicionais.filter(
				(adicional) => adicional.id_atendimento == chat.id
			);
			chat.queueChatTags = tags.filter(
				(adicional) => adicional.id_atendimento == chat.id
			);
		}

		reply(registros);
	} catch (e) {
		reply(Boom.badData(e));
	}
};

exports.getMessagesByCliente = async (request, reply) => {
	try {
		var pag =
			request.query.max && request.query.max > 0
				? { page: request.query.page, perPage: request.query.max }
				: undefined;
		const query = vAtendimentosOmnichannel.query;
		query.orderBy("atendimentos_omnichannel.id DESC");
		query.where({
			"atendimentos_omnichannel.cliente_id": request.query.idCliente,
		});
		// if (request.query.agente) {

		// }
		// query.where({ '"queueChat".status': 'Encerrado' });
		query.where(
			query.sql(
				`(atendimentos_omnichannel.status ='Encerrado' OR (atendimentos_omnichannel.status = 'Entrou' AND atendimentos_omnichannel.agente = $1))`,
				[request.query.agente]
			)
		);
		let atendimentos = await query.require(false).fetch(pag);
		if (atendimentos.rows) {
			atendimentos = atendimentos.rows;
			let mensagens = [];
			atendimentos.map((a) => {
				if (a.canal_tipo == "1") {
					mensagens.push({
						texto: `Data: ${moment(a.dataIni).format("llll")} - #${a.id
							} - tempo: ${a.duracao} minutos - fila: ${a.fila} - sentido: ${a.sentido
							} - agente: ${a.agente}`,
						inOut: 3,
						idAtendimento: a.id,
						queueTypeID: a.canal_tipo,
						dataHora: a.dataIni,
						timestamp: a.dataIni,
					});
				} else {
					mensagens.push({
						texto: `Data: ${moment(a.dataIni).format("llll")} - #${a.id
							} - fila: ${a.fila} - sentido: ${a.sentido} - agente: ${a.agente}`,
						inOut: 3,
						idAtendimento: a.id,
						queueTypeID: a.canal_tipo,
						dataHora: a.dataIni,
						timestamp: a.dataIni,
					});
				}
			});
			let idAtendimentos = atendimentos
				.filter((a) => a.canal_tipo != "1")
				.map((a) => a.id);
			const q = AtendimentoMensagem.query;
			q.leftJoin(MensagemAnexos);
			q.leftJoin(RepliedMsg.query.leftJoin(RepliedAnexos));
			q.where(AtendimentoMensagem.where.in("idAtendimento", idAtendimentos));
			q.orderBy('"queueChatMessages".id ASC');
			console.log(`PROTOCOLOS++++++++++++++++++++`, idAtendimentos);
			let msgs = await q.require(false).fetch();
			for (let i = 0; i < msgs.length; i++) {
				let atendimento = atendimentos.find(
					(a) => a.id == msgs[i].idAtendimento
				);
				msgs[i].timestamp = atendimento.dataIni;
				msgs[i].queueTypeID = atendimentos.find(
					(a) => a.id == msgs[i].idAtendimento
				)
					? atendimentos.find((a) => a.id == msgs[i].idAtendimento).canal_tipo
					: null;
				mensagens.push(msgs[i]);
			}
			mensagens = mensagens.sort((a, b) => a.timestamp - b.timestamp);
			reply(mensagens);
		} else {
			reply([]);
		}

		//	query.leftJoin(AtendimentoMensagem);
		// AtendimentoMensagem.query.leftJoin(MensagemAnexos);
		// AtendimentoMensagem.query.leftJoin(RepliedMsg);
		// RepliedMsg.query.leftJoin(RepliedAnexos);

		//reply(registros);
	} catch (error) {
		reply(Boom.badData(error));
	}
};

exports.updateDocumento = async (request, reply) => {
	try {
		let atendimento = await ClienteAtendimento.fetchById(
			request.params.protocolo
		);
		if (atendimento) {
			atendimento.assunto = request.params.documento;
			atendimento.save();
			reply(atendimento);
		}
	} catch (error) {
		console.error(error);
		reply(Boom.badData(error));
	}
};

exports.marcaNaoLida = async (request, reply) => {
	try {
		const atendimento = await ClienteAtendimento.query
			.require(false)
			.where("idCliente", request.params.idCliente)
			.where("ultimoAgente", request.params.agente)
			.first()
			.require()
			.update({ qtd_msg_nao_lidas: -1 });

		reply(atendimento);
	} catch (error) {
		console.error(error);
		reply(Boom.badData(error));
	}
};

exports.encerrarAllAtendimentosAguardando = async (request, reply) => {
	try {
		await ClienteAtendimento.query
			.where({ ultimoAgente: null })
			.update({ status: "Encerrado" }); // updates all rows
		reply(true);
	} catch (error) {
		reply(Boom.badData(error));
	}
};
exports.getMensagemAnexos = async (request, reply) => {
	try {
		const ret = await MensagemAnexos.query
			.where({idMensagem: request.params.idMensagem})
			.fetch();
		reply(ret);
	} catch (error) {
		reply(Boom.badData(error));
	}
}
exports.getAnexo = async function (request, reply) {
	try {
		let substr = request.query.caminho.split("/");
		let arq = substr[substr.length - 1];
		if (!request.query.caminho.startsWith("/gravacoes")) {
			reply(
				Boom.forbidden(
					"Você só pode acessar arquivos que estejam em /gravacoes"
				)
			);
		} else {
			if (fs.existsSync(request.query.caminho)) {
				reply.file(request.query.caminho, {
					name: arq,
					confine: false,
				});
			} else {
				reply(Boom.badData(`Arquivo não encontrado`));
			}
		}
	} catch (error) {
		reply(Boom.badData(error));
	}
};
exports.verificaAtendimentos = async (request, reply) => {
	try {
		const sqlChats = `
			SELECT COUNT(id)
			FROM "queueChat"
			WHERE status = 'Entrou'
				AND "ultimoAgente" IS NULL
				AND EXTRACT(EPOCH FROM (NOW() - "dataIni")) / 60 > 3
		`;
		const retChats = await db.query(sqlChats);
		reply(retChats.rows[0].count);
	} catch (error) {
		reply(Boom.badData(error));
	}
};

exports.uploadAttatch = async (request, reply) => {
	try {
		let atendimento = await ClienteAtendimento.fetchById(request.params.atendimentoID);
		const Omni = require("../../../xc_modules/omni/index");
		if (!Array.isArray(request.payload.files)) {
			request.payload.files = [request.payload.files];
		}
		const date = new Date();
		const dir = `/gravacoes/${atendimento.queueFrom ? atendimento.queueFrom : atendimento.queue}/${date.getFullYear()}/${date.getMonth() + 1}/${date.getDate()}/`;
		if (!fs.existsSync(path.resolve(dir))) {
			fs.mkdirSync(path.resolve(dir), { recursive: true });
		}
		if (request.payload.isEmail) {
			console.log("ERA EMAIL====================================================")
			let data = {
				file: true,
				protocolo: request.params.atendimentoID,
				queue: atendimento.queue,
				queueFrom: atendimento.queueFrom,
				queueAnexos: [],
				message: request.payload.legendas,
				client: `newClient`,
				agente: request.params.agente,
				email: request.payload.email,
				cco:request.payload.cco,
				cc:request.payload.cc,
				encaminha: request.payload.encaminha,
				envelope: request.payload.encaminha
			};
			let promises = request.payload.files.map((file) => {

				return new Promise(async (resolve, reject) => {
					const name = file.hapi.filename;
					const newName = name.substring(0, name.lastIndexOf(".")) + name.substring(name.lastIndexOf("."));
					const file_path = path.resolve(dir, newName);
					const writeStream = fs.createWriteStream(file_path);
					const mime = file.hapi.headers["content-type"] ? file.hapi.headers["content-type"] : "application/octet-stream";
					writeStream.on("error", function (err) {
						fs.unlinkSync(file_path);
						reject(err);
						console.error(err)
					});
					file.pipe(writeStream);
					file.on("end", async function (err) {
						if (err) {
							fs.unlinkSync(file_path);
							reject(err)
							console.error(err)
						} else {			
							console.log("RESOLVE====================================================")				
							data.queueAnexos.push({ caminho: file_path, mimeType: mime, filename: newName })					
							resolve(true);
						}
					});
				});
			})
			await Promise.all(promises);
			const ret = await Omni.sendFileMsg(data);
			console.log('RETORNO OMNI==================================================================================', ret)
			reply(ret);
		}else{
			request.payload.legendas = JSON.parse(request.payload.legendas);
			let promises = request.payload.files.map((file, i) => {
				return new Promise(async (resolve, reject) => {
					const name = file.hapi.filename;
					const newName = name.substring(0, name.lastIndexOf(".")) + uuid() + name.substring(name.lastIndexOf("."));
					const file_path = path.resolve(dir, newName);
					const writeStream = fs.createWriteStream(file_path);
					const mime = file.hapi.headers["content-type"] ? file.hapi.headers["content-type"] : "application/octet-stream";
					writeStream.on("error", function (err) {
						fs.unlinkSync(file_path);
						resolve({ ...file, error: err, size: file.bytes });
					});
					file.pipe(writeStream);
					file.on("end", async function (err) {
						if (err) {
							fs.unlinkSync(file_path);
							resolve({ ...file, error: err, size: file.bytes });
						} else {
							let data = {
								file: true,
								protocolo: request.params.atendimentoID,
								queue: atendimento.queue,
								queueFrom: atendimento.queueFrom,
								queueAnexos: [
									{ caminho: file_path, mimeType: mime, filename: newName },
								],
								type: mime,
								filename: newName,
								file_name: newName,
								Msg: request.payload.legendas[i],
								template: request.payload.template,
								client: `newClient`,
								agente: request.params.agente,
							};
							let result = await Omni.sendFileMsg(data);
							resolve(result);
						}
					});
				});
			})
	
			let results = await Promise.all(promises);
			reply(results);
		}
	
	} catch (error) {
		reply(Boom.badData(error));
	}
};
