const db = require('../../../util/db')
const _ = require('underscore')
const path = require('path')
const fs = require('fs')
const Boom = require('boom')
const Cliente = require('../../../models/Cliente');

exports.getContato = async (request, reply) => {
    try {
        const sqlChats = `select json_agg(chats) as chats FROM
        (SELECT 
        a.*,
        ( 
            SELECT json_agg(item)
            FROM (
            SELECT b.*
            FROM cliente b where id = a."idCliente"
            ) item
        ) AS cliente,
        ( 
            SELECT json_agg(item)
            FROM (
            SELECT b.agente,b.obs,c.*
            FROM "queueChatClassificacao" b 
            INNER JOIN "classificacoes" c on b."classificacaoID" = c.id
            where b."queueChatID" = a.id	  
            ) item
        ) AS classificacao,
        ( 
            SELECT json_agg(item)
            FROM (
            SELECT m.*,
            ( 
                SELECT json_agg(item)
                FROM (
                SELECT an.*
                FROM "queueAnexos" an
                where an."id_message" = m.id	  
                ) item
            ) AS anexos
            FROM "queueChatMessages" m
            where m."idQueueChat" = a.id
            ORDER BY m."dataHora" ASC
            ) item
        ) AS mensagens
        FROM "queueChat" a 
        WHERE a."queueTypeID" != 1 AND a."ultimoAgente" = $1 AND a."dataIni" >= $2 AND a."idCliente" = $3
        ORDER BY id ASC
        ) as chats`

        const sqlCalls = `select json_agg(calls) as calls FROM
        (SELECT 
        a.*,a."calldate" as "data_hora_last_msg", 1 as "queueTypeID",
        ( 
            SELECT json_agg(item)
            FROM (
            SELECT b.*
            FROM cliente b where b.id = a."idCliente"
            ) item
        ) AS cliente,
        ( 
            SELECT json_agg(item)
            FROM (
            SELECT b."data",b.obs,b.protocolo,c.*
            FROM "cdr_classificacoes" b 
            INNER JOIN "classificacoes" c on b."id_classificacao" = c.id
            where b."uniqueid" = a."callid"	  
            ) item
        ) AS classificacao
        FROM "queue_calls" a 
        WHERE a."agent" = $1 AND a."calldate" >= $2 AND a."idCliente" = $3
        ORDER BY calldate ASC
        ) as calls`

        let d = new Date()
        if (request.query.dataStart) {
            d = new Date(request.query.dataStart)
        }
        else {
            d.setMonth(d.getMonth() - 3);
            d.setHours(0, 0, 0, 0);
        }

        const retChats = await db.query(sqlChats, [request.params.AgenteFullName, d, request.params.ContatoID])
        const retCall = await db.query(sqlCalls, [request.params.AgenteFullName, d, request.params.ContatoID])

        const ret = _.union(retChats.rows[0].chats, retCall.rows[0].calls)

        const ret2 = _.chain(ret)
            .groupBy('idCliente')
            .sortBy('data_hora_last_msg')
            .map((clientes) => {
                let ob = clientes[0].cliente ? clientes[0].cliente[0] : { naocliente: true }
                let atendimentos = clientes.map(at => {
                    const { cliente, ...dados } = at
                    return dados
                })
                ob.data_hora_last_msg = atendimentos.reduce((a, b) => (new Date(a.data_hora_last_msg).getTime() > new Date(b.data_hora_last_msg).getTime() ? a : b)).data_hora_last_msg;

                const allAtendimentos = _.groupBy(atendimentos, 'queueTypeID')
                ob.atendimentos = Object.entries(allAtendimentos).map(tipo => {
                    const at = {}
                    at.tipo = tipo[0]
                    at.atendimentos = tipo[1]
                    return at
                })
                return ob
            })
            .reverse()
            .value()

        reply(ret2);
    } catch (error) {
        reply(Boom.badData(error));
    }
}

exports.getFotoContato = async (request, reply) => {
    try {
        const contato = await Cliente.fetchById(request.params.ContatoID)
        if (!contato || !contato.profileImage || !fs.existsSync(contato.profileImage)) {
            return reply(Boom.badData(`Arquivo não encontrado`));
        }
        return reply.file(contato.profileImage, {
            name: path.basename(contato.profileImage),
            confine: false,
        });
    } catch (error) {
        console.log(error);
        reply(Boom.badData(error));
    }
};
exports.findCliente = async (request, reply) => {
    try {
       const cliente = await  Cliente.BuscaClientePorNumero(request.params.Numero)
       console.log('CLIENTE', cliente)
       reply(cliente[0])
    } catch (error) {
        console.log(error);
        reply(Boom.badData(error));
    }
}


exports.uploadFotoContato = async (request, reply) => {
    try {
        const dir = '/gravacoes/clientes/'
        const file_path = path.resolve(dir, `${request.params.ContatoID.toString()}.jpg`)
        if (!fs.existsSync(path.resolve(dir))) {
            fs.mkdirSync(path.resolve(dir), { recursive: true })
        }
        const file = fs.createWriteStream(file_path)
        file.on('error', function (err) {
            console.error(err);
            fs.unlinkSync(file_path);
            reply(Boom.badData(err));
        });
        request.payload.file.pipe(file);
        request.payload.file.on('end', async function (err) {
            if (err) {
                fs.unlinkSync(file_path);
                reply(Boom.badData(err));
            }
            else {
                let dados = {
                    id: request.params.ContatoID,
                    profileImage: file_path
                }
                let ret = await Cliente.update(dados);
                reply(ret);
            }
        })
    } catch (error) {
        console.error(error);
        reply(Boom.badData(error));
    }
}