const Joi = require('joi');
const handlers = require('./handlers');

module.exports = [

    {
        method: 'GET',
        path: '/api/v4/contato/{AgenteFullName}/{ContatoID}',
        handler: handlers.getContato,
        config: {
            description: 'Obtem os atendimentos do agente',
            tags: ['api', 'Agente'],
            auth: {
                strategy: 'bearer',
                scope: ['app.agentes', 'admin']
            },
            validate: {
                params: {
                    AgenteFullName: Joi.string().required(),
                    ContatoID: Joi.number().required()
                },
                query: {
                    dataStart: Joi.date().allow(null).description('Atendimentos a partir da data informada (padrão 1 ano)')
                }
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/contato/{ContatoID}/foto',
        handler: handlers.getFotoContato,
        config: {
            description: 'Obtem a foto do contato',
            tags: ['api', 'Contato'],
            cache: {
                expiresIn: 10 * 60 * 1000, // 10 minutos em milissegundos
                privacy: 'private'
            },
            validate: {
                params: {
                    ContatoID: Joi.number().required()
                },
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/contato/findCliente/{Numero}',
        handler: handlers.findCliente,
        config: {
            description: 'Obtem os dados do cliente',
            tags: ['api', 'Contato'],
            validate: {
                params: {
                    Numero: Joi.string().required()
                },
            }
        }
    },
    {
        method: 'POST',
        path: '/api/v4/contato/{ContatoID}/foto',
        handler: handlers.uploadFotoContato,
        config: {
            description: 'Envia uma imgagem para o profile do contato',
            tags: ['api', 'Contato'],
            auth: {
                strategy: 'bearer',
                scope: ['app.clientes', 'admin']
            },
            payload: {
                output: 'stream',
                allow: 'multipart/form-data',
                maxBytes: 50 * 1024 * 1024, // 50mb
                parse: true,
            },
            validate: {
                params: {
                    ContatoID: Joi.number().required()
                },
                payload: {
                    file: Joi.object().unknown().description('Arquivo')
                }
            }
        }
    },
]