const Joi = require('joi');
const handlers = require('./handlers');

module.exports = [
    {
        method: 'GET',
        path: '/api/v4/contextos',
        handler: handlers.getAll,
        config: {
            description: 'Retorna os grupos de rotas',
            tags: ['api', 'Contexto'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.rotas']
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/contexto/{id}',
        handler: handlers.getOne,
        config: {
            description: 'Retorna o Contexto cadastrado',
            tags: ['api', 'Contexto'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.rotas']
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v4/contexto/{id}',
        handler: handlers.update,
        config: {
            description: 'Atualiza um Contexto',
            tags: ['api', 'Contexto'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.rotas']
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                },
                payload: Joi.object({
                    nome: Joi.string().required(),
                    sentido: Joi.string().required(),
                    descricao: Joi.string().optional().allow([null, '']),
                })
            }
        }
    },
    {
        method: 'POST',
        path: '/api/v4/contexto',
        handler: handlers.create,
        config: {
            description: 'Cria um Contexto',
            tags: ['api', 'Contexto'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.rotas']
            },
            validate: {
                payload: Joi.object({
                    nome: Joi.string().required(),
                    sentido: Joi.string().required(),
                    descricao: Joi.string().optional().allow([null, '']),
                })
            }
        }
    },
    {
        method: 'DELETE',
        path: '/api/v4/contexto/{id}',
        handler: handlers.delete,
        config: {
            description: 'Remove um Contexto',
            tags: ['api', 'Contexto'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.rotas']
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                }
            }
        }
    },
];
