const Boom = require('boom');
const db = require('../../../util/db');
const QueueCalls = require('../../../models/QueueCalls');
const QueueCallsTags = require('../../../models/QueueCallsTags');
const QueueCallsCamposAdicionais = require('../../../models/QueueCallsCamposAdicionais')
const CdrGravacao = require('../../../models/CdrGravacao')
const vCliente = require('../../../models/vCliente')
const Tags = require('../../../models/Tags');
const fs = require('fs');
const _ = require('underscore');
const Fila = require('../../../models/Fila')
const FilasGrupo = require('../../../models/FilasGrupo')
const FilasGruposFilas = require('../../../models/FilasGruposFilas')

exports.getLigacoesTags = async function (request, reply) {
    try {
        const max = request.query.max ? (request.query.max === -1 ? null : parseInt(request.query.max)) : 15
        const page = parseInt(request.query.page) || 1;
        const offset = (page - 1) * max;

        let query = db.knex('queue_calls')
            .select('queue_calls.*')
        // .select('queue_calls.*', 'cdr_gravacao.link as gravacao')
        // .leftJoin('cdr_gravacao', 'queue_calls.callid', 'cdr_gravacao.uniqueid');

        // Aplicar filtros
        if (request.query.data_ini) {
            query.where('queue_calls.calldate', '>=', request.query.data_ini);
        }
        if (request.query.data_fim) {
            query.where('queue_calls.calldate', '<=', request.query.data_fim);
        }
        if (request.query.numero) {
            query.where('queue_calls.numero', 'like', `%${request.query.numero}%`);
        }
        if (request.query.filas) {
            query.whereIn('queue_calls.queuename', request.query.filas);
        }
        if (request.query.agentes) {
            query.whereIn('queue_calls.agent', request.query.agentes);
        }
        if (request.query.status) {
            query.whereIn('queue_calls.status', request.query.status);
        }
        if (request.query.protocolo) {
            query.where('queue_calls.callid', 'like', `%${request.query.protocolo}%`);
        }
        if (request.query.cliente_nome) {
            query.where('queue_calls.cliente_nome', 'like', `%${request.query.cliente_nome}%`);
        }
        if (request.query.sentido) {
            query.where('queue_calls.inout', request.query.sentido);
        }
        if (request.query.finalizadapor) {
            query.where('queue_calls.terminatedby', request.query.finalizadapor);
        }
        if (request.query.tags_id && request.query.tags_id.length > 0) {
            const tagsSubquery = db.knex('tags_atendimentos')
                .select('id_atendimento')
                .whereIn('id_tag', request.query.tags_id)
                .groupBy('id_atendimento')
            query.whereIn('queue_calls.callid', tagsSubquery);
        }

        // Adicionar ordenação
        if (request.query.order && request.query.order.length > 0) {
            request.query.order.forEach(orderItem => {
                query.orderBy(`queue_calls.${orderItem.campo}`, orderItem.ordem);
            });
        }

        // Contar o total de registros
        const totalResult = await query.clone().clearSelect().clearOrder().count('* as total').first();

        // Executar consulta
        const result = max ? await query.clone().limit(max).offset(offset) : await query.clone()
        const resultIds = result.map(row => row.callid)

        const tagsAtendimento = await db.knex('tags_atendimentos').select('*').whereIn('id_atendimento', resultIds)
        const tags = await db.knex('tags').select('*').whereIn('id', tagsAtendimento.map(tagAt => tagAt.id_tag))
        tagsAtendimento.forEach(tagAt => { tagAt.tags = tags.filter(tag => tag.id === tagAt.id_tag) })

        const camposAdicionais = await db.knex('atendimento_campos_adicionais').select('*').whereIn('id_atendimento', resultIds)

        const clientes = await db.knex('v_cliente').select('*').whereIn('id', result.map(row => row.idCliente))

        const gravacoes = await db.knex('cdr_gravacao').select('*').whereIn('uniqueid', resultIds)
        // 'cdr_gravacao', 'queue_calls.callid', 'cdr_gravacao.uniqueid'

        result.forEach(row => {
            row.queueCallsCamposAdicionais = camposAdicionais.filter(ca => ca.id_atendimento === row.callid)
            row.queueCallsTags = tagsAtendimento.filter(ta => ta.id_atendimento === row.callid)
            const gravacao = gravacoes.find(grav => grav.uniqueid === row.callid)
            row.vCliente = clientes.filter(cli => cli.id === row.idCliente)
            row.gravacao = gravacao ? gravacao.link : null
        })

        reply({
            page,
            max,
            rows: result,
            pages: max ? Math.ceil(totalResult.total / max) : 1,
            total: totalResult.total
        });
    } catch (error) {
        reply(Boom.badImplementation('Erro ao buscar ligações com tags', error));
    }
};

exports.getLigacoesTagsIds = async function (request, reply) {
    try {
        const q = QueueCalls.query
        q.leftJoin(vCliente)
        const qTotal = QueueCalls.query
        q.leftJoin(
            QueueCallsTags.query
                .leftJoin(Tags)
        );
        q.leftJoin(QueueCallsCamposAdicionais);

        const max = request.payload.max ? request.payload.max : 15;
        const page = request.payload.page ? request.payload.page : 1;

        q.where(QueueCalls.where.in({ callid: request.payload.callids }))
        qTotal.where(QueueCalls.where.in({ callid: request.payload.callids }))

        const resultTotal = await qTotal.count();

        const pagination = {
            withPaginationData: false,
            perPage: max && max > -1 ? max : null,
            page: max && max > -1 && page && page > 0 ? page : null
        }
        const result = pagination.perPage && pagination.page ? await q.fetch(pagination) : await q.fetch();
        const callids = result.map(row => row.callid)

        const gravacoes = await CdrGravacao.query.where(CdrGravacao.where.in('uniqueid', callids)).fetch()

        // Isso aqui é uma gambiarra, pois pelo o SQL ele está duplicando os registros e não consegui mudar isso
        function removeDuplicate (call) {
            const uniqueTags = [];
            const seenIdsTags = new Set();
            for (const tag of call.queueCallsTags) {
                if (!seenIdsTags.has(tag.id)) {
                    uniqueTags.push(tag);
                    seenIdsTags.add(tag.id);
                }
            }

            const uniqueCamposAdicionais = [];
            const seenIdsCamposAdicionais = new Set();
            for (const tag of call.queueCallsCamposAdicionais) {
                if (!seenIdsCamposAdicionais.has(tag.id)) {
                    uniqueCamposAdicionais.push(tag);
                    seenIdsCamposAdicionais.add(tag.id);
                }
            }
            call.queueCallsCamposAdicionais = uniqueCamposAdicionais;
            call.queueCallsTags = uniqueTags;
            const grava = gravacoes.find(record => record.uniqueid == call.callid)
            call.gravacao = grava ? grava.link : null
        }

        // Use este loop para processar todos os resultados
        for (const call of result) {
            removeDuplicate(call);
        }

        const ret = { page, max, rows: result, pages: max < 1 ? resultTotal : Math.ceil(resultTotal / max), total: resultTotal }

        reply(ret);
    } catch (error) {
        reply(Boom.badImplementation('Erro ao buscar ligações com tags', error));
    }
};

exports.getLigacoes = (request, reply) => {
    const knex = db.knex('public.queue_calls');

    if (request.query.data_ini || request.query.data_fim) {
        if (request.query.data_ini) {
            let data = new Date(request.query.data_ini).toISOString().split("T")[0];
            knex.whereRaw(`calldate::date >= '${data}'`);

            console.log(`Data ini -> ${data}`);
        }

        if (request.query.data_fim) {
            let data = new Date(request.query.data_fim).toISOString().split("T")[0];
            knex.whereRaw(`calldate::date <= '${data}'`);

            console.log(`Data fim -> ${data}`);
        }
    } else if (request.query.calldate) {
        console.log(`Calldate array -> `, request.query.calldate);

        if (request.query.calldate.length > 0) {
            knex.whereRaw("calldate::date = ANY (ARRAY " + JSON.stringify(request.query.calldate).replace(/"/g, "'") + "::date[])");
        } else {
            knex.whereRaw("queue_calls.calldate::date >= '" + new Date().toISOString().slice(0, 10) + "'");
        }
    } else {
        // return reply(Boom.badData(`O Filtro de data precisa ser especificado!`));
    }

    if (request.query.hora_ini) knex.whereRaw("queue_calls.calldate::time >= '" + request.query.hora_ini + "'");
    if (request.query.hora_fim) knex.whereRaw("queue_calls.calldate::time <= '" + request.query.hora_fim + "'");
    if (request.query.status && request.query.status.length > 0) knex.whereRaw("queue_calls.status = ANY (ARRAY " + JSON.stringify(request.query.status).replace(/"/g, "'") + ")");
    if (request.query.filas && request.query.filas.length > 0) knex.whereRaw("queue_calls.queuename = ANY (ARRAY " + JSON.stringify(request.query.filas).replace(/"/g, "'") + ")");

    knex.leftJoin('cdr_gravacao', function () {
        this.on('queue_calls.callid', '=', 'cdr_gravacao.uniqueid').andOn('queue_calls.queuename', '=', 'cdr_gravacao.queuename')
    });
    knex.leftJoin('cdr_classificacoes', 'queue_calls.callid', 'cdr_classificacoes.uniqueid');
    knex.leftJoin('classificacoes', 'cdr_classificacoes.id_classificacao', 'classificacoes.id');
    knex.leftJoin('subclassificacao', 'cdr_classificacoes.id_subclassificacao', 'subclassificacao.id');
    knex.leftJoin('cdr_classificacoes as c2', 'queue_calls.callid2', 'c2.uniqueid');
    knex.leftJoin('classificacoes as cla2', 'c2.id_classificacao', 'cla2.id');
    knex.leftJoin('subclassificacao as sub2', 'c2.id_subclassificacao', 'sub2.id');

    knex.select(['queue_calls.callid',
        'queue_calls.calldate',
        'queue_calls.queuename as fila',
        'queue_calls.agent as agente',
        'queue_calls.numero',
        'queue_calls.status',
        db.knex.raw("TO_CHAR((queue_calls.holdtime || ' second'::text)::interval,'HH24:MI:SS') as espera"),
        db.knex.raw("TO_CHAR((queue_calls.calltime || ' second'::text)::interval,'HH24:MI:SS') as duracao"),
        'queue_calls.terminatedby as finalizadapor',
        'queue_calls.position as posicao_final',
        'queue_calls.origposition as posicao_entrada',
        'queue_calls.inout as sentido',
        'queue_calls.transferredto as transferida_para',
        'queue_calls.identclient',
        'queue_calls.idCliente',
        db.knex.raw("COALESCE ( classificacoes.id, cla2.id) as id_classificacao"),
        db.knex.raw("COALESCE ( classificacoes.nome, cla2.nome) as classificacao"),
        db.knex.raw("COALESCE ( cdr_classificacoes.obs, c2.obs) as observacao"),
        db.knex.raw("COALESCE ( cdr_classificacoes.protocolo, c2.protocolo) as protocolo"),
        db.knex.raw("COALESCE ( cdr_classificacoes.id_subclassificacao, c2.id_subclassificacao) as id_subclassificacao"),
        db.knex.raw("COALESCE ( subclassificacao.nome, sub2.nome) as subclassificacao"),
        'cdr_gravacao.link as gravacao'])
        .then(function (rows) {
            const ret = {};
            ret.error = false;
            ret.full = true;
            ret.total = rows.length;
            ret.message = 'Listagem full';
            ret.dados = rows;
            reply(ret);
        })
        .catch(function (error) {
            reply(Boom.badData(error));
        });
}

exports.getFilaGravacao = async function (request, reply) {
    try {
        let substr = request.query.caminho.split('/');
        let arq = substr[substr.length - 1]
        let sempermicao = true;
        const token = '1e5a475b7421c20cf0edb60543b20d406a2e55fd2619f5f04ee10dbdc41a007b'
        console.log('ENTROU AQUI=======================', request.query.access_token)
        if (request.query.access_token === token) {

            reply.file(request.query.caminho, {
                name: arq,
                confine: false
            })
            return
        }
        if (request.auth && request.auth.credentials && request.auth.credentials.extra) {
            const permi = request.auth.credentials.extra.find(item => item.modulo === 'app.cdrfilas');
            if (permi && permi.extra) {
                if (permi.extra.includes('G')) sempermicao = false;
            }
        }
        if (sempermicao) {
            reply(Boom.forbidden('Você não tem acesso a esse recurso'));
        }
        else if (!request.query.caminho.startsWith('/gravacoes')) {
            reply(Boom.forbidden('Você só pode acessar arquivos que estejam em /gravacoes'));
        }
        else {
            reply.file(request.query.caminho, {
                name: arq,
                confine: false
            })
        }
    } catch (error) {
        reply(Boom.badData(error));
    }
};
exports.getGravacaoAgenteWeb = async function (request, reply) {
    try {
        let substr = request.query.caminho.split('/');
        let arq = substr[substr.length - 1]
        reply.file(request.query.caminho, {
            name: arq,
            confine: false
        })
    } catch (error) {
        reply(Boom.badData(error));
    }

}

exports.getGruposFilas = async function (request, reply) {
    const max = request.query.max ? request.query.max : 15;
    const page = request.query.page ? request.query.page : 1;
    const knex = db.knex('queue_groups').leftJoin('queue_groups_queues', 'queue_groups.id', 'queue_groups_queues.idGrupo');

    if (request.query.nome) {
        knex.where('nome', 'ilike', `%${request.query.nome}%`);
    }

    if (request.query.nomes) {
        knex.where('nome', 'in', request.query.nomes);
    }

    if (request.query.filas) {
        knex.where(builder => {
            builder.whereIn('queue_groups_queues.fila', request.query.filas)
                .orWhereNull('queue_groups_queues.fila');
        });
    }

    if (max === -1) {
        knex.clearSelect();
        if (request.query.order) {
            for (let p = 0; p < request.query.order.length; ++p) {
                if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                    knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
                }
                else if (request.query.order[p] && request.query.order[p].campo) {
                    knex.orderBy(request.query.order[p].campo, 'ASC');
                }
            }
        }

        knex.select('*')
            .then(async function (rows) {
                // Agora eu vou tratar o array, pois ele duplica o registro por conta do innerJoin
                const gruposMap = {};
                rows.forEach((row) => {
                    const { idGrupo, nome, fila } = row;
                    if (!gruposMap[idGrupo]) {
                        gruposMap[idGrupo] = {
                            id: idGrupo,
                            nome: nome,
                            filas: []
                        };
                    }
                    gruposMap[idGrupo].filas.push(fila);
                });

                const grupos = Object.values(gruposMap);

                const ret = {};
                ret.error = false;
                ret.total = grupos.length;
                ret.message = 'Lista de todos os grupos de filas';
                ret.dados = grupos;
                reply(ret);
            })
            .catch(function (error) {
                console.error(error);
                reply(Boom.badData(error));
            });
    }
    else {
        const offset = max * (page - 1);

        knex.countDistinct('queue_groups.id as total')
            .then(function (total) {
                if (total[0].total > 0) {
                    knex.clearSelect();

                    if (request.query.order) {
                        for (let p = 0; p < request.query.order.length; ++p) {
                            if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                                knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
                            }
                            else if (request.query.order[p] && request.query.order[p].campo) {
                                knex.orderBy(request.query.order[p].campo, 'ASC');
                            }
                        }
                    }

                    knex.select('queue_groups.id as idGrupo_queue_groups', 'queue_groups.*', 'queue_groups_queues.*')
                        .limit(max)
                        .offset(offset)
                        .then(async function (rows) {
                            const pages = Math.ceil(parseInt(total[0].total) / max);

                            // Agora eu vou tratar o array, pois ele duplica o registro por conta do innerJoin
                            const gruposMap = {};
                            rows.forEach((row) => {
                                const { idGrupo_queue_groups, idGrupo, nome, fila } = row;
                                if (!gruposMap[idGrupo]) {
                                    gruposMap[idGrupo] = {
                                        id: idGrupo || idGrupo_queue_groups,
                                        nome: nome,
                                        filas: []
                                    };
                                }
                                gruposMap[idGrupo].filas.push(fila);
                            });

                            const grupos = Object.values(gruposMap);

                            const ret = {};
                            ret.error = false;
                            ret.full = pages === 1 ? true : false;
                            ret.page = page;
                            ret.max = max;
                            ret.offset = offset;
                            ret.total = parseInt(total[0].total);
                            ret.pages = pages;
                            ret.message = 'Lista de grupos de filas paginada';
                            ret.dados = grupos;
                            reply(ret);
                        })
                        .catch(function (error) {
                            console.error(error);
                            reply(Boom.badData(error));
                        });
                }
                else {
                    const ret = {};
                    ret.error = true;
                    ret.total = 0;
                    ret.message = 'Nenhum registro encontrado';
                    reply(ret);
                }
            })
            .catch(function (error) {
                console.error(error);
                reply(Boom.badData(error));
            });
    }
};

exports.getGrupoFilas = async function (request, reply) {
    try {
        const grupo = await db.knex('queue_groups').select('*').where({ id: request.params.idGrupo }).first()
        grupo.filas = await db.knex('queue_groups_queues').select('*').where({ idGrupo: request.params.idGrupo })
        reply(grupo)
    } catch (error) {
        reply(Boom.badData(error))
    }
}

exports.createGrupoFilas = async function (request, reply) {
    try {
        await db.knex.transaction(async trx => {
            const gf = (await trx('queue_groups').insert({ nome: request.payload.nome }).returning('*'))[0]

            if (gf) {
                const filas = request.payload.filas.map((fila) => { return { idGrupo: gf.id, fila: fila } })
                gf.filas = request.payload.filas
                await trx.batchInsert('queue_groups_queues', filas);
            }

            reply(gf)
        })
    } catch (error) {
        reply(Boom.badData(error));
    }
}

exports.updateGrupoFilas = async function (request, reply) {
    try {
        await db.knex.transaction(async trx => {
            const atualizado = { nome: request.payload.nome }
            await trx('queue_groups').where({ id: request.params.idGrupo }).update(atualizado)

            // Agora a parte das filas
            const filas = request.payload.filas.map((fila) => { return { idGrupo: request.params.idGrupo, fila: fila } })
            await trx('queue_groups_queues').where({ idGrupo: request.params.idGrupo }).del()
            await trx.batchInsert('queue_groups_queues', filas)

            // Vou verificar se esse grupo possui agentes e/ou supervisores
            const agentesGrupos = await db.knex('public.agentes_grupos_filas').select('*')
            const agentesDoGrupoID = agentesGrupos.filter(ag => ag.fila_grupo === request.payload.nome).map(ag => ag.id_agente)
            const supervisoresGrupos = await db.knex('public.supervisores_grupos_filas').select('*')
            const supervisoresDoGrupoID = supervisoresGrupos.filter(user => user.fila_grupo === request.payload.nome).map(user => user.id_supervisor)

            if ((agentesDoGrupoID && agentesDoGrupoID.length > 0) || (supervisoresDoGrupoID && supervisoresDoGrupoID.length > 0)) {
                // Aqui eu vou pegar todos os outros grupos existentes com suas respectivas filas
                const todosGrupos = await db.knex('public.queue_groups').select('*')
                const todosGruposFilas = await db.knex('public.queue_groups_queues').select('*')
                for (let grupo of todosGrupos) {
                    grupo.fila = grupo.id === request.params.idGrupo ? request.payload.filas : todosGruposFilas.filter(f => f.idGrupo === grupo.id).map(f => f.fila)
                }
                if (agentesDoGrupoID && agentesDoGrupoID.length > 0) {
                    // Aqui eu deleto as filas dos agentes do grupo e adiciono-as novamente já atualizadas
                    const filasAgs = []
                    for (let agente of agentesDoGrupoID) {
                        const ag = {}
                        ag.id = agente
                        ag.gruposNomes = agentesGrupos.filter(agg => agg.id_agente === ag.id).map(agg => agg.fila_grupo)
                        ag.grupos = todosGrupos.filter(grupo => ag.gruposNomes.includes(grupo.nome))
                        ag.filasNomes = _.union(ag.grupos.flatMap(grupo => grupo.fila))
                        ag.filas = ag.filasNomes.map((fila) => { return { id_agente: ag.id, fila: fila } })
                        filasAgs.push(...ag.filas)
                    }

                    await trx('agentes_filas').whereIn('id_agente', agentesDoGrupoID).del()
                    await trx.batchInsert('agentes_filas', filasAgs)
                }

                if (supervisoresDoGrupoID && supervisoresDoGrupoID.length > 0) {
                    // Aqui eu deleto as filas dos usuários do grupo e adiciono-as novamente já atualizadas
                    const filasAgs = []
                    for (let agente of agentesDoGrupoID) {
                        const ag = {}
                        ag.id = agente
                        ag.gruposNomes = agentesGrupos.filter(agg => agg.id_agente === ag.id).map(agg => agg.fila_grupo)
                        ag.grupos = todosGrupos.filter(grupo => ag.gruposNomes.includes(grupo.nome))
                        ag.filasNomes = _.union(ag.grupos.flatMap(grupo => grupo.fila))
                        ag.filas = ag.filasNomes.map((fila) => { return { id_agente: ag.id, fila: fila } })
                        filasAgs.push(...ag.filas)
                    }

                    await trx('agentes_filas').whereIn('id_agente', agentesDoGrupoID).del()
                    await trx.batchInsert('agentes_filas', filasAgs)
                }
            }

            atualizado.filas = request.payload.filas
            atualizado.id = request.params.idGrupo
            reply(atualizado)
        })
    } catch (error) {
        reply(Boom.badData(error))
    }
}

exports.deleteGrupoFilas = async function (request, reply) {
    try {
        await db.knex.transaction(async trx => {
            const deletado = await db.knex('queue_groups').select('*').where({ id: request.params.idGrupo }).first()
            deletado.filas = await db.knex('queue_groups_queues').select('*').where({ idGrupo: request.params.idGrupo })
            // Vou verificar quais agentes estão nesse grupo
            const agentesGrupos = await db.knex('public.agentes_grupos_filas').select('*')
            const agentesDoGrupoID = agentesGrupos.filter(ag => ag.fila_grupo === deletado.nome).map(ag => ag.id_agente)
            const supervisoresGrupos = await db.knex('public.supervisores_grupos_filas').select('*')
            const supervisoresDoGrupoID = supervisoresGrupos.filter(user => user.fila_grupo === deletado.nome).map(user => user.id_supervisor)

            await trx('agentes_grupos_filas').where('fila_grupo', deletado.nome).del()
            await trx('queue_groups_queues').where('idGrupo', request.params.idGrupo).del()
            await trx('queue_groups').where('id', request.params.idGrupo).del()

            if ((agentesDoGrupoID && agentesDoGrupoID.length > 0) || (supervisoresDoGrupoID && supervisoresDoGrupoID.length > 0)) {
                // Aqui eu vou pegar todos os outros grupos existentes com suas respectivas filas
                const todosGrupos = await db.knex('public.queue_groups').select('*')
                const todosGruposFilas = await db.knex('public.queue_groups_queues').select('*')
                for (let grupo of todosGrupos) {
                    grupo.fila = grupo.id === request.params.idGrupo ? [] : todosGruposFilas.filter(f => f.idGrupo === grupo.id).map(f => f.fila)
                }

                if (agentesDoGrupoID && agentesDoGrupoID.length > 0) {
                    // Aqui eu deleto as filas dos agentes do grupo e adiciono-as novamente já atualizadas
                    const filasAgs = []
                    for (let agente of agentesDoGrupoID) {
                        const ag = {}
                        ag.id = agente
                        ag.gruposNomes = agentesGrupos.filter(agg => agg.id_agente === ag.id).map(agg => agg.fila_grupo)
                        ag.grupos = todosGrupos.filter(grupo => ag.gruposNomes.includes(grupo.nome))
                        ag.filasNomes = _.union(ag.grupos.flatMap(grupo => grupo.fila))
                        ag.filas = ag.filasNomes.map((fila) => { return { id_agente: ag.id, fila: fila } })
                        filasAgs.push(...ag.filas)
                    }

                    await trx('agentes_filas').whereIn('id_agente', agentesDoGrupoID).del()
                    await trx.batchInsert('agentes_filas', filasAgs)
                }

                if (supervisoresDoGrupoID && supervisoresDoGrupoID.length > 0) {
                    // Aqui eu deleto as filas dos usuários do grupo e adiciono-as novamente já atualizadas
                    const filasAgs = []
                    for (let agente of agentesDoGrupoID) {
                        const ag = {}
                        ag.id = agente
                        ag.gruposNomes = agentesGrupos.filter(agg => agg.id_agente === ag.id).map(agg => agg.fila_grupo)
                        ag.grupos = todosGrupos.filter(grupo => ag.gruposNomes.includes(grupo.nome))
                        ag.filasNomes = _.union(ag.grupos.flatMap(grupo => grupo.fila))
                        ag.filas = ag.filasNomes.map((fila) => { return { id_agente: ag.id, fila: fila } })
                        filasAgs.push(...ag.filas)
                    }

                    await trx('agentes_filas').whereIn('id_agente', agentesDoGrupoID).del()
                    await trx.batchInsert('agentes_filas', filasAgs)
                }
            }

            reply(deletado)
        })
    } catch (error) {
        reply(Boom.badData(error))
    }
}