// Imports
const db = require('../../../util/db');
const Ami = require('../../../ami');
const log = require("../../../util/Log.js");
const Boom = require('boom');
const fs = require('fs');

// Functions
function enfileirarLigacaoOriginate(origem, callid) {
    // Cria a função que vai receber os eventos de hangup do AMI
    let handler = (function (origem, callid, event) {
        // Espera 5 segundos antes de prosseguir, importante para garantir que o Asterisk tenha inserido na CDR
        setTimeout(async () => {
            try {
                log.info(`[AMI][HANGUP][EFETUAR] Processando hangup da ligação originate ${callid}`);

                // Ignora eventos que tenham um callid diferente do esperado
                if (event.uniqueid != callid && event.linkedid != callid) return;

                // Altera a linha no banco
                let response = await db.knex(`cdr`)
                    .update({ "src": origem })
                    .where({ "linkedid": callid });

                // Verifica a quantidade de linhas que foram alteradas no banco
                if (response <= 0) log.warn(`AMI_HANGUP: Não foi possível encontrar a ligação originate ${callid}`);
            } catch (error) {
                console.trace(error);
                log.error(`AMI_HANGUP: Não foi possível atualizar a ligação originate ${callid}.\n${error}`);
            }

            // Remove o listener do evento, ele só serve para essa ligação e não tem uso adiante
            Ami.XCeventos.removeListener("Desliga", handler);
        }, 5000);
    }).bind(this, origem, callid);

    Ami.XCeventos.on("Desliga", handler);
}

// Exports
exports.discar = function (request, reply) {
    Ami.Discar(request.payload, async (resposta) => {
        const ret = {};
        ret.error = resposta.response === 'Error' ? true : false;
        ret.callid = resposta.response === 'Error' ? undefined : resposta.callid;
        ret.message = resposta.message;

        // Enfileira a ligação para ser atualizado o campo "src" quando houver hangup
        enfileirarLigacaoOriginate(request.payload.origem, resposta.callid);

        reply(ret);
    });
};
exports.conferencia = function (request, reply) {
    Ami.Conferencia(request.payload, async (reposta) => {
        const ret = {};
        ret.error = reposta.response === 'Error' ? true : false;
        ret.callid = reposta.response === 'Error' ? undefined : reposta.callid;
        ret.message = reposta.message;
        reply(ret);
    })
};

exports.TransferirComConsulta = async function (request, reply) {
    try {
        const ret = await Ami.TransferirComConsulta({
            CanalOrigem: request.payload.canal_origem,
            NumeroDestino: request.payload.destino,
            ContextoDestino: request.payload.rota,
            CanalDestino: request.payload.canal_destino
        })
        reply(ret);
    } catch (error) {
        reply(Boom.badData(error));
    }
};

exports.CoreShowChannels = async function (request, reply) {
    try {
        const ret = await Ami.CoreShowChannels();
        reply(ret);
    } catch (error) {
        reply(Boom.badData(error));
    }
};

exports.getAudioFile = async function (request, reply) {
    try {
        let substr = request.query.caminho.split('/');
        let arq = substr[substr.length - 1]
        let sempermicao = true;
        if (request.auth && request.auth.credentials && request.auth.credentials.extra) {
            const permi = request.auth.credentials.extra.find(item => item.modulo === 'app.cdr');
            if (permi && permi.extra) {
                if (permi.extra.includes('G')) sempermicao = false;
            }
        }
        if (sempermicao) {
            reply(Boom.forbidden('Você não tem acesso a esse recurso'));
        }
        else {
            reply.file(request.query.caminho, {
                name: arq,
                confine: false
            })
        }
    } catch (error) {
        reply(Boom.badData(error));
    }
};

exports.getClassificacaoLigacao = async function (request, reply) {
    try {
        const knex = db.knex('public.cdr_classificacoes')
        knex.where('uniqueid', request.params.uniqueid)
        const ret = await knex.select('cdr_classificacoes.*')
                        .orderBy('data', 'DESC')
        reply(ret)
    } catch (error) {
        reply(Boom.badData(error));
    }
}