const Joi = require('joi');
const handlers = require('./handlers');

module.exports = [
    {
        method: 'POST',
        path: '/api/v4/ligacao/efetuar',
        handler: handlers.discar,
        config: {
            description: 'Efetua uma ligação.',
            tags: ['api', 'v4', 'Ligação'],
            auth: {
                strategy: 'bearer',
                scope: ['admin']
            },
            validate: {
                payload: Joi.object({
                    origem: Joi.string().required().example('SIP/1000').description('Ramal de origem.'),
                    destino: Joi.string().required().description('Destino').example('05130305500'),
                    destino_nome: Joi.string().description('Opcional. Nome para o destino').example('João da Silva'),
                    rota: Joi.string().required().description('Rota de saída. Grupo de rota informado do cadastro do Ramal.').example('Ramais'),
                    max_chamar: Joi.number().integer().description('Tempo máximo chamando').default(30).example(30),
                    campos: Joi.object().keys().unknown().example('{"CPF":"123456"}').description('Campos associados a ligação para fins de integração com o Cliente.')
                })
            }
        }
    },
    {
        method: 'POST',
        path: '/api/v4/ligacao/Conferencia',
        handler: handlers.conferencia,
        config: {
            description: 'Efetua uma ligação.',
            tags: ['api', 'v4', 'Ligação'],
            auth: {
                strategy: 'bearer',
                scope: ['admin']
            },
            validate: {
                payload: Joi.object({
                    sala: Joi.string().required().example('123').description('Id sala'),
                    destino: Joi.string().required().description('Destino').example('05130305500'),
                    destino_nome: Joi.string().description('Opcional. Nome para o destino').example('João da Silva'),
                    rota: Joi.string().required().description('Rota de saída. Grupo de rota informado do cadastro do Ramal.').example('Ramais'),
                    max_chamar: Joi.number().integer().description('Tempo máximo chamando').default(30).example(30),
                    campos: Joi.object().keys().unknown().example('{"CPF":"123456"}').description('Campos associados a ligação para fins de integração com o Cliente.')
                })
            }
        }
    },
    {
        method: 'POST',
        path: '/api/v4/ligacao/transferir_com_consulta',
        handler: handlers.TransferirComConsulta,
        config: {
            description: 'Transfere uma ligação.',
            tags: ['api', 'Ligacao'],
            auth: {
                strategy: 'bearer',
                scope: ['admin']
            },
            validate: {
                payload: Joi.object({
                    canal_origem: Joi.string(),
                    canal_destino: Joi.string(),
                    destino: Joi.string(),
                    rota: Joi.string(),
                })
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/ligacao/canais_ativos',
        handler: handlers.CoreShowChannels,
        config: {
            description: 'Exibe os canais ativos (CoreShowChannels)',
            tags: ['api', 'Ligacao'],
            auth: {
                strategy: 'bearer',
                scope: ['admin']
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/ligacoes/gravacao',
        handler: handlers.getAudioFile,
        config: {
            description: 'Ouviu gravação de ligações',
            tags: ['api', 'Ligação'],
            auth: {
                strategy: 'bearer',
                scope: ['app.cdr']
            },
            validate: {
                query: {
                    access_token: Joi.string().description('Token'),
                    caminho: Joi.string().required().description('Caminho do arquivo'),
                }
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/ligacoes/downloadgravacao',
        handler: handlers.getAudioFile,
        config: {
            description: 'Fez download da gravação de ligações',
            tags: ['api', 'Ligação'],
            auth: {
                strategy: 'bearer',
                scope: ['app.cdr']
            },
            validate: {
                query: {
                    access_token: Joi.string().description('Token'),
                    caminho: Joi.string().required().description('Caminho do arquivo'),
                }
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/ligacoes/getClassificacaoLigacao/{uniqueid}',
        handler: handlers.getClassificacaoLigacao,
        config: {
            description: 'Obtem as calls do cliente em todas as filas que o agente faz parte',
            tags: ['api', 'Agente'],
            auth: {
                strategy: 'bearer',
                scope: ['app.agentes', 'admin']
            },
            validate: {
                params: {
                    uniqueid: Joi.string().required().description('uniqueid da ligacao'),
                }                
            }
        }
    },
];
