const Boom = require("boom");
const csv = require("fast-csv");
const TextoUtil = require("../../../util/texto");
const db = require("../../../util/db");
const MimeType = require("mime");
const uuid = require("uuid/v1");
const moment = require("moment");
const fs = require("fs");
const ListaTransmissao = require("../../../models/ListaTransmissao");
const ListaTransmissaoArquivos = require("../../../models/ListaTransmissaoArquivos");
const ListaTransmissaoCampos = require("../../../models/ListaTransmissaoCampos");
const ListaTransmissaoNumeros = require("../../../models/ListaTransmissaoNumeros");
const RegrasTransbordo = require("../../../models/ListaTransmissaoRegrasTransbordo");
const ListaTransmissaoTransbordo = require("../../../models/ListaTransmissaoTransbordo");
const Agente = require("../../../models/Agente");
const { processaLista, setarCron } = require("../../../lista_transmissao");

let processarCSV = function (dadosDoArquivo) {
  return new Promise((resolve, reject) => {
    let registros = [];
    csv
      .fromString(dadosDoArquivo, { headers: false, delimiter: ";" })
      .on("data", (data) => registros.push(data))
      .on("error", (error) => reject(`Erro ao processar o CSV: ${error}`))
      .on("end", () => resolve(registros));
  });
};

exports.createLista = async (request, reply) => {
  try {
    let ob = await ListaTransmissao.save(request.payload);
    let dados = request.payload;
    if (dados.agendaData) {
      console.log("VAMOS MARCAR NO CRON");
      setarCron(ob[0], new Date(dados.agendaData));
    }
    reply(ob);
  } catch (error) {
    console.error(error);
    reply(Boom.badData(error));
  }
};

exports.updateLista = async (request, reply) => {
  try {
    let dados = request.payload;
    dados.id = request.params.id;
    let ob = await ListaTransmissao.save(dados);
    if (dados.agendaData) {
      setarCron(ob[0], new Date(dados.agendaData));
      console.log("VAMOS MARCAR NO CRON", ob);
    }
    reply(ob);
  } catch (error) {
    console.error(error);
    reply(Boom.badData(error));
  }
};

exports.getLista = async (request, reply) => {
  try {
    const query = ListaTransmissao.query;
    query.where({ id: request.params.id });
    query.innerJoin(Agente, { fields: ["fullname"] });
    var registros = await query.require(true).fetch();
    reply(registros[0]);
  } catch (e) {
    reply(Boom.badData(e));
  }
};

exports.getListas = async (request, reply) => {
  try {
    const query = ListaTransmissao.query;
    query.innerJoin(Agente, { fields: ["fullname"] });
    var registros = await query.require(false).fetch();
    reply(registros);
  } catch (e) {
    reply(Boom.badData(e));
  }
};

exports.getListasAgente = async (request, reply) => {
  try {
    const query = ListaTransmissao.query;
    query.where({ idAgente: request.params.idAgente });
    query.innerJoin(Agente, { fields: ["fullname"] });
    var registros = await query.require(false).fetch();
    reply(registros);
  } catch (e) {
    reply(Boom.badData(e));
  }
};

exports.getListaArquivos = async (request, reply) => {
  try {
    const query = ListaTransmissaoArquivos.query;
    query.where({ idLista: request.params.id });
    var registros = await query.require(false).fetch();
    reply(registros);
  } catch (e) {
    reply(Boom.badData(e));
  }
};

exports.getListaCampos = async (request, reply) => {
  try {
    const query = ListaTransmissaoCampos.query;
    query.where({ idLista: request.params.id });
    var registros = await query.require(false).fetch();
    reply(registros);
  } catch (e) {
    reply(Boom.badData(e));
  }
};

exports.createNumero = async (request, reply) => {
  try {
    let dados = request.payload;
    dados.idLista = request.params.id;
    const count = await ListaTransmissaoNumeros.query
      .where({ idLista: request.params.id })
      .fetch();
    if (count.length + 1 > 15000) {
      reply(
        Boom.badData("Não foi possível subir, excedeu o limite de contatos.")
      );
      return;
    }
    let ob = await ListaTransmissaoNumeros.save(dados);
    reply(ob);
  } catch (error) {
    console.error(error);
    reply(Boom.badData(error));
  }
};

exports.updateNumero = async (request, reply) => {
  try {
    let dados = request.payload;
    dados.id = request.params.id;
    let ob = await ListaTransmissaoNumeros.save(dados);
    reply(ob);
  } catch (error) {
    console.error(error);
    reply(Boom.badData(error));
  }
};

exports.getListaNumeros = async (request, reply) => {
  try {
    const query = ListaTransmissaoNumeros.query;
    query.where({ idLista: request.params.id });
    var registros = await query.require(false).fetch();
    reply(registros);
  } catch (e) {
    reply(Boom.badData(e));
  }
};

exports.getListaRegras = async (request, reply) => {
  try {
    const query = ListaTransmissaoTransbordo.query;
    query.where({ idLista: request.params.id });
    query.innerJoin(RegrasTransbordo, { fields: ["nome", "regra"] });
    var registros = await query.require(false).fetch();
    reply(registros);
  } catch (e) {
    reply(Boom.badData(e));
  }
};

exports.getRegrasTransbordos = async (request, reply) => {
  try {
    const query = RegrasTransbordo.query;
    var registros = await query.require(false).fetch();
    reply(registros);
  } catch (e) {
    reply(Boom.badData(e));
  }
};

exports.importNumeros = async (request, reply) => {
  const file = request.payload.file;
  const aceitos = [
    "application/vnd.ms-excel",
    "text/plain",
    "text/csv",
    "text/tsv",
  ];
  if (aceitos.includes(file.hapi.headers["content-type"])) {
    let body = "";
    file.on("data", (chunk) => {
      body += chunk;
    });
    file.on("end", async () => {
      try {
        let registros = await processarCSV(body);
        const numeros = [];
        for (let i = 0; i < registros.length; i++) {
          let ob = {};
          ob.idLista = request.params.id;
          ob.numero = TextoUtil.limparTextoNumerico(registros[i][0]);
          ob.nome = registros[i][1];
          numeros.push(ob);
        }
        if (numeros.length > 15000) {
          reply(
            Boom.badData(
              "Não foi possível subir o arquivo, excedeu o limite de contatos."
            )
          );
          return;
        }
        const count = await ListaTransmissaoNumeros.query
          .where({ idLista: request.params.id })
          .fetch();
        if (count.length + numeros.length > 15000) {
          reply(
            Boom.badData(
              "Não foi possível subir o arquivo, excedeu o limite de contatos."
            )
          );
          return
        }
        const ret = await ListaTransmissaoNumeros.insert(numeros);
        reply({
          error: false,
          msg: `${ret.length} registros importados com sucesso.`,
          importados: ret.length,
          enviados: registros.length,
        });
      } catch (error) {
        reply(Boom.badData(error));
      }
    });
    file.on("error", (err) => {
      reply(Boom.badData(err));
    });
  } else {
    reply(Boom.badData("Formato de arquivo não aceito."));
  }
};

exports.delLista = async (request, reply) => {
  const transaction = new db.orm.Transaction();
  const ListaTransmissaoNumerosT = transaction.models.ListaTransmissaoNumeros;
  const ListaTransmissaoCamposT = transaction.models.ListaTransmissaoCampos;
  const ListaTransmissaoTransbordoT =
    transaction.models.ListaTransmissaoTransbordo;
  const ListaTransmissaoArquivosT = transaction.models.ListaTransmissaoArquivos;
  const ListaTransmissaoT = transaction.models.ListaTransmissao;
  try {
    await ListaTransmissaoNumerosT.query
      .require(false)
      .where({ idLista: request.params.id })
      .delete();

    await ListaTransmissaoCamposT.query
      .require(false)
      .where({ idLista: request.params.id })
      .delete();

    await ListaTransmissaoTransbordoT.query
      .require(false)
      .where({ idLista: request.params.id })
      .delete();

    await ListaTransmissaoArquivosT.query
      .require(false)
      .where({ idLista: request.params.id })
      .delete();

    await ListaTransmissaoT.query
      .require(false)
      .where({ id: request.params.id })
      .delete();

    await transaction.commit();

    reply({ error: false, msg: "Lista deletetada" });
  } catch (error) {
    transaction.rollback();
    reply(Boom.badData(error));
  }
};

exports.delNumero = async (request, reply) => {
  try {
    const ob = await ListaTransmissaoNumeros.fetchById(request.params.id);
    ob.delete();
    reply(ob);
  } catch (error) {
    reply(Boom.badData(error));
  }
};

exports.delAllNumeros = async (request, reply) => {
  try {
    const query = ListaTransmissaoNumeros.query;
    query.where({ idLista: request.params.idLista });
    var registros = await query.delete();
    reply({
      error: false,
      msg: `${registros.length} registros apagados.`,
      total: registros.length,
    });
  } catch (e) {
    reply(Boom.badData(e));
  }
};

exports.uploadArquivoLista = async (request, reply) => {
  try {
    const file = request.payload.file;
    // Gera caminho a salvar o arquivo
    let path = `/gravacoes/ListaTransmissao_${
      request.params.id
    }/${moment().year()}/${moment().month()}/${moment().date()}/`;
    let filepath = path + file.hapi.filename;

    // Cria a estrutura de pastas, caso não exista
    if (!fs.existsSync(path)) fs.mkdirSync(path, { recursive: true });

    // Cria a stream do arquivo local
    let localFileStream = fs.createWriteStream(filepath);
    localFileStream.on("error", (e) => {
      // Em caso de erro ao escrever no arquivo
      // Deleta o arquivo local
      fs.unlinkSync(filepath);
      return reply(Boom.badData(e));
    });

    // Redireciona todos os chunks do arquivo pra stream do arquivo local
    file.pipe(localFileStream);

    // Quando receber todos os pedaços do arquivo
    file.on("end", async (e) => {
      if (e) {
        // Em caso de erro deleta o arquivo local
        fs.unlinkSync(filepath);
        return reply(Boom.badData(e));
      } else {
        try {
          const listaArquivo = new ListaTransmissaoArquivos();
          listaArquivo.idLista = request.params.id;
          listaArquivo.caminho = filepath;
          listaArquivo.legenda = request.payload.legenda;
          const ret = await listaArquivo.save();
          return reply(ret);
        } catch (e) {
          // Deleta o arquivo local
          fs.unlinkSync(filepath);
          return reply(Boom.badData(e));
        }
      }
    });
  } catch (error) {
    reply(Boom.badData(error));
  }
};

exports.delArquivo = async (request, reply) => {
  try {
    const ob = await ListaTransmissaoArquivos.fetchById(request.params.id);
    ob.delete();
    try {
      fs.unlinkSync(ob.caminho);
    } catch (error) {
      console.log("Erro ao apagar arquivo", error);
    }
    reply(ob);
  } catch (error) {
    reply(Boom.badData(error));
  }
};

exports.delAllArquivos = async (request, reply) => {
  try {
    const query = ListaTransmissaoArquivos.query;
    query.where({ idLista: request.params.idLista });
    var registros = await query.delete();
    for (let i = 0; i < registros.length; i++) {
      try {
        fs.unlinkSync(registros[i].caminho);
      } catch (error) {
        console.log("Erro ao apagar arquivo", error);
      }
    }
    reply({
      error: false,
      msg: `${registros.length} registros apagados.`,
      total: registros.length,
    });
  } catch (e) {
    reply(Boom.badData(e));
  }
};

exports.executarLista = async (request, reply) => {
  try {
    const retorno = await processaLista(request.params.id);
    reply(retorno);
  } catch (error) {
    console.error(error);
    reply(Boom.badData(error));
  }
};
