const Boom = require('boom');
const ApiLog = require('../../../models/ApiLog');

exports.getAll = async function (request, reply) {
    try {
        const permi = request.auth.credentials.extra.find(item => item.modulo === 'app.log');

        const fields = ['DataHora', 'rota', 'RotaDescricao', 'metodo', 'UserLogin', 'ip', 'pc', 'status'];
        const query = ApiLog.query;

        if (permi && permi.extra) {
            if (permi.extra.includes('P')) fields.push('payload');
            if (permi.extra.includes('Q')) fields.push('query');
            if (permi.extra.includes('R')) fields.push('retorno');
            if (permi.extra.includes('F')) fields.push('parametros');
        }

        query.fields(fields);
        const dataini = request.query.data_ini ? request.query.data_ini : new Date().toDateString();
        const datafim = request.query.data_fim ? request.query.data_fim : new Date().toDateString();
        query.where(query.sql('"DataHora"::date BETWEEN $1 AND $2', [dataini, datafim]));
        if (request.query.hora_ini) query.where(query.sql('"DataHora"::time >= $1', [request.query.hora_ini]));
        if (request.query.hora_fim) query.where(query.sql('"DataHora"::time <= $1', [request.query.hora_fim]));
        if (request.query.usuarios) query.where(ApiLog.where.in('UserLogin', request.query.usuarios));

        //query.leftJoin(Fila, { fields: ['name'] });
        var registros = await query.require(false).fetch();
        reply(registros);
    } catch (error) {
        reply(Boom.badData(error));
    }
};