const Boom = require('boom');
const db = require('../../../util/db');
const Rota = require('../../../models/Rota');
const RotaGrupo = require('../../../models/RotaGrupo');

exports.getAll = async function (request, reply) {
    try {
        const query = Rota.query;
        query.leftJoin(RotaGrupo);
        var registros = await query.require(false).fetch();
        reply(registros);
    } catch (error) {
        reply(Boom.badData(error));
    }
};

exports.getOne = async function (request, reply) {
    try {
        const query = Rota.query;
        query.where({ id: request.params.id });
        query.leftJoin(RotaGrupo);
        var registros = await query.require(true).fetch();
        reply(registros[0]);
    } catch (error) {
        reply(Boom.badData(error));
    }
};

exports.getPorPrefixo = async function (request, reply) {
    try {
        const query = Rota.query;
        query.where({ exten: request.params.prefixo });
        query.where({ grupo: request.params.grupo });
        query.orderBy(['priority']);
        query.leftJoin(RotaGrupo);
        var registros = await query.require(true).fetch();
        reply(registros);
    } catch (error) {
        reply(Boom.badData(error));
    }
};

exports.replace = async function (request, reply) {
    const transaction = new db.orm.Transaction();
    try {
        const RotaT = transaction.models.Rota;
        await RotaT.query
            .where({ exten: request.params.prefixo })
            .where({ grupo: request.params.grupo })
            .require(false)
            .delete();

        var novo = await RotaT.insert(request.payload);
        await transaction.commit();
        reply(novo);
    } catch (error) {
        await transaction.rollback();
        reply(Boom.badData(error));
    }
};

exports.update = async (request, reply) => {
    try {
        let dados = request.payload;
        dados.id = request.params.id;
        let ret = await Rota.update(dados);
        reply(ret);
    } catch (error) {
        reply(Boom.badData(error));
    }
}

exports.create = async (request, reply) => {
    try {
        let ret = await Rota.save(request.payload);
        reply(ret);
    } catch (error) {
        reply(Boom.badData(error));
    }
}

exports.delete = async (request, reply) => {
    try {
        const query = Rota.query;
        query.where({ id: request.params.id });
        var registros = await query.require(true).delete();
        reply(registros);
    } catch (e) {
        reply(Boom.badData(e));
    }
};

exports.deletePrefixo = async function (request, reply) {
    try {
        let ret = await Rota.query
            .where({ exten: request.params.prefixo })
            .where({ grupo: request.params.grupo })
            .require(false)
            .delete();

        reply(ret);
    } catch (error) {
        reply(Boom.badData(error));
    }
};

exports.limpaGrupo = async function (request, reply) {
    try {
        let ret = await Rota.query
            .where({ grupo: request.params.grupo })
            .require(false)
            .delete();

        reply(ret);
    } catch (error) {
        reply(Boom.badData(error));
    }
};

exports.clone = async (request, reply) => {
    try {
        await Rota.query.execute({
            text: `INSERT INTO rotas(context, exten, priority, app, appdata, nome) SELECT $1 as grupo, exten, priority, app, appdata, nome FROM rotas where context = $2`,
            values: [request.payload.grupo_destino, request.payload.grupo_origem]
        });
        reply({ error: false, msg: `Clone das rotas de ${request.payload.grupo_origem} para ${request.payload.grupo_destino} feito com sucesso.` });
    } catch (e) {
        reply(Boom.badData(e));
    }
};

exports.cloneRota = async (request, reply) => {
    try {
        await Rota.query.execute({
            text: `INSERT INTO rotas(context, exten, priority, app, appdata, nome) SELECT $1 as grupo, exten, priority, app, appdata, nome FROM rotas where context = $2 AND exten = $3`,
            values: [request.payload.grupo_destino, request.payload.grupo_origem, request.payload.rota]
        });
        reply({ error: false, msg: `Clone da rota ${request.payload.rota} do grupo ${request.payload.grupo_origem} para ${request.payload.grupo_destino} feito com sucesso.` });
    } catch (e) {
        reply(Boom.badData(e));
    }
};