"use strict"
const Joi = require('joi');
const Boom = require('boom');
const Handlers = require("./handlers");
const Config = require('../../../models/Config');
module.exports = [
    {
        method: "GET",
        path: "/api/v4/servidor/horario",
        handler: Handlers.getHorarioServidor,
        config: {
            description: "Retorna o horário atual do servidor",
            tags: ["api", "v4", "Servidor"],
            auth: {
                strategy: "bearer",
                scope: "user" // or [ "user", "admin" ]
            }
        }
    },
    {
        method: "GET",
        path: "/api/v4/servidor/config",
        handler: async (request, reply) => {
            try {
                let query = Config.query;
                if (request.query.nome) query.where({ nome: request.query.nome });
                let config = await query.require(false).fetch();
                reply(config);
            } catch (error) {
                reply(Boom.badData(error));
            }
        },
        config: {
            description: "Retorna as configurações",
            tags: ["api", "v4", "Servidor"],
            auth: {
                strategy: "bearer",
                scope: "user" // or [ "user", "admin" ]
            },
            validate: {
                query: {
                    nome: Joi.string().allow('').allow(null).description('Filtra pelo nome da configuração'),
                }
            }
        }
    },

];
