const Boom = require('boom')
const db = require('../../../util/db')
const Tags = require('../../../models/Tags')
const TagsAtendimentos = require('../../../models/TagsAtendimentos')
const TagsGrupos = require('../../../models/TagsGrupos')
const TagsGruposAgentes = require('../../../models/TagsGruposAgentes')

exports.criarTag = async (request, reply) => {
    try {
        let ob = await Tags.save(request.payload)
        reply(ob)
    } catch (error) {
        console.error(error)
        reply(Boom.badData(error))
    }
}

exports.updateTag = async (request, reply) => {
    try {
        const dados = request.payload
        dados.id = request.params.id
        let ob = await Tags.save(dados)
        reply(ob)
    } catch (error) {
        console.error(error)
        reply(Boom.badData(error))
    }
}

exports.getTags = async (request, reply) => {
    try {
        const query = Tags.query
        query.where(Tags.where.notEqual({ id: 999999 }))
        const registros = await query.require(false).fetch()
        reply(registros)
    } catch (error) {
        console.error(error)
        reply(Boom.badData(error))
    }
}

exports.getTagById = async (request, reply) => {
    try {
        const query = Tags.query
        query.where({ id: request.params.id })
        const registros = await query.require(true).fetch()
        reply(registros[0])
    } catch (e) {
        reply(Boom.badData(e))
    }
}

exports.removeTagAtendimentoById = async (request, reply) => {
    try {
        const retorno = await TagsAtendimentos.query
            .require(true)
            .where({ id_tag: request.params.IdTag })
            .where({ id_atendimento: request.params.idAtendimento })
            .delete();
        reply(retorno)
    } catch (error) {
        console.error(error)
        reply(Boom.badData(error))
    }
}

exports.addTagAtendimento = async (request, reply) => {
    try {
        let ob = await TagsAtendimentos.save(request.payload)
        reply(ob)
    } catch (error) {
        console.error(error)
        reply(Boom.badData(error))
    }
}

exports.updateTagAtendimento = async (request, reply) => {
    try {
        const dados = request.payload
        dados.id = request.params.idTagAtendimento
        let ob = await TagsAtendimentos.save(dados)
        reply(ob)
    } catch (error) {
        console.error(error)
        reply(Boom.badData(error))
    }
}

exports.getTagsByIdAtendimento = async (request, reply) => {
    try {
        const query = TagsAtendimentos.query
        query.where({ id_atendimento: request.params.idAtendimento })
        const registros = await query.require(true).fetch()
        reply(registros)
    } catch (e) {
        reply(Boom.badData(e))
    }
}

exports.getGruposTagsById = async (request, reply) => {
    try {
        const grupo = await TagsGrupos.query.where({ id: request.params.idGrupo }).require(true).first().fetch()
        const agentes = await TagsGruposAgentes.query.where({ id_grupo: grupo.id }).fetch()
        grupo.agentes = agentes.map((ag) => ag.agente)
        reply(grupo)
    } catch (e) {
        reply(Boom.badData(e))
    }
}

exports.getGruposTags = async (request, reply) => {
    try {
        const grupos = await TagsGrupos.query.require(false).fetch()
        const agentes = await TagsGruposAgentes.query.require(false).fetch()
        grupos.map((g) => {
            const ags = agentes.filter((ag) => ag.id_grupo === g.id)
            g.agentes = ags.map((ag) => ag.agente)
        })
        reply(grupos)
    } catch (error) {
        console.error(error)
        reply(Boom.badData(error))
    }
}

exports.criarGrupoDeTags = async function (request, reply) {
    try {
        await db.knex.transaction(async trx => {
            const gt = (await trx('tags_grupos').insert({ nome: request.payload.nome }).returning('*'))[0]

            if (gt) {
                const agentes = request.payload.agentes.map((agente) => { return { id_grupo: gt.id, agente: agente } })
                gt.agentes = request.payload.agentes
                console.log(agentes)
                await trx.batchInsert('tags_grupos_agentes', agentes);
            }

            reply(gt)
        })
    } catch (error) {
        reply(Boom.badData(error));
    }
}

exports.updateGrupoDeTags = async function (request, reply) {
    try {
        await db.knex.transaction(async trx => {
            const atualizado = { nome: request.payload.nome }
            await trx('tags_grupos').where({ id: request.params.idGrupo }).update(atualizado)

            const agentes = request.payload.agentes.map((agente) => { return { id_grupo: request.params.idGrupo, agente: agente } })
            await trx('tags_grupos_agentes').where({ id_grupo: request.params.idGrupo }).del()
            await trx.batchInsert('tags_grupos_agentes', agentes)

            atualizado.agentes = request.payload.agentes
            atualizado.id = request.params.idGrupo
            reply(atualizado)
        })
    } catch (error) {
        reply(Boom.badData(error))
    }
}

exports.addAgenteNoGrupo = async (request, reply) => {
    try {
        let ob = await TagsGruposAgentes.save(request.payload)
        reply(ob)
    } catch (error) {
        console.error(error)
        reply(Boom.badData(error))
    }
}

exports.deleteGrupoTags = async function (request, reply) {
    try {
        await db.knex.transaction(async trx => {
            await trx('tags_grupos_agentes').where('id_grupo', request.params.idGrupo).del()
            await trx('tags_grupos').where('id', request.params.idGrupo).del()

            reply(`Grupo com o id ${request.params.idGrupo} deletado com sucesso!`)
        })
    } catch (error) {
        reply(Boom.badData(error))
    }
}

exports.deleteTag = async function (request, reply) {
    try {
        await db.knex.transaction(async trx => {
            await trx('tags').where('id', request.params.idTag).del()

            reply(`Tag com o id ${request.params.idGrupo} deletado com sucesso!`)
        })
    } catch (error) {
        reply(Boom.badData(error))
    }
}

