const Joi = require('joi');
const handlers = require('./handlers');

module.exports = [
    {
        method: 'POST',
        path: '/api/v4/tags',
        handler: handlers.criarTag,
        config: {
            description: 'Crie uma tag',
            tags: ['api', 'Tags'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                payload: Joi.object({
                    nome: Joi.string().required(),
                    id_grupo: Joi.number().optional().allow(null).description('Id do grupo de tags'),
                    cor: Joi.string().required()
                })
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v4/tags/{id}',
        handler: handlers.updateTag,
        config: {
            description: 'Atualiza uma Tag',
            tags: ['api', 'Tags'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                },
                payload: Joi.object({
                    nome: Joi.string().required(),
                    id_grupo: Joi.number().optional().allow(null).description('Id do grupo de tags'),
                    cor: Joi.string().required()
                })
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/tags/{id}',
        handler: handlers.getTagById,
        config: {
            description: 'Obtem uma Tag',
            tags: ['api', 'Tags'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/tags',
        handler: handlers.getTags,
        config: {
            description: 'Obtem todas as tags',
            tags: ['api', 'Tags'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            }
        }
    },
    {
        method: 'POST',
        path: '/api/v4/tags/atendimento',
        handler: handlers.addTagAtendimento,
        config: {
            description: 'Add uma tag em um atendimento',
            tags: ['api', 'Tags'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                payload: Joi.object({
                    id_tag: Joi.number().required().description('Id da tag'),
                    id_atendimento: Joi.string().required().description('Id do atendimento que foi tageado'),
                    agente: Joi.string().required().description('Fullname do agente que tageou'),
                    tipo_atendimento: Joi.string().required().description('Tipo do atenndimento voz | texto')                   
                })
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v4/tags/atendimento/{idTagAtendimento}',
        handler: handlers.updateTagAtendimento,
        config: {
            description: 'Add uma tag em um atendimento',
            tags: ['api', 'Tags'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    idTagAtendimento: Joi.number().integer().required()
                },
                payload: Joi.object({
                    id_tag: Joi.number().optional().description('Id da tag'),
                    id_atendimento: Joi.string().optional().description('Id do atendimento que foi tageado'),
                    agente: Joi.string().optional().description('Fullname do agente que tageou'),
                    tipo_atendimento: Joi.string().optional().description('Tipo do atenndimento voz | texto')                  
                })
            }
        }
    },
    {
        method: 'DELETE',
        path: '/api/v4/tags/atendimento/{IdTag}/{idAtendimento}',
        handler: handlers.removeTagAtendimentoById,
        config: {
            description: 'Add uma tag em um atendimento',
            tags: ['api', 'Tags'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    IdTag: Joi.number().integer().required(),
                    idAtendimento: Joi.string().required()
                }
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/tags/atendimento/{idAtendimento}',
        handler: handlers.getTagsByIdAtendimento,
        config: {
            description: 'Obtem as tags adicionados no atendimento',
            tags: ['api', 'Tags'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    idAtendimento: Joi.string().required()
                }
            }
        }
    },
    {
        method: 'POST',
        path: '/api/v4/tags/grupos',
        handler: handlers.criarGrupoDeTags,
        config: {
            description: 'Add uma grupo de tags',
            tags: ['api', 'Tags'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                payload: Joi.object({
                    nome: Joi.string().required().description('Nome do Grupo'),
                    agentes: Joi.array().items(Joi.string()).example(['agente1', 'agente2']).required().description('Todas os agentes que esse grupo terá'),
                })
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v4/tags/grupos/{idGrupo}',
        handler: handlers.updateGrupoDeTags,
        config: {
            description: 'Edita um grupo de tags',
            tags: ['api', 'Tags'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    idGrupo: Joi.string().required()
                },
                payload: Joi.object({
                    nome: Joi.string().required().description('Nome do Grupo'),
                    agentes: Joi.array().items(Joi.string()).example(['agente1', 'agente2']).required().description('Todas os agentes que esse grupo terá'),
                })
            }
        }
    },
    {
        method: 'POST',
        path: '/api/v4/tags/grupos/addAgente',
        handler: handlers.addAgenteNoGrupo,
        config: {
            description: 'Add uma grupo de tags',
            tags: ['api', 'Tags'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                payload: Joi.object({
                    id_grupo: Joi.number().required().description('id grupo de tags'),
                    agente: Joi.string().required().description('Fullname do agente')
                })
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/tags/grupos/{idGrupo}',
        handler: handlers.getGruposTagsById,
        config: {
            description: 'Obtem as tags adicionados no atendimento',
            tags: ['api', 'Tags'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    idGrupo: Joi.string().required()
                }
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/tags/grupos',
        handler: handlers.getGruposTags,
        config: {
            description: 'Obtem as tags adicionados no atendimento',
            tags: ['api', 'Tags'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            }
        }
    },
    {
        method: 'DELETE',
        path: '/api/v4/tags/grupos/{idGrupo}',
        handler: handlers.deleteGrupoTags,
        config: {
            description: 'Deleta o grupo de tags',
            tags: ['api', 'Tags'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    idGrupo: Joi.string().required()
                }
            }
        }
    },
    {
        method: 'DELETE',
        path: '/api/v4/tags/{idTag}',
        handler: handlers.deleteTag,
        config: {
            description: 'Deleta uma tag',
            tags: ['api', 'Tags'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    idTag: Joi.string().required()
                }
            }
        }
    }
]