const db = require('../../../util/db');
const Ami = require('../../../ami');
const Boom = require('boom');
const Tronco = require('../../../models/Tronco');

exports.getAll = async function (request, reply) {
    try {
        const ret = await Tronco.fetch();
        for (let tronco of ret) {
            let peer = await Ami.SipShowPeer(tronco.nome);
            tronco.status = peer.status;
            tronco.ip = peer.address_ip;
        }
        reply(ret);
    } catch (error) {
        reply(Boom.badData(error));
    }
};

exports.getOne = async function (request, reply) {
    try {
        const ret = await Tronco.fetchById(request.params.id);
        ret.conf = await Ami.getConfig({ arquivo: 'sip_trunk.conf', categoria: ret.nome });
        if (ret.conf) {
            //ret.conf = ret.conf[ret.nome];
            ret.conf = Object.entries(ret.conf[ret.nome]).map(conf => {
                return {
                    parametro: conf[0],
                    valor: conf[1]
                }
            })
        }
        reply(ret);
    } catch (error) {
        reply(Boom.badData(error));
    }
};

exports.update = async function (request, reply) {
    const transaction = new db.orm.Transaction();
    try {
        const TroncoT = transaction.models.Tronco;
        const tronco = await TroncoT.fetchById(request.params.id);
        let up = JSON.parse(JSON.stringify(request.payload));
        delete up.conf;
        up.id = request.params.id;
        let ret = await TroncoT.update(up);

        const ami = await Ami.updateTroncoConfig({
            tronco: request.payload.nome,
            registro_novo: request.payload.registro,
            registro_antigo: tronco.registro,
            dados: request.payload.conf
        });
        ret[0].ami = ami;
        await transaction.commit();
        reply(ret[0]);
    } catch (error) {
        await transaction.rollback();
        reply(Boom.badData(error));
    }
};

exports.create = async function (request, reply) {
    const transaction = new db.orm.Transaction();
    try {
        const TroncoT = transaction.models.Tronco;
        let up = JSON.parse(JSON.stringify(request.payload));
        delete up.conf;
        let ret = await TroncoT.save(up);

        const ami = await Ami.updateTroncoConfig({
            tronco: request.payload.nome,
            registro_novo: request.payload.registro,
            dados: request.payload.conf,
            isNew: true
        });
        ret.ami = ami;
        await transaction.commit();
        reply(ret);
    } catch (error) {
        await transaction.rollback();
        reply(Boom.badData(error));
    }
};

exports.delete = async function (request, reply) {
    const transaction = new db.orm.Transaction();
    try {
        const TroncoT = transaction.models.Tronco;
        const tronco = await TroncoT.fetchById(request.params.id);

        await TroncoT.query
            .require(false)
            .where({ id: request.params.id })
            .delete();

        const ami = await Ami.deleteTroncoConfig({
            tronco: tronco.nome,
            registro: tronco.registro,
        });
        tronco.ami = ami;
        await transaction.commit();
        reply(tronco);
    } catch (error) {
        await transaction.rollback();
        reply(Boom.badData(error));
    }
};