const Joi = require('joi');
const handlers = require('./handlers');

module.exports = [
    {
        method: 'GET',
        path: '/api/v4/troncos',
        handler: handlers.getAll,
        config: {
            description: 'Retorna os troncos cadastrados',
            tags: ['api', 'Troncos'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.troncos']
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/tronco/{id}',
        handler: handlers.getOne,
        config: {
            description: 'Retorna os troncos cadastrados',
            tags: ['api', 'Troncos'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.troncos']
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v4/tronco/{id}',
        handler: handlers.update,
        config: {
            description: 'Atualiza um troncos',
            tags: ['api', 'Troncos'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.troncos']
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                },
                payload: Joi.object({
                    conf: Joi.array().items(
                        Joi.object({
                            parametro: Joi.string().required(),
                            valor: Joi.string().required()
                        })
                    ).required(),
                    nome: Joi.string().required(),
                    rota: Joi.string().required(),
                    registro: Joi.string().optional().allow([null, '']),
                })
            }
        }
    },
    {
        method: 'POST',
        path: '/api/v4/tronco',
        handler: handlers.create,
        config: {
            description: 'Cria um troncos',
            tags: ['api', 'Troncos'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.troncos']
            },
            validate: {
                payload: Joi.object({
                    conf: Joi.array().items(
                        Joi.object({
                            parametro: Joi.string().required(),
                            valor: Joi.string().required()
                        })
                    ).required(),
                    nome: Joi.string().required(),
                    rota: Joi.string().required(),
                    registro: Joi.string().optional().allow([null, '']),
                })
            }
        }
    },
    {
        method: 'DELETE',
        path: '/api/v4/tronco/{id}',
        handler: handlers.delete,
        config: {
            description: 'Remove um tronco',
            tags: ['api', 'Troncos'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.troncos']
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                }
            }
        }
    },
];
