const Joi = require('joi');
const handlers = require('./handlers');

const fluxoCampos = Joi.object({
    callid: Joi.string().required().description('Callis da ligação'),
    calldate: Joi.date().iso().optional().allow('').allow(null).description('Data e hora'),
    src: Joi.string().optional().allow('').allow(null).description('Numero de origem'),
    dst: Joi.string().optional().allow('').allow(null).description('Numero de destino'),
    context: Joi.string().optional().allow('').allow(null).description('Contexto'),
    menu: Joi.string().optional().allow('').allow(null).description('Menu'),
    id_client: Joi.string().optional().allow('').allow(null).description('Id client'),
    opcao: Joi.string().optional().allow('').allow(null).description('Opção'),
    src_menu: Joi.string().optional().allow('').allow(null).description('Menu de origem'),
    data_a: Joi.string().optional().allow('').allow(null).description('Dado adicional 1'),
    data_b: Joi.string().optional().allow('').allow(null).description('Dado adicional 2')
});

module.exports = [
    {
        method: 'GET',
        path: '/api/v4/ura_fluxo',
        handler: handlers.getAll,
        config: {
            description: 'Obtem os registros da URA_FLUXO',
            tags: ['api', 'v4', 'Ura Fluxo'],
            auth: {
                strategy: 'bearer',
                scope: ['app.ligacoesura']
            },
            validate: {
                query: {
                    data_ini: Joi.date().raw().optional().allow(null).description('Data inicial'),
                    data_fim: Joi.date().raw().optional().allow(null).description('Data Final'),
                    hora_ini: Joi.string().optional().allow(null).description('Hora inicial'),
                    hora_fim: Joi.string().optional().allow(null).description('Hora final')
                }
            }
        }
    },
    {
        method: 'POST',
        path: '/api/v4/ura_fluxo',
        handler: handlers.create,
        config: {
            description: 'Insere um registro no URA_FLUXO',
            tags: ['api', 'v4', 'Ura Fluxo'],
            auth: {
                strategy: 'bearer',
                scope: ['app.ligacoesura'] // or [ 'user', 'admin' ]
            },
            validate: {
                payload: fluxoCampos
            }
        }
    },
];
