const Boom = require('boom');
const UraTexto = require('../../../models/uraTexto');
const UraBlocos = require('../../../models/uraTextoBlocos');

exports.getAll = async (request, reply) => {
    try {
        reply(await UraTexto.fetch());
    } catch (error) {
        reply(Boom.badData(error));
    }
};

exports.getOne = async (request, reply) => {
    try {
        reply(await UraTexto.fetchById(request.params.ID));
    } catch (error) {
        reply(Boom.badData(error));
    }
};

exports.create = async (request, reply) => {
    try {
        reply(await new UraTexto(request.payload).save());
    } catch (error) {
        reply(Boom.badData(error));
    }
};

exports.update = async (request, reply) => {
    try {
        const ura = request.payload;
        ura.id = request.params.ID;
        reply(await new UraTexto(ura).save());
    } catch (error) {
        reply(Boom.badData(error));
    }
};

exports.delete = async (request, reply) => {
    try {
        reply(await UraTexto.query
            .where({ id: request.params.ID })
            .require(true)
            .delete());
    } catch (error) {
        reply(Boom.badData(error));
    }
};

exports.getBlocos = async (request, reply) => {
    try {
        reply(await UraBlocos.query
            .where({ uraTextoID: request.params.uraTextoID })
            .require(true)
            .fetch());

    } catch (error) {
        reply(Boom.badData(error));
    }
};

exports.getBloco = async (request, reply) => {
    try {
        reply(await UraBlocos.query
            .where({ uraTextoID: request.params.uraTextoID })
            .where({ id: request.params.ID })
            .first()
            .require(true)
            .fetch());

    } catch (error) {
        reply(Boom.badData(error));
    }
};

exports.delBloco = async (request, reply) => {
    try {
        reply(await UraBlocos.query
            .where({ uraTextoID: request.params.uraTextoID })
            .where({ id: request.params.ID })
            .require(true)
            .delete());

    } catch (error) {
        reply(Boom.badData(error));
    }
};

exports.upBloco = async (request, reply) => {
    try {
        const bloco = request.payload;
        bloco.id = request.params.ID;
        bloco.uraTextoID = request.params.uraTextoID;
        reply(await new UraBlocos(bloco).save());
    } catch (error) {
        reply(Boom.badData(error));
    }
};
