const Boom = require('boom');
const Supervisores = require('../../../models/Supervisores');
const SupervisoresModulos = require('../../../models/SupervisoresModulos');
const SupervisoresFilas = require('../../../models/SupervisoresFilas');
const SupervisoresGruposFila = require('../../../models/SupervisoresGruposFila');
const AutenticacaoFontes = require('../../../models/AutenticacaoFontes');
const Modulos = require('../../../models/Modulos');
const db = require('../../../util/db');
const md5 = require('md5');

exports.create = async (request, reply) => {
    try {
        if (request.payload.senha) request.payload.senha = md5(request.payload.senha);
        let ret = await Supervisores.save(request.payload);
        reply(ret);
    } catch (error) {
        reply(Boom.badData(error));
    }
}

exports.update = async (request, reply) => {
    try {
        let dados = request.payload;
        dados.id = request.params.id;
        if (dados.senha) dados.senha = md5(dados.senha);
        let ret = await Supervisores.update(dados);
        reply(ret);
    } catch (error) {
        reply(Boom.badData(error));
    }
}

exports.getOne = async (request, reply) => {
    try {
        const query = Supervisores.query;
        query.fields(['id', 'nome', 'senha', 'fullname', 'token', 'token_create', 'status', 'autenticacao_fonte', 'usa_grupo_fila']);
        query.where({ id: request.params.id });
        query.leftJoin(AutenticacaoFontes, { fields: ['id', 'nome', 'tipo'] });
        var registros = await query.require(true).fetch();
        reply(registros[0]);
    } catch (e) {
        reply(Boom.badData(e));
    }
};

exports.getPermicoes = async (request, reply) => {
    try {
        let supModulos = await SupervisoresModulos.query
            .fields(['id_modulo', 'extra_usuario'])
            .innerJoin(Modulos, { fields: ['nome', 'descricao'] })
            .require(false)
            .where({ id_supervisor: request.params.id })
            .fetch();

        supModulos = supModulos.map(item => {
            item.modulo = item.modulos[0].nome ? item.modulos[0].nome : item.modulos[0].descricao;
            delete item.modulos;
            return item;
        })


        let supgrupos_fila = await SupervisoresGruposFila.query
            .where({ id_supervisor: request.params.id })
            .fields(['fila_grupo'])
            .require(false)
            .fetch();
        if (supgrupos_fila) {
            supgrupos_fila = supgrupos_fila.map(f => {
                return f.fila_grupo;
            })
        }

        let supfilas = await SupervisoresFilas.query
            .where({ id_supervisor: request.params.id })
            .fields(['id_fila'])
            .require(false)
            .fetch();
        if (supfilas) {
            supfilas = supfilas.map(f => {
                return f.id_fila;
            })
        }

        let sup_usa_grupo_fila = await Supervisores.query
            .where({ id: request.params.id })
            .fields(['usa_grupo_fila'])
            .first()
            .fetch()

        const permicoes = {
            usa_grupo_fila: sup_usa_grupo_fila.usa_grupo_fila,
            modulos: supModulos,
            filas: supfilas,
            grupos_fila: supgrupos_fila
        }

        reply(permicoes);
    } catch (e) {
        reply(Boom.badData(e));
    }
};

exports.setPermicoes = async (request, reply) => {
    const transaction = new db.orm.Transaction();

    try {
        let ret = {};
        let dados;
        let filas;
        let gruposFila;

        if (request.payload.modulos.length > 0) {
            dados = request.payload.modulos.map(modulo => {
                modulo.id_supervisor = request.params.id;
                return modulo;
            });
        }

        if (request.payload.grupos_fila.length > 0) {
            gruposFila = request.payload.grupos_fila.map(grupoF => {
                return {
                    id_supervisor: request.params.id,
                    fila_grupo: grupoF
                };
            });
        }

        if (request.payload.filas.length > 0) {
            filas = request.payload.filas.map(fila => {
                return {
                    id_supervisor: request.params.id,
                    id_fila: fila
                };
            });
        }

        const SupervisoresT = transaction.models.Supervisores;
        const SupervisoresModulosT = transaction.models.SupervisoresModulos;
        const SupervisoresGruposFilaT = transaction.models.SupervisoresGruposFila;
        const SupervisoresFilasT = transaction.models.SupervisoresFilas;

        await SupervisoresT.query
            .update({ id: request.params.id, usa_grupo_fila: request.payload.usa_grupo_fila })

        await SupervisoresModulosT.query
            .require(false)
            .where({ id_supervisor: request.params.id })
            .delete();

        if (request.payload.modulos.length > 0) {
            ret.modulos = await SupervisoresModulosT.insert(dados);
        }

        await SupervisoresGruposFilaT.query
            .require(false)
            .where({ id_supervisor: request.params.id })
            .delete();

        if (request.payload.grupos_fila.length > 0) {
            ret.gruposFila = await SupervisoresGruposFilaT.insert(gruposFila);
        }

        await SupervisoresFilasT.query
            .require(false)
            .where({ id_supervisor: request.params.id })
            .delete();

        if (request.payload.filas.length > 0) {
            ret.filas = await SupervisoresFilasT.insert(filas);
        }

        await transaction.commit();

        reply(ret);
    } catch (error) {
        await transaction.rollback();
        reply(Boom.badData(error));
    }
}

exports.getAll = async (request, reply) => {
    try {
        const query = Supervisores.query;
        query.fields(['id', 'nome', 'senha', 'fullname', 'token', 'token_create', 'status', 'autenticacao_fonte', 'usa_grupo_fila']);
        query.leftJoin(AutenticacaoFontes, { fields: ['id', 'nome', 'tipo'] });
        var registros = await query.require(false).fetch();
        reply(registros);
    } catch (e) {
        reply(Boom.badData(e));
    }
};

exports.delete = async (request, reply) => {
    try {
        const query = Supervisores.query;
        query.where({ id: request.params.id });
        var registros = await query.require(true).delete();
        reply(registros);
    } catch (e) {
        reply(Boom.badData(e));
    }
};