const Boom = require('boom');
const config = require('../../config.json');
const log = require('simple-node-logger').createSimpleLogger(config.log_ops);
log.setLevel(config.log ? config.log : 'error');
let WhatsApi = require('../../xc_modules/whatsapi');
let Gup = require('../../xc_modules/gupshup');
const Cliente = require('../../models/Cliente');
const ClienteCamposAdicionais = require('../../models/ClienteCamposAdicionais');
const ClienteAtendimento = require('../../models/ClienteAtendimento');
const AtendimentoMensagem = require('../../models/AtendimentoMensagem');
const QueueTable = require('../../models/Fila')
const Ami = require('../../ami/index');
const ws = require('../../ws/index');
const uuid = require(`uuid`)
const toStream = require('buffer-to-stream');
const mime = require('mime-to-extensions');
const Omni = require('../../xc_modules/omni/index');
WhatsApi = new WhatsApi(config.whatsApi.host, config.whatsApi.clientID);
if (config.whatsApi.ativado && config.whatsApi.clientID !== 'SEM') {
    WhatsApi.connect();
}
Gup = new Gup(config.gupshup.host, config.gupshup.clientID);

if (config.gupshup.ativado && config.gupshup !== 'SEM') {
    Gup.connect();
}
const sendMessage = (instanceID, chatId, msg) => {
    return new Promise((resolve, reject) => {
        try {
            WhatsApi.socket.emit('newWhatsMsg', { instanceID: instanceID, chatId: chatId, msg: msg }, (retorno) => {
                console.log(`WhatsApp para ${chatId} VIA API REST Retornou:`);
                resolve(retorno);
            });
        } catch (error) {
            reject({ error: true, msg: error });
        }

    })
}
const sendMessageGup = (apikey, appName, source, instanceID, chatId, msg, template) => {
    return new Promise((resolve, reject) => {
        try {
            if (!template || Object.values(template).length == 0) {
                console.log(`Entrou==================`);
                Gup.socket.emit('newWhatsMsgGupshup', { apikey: apikey, source: source, to: chatId, text: msg }, (retorno) => {
                });

                resolve({ error: false, msg: 'Mensagem enviada' });
            } else {
                Gup.socket.emit('newWhatsMsgGupshup', { apikey: apikey, appName: appName, source: source, to: chatId, text: msg, template: { id: template.templateId, fields: template.fields } }, (retorno) => {
                    console.log(`GupShup para ${chatId} VIA API REST Retornou:`);

                });
                resolve({ error: false, msg: 'Template enviado' });
            }

        } catch (error) {
            console.error(error);
            reject({ error: true, msg: error });
        }

    })
}
exports.sendMessageFileGup = (request, reply) => {
    try {
        let dados = request.payload;
        var file = request.payload.file;
        dados.caption = request.payload.legenda ? request.payload.legenda : file.hapi.filename;
        dados.file_name = file.hapi.filename;
        var type = dados.file_name.split('.');

        dados.mimeType = mime.lookup(type[1]);
        dados.mimetype = mime.lookup(type[1]);
        dados.Upload = 'start';
        dados.legenda = request.payload.legenda ? request.payload.legenda : file.hapi.filename;
        dados.uid = uuid();


        Gup.socket.emit('sendMsgFileGupshup', { apikey: request.payload.apiKey, source: request.payload.source, instanceID: dados.fila, chatId: dados.destino, dados: dados });

        var stream = toStream(file._data)
        stream.on('readable', () => {
            let chunk
            while ((chunk = stream.read()) !== null) {
                dados.Upload = 'sending';
                Gup.socket.emit('sendMsgFileGupshup', { apikey: request.payload.apiKey, source: request.payload.source, instanceID: dados.fila, chatId: dados.destino, dados: dados }, null, chunk);
            }
        })
        stream.on('end', async () => {
            dados.Upload = 'end';
            Gup.socket.emit('sendMsgFileGupshup', { apikey: request.payload.apiKey, source: request.payload.source, instanceID: dados.fila, to: dados.destino, dados: dados }, async (retorno) => {
                try {
                    if(request.payload.protocolo) {
                        saveMsgAux(request, retorno)
                    }
                    reply(retorno);
                } catch (error) {
                    reply(Boom.badData('ERRO AO UPAR ANEXO'));

                }
            });
        });
        stream.on('error', (err) => {
            reply(Boom.badData('Erro ao ler o arquivo'));
        })
    } catch (error) {
        console.error(error);
        reply(Boom.badData('erro'));
    }
}
const saveAnexo = async (msg, file) => {
    try {       
        let anexo = await new MensagemAnexos({ idMensagem: msg.id, caminho: file.caminho, mimeType: file.mimeType }).save();
    } catch (error) {
        log.error('erro ao salvar msg', error);
    }

};

exports.sendMessageFile = (request, reply) => {
    try {
        let dados = request.payload;
        var file = request.payload.file;
        dados.caption = request.payload.legenda ? request.payload.legenda : file.hapi.filename;
        dados.file_name = file.hapi.filename;
        var type = dados.file_name.split('.');

        dados.mimeType = mime.lookup(type[1]);
        dados.mimetype = mime.lookup(type[1]);
        dados.Upload = 'start';
        dados.legenda = request.payload.legenda ? request.payload.legenda : file.hapi.filename;
        dados.uid = uuid();


        WhatsApi.socket.emit('sendMsgFile', { instanceID: dados.fila, chatId: dados.destino, dados: dados });

        var stream = toStream(file._data)
        stream.on('readable', () => {
            let chunk
            while ((chunk = stream.read()) !== null) {
                dados.Upload = 'sending';
                WhatsApi.socket.emit('sendMsgFile', { instanceID: dados.fila, chatId: dados.destino, dados: dados }, null, chunk);
            }
        })
        stream.on('end', async () => {
            dados.Upload = 'end';
            WhatsApi.socket.emit('sendMsgFile', { instanceID: dados.fila, chatId: dados.destino, dados: dados }, async (retorno) => {
                try {
                    reply(retorno);
                } catch (error) {
                    reply(Boom.badData('ERRO AO UPAR ANEXO'));

                }
            });
        });
        stream.on('error', (err) => {
            reply(Boom.badData('Erro ao ler o arquivo'));
        })
    } catch (error) {
        console.error(error);
        reply(Boom.badData('erro'));
    }
}
async function saveMsgAux (request, retorno) {
    try {
        const atendimento = await ClienteAtendimento.query.require(false).first()
            .where({ id: request.payload.protocolo })           
            .fetch();
            const m = await new AtendimentoMensagem({ idAtendimento: request.payload.protocolo, texto: request.payload.mensagem, inOut: 2, agente: atendimento.ultimoAgente, status: retorno.error ? `pending` : `seen` }).save()
    } catch (error) {
        console.error(error)
    }
}
exports.send = async (request, reply) => {
    try {
        log.info(`Enviando WhatsApp para ${request.payload.destino} VIA API REST:`);
        const retorno = await sendMessage(request.payload.fila, request.payload.destino, request.payload.mensagem);
        console.log('RETORNOU', retorno)
        if(request.payload.protocolo) {
          saveMsgAux(request, retorno)
        }
        if (retorno.error) {
            reply(Boom.badData(retorno.msg));
        }
        else {
            reply(retorno);
        }
    } catch (error) {
        console.error(error)
        reply(Boom.badData(error));

    }

}
exports.criarAtendimento = async (request, reply) => {
    try {
        const fila = await QueueTable.fetchByName(request.payload.fila)
        let cliente = await Cliente.query.execute({ text: 'select * from find_cliente($1)', values: [request.payload.numero] });
        cliente = cliente[0];
        if (!cliente) {
            cliente = await new Cliente({ numero: request.payload.numero, nome: request.payload.nome }).save();
            console.log('TESTE', cliente)
            cliente = cliente[0]
        }
        const atendimento = await new ClienteAtendimento({ fromID: request.payload.numero, fromName: cliente.nome, queue: request.payload.fila, queueFrom: request.payload.instancia, idCliente: cliente.id, ativo: true, status: 'Entrou', sentido:'Sainte', queueTypeID:fila.queue_type, action: 'WhatsAppIn', dadosAdicionais: request.payload.dados }).save();
        reply(atendimento)
    } catch (error) {
        console.error(error)
        reply(Boom.badData(error))
    }
}
exports.sendGup = async (request, reply) => {
    try {
        log.info(`Enviando WhatsApp para VIA API REST:`, request.payload);
        const retorno = await sendMessageGup(request.payload.apiKey, request.payload.appName, request.payload.source, request.payload.fila, request.payload.destino, request.payload.mensagem, request.payload.template);
        if(request.payload.protocolo) {
          saveMsgAux(request, retorno)
        }
        if (retorno.error) {
            reply(Boom.badData(retorno.msg));
        }
        else {
            reply(retorno);
        }
    } catch (error) {
        console.error(error);
        reply(Boom.badData(error));

    }

}

exports.sendtoqueue = async (request, reply) => {
    try {
        const fila = Ami.getFila(request.payload.fila);
        console.log('FILA', request.payload.fila)
        var mensagem = request.payload.mensagem;
        var cliente = await Cliente.query.execute({ text: 'select * from find_cliente($1)', values: [request.payload.destino] });
        cliente = cliente[0];
        if (!cliente) {
            cliente = await new Cliente({ numero: request.payload.destino, nome: request.payload.nome }).save();
            console.log('TESTE', cliente)
            cliente = cliente[0]
        }
        console.log('CLEINTE', cliente)
        let dados = request.payload.dados;
        console.log('DADOS PASSADOS', dados)
        for (var name in dados) {
            let verifica = await ClienteCamposAdicionais.query.require(false).first()
                .where({ nome: name, clienteID: cliente.id }).fetch();
            //vamos atualizar os que ja tem
            if (verifica) {
                verifica.valor = dados[name];
                verifica.save();
            }
            //Vamos adicionar caso nao esteja no verifica
            if (!verifica) {
                //vamos verificar se e tipo telefone ou tipo texto
                let campo = {};
                let valor = dados[name];
                campo.nome = name;
                campo.valor = valor;
                campo.clienteID = cliente.id;
                console.log(`CAMPO`, campo);
                new ClienteCamposAdicionais(campo).save()
            }
        }
        console.log('CLIENTE ++++++', cliente.nome)
        var Ativo = await ClienteAtendimento.query.require(false).first()
            .where({ idCliente: cliente.id })
            .where(ClienteAtendimento.where.or(
                ClienteAtendimento.where.equal({ queue: request.payload.instancia }),
                ClienteAtendimento.where.equal({ queueFrom: request.payload.instancia })
            ))
            .where({ status: 'Entrou' })
            .fetch();
        var atendimento = Ativo ? Ativo : await new ClienteAtendimento({ fromID: request.payload.destino, fromName: cliente.nome, queue: request.payload.fila, queueFrom: request.payload.instancia, idCliente: cliente.id, ativo: true, status: 'Entrou', queueTypeID: 3, action: 'WhatsAppIn', dadosAdicionais: request.payload.dados }).save();

        if (!Ativo) {
            Omni.addAguardando(fila, atendimento)
            console.log('nao estava ativo');
            let retorno = await sendMessage(request.payload.instancia, request.payload.destino, mensagem);
            var m = await new AtendimentoMensagem({ idAtendimento: atendimento.id, texto: mensagem, inOut: 3, agente: null, status: retorno.error ? `pending` : `seen` }).save();
            if (retorno.error) {
                reply(Boom.badData(retorno.errorDetails));
                return
            }
        }
        else {
            if (request.payload.notificacao) {
                mensagem = request.payload.notificacao;
                let retorno = await sendMessage(request.payload.instancia, request.payload.destino, mensagem);
                console.log('ja estava ativo', retorno)

                var m = await new AtendimentoMensagem({ idAtendimento: atendimento.id, texto: mensagem, inOut: 3, agente: null, status: retorno.error ? `pending` : `seen` }).save();
                if (retorno.error) {
                    reply(Boom.badData(retorno.errorDetails));
                    return
                }
            }
        }
        // Salva Mensagens enviada para o cliente.
        console.log(' passou aqui')

        reply({ error: false, msg: 'ok' });
    } catch (error) {
        console.log(' deu erro', error)
        reply(Boom.badData(error));
    }

};
exports.sendtoagent = async (request, reply) => {
    try {
        const fila = Ami.getFila(request.payload.fila);
        var mensagem = request.payload.mensagem;
        var num = request.payload.destino.substr(-8);
        var cliente = await Cliente.query.require(false).first()
            .where(Cliente.where.like({ numero: `%${num}` }))
            .fetch();
        if (!cliente) {
            cliente = await new Cliente({ numero: request.payload.destino, nome: 'Cliente Ura' }).save();
        }
        var Ativo = await ClienteAtendimento.query.require(false).first()
            .where({ idCliente: cliente.id })
            .where(ClienteAtendimento.where.or(
                ClienteAtendimento.where.equal({ queue: request.payload.fila }),
                ClienteAtendimento.where.equal({ queueFrom: request.payload.fila })
            ))
            .where({ status: 'Entrou' })
            .fetch();
        var atendimento = Ativo ? Ativo : await new ClienteAtendimento({ fromID: request.payload.destino, fromName: cliente.nome, queue: fila.nome, idCliente: cliente.id, ativo: true, status: 'Entrou', queueTypeID: 3, action: 'WhatsAppIn', dadosAdicionais: request.payload.dados, sentido: 'Sainte' }).save();
        atendimento.ultimoAgente = request.payload.agente;
        atendimento.save();
        //mensagem = request.payload.notificacao;
        let retorno = await sendMessage(request.payload.fila, request.payload.destino, mensagem);
        console.log('ja estava ativo', retorno)

        var m = await new AtendimentoMensagem({ idAtendimento: atendimento.id, texto: mensagem, inOut: 3, agente: atendimento.ultimoAgente, status: retorno.error ? `pending` : `seen` }).save();
        if (retorno.error) {
            reply(Boom.badData(retorno.msg));
        }


        // Salva Mensagens enviada para o cliente.
        //console.log(' passou aqui')
        ws.wsEvento.emit(`sendToAgent`, atendimento);
        reply({ error: false, atendimento: atendimento });
    } catch (error) {
        console.log(' deu erro', error)
        reply(Boom.badData(error));
    }

};

