'use strict'
const Joi = require('joi');
const handlers = require('./handlers');


module.exports = [
  {
    method: 'POST',
    path: '/api/v2/whatsapp/send',
    handler: handlers.send,
    config: {
      description: 'Apenas envia uma mensagem de Texto via WhatsApp para um destino, não cria atendimento.',
      tags: ['api', 'v2', 'WhatsApp'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        payload: {
          fila: Joi.string().required().description('Nome da fila do Xcontact vinculada a conta de WhatsApp').example('FilaA'),
          destino: Joi.string().required().description('Número de destino no formato COD_PAIS COD_REGIAO NUMERO. Ex: 555130305501').example('555130305501'),
          mensagem: Joi.string().required().description('Mensagem').example('Oi Xcontact'),
          dados: Joi.object().unknown(),
          protocolo:Joi.number().description('id do atendimento').example('123')
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v2/whatsapp/sendGup',
    handler: handlers.sendGup,
    config: {
      description: 'Apenas envia uma mensagem de Texto via WhatsApp para um destino, não cria atendimento.',
      tags: ['api', 'v2', 'WhatsApp'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        payload: {
          apiKey: Joi.string().required().description('Chave de envio gupshup'),
          source: Joi.string().required().description('Numero de whats registrado na gupshup'),
          fila: Joi.string().required().description('Nome da fila do Xcontact vinculada a conta de WhatsApp').example('FilaA'),
          destino: Joi.string().required().description('Número de destino no formato COD_PAIS COD_REGIAO NUMERO. Ex: 555130305501').example('555130305501'),
          mensagem: Joi.string().required().description('Mensagem').example('Oi Xcontact'),
          dados: Joi.object().unknown(),
          template: Joi.object().unknown().optional(),
          appName: Joi.string().optional().description('nome do app na gupshup'),
          protocolo:Joi.number().description('id do atendimento').example('123')
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v2/whatsapp/sendFileGup',
    handler: handlers.sendMessageFileGup,
    config: {
      description: 'Apenas envia uma mensagem de Texto via WhatsApp para um destino, não cria atendimento.',
      tags: ['api', 'v2', 'WhatsApp'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      plugins: {
        'hapi-swagger': {
          payloadType: 'form'
        }
      },
      payload: {
        output: 'stream',
        allow: 'multipart/form-data',
        maxBytes: 300971520, // 300mb
        parse: true,
      },
      validate: {
        payload: Joi.object({
          fila: Joi.string().required().description('Nome da fila do Xcontact vinculada a conta de WhatsApp').example('FilaA'),
          destino: Joi.string().required().example('55511212').description('Destino, ID ou Número'),
          legenda: Joi.string().allow(null).description('Legenda/nome para o arquivo'),
          apiKey: Joi.string().required().description('Chave de envio gupshup'),
          source: Joi.string().required().description('Numero de whats registrado na gupshup'),
          file: Joi.any().meta({ swaggerType: 'file' }).required().description('Arquivo'),
          protocolo:Joi.number().description('id do atendimento').example('123')
        }).label('SendFileObject')
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v2/whatsapp/sendFile',
    handler: handlers.sendMessageFile,
    config: {
      description: 'Apenas envia uma mensagem de Texto via WhatsApp para um destino, não cria atendimento.',
      tags: ['api', 'v2', 'WhatsApp'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      plugins: {
        'hapi-swagger': {
          payloadType: 'form'
        }
      },
      payload: {
        output: 'stream',
        allow: 'multipart/form-data',
        maxBytes: 300971520, // 300mb
        parse: true,
      },
      validate: {
        payload: Joi.object({
          fila: Joi.string().required().description('Nome da fila do Xcontact vinculada a conta de WhatsApp').example('FilaA'),
          destino: Joi.string().required().example('55511212').description('Destino, ID ou Número'),
          legenda: Joi.string().allow(null).description('Legenda/nome para o arquivo'),
          file: Joi.any().meta({ swaggerType: 'file' }).required().description('Arquivo'),
          protocolo:Joi.number().description('id do atendimento').example('123')
        }).label('SendFileObject')
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v2/whatsapp/sendtoqueue',
    handler: handlers.sendtoqueue,
    config: {
      description: 'Envia uma mensagem de Texto via WhatsApp para um destino, e enfilera o atendimento na fila.',
      tags: ['api', 'v2', 'WhatsApp'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        payload: {
          fila: Joi.string().required().description('Nome da fila do Xcontact vinculada a conta de WhatsApp').example('FilaA'),
          instancia: Joi.string().required().description('Nome da fila do Xcontact vinculada a conta de WhatsApp').example('FilaA'),
          nome: Joi.string().required().description('Nome da fila do Xcontact vinculada a conta de WhatsApp').example('Juca Bala'),
          destino: Joi.string().required().description('Número de destino no formato COD_PAIS COD_REGIAO NUMERO. Ex: 555130305501').example('555130305501'),
          mensagem: Joi.string().required().description('Mensagem').example('Oi Xcontact'),
          notificacao: Joi.string().optional().allow(null).allow('').description('Mensagem opcional de notificação se Já estiver em fila').example('Você já está em atendimento.'),
          dados: Joi.object().unknown()
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v2/whatsapp/sendtoagent',
    handler: handlers.sendtoagent,
    config: {
      description: 'Envia uma mensagem de Texto via WhatsApp para um destino, e enfilera o atendimento na fila.',
      tags: ['api', 'v2', 'WhatsApp'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        payload: {
          fila: Joi.string().required().description('Nome da fila do Xcontact vinculada a conta de WhatsApp').example('FilaA'),
          agente: Joi.string().required().description('Nome do agente do Xcontact que ira atender').example('Aristeu'),
          destino: Joi.string().required().description('Número de destino no formato COD_PAIS COD_REGIAO NUMERO. Ex: 555130305501').example('555130305501'),
          mensagem: Joi.string().required().description('Mensagem').example('Oi Xcontact'),
          dados: Joi.object().unknown()
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v2/whatsapp/criarAtendimento',
    handler: handlers.criarAtendimento,
    config: {
      description: 'Cria um atendimento',
      tags: ['api', 'v2', 'WhatsApp'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        payload: {
          fila: Joi.string().required().description('Nome da fila do Xcontact vinculada a conta de WhatsApp').example('FilaA'),
          instancia: Joi.string().required().description('Nome da fila do Xcontact vinculada a conta de WhatsApp').example('FilaA'),
          nome: Joi.string().required().description('Nome da fila do Xcontact vinculada a conta de WhatsApp').example('Juca Bala'),
          numero: Joi.string().required().description('Número de destino no formato COD_PAIS COD_REGIAO NUMERO. Ex: 555130305501').example('555130305501'),
          dados: Joi.object().unknown()
        }
      }
    }
  },

];
