const dotenv = require('dotenv');
dotenv.config();

const db = require('./util/db');
const Layout = require('./models/Layouts');
const LayoutCampos = require('./models/LayoutsCampos');
const csv = require("fast-csv");
const uuid = require('uuid');
const fs = require("fs");
const TextoUtil = require('./util/texto');

let id_layout = 11;

let processarCSV = function (dadosDoArquivo, delimitador) {
    return new Promise((resolve, reject) => {
        let registros = [];
        csv.fromString(dadosDoArquivo, { headers: false, delimiter: delimitador })
            .on("data", data => registros.push(data))
            .on("error", error => reject(`Erro ao processar o CSV: ${error}`))
            .on("end", () => resolve(registros));
    });
};

async function run() {
    // Pega layout
    const layout = await Layout.query
        .innerJoin(LayoutCampos)
        .where({ id: id_layout })
        .fetch();

    // Se o layout possuir cabeçalho, deve-se considerar apenas da linha 1 pra baixo
    let indiceInicialRegistros = layout[0].cabecalho ? 1 : 0;
    let campos = layout[0].layoutsCampos.filter(x => !["telefone_adicional", "campo_adicional"].includes(x.xcontact));
    let numeros_adicionais = layout[0].layoutsCampos.filter(x => x.xcontact === "telefone_adicional");
    let campos_adicionais = layout[0].layoutsCampos.filter(x => x.xcontact === "campo_adicional");

    // Na rota
    //let dadosDoArquivo = request.payload.file.toString("latin1", 0, request.payload.file.length);

    // No teste
    let dadosDoArquivo = fs.readFileSync('Clientes.csv', 'latin1');

    // Da parse no conteúdo do arquivo
    let registros = await processarCSV(dadosDoArquivo, layout[0].delimitador);

    // Monta retorno
    let result = {
        total: 0,
        adicionados: 0,
        atualizados: 0,
        falha: 0
    };
    result.total = registros.length - indiceInicialRegistros;

    const uid = uuid.v4();

    // Popula registros para insert.
    let clientesList = [];
    let clientesCamposAdicionaisList = [];
    let clientesNumerosAdicionaisList = [];
    for (let i = indiceInicialRegistros; i < registros.length; i++) {
        let cliente = {};
        cliente.uid = uid;
        for (let campo of campos) {
            let registro = registros[i][campo.posicao_csv];
            if (registro && registro.trim().length > 0) {
                cliente[campo.xcontact] = campo.xcontact == 'numero' ? TextoUtil.limparTextoNumerico(registro) : registro;
            }
        }
        clientesList.push(cliente);

        // Registra Numeros adicionais
        for (let campo of numeros_adicionais) {
            let registro = registros[i][campo.posicao_csv];
            if (registro && registro.trim().length > 0) {
                let newReg = {
                    uid: uid,
                    clienteNumero: cliente.numero,
                    clienteEmail: cliente.email,
                    nome: campo.nome,
                    numero: TextoUtil.limparTextoNumerico(registro)
                };
                clientesNumerosAdicionaisList.push(newReg);
            }
        }

        // Registra campos adicionais
        for (let campo of campos_adicionais) {
            let registro = registros[i][campo.posicao_csv];
            if (registro && registro.trim().length > 0) {
                let newReg = {
                    uid: uid,
                    clienteNumero: cliente.numero,
                    clienteEmail: cliente.email,
                    nome: campo.nome,
                    valor: registro
                };
                clientesCamposAdicionaisList.push(newReg);
            }
        }
    }

    // Insere na tabela temp
    try {
        await db.knex.batchInsert('clientes_import', clientesList);
        if (clientesNumerosAdicionaisList.length > 0) {
            await db.knex.batchInsert('clientes_import_numeros', clientesNumerosAdicionaisList);
        }
        if (clientesCamposAdicionaisList.length > 0) {
            await db.knex.batchInsert('clientes_import_campos', clientesCamposAdicionaisList);
        }

        // Agora vem os pesados...
        await db.knex.raw(`UPDATE cliente b
        SET numero = COALESCE(a.numero, b.numero), nome = COALESCE(a.nome, b.nome), "profileImage" = COALESCE(a."profileImage", b."profileImage"), "idGrupo" = COALESCE(a."idGrupo", b."idGrupo"), email = COALESCE(a.email, b.email), documento = COALESCE(a.documento, b.documento), url = COALESCE(a.url, b.url), "facebookID" = COALESCE(a."facebookID", b."facebookID")
        FROM clientes_import a
        WHERE ((a.email is not null AND a.email = b.email) OR (a.numero is not null AND a.numero = b.numero)) AND a.uid = '${uid}'`);

        await db.knex.raw(`INSERT INTO cliente("numero", "isBusiness", "nome", "profileImage", "idGrupo", "email", "documento", "url", "facebookID")
        select "numero", "isBusiness", "nome", "profileImage", "idGrupo", "email", "documento", "url", "facebookID" from clientes_import WHERE uid = '${uid}' ON CONFLICT DO NOTHING`);

        await db.knex.raw(`UPDATE "clienteCamposAdicionais" up
        SET valor = imp."valor"
        FROM "clientes_import_campos" imp, "cliente" j
        WHERE (j.email = imp."clienteEmail" OR j.numero = imp."clienteNumero") AND j.id = up."clienteID" AND up.nome = imp.nome AND imp.uid = '${uid}'`);

        await db.knex.raw(`INSERT INTO "clienteCamposAdicionais"("clienteID", nome, valor)
        SELECT j.id,imp.nome,imp.valor
        FROM "clientes_import_campos" imp, "cliente" j
        WHERE (j.email = imp."clienteEmail" OR j.numero = imp."clienteNumero") AND imp.uid = '${uid}' ON CONFLICT DO NOTHING`);

        await db.knex.raw(`UPDATE "clienteNumerosAdicionais" up
        SET numero = imp."numero"
        FROM "clientes_import_numeros" imp, "cliente" j
        WHERE (j.email = imp."clienteEmail" OR j.numero = imp."clienteNumero") AND j.id = up."clienteID" AND up.nome = imp.nome AND imp.uid = '${uid}'`);

        await db.knex.raw(`INSERT INTO "clienteNumerosAdicionais"("clienteID", nome, numero)
        SELECT j.id,imp.nome,imp.numero
        FROM "clientes_import_numeros" imp, "cliente" j
        WHERE (j.email = imp."clienteEmail" OR j.numero = imp."clienteNumero") AND imp.uid = '${uid}' ON CONFLICT DO NOTHING`);

        // Limpar tabelas de import
        await db.knex.raw(`DELETE FROM clientes_import WHERE uid = '${uid}'`);
        await db.knex.raw(`DELETE FROM clientes_import_campos WHERE uid = '${uid}'`);
        await db.knex.raw(`DELETE FROM clientes_import_numeros WHERE uid = '${uid}'`);

    } catch (error) {
        console.log(error);
    }
    //await db.knex.batchInsert('clientes_import', clientesList, clientesList.length);

    //console.log(clientesList);
    //console.log(clientesNumerosAdicionaisList);
    //console.log(clientesCamposAdicionaisList);
    console.log(new Date());
}

console.log(new Date());
run();