'use strict'
const Boom = require('boom');
const db = require('./db');
const Paginacao = require('./Paginacao');
const knexnest = require('knexnest');

module.exports = class CRUD {
  constructor(tabela) {
    this.tabela = tabela;
  }
  returnAll (request) {
    return new Promise((resolve, reject) => {
      const max = request.query.max ? request.query.max : 15;
      const page = request.query.page ? request.query.page : 1;
      const knex = db.knex(this.tabela);

      if (this.filtros) {
        for (let f = 0; f < this.filtros.length; ++f) {
          if (!this.filtros[f].operador) {
            if (Array.isArray(this.filtros[f].valor))
              knex.whereIn(this.filtros[f].campo, this.filtros[f].valor);
            else
              knex.whereRaw(this.filtros[f].campo + ' = ?', this.filtros[f].valor);
          }
          else {
            knex.whereRaw(this.filtros[f].campo + ' ' + this.filtros[f].operador + ' ?', this.filtros[f].valor);
          }
        }
      }
      if (this.filtrosraw) {
        for (let f = 0; f < this.filtrosraw.length; ++f) {
          knex.whereRaw(this.filtrosraw[f]);
        }
      }
      if (this.innerjoin) {
        for (let f = 0; f < this.innerjoin.length; ++f) {
          knex.innerJoin(this.innerjoin[f].tabela, this.innerjoin[f].campo1, this.innerjoin[f].campo2);
        }
      }
      if (this.leftjoin) {
        for (let f = 0; f < this.leftjoin.length; ++f) {
          if (this.leftjoin[f].campos && this.leftjoin[f].campos.length > 0) {
            var campos = this.leftjoin[f].campos;
            knex.leftJoin(this.leftjoin[f].tabela, function () {
              for (let ci = 1; ci < campos.length; ci += 2) {
                this.on(campos[ci - 1], campos[ci]);
              }
            });
          }
          else {
            knex.leftJoin(this.leftjoin[f].tabela, this.leftjoin[f].campo1, this.leftjoin[f].campo2);
          }
        }
      }

      const pag = new Paginacao(max, page);
      pag.knexnest_op = this.knexnest_op;
      pag.knex = knex;
      pag.order = this.order ? this.order : undefined;
      pag.select = this.select ? this.select : undefined;

      pag.getRet((ret, err) => {
        if (err) {
          console.error(err);
          reject(Boom.badData(err));
        } else {
          resolve(ret);
        }
      });
    });
  }
  getAll (request, reply) {
    const max = request.query.max ? request.query.max : 15;
    const page = request.query.page ? request.query.page : 1;
    const knex = db.knex(this.tabela);

    if (this.filtros) {
      for (let f = 0; f < this.filtros.length; ++f) {
        if (!this.filtros[f].operador) {
          if (Array.isArray(this.filtros[f].valor))
            knex.whereIn(this.filtros[f].campo, this.filtros[f].valor);
          else
            knex.whereRaw(this.filtros[f].campo + ' = ?', this.filtros[f].valor);
        }
        else {
          knex.whereRaw(this.filtros[f].campo + ' ' + this.filtros[f].operador + ' ?', this.filtros[f].valor);
        }
      }
    }
    if (this.filtrosraw) {
      for (let f = 0; f < this.filtrosraw.length; ++f) {
        knex.whereRaw(this.filtrosraw[f]);
      }
    }
    if (this.innerjoin) {
      for (let f = 0; f < this.innerjoin.length; ++f) {
        knex.innerJoin(this.innerjoin[f].tabela, this.innerjoin[f].campo1, this.innerjoin[f].campo2);
      }
    }
    if (this.leftjoin) {
      for (let f = 0; f < this.leftjoin.length; ++f) {
        if (this.leftjoin[f].campos && this.leftjoin[f].campos.length > 0) {
          var campos = this.leftjoin[f].campos;
          knex.leftJoin(this.leftjoin[f].tabela, function () {
            for (let ci = 1; ci < campos.length; ci += 2) {
              this.on(campos[ci - 1], campos[ci]);
            }
          });
        }
        else {
          knex.leftJoin(this.leftjoin[f].tabela, this.leftjoin[f].campo1, this.leftjoin[f].campo2);
        }
      }
    }

    const pag = new Paginacao(max, page);
    pag.knexnest_op = this.knexnest_op;
    pag.knex = knex;
    pag.order = this.order ? this.order : undefined;
    pag.select = this.select ? this.select : undefined;
    pag.getRet((ret, err) => {
      if (err) {
        console.error(err);
        reply(Boom.badData(err));
      }
      else {
        reply(ret);
      }
    });
  }
  getOne (request, reply) {
    const id = this.id ? this.id : 'id';
    const knex = db.knex(this.tabela);
    const select = this.select ? this.select : '*';
    const knexnest_op = this.knexnest_op;

    if (this.innerjoin) {
      for (let f = 0; f < this.innerjoin.length; ++f) {
        knex.innerJoin(this.innerjoin[f].tabela, this.innerjoin[f].campo1, this.innerjoin[f].campo2);
      }
    }
    if (this.leftjoin) {
      for (let f = 0; f < this.leftjoin.length; ++f) {
        knex.leftJoin(this.leftjoin[f].tabela, this.leftjoin[f].campo1, this.leftjoin[f].campo2);
      }
    }

    knex.where(id, request.params.ID);
    // Usa o https://www.npmjs.com/package/knexnest
    if (this.knexnest_op) {
      knex.select(select);
      knexnest(knex)
        .then(function (rows) {
          const ret = {};
          if (rows.length > 0) {
            ret.error = false;
            ret.full = true;
            ret.total = rows.length;
            ret.message = 'Dados do ' + id + ' = ' + request.params.ID;
            ret.dados = rows[0];
          }
          else {
            ret.error = true;
            ret.total = rows.length;
            ret.message = 'Nenhum registro encontrado para ' + id + ' = ' + request.params.ID;
            ret.dados = rows[0];
          }
          reply(ret);
        })
        .catch(function (error) {
          console.error(error);
          reply(Boom.badData(error));
        });
    }
    else {
      knex.select(select)
        .then(function (rows) {
          const ret = {};
          if (rows.length > 0) {
            ret.error = false;
            ret.full = true;
            ret.total = rows.length;
            ret.message = 'Dados do ' + id + ' = ' + request.params.ID;
            ret.dados = rows[0];
          }
          else {
            ret.error = true;
            ret.total = rows.length;
            ret.message = 'Nenhum registro encontrado para ' + id + ' = ' + request.params.ID;
            ret.dados = rows[0];
          }
          reply(ret);
        })
        .catch(function (error) {
          console.error(error);
          reply(Boom.badData(error));
        });
    }
  }
  create (request, reply) {
    const knex = db.knex(this.tabela);
    const id = this.id ? this.id : 'id';
    knex.insert(request.payload);
    knex.returning(id)
      .then(function (idc) {
        const ret = {};
        ret.error = false;
        ret.id = idc[0];
        ret.message = 'ID = ' + idc[0] + ' criado com sucesso.';
        reply(ret);
      })
      .catch(function (error) {
        console.error(error);
        reply(Boom.badData(error));
      });
  }
  update (request, reply) {
    const knex = db.knex(this.tabela);
    const id = this.id ? this.id : 'id';
    for (const key in request.payload) {
      if (request.payload[key] === '') request.payload[key] = null;
    }
    knex.update(request.payload);

    knex.where(id, request.params.ID)
      .then(function (rows) {
        const ret = {};
        ret.error = false;
        ret.total = rows;
        ret.message = 'ID = ' + request.params.ID + ' atualizado com sucesso.';
        reply(ret);
      })
      .catch(function (error) {
        console.error(error);
        reply(Boom.badData(error));
      });
  }
  delete (request, reply) {
    const knex = db.knex(this.tabela);
    const id = this.id ? this.id : 'id';
    knex.where(id, request.params.ID)
      .del()
      .then(function (rows) {
        if (rows > 0) {
          const ret = {};
          ret.error = false;
          ret.total = rows;
          ret.message = 'ID = ' + request.params.ID + ' apagado com sucesso.';
          reply(ret);
        }
        else {
          reply(Boom.badData("Nenhum resgistro encontrado para ID = " + request.params.ID));
        }
      })
      .catch(function (error) {
        console.error(error);
        reply(Boom.badData(error));
      });
  }
}