/**
 * Fila temporaria pega do modelo novo só para colocar a Api nova funcionar logo..
 */
const Cliente = require('../models/Cliente');
const ClienteAtendimento = require('../models/ClienteAtendimento');
const AtendimentoMensagem = require('../models/AtendimentoMensagem');
const MensagemAnexos = require('../models/MensagemAnexos');
const Agente = require('../models/Agente');
const Pesquisa = require('../models/Pesquisa');
const PesquisaNota = require('../models/PesquisaNota');
const turno = require('../models/Turno');
const filaTurno = require('../models/FilaTurnos');
const config = require('../config.json');
const log = require('simple-node-logger').createSimpleLogger(config.log_ops);
log.setLevel(config.log ? config.log : 'error');


module.exports = class FilaTemp {

    constructor(fila) {
        this.fila = fila;

        // Verifica e valida os campos de métricas
        this.fila.atendimentos = this.fila.atendimentos || [];
        this.fila.atendimentosEncerrados = this.fila.atendimentosEncerrados || [];
        this.fila.totalAtendimentos = this.fila.totalAtendimentos || 0;
        this.fila.total_espera = this.fila.total_espera || 0;
        this.fila.total_falando = this.fila.total_falando || 0;
        this.fila.total_abandono = this.fila.total_abandono || 0;
    }

    async getAgentesDisponiveis() {
        if (!this.fila.agentes || !Array.isArray(this.fila.agentes)) return undefined;
        let ag = await this.fila.agentes.filter(async function (agente) {
            if (!agente.atendimentos) agente.atendimentos = [];
            let ag = await Agente.fetchByFullname(agente.nome);
            agente.max_chat_atende = ag.max_chat_atende;
            // O Atendimento Ativo pode ter sido redirecionado pela URA ou Bot
            var atendimentos = await ClienteAtendimento.query.require(false)
                .where({ ultimoAgente: agente.nome })
                .where({ status: 'Entrou' })
                .fetch();
            if (!atendimentos) atendimentos = [];
            // agente.atendimentos = atendimentos
            var qtd = agente.max_chat_atende - atendimentos.length
            return qtd > 0; // FILTRANDO POR STATUS E PELO MAX-CHAT-ATENDE
        });
        return ag;
    }

    async verificaTurno() {
        var filaTurnos = await filaTurno.query
            .where({ FilaID: this.fila.nome })
            .fetch();
        return filaTurnos;
    }

    //#region Cálculo de tempo médio de atendimento
    adicionarAtendimentoNaListagem(atendimento) {
        // Verifica se o atendimento já está na lista, ignorando duplicatas
        if (!this.fila.atendimentos.find(x => x.id === atendimento.id)) {
            log.info(`[FilaTemp] Novo atendimento! Atendimento#${atendimento.id}`);

            // Salva o timestamp de quando o atendimento começou
            atendimento.timestampInicioAtendimento = Date.now();

            // Verifica se o atendimento já está vinculado a um agente
            if (atendimento.ultimoAgente) {
                // Salva o timestamp do fim da espera por agente
                atendimento.timestampFimEspera = atendimento.timestampInicioAtendimento;// Usando o mesmo timestamp do inicio para não ter discrepância de ms
            }

            // Adiciona na lista
            this.fila.atendimentos.push(atendimento);

            // Incrementa o número de atendimentos totais da fila no dia
            this.fila.totalAtendimentos++;
        } else {
            log.info(`[FilaTemp] Ignorando duplicata de atendimento! Atendimento#${atendimento.id}`);
        }
    }

    removerAtendimentoDaListagem(at) {
        log.info(`[FilaTemp] Removendo atendimento! Atendimento#${at.id}`);

        // Procura o índice do atendimento na lista
        let index = this.fila.atendimentos.findIndex(x => x.id === at.id);
        // Se encontrado, o remove
        if (index !== -1) {
            const atendimento = this.fila.atendimentos[index];

            // Verifica se existe um agente vinculado ao atendimento
            if (atendimento.ultimoAgente) {
                atendimento.timestampFimAtendimento = Date.now();

                // Adiciona o tempo corrido nas métricas da fila
                // Calcula o tempo de espera e converte-o em segundos
                let tempoTotalEspera = Math.max((atendimento.timestampFimEspera - atendimento.timestampInicioAtendimento) / 1000, 0);
                if (tempoTotalEspera) this.fila.total_espera += tempoTotalEspera;

                // Calcula o tempo corrido do atendimento, converte em segundos e subtrai o tempo de espera da contagem
                let tempoTotalAtendimento = Math.max((atendimento.timestampFimAtendimento - atendimento.timestampInicioAtendimento) / 1000 - tempoTotalEspera, 0);
                if (tempoTotalAtendimento) this.fila.total_falando += tempoTotalAtendimento;

                log.info(`[FilaTemp] Atendimento#${atendimento.id} -> \n\tTempo total espera: ${tempoTotalEspera}\n\tTempo total atendimento: ${tempoTotalAtendimento}`);

                // Atualiza os cálculos
                this.calcularMediaDeTempoDosAtendimentos();
            }

            // Remove o atendimento da listagem
            this.fila.atendimentos.splice(index, 1);

            // Adiciona o atendimento pra listagem de atendimentos encerrados
            this.fila.atendimentosEncerrados.push(atendimento);
        } else {
            log.info(`[FilaTemp] Atendimento#${atendimento.id} não foi encontrado nos atendimentos ativos em cache!`);
        }
    }

    calcularMediaDeTempoDosAtendimentos() {
        log.info(`[FilaTemp] Calculando média dos tempos de atendimento...\nTotal falando:${this.fila.total_falando}\nTotal espera:${this.fila.total_espera}\nTotal atendimentos:${this.fila.totalAtendimentos}`);

        // Calcula o TMA
        this.fila.tma = (this.fila.total_falando / this.fila.totalAtendimentos) || 0;

        // Calcula o TME
        this.fila.tme = (this.fila.total_espera / this.fila.totalAtendimentos) || 0;
    }

    notificarInicioAtendimentoPorAgente(at) {
        // Busca o atendimento na lista
        let atendimento = this.fila.atendimentos.find(x => x.id === at.id);
        if (atendimento) {
            // Salva o timestamp do fim da espera por agente
            atendimento.timestampFimEspera = Date.now();

            log.info(`[FilaTemp] Atualizado o tempo de espera corrido do Atendimento#${at.id}`);
        } else {
            log.error(`[FilaTemp] Atendimento vinculado a agente porém não consta na listagem de atendimentos! Atendimento#${at.id}`);
        }
    }
    //#endregion

    getAgenteByRamal(ramal) {
        if (!this.fila.agentes || !Array.isArray(this.fila.agentes)) return undefined;
        return this.fila.agentes.filter(function (agente) {
            return agente.ramal === ramal;
        });

    }

    getAgenteByNome(nome) {
        if (!this.fila.agentes || !Array.isArray(this.fila.agentes)) return undefined;
        return this.fila.agentes.filter(function (agente) {
            return agente.nome === nome;
        });
    }

    async loadAtendimentosPendentes() {
        const atendimentos = await ClienteAtendimento.query
            .where({ status: 'Entrou' })
            .where({ queue: this.fila.nome })
            .where({ ultimoAgente: null })
            .orderBy(`"dataIni" ASC`)
            .fetch();
        if (atendimentos && atendimentos.length > 0) {
            if (!this.fila.ligacoes) this.fila.ligacoes = [];
            this.fila.ligacoes = atendimentos;
        }
    }

    async addTopoFila(atendimento) {
        if (!this.fila.ligacoes) this.fila.ligacoes = [];
        this.fila.ligacoes.unshift(atendimento);

    }

    async removerAtendimento(protocolo, ramal) {
        var agente = await this.getAgenteByRamal(ramal);
        if (agente && agente.length > 0) {
            agente = agente[0];
            agente.atendimentos = await ClienteAtendimento.query.require(false)
                .where({ ultimoAgente: agente.nome })
                .where({ status: 'Entrou' })
                .where({ ativo: true })
                .fetch();
            if (!agente.atendimentos) agente.atendimentos = [];
            for (let i = 0; i < agente.atendimentos.length; i++) {
                if (agente.atendimentos[i].id === protocolo) {
                    //agente.atendimentos.splice(i,1);
                    delete agente.atendimentos[i];
                    return true;
                }
            }
        }
        return undefined;
    }

    async ProcessPesquisa(socket, dados, fila) {
        if (fila.id_pesquisa && fila.id_pesquisa > 0) {

            // Pega a pesquisa
            var pesquisa = await Pesquisa.fetchById(fila.id_pesquisa);
            // Muda status do atendimento para Pesquisa
            var atendimento = await new ClienteAtendimento({ id: dados.atendimento, status: 'Pesquisa' }).save();
            // Envia msg de pesquisa
            if (fila.queue_type === 3) {
                socket.emit('newWhatsMsg', { instanceID: dados.fila, chatId: dados.destinatario, msg: pesquisa.descricao }, (retorno) => {

                });
            }

            if (fila.queue_type === 2) {
                dados.protocolo = dados.atendimento;
                dados.queue = dados.fila;
                dados.ClientID = dados.xcontactID;
                dados.Msg = pesquisa.descricao;
                dados.event = 'QueueSemAgentes';
                dados.resposta = pesquisa.descricao
                try {
                    socket.emit('message', JSON.stringify(dados));
                } catch (error) {
                }


            }
            // Fecha pesquisa depois depois de 5min
            // setTimeout(async (atendimento) => { 
            //     await new ClienteAtendimento({id: atendimento.id, status: 'Encerrado'}).save({require: false});
            // }, 1 * 60 * 1000, atendimento);
        } else {
            await new ClienteAtendimento({ id: dados.atendimento, status: 'Encerrado' }).save({ require: false });
        }
    }

}