'use strict'
const knexnest = require('knexnest');

module.exports = class Paginacao {
  constructor(max, page) {
    this.max = max;
    this.page = page;
  }

  /**
   * Retorna os registros.
   */
  getRet (callback) {
    if (!this.knex) {
      callback(null, 'Knex não definido.');
      return 'Knex não definido.';
    }
    if (this.max === -1 || this.knexnest_op) {
      this.knex.clearSelect();
      if (this.order) {
        for (let p = 0; p < this.order.length; ++p) {
          if (this.order[p] && this.order[p].campo && this.order[p].ordem) {
            this.knex.orderBy(this.order[p].campo, this.order[p].ordem);
          }
          else if (this.order[p] && this.order[p].campo) {
            this.knex.orderBy(this.order[p].campo, 'ASC');
          }
        }
      }
      // Usa o https://www.npmjs.com/package/knexnest
      if (this.knexnest_op) {
        this.knex.select(this.select);
        knexnest(this.knex)
          .then(function (rows) {
            if (!rows) {
              const ret = {};
              ret.error = true;
              ret.total = 0;
              ret.message = 'Nenhum registro encontrado';
              return callback(ret);
            }
            const ret = {};
            ret.error = false;
            ret.full = true;
            ret.total = rows.length;
            ret.message = 'Listagem full';
            ret.dados = rows;
            callback(ret);
          })
          .catch(function (error) {
            callback(null, error);
          });
      }
      else {
        this.knex.select(this.select ? this.select : '*')
          .then(function (rows) {
            const ret = {};
            ret.error = false;
            ret.full = true;
            ret.total = rows.length;
            ret.message = 'Listagem full';
            ret.dados = rows;
            callback(ret);
          })
          .catch(function (error) {
            callback(null, error);
          });
      }
    }
    else {
      const knex = this.knex;
      const order = this.order;
      const max = this.max;
      const page = this.page;
      const offset = max * (page - 1);
      const select = this.select;
      const knexnest_op = this.knexnest_op;
      //console.log('SQL = '+knex.count('* as total').toString());
      knex.clearSelect();
      knex.count('* as total')
        .then(function (total) {
          if (total[0].total > 0) {
            knex.clearSelect();
            // Parametros de ordenação, fora do count
            if (order) {
              for (let p = 0; p < order.length; ++p) {
                if (order[p] && order[p].campo && order[p].ordem) {
                  knex.orderBy(order[p].campo, order[p].ordem);
                }
                else if (order[p] && order[p].campo) {
                  knex.orderBy(order[p].campo, 'ASC');
                }
              }
            }
            knex.select(select ? select : '*')
              .limit(max)
              .offset(offset)
              .then(function (rows) {
                const pages = Math.ceil(parseInt(total[0].total) / max);
                const ret = {};
                ret.error = false;
                ret.full = pages === 1 ? true : false;
                ret.page = page;
                ret.max = max;
                ret.total = parseInt(total[0].total);
                ret.pages = pages;
                ret.message = pages > 1 ? 'Listagem paginada' : 'Listagem full';
                ret.dados = rows;
                return callback(ret);
              })
              .catch(function (error) {
                return callback(null, error);
              });
          }
          else {
            const ret = {};
            ret.error = true;
            ret.total = 0;
            ret.message = 'Nenhum registro encontrado';
            return callback(ret);
          }
        })
        .catch(function (error) {
          return callback(null, error);
        });
    }
  }

  returnAll () {
    return new Promise((resolve, reject) => {
      if (!this.knex) {
        reject('Knex não definido.');
        return;
      }

      this.knex.clearSelect();
      if (this.order) {
        for (let p = 0; p < this.order.length; ++p) {
          if (this.order[p] && this.order[p].campo && this.order[p].ordem) {
            this.knex.orderBy(this.order[p].campo, this.order[p].ordem);
          } else if (this.order[p] && this.order[p].campo) {
            this.knex.orderBy(this.order[p].campo, 'ASC');
          }
        }
      }

      if (this.knexnest_op) {
        this.knex.select(this.select);
        knexnest(this.knex)
          .then(rows => {
            const ret = {
              error: false,
              full: true,
              total: rows.length,
              message: 'Listagem completa',
              dados: rows
            };
            resolve(ret);
          })
          .catch(error => {
            reject(error);
          });
      } else {
        this.knex.select(this.select ? this.select : '*')
          .then(rows => {
            const ret = {
              error: false,
              full: true,
              total: rows.length,
              message: 'Listagem completa',
              dados: rows
            };
            resolve(ret);
          })
          .catch(error => {
            reject(error);
          });
      }
    });
  }
}