'use strict'
const config = require('../config.json');
const NodeCache = require("node-cache");
const Supervisores = require('../models/Supervisores');
const SupervisoresModulos = require('../models/SupervisoresModulos');
const Modulos = require('../models/Modulos');
// Token em cache por 24h, enquanto tiver o token via config isso não pode ser usado...
// Tem que implementar também a remoção do cache quando um usário relogar ou invalidar o token
//const XCache = new NodeCache({ stdTTL: 86400 });
const XCache = new NodeCache();
exports.XCache = XCache

const admin = {
    id: 999999,
    nome: "API",
    login: "API",
    email: "admin@xcontact.com.br",
    scope: ['user', 'admin', 'client']
};
XCache.set(config.api_key, admin);

exports.ValidaToken = async (key, callback) => {
    //console.log(`Valida Token chamada para token = ${key}`);
    var cached = XCache.get(key);
    if (cached) {
        //console.log(`Token em cache`);
        //console.log(cached);
        // User em cache...
        callback(cached);
    }
    else {
        try {
            const query = Supervisores.query;
            query.fields(['id', 'nome', 'fullname', 'token', 'autenticacao_fonte']);
            query.where({ token: key });
            query.leftJoin(
                SupervisoresModulos.query.innerJoin(
                    Modulos.query.where(Modulos.where.isNotNull('nome')), { fields: ['nome', 'extra_usuario'] }
                )
            );

            const ret = await query.require(true).fetch();
            const modulos = ret[0].supervisoresModulos.filter(item => item.modulos.length > 0).map(item => item.modulos[0].nome);

            let extra = ret[0].supervisoresModulos.filter(item => item.extra_usuario).map(item => {
                return {
                    modulo: item.modulos[0].nome,
                    extra: item.extra_usuario
                }
            });

            modulos.push('user'); // Remover isso futuramente, usado em rotas que o scopo não está definido com o nome do módulo
            const user = {
                id: ret[0].id,
                login: ret[0].nome,
                fullname: ret[0].fullname,
                token: ret[0].token,
                scope: modulos,
                extra: extra
            }
            // User encontrado, colocando em cache e retornando ele
            XCache.set(key, user);
            callback(user);
        } catch (error) {
            callback(null, 'User nao encontrado, sem permissao ou token vencido.');
        }
    }
}