const os = require("os");
const config = require('../config.json');
const fs = require("fs");

function appendLogToFile(message) {
    try {
        // Function for padding the string with zeros
        let f = (x, length = 2) => x.toString().padStart(length, "0");

        // Mount the timestamp string
        let t = new Date();
        let timestamp = `${f(t.getDate())}-${f(t.getMonth() + 1)}-${t.getFullYear()} ${f(t.getHours())}:${f(t.getMinutes())}:${f(t.getSeconds())}:${f(t.getMilliseconds(), 3)}`;

        // Gather all the information to log on the file
        let log = `${timestamp} SHUTDOWN ${message}\n`;

        // Appends to the system default log file
        fs.appendFileSync(config.log_ops.logFilePath, log, { encoding: "utf-8" });
    } catch (error) {
        console.error(error);
        console.trace(error);
    }
}

if (os.platform() != "win32") {
    // Define all events to register on the process
    const haltEvents = [
        `SIGINT`, `SIGQUIT`, `SIGHUP`, `SIGUSR2`, `SIGTERM`, `uncaughtException`
        /* `exit`, */ /* `unhandledRejection`, */
    ];

    function onProcessHalt() {
        try {
            console.error(`\n\nProcesso do XCONTACT encerrado!\n\n`, arguments);

            // Deixa as exceptions por último
            let parts = [...arguments].sort((a, b) => {
                if (a instanceof Error) return 1;
                if (b instanceof Error) return -1;
                return 0;
            });

            // Loga todos os argumentos no buffer
            let buffer = "{\n" + parts.map((x, i) => {
                if (x instanceof Error) {
                    return `${i} => "${x.stack || x.message || x.toString()}"`;
                } else {
                    return `${i} => "${x.toString()}"`;
                }
            }).join(", \n") + "\n}";

            // Log o buffer no arquivo de logs
            appendLogToFile(`Processo do xcontact recebeu -> ` + buffer);
        } catch (e) {
            console.trace(e);
            appendLogToFile(`Erro ao mostrar log -> ${e.toString()}`);
        }

        process.exitCode = 1;
        process.exit();
    }

    // Register hooks on the process
    haltEvents.forEach(event => {
        process.on(event, onProcessHalt.bind(this, event));
    });
}