'use strict'

const Joi = require('joi');

//Exporta objetos comuns
//Objeto de retorno comum para as requisições da API
let returnSchema = {
  error: Joi.boolean().example(false).description('Indica se o processo gerou algum erro.'),
  full: Joi.boolean().example(false).description('Indica se o retorno é completo, caso seja false será paginado.'),
  total: Joi.number().integer().example('10').description('Quantidade de registros retornados'),
  page: Joi.number().integer().example('1').description('Pagina atual dos registros'),
  offset: Joi.number().integer().example('1').description('Offset'),
  pages: Joi.number().integer().example('1').description('total de página'),
  max: Joi.number().integer().example('15').description('Máximo de registros por página'),
  pages: Joi.number().integer().example('1').description('total de páginas'),
  message: Joi.string().example('Lista de registros').description('Descrição da lista')
};

//Define o schema de retorno padrão da API e substitui a area de dados pela customPart suprida
module.exports.CreateReturnSchema = (customPart) => {
  if (customPart != null)
    return Object.assign(returnSchema, customPart);
  else
    return Object.assign(returnSchema, { dados: Joi.array().items().example([{ id: 1, nome: "Registro1" }, { id: 2, nome: "Registro2" }]).description('Lista dos registros') });
};

//Gera um identificador único
module.exports.GenerateUID = () => {
  var dt = new Date().getTime();
  var uid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
    var r = (dt + Math.random() * 16) % 16 | 0;
    dt = Math.floor(dt / 16);
    return (c == 'x' ? r : (r & 0x3 | 0x8)).toString(16);
  });
  return uid;
};

//Rercupera o endereço IP do cliente remoto
module.exports.GetRequestIP = (request) => {
  let xf = request.headers['x-forwarded-for'];
  return xf ? xf.split(',')[0] : request.info.remoteAddress;
}

//Objeto para filtro
module.exports.SearchExpressionObject = () => {
  return {
    operador: Joi.string().allow("").description('Operador aplicável. "1-Igual, 2-INiciando com, 3-Terminando Com, 4-Contendo, 5-Diferente, vazio - Contendo."'),
    valor: Joi.string().description('Valor do Filtro')
  }
};
//Construtor de expressões de filtro
module.exports.buildObjectSearchExpression = (field, obj) => {
  return this.buildSearchExpression(field, obj.operador, obj.valor);
};

//Monta cláusula where para filtros de condição
module.exports.buildSearchExpression = (field, condition, value) => {
  let exp = null;
  switch (condition) {
    case "1":
      //Equal
      exp = `LOWER(${field}) = '${value.toLowerCase()}'`;
      break;
    case "2":
      //Starting With
      exp = `LOWER(${field} LIKE '${value.toLowerCase()}%'`;
      break;
    case "3":
      //Ends With
      exp = `LOWER(${field} LIKE '%${value.toLowerCase()}'`;
      break;
    case "4":
      //Not Containing
      exp = `LOWER(${field} NOT LIKE '%${value.toLowerCase()}%'`;
      break;
    case "5":
      //Not equal
      exp = `LOWER(${field}) != '${value.toLowerCase()}'`;
      break;
    default:
      //Containing
      exp = `LOWER(${field}) LIKE '%${value.toLowerCase()}%'`;
      break;
  }
  return exp;
};


