const config = require('../config.json');
const log = require('simple-node-logger').createSimpleLogger(config.log_ops);
log.setLevel(config.log ? config.log : 'error');

module.exports = class WhatsApiClient {
    constructor() {
        this.Apisocket = require('socket.io-client')(config.whatsApi.host);

        this.Apisocket.on("error", (error) => {
            console.error(error);
            console.trace(error);
            log.error(`[Socket-Server-WhatsApiClient-Error] Erro! ` + error);
        });

        this.Apisocket.on('connect', () => {
            log.info('Conectou no whatsApi X5, enviando Login....');
            this.Apisocket.emit('Login', { clientID: config.whatsApi.clientID }, (ret) => {
                ret.error ? log.error(ret) : log.info(ret);
            });
        });

        this.Apisocket.on('newQrCode', (data) => {
            log.info('Recebeu novo Qrcod...');
            if (this.socket) {
                this.socket.emit('newQrCode', data);
            }
            else {
                log.warn('Recebi novo QRcode mas não tem ninguem escutando aqui...');
            }
        });

        this.Apisocket.on('WhatsStatus', (data) => {
            // log.info('Recebeu novo WhatsStatus...');
            if (this.socket) {
                this.socket.emit('WhatsStatus', data);
            } else {
                // log.warn('Recebi novo WhatsStatus mas não tem ninguem escutando aqui...');
            }
        });
    }
    process(socket) {
        this.socket = socket;

        socket.on('getWhatsStatus', (request, reply) => {
            log.info('Cliente chamou getWhatsStatus no processes...', request);
            this.Apisocket.emit('getWhatsStatus', { instanceID: request.instanceID }, (retorno) => {
                log.info('Recebi: ', retorno);
                if (typeof reply === 'function') reply(retorno);
            });
        });

        socket.on('startInstance', (request, reply) => {
            log.info('Cliente chamou startInstance no processes...', request);
            this.Apisocket.emit('startInstance', { instanceID: request.instanceID }, (retorno) => {
                log.info('Recebi: ', retorno);
                if (typeof reply === 'function') reply(retorno);
            });
        });

        socket.on('stopInstance', (request, reply) => {
            log.info('Cliente chamou stopInstance no processes...', request);
            this.Apisocket.emit('stopInstance', { instanceID: request.instanceID }, (retorno) => {
                log.info('Recebi: ', retorno);
                if (typeof reply === 'function') reply(retorno);
            });
        });

        socket.on('refreshQrRequest', (request, reply) => {
            log.info('Cliente chamou refreshQrRequest no processes...', request);
            this.Apisocket.emit('refreshQrRequest', { instanceID: request.instanceID }, (retorno) => {
                log.info('Recebi: ', retorno);
                if (typeof reply === 'function') reply(retorno);
            });
        });

    }
}