'use strict'
const namiLib = require("nami");
const Ami = require('../ami');
const Discador = require('../discador');
const Depurador = require('../discador/Depurador');
const ws = require('../ws');
const db = require('../util/db');
const XClientes = require('../util/XClients');
const ChatEvent = require('./ChatEvent');
const config = require('../config.json');
const moment = require('moment-timezone');
const uuid = require('uuid/v1');
const mail = require('../mail_notifier/index');
moment.tz.setDefault(config.servidor.timezone);
const xc = new XClientes();
const csa = require('../csa/index');
var Queue = [];
const FilaTemp = require('../util/FilaTemp');
const FilaPreAtendimento = require(`../models/FilaPreAtendimento`);
var mime = require('mime-to-extensions')
const opts = {
	logFilePath: '/var/log/xcontact.log',
	timestampFormat: 'DD-MM-YYYY HH:mm:ss.SSS'
};
var Atendimento = require('../models/ClienteAtendimento');
const log = require('simple-node-logger').createSimpleLogger(opts);
const log_level = config.log ? config.log : 'error';
log.setLevel(log_level);
let Omni = require('../xc_modules/omni/index');
var requestMetodos = {};
let AtendimentoMensagem = require('../models/AtendimentoMensagem');
let MensagemAnexos = require('../models/MensagemAnexos');
function sendRetorno(socket, event) {
	try {
		const msg = JSON.stringify(event);
		socket.emit('message', msg);
	} catch (ex) {
	}
}

function sendRetornoCMD(socket, event) {
	try {
		const msg = JSON.stringify(event);
		socket.emit('retornocmd', msg);
	} catch (ex) {
	}
}

exports.reloadCampanhas = function (socket, dados) {
	Discador.reloadCampanhas();
	//if(socket) sendRetorno(socket,response);
};

exports.getCampanhas = function (socket, dados) {
	let campanhas = {};
	if (dados && dados.id) {
		campanhas = Discador.getCampanhaByID(dados.id);
	}
	else campanhas = Discador.getCampanhas();
	if (socket) sendRetornoCMD(socket, campanhas);
};

exports.Disca = function (socket, dados) {
	const ramal = dados.origem.includes("/") ? dados.origem.split("/")[1] : dados.origem;
	const ac = new namiLib.Actions.Originate();
	ac.channel = dados.origem;
	ac.exten = dados.destino;
	ac.callerid = `"${dados.destino}" < ${ramal} >`;
	ac.priority = 1;
	ac.timeout = dados.timeout ? dados.timeout * 1000 : 30 * 1000;
	ac.context = dados.contexto;
	ac.async = true;
	if (dados.actionid) ac.ActionID = dados.actionid;
	if (dados.vars) ac.variable = dados.vars.join("|");

	Ami.nami.send(ac, function (response) {
		if (socket) sendRetorno(socket, response);
	});
};

exports.Teste = function (socket, dados) {
	Discador.ProcessaCampanhas();
};

exports.DepuradorCampanhasIniciar = function (socket, dados) {
	if (!dados || !dados.id) return;

	// DEBUG: Inicia a depuração da campanha especificada
	Depurador.iniciarDepuracao(socket, dados.id);
};

exports.DepuradorCampanhasFinalizar = function (socket, dados) {
	if (!dados || !dados.id) return;

	// DEBUG: Finaliza a depuração da campanha especificada
	Depurador.finalizarDepuracao(socket, dados.id);
};

exports.QueueAdd = function (socket, dados) {

	if (dados.queue instanceof Array) {
		for (let i = 0; i < dados.queue.length; ++i) {
			const ac = new namiLib.Actions.QueueAdd(dados.location, dados.queue[i], dados.paused, dados.membername, dados.penalty);
			Ami.nami.send(ac);
		}
	} else {
		const ac = new namiLib.Actions.QueueAdd(dados.location, dados.queue, dados.paused, dados.membername, dados.penalty);
		Ami.nami.send(ac);
	}
};

exports.QueueRemove = function (socket, dados) {
	dados.location = dados.location ? dados.location : dados.interface;
	if (dados.queue instanceof Array) {
		for (let i = 0; i < dados.queue.length; ++i) {
			const ac = new namiLib.Actions.QueueRemove(dados.location, dados.queue[i]);
			Ami.nami.send(ac);
		}
	}
	else {
		const ac = new namiLib.Actions.QueueRemove(dados.location, dados.queue);
		Ami.nami.send(ac);
	}
};

exports.QueuePause = function (socket, dados) {
	dados.location = dados.location ? dados.location : dados.stateinterface;
	dados.reason = dados.reason ? dados.reason : dados.motivo;
	const ac = new namiLib.Actions.QueuePause(dados.location, undefined, dados.reason);
	Ami.nami.send(ac);
};

exports.QueueUnpause = function (socket, dados) {
	dados.location = dados.location ? dados.location : dados.interface;
	const ac = new namiLib.Actions.QueueUnpause(dados.location);
	Ami.nami.send(ac);
};

exports.QueueStatus = (socket, dados) => {
	var ret = { event: 'FilasStatus' };
	let filas = [];
	ret.filas = [];
	if (dados && dados.queue) {
		filas.push(Ami.getFila(dados.queue));
	}
	if (filas.length == 0) filas = Ami.getFilas();
	for (let fila of filas) {
		let newfila = JSON.parse(JSON.stringify(fila));
		newfila.atendimentos = [];
		newfila.ligacoes_atendidas = [];
		newfila.agentes = newfila.agentes.map(agente => {
			if (agente) agente.atendimentos = [];
			return agente;
		})
		newfila.ligacoes = newfila.ligacoes.map(l => {

			if (l) {
				l.origem = l.origem ? l.origem : l.fromID.split('@c.us')[0];
				l.inicio = l.inicio ? l.inicio : l.dataIni;
			}

			return l;
		})
		ret.filas.push(newfila);
	}

	sendRetorno(socket, ret);
};

exports.Desliga = (socket, dados) => {
	Ami.Desligar(dados, (retorno) => {
		if (socket) sendRetorno(socket, retorno);
	});
};
// exports.DesligarByDestino = (socket,dados)=>{
// 	Ami.Desligar(dados, (retorno) => {
// 		if (socket) sendRetorno(socket, retorno);
// 	});
// }

requestMetodos.getAgentes = (request, reply) => {
	var filas = Ami.getFilas();
	var agentes = [];

	filas.forEach(fila => {
		fila.agentes.forEach(agente => {
			const temp = agentes.find(item => item.nome === agente.nome);
			if (temp) {
				if (temp.filas.indexOf(fila.nome) === -1) {
					temp.filas.push(fila.nome);
				}
			}
			else {
				agente.filas = [fila.nome];
				agente.StatusTimeMs = new Date(new Date() - agente.StatusDate).getTime();
				agentes.push(agente);
			}
		});
	});
	reply({ error: false, data: agentes });
}

requestMetodos.getFilas = (request, reply) => {
	let filas = JSON.parse(JSON.stringify(Ami.getFilas())).map(function (f) {
		f.ligacoes_atendidas = [];
		f.atendimentos = [];
		f.agentes = f.agentes.map(agente => {
			if (agente) {
				agente.StatusTimeMs = new Date(new Date() - new Date(agente.StatusDate)).getTime();
				agente.atendimentos = [];
			}
			return agente;
		})
		if (f.ligacoes) {
			f.ligacoes = f.ligacoes.map(l => {
				if (l) {
					l.origem = l.origem ? l.origem : l.fromID.split('@c.us')[0];
					l.inicio = l.inicio ? l.inicio : l.dataIni;
				}
				return l;
			})
		}
		return f;
	});

	if (request.filas) {
		filas = filas.filter(f => request.filas.includes(f.nome));
	}
	reply({ error: false, data: filas });
}

requestMetodos.getFila = (request, reply) => {
	var fila = Ami.getFila(request.fila);
	fila.agentes.forEach(agente => {
		agente.StatusTimeMs = new Date(new Date() - agente.StatusDate).getTime();
	});
	reply({ error: false, data: fila });
}

requestMetodos.getAgente = (request, reply) => {
	const agente = Ami.getAgentePeloRamal(request.ramal)
	if(agente) {
		reply(agente)
	}
	else {
		reply({ error: true, msg: 'Agente não encontrado, use SIP/ramal' });
	}
}

requestMetodos.QueueStatus = (request, reply) => {
	Ami.QueueStatus(request.fila, retorno => {
		reply({ error: false, data: retorno });
	});
}

// Funciona, quando usado para ligações em fila registra como abandono.
requestMetodos.Redirect = (request, reply) => {
	log.info(`Executando Redirect: Channel: ${request.Channel}, Context: ${request.Context}, Exten: ${request.Exten}`);
	const qa = new namiLib.Actions.Redirect();

	qa.ActionID = 'alakazan';
	qa.Channel = request.Channel;
	qa.Exten = request.Exten;
	qa.Context = request.Context;
	qa.Priority = 1;

	Ami.nami.send(qa, (resposta) => {
		reply({ error: false, data: resposta });
	});
}

requestMetodos.setLog = (request, reply) => {
	const log = require('../util/Log');
	log.setLevel(request.level);
	reply({ error: false, msg: `Log defindo para ${log.getLevel()}` });
}

/**
 * Recarrega um arquivo/modulo carregado por um require.
 * ###### Atenção ######
 * SÓ FUNCIONA SE USAR OS REQUIRE NAS FUNÇÕES E NÃO NO INICIO DO ARQUIVO.
 * Ex: em vez de usar no inicio do arquivo um const ev = require('./Events'); e depois usar um ev.FuncaoNova() usar direto require('./Events').FuncaoNova();
 * @param {Object} request Objeto contendo key module = nome_modulo
 * @param {Function} reply Função executado no retorno
 */
requestMetodos.moduleRestart = (request, reply) => {
	if (!request.module) {
		reply({ error: true, msg: `Precisa informa o módulo jumento!!!` });
		return;
	}
	switch (request.module) {
		case 'AmiEvents':
			try {
				delete require.cache[require.resolve('../ami/Events')];
				require('../ami/Events');
				reply({ error: false, msg: `O modulo ${request.module} foi reiniciado com sucesso!!` });
			} catch (error) {
				log.error('[WS:handlers] Erro ao reiniciar o modulo, ERRO: ', error);
			}
			break;

		default:
			reply({ error: true, msg: `O modulo ${request.module} não suporta Restart` });
			break;
	}
}

exports.AgenteShow = (socket, dados) => {

	var filtro = undefined;
	if (dados && dados.filtros) {
		filtro = dados.filtros;
	}
	Ami.getAgentes(filtro, (agentes) => {
		const ret = { event: 'AgenteShow' };
		ret.agentes = agentes;
		sendRetorno(socket, ret);
	});
};

exports.OnAuthentication = async (socket, data) => {
	data = JSON.parse(data);
	socket.XCliente = data;
	// if (data.action === 'webChat') {
	//   if (!data.protocolo || data.protocolo === undefined || data.protocolo === null) {
	//     // Sem procotocolo
	//     const protocolo_data = {
	//       socketID: socket.id,
	//       fromID: 'website_' + data.nome,
	//       fromName: data.nome,
	//       assunto: 'Webchat',
	//       action: 'QueueDisponivel',
	//       ativo: true,
	//       queue: data.queue,
	//       email: null,
	//       status: 'Entrou',
	//       dados_adicionais: JSON.stringify(data.dados_adicionais)
	//     };
	//     var protocolo = await xc.geraProtocolo(protocolo_data);
	//     data.protocolo = protocolo;
	//   }
	//   else {
	//     // Protocolo na conexao
	//     var at = await xc.ProtocoloEmAtendimento(data.protocolo);
	//     if (!at) {
	//       const protocolo_data = {
	//         socketID: socket.id,
	//         fromID: 'website_' + data.nome,
	//         fromName: data.nome,
	//         assunto: 'Webchat',
	//         action: 'QueueDisponivel',
	//         ativo: true,
	//         queue: data.queue,
	//         status: 'Entrou',
	//         email: null
	//       };
	//       var protocolo = await xc.geraProtocolo(protocolo_data);
	//       data.protocolo = protocolo;
	//     }
	//   }
	//   const qd = new ChatEvent("QueueDisponivel");
	//   qd.queue = data.queue;
	//   qd.protocolo = data.protocolo;
	//   qd.resposta = mensagensAutomaticas.protocolo_webchat;
	//   qd.nome = config.csa.nome;
	//   try {
	//     socket.emit('message', JSON.stringify(qd));
	//   } catch (error) {
	//   }

	// }
	// Coloca os dados do agente ou site no socket
	// if(socket.XCliente){
	//   await csa.getMsgPendentesFromBd();
	// }


};


//#region Omni

Ami.XCeventos.on('filasCarregadas', (filas) => {
	for (let i = 0; i < filas.length; i++) {
		if (filas[i].queue_type === 3 || filas[i].queue_type === 4 || filas[i].queue_type === 5) {
			const fila = new FilaTemp(filas[i]);
			fila.loadAtendimentosPendentes();
		}
	}
})
exports.replyMessage = async (socket, data) => {
	Omni.replyMsg(data);
	if (data.queueTypeID === 4) {
		if (data.file) {
			Omni.sendFileMsg(data);
		} else {
			Omni.sendMsg(data);
		}
	}
}
exports.getTemplates = async (socket, data) => {
	try {
		Omni.getTemplatesZenvia(data);

	} catch (error) {
		console.error(error);
	}
}
exports.sendTemplate = async (socket, data) => {
	try {
		Omni.sendTemplateMsgZenvia(data);
	} catch (error) {
		console.error(error);
	}
}
exports.deleteMessage = async (socket, data) => {
	Omni.deleteMsg(data);
}
exports.retryMessage = async (socket, data) => {
	Omni.retryMsg(data);
}
exports.ClientChatMessage = async (socket, data) => {
	if (data.file) {
		Omni.sendFileMsg(data);
	} else {
		Omni.sendMsg(data);
	}

	// if (data.queueTypeID === 2) {
	//   ws.wsEvento.emit('sendChat', data);
	// }
	// if (data.queueTypeID === 3) {
	//   ws.wsEvento.emit('sendWhats', data);
	// }
	// if (data.queueTypeID === 5) {
	//   ws.wsEvento.emit('sendWhatsZenvia', data);
	// }

	// if (data.queueTypeID === 4) {
	//   ws.wsEvento.emit('sendEmail', data);
	// }

}
async function saveMsg(msg, file) {
	try {
		var m = await new AtendimentoMensagem(msg).save();
		if (file) {
			new MensagemAnexos({ idMensagem: m.id, caminho: file.caminho, mimeType: file.mimeType }).save();
		}
		return m;
	} catch (error) {
		log.error('erro ao salvar msg', error);
	}

}
exports.TransfereAtendimentoParaFila = async (socket, data) => {
	try {
		let omniws = ws.io.of('/omni');

		let retorno = await Atendimento.query.first()
			.where({ id: data.protocolo })
			.fetch();
		
		if (retorno) {
			Omni.OmniEventos.emit('removeAtendimentoNaFila', {transferTo:data.queue, fila: retorno.queue, agente: retorno.ultimoAgente, data:new Date(), atendimentoId:retorno.id, tipo:'transferencia'})
			retorno.id = data.protocolo;
			let agenteRemetente = retorno.ultimoAgente;
			retorno.ultimoAgente = null;
			retorno.queue = data.queue;
			retorno.queueFrom = data.queueFrom;
			retorno.status = 'Entrou';
			var aux = await new Atendimento(retorno).save();
			Omni.OmniEventos.emit('removeAguardando',{ fila: fila, atendimento: retorno })
			aux.agenteRemetente = agenteRemetente;
			if (aux) {
				let preAux = await FilaPreAtendimento.query.require(false).first().where({ queueName: data.queue }).fetch();
				if (preAux) {
					const {sendMsg} = require('../xc_modules/omni/api/client/events/handlers')
					console.log(preAux);
					aux.status = `PreAtendimento`;
					aux.ativo = false;
					sendMsg(null, { protocolo: retorno.id, message: preAux.msgInicial, queue: retorno.queue, resposta: preAux.msgInicial });
				}

				aux.save();
				var fila = await Ami.getFila(data.queue);
				var ft = new FilaTemp(fila);
				ft.addTopoFila(aux);

				ws.wsEvento.emit(`transferToQueue`, aux, agenteRemetente);
				var aux1 = {
					toID: data.toID,
					name: data.nome,
					ativo: data.ativo,
					protocolo: data.protocolo,
					queue: data.queue,
					location: data.location,
					event: 'TransfereAtendimento'
				}
				Omni.removeTimerTransbordo(aux);
				let retornoTransfer = await Omni.AddHistoricoTransferencia(aux, 'fila');
				aux1.transfers = retornoTransfer.dadosAdicionais.transfers;
				var Query = await db.query('SELECT texto FROM public."queueRespostasAutomaticas" where fila = $1 AND "idRespostaTipo" = $2', [data.queueFrom, 8]);
				if (Query.rowCount > 0) {
					var texto = Query.rows[0].texto;
					console.log(`teta`, texto)
					let trocas = { "$AGENTEREMETENTE": agenteRemetente }
					if (trocas) {
						if (typeof trocas === 'object') {
							Object.entries(trocas).forEach(([key, value]) => {
								texto = texto.split(key).join(value);
							});
						}
					}
					var lastMsg = await AtendimentoMensagem.query.require(false).first()
						.where({ idAtendimento: retorno.id })
						.orderBy(`id DESC`)
						.fetch();
					let message = await saveMsg({ idAtendimento: retorno.id, texto: texto, inOut: 3, agente: 'Xcontact', mailMessageID: retorno.queueTypeID === 4 && lastMsg ? lastMsg.mailMessageID : 'Xcontact' }, null);
					//ws.broadcast(aux1);
					omniws.to(`Fila.${aux.queue}`).emit('TransfereAtendimento', aux);

				} else {
					// ws.broadcast(aux1);
					omniws.to(`Fila.${aux.queue}`).emit('TransfereAtendimento', aux);
				}
			}

		}
	} catch (error) {
		console.error('Por algum motivo nao achou o atendimento na hora de transferir ', error)
	}
}
exports.TransferenciaAtendimento = async (socket, data) => {
	try {
		let omniws = ws.io.of('/omni');

		data.fila = data.queue;
		let retorno = await Atendimento.query.first()
			.where({ id: data.protocolo })
			.fetch();
		const atendimentoAntigo = {...retorno}
		if (retorno) {
			retorno.id = data.protocolo;
			let agenteRemetente = retorno.ultimoAgente;
			retorno.ultimoAgente = data.nome;
			retorno.queueFrom = retorno.queueFrom ? retorno.queueFrom : data.queueFrom;
			retorno.queue = data.queue;
			retorno.status = "Entrou";
			var aux = await new Atendimento(retorno).save();
			const fila = await Ami.getFila(data.queue);
			Omni.OmniEventos.emit('removeAguardando',{ fila: fila, atendimento: retorno })			
			aux.agenteRemetente = agenteRemetente;
			var aux1 = {
				toID: data.toID,
				name: data.nome,
				ativo: data.ativo,
				protocolo: data.protocolo,
				queue: data.queue,
				location: data.location,
				event: 'TransfereAtendimento'
			}
			Omni.removeTimerTransbordo(aux);
			let retornoTransfer = await Omni.AddHistoricoTransferencia(aux, 'agente');
			aux1.transfers = retornoTransfer.dadosAdicionais.transfers;
			var Query = await db.query('SELECT texto FROM public."queueRespostasAutomaticas" where fila = $1 AND "idRespostaTipo" = $2', [retorno.queueFrom, 9]);
			if (Query.rowCount > 0) {
				var texto = Query.rows[0].texto;
				let trocas = { "$AGENTEREMETENTE": agenteRemetente, "$AGENTEDESTINATARIO": retorno.ultimoAgente }
				if (trocas) {
					if (typeof trocas === 'object') {
						Object.entries(trocas).forEach(([key, value]) => {
							texto = texto.split(key).join(value);
						});
					}
				}
				var lastMsg = await AtendimentoMensagem.query.require(false).first()
					.where({ idAtendimento: retorno.id })
					.orderBy(`id DESC`)
					.fetch();
				let message = await saveMsg({ idAtendimento: retorno.id, texto: texto, inOut: 3, agente: 'Xcontact', mailMessageID: retorno.queueTypeID === 4 && lastMsg ? lastMsg.mailMessageID : 'Xcontact' }, null);
				if (aux.ultimoAgente){
					 omniws.to(`Agente.${atendimentoAntigo.ultimoAgente}`).emit('TransfereAtendimento', aux);
					 console.log('AGENTE ANTIGO---------', atendimentoAntigo.ultimoAgente)
					ws.wsEvento.emit(`transferToAgent`, aux, retorno.ultimoAgente);
					 return omniws.to(`Agente.${aux.ultimoAgente}`).emit('TransfereAtendimento', aux);
				} 

				//ws.broadcast(aux1);
			} else {
				console.log(`ENTROU AQUI`);
				if (aux.ultimoAgente)  omniws.to(`Agente.${aux.ultimoAgente}`).emit('TransfereAtendimento', aux);
				//Vamos emitir o evento para o agente antigo
				 console.log('AGENTE ANTIGO---------', atendimentoAntigo.ultimoAgente)
				 omniws.to(`Agente.${atendimentoAntigo.ultimoAgente}`).emit('TransfereAtendimento', aux);
				 ws.wsEvento.emit(`transferToAgent`, aux, retorno.ultimoAgente);

				return
				// ws.broadcast(aux1);
			}
			//ws.broadcast(aux1);
			ws.wsEvento.emit(`transferToAgent`, aux, retorno.ultimoAgente);
			var faux = await Ami.getFila(data.queue);
			Omni.removeAguardando(faux, aux);

		}
	} catch (error) {
		console.error('ERRO NA TRANSFERENCIA DE ATENDIMENTO', error);
	}

}
exports.EncerrarAtendimento = async (socket, data) => {
	Omni.encerraAtendimento(data);
	//ws.wsEvento.emit('AtendimentoEncerrado', data);
}
exports.addWhats = (socket, data) => {
	ws.wsEvento.emit('addWhats', socket, data);
}
exports.addAtendimento = (socket, data) => {
	Omni.addAtendimento(data);
}
exports.addEmail = async (socket, data) => {
	ws.wsEvento.emit('addWhats', socket, data);
}
exports.NewCall = async (socket, data) => {
	ws.wsEvento.emit('NewCall', socket, data);
}

//#endregion

exports.requestMetodos = requestMetodos;
