const db = require('../util/db');
const ClienteGrupo = require('./ClienteGrupo');
const fs = require('fs');
const request = require('request');
const { exception } = require('console');
const log = require('simple-node-logger').createSimpleLogger({ logFilePath: "/var/log/xcontact.log", timestampFormat: "DD-MM-YYYY HH:mm:ss.SSS" });
const log_level = 'error';
const NumerosAdicionais = require(`../../../models/ClienteNumerosAdicionais`);
log.setLevel(log_level);
class Cliente extends db.orm.Model { }

Cliente.table = 'cliente';
Cliente.fields = {
    id: { type: 'integer', primary: true, updated: false, methods: true },
    numero: { type: 'string', unique: true, methods: true },
    isBusiness: 'boolean',
    nome: { type: 'string' },
    documento: { type: 'string' },
    profileImage: 'string',
    idGrupo: {
        type: 'integer',
        references: ClienteGrupo.fields.id
    },
    email: { type: 'string', unique: true, methods: true }
}
Cliente.getClientFromMsg = async (msg) => {
    msg.idGrupo = msg.idGrupo ? parseInt(msg.idGrupo) : null;
    try {

        var num = msg.idCliente.substr(-8);
        let numeroAdicional = await NumerosAdicionais.query.first()
            .where(NumerosAdicionais.where.like({ numero: `%${num}` })).fetch();
        //Caso nao ache na tabela de numeros adicionais
        if (!numeroAdicional) {
            var cliente = await Cliente.query.require(true).first()
                .where(Cliente.where.like({ numero: `%${num}` }))
                .fetch();
            return cliente;
        }
        var cliente = await Cliente.query.require(true).first()
            .where(Cliente.where.like({ id: numeroAdicional.clienteID }))
            .fetch();
        return cliente;

    } catch (error) {
        return await new Cliente({ numero: msg.idCliente, isBusiness: false, nome: msg.nomeCliente, idGrupo: msg.idGrupo }).save();
    }
}
Cliente.downloadImageNew = (url, cliente) => {
    try {
        var dir = `/gravacoes/clientes`;
        var stream = function () {
            fs.mkdirSync(dir, { recursive: true });
            request(url)
                .pipe(fs.createWriteStream(`${dir}/${cliente.id}.jpg`))
                .on("error", (error) => {
                    console.error(error);
                    console.trace(error);
                    log.error(`[Socket-Server-OmniXcModulesDownloadImageNew-Error] Erro! ` + error);
                });;
        }
        stream();
        return `${dir}/${cliente.id}.jpg`;
    } catch (error) {
        console.error('ERRRO AO BAIXAR IMAGEM DO WHATS');
    }
}
Cliente.getClienteByEmail = async (msg) => {
    try {
        return await Cliente.fetchByEmail(msg.idCliente)
    } catch (error) {
        if (msg.numero) {
            let msgAux = msg;
            msgAux.idCliente = msgAux.numero;
            return await Cliente.getClientFromMsg(msgAux);
        }
        return await new Cliente({ numero: null, isBusiness: false, nome: msg.name, profileImage: null, email: msg.idCliente }).save();

    }
}

Cliente.getClienteByMsg = async (msg) => {
    msg.idCliente = msg.from;
    msg.nomeCliente = msg.name;
    try {
        var cliente = (msg.queueTypeID === 3 || msg.queueTypeID === 5) ? await Cliente.getClientFromMsg(msg) : await Cliente.getClienteByEmail(msg);
        if (cliente) {
            if (msg.queueTypeID === 3) {
                if (msg.image !== null) cliente.profileImage = await Cliente.downloadImageNew(msg.image, cliente);
            }
            return await new Cliente({ id: cliente.id, numero: cliente.numero, isBusiness: msg.isBusiness, email: cliente.email, documento: cliente.documento, profileImage: cliente.profileImage, idGrupo: cliente.idGrupo ? cliente.idGrupo : null }).save();
        }
        // if (msg.queueTypeID === 3 || msg.queueTypeID === 5) {
        //     if (!cliente) return await new Cliente({ numero: null, isBusiness: false, nome: msg.nomeCliente, profileImage: null, numero: msg.idCliente }).save();
        // }

    } catch (err) {
        log.error(err);
    }
}
module.exports = Cliente;