const Atendimento = require("./class/Atendimento");
const Ami = require("../../ami/index");
const FilaTemp = require("../../util/FilaTemp");
const cron = require("node-cron");
const { Estrategia } = require("../EstrategiasDistribuicao");
const Omni = require("./index");
// Tipos de filas processadas por esse modulo
const filas_onmi = [2, 3,5, 4, 6, 7, 8, 10];
const { Log } = require("../../util/LogNew");
const log = new Log("distribuicao-modulo", Log.levelDebug);
exports.initDistribuicao = async (myEmitter, ws, autoEncerramento) => {
  const job = cron.schedule("*/10 * * * * *", () => {
    // log.debug('Executando função a cada 10 segundos');
    // log.info(`[INFO] Processing queue at ${new Date().toISOString()}`);
    pegaFilasParaDistribuir();
  });

  // Iniciando o cron job
  job.start();

  // Função para verificar atendimentos na fila
  const verificaAtendimentos = async () => {
    // Obtenha todos os atendimentos na fila com ultimoAgente igual a null
    // log.info(
    //   `[INFO] verificando atendimentos a mais tempo na fila ${new Date().toISOString()}`
    // );

    const atendimentos = await Atendimento.query
      .require(false)
      .where({ status: "Entrou" })
      .where({ ultimoAgente: null })
      .fetch();

    // Verifique cada atendimento
    for (let atendimento of atendimentos) {
      // Calcule a diferença de tempo entre agora e a dataIni do atendimento
      const diff = Date.now() - new Date(atendimento.dataIni);

      // Se a diferença for maior que 3 minutos (180000 milissegundos), distribua o atendimento
      if (diff > 180000) {
        log.warn(
          `[WARN] ATENDIMENTO => ${atendimento.id} ${new Date().toISOString()}`
        );
        let fila = await Ami.getFila(atendimento.queue);
        if (!fila.ligacoes) fila.ligacoes = [];
        atendimento.processando = false;
        const idx = fila.ligacoes.findIndex((a) => a.id === atendimento.id);
        if (idx === -1) return fila.ligacoes.unshift(atendimento);
        fila.ligacoes.splice(idx, 1, atendimento);
      }
    }
  };

  // Execute a função verificaAtendimentos a cada minuto (30000 milissegundos)
  setInterval(verificaAtendimentos, 30000);
  const pegaFilasParaDistribuir = async () => {
    try {
      let filas = Ami.getFilas();
      filas = filas.filter((f) => filas_onmi.includes(f.queue_type));
      if (filas && filas.length > 0) {
        for (let f of filas) {
          if (f) {
            if (f.ligacoes && f.ligacoes.length > 0)
              await distribuiAtendimento(f);
          }
        }
      }
    } catch (error) {
      log.error("A FILA NAO POSSUIA ATENDIMENTOS PARA DISTRIBUIR");
    }
  };

  const distribuiAtendimento = async (fila) => {
    try {
      if (!fila.ligacoes) fila.ligacoes = [];
      log.info(fila.nome, fila.ligacoes.length)
      let atendimentos = fila.ligacoes
        .filter((a) => !a.processando) // Primeiro filtra os elementos não processados
        .sort((a, b) => a.id - b.id) // Depois ordena com base no id
        .filter(
          (function () {
            const seen = new Set();
            return function (a) {
              if (seen.has(a.id)) {
                return false; // Filtra os elementos com ids duplicados
              }
              seen.add(a.id);
              return true;
            };
          })()
        );
      log.debug(
        `[INFO] TODOS OS ATENDIMENTOS NA FILA =>${
          fila.nome
        } ${new Date().toISOString()}`,
        fila.ligacoes
      );
      log.debug(fila.nome, atendimentos.length)
      if (atendimentos.length == 0)
        return log.info(
          `[INFO] NAO TEMOS ATENDIMENTOS ${new Date().toISOString()} na fila == ${
            fila.nome
          } `
        );
      log.debug(
        `[INFO] ATENDIMENTO ESCOLHIDO => ${
          atendimentos[0].id
        } ${new Date().toISOString()}`
      );
      let atendimento = atendimentos[0];
      if (!atendimento) return;
      atendimento.processando = true;
      log.debug('ESTAMOS PROCESSANDO ESSE ATENDIMENTO', atendimento.id, atendimento.queue)
      let retorno = await sortearAgente(atendimento);
    } catch (error) {
      log.error(error);
    }
  };

  const sortearAgente = async (atendimento) => {
    try {
      log.debug(
        `SORTEAR AGENTE ATENDIMENTO => ${
          atendimento.id
        } ${new Date().toISOString()}`
      );
      let fila = await Ami.getFila(atendimento.queue);
      let getAgente = Estrategia.get(fila.strategy);
      log.debug(
        `ESTRATEGIA, ATENDIMENTO => ${
          atendimento.id
        } ${new Date().toISOString()}`,
        fila.strategy
      );

      if (typeof getAgente !== "function") {
        getAgente = Estrategia.get("rrmemory");
      }
      const agente = getAgente(fila);
      if (agente) {
        const sorteado = agente;
        log.debug(
          `[INFO] SORTEADO, ATENDIMENTO => ${atendimento.id} AGENTE = ${
            sorteado.nome
          } ${new Date().toISOString()}`
        );
        let auxA = [];
        auxA = await Atendimento.query
          .require(false)
          .where({ ultimoAgente: sorteado.nome })
          .where({ status: "Entrou" })
          .where(Atendimento.where.notEqual({ queueTypeID: 1 }))
          .fetch();
        let count = sorteado.max_chat_atende - auxA.length;

        log.info(
          `[INFO] SORTEADO PODE PEGAR ${count} => ${atendimento.id} AGENTE = ${
            sorteado.nome
          } MAXIMO DE => ${sorteado.max_chat_atende} JA TEM => ${
            auxA.length
          }  = ${auxA.map((a) => a.id).join(" ")} ${new Date().toISOString()}`
        );
        if (sorteado && count > 0) {
          log.info(
            `[INFO] SORTEADO PODE PEGAR ATENDIMENTO => ${
              atendimento.id
            } AGENTE = ${sorteado.nome} ${new Date().toISOString()}`
          );
          const socketAgente = ws.getSocketAgenteByFullName(sorteado.nome);
          if (socketAgente && socketAgente.web) {
            log.info(
              `[INFO] ENVIEI O INVITE PARA O AGENTE => ${
                sorteado.nome
              } ATENDIMENTO => ${atendimento.id}${new Date().toISOString()}`
            );
            enviaInviteParaAgente(atendimento, sorteado.nome);
          } else {
            if (!fila.ligacoes) fila.ligacoes = [];
            const idx = fila.ligacoes.findIndex((a) => a.id === atendimento.id);
            fila.ligacoes.splice(idx, 1);
            await distribuiAtendimentoOld(sorteado, atendimento);
          }
        } else {
          atendimento.processando = false;
        }
        myEmitter.emit("filaStatus", { atendimento: atendimento, tipo: null });
        if (!fila.ligacoes) fila.ligacoes = [];
        return;
      } else {
        atendimento.processando = false;
      }
      myEmitter.emit("filaStatus", { atendimento: atendimento, tipo: null });
      if (!fila.ligacoes) fila.ligacoes = [];
      return;
    } catch (error) {
      console.error(error)
      log.error("[ERROR] ERRO NO SORTEAR AGENTE", error);
    }
  };
  const enviaInviteParaAgente = async (atendimento, agenteName) => {
    try {
      log.info(
        `[INFO] METODO ENVIA O INVITE PARA O AGENTE => ${agenteName} ATENDIMENTO => ${
          atendimento.id
        } ${new Date().toISOString()}`
      );

      myEmitter.emit("inviteAtendimento", { atendimento, agente: agenteName });
      myEmitter.emit("FilaInviteAtendimento", {
        fila: atendimento.queue,
        agente: agenteName,
        data: new Date(),
      });
      //Apos 30 segundos vamos liberar o atendimento para ser distribuido novamente
      const idxT = Omni.timers.findIndex((t) => t.id == atendimento.id);
      if (idxT !== -1) {
        clearTimeout(Omni.timers[idxT].timer);
        Omni.timers.splice(idxT, 1);
      }
      Omni.timers.push({
        id: atendimento.id,
        timer: setTimeout(async () => {
          log.debug(
            `[INFO] VAMOS MANDAR O REMOVEINVITE O AGENTE => ${agenteName} ATENDIMENTO => ${
              atendimento.id
            } ${new Date().toISOString()}`
          );
          atendimento.processando = false;
          myEmitter.emit("removeInviteAgente", {
            agente: agenteName,
            atendimento: atendimento,
          });
        }, 30000),
      });
    } catch (error) {
      log.error("[ERROR]", error);
    }
  };

  const distribuiAtendimentoOld = async (sorteado, atendimento) => {
    try {
      let fila = await Ami.getFila(atendimento.queue);
      myEmitter.emit("removeAguardando", {
        fila: fila,
        atendimento: atendimento,
      });
      myEmitter.emit("removeAtendimentoNaFila", {
        fila: atendimento.queue,
        agente: sorteado.nome,
        atendimentoId: atendimento.id,
        nome: atendimento.fromName,
        numero: atendimento.fromID,
        data: new Date(),
        tipo: "atendido",
      });
      myEmitter.emit("filaStatus", { atendimento: atendimento, tipo: null });
      atendimento.ultimoAgente = sorteado.nome;
      myEmitter.emit("filaStatus", { atendimento: atendimento, tipo: null });
      await new Atendimento(atendimento).save();
      addArrayMedias(atendimento, fila);
      await enviaParaAgente(atendimento);
      return;
    } catch (error) {
      log.error("[ERROR] ERRO NO distribuiAtendimentoOld AGENTE", error);
    }
  };
  const addArrayMedias = (atendimento, fila) => {
    atendimento.holdtime =
      (new Date().getTime() - new Date(atendimento.dataIni).getTime()) / 1000;
    atendimento.dataDistribuicao = new Date();
    fila.ligacoes_atendidas.push(atendimento);
    myEmitter.emit("filaStatus", { atendimento: atendimento, tipo: "" });
  };

  const enviaParaAgente = (atendimento, msg = null) => {
    return new Promise((resolve, reject) => {
      try {
        log.info(`[INFO] ENVIAPARAAGENTE`, atendimento);
        if (msg) {
          let aux = {};
          aux.message = msg;
          aux.atendimento = atendimento;
          myEmitter.emit("newMessage", aux);
          autoEncerramento.verificaTimer(null, atendimento);
          resolve();
        } else {
          autoEncerramento.verificaTimer(null, atendimento);
          myEmitter.emit("addAtendimento", atendimento);
          resolve();
        }
      } catch (error) {
        log.error(`[ERROR] [ENVIAPARAAGENTE ERROR]`, error);
        reject();
      }
    });
  };
};
