const { Log } = require('../../../../util/LogNew')
const log = new Log('WebRTCNativo-csaHandler', Log.levelDebug)

/**
 * Solicita que um INVITE seja confirmado
 * @param {*} client 
 * @param {*} call 
 * @returns Promise
 */
exports.confirmInvite = (client, call) => {
    return new Promise((resolve, reject) => {   
        // 10s para confirmar
        let timeout = setTimeout(() => {
            log.warn(`Timeout na confirmação do INVITE`)
            client.off('msg', confirmInviteListener)
            reject(new Error('Timeout na confirmação do INVITE'))
        }, 10 * 1000)
        
        // Cria listener para aguardar resposta
        const confirmInviteListener = (data) => {
            if(data.type && data.type === 'InviteConfirmed' && data.callid === call.callid) {
                log.debug(`CSA confirmou o invite para callid = ${data.callid}`)
                // Para o timeout
                clearTimeout(timeout)                
                // Removendo o listener
                client.off('msg', confirmInviteListener)
                resolve(`INVITE para callid = ${data.callid} confirmado`)
            }
        }

        // Atribui o listener ao evento msg
        client.on('msg', confirmInviteListener)

        // Msg pedindo confirmacao
        const confirmEv = {
            type: 'inviteConfirm',
            callid: call.callid,
            room: call.room,
            msg: 'Você confirma esse INVITE?'
        }

        client.emit('msg', confirmEv)
    })
}

exports.desligar = (client, call) => {
    client.emit('msg', { call, type: 'bye', room: call.room })
}