const WebrtcNativo = require('../src')
const fs = require('fs')
const wsport = 443

const options = {
    key: fs.readFileSync('cert/key.pem'),
    cert: fs.readFileSync('cert/cert.pem')
}

const httpServer = require("https").createServer(options)
const io = require("socket.io")(httpServer)



httpServer.listen(wsport, '0.0.0.0')
console.log('CallManager listening at port ', wsport)

// Controle de multiplas ligações 
// const calls = new Map()
const filas = new Map()

filas.set('Totem', {
    name: 'Totem',
    aguardando: new Map(),
    agentes: new Map()
})

filas.set('FilaB', {
    name: 'FilaB',
    aguardando: new Map(),
    agentes: new Map()
})

const config = {
    server: 'ws://localhost:1000',
    io: {
        reconnectionAttempts: 50,
        autoConnect: false
    }
}
const conn = new WebrtcNativo(config)

async function connCSA() {    
    conn.start()
    conn.on('status', status => {
        console.log('Recebendo evento de status, dados: ', status)
        if(status.conectado) {
            console.log('--- CONETADO ---')
            // O servidor escuta eventos da sala trunk
            conn._send('join', { room: 'trunk', nome: 'Xcontact' })
        }
    })

    conn.on('msg', data => {
        console.log('Recebendo mensagem: ', data)
        if (data.type === 'invite') {
            // Busca a fila pelo destino informado no invite
            const fila = filas.get(data.to)
            if(fila) {
                // cria sala no CSA para trafegar dados dessa ligação
                const room = `call_${data.callid}`
                conn._send('join', { room, nome: 'Xcontact' })

                // Responde direto pro socket que enviou o INVITE que a ligação foi aceita e indica em que sala a partir de agora devem ser trocadas as mensagens.
                const acceptMsg = {
                    type: 'accept',
                    callid: data.callid,
                    room: data.SocketID,
                    msg: 'Queued',
                    new_room: room
                }
                conn._send('msg', acceptMsg)
                const call = {
                    callid: data.callid,
                    from: data.from,
                    to: data.to,
                    dateIn: new Date(),
                    queue: fila.name,
                    room
                }
                // Coloca a ligação na fila
                fila.aguardando.set(call.callid, call)
                // Broadcast, todos agentes irão receber, depois isso vai mudar pois vai ser pro agente selecionado
                const ev = {
                    type: 'invite',
                    call
                }
                io.emit('msg', ev)
            }
            else {
                const rejectMsg = {
                    type: 'reject',
                    callid: data.callid,
                    msg: 'Destination not FOUND',
                }
                conn._send('msg', rejectMsg)                                
            }                   
        }
        else {
            // Caso não seja INVITE e tem sala manda pra sala
            if(data.room) {
                io.to(data.room).emit(`msg`, data)                
            }
        }
    })
}

connCSA()

// Logica do Xcontact com os agentes.
io.on('connection', (socket) => {
    console.log(`Nova conexao de agente no Xcontact, IP: ${socket.handshake.address}, SocketID: ${socket.id}`)
    
    socket.on(`login`, (request, reply) => {
        console.log('Agente chamando login: ', request)
        socket.LoginOk = true;
        socket.User = {
            id: 1,
            nome: 'Teste',
            fullname: 'Agente Teste'
        }
        if (typeof reply === 'function') {
            reply(socket.User)
        }
    })

    socket.on('disconnect', (e) => {
        console.log(`Cliente desconectou, IP: ${socket.handshake.address}, SocketID: ${socket.id}`)
        // Verifica se o socket estava logado em alguma fila
        filas.forEach(fila => {
            const agente = [...fila.agentes.entries()].filter(([_key, value]) => value.sockeid === socket.id).map(([k]) => k)            
            if(agente && agente.length > 0) {
                console.log('Agente encontrado, removendo ', agente[0])
                fila.agentes.delete(agente[0])
            }
        });
    })

    function MapToObject (map) {
        if (!(map instanceof Map)) return map
        return Object.fromEntries(Array.from(map.entries(), ([k, v]) => {
          if (v instanceof Array) {
            return [k, v.map(MapToObject)]
          } 
          else if (v instanceof Map) {
            return [k, MapToObject(v)]
          } 
          else if (v instanceof Object) {
            return [k, MapToObject(new Map(Object.entries(v)))]
          } else {
            return [k, v]
          }
        }))
    }

    socket.on(`getFilas`, (request, reply) => {
        console.log('Agente chamando getFilas: ', request)

        if (typeof reply === 'function') {
            let result = MapToObject(filas)

            console.log('Retornou: ', JSON.stringify(result, null, ' '))
            reply(result)
        }
    })

    socket.on(`joinFila`, (request, reply) => {
        console.log('Agente chamando loginFila: ', request)
        let msg
        if(request.fila) {
            const fila = filas.get(request.fila)
            if(fila) {
                socket.join(request.fila)
                const agente = request.agente || {}
                agente.sockeid = socket.id
                fila.agentes.set(agente.sockeid, agente)
                msg = {error: false, msg: `Parabéns, você logou na fila ${request.fila}`}
            }
            else {
                msg = {error: true, msg: `A fila ${request.fila} não existe ou não é do tipo compativel`}
            }            
        }
        else {
            msg = {error: true, msg: 'Você deve informar a fila que quer logar'}
        }

        if (typeof reply === 'function') {
            console.log('Retornou: ', msg)
            reply(msg)
        }
    })

    socket.on(`atender`, (request, reply) => {
        request.agente = {
            id: socket.User.id,
            nome: socket.User.fullname
        }
        console.log('Agente chamando atender: ', request)
        request.room = request.call.room    
        socket.join(request.room)
        conn._send('msg', request)
        const result = 'Comando enviado em async'
        if (typeof reply === 'function') {
            reply(result)
        }
    })

    socket.on(`desligar`, (request, reply) => {
        request.agente = {
            id: socket.User.id,
            nome: socket.User.fullname
        }
        console.log('Agente chamando desligar: ', request)
        request.room = request.call.room    
        conn._send('msg', request)
        const result = 'Comando enviado em async'
        if (typeof reply === 'function') {
            reply(result)
        }
    })

    socket.on(`msg`, (request, reply) => {
        console.log('Agente chamando msg: ', request)
        conn._send('msg', request)
        const result = 'Comando enviado em async'
        if (typeof reply === 'function') {
            reply(result)
        }
    })
})