/**
 * Emula um cliente externo conectado no CSA webRTC que ira enviar uma ligação que devera ser atendida por um agente
 */

const io = require('socket.io-client')
const readline = require('readline')

const rl = readline.createInterface({
    input: process.stdin,
    output: process.stdout,
    prompt: 'Totem CLI> '
})

const id = new Date().getMilliseconds()+ Math.floor(Math.random()*1000) 
const totem = {
    id, 
    name: `Totem_${id}`
}

let callid

const client = io('ws://localhost:1000')

client.on('connect', () => {
    console.log('Opa...conectou')
    rl.prompt();
})

client.on('disconnect', (e) => {
    console.log(e);
    console.log('Desconectou do servidor');
    rl.prompt();
});

client.on('msg', (msg) => {    
    console.log('Mensagem do servidor: ', msg);
    if(msg.type === 'accept') {
        console.log(`Aceitaram ligação ${msg.callid}, meu callid = ${callid}`)
        if(msg.callid === callid) {
            console.log('** Ae sim, atenderam minha ligação')
            client.emit(`join`, { room: msg.new_room, nome: totem.name })
            client.emit(`msg`, { room: msg.new_room, type: 'ACK', callid, extra: 'Blz, vamos falar por aqui' })
        }
    }
    if(msg.type === 'atender') {
        console.log(`Opa, atendimento pelo agente`)
        client.emit(`msg`, { room: msg.room, type: 'ACK', data_msg: 'Obrigado por atender minha ligacao' })
    }
    rl.prompt();
});

rl.on('line', (line) => {
    let pars = line.trim().split(' ');

    switch (pars[0]) {
        case 'sair':
        case 'quit':
        case 'q':
            rl.close();
            break;
        case '':
            break;
        case 'invite':
            callid = new Date().getMilliseconds()+ Math.floor(Math.random()*1000)
            const invite = {
                type: 'invite',
                from: totem.name,
                to: pars[1],
                callid
            }
            client.emit(`msg`, invite, response => {
                //console.log(`Retorno de getFilas ${JSON.stringify(response, null, 2)}`);
                console.log(`Retorno de INVITE: `, response);
                rl.prompt();
            })
            break;
        case 'cancel':
            const callcancel = {
                from: totem.name,
                callid,
                type: 'cancel'
            }
            client.emit(`msg`, { room: pars[1], type: 'cancel', call:  callcancel}, response => {
                console.log(`Retorno de msg ${JSON.stringify(response, null, 2)}`);
                rl.prompt();
            })
            break;
        case 'msg':
            client.emit(`msg`, { room: pars[1], type: 'teste', data_msg: pars[2] }, response => {
                console.log(`Retorno de msg ${JSON.stringify(response, null, 2)}`);
                rl.prompt();
            })
            break;
        default:
            break;
    }
    rl.prompt();
}).on('close', () => {
    console.log('Até logo!');
    process.exit(0);
});
