const EventEmitter = require('events');
const log = require("../../util/Log.js");
class MyEmitter extends EventEmitter { }

module.exports = class WhatsApi {
    WhatsEvents = [
        'WhatsStatus',
        'newWhats',
        'newQrCode',
        'newFileMsg',
        'newFileMsgQuoted',
        'sendMsgReturn',
        'message_ack'
    ];
    constructor(host, clientID) {
        this.host = host;
        this.clientID = clientID;
        this.emitter = new MyEmitter();
    }
    on(event, fn, fn2 = null) {
        return this.emitter.addListener(event, fn, fn2);
    }
    connect() {
        this.socket = require('socket.io-client')(this.host);

        this.socket.on("error", (error) => {
            try {
                console.error(error);
                console.trace(error);
                log.error(`[Socket-Server-XcModulesWhatsApi-Error] Erro! ` + error);
            } catch (e) {
                console.error(e);
                console.trace(e);
            }
        });

        this.socket.on('connect', () => {
            this.socket.emit('Login', { clientID: this.clientID }, (retorno) => {
                this.emitter.emit('LoginEvent', retorno);
            });
        });

        // Add listener for All Events in array WhatsEvents
        this.WhatsEvents.forEach(event => this.socket.on(event, (args, args2 = null) => this.emitter.emit(event, args, args2)))
    }
    // Async, retorna quando o Socket retornar...
    async startInstance(instanceID) {
        return new Promise((resolve) => {
            this.socket.emit('startInstance', { instanceID: instanceID }, (retorno) => {
                resolve(retorno);
            });
        })
    }
    async showMyInstances() {
        return new Promise((resolve) => {
            this.socket.emit('showMyInstances', (retorno) => {
                resolve(retorno);
            });
        })
    }
    sendMessage(msg, callback = null) {
        try {
            this.socket.emit('newWhatsMsg', msg, (retorno) => {
                if (callback !== null) {
                    callback(retorno);
                }
            });
        } catch (error) {
            console.error('ERRO NO SENDMESSAGE')
        }

    }
}